Package: multinma
Title: Bayesian Network Meta-Analysis of Individual and Aggregate Data
Version: 0.9.1
Authors@R: 
    c(person(given = c("David", "M."),
           family = "Phillippo",
           role = c("aut", "cre"),
           email = "david.phillippo@bristol.ac.uk",
           comment = c(ORCID = "0000-0003-2672-7841")),
    person(given = c("Samuel", "J."),
            family = "Perren",
            role = "ctb",
            email = "samuel.perren@bristol.ac.uk",
            comment = c(ORCID = "0009-0005-1921-6957")),
    person(given = "Niels",
            family = "Dunnewind",
            role = "ctb",
            comment = c(ORCID = "0009-0009-8675-9109")))
Description: Network meta-analysis and network meta-regression models for 
    aggregate data, individual patient data, and mixtures of both individual 
    and aggregate data using multilevel network meta-regression as described by
    Phillippo et al. (2020) <doi:10.1111/rssa.12579>. Models are estimated in a
    Bayesian framework using 'Stan'.
License: GPL-3
Encoding: UTF-8
LazyData: true
Biarch: true
Depends: R (>= 3.4.0)
Imports: methods, stats, graphics, survival, Rcpp (>= 0.12.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), rstantools (>=
        2.0.0), Rdpack (>= 0.7), tibble (>= 2.1.3), dplyr (>= 1.0.0),
        rlang, purrr, forcats, glue, randtoolbox, copula, tidyr (>=
        1.0.0), stringr, Matrix, igraph, ggraph, ggplot2 (>= 3.3.0),
        ggdist (>= 2.1.1), truncdist, bayesplot
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>=
        2.32.9)
SystemRequirements: GNU make
RoxygenNote: 7.3.3
Suggests: testthat (>= 2.1.0), vdiffr, withr, knitr, rmarkdown,
        parallel, R.rsp, rprojroot, loo (>= 2.0.0), crayon, tidygraph,
        pkgdown, splines2 (>= 0.5.0), flexsurv, rstpm2
RdMacros: Rdpack
URL: https://dmphillippo.github.io/multinma/,
        https://github.com/dmphillippo/multinma
BugReports: https://github.com/dmphillippo/multinma/issues
VignetteBuilder: knitr, R.rsp
NeedsCompilation: yes
Packaged: 2026-04-16 09:58:05 UTC; dp14189
Author: David M. Phillippo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2672-7841>),
  Samuel J. Perren [ctb] (ORCID: <https://orcid.org/0009-0005-1921-6957>),
  Niels Dunnewind [ctb] (ORCID: <https://orcid.org/0009-0009-8675-9109>)
Maintainer: David M. Phillippo <david.phillippo@bristol.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-16 23:10:09 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-20 23:47:17 UTC; unix
Archs: multinma.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("names<-",mcmc_array)
S3method(add_integration,data.frame)
S3method(add_integration,default)
S3method(add_integration,nma_data)
S3method(as.array,nma_dic)
S3method(as.array,nma_rank_probs)
S3method(as.array,nma_summary)
S3method(as.array,stan_nma)
S3method(as.data.frame,nma_dic)
S3method(as.data.frame,nma_summary)
S3method(as.data.frame,nodesplit_summary)
S3method(as.data.frame,stan_nma)
S3method(as.igraph,nma_data)
S3method(as.matrix,nma_dic)
S3method(as.matrix,nma_rank_probs)
S3method(as.matrix,nma_summary)
S3method(as.matrix,stan_nma)
S3method(as.stanfit,default)
S3method(as.stanfit,stan_nma)
S3method(as.tibble,nma_dic)
S3method(as.tibble,nma_summary)
S3method(as.tibble,stan_nma)
S3method(as_tibble,nma_dic)
S3method(as_tibble,nma_summary)
S3method(as_tibble,nodesplit_summary)
S3method(as_tibble,stan_nma)
S3method(names,mcmc_array)
S3method(pairs,stan_nma)
S3method(plot,mcmc_array)
S3method(plot,nma_data)
S3method(plot,nma_dic)
S3method(plot,nma_nodesplit)
S3method(plot,nma_nodesplit_df)
S3method(plot,nma_parameter_summary)
S3method(plot,nma_rank_probs)
S3method(plot,nma_summary)
S3method(plot,nodesplit_summary)
S3method(plot,stan_nma)
S3method(plot,surv_nma_summary)
S3method(predict,stan_nma)
S3method(predict,stan_nma_surv)
S3method(print,mcmc_array)
S3method(print,mlnmr_data)
S3method(print,nma_data)
S3method(print,nma_dic)
S3method(print,nma_nodesplit)
S3method(print,nma_nodesplit_df)
S3method(print,nma_prior)
S3method(print,nma_summary)
S3method(print,nodesplit_summary)
S3method(print,stan_nma)
S3method(stats::knots,stan_nma)
S3method(summary,mcmc_array)
S3method(summary,nma_nodesplit)
S3method(summary,nma_nodesplit_df)
S3method(summary,nma_prior)
S3method(summary,stan_nma)
export(.default)
export(.is_default)
export(Hmspline)
export(RE_cor)
export(Surv)
export(add_integration)
export(as.stanfit)
export(as.tibble.nodesplit_summary)
export(cauchy)
export(combine_network)
export(dbern)
export(dgamma)
export(dgent)
export(dic)
export(distr)
export(dlogitnorm)
export(dlogt)
export(dmspline)
export(exponential)
export(flat)
export(geom_km)
export(get_nodesplits)
export(half_cauchy)
export(half_normal)
export(half_student_t)
export(has_direct)
export(has_indirect)
export(hmspline)
export(inv_softmax)
export(is_network_connected)
export(log_normal)
export(log_student_t)
export(make_knots)
export(marginal_effects)
export(multi)
export(nma)
export(normal)
export(pbern)
export(pgamma)
export(pgent)
export(plogitnorm)
export(plogt)
export(plot_integration_error)
export(plot_prior_posterior)
export(pmspline)
export(posterior_rank_probs)
export(posterior_ranks)
export(qbern)
export(qgamma)
export(qgent)
export(qlogitnorm)
export(qlogt)
export(qmspline)
export(relative_effects)
export(rmst_mspline)
export(set_agd_arm)
export(set_agd_contrast)
export(set_agd_surv)
export(set_ipd)
export(softmax)
export(student_t)
export(theme_multinma)
export(unnest_integration)
export(which_RE)
import(Rcpp)
import(methods)
importFrom(RcppParallel,CxxFlags)
importFrom(Rdpack,reprompt)
importFrom(dplyr,"%>%")
importFrom(grDevices,nclass.Sturges)
importFrom(graphics,pairs)
importFrom(graphics,plot)
importFrom(igraph,as.igraph)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,enquo)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,dbinom)
importFrom(stats,dnorm)
importFrom(stats,dunif)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,optim)
importFrom(stats,pbinom)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qbinom)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(tibble,as.tibble)
importFrom(tibble,as_tibble)
importFrom(truncdist,dtrunc)
importFrom(truncdist,ptrunc)
importFrom(truncdist,qtrunc)
importFrom(utils,head)
importFrom(utils,packageVersion)
useDynLib(multinma, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# multinma: Network Meta-Analysis of individual and aggregate data in Stan <img src='man/figures/logo.svg' style="float:right" height="139" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/multinma)](https://CRAN.R-project.org/package=multinma)
[![R-universe](https://dmphillippo.r-universe.dev/badges/multinma)](https://dmphillippo.r-universe.dev)
[![R-CMD-check](https://github.com/dmphillippo/multinma/workflows/R-CMD-check/badge.svg)](https://github.com/dmphillippo/multinma/actions)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3904454.svg)](https://doi.org/10.5281/zenodo.3904454)
<!-- badges: end -->

The `multinma` package implements network meta-analysis, network
meta-regression, and multilevel network meta-regression models which
combine evidence from a network of studies and treatments using either
aggregate data or individual patient data from each study (Phillippo et
al. 2020; Phillippo 2019). Models are estimated in a Bayesian framework
using Stan (Carpenter et al. 2017).

## Installation

You can install the released version of `multinma` from
[CRAN](https://CRAN.R-project.org/package=multinma) with:

``` r
install.packages("multinma")
```

The development version can be installed from
[R-universe](https://dmphillippo.r-universe.dev) with:

``` r
install.packages("multinma", repos = c("https://dmphillippo.r-universe.dev", getOption("repos")))
```

or from source on [GitHub](https://github.com/dmphillippo/multinma)
with:

``` r
# install.packages("devtools")
devtools::install_github("dmphillippo/multinma")
```

Installing from source requires that the `rstan` package is installed
and configured. See the installation guide
[here](https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started).

## Getting started

A good place to start is with the package vignettes which walk through
example analyses, see `vignette("vignette_overview")` for an overview.
The series of NICE Technical Support Documents on evidence synthesis
gives a detailed introduction to network meta-analysis:

> Dias, S. et al. (2011). “NICE DSU Technical Support Documents 1-7:
> Evidence Synthesis for Decision Making.” *National Institute for
> Health and Care Excellence.* Available from
> <https://sheffield.ac.uk/nice-dsu/tsds>.

Multilevel network meta-regression is set out in the following methods
papers:

> Phillippo, D. M. et al. (2020). “Multilevel Network Meta-Regression
> for population-adjusted treatment comparisons.” *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*,
> 183(3):1189-1210. doi:
> [10.1111/rssa.12579](https://doi.org/10.1111/rssa.12579).

> Phillippo, D. M. et al. (2025). “Multilevel network meta-regression
> for general likelihoods: synthesis of individual and aggregate data
> with applications to survival analysis”. *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*, qnaf169. doi:
> [10.1093/jrsssa/qnaf169](https://doi.org/10.1093/jrsssa/qnaf169).

## Citing multinma

The `multinma` package can be cited as follows:

> Phillippo, D. M. (2026). *multinma: Bayesian Network Meta-Analysis of
> Individual and Aggregate Data*. R package version 0.9.1, doi:
> [10.5281/zenodo.3904454](https://doi.org/10.5281/zenodo.3904454).

When fitting ML-NMR models, please cite the methods paper:

> Phillippo, D. M. et al. (2020). “Multilevel Network Meta-Regression
> for population-adjusted treatment comparisons.” *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*,
> 183(3):1189-1210. doi:
> [10.1111/rssa.12579](https://doi.org/10.1111/rssa.12579).

For ML-NMR models with time-to-event outcomes, please cite:

> Phillippo, D. M. et al. (2025). “Multilevel network meta-regression
> for general likelihoods: synthesis of individual and aggregate data
> with applications to survival analysis”. *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*, qnaf169. doi:
> [10.1093/jrsssa/qnaf169](https://doi.org/10.1093/jrsssa/qnaf169).

## References

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-Carpenter2017" class="csl-entry">

Carpenter, B., A. Gelman, M. D. Hoffman, D. Lee, B. Goodrich, M.
Betancourt, M. Brubaker, J. Guo, P. Li, and A. Riddell. 2017. “Stan: A
Probabilistic Programming Language.” *Journal of Statistical Software*
76 (1). <https://doi.org/10.18637/jss.v076.i01>.

</div>

<div id="ref-Phillippo_thesis" class="csl-entry">

Phillippo, D. M. 2019. “Calibration of Treatment Effects in Network
Meta-Analysis Using Individual Patient Data.” PhD thesis, University of
Bristol.

</div>

<div id="ref-methods_paper" class="csl-entry">

Phillippo, D. M., S. Dias, A. E. Ades, M. Belger, A. Brnabic, A.
Schacht, D. Saure, Z. Kadziola, and N. J. Welton. 2020. “Multilevel
Network Meta-Regression for Population-Adjusted Treatment Comparisons.”
*Journal of the Royal Statistical Society: Series A (Statistics in
Society)* 183 (3): 1189–1210. <https://doi.org/10.1111/rssa.12579>.

</div>

</div>

<!-- README.md is generated from README.Rmd. Please edit that file -->

# multinma: Network Meta-Analysis of individual and aggregate data in Stan <img src='man/figures/logo.svg' style="float:right" height="139" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/multinma)](https://CRAN.R-project.org/package=multinma)
[![R-universe](https://dmphillippo.r-universe.dev/badges/multinma)](https://dmphillippo.r-universe.dev)
[![R-CMD-check](https://github.com/dmphillippo/multinma/workflows/R-CMD-check/badge.svg)](https://github.com/dmphillippo/multinma/actions)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3904454.svg)](https://doi.org/10.5281/zenodo.3904454)
<!-- badges: end -->

The `multinma` package implements network meta-analysis, network
meta-regression, and multilevel network meta-regression models which
combine evidence from a network of studies and treatments using either
aggregate data or individual patient data from each study (Phillippo et
al. 2020; Phillippo 2019). Models are estimated in a Bayesian framework
using Stan (Carpenter et al. 2017).

## Installation

You can install the released version of `multinma` from
[CRAN](https://CRAN.R-project.org/package=multinma) with:

``` r
install.packages("multinma")
```

The development version can be installed from
[R-universe](https://dmphillippo.r-universe.dev) with:

``` r
install.packages("multinma", repos = c("https://dmphillippo.r-universe.dev", getOption("repos")))
```

or from source on [GitHub](https://github.com/dmphillippo/multinma)
with:

``` r
# install.packages("devtools")
devtools::install_github("dmphillippo/multinma")
```

Installing from source requires that the `rstan` package is installed
and configured. See the installation guide
[here](https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started).

## Getting started

A good place to start is with the package vignettes which walk through
example analyses, see `vignette("vignette_overview")` for an overview.
The series of NICE Technical Support Documents on evidence synthesis
gives a detailed introduction to network meta-analysis:

> Dias, S. et al. (2011). “NICE DSU Technical Support Documents 1-7:
> Evidence Synthesis for Decision Making.” *National Institute for
> Health and Care Excellence.* Available from
> <https://sheffield.ac.uk/nice-dsu/tsds>.

Multilevel network meta-regression is set out in the following methods
papers:

> Phillippo, D. M. et al. (2020). “Multilevel Network Meta-Regression
> for population-adjusted treatment comparisons.” *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*,
> 183(3):1189-1210. doi:
> [10.1111/rssa.12579](https://doi.org/10.1111/rssa.12579).

> Phillippo, D. M. et al. (2025). “Multilevel network meta-regression
> for general likelihoods: synthesis of individual and aggregate data
> with applications to survival analysis”. *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*, qnaf169. doi:
> [10.1093/jrsssa/qnaf169](https://doi.org/10.1093/jrsssa/qnaf169).

## Citing multinma

The `multinma` package can be cited as follows:

> Phillippo, D. M. (2026). *multinma: Bayesian Network Meta-Analysis of
> Individual and Aggregate Data*. R package version 0.9.0, doi:
> [10.5281/zenodo.3904454](https://doi.org/10.5281/zenodo.3904454).

When fitting ML-NMR models, please cite the methods paper:

> Phillippo, D. M. et al. (2020). “Multilevel Network Meta-Regression
> for population-adjusted treatment comparisons.” *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*,
> 183(3):1189-1210. doi:
> [10.1111/rssa.12579](https://doi.org/10.1111/rssa.12579).

For ML-NMR models with time-to-event outcomes, please cite:

> Phillippo, D. M. et al. (2025). “Multilevel network meta-regression
> for general likelihoods: synthesis of individual and aggregate data
> with applications to survival analysis”. *Journal of the Royal
> Statistical Society: Series A (Statistics in Society)*, qnaf169. doi:
> [10.1093/jrsssa/qnaf169](https://doi.org/10.1093/jrsssa/qnaf169).

## References

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-Carpenter2017" class="csl-entry">

Carpenter, B., A. Gelman, M. D. Hoffman, D. Lee, B. Goodrich, M.
Betancourt, M. Brubaker, J. Guo, P. Li, and A. Riddell. 2017. “Stan: A
Probabilistic Programming Language.” *Journal of Statistical Software*
76 (1). <https://doi.org/10.18637/jss.v076.i01>.

</div>

<div id="ref-Phillippo_thesis" class="csl-entry">

Phillippo, D. M. 2019. “Calibration of Treatment Effects in Network
Meta-Analysis Using Individual Patient Data.” PhD thesis, University of
Bristol.

</div>

<div id="ref-methods_paper" class="csl-entry">

Phillippo, D. M., S. Dias, A. E. Ades, M. Belger, A. Brnabic, A.
Schacht, D. Saure, Z. Kadziola, and N. J. Welton. 2020. “Multilevel
Network Meta-Regression for Population-Adjusted Treatment Comparisons.”
*Journal of the Royal Statistical Society: Series A (Statistics in
Society)* 183 (3): 1189–1210. <https://doi.org/10.1111/rssa.12579>.

</div>

</div>

#' Plot theme for multinma plots
#'
#' A simple `ggplot2` theme for plots in the `multinma` package.
#'
#' @param ... Arguments passed to
#'   \code{\link[ggplot2:ggtheme]{ggplot2::theme_light()}}
#'
#' @return A `ggplot2` theme
#' @export
#'
#' @seealso [ggplot2::theme()], \code{\link[ggplot2:theme_get]{ggplot2::theme_set()}}
#'
#' @examples
#' library(ggplot2)
#' theme_set(theme_multinma())
#'
theme_multinma <- function(...) {
  ggplot2::theme_light(...) +
    ggplot2::theme(
      panel.border = ggplot2::element_rect(colour = "grey70", fill = NA),
      panel.grid.major = ggplot2::element_line(colour = "grey95"),
      panel.grid.minor = ggplot2::element_line(colour = "grey95"),
      strip.background = ggplot2::element_rect(colour = "grey70", fill = "grey90"),
      strip.text = ggplot2::element_text(colour = "black")
    )
}

.onAttach <- function(...) {
  packageStartupMessage("For execution on a local, multicore CPU with excess RAM we recommend calling")
  packageStartupMessage("options(mc.cores = parallel::detectCores())")

  invisible()
}

.onLoad <- function(...) {
  s3_register("loo::loo", "stan_nma")
  s3_register("loo::waic", "stan_nma")
  s3_register("tidygraph::as_tbl_graph", "nma_data")

  invisible()
}

require_pkg <- function(p, error = TRUE) {
  if (!rlang::is_string(p)) abort("`p` must be a package name as a string")
  if (!rlang::is_bool(error)) abort("`error` must be a logical value TRUE/FALSE")

  if (requireNamespace(p, quietly = TRUE)) {
    invisible(TRUE)
  } else if (error) {
    abort(glue::glue('Install suggested package `{p}` to use this function: install.packages("{p}")'))
  } else {
    invisible(FALSE)
  }
}

#' Treatment rankings and rank probabilities
#'
#' Produce posterior treatment rankings and rank probabilities from a fitted NMA
#' model. When a meta-regression is fitted with effect modifier interactions
#' with treatment, these will differ by study population.
#'
#' @param x A `stan_nma` object created by [nma()]
#' @param newdata Only used if a regression model is fitted. A data frame of
#'   study details, one row per study, giving the covariate values at which to
#'   produce relative effects. Column names must match variables in the
#'   regression model. If `NULL`, relative effects are produced for all studies
#'   in the network.
#' @param study Column of `newdata` which specifies study names, otherwise
#'   studies will be labelled by row number.
#' @param lower_better Logical, are lower treatment effects better (`TRUE`;
#'   default) or higher better (`FALSE`)? See details.
#' @param probs Numeric vector of quantiles of interest to present in computed
#'   summary, default `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#' @param summary Logical, calculate posterior summaries? Default `TRUE`.
#' @param sucra Logical, calculate the surface under the cumulative ranking
#'   curve (SUCRA) for each treatment? Default `FALSE`.
#' @param subset Names of treatments to include in the rank calculation, e.g. if
#'   the decision set is smaller than the analysis set. By default, all
#'   treatments in the network will be included.
#'
#' @return A [nma_summary] object if `summary = TRUE`, otherwise a list
#'   containing a 3D MCMC array of samples and (for regression models) a data
#'   frame of study information.
#' @export
#'
#' @seealso [plot.nma_summary()] for plotting the ranks and rank probabilities.
#'
#' @details The function `posterior_ranks()` produces posterior rankings, which
#'   have a distribution (e.g. mean/median rank and 95% Credible Interval). The
#'   function `posterior_rank_probs()` produces rank probabilities, which give
#'   the posterior probabilities of being ranked first, second, etc. out of all
#'   treatments.
#'
#'   The argument `lower_better` specifies whether lower treatment
#'   effects or higher treatment effects are preferred. For example, with a
#'   negative binary outcome lower (more negative) log odds ratios are
#'   preferred, so `lower_better = TRUE`. Conversely, for example, if treatments
#'   aim to increase the rate of a positive outcome then `lower_better = FALSE`.
#'
#' @examples
#' ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Produce posterior ranks
#' smk_rank_RE <- posterior_ranks(smk_fit_RE, lower_better = FALSE)
#' smk_rank_RE
#' plot(smk_rank_RE)
#'
#' # Produce rank probabilities
#' smk_rankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE)
#' smk_rankprob_RE
#' plot(smk_rankprob_RE)
#'
#' # Produce cumulative rank probabilities
#' smk_cumrankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE,
#'                                            cumulative = TRUE)
#' smk_cumrankprob_RE
#' plot(smk_cumrankprob_RE)
#'
#' # Further customisation is possible with ggplot commands
#' plot(smk_cumrankprob_RE) +
#'   ggplot2::facet_null() +
#'   ggplot2::aes(colour = Treatment)
#' }
#'
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_mlnmr_example
#' @examples \donttest{
#' # Produce population-adjusted rankings for all study populations in
#' # the network
#'
#' # Ranks
#' pso_rank <- posterior_ranks(pso_fit)
#' pso_rank
#' plot(pso_rank)
#'
#' # Rank probabilities
#' pso_rankprobs <- posterior_rank_probs(pso_fit)
#' pso_rankprobs
#' plot(pso_rankprobs)
#'
#' # Cumulative rank probabilities
#' pso_cumrankprobs <- posterior_rank_probs(pso_fit, cumulative = TRUE)
#' pso_cumrankprobs
#' plot(pso_cumrankprobs)
#'
#' # Produce population-adjusted rankings for a different target
#' # population
#' new_agd_means <- data.frame(
#'   bsa = 0.6,
#'   prevsys = 0.1,
#'   psa = 0.2,
#'   weight = 10,
#'   durnpso = 3)
#'
#' # Ranks
#' posterior_ranks(pso_fit, newdata = new_agd_means)
#'
#' # Rank probabilities
#' posterior_rank_probs(pso_fit, newdata = new_agd_means)
#'
#' # Cumulative rank probabilities
#' posterior_rank_probs(pso_fit, newdata = new_agd_means,
#'                      cumulative = TRUE)
#' }
posterior_ranks <- function(x, newdata = NULL, study = NULL,
                            lower_better = TRUE,
                            probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                            sucra = FALSE,
                            summary = TRUE,
                            subset = NULL) {
  # Checks
  if (!rlang::is_bool(lower_better))
    abort("`lower_better` should be TRUE or FALSE.")

  if (!rlang::is_bool(summary))
    abort("`summary` should be TRUE or FALSE.")

  if (!rlang::is_bool(sucra))
    abort("`sucra` should be TRUE or FALSE.")

  check_probs(probs)

  # Cannot produce relative effects for inconsistency models
  if (x$consistency != "consistency")
    abort(glue::glue("Cannot produce ranks under inconsistency '{x$consistency}' model."))

  # Check subset
  if (!is.null(subset)) {
    if (!all(subset %in% levels(x$network$treatments))) abort("Treatments listed in `subset` not present in the network.")
    subset <- factor(subset, levels = levels(x$network$treatments))
    subset <- sort(unique(subset))
    if (length(subset) == 1) abort("Must have 2 or more treatments included in `subset` to rank.")
  } else {
    subset <- x$network$treatments
  }

  # Get reference treatment, number of treatments
  trt_ref <- levels(x$network$treatments)[1]
  ntrt <- length(subset)

  # All other checks handled by relative_effects()
  rel_eff <- relative_effects(x = x, newdata = newdata, study = {{ study }},
                              all_contrasts = FALSE, summary = FALSE)

  studies <- rel_eff$studies

  if (is.null(studies)) { # No study-specific treatment effects

    # Subset
    sim <- rel_eff$sim
    sim <- sim[ , , grepl(paste0("^d\\[\\Q", subset, "\\E\\]$", collapse = "|"), dimnames(sim)[[3]]), drop = FALSE]

    # Add zeros for d[1]
    dim_d <- dim(sim)
    if (trt_ref %in% subset) dim_d[3] <- dim_d[3] + 1
    dimnames_d <- dimnames(sim)
    if (trt_ref %in% subset) dimnames_d[[3]] <- c(paste0("d[", trt_ref, "]"), dimnames_d[[3]])
    d <- array(NA_real_, dim = dim_d, dimnames = dimnames_d)
    if (trt_ref %in% subset) {
      d[ , , 1] <- 0
      d[ , , 2:ntrt] <- sim
    } else {
      d <- sim
    }

    # Get ranks at each iteration
    rk <- aperm(apply(d, 1:2, rank, ties.method = "min"),
                c("iterations", "chains", "parameters"))

    # If higher treatment effects are better
    if (!lower_better) rk <- ntrt + 1 - rk

    # Rename parameters
    dimnames(rk)[[3]] <- paste0("rank[", subset, "]")

    # Get summaries
    if (summary) {
      rk_summary <- summary_mcmc_array(rk, probs = probs) %>%
        tibble::add_column(.trt = subset, .before = 1)

      if (sucra) {
        # Calculate SUCRA using scaled mean rank relation of Rucker and Schwarzer (2015)
        sucras <- unname((ntrt - rk_summary$mean) / (ntrt - 1))
        rk_summary <- tibble::add_column(rk_summary, sucra = sucras, .after = "sd")
      }

      out <- list(summary = rk_summary, sims = rk)
    } else {
      out <- list(sims = rk)
    }

  } else { # Study-specific treatment effects
    nstudy <- nrow(studies)

    # Subset
    sim <- rel_eff$sim
    sim <- sim[ , , grepl(paste0("\\: \\Q", subset, "\\E\\]$", collapse = "|"), dimnames(sim)[[3]]), drop = FALSE]

    d <- sim
    d_names <- dimnames(d)[[3]]

    # Calculate ranks within each study population
    dim_rk <- dim(d)
    if (trt_ref %in% subset) dim_rk[3] <- dim_rk[3] + nstudy
    rk_names <- vector("character", dim_rk[[3]])
    dimnames_rk <- dimnames(d)
    dimnames_rk[[3]] <- rk_names
    rk <- array(NA_real_, dim = dim_rk, dimnames = dimnames_rk)

    dim_temp_d <- dim(d)
    dim_temp_d[[3]] <- ntrt
    dimnames_temp_d <- dimnames(d)
    dimnames_temp_d[[3]] <- paste0("d[", subset, "]")
    temp_d <- array(NA_real_, dim = dim_temp_d, dimnames = dimnames_temp_d)
    if (trt_ref %in% subset) temp_d[ , , 1] <- 0

    for (i in seq_len(nstudy)) {
      rk_names[(i - 1)*ntrt + 1:ntrt] <-
        if (trt_ref %in% subset) {
          c(paste0("d[", studies$.study[i], ": ", trt_ref, "]"),
            d_names[(i - 1)*(ntrt - 1) + 1:(ntrt - 1)])
        } else {
          d_names[(i - 1)*ntrt + 1:ntrt]
        }

      if (trt_ref %in% subset) {
        temp_d[ , , 2:ntrt] <- d[ , , (i - 1)*(ntrt - 1) + 1:(ntrt - 1)]
      } else {
        temp_d <- d[ , , (i - 1)*ntrt + 1:ntrt]
      }

      rk[ , , (i - 1)*ntrt + 1:ntrt] <-
        aperm(apply(temp_d, 1:2, rank, ties.method = "min"),
              c("iterations", "chains", "parameters"))
    }

    # If higher treatment effects are better
    if (!lower_better) rk <- ntrt + 1 - rk

    # Rename parameters
    dimnames(rk)[[3]] <- gsub("^d\\[", "rank\\[", rk_names)

    # Get summaries
    if (summary) {
      rk_summary <- summary_mcmc_array(rk, probs = probs) %>%
        # Add in study info
        tibble::add_column(.study = rep(studies$.study, each = ntrt),
                           .trt = rep(subset, times = nstudy),
                           .before = 1)

      if (sucra) {
        # Calculate SUCRA using scaled mean rank relation of Rucker and Schwarzer (2015)
        sucras <- unname((ntrt - rk_summary$mean) / (ntrt - 1))
        rk_summary <- tibble::add_column(rk_summary, sucra = sucras, .after = "sd")
      }

      out <- list(summary = rk_summary, sims = rk, studies = studies)
    } else {
      out <- list(sims = rk, studies = studies)
    }
  }

  if (summary) {
    class(out) <- c("nma_ranks", "nma_summary")
    attr(out, "xlab") <- "Treatment"
    attr(out, "ylab") <- "Posterior Rank"
  }
  return(out)
}

#' @param cumulative Logical, return cumulative rank probabilities? Default is
#'   `FALSE`, return posterior probabilities of each treatment having a given
#'   rank. If `TRUE`, cumulative posterior rank probabilities are returned for
#'   each treatment having a given rank or better.
#' @export
#' @rdname posterior_ranks
posterior_rank_probs <- function(x, newdata = NULL, study = NULL, lower_better = TRUE,
                                 cumulative = FALSE, sucra = FALSE, subset = NULL) {
  # Checks
  if (!rlang::is_bool(cumulative))
    abort("`cumulative` should be TRUE or FALSE.")

  if (!rlang::is_bool(sucra))
    abort("`sucra` should be TRUE or FALSE.")

  # All other checks handled by posterior_ranks()
  rk <- posterior_ranks(x = x, newdata = newdata, study = {{ study }},
                        lower_better = lower_better, subset = subset,
                        summary = FALSE)

  # Check subset
  if (!is.null(subset)) {
    if (!all(subset %in% levels(x$network$treatments))) abort("Treatments listed in `subset` not present in the network.")
    subset <- factor(subset, levels = levels(x$network$treatments))
    subset <- sort(unique(subset))
  } else {
    subset <- x$network$treatments
  }

  ntrt <- length(subset)
  studies <- rk$studies

  if (is.null(studies)) { # No study-specific treatment effects

    p_rank <- apply(apply(rk$sims, 1:3, `==`, 1:ntrt), c(4, 1), mean)
    if (cumulative) p_rank <- t(apply(p_rank, 1, cumsum))

    if (sucra) {
      if (cumulative) sucras <- rowMeans(p_rank[, -ntrt, drop = FALSE])
      else sucras <- rowMeans(t(apply(p_rank, 1, cumsum))[, -ntrt, drop = FALSE])
    }

    rownames(p_rank) <- stringr::str_replace(rownames(p_rank), "^rank\\[", "d\\[")
    colnames(p_rank) <- paste0("p_rank[", 1:ntrt, "]")

    p_rank <- tibble::as_tibble(p_rank, rownames = "parameter") %>%
      tibble::add_column(.trt = subset, .before = 1)

    if (sucra) p_rank$sucra <- unname(sucras)

    out <- list(summary = p_rank)

  } else { # Study-specific treatment effects

    nstudy <- nrow(studies)

    p_rank <- matrix(NA_real_, nrow = ntrt * nstudy, ncol = ntrt)
    rk_sims <- rk$sims
    for (i in seq_len(nstudy)) {
      p_rank[(i - 1)*ntrt + 1:ntrt, ] <-
        apply(apply(rk_sims[ , , (i - 1)*ntrt + 1:ntrt], 1:3, `==`, 1:ntrt), c(4, 1), mean)
    }

    if (cumulative) p_rank <- t(apply(p_rank, 1, cumsum))

    if (sucra) {
      if (cumulative) sucras <- rowMeans(p_rank[, -ntrt, drop = FALSE])
      else sucras <- rowMeans(t(apply(p_rank, 1, cumsum))[, -ntrt, drop = FALSE])
    }

    rownames(p_rank) <- stringr::str_replace(dimnames(rk_sims)[[3]], "^rank\\[", "d\\[")
    colnames(p_rank) <- paste0("p_rank[", 1:ntrt, "]")

    p_rank <- tibble::as_tibble(p_rank, rownames = "parameter") %>%
      # Add in study info
      tibble::add_column(.study = rep(studies$.study, each = ntrt),
                         .trt = rep(subset, times = nstudy),
                         .before = 1)

    if (sucra) p_rank$sucra <- unname(sucras)

    out <- list(summary = p_rank, studies = studies)

  }

  class(out) <- c("nma_rank_probs", "nma_summary")
  attr(out, "xlab") <- "Rank"
  attr(out, "ylab") <- if (cumulative) "Cumulative Rank Probability" else "Rank Probability"
  return(out)
}

# Generated by vignette example_diabetes.Rmd: do not edit by hand
# Instead edit example_diabetes.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(diabetes)


## -------------------------------------------------------------------------------------------------
db_net <- set_agd_arm(diabetes, 
                      study = studyc,
                      trt = trtc,
                      r = r, 
                      n = n)
db_net


## ----eval=FALSE-----------------------------------------------------------------------------------
## plot(db_net, weight_edges = TRUE, weight_nodes = TRUE)

## ----diabetes_network_plot, echo=FALSE------------------------------------------------------------
plot(db_net, weight_edges = TRUE, weight_nodes = TRUE) + ggplot2::theme(legend.box.margin = ggplot2::unit(c(0, 0, 0, 4), "lines"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## -------------------------------------------------------------------------------------------------
db_fit_FE <- nma(db_net, 
                 trt_effects = "fixed",
                 link = "cloglog",
                 regression = ~offset(log(time)),
                 prior_intercept = normal(scale = 100),
                 prior_trt = normal(scale = 100))


## -------------------------------------------------------------------------------------------------
db_fit_FE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(db_fit_FE, pars = c("d", "mu"))


## ----db_FE_pp_plot, fig.width=8, fig.height=6, out.width="100%"-----------------------------------
plot_prior_posterior(db_fit_FE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## -------------------------------------------------------------------------------------------------
db_fit_RE <- nma(db_net, 
                 trt_effects = "random",
                 link = "cloglog",
                 regression = ~offset(log(time)),
                 prior_intercept = normal(scale = 10),
                 prior_trt = normal(scale = 10),
                 prior_het = half_normal(scale = 5),
                 init_r = 0.5)


## -------------------------------------------------------------------------------------------------
db_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(db_fit_RE, pars = c("d", "mu", "delta"))


## ----db_RE_pp_plot--------------------------------------------------------------------------------
plot_prior_posterior(db_fit_RE, prior = c("trt", "het"))


## -------------------------------------------------------------------------------------------------
(dic_FE <- dic(db_fit_FE))

## -------------------------------------------------------------------------------------------------
(dic_RE <- dic(db_fit_RE))


## ----db_FE_resdev_plot----------------------------------------------------------------------------
plot(dic_FE)


## ----db_RE_resdev_plot----------------------------------------------------------------------------
plot(dic_RE)


## ----diabetes_releff_FE, fig.height=3-------------------------------------------------------------
(db_releff_FE <- relative_effects(db_fit_FE, trt_ref = "Diuretic"))
plot(db_releff_FE, ref_line = 0)

## ----diabetes_releff_RE, fig.height=3-------------------------------------------------------------
(db_releff_RE <- relative_effects(db_fit_RE, trt_ref = "Diuretic"))
plot(db_releff_RE, ref_line = 0)


## ----db_pred_FE, fig.height = 2-------------------------------------------------------------------
db_pred_FE <- predict(db_fit_FE, 
                      newdata = data.frame(time = 3),
                      baseline = distr(qnorm, mean = -4.2, sd = 1.11^-0.5), 
                      baseline_trt = "Diuretic",
                      type = "response")
db_pred_FE
plot(db_pred_FE)

## ----db_pred_RE, fig.height = 2-------------------------------------------------------------------
db_pred_RE <- predict(db_fit_RE, 
                      newdata = data.frame(time = 3),
                      baseline = distr(qnorm, mean = -4.2, sd = 1.11^-0.5), 
                      baseline_trt = "Diuretic",
                      type = "response")
db_pred_RE
plot(db_pred_RE)


## ----db_pred_RE_all, fig.height=16----------------------------------------------------------------
db_pred_RE_studies <- predict(db_fit_RE, type = "response")
db_pred_RE_studies
plot(db_pred_RE_studies)


## ----diabetes_ranks-------------------------------------------------------------------------------
(db_ranks <- posterior_ranks(db_fit_RE))
plot(db_ranks)

## ----diabetes_rankprobs---------------------------------------------------------------------------
(db_rankprobs <- posterior_rank_probs(db_fit_RE))
plot(db_rankprobs)

## ----diabetes_cumrankprobs------------------------------------------------------------------------
(db_cumrankprobs <- posterior_rank_probs(db_fit_RE, cumulative = TRUE))
plot(db_cumrankprobs)


## ----diabetes_tests, include=FALSE, eval=params$run_tests-----------------------------------------
#--- Test against TSD 2 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# TSD treatment codes
# trt_codes <- c(1 = "Diuretic",
#                2 = "Placebo",
#                3 = "Beta Blocker",
#                4 = "CCB",
#                5 = "ACE Inhibitor",
#                6 = "ARB")

# FE Relative effects
tsd_FE <- tribble(
~trt           , ~mean, ~sd , ~median, ~lower, ~upper,
"Placebo"      , -0.25, 0.06, -0.25  , -0.36 , -0.14 ,
"Beta Blocker" , -0.06, 0.06, -0.06  , -0.17 ,  0.05 ,
"CCB"          , -0.25, 0.05, -0.25  , -0.36 , -0.15 ,
"ACE Inhibitor", -0.36, 0.05, -0.36  , -0.46 , -0.25 ,
"ARB"          , -0.45, 0.06, -0.45  , -0.58 , -0.33 ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(db_net$treatments))) %>%
  arrange(trt)

db_releff_FE <- as.data.frame(db_releff_FE)

test_that("FE relative effects", {
  expect_equivalent(db_releff_FE$mean, tsd_FE$mean, tolerance = tol)
  expect_equivalent(db_releff_FE$sd, tsd_FE$sd, tolerance = tol)
  expect_equivalent(db_releff_FE$`50%`, tsd_FE$median, tolerance = tol)
  expect_equivalent(db_releff_FE$`2.5%`, tsd_FE$lower, tolerance = tol)
  expect_equivalent(db_releff_FE$`97.5%`, tsd_FE$upper, tolerance = tol)
})

# RE Relative effects
tsd_RE <- tribble(
~trt           , ~mean, ~sd , ~median, ~lower, ~upper,
"Placebo"      , -0.29, 0.09, -0.29  , -0.47 , -0.12 ,
"Beta Blocker" , -0.07, 0.09, -0.07  , -0.25 ,  0.10 ,
"CCB"          , -0.24, 0.08, -0.24  , -0.41 , -0.08 ,
"ACE Inhibitor", -0.40, 0.09, -0.40  , -0.58 , -0.24 ,
"ARB"          , -0.47, 0.11, -0.47  , -0.70 , -0.27 ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(db_net$treatments))) %>%
  arrange(trt)

db_releff_RE <- as.data.frame(db_releff_RE)

test_that("RE relative effects", {
  expect_equivalent(db_releff_RE$mean, tsd_RE$mean, tolerance = tol)
  expect_equivalent(db_releff_RE$sd, tsd_RE$sd, tolerance = tol)
  expect_equivalent(db_releff_RE$`50%`, tsd_RE$median, tolerance = tol)
  expect_equivalent(db_releff_RE$`2.5%`, tsd_RE$lower, tolerance = tol)
  expect_equivalent(db_releff_RE$`97.5%`, tsd_RE$upper, tolerance = tol)
})

# Heterogeneity SD
db_tau <- as.data.frame(summary(db_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(db_tau$mean, 0.13, tolerance = tol)
  expect_equivalent(db_tau$sd, 0.04, tolerance = tol)
  expect_equivalent(db_tau$`50%`, 0.12, tolerance = tol)
  expect_equivalent(db_tau$`2.5%`, 0.05, tolerance = tol)
  expect_equivalent(db_tau$`97.5%`, 0.23, tolerance = tol)
})

# FE probabilities
tsd_pred_FE <- tribble(
~trt           , ~mean, ~sd  , ~median, ~lower, ~upper,
"Diuretic"     , 0.065, 0.067, 0.044  , 0.01  ,0.25   ,
"Placebo"      , 0.052, 0.055, 0.034  , 0.01  ,0.20   ,
"Beta Blocker" , 0.062, 0.064, 0.042  , 0.01  ,0.24   ,
"CCB"          , 0.051, 0.055, 0.034  , 0.01  ,0.20   ,
"ACE Inhibitor", 0.047, 0.050, 0.031  , 0.00  ,0.18   ,
"ARB"          , 0.043, 0.046, 0.028  , 0.00  ,0.17   ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(db_net$treatments))) %>%
  arrange(trt)

db_pred_FE <- as.data.frame(db_pred_FE)

test_that("FE predicted probabilities at 3 years", {
  expect_equivalent(db_pred_FE$mean, tsd_pred_FE$mean, tolerance = tol)
  expect_equivalent(db_pred_FE$sd, tsd_pred_FE$sd, tolerance = tol)
  expect_equivalent(db_pred_FE$`50%`, tsd_pred_FE$median, tolerance = tol)
  expect_equivalent(db_pred_FE$`2.5%`, tsd_pred_FE$lower, tolerance = tol)
  expect_equivalent(db_pred_FE$`97.5%`, tsd_pred_FE$upper, tolerance = tol)
})

# RE probabilities
tsd_pred_RE <- tribble(
~trt           , ~mean, ~sd  , ~median, ~lower, ~upper,
"Diuretic"     , 0.065, 0.067, 0.044  , 0.01  ,0.25   ,
"Placebo"      , 0.050, 0.053, 0.033  , 0.01  ,0.20   ,
"Beta Blocker" , 0.061, 0.064, 0.041  , 0.01  ,0.24   ,
"CCB"          , 0.052, 0.056, 0.035  , 0.01  ,0.20   ,
"ACE Inhibitor", 0.045, 0.048, 0.030  , 0.00  ,0.18   ,
"ARB"          , 0.042, 0.046, 0.028  , 0.00  ,0.17   ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(db_net$treatments))) %>%
  arrange(trt)

db_pred_RE <- as.data.frame(db_pred_RE)

test_that("RE predicted probabilities at 3 years", {
  expect_equivalent(db_pred_RE$mean, tsd_pred_RE$mean, tolerance = tol)
  expect_equivalent(db_pred_RE$sd, tsd_pred_RE$sd, tolerance = tol)
  expect_equivalent(db_pred_RE$`50%`, tsd_pred_RE$median, tolerance = tol)
  expect_equivalent(db_pred_RE$`2.5%`, tsd_pred_RE$lower, tolerance = tol)
  expect_equivalent(db_pred_RE$`97.5%`, tsd_pred_RE$upper, tolerance = tol)
})

# RE DIC
test_that("RE DIC", {
  expect_equivalent(dic_RE$resdev, 53.7, tolerance = tol_dic)
  expect_equivalent(dic_RE$pd, 38.0, tolerance = tol_dic)
  expect_equivalent(dic_RE$dic, 91.7, tolerance = tol_dic)
})

# FE DIC
test_that("FE DIC", {
  expect_equivalent(dic_FE$resdev, 78.25, tolerance = tol_dic)
  expect_equivalent(dic_FE$pd, 27.0, tolerance = tol_dic)
  expect_equivalent(dic_FE$dic, 105.2, tolerance = tol_dic)
})

# Check that predictions for multiple studies works in any order
times <- 1:3
# Baselines named by time
bls <- list("1" = distr(qnorm, mean = -4.2, sd = 1.11^-0.5),
            "2" = distr(qnorm, mean = -4.2, sd = 1.11^-0.5),
            "3" = distr(qnorm, mean = -4.2, sd = 1.11^-0.5))

db_pred_FE_multi1 <- as.data.frame(predict(db_fit_FE, 
                      newdata = data.frame(time = times),
                      baseline = unname(bls), 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

db_pred_RE_multi1 <- as.data.frame(predict(db_fit_RE, 
                      newdata = data.frame(time = times),
                      baseline = unname(bls), 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

db_pred_FE_multi2 <- as.data.frame(predict(db_fit_FE, 
                      newdata = data.frame(time = times),
                      baseline = bls, 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

db_pred_RE_multi2 <- as.data.frame(predict(db_fit_RE, 
                      newdata = data.frame(time = times),
                      baseline = bls, 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

db_pred_FE_multi3 <- as.data.frame(predict(db_fit_FE, 
                      newdata = data.frame(time = rev(times)),
                      baseline = bls, 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response")) %>% 
  arrange(.study)

db_pred_RE_multi3 <- as.data.frame(predict(db_fit_RE, 
                      newdata = data.frame(time = rev(times)),
                      baseline = bls, 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response")) %>% 
  arrange(.study)

db_pred_FE_multi4 <- as.data.frame(predict(db_fit_FE, 
                      newdata = data.frame(time = times),
                      baseline = rev(bls), 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

db_pred_RE_multi4 <- as.data.frame(predict(db_fit_RE, 
                      newdata = data.frame(time = times),
                      baseline = rev(bls), 
                      study = time,
                      baseline_trt = "Diuretic",
                      type = "response"))

test_that("Predictions for reordered newdata/baselines works", {
  expect_equivalent(db_pred_FE_multi1$mean,    db_pred_FE_multi2$mean, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$sd,      db_pred_FE_multi2$sd, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`50%`,   db_pred_FE_multi2$`50%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`2.5%`,  db_pred_FE_multi2$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`97.5%`, db_pred_FE_multi2$`97.5%`, tolerance = tol)
  
  expect_equivalent(db_pred_FE_multi1$mean,    db_pred_FE_multi3$mean, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$sd,      db_pred_FE_multi3$sd, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`50%`,   db_pred_FE_multi3$`50%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`2.5%`,  db_pred_FE_multi3$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`97.5%`, db_pred_FE_multi3$`97.5%`, tolerance = tol)
  
  expect_equivalent(db_pred_FE_multi1$mean,    db_pred_FE_multi4$mean, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$sd,      db_pred_FE_multi4$sd, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`50%`,   db_pred_FE_multi4$`50%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`2.5%`,  db_pred_FE_multi4$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_FE_multi1$`97.5%`, db_pred_FE_multi4$`97.5%`, tolerance = tol)
  
  expect_equivalent(db_pred_RE_multi1$mean,    db_pred_RE_multi2$mean, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$sd,      db_pred_RE_multi2$sd, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`50%`,   db_pred_RE_multi2$`50%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`2.5%`,  db_pred_RE_multi2$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`97.5%`, db_pred_RE_multi2$`97.5%`, tolerance = tol)
  
  expect_equivalent(db_pred_RE_multi1$mean,    db_pred_RE_multi3$mean, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$sd,      db_pred_RE_multi3$sd, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`50%`,   db_pred_RE_multi3$`50%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`2.5%`,  db_pred_RE_multi3$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`97.5%`, db_pred_RE_multi3$`97.5%`, tolerance = tol)
  
  expect_equivalent(db_pred_RE_multi1$mean,    db_pred_RE_multi4$mean, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$sd,      db_pred_RE_multi4$sd, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`50%`,   db_pred_RE_multi4$`50%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`2.5%`,  db_pred_RE_multi4$`2.5%`, tolerance = tol)
  expect_equivalent(db_pred_RE_multi1$`97.5%`, db_pred_RE_multi4$`97.5%`, tolerance = tol)
})


# Force clean up
rm(list = ls())
gc()

# Generated by vignette example_certolizumab.Rmd: do not edit by hand
# Instead edit example_certolizumab.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------

## ----include=FALSE--------------------------------------------------------------------------------
set.seed(76441)


## ----eval = FALSE---------------------------------------------------------------------------------
# library(multinma)
# options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)

## -------------------------------------------------------------------------------------------------
library(dplyr)
library(ggplot2)


## -------------------------------------------------------------------------------------------------
head(certolizumab)


## ----certolizumab_baseline_risk_plot--------------------------------------------------------------
certolizumab <-
  certolizumab %>%
  group_by(study) %>% 
  mutate(
    cc = any(r == 0),
    probability = if_else(cc, (r + 0.5) / (n + 0.5), r / n),
    odds = probability / (1 - probability),
    log_odds = log(odds)
  )

p_baseline_risk <-
  left_join(
    filter(certolizumab, trt != "Placebo"),
    filter(certolizumab, trt == "Placebo"),
    by = "study",
    suffix = c("", "_baseline")
  ) %>%
  mutate(log_odds_ratio = log_odds - log_odds_baseline, n_total = n + n_baseline) %>%
  ggplot(aes(log_odds_baseline)) +
  geom_hline(yintercept = 0, linetype = "dashed") +
  labs(x = "Placebo log odds", y = "log Odds Ratio", size = "Sample Size") +
  theme_multinma()

p_baseline_risk +
  geom_point(aes(y = log_odds_ratio, size = n_total))


## -------------------------------------------------------------------------------------------------
cert_net <- set_agd_arm(certolizumab,
                        study = study, trt = trt, n = n, r = r,
                        trt_class = if_else(trt == "Placebo", "Placebo", "Treatment"))
cert_net


## ----eval=FALSE-----------------------------------------------------------------------------------
# plot(cert_net, weight_edges = TRUE, weight_nodes = TRUE)

## ----certolizumab_network_plot, echo=FALSE--------------------------------------------------------
plot(cert_net, weight_edges = TRUE, weight_nodes = TRUE) +
  ggplot2::theme(legend.box.margin = ggplot2::unit(c(0, 0, 0, 4), "lines"))


## ----eval=!params$run_tests-----------------------------------------------------------------------
# cert_fit_FE <- nma(cert_net,
#                    trt_effects = "fixed",
#                    regression = ~.mu:.trt,
#                    prior_intercept = normal(scale = sqrt(1000)),
#                    prior_trt = normal(scale = 100),
#                    prior_reg = normal(scale = 100),
#                    adapt_delta = 0.95)

## ----echo=FALSE, eval=params$run_tests------------------------------------------------------------
cert_fit_FE <- nowarn_on_ci(nma(cert_net,
                   trt_effects = "fixed",
                   regression = ~.mu:.trt,
                   prior_intercept = normal(scale = sqrt(1000)),
                   prior_trt = normal(scale = 100),
                   prior_reg = normal(scale = 100),
                   adapt_delta = 0.95))


## -------------------------------------------------------------------------------------------------
cert_fit_FE


## ----eval=!params$run_tests-----------------------------------------------------------------------
# cert_fit_RE <- nma(cert_net,
#                    trt_effects = "random",
#                    regression = ~.mu:.trt,
#                    prior_intercept = normal(scale = sqrt(1000)),
#                    prior_trt = normal(scale = 100),
#                    prior_reg = normal(scale = 100),
#                    prior_het = half_normal(2.5),
#                    adapt_delta = 0.95)

## ----echo=FALSE, eval=params$run_tests------------------------------------------------------------
cert_fit_RE <- nowarn_on_ci(nma(cert_net,
                   trt_effects = "random",
                   regression = ~.mu:.trt,
                   prior_intercept = normal(scale = sqrt(1000)),
                   prior_trt = normal(scale = 100),
                   prior_reg = normal(scale = 100),
                   prior_het = half_normal(2.5),
                   iter = 8000,
                   adapt_delta = 0.95))


## -------------------------------------------------------------------------------------------------
cert_fit_RE


## -------------------------------------------------------------------------------------------------
(dic_FE <- dic(cert_fit_FE))
(dic_RE <- dic(cert_fit_RE))


## -------------------------------------------------------------------------------------------------
plot(dic_FE)


## -------------------------------------------------------------------------------------------------
plot(dic_RE)


## ----certolizumab_reg_plot------------------------------------------------------------------------
cert_mu_reg <-
  cert_fit_FE %>%
  relative_effects(
    newdata = tibble(.mu = seq(log(0.01), log(0.5), length.out = 20)),
    study = .mu
  ) %>%
  as_tibble() %>%
  mutate(
    trt = .trtb,
    log_odds_baseline = as.numeric(as.character(.study))
  )

p_baseline_risk +
  facet_wrap(vars(trt)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), data = cert_mu_reg,
              fill = "darkred", alpha = 0.3) +
  geom_line(aes(y = mean), data = cert_mu_reg,
            colour = "darkred") +
  geom_point(aes(y = log_odds_ratio, size = n_total), alpha = 0.6)


## ----certolizumab_releff_FE, fig.height=3---------------------------------------------------------
newdata <- data.frame(.mu = cert_fit_FE$xbar[[".mu"]])
(cert_releff_FE <- relative_effects(cert_fit_FE, newdata = newdata))
plot(cert_releff_FE, ref_line = 0)

## ----certolizumab_releff_RE, fig.height=3---------------------------------------------------------
(cert_releff_RE <- relative_effects(cert_fit_RE, newdata = newdata))
plot(cert_releff_RE, ref_line = 0)


## ----certolizumab_releff_study_RE-----------------------------------------------------------------
(cert_releff_study_RE <- relative_effects(cert_fit_RE))


## -------------------------------------------------------------------------------------------------
predict(cert_fit_RE, baseline = distr(qnorm, mean = cert_fit_RE$xbar[[".mu"]], sd = 0.5))


## -------------------------------------------------------------------------------------------------
predict(cert_fit_RE)


## ----certolizumab_ranks---------------------------------------------------------------------------
(cert_ranks <- posterior_ranks(cert_fit_RE, newdata = newdata,
                               lower_better = FALSE))
plot(cert_ranks)

## ----certolizumab_rankprobs-----------------------------------------------------------------------
(cert_rankprobs <- posterior_rank_probs(cert_fit_RE, newdata = newdata,
                                        lower_better = FALSE))
plot(cert_rankprobs)

## ----certolizumab_cumrankprobs--------------------------------------------------------------------
(cert_cumrankprobs <- posterior_rank_probs(cert_fit_RE, cumulative = TRUE,
                                           newdata = newdata, lower_better = FALSE))
plot(cert_cumrankprobs)


## ----eval=!params$run_tests-----------------------------------------------------------------------
# nma(cert_net,
#     trt_effects = "fixed",
#     regression = ~(disease_duration + .mu):.trt,
#     prior_intercept = normal(scale = sqrt(1000)),
#     prior_trt = normal(scale = 100),
#     prior_reg = normal(scale = 100),
#     adapt_delta = 0.95)

## ----echo=FALSE, eval=params$run_tests------------------------------------------------------------
nowarn_on_ci(nma(cert_net,
    trt_effects = "fixed",
    regression = ~(disease_duration + .mu):.trt,
    prior_intercept = normal(scale = sqrt(1000)),
    prior_trt = normal(scale = 100),
    prior_reg = normal(scale = 100),
    adapt_delta = 0.95))


## ----certolizumab_tests, include=FALSE, eval=params$run_tests-------------------------------------
#--- Test against TSD 3 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# FE Relative effects
tsd_FE <- tribble(
~trt          , ~mean, ~sd , ~lower, ~upper,
"CZP"         , 1.85 , 0.10, 1.67  , 2.06  ,
"Adalimumab"  , 2.13 , 0.11, 1.90  , 2.35  ,
"Etanercept"  , 2.08 , 0.34, 1.47  , 2.80  ,
"Infliximab"  , 1.68 , 0.10, 1.49  , 1.86  ,
"Rituximab"   , 0.36 , 0.50, -0.72 , 1.27  ,
"Tocilizumab" , 2.20 , 0.14, 1.93  , 2.46  ,
) %>%
  arrange(ordered(trt, levels = levels(cert_net$treatments)))

cert_releff_FE <- as.data.frame(cert_releff_FE)

test_that("FE relative effects", {
  expect_equivalent(cert_releff_FE$mean, tsd_FE$mean, tolerance = tol)
  expect_equivalent(cert_releff_FE$sd, tsd_FE$sd, tolerance = tol)
  expect_equivalent(cert_releff_FE$`2.5%`, tsd_FE$lower, tolerance = tol)
  expect_equivalent(cert_releff_FE$`97.5%`, tsd_FE$upper, tolerance = tol)
})

# RE Relative effects
tsd_RE <- tribble(
~trt          , ~mean, ~sd , ~lower, ~upper,
"CZP"         , 1.83 , 0.24, 1.35  , 2.29  ,
"Adalimumab"  , 2.18 , 0.22, 1.79  , 2.63  ,
"Etanercept"  , 2.04 , 0.46, 1.19  , 2.94  ,
"Infliximab"  , 1.71 , 0.22, 1.30  , 2.16  ,
"Rituximab"   , 0.37 , 0.59, -0.86 , 1.45  ,
"Tocilizumab" , 2.25 , 0.27, 1.75  , 2.79  ,
) %>%
  arrange(ordered(trt, levels = levels(cert_net$treatments)))

cert_releff_RE <- as.data.frame(cert_releff_RE)

test_that("RE relative effects", {
  expect_equivalent(cert_releff_RE$mean, tsd_RE$mean, tolerance = tol)
  expect_equivalent(cert_releff_RE$sd, tsd_RE$sd, tolerance = tol)
  expect_equivalent(cert_releff_RE$`2.5%`, tsd_RE$lower, tolerance = tol)
  expect_equivalent(cert_releff_RE$`97.5%`, tsd_RE$upper, tolerance = tol)
})

cert_fit_RE_mu <- summary(cert_fit_RE, pars = "mu")
test_that("mean study-specific intercepts identical to those in parameter summary", {
  expect_equal(cert_releff_study_RE$studies$.mu, unname(cert_fit_RE_mu$summary$mean))
})

# Heterogeneity SD
cert_tau <- as.data.frame(summary(cert_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(cert_tau$`50%`, 0.19, tolerance = tol)
  expect_equivalent(cert_tau$sd, 0.19, tolerance = tol)
  expect_equivalent(cert_tau$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(cert_tau$`97.5%`, 0.70, tolerance = tol*2)
})

# Baseline risk meta-regression
expect_equal(cert_fit_FE$xbar[[".mu"]], -2.421, tolerance = 0.01)
expect_equal(cert_fit_RE$xbar[[".mu"]], -2.421, tolerance = 0.01)

cert_beta_FE <- as.data.frame(summary(cert_fit_FE, pars = "beta"))

test_that("FE baseline risk meta-regression", {
  expect_equal(cert_beta_FE$mean, -0.93, tolerance = tol)
  expect_equal(cert_beta_FE$sd, 0.09, tolerance = tol)
  expect_equal(cert_beta_FE$`2.5%`, -1.03, tolerance = tol)
  expect_equal(cert_beta_FE$`97.5%`, -0.69, tolerance = tol)
})

cert_beta_RE <- as.data.frame(summary(cert_fit_RE, pars = "beta"))

test_that("RE baseline risk meta-regression", {
  expect_equal(cert_beta_RE$mean, -0.95, tolerance = tol)
  expect_equal(cert_beta_RE$sd, 0.10, tolerance = tol)
  expect_equal(cert_beta_RE$`2.5%`, -1.10, tolerance = tol)
  expect_equal(cert_beta_RE$`97.5%`, -0.70, tolerance = tol)
})

# DIC
test_that("FE DIC", {
  expect_equivalent(dic_FE$resdev, 27.3, tolerance = tol_dic)
  expect_equivalent(dic_FE$pd, 19.0, tolerance = tol_dic)
  expect_equivalent(dic_FE$dic, 46.3, tolerance = tol_dic)
})

test_that("RE DIC", {
  expect_equivalent(dic_RE$resdev, 24.2, tolerance = tol_dic)
  #expect_equivalent(dic_RE$pd, 19.4, tolerance = tol_dic)
  #expect_equivalent(dic_RE$dic, 43.6, tolerance = tol_dic)
  expect_equivalent(dic_RE$pd, 21.5, tolerance = tol_dic)
  expect_equivalent(dic_RE$dic, 45.8, tolerance = tol_dic)
})


# Force clean up
rm(list = ls())
gc()

# Generated by vignette example_atrial_fibrillation.Rmd: do not edit by hand
# Instead edit example_atrial_fibrillation.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE-----------------------------------------------------------------

## ----include=FALSE-----------------------------------------------------------------------------------------------------------
set.seed(4783982)


## ----eval = FALSE------------------------------------------------------------------------------------------------------------
# library(multinma)
# options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE-----------------------------------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## ----------------------------------------------------------------------------------------------------------------------------
head(atrial_fibrillation)


## ----------------------------------------------------------------------------------------------------------------------------
af_net <- set_agd_arm(atrial_fibrillation[atrial_fibrillation$studyc != "WASPO", ], 
                      study = studyc,
                      trt = trtc,
                      r = r, 
                      n = n,
                      trt_class = trt_class)
af_net


## ----af_network_plot, fig.width=8, fig.height=6, out.width="100%"------------------------------------------------------------
plot(af_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE) + 
  ggplot2::theme(legend.position = "bottom", legend.box = "vertical")


## ----------------------------------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=FALSE--------------------------------------------------------------------------------------------------------------
# af_fit_1 <- nma(af_net,
#                 trt_effects = "random",
#                 prior_intercept = normal(scale = 100),
#                 prior_trt = normal(scale = 100),
#                 prior_het = half_normal(scale = 5),
#                 adapt_delta = 0.99)

## ----echo=FALSE--------------------------------------------------------------------------------------------------------------
af_fit_1 <- nma(af_net, 
                seed = 103533305,
                trt_effects = "random",
                prior_intercept = normal(scale = 100),
                prior_trt = normal(scale = 100),
                prior_het = half_normal(scale = 5),
                adapt_delta = 0.99,
                iter = 5000)


## ----------------------------------------------------------------------------------------------------------------------------
af_fit_1


## ----eval=FALSE--------------------------------------------------------------------------------------------------------------
# # Not run
# print(af_fit_1, pars = c("d", "mu", "delta"))


## ----af_1_pp_plot, fig.width=8, fig.height=6, out.width="100%"---------------------------------------------------------------
plot_prior_posterior(af_fit_1, prior = c("trt", "het"))


## ----------------------------------------------------------------------------------------------------------------------------
(af_1_releff <- relative_effects(af_fit_1, trt_ref = "Placebo/Standard care"))


## ----af_1_releff_plot--------------------------------------------------------------------------------------------------------
plot(af_1_releff, ref_line = 0)


## ----af_1_ranks--------------------------------------------------------------------------------------------------------------
(af_1_ranks <- posterior_ranks(af_fit_1))
plot(af_1_ranks)

## ----af_1_rankprobs----------------------------------------------------------------------------------------------------------
(af_1_rankprobs <- posterior_rank_probs(af_fit_1))
plot(af_1_rankprobs)

## ----af_1_cumrankprobs-------------------------------------------------------------------------------------------------------
(af_1_cumrankprobs <- posterior_rank_probs(af_fit_1, cumulative = TRUE))
plot(af_1_cumrankprobs)


## ----eval=FALSE--------------------------------------------------------------------------------------------------------------
# af_fit_4b <- nma(af_net,
#                  trt_effects = "random",
#                  regression = ~ .trt:stroke,
#                  class_interactions = "common",
#                  QR = TRUE,
#                  prior_intercept = normal(scale = 100),
#                  prior_trt = normal(scale = 100),
#                  prior_reg = normal(scale = 100),
#                  prior_het = half_normal(scale = 5),
#                  adapt_delta = 0.99)

## ----echo=FALSE, eval=!params$run_tests--------------------------------------------------------------------------------------
# af_fit_4b <- nma(af_net,
#                  seed = 579212814,
#                  trt_effects = "random",
#                  regression = ~ .trt:stroke,
#                  class_interactions = "common",
#                  QR = TRUE,
#                  prior_intercept = normal(scale = 100),
#                  prior_trt = normal(scale = 100),
#                  prior_reg = normal(scale = 100),
#                  prior_het = half_normal(scale = 5),
#                  adapt_delta = 0.99)

## ----echo=FALSE, eval=params$run_tests---------------------------------------------------------------------------------------
af_fit_4b <- nowarn_on_ci(nma(af_net, 
                 seed = 579212814,
                 trt_effects = "random",
                 regression = ~ .trt:stroke,
                 class_interactions = "common",
                 QR = TRUE,
                 prior_intercept = normal(scale = 100),
                 prior_trt = normal(scale = 100),
                 prior_reg = normal(scale = 100),
                 prior_het = half_normal(scale = 5),
                 adapt_delta = 0.99,
                 iter = 5000))


## ----------------------------------------------------------------------------------------------------------------------------
af_fit_4b


## ----eval=FALSE--------------------------------------------------------------------------------------------------------------
# # Not run
# print(af_fit_4b, pars = c("d", "mu", "delta"))


## ----af_4b_pp_plot-----------------------------------------------------------------------------------------------------------
plot_prior_posterior(af_fit_4b, prior = c("reg", "het"))


## ----af_4b_releff_plot, fig.height = 16, eval=FALSE--------------------------------------------------------------------------
# # Not run
# (af_4b_releff <- relative_effects(af_fit_4b, trt_ref = "Placebo/Standard care"))
# plot(af_4b_releff, ref_line = 0)


## ----af_4b_releff_01_plot----------------------------------------------------------------------------------------------------
(af_4b_releff_01 <- relative_effects(af_fit_4b, 
                                     trt_ref = "Placebo/Standard care",
                                     newdata = data.frame(stroke = c(0, 1), 
                                                          label = c("stroke = 0", "stroke = 1")),
                                     study = label))
plot(af_4b_releff_01, ref_line = 0)


## ----af_4b_betas-------------------------------------------------------------------------------------------------------------
plot(af_fit_4b, pars = "beta", stat = "halfeye", ref_line = 0)


## ----af_4b_betas_transformed-------------------------------------------------------------------------------------------------
af_4b_beta <- as.array(af_fit_4b, pars = "beta")

# Subtract beta[Control:stroke] from the other class interactions
af_4b_beta[ , , 2:3] <- sweep(af_4b_beta[ , , 2:3], 1:2, 
                              af_4b_beta[ , , "beta[.trtclassControl:stroke]"], FUN = "-")

# Set beta[Anti-coagulant:stroke] = -beta[Control:stroke]
af_4b_beta[ , , "beta[.trtclassControl:stroke]"] <- -af_4b_beta[ , , "beta[.trtclassControl:stroke]"]
names(af_4b_beta)[1] <- "beta[.trtclassAnti-coagulant:stroke]"

# Summarise
summary(af_4b_beta)
plot(summary(af_4b_beta), stat = "halfeye", ref_line = 0)


## ----af_4b_ranks-------------------------------------------------------------------------------------------------------------
(af_4b_ranks <- posterior_ranks(af_fit_4b,
                                newdata = data.frame(stroke = c(0, 1), 
                                                     label = c("stroke = 0", "stroke = 1")), 
                                study = label))
plot(af_4b_ranks)

## ----af_4b_rankprobs, fig.height=12------------------------------------------------------------------------------------------
(af_4b_rankprobs <- posterior_rank_probs(af_fit_4b,
                                         newdata = data.frame(stroke = c(0, 1), 
                                                              label = c("stroke = 0", "stroke = 1")), 
                                         study = label))

# Modify the default output with ggplot2 functionality
library(ggplot2)
plot(af_4b_rankprobs) + 
  facet_grid(Treatment~Study, labeller = label_wrap_gen(20)) + 
  theme(strip.text.y = element_text(angle = 0))

## ----af_4b_cumrankprobs, fig.height=12---------------------------------------------------------------------------------------
(af_4b_cumrankprobs <- posterior_rank_probs(af_fit_4b, cumulative = TRUE,
                                            newdata = data.frame(stroke = c(0, 1), 
                                                                 label = c("stroke = 0", "stroke = 1")), 
                                            study = label))

plot(af_4b_cumrankprobs) + 
  facet_grid(Treatment~Study, labeller = label_wrap_gen(20)) + 
  theme(strip.text.y = element_text(angle = 0))


## ----------------------------------------------------------------------------------------------------------------------------
(af_dic_1 <- dic(af_fit_1))

## ----------------------------------------------------------------------------------------------------------------------------
(af_dic_4b <- dic(af_fit_4b))


## ----af_1_resdev_plot--------------------------------------------------------------------------------------------------------
plot(af_dic_1)


## ----af_4b_resdev_plot-------------------------------------------------------------------------------------------------------
plot(af_dic_4b)


## ----atrial_fibrillation_tests, include=FALSE, eval=params$run_tests---------------------------------------------------------
#--- Test against TSD 2 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# No covariates

Cooper_1_releff <- tribble(
~trt                                       , ~est , ~lower, ~upper,
"Low adjusted dose anti-coagulant"         , -1.08,-1.77  , -0.37 ,
"Standard adjusted dose anti-coagulant"    , -0.76,-1.16  , -0.36 ,
"Fixed dose warfarin"                      , 0.18 ,-0.73  , 1.06  ,
"Low dose aspirin"                         , -0.15,-0.56  , 0.27  ,
"Medium dose aspirin"                      , -0.37,-0.83  , 0.07  ,
"High dose aspirin"                        , -0.25,-1.72  , 1.23  ,
"Alternate day aspirin"                    , -1.67,-4.54  , 0.41  ,
"Ximelagatran"                             , -0.84,-1.50  , -0.18 ,
"Triflusal"                                , -0.11,-1.35  , 1.20  ,
"Indobufen"                                , -0.52,-1.47  , 0.47  ,
"Dipyridamole"                             , -0.18,-1.02  , 0.66  ,
"Fixed dose warfarin + low dose aspirin"   , -0.29,-1.09  , 0.51  ,
"Fixed dose warfarin + medium dose aspirin", 0.13 ,-0.60  , 0.83  ,
"Acenocoumarol"                            , -1.56,-3.31  , 0.06  ,
"Low dose aspirin + copidogrel"            , -0.24,-1.06  , 0.57  ,
"Low dose aspirin + dipyridamole"          , -0.49,-1.38  , 0.38  ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(af_net$treatments))) %>%
  arrange(trt)

af_1_releff_df <- as.data.frame(af_1_releff)

test_that("Relative effects (no covariates)", {
  expect_equivalent(af_1_releff_df$mean, Cooper_1_releff$est, tolerance = tol)
  expect_equivalent(af_1_releff_df$`2.5%`, Cooper_1_releff$lower, tolerance = tol)
  expect_equivalent(af_1_releff_df$`97.5%`, Cooper_1_releff$upper, tolerance = tol)
})

af_1_tau <- as.data.frame(summary(af_fit_1, pars = "tau"))

test_that("Heterogeneity SD (no covariates)", {
  expect_equivalent(af_1_tau$`50%`, 0.28, tolerance = tol)
  expect_equivalent(af_1_tau$`2.5%`, 0.02, tolerance = tol)
  expect_equivalent(af_1_tau$`97.5%`, 0.57, tolerance = tol)
})

test_that("DIC (no covariates)", {
  expect_equivalent(af_dic_1$resdev, 60.22, tolerance = tol_dic)
  expect_equivalent(af_dic_1$pd, 48.35, tolerance = tol_dic)
  expect_equivalent(af_dic_1$dic, 108.57, tolerance = tol_dic)
})

test_that("SUCRAs", {
  af_ranks_1 <- posterior_ranks(af_fit_1, sucra = TRUE)
  af_rankprobs_1 <- posterior_rank_probs(af_fit_1, sucra = TRUE)
  af_cumrankprobs_1 <- posterior_rank_probs(af_fit_1, cumulative = TRUE, sucra = TRUE)
  
  expect_equal(af_ranks_1$summary$sucra, af_rankprobs_1$summary$sucra)
  expect_equal(af_ranks_1$summary$sucra, af_cumrankprobs_1$summary$sucra)
})


# Check construction of all contrasts
af_1_releff_all_contr <- relative_effects(af_fit_1, all_contrasts = TRUE)

# Reconstruct from basic contrasts in each study
dk <- function(study, trt, sims) {
  if (trt == "Placebo/Standard care") return(0)
  else if (is.na(study)) {
    return(sims[ , , paste0("d[", trt, "]"), drop = FALSE])
  } else {
    return(sims[ , , paste0("d[", study, ": ", trt, "]"), drop = FALSE])
  }
}

test_af_1_all_contr <- tibble(
  contr = af_1_releff_all_contr$summary$parameter,
  .trtb = factor(stringr::str_extract(contr, "(?<=\\[)(.+)(?= vs\\.)"), levels = levels(af_net$treatments)),
  .trta = factor(stringr::str_extract(contr, "(?<=vs\\. )(.+)(?=\\])"), levels = levels(af_net$treatments))
) %>%
  rowwise() %>%
  mutate(as_tibble(multinma:::summary.mcmc_array(dk(NA, .trtb, af_1_releff$sims) - dk(NA, .trta, af_1_releff$sims)))) %>%
  select(.trtb, .trta, parameter = contr, mean:Rhat)

test_that("Construction of all contrasts is correct (no covariates)", {
  ntrt <- nlevels(af_net$treatments)
  expect_equal(nrow(af_1_releff_all_contr$summary), ntrt * (ntrt - 1) / 2)
  expect_equal(select(af_1_releff_all_contr$summary, -Rhat),
               select(test_af_1_all_contr, -Rhat),
               check.attributes = FALSE)
})

# With stroke covariate, shared interactions

Cooper_4b_releff <- tribble(
~trt                                       , ~est  , ~lower, ~upper, 
"Low adjusted dose anti-coagulant"         , -1.20 ,-1.89  , -0.54 ,
"Standard adjusted dose anti-coagulant"    , -0.77 ,-1.14  , -0.38 ,
"Fixed dose warfarin"                      , -0.11 ,-0.90  , 0.72  ,
"Low dose aspirin"                         , -0.08 ,-0.47  , 0.30  ,
"Medium dose aspirin"                      , -0.45 ,-0.87  , -0.03 ,
"High dose aspirin"                        , -0.39 ,-1.86  , 1.11  ,
"Alternate day aspirin"                    , -1.74 ,-5.16  , 0.48  ,
"Ximelagatran"                             , -0.86 ,-1.42  , -0.27 ,
"Triflusal"                                , 0.13  ,-1.05  , 1.38  ,
"Indobufen"                                , -1.21 ,-2.26  , -0.13 ,
"Dipyridamole"                             , -0.21 ,-1.01  , 0.58  ,
"Fixed dose warfarin + low dose aspirin"   , 0.54  ,-0.80  , 1.85  ,
"Fixed dose warfarin + medium dose aspirin", 0.12  ,-0.53  , 0.80  ,
"Acenocoumarol"                            , -0.534,-2.67  , 1.38  ,
"Low dose aspirin + copidogrel"            , -0.14 ,-0.82  , 0.53  ,
"Low dose aspirin + dipyridamole"          , -0.53 ,-1.38  , 0.30  ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(af_net$treatments))) %>%
  arrange(trt)

af_4b_releff_Cooper <- as.data.frame(relative_effects(af_fit_4b, 
                                                      newdata = tibble(stroke = 0.27),
                                                      trt_ref = "Placebo/Standard care"))

test_that("Relative effects (common interaction)", {
  expect_equivalent(af_4b_releff_Cooper$mean, Cooper_4b_releff$est, tolerance = tol)
  expect_equivalent(af_4b_releff_Cooper$`2.5%`, Cooper_4b_releff$lower, tolerance = tol)
  expect_equivalent(af_4b_releff_Cooper$`97.5%`, Cooper_4b_releff$upper, tolerance = tol)
})

Cooper_4b_beta <- tribble(
~trt_class       , ~est , ~lower, ~upper,
"Anti-coagulant", -0.71,-1.58  , 0.15  ,
"Anti-platelet" , 0.23 ,-0.45  , 0.93  ,
#"Mixed"          , 3.05 ,-1.26  , 7.30  ,
"Mixed"          , 3.21 ,-0.91  , 7.30  ,
)

af_4b_beta_df <- as.data.frame(summary(af_4b_beta))

test_that("Interaction estimates (common interaction)", {
  expect_equivalent(af_4b_beta_df$mean, Cooper_4b_beta$est, tolerance = tol)
  skip_on_ci()
  expect_equivalent(af_4b_beta_df$`2.5%`, Cooper_4b_beta$lower, tolerance = tol)
  expect_equivalent(af_4b_beta_df$`97.5%`, Cooper_4b_beta$upper, tolerance = tol)
})

af_4b_tau <- as.data.frame(summary(af_fit_4b, pars = "tau"))

test_that("Heterogeneity SD (common interaction)", {
  expect_equivalent(af_4b_tau$`50%`, 0.19, tolerance = tol)
  expect_equivalent(af_4b_tau$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(af_4b_tau$`97.5%`, 0.48, tolerance = tol)
})

test_that("DIC (common interaction)", {
  expect_equivalent(af_dic_4b$resdev, 58.74, tolerance = tol_dic)
  expect_equivalent(af_dic_4b$pd, 48.25, tolerance = tol_dic)
  expect_equivalent(af_dic_4b$dic, 106.99, tolerance = tol_dic)
})

test_that("SUCRAs", {
  stroke_01 <- data.frame(stroke = c(0, 1), label = c("stroke = 0", "stroke = 1"))
  af_ranks_4b <- posterior_ranks(af_fit_4b, newdata = stroke_01, 
                                study = label, sucra = TRUE)
  af_rankprobs_4b <- posterior_rank_probs(af_fit_4b, newdata = stroke_01, 
                                           study = label, sucra = TRUE)
  af_cumrankprobs_4b <- posterior_rank_probs(af_fit_4b, cumulative = TRUE, newdata = stroke_01,
                                              study = label, sucra = TRUE)
  
  expect_equal(af_ranks_4b$summary$sucra, af_rankprobs_4b$summary$sucra)
  expect_equal(af_ranks_4b$summary$sucra, af_cumrankprobs_4b$summary$sucra)
})

# Check construction of all contrasts
af_4b_releff <- relative_effects(af_fit_4b, trt_ref = "Placebo/Standard care")
af_4b_releff_all_contr <- relative_effects(af_fit_4b, all_contrasts = TRUE)

test_af_4b_all_contr <- tibble(
  contr = af_4b_releff_all_contr$summary$parameter,
  .study = factor(stringr::str_extract(contr, "(?<=\\[)(.+)(?=:)")),
  .trtb = factor(stringr::str_extract(contr, "(?<=\\: )(.+)(?= vs\\.)"), levels = levels(af_net$treatments)),
  .trta = factor(stringr::str_extract(contr, "(?<=vs\\. )(.+)(?=\\])"), levels = levels(af_net$treatments))
) %>% 
  rowwise() %>% 
  mutate(as_tibble(multinma:::summary.mcmc_array(dk(.study, .trtb, af_4b_releff$sims) - dk(.study, .trta, af_4b_releff$sims)))) %>% 
  select(.study, .trtb, .trta, parameter = contr, mean:Rhat)

test_that("Construction of all contrasts is correct (common interaction)", {
  ntrt <- nlevels(af_net$treatments)
  nstudy <- nlevels(test_af_4b_all_contr$.study)
  expect_equal(nrow(af_4b_releff_all_contr$summary), nstudy * ntrt * (ntrt - 1) / 2)
  expect_equal(select(af_4b_releff_all_contr$summary, -Rhat), 
               select(test_af_4b_all_contr, -Rhat), 
               check.attributes = FALSE)
})

# Check construction of all contrasts in target population
af_4b_releff_new <- relative_effects(af_fit_4b, newdata = tibble(stroke = 0.27), trt_ref = "Placebo/Standard care")
af_4b_releff_all_contr_new <- relative_effects(af_fit_4b, newdata = tibble(stroke = 0.27), all_contrasts = TRUE)

test_af_4b_all_contr_new <- tibble(
  contr = af_4b_releff_all_contr_new$summary$parameter,
  .study = factor(stringr::str_extract(contr, "(?<=\\[)(.+)(?=:)")),
  .trtb = factor(stringr::str_extract(contr, "(?<=\\: )(.+)(?= vs\\.)"), levels = levels(af_net$treatments)),
  .trta = factor(stringr::str_extract(contr, "(?<=vs\\. )(.+)(?=\\])"), levels = levels(af_net$treatments))
) %>% 
  rowwise() %>% 
  mutate(as_tibble(multinma:::summary.mcmc_array(dk(.study, .trtb, af_4b_releff_new$sims) - dk(.study, .trta, af_4b_releff_new$sims)))) %>% 
  select(.study, .trtb, .trta, parameter = contr, mean:Rhat)

test_that("Construction of all contrasts in target population is correct (common interaction)", {
  ntrt <- nlevels(af_net$treatments)
  nstudy <- nlevels(test_af_4b_all_contr_new$.study)
  expect_equal(nrow(af_4b_releff_all_contr_new$summary), nstudy * ntrt * (ntrt - 1) / 2)
  expect_equal(select(af_4b_releff_all_contr_new$summary, -Rhat), 
               select(test_af_4b_all_contr_new, -Rhat), 
               check.attributes = FALSE)
})

test_that("Robust to custom options(contrasts) settings", {
  af_fit_4b_SAS <- withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                                       unordered = "contr.SAS")),
             nowarn_on_ci(nma(af_net, 
                 seed = 579212814,
                 trt_effects = "random",
                 regression = ~ .trt:stroke,
                 class_interactions = "common",
                 QR = TRUE,
                 prior_intercept = normal(scale = 100),
                 prior_trt = normal(scale = 100),
                 prior_reg = normal(scale = 100),
                 prior_het = half_normal(scale = 5),
                 adapt_delta = 0.99,
                 iter = 5000)))

  expect_equal(as_tibble(summary(af_fit_4b_SAS))[, c("parameter", "mean", "sd")],
               as_tibble(summary(af_fit_4b))[, c("parameter", "mean", "sd")],
               tolerance = tol)
  expect_equal(as_tibble(relative_effects(af_fit_4b_SAS))[, c("parameter", "mean", "sd")],
               as_tibble(relative_effects(af_fit_4b))[, c("parameter", "mean", "sd")],
               tolerance = tol)
})



# Force clean up
rm(list = ls())
gc()

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Only test gradients, no sampling
smk_fit_RE <- nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5),
                  test_grad = TRUE)


test_that("probs argument", {
  m <- "numeric vector of probabilities"
  expect_error(posterior_ranks(smk_fit_RE, probs = "a"), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = -1), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = 1.5), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = Inf), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = list()), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = NA), m)
  expect_error(posterior_ranks(smk_fit_RE, probs = NULL), m)
})

test_that("summary argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_ranks(smk_fit_RE, summary = "a"), m)
  expect_error(posterior_ranks(smk_fit_RE, summary = 1), m)
  expect_error(posterior_ranks(smk_fit_RE, summary = list()), m)
  expect_error(posterior_ranks(smk_fit_RE, summary = NA), m)
  expect_error(posterior_ranks(smk_fit_RE, summary = NULL), m)
})

test_that("sucra argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_ranks(smk_fit_RE, sucra = "a"), m)
  expect_error(posterior_ranks(smk_fit_RE, sucra = 1), m)
  expect_error(posterior_ranks(smk_fit_RE, sucra = list()), m)
  expect_error(posterior_ranks(smk_fit_RE, sucra = NA), m)
  expect_error(posterior_ranks(smk_fit_RE, sucra = NULL), m)
})

test_that("newdata argument", {
  m <- "not a data frame"
  expect_error(posterior_ranks(smk_fit_RE, newdata = "a"), m)
  expect_error(posterior_ranks(smk_fit_RE, newdata = 1), m)
  expect_error(posterior_ranks(smk_fit_RE, newdata = list()), m)
  expect_error(posterior_ranks(smk_fit_RE, newdata = NA), m)
})

test_that("lower_better argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_ranks(smk_fit_RE, lower_better = "a"), m)
  expect_error(posterior_ranks(smk_fit_RE, lower_better = 1), m)
  expect_error(posterior_ranks(smk_fit_RE, lower_better = list()), m)
  expect_error(posterior_ranks(smk_fit_RE, lower_better = NA), m)
  expect_error(posterior_ranks(smk_fit_RE, lower_better = NULL), m)
})

test_that("newdata argument", {
  m <- "not a data frame"
  expect_error(posterior_rank_probs(smk_fit_RE, newdata = "a"), m)
  expect_error(posterior_rank_probs(smk_fit_RE, newdata = 1), m)
  expect_error(posterior_rank_probs(smk_fit_RE, newdata = list()), m)
  expect_error(posterior_rank_probs(smk_fit_RE, newdata = NA), m)
})

test_that("lower_better argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_rank_probs(smk_fit_RE, lower_better = "a"), m)
  expect_error(posterior_rank_probs(smk_fit_RE, lower_better = 1), m)
  expect_error(posterior_rank_probs(smk_fit_RE, lower_better = list()), m)
  expect_error(posterior_rank_probs(smk_fit_RE, lower_better = NA), m)
  expect_error(posterior_rank_probs(smk_fit_RE, lower_better = NULL), m)
})

test_that("cumulative argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_rank_probs(smk_fit_RE, cumulative = "a"), m)
  expect_error(posterior_rank_probs(smk_fit_RE, cumulative = 1), m)
  expect_error(posterior_rank_probs(smk_fit_RE, cumulative = list()), m)
  expect_error(posterior_rank_probs(smk_fit_RE, cumulative = NA), m)
  expect_error(posterior_rank_probs(smk_fit_RE, cumulative = NULL), m)
})

test_that("sucra argument", {
  m <- "should be TRUE or FALSE"
  expect_error(posterior_rank_probs(smk_fit_RE, sucra = "a"), m)
  expect_error(posterior_rank_probs(smk_fit_RE, sucra = 1), m)
  expect_error(posterior_rank_probs(smk_fit_RE, sucra = list()), m)
  expect_error(posterior_rank_probs(smk_fit_RE, sucra = NA), m)
  expect_error(posterior_rank_probs(smk_fit_RE, sucra = NULL), m)
})

skip_on_cran()  # Reduce CRAN check time

# Only small number of samples to test
smk_fit_RE <- suppressWarnings(nma(smk_net,
                                   trt_effects = "random",
                                   prior_intercept = normal(scale = 100),
                                   prior_trt = normal(scale = 100),
                                   prior_het = normal(scale = 5),
                                   iter = 10))

test_that(".trt column is correct", {
  rk <- tibble::as_tibble(posterior_ranks(smk_fit_RE))
  expect_identical(paste0("rank[", rk$.trt, "]"),
                   rk$parameter)

  rp <- tibble::as_tibble(posterior_rank_probs(smk_fit_RE))
  expect_identical(paste0("d[", rp$.trt, "]"),
                   rp$parameter)
})


pso_net <- set_ipd(plaque_psoriasis_ipd[complete.cases(plaque_psoriasis_ipd), ],
                   studyc, trtc,
                   r = pasi75)

# Only small number of samples to test
pso_fit <- suppressWarnings(nma(pso_net,
                                trt_effects = "fixed",
                                regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
                                prior_intercept = normal(scale = 10),
                                prior_trt = normal(scale = 10),
                                prior_reg = normal(scale = 10),
                                init_r = 0.1,
                                iter = 10))

test_that(".study, .trt columns are correct", {
  rk <- tibble::as_tibble(posterior_ranks(pso_fit))
  expect_identical(paste0("rank[", rk$.study, ": ", rk$.trt, "]"),
                   rk$parameter)

  rp <- tibble::as_tibble(posterior_rank_probs(pso_fit))
  expect_identical(paste0("d[", rp$.study, ": ", rp$.trt, "]"),
                   rp$parameter)
})

# Generated by vignette example_thrombolytics.Rmd: do not edit by hand
# Instead edit example_thrombolytics.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(thrombolytics)


## -------------------------------------------------------------------------------------------------
thrombo_net <- set_agd_arm(thrombolytics, 
                           study = studyn,
                           trt = trtc,
                           r = r, 
                           n = n)
thrombo_net

## ----include=FALSE, eval=params$run_tests---------------------------------------------------------
# Make trtf factor to order treatments in same way as Dias analysis - needed to
# recreate inconsistency analyses
trts <- dplyr::distinct(thrombolytics, trtn, trtc)
trts <- dplyr::arrange(trts, trtn)
thrombolytics$trtf <- factor(thrombolytics$trtn, levels = trts$trtn, labels = trts$trtc)
thrombo_net2 <- set_agd_arm(thrombolytics, 
                            study = studyn,
                            trt = trtf,
                            r = r, 
                            n = n)


## ----eval=FALSE-----------------------------------------------------------------------------------
## plot(thrombo_net, weight_edges = TRUE, weight_nodes = TRUE)

## ----thrombo_net_plot, echo=FALSE-----------------------------------------------------------------
plot(thrombo_net, weight_edges = TRUE, weight_nodes = TRUE) + ggplot2::theme(legend.margin = ggplot2::margin(l = 4, unit = "lines"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## -------------------------------------------------------------------------------------------------
thrombo_fit <- nma(thrombo_net, 
                   trt_effects = "fixed",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100))


## -------------------------------------------------------------------------------------------------
thrombo_fit


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(thrombo_fit, pars = c("d", "mu"))


## ----thrombo_pp_plot------------------------------------------------------------------------------
plot_prior_posterior(thrombo_fit, prior = "trt")


## -------------------------------------------------------------------------------------------------
(dic_consistency <- dic(thrombo_fit))


## ----thrombo_resdev_plot, fig.width=12------------------------------------------------------------
plot(dic_consistency)


## -------------------------------------------------------------------------------------------------
thrombo_fit_ume <- nma(thrombo_net, 
                       consistency = "ume",
                       trt_effects = "fixed",
                       prior_intercept = normal(scale = 100),
                       prior_trt = normal(scale = 100))
thrombo_fit_ume


## -------------------------------------------------------------------------------------------------
dic_consistency
(dic_ume <- dic(thrombo_fit_ume))


## ----thrombo_devdev_plot--------------------------------------------------------------------------
plot(dic_consistency, dic_ume, show_uncertainty = FALSE)


## ----eval=!params$run_tests-----------------------------------------------------------------------
## thrombo_nodesplit <- nma(thrombo_net,
##                          consistency = "nodesplit",
##                          trt_effects = "fixed",
##                          prior_intercept = normal(scale = 100),
##                          prior_trt = normal(scale = 100))

## ----include=FALSE, eval=params$run_tests---------------------------------------------------------
# Run node-splits with treatments ordered as per Dias
thrombo_nodesplit <- nma(thrombo_net2, 
                         consistency = "nodesplit",
                         trt_effects = "fixed",
                         prior_intercept = normal(scale = 100),
                         prior_trt = normal(scale = 100))


## -------------------------------------------------------------------------------------------------
summary(thrombo_nodesplit)


## ----thrombo_nodesplit, fig.width = 7-------------------------------------------------------------
plot(thrombo_nodesplit)


## ----thrombo_nodesplit_omega, fig.height = 6------------------------------------------------------
plot(thrombo_nodesplit, pars = "omega", stat = "halfeye", ref_line = 0) +
  ggplot2::aes(y = comparison) +
  ggplot2::facet_null()


## ----thrombo_releff-------------------------------------------------------------------------------
(thrombo_releff <- relative_effects(thrombo_fit, all_contrasts = TRUE))
plot(thrombo_releff, ref_line = 0)


## ----thrombo_ranks--------------------------------------------------------------------------------
(thrombo_ranks <- posterior_ranks(thrombo_fit))
plot(thrombo_ranks)

## ----thrombo_rankprobs----------------------------------------------------------------------------
(thrombo_rankprobs <- posterior_rank_probs(thrombo_fit))
plot(thrombo_rankprobs)

## ----thrombo_cumrankprobs-------------------------------------------------------------------------
(thrombo_cumrankprobs <- posterior_rank_probs(thrombo_fit, cumulative = TRUE))
plot(thrombo_cumrankprobs)


## ----thrombo_tests, include=FALSE, eval=params$run_tests------------------------------------------
#--- Test against TSD 4 results ---
library(testthat)
library(dplyr)
library(tidyr)

test_that("Reference trt is SK", {
  expect_equivalent(levels(thrombo_net$treatments)[1], "SK")
})

tol <- 0.05
tol_dic <- 0.1

# Relative effects
tsd_releff <- tribble(
~trt_b    , ~trt       , ~mean , ~sd  , ~lower, ~upper,
"SK"      , "t-PA"     , 0.002 , 0.030, -0.06 , 0.06  ,
"SK"      , "Acc t-PA" , -0.177, 0.043, -0.26 , -0.09 ,
"SK"      , "SK + t-PA", -0.049, 0.046, -0.14 , 0.04  ,
"SK"      , "r-PA"     , -0.124, 0.060, -0.24 , -0.01 ,
"SK"      , "PTCA"     , -0.476, 0.101, -0.67 , -0.28 ,
"SK"      , "UK"       , -0.203, 0.221, -0.64 , 0.23  ,
"SK"      , "ASPAC"    , 0.016 , 0.037, -0.06 , 0.09  ,
"t-PA"    , "PTCA"     , -0.478, 0.104, -0.68 , -0.27 ,
"t-PA"    , "UK"       , -0.206, 0.221, -0.64 , 0.23  ,
"t-PA"    , "ASPAC"    , 0.013 , 0.037, -0.06 , 0.09  ,
"Acc t-PA", "r-PA"     , 0.054 , 0.055, -0.05 , 0.16  ,
"Acc t-PA", "TNK"      , 0.005 , 0.064, -0.12 , 0.13  ,
"Acc t-PA", "PTCA"     , -0.298, 0.098, -0.49 , -0.11 ,
"Acc t-PA", "UK"       , -0.026, 0.221, -0.46 , 0.41  ,
"Acc t-PA", "ASPAC"    , 0.193 , 0.056, 0.08  , 0.30  ) %>% 
  mutate(.trt_b = ordered(trt_b, levels = levels(thrombo_net$treatments)),
         .trt = ordered(trt, levels = levels(thrombo_net$treatments)),
         rev = if_else(.trt_b > .trt, -1, 1),
         .l = lower, .u = upper,
         lower = if_else(.trt_b > .trt, .u, .l),
         upper = if_else(.trt_b > .trt, .l, .u),
         trt_b = if_else(.trt_b > .trt, .trt, .trt_b),
         trt = if_else(.trt_b > .trt, .trt_b, .trt),
         lab = paste0("d[", trt, " vs. ", trt_b, "]")) %>% 
  arrange(trt_b, trt) %>% 
  mutate_at(vars(mean, lower, upper), ~.*rev)

thrombo_releff_summary <- as.data.frame(thrombo_releff) %>% 
  filter(parameter %in% tsd_releff$lab)

test_that("FE relative effects", {
  expect_equivalent(thrombo_releff_summary$mean, tsd_releff$mean, tolerance = tol)
  expect_equivalent(thrombo_releff_summary$sd, tsd_releff$sd, tolerance = tol)
  expect_equivalent(thrombo_releff_summary$`2.5%`, tsd_releff$lower, tolerance = tol)
  expect_equivalent(thrombo_releff_summary$`97.5%`, tsd_releff$upper, tolerance = tol)
})

test_that("SUCRAs", {
  thrombo_ranks <- posterior_ranks(thrombo_fit, sucra = TRUE)
  thrombo_rankprobs <- posterior_rank_probs(thrombo_fit, sucra = TRUE)
  thrombo_cumrankprobs <- posterior_rank_probs(thrombo_fit, cumulative = TRUE, sucra = TRUE)
  
  expect_equal(thrombo_ranks$summary$sucra, thrombo_rankprobs$summary$sucra)
  expect_equal(thrombo_ranks$summary$sucra, thrombo_cumrankprobs$summary$sucra)
})

# DIC
test_that("DIC", {
  expect_equivalent(dic_consistency$resdev, 105.9, tolerance = tol_dic)
  expect_equivalent(dic_consistency$pd, 58, tolerance = tol_dic)
  expect_equivalent(dic_consistency$dic, 163.9, tolerance = tol_dic)
})

# Relative effects (UME)

# FE UME model, so no differences by reference treatment, no multi-arm correction
tsd_ume <- tribble(
~trt_b    , ~trt       , ~mean , ~sd  , ~lower, ~upper,
"SK"      , "t-PA"     , -0.004, 0.030, -0.06 , 0.06  ,
"SK"      , "Acc t-PA" , -0.158, 0.049, -0.25 , -0.06 ,
"SK"      , "SK + t-PA", -0.044, 0.047, -0.14 , 0.05  ,
"SK"      , "r-PA"     , -0.060, 0.089, -0.23 , 0.11  ,
"SK"      , "PTCA"     , -0.665, 0.185, -1.03 , -0.31 ,
"SK"      , "UK"       , -0.369, 0.518, -1.41 , 0.63  ,
"SK"      , "ASPAC"    , 0.005 , 0.037, -0.07 , 0.08  ,
"t-PA"    , "PTCA"     , -0.544, 0.417, -1.38 , 0.25  ,
"t-PA"    , "UK"       , -0.294, 0.347, -0.99 , 0.37  ,
"t-PA"    , "ASPAC"    , -0.290, 0.361, -1.01 , 0.41  ,
"Acc t-PA", "r-PA"     , 0.019 , 0.066, -0.11 , 0.15  ,
"Acc t-PA", "TNK"      , 0.006 , 0.064, -0.12 , 0.13  ,
"Acc t-PA", "PTCA"     , -0.216, 0.119, -0.45 , 0.02  ,
"Acc t-PA", "UK"       , 0.146 , 0.358, -0.54 , 0.86  ,
"Acc t-PA", "ASPAC"    , 1.405 , 0.417, 0.63  , 2.27  ) %>% 
  mutate(.trt_b = ordered(trt_b, levels = levels(thrombo_net$treatments)),
         .trt = ordered(trt, levels = levels(thrombo_net$treatments)),
         rev = if_else(.trt_b > .trt, -1, 1),
         .l = lower, .u = upper,
         lower = if_else(.trt_b > .trt, .u, .l),
         upper = if_else(.trt_b > .trt, .l, .u),
         trt_b = if_else(.trt_b > .trt, .trt, .trt_b),
         trt = if_else(.trt_b > .trt, .trt_b, .trt),
         lab = paste0("d[", trt, " vs. ", trt_b, "]")) %>%
  arrange(trt_b, trt) %>%
  mutate_at(vars(mean, lower, upper), ~.*rev)

thrombo_ume_releff <- summary(thrombo_fit_ume, pars = "d")

test_that("UME relative effects", {
  expect_equivalent(thrombo_ume_releff$summary$mean, tsd_ume$mean, tolerance = tol)
  expect_equivalent(thrombo_ume_releff$summary$sd, tsd_ume$sd, tolerance = tol)
  expect_equivalent(thrombo_ume_releff$summary$`2.5%`, tsd_ume$lower, tolerance = tol)
  expect_equivalent(thrombo_ume_releff$summary$`97.5%`, tsd_ume$upper, tolerance = tol)
})

# DIC (UME)
test_that("UME DIC", {
  expect_equivalent(dic_ume$resdev, 99.7, tolerance = tol_dic)
  expect_equivalent(dic_ume$pd, 65, tolerance = tol_dic)
  expect_equivalent(dic_ume$dic, 164.7, tolerance = tol_dic)
})


# Node-splitting
trt_code <- c("SK", "t-PA", "Acc t-PA", "SK + t-PA", "r-PA",
              "TNK", "PTCA", "UK", "ASPAC")

dias2010_nodesplit_est <- tribble(
  ~trt1, ~trt2, ~net_mean, ~net_sd, ~dir_mean, ~dir_sd, ~ind_mean, ~ind_sd, ~omega_mean, ~omega_sd, ~p_value,
 1, 2,  0.002,  0.030,  0.000,  0.030,  0.189,  0.235, -0.190,  0.236,  0.42,
 1, 3, -0.177,  0.043, -0.158,  0.048, -0.247,  0.092,  0.088,  0.104,  0.39,
 1, 5, -0.124,  0.060, -0.060,  0.089, -0.175,  0.081,  0.115,  0.121,  0.34,
 1, 7, -0.475,  0.101, -0.666,  0.185, -0.393,  0.120, -0.272,  0.222,  0.22,
 1, 8, -0.203,  0.219, -0.369,  0.518, -0.168,  0.244, -0.207,  0.575,  0.73,
 1, 9,  0.016,  0.037,  0.009,  0.037,  0.424,  0.252, -0.413,  0.253,  0.10,
 2, 7, -0.477,  0.104, -0.545,  0.417, -0.475,  0.108, -0.073,  0.432,  0.88,
 2, 8, -0.205,  0.220, -0.295,  0.347, -0.144,  0.290, -0.155,  0.452,  0.74,
 2, 9,  0.014,  0.037,  0.006,  0.037,  0.471,  0.241, -0.468,  0.241,  0.05,
 3, 4,  0.128,  0.054,  0.126,  0.054,  0.630,  0.697, -0.506,  0.696,  0.47,
 3, 5,  0.053,  0.056,  0.019,  0.066,  0.135,  0.101, -0.116,  0.120,  0.34,
 3, 7, -0.298,  0.097, -0.216,  0.118, -0.477,  0.174,  0.260,  0.211,  0.21,
 3, 8, -0.026,  0.220,  0.143,  0.356, -0.136,  0.288,  0.277,  0.461,  0.55,
 3, 9,  0.193,  0.056,  1.409,  0.415,  0.165,  0.057,  1.239,  0.420,  0.001
) %>% 
  mutate(trt1 = forcats::fct_relevel(factor(trt1, levels = 1:9, labels = trt_code), 
                                     !! levels(thrombo_net2$treatments)),
         trt2 = forcats::fct_relevel(factor(trt2, levels = 1:9, labels = trt_code), 
                                     !! levels(thrombo_net2$treatments)))

for (i in 1:nrow(dias2010_nodesplit_est)) {
  if (as.numeric(dias2010_nodesplit_est$trt1[i]) > as.numeric(dias2010_nodesplit_est$trt2[i])) {
    dias2010_nodesplit_est[i, c("trt1", "trt2")] <- dias2010_nodesplit_est[i, c("trt2", "trt1")]
    dias2010_nodesplit_est[i, c("net_mean", "dir_mean", "ind_mean")] <- -dias2010_nodesplit_est[i, c("net_mean", "dir_mean", "ind_mean")]
  }
}

dias2010_nodesplit_est <- arrange(dias2010_nodesplit_est, trt1, trt2)

thrombo_nodesplit_est <- as_tibble(summary(thrombo_nodesplit), nest = FALSE) %>% 
  mutate(parameter = stringr::str_replace(parameter, "(^d_)?(.+)\\[.+\\]$", "\\2")) %>% 
  pivot_wider(names_from = "parameter", 
              values_from = c("mean", "sd"),
              names_glue = "{parameter}_{.value}",
              id_cols = c("trt1", "trt2", "p_value")) %>% 
  mutate(across(where(is.numeric), unname)) %>% 
  select(!!! colnames(dias2010_nodesplit_est))

test_that("Node-splitting estimates", {
  # expect_equal(thrombo_nodesplit_est, dias2010_nodesplit_est, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$net_mean, dias2010_nodesplit_est$net_mean, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$net_sd, dias2010_nodesplit_est$net_sd, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$ind_mean, dias2010_nodesplit_est$ind_mean, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$ind_sd, dias2010_nodesplit_est$ind_sd, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$dir_mean, dias2010_nodesplit_est$dir_mean, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$dir_sd, dias2010_nodesplit_est$dir_sd, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$omega_mean, dias2010_nodesplit_est$omega_mean, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$omega_sd, dias2010_nodesplit_est$omega_sd, tolerance = tol)
  expect_equal(thrombo_nodesplit_est$p_value, dias2010_nodesplit_est$p_value, tolerance = tol)
})

dias2010_nodesplit_dic <- tribble(
  ~trt1, ~trt2, ~resdev,   ~pd,   ~dic,
      1,     2,   106.4,  58.7,  165.1,
      1,     3,   106.2,  58.7,  165.0,
      1,     5,   106.1,  58.7,  164.8,
      1,     7,   105.5,  58.7,  164.2,
      1,     8,   106.9,  58.7,  165.6,
      1,     9,   104.3,  58.7,  163.0,
      2,     7,   106.9,  58.7,  165.6,
      2,     8,   106.8,  58.7,  165.5,
      2,     9,   103.3,  58.7,  162.0,
      3,     4,   106.5,  58.7,  165.2,
      3,     5,   106.1,  58.7,  164.8,
      3,     7,   105.5,  58.7,  164.2,
      3,     8,   106.6,  58.7,  165.3,
      3,     9,    96.9,  58.7,  155.6) %>% 
  mutate(trt1 = forcats::fct_relevel(factor(trt1, levels = 1:9, labels = trt_code), 
                                     !! levels(thrombo_net$treatments)),
         trt2 = forcats::fct_relevel(factor(trt2, levels = 1:9, labels = trt_code), 
                                     !! levels(thrombo_net$treatments)))

for (i in 1:nrow(dias2010_nodesplit_dic)) {
  if (as.numeric(dias2010_nodesplit_dic$trt1[i]) > as.numeric(dias2010_nodesplit_dic$trt2[i])) {
    dias2010_nodesplit_dic[i, c("trt1", "trt2")] <- dias2010_nodesplit_dic[i, c("trt2", "trt1")]
  }
}

dias2010_nodesplit_dic <- arrange(dias2010_nodesplit_dic, trt1, trt2)

thrombo_nodesplit_dic <- as_tibble(summary(thrombo_nodesplit), nest = FALSE) %>% 
  distinct(trt1, trt2, resdev, pd, dic)

test_that("Node-splitting DIC", {
  expect_equal(thrombo_nodesplit_dic$dic, dias2010_nodesplit_dic$dic, tolerance = tol_dic)
  expect_equal(thrombo_nodesplit_dic$resdev, dias2010_nodesplit_dic$resdev, tolerance = tol_dic)
  expect_equal(thrombo_nodesplit_dic$pd, dias2010_nodesplit_dic$pd, tolerance = tol_dic)
})


# Force clean up
rm(list = ls())
gc()

#' Predictions of absolute effects from NMA models
#'
#' Obtain predictions of absolute effects from NMA models fitted with [nma()].
#' For example, if a model is fitted to binary data with a logit link, predicted
#' outcome probabilities or log odds can be produced. For survival models,
#' predictions can be made for survival probabilities, (cumulative) hazards,
#' (restricted) mean survival times, and quantiles including median survival
#' times.
#' When an IPD NMA or ML-NMR model has been fitted, predictions can be
#' produced either at an individual level or at an aggregate level.
#' Aggregate-level predictions are population-average absolute effects; these
#' are marginalised or standardised over a population. For example, average
#' event probabilities from a logistic regression, or marginal (standardised)
#' survival probabilities from a survival model.
#'
#' @param object A `stan_nma` object created by [nma()].
#' @param ... Additional arguments, passed to [uniroot()] for regression models
#'   if `baseline_level = "aggregate"`.
#' @param baseline An optional [distr()] distribution for the baseline response
#'   (i.e. intercept), about which to produce absolute effects. Can also be a
#'   character string naming a study in the network to take the estimated
#'   baseline response distribution from. If `NULL`, predictions are produced
#'   using the baseline response for each study in the network with IPD or
#'   arm-based AgD.
#'
#'   For regression models, this may be a list of [distr()] distributions (or
#'   study names in the network to use the baseline distributions from) of the
#'   same length as the number of studies in `newdata`, possibly named by the
#'   studies in `newdata` or otherwise in order of appearance in `newdata`.
#'
#'   Use the `baseline_type` and `baseline_level` arguments to specify whether
#'   this distribution is on the response or linear predictor scale, and (for
#'   ML-NMR or models including IPD) whether this applies to an individual at
#'   the reference level of the covariates or over the entire `newdata`
#'   population, respectively. For example, in a model with a logit link with
#'   `baseline_type = "link"`, this would be a distribution for the baseline log
#'   odds of an event. For survival models, `baseline` always corresponds to the
#'   intercept parameters in the linear predictor (i.e. `baseline_type` is
#'   always `"link"`, and `baseline_level` is `"individual"` for IPD NMA or
#'   ML-NMR, and `"aggregate"` for AgD NMA).
#'
#'   Use the `baseline_trt` argument to specify which treatment this
#'   distribution applies to.
#' @param newdata Only required if a regression model is fitted and `baseline`
#'   is specified. A data frame of covariate details, for which to produce
#'   predictions. Column names must match variables in the regression model.
#'
#'   If `level = "aggregate"` this should either be a data frame with integration
#'   points as produced by [add_integration()] (one row per study), or a data
#'   frame with individual covariate values (one row per individual) which are
#'   summarised over.
#'
#'   If `level = "individual"` this should be a data frame of individual
#'   covariate values, one row per individual.
#'
#'   If `NULL`, predictions are produced for all studies with IPD and/or
#'   arm-based AgD in the network, depending on the value of `level`.
#' @param study Column of `newdata` which specifies study names or IDs. When not
#'   specified: if `newdata` contains integration points produced by
#'   [add_integration()], studies will be labelled sequentially by row;
#'   otherwise data will be assumed to come from a single study.
#' @param type Whether to produce predictions on the `"link"` scale (the
#'   default, e.g. log odds) or `"response"` scale (e.g. probabilities).
#'
#'   For survival models, the options are `"survival"` for survival
#'   probabilities (the default), `"hazard"` for hazards, `"cumhaz"` for
#'   cumulative hazards, `"mean"` for mean survival times, `"quantile"` for
#'   quantiles of the survival time distribution, `"median"` for median survival
#'   times (equivalent to `type = "quantile"` with `quantiles = 0.5`), `"rmst"`
#'   for restricted mean survival times, or `"link"` for the linear predictor.
#'   For `type = "survival"`, `"hazard"` or `"cumhaz"`, predictions are given at
#'   the times specified by `times` or at the event/censoring times in the
#'   network if `times = NULL`. For `type = "rmst"`, the restricted time horizon
#'   is specified by `times`, or if `times = NULL` the earliest last follow-up
#'   time amongst the studies in the network is used. When `level =
#'   "aggregate"`, these all correspond to the standardised survival function
#'   (see details).
#' @param level The level at which predictions are produced, either
#'   `"aggregate"` (the default), or `"individual"`. If `baseline` is not
#'   specified, predictions are produced for all IPD studies in the network if
#'   `level` is `"individual"` or `"aggregate"`, and for all arm-based AgD
#'   studies in the network if `level` is `"aggregate"`.
#' @param baseline_trt Treatment to which the `baseline` response distribution
#'   refers, if `baseline` is specified. By default, the baseline response
#'   distribution will refer to the network reference treatment. Coerced to
#'   character string.
#' @param baseline_type When a `baseline` distribution is given, specifies
#'   whether this corresponds to the `"link"` scale (the default, e.g. log odds)
#'   or `"response"` scale (e.g. probabilities). For survival models, `baseline`
#'   always corresponds to the intercept parameters in the linear predictor
#'   (i.e. `baseline_type` is always `"link"`).
#' @param baseline_level When a `baseline` distribution is given, specifies
#'   whether this corresponds to an individual at the reference level of the
#'   covariates (`"individual"`, the default), or from an (unadjusted) average
#'   outcome on the reference treatment in the `newdata` population
#'   (`"aggregate"`). Ignored for AgD NMA, since the only option is
#'   `"aggregate"` in this instance. For survival models, `baseline` always
#'   corresponds to the intercept parameters in the linear predictor (i.e.
#'   `baseline_level` is `"individual"` for IPD NMA or ML-NMR, and `"aggregate"`
#'   for AgD NMA).
#' @param probs Numeric vector of quantiles of interest to present in computed
#'   summary, default `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#' @param predictive_distribution Logical, when a random effects model has been
#'   fitted, should the predictive distribution for absolute effects in a new
#'   study be returned? Default `FALSE`.
#' @param expand Logical, expand out predictions for every treatment (`TRUE`),
#'   or only produce predictions for observed treatments (`FALSE`). Default
#'   `TRUE`.
#' @param summary Logical, calculate posterior summaries? Default `TRUE`.
#' @param progress Logical, display progress for potentially long-running
#'   calculations? Population-average predictions from ML-NMR models are
#'   computationally intensive, especially for survival outcomes. Currently the
#'   default is to display progress only when running interactively and
#'   producing predictions for a survival ML-NMR model.
#' @param trt_ref Deprecated, renamed to `baseline_trt`.
#'
#' @details
#' # Aggregate-level predictions from IPD NMA and ML-NMR models
#' Population-average absolute effects can be produced from IPD NMA and ML-NMR
#' models with `level = "aggregate"`. Predictions are averaged over the target
#' population (i.e. standardised/marginalised), either by (numerical)
#' integration over the joint covariate distribution (for AgD studies in the
#' network for ML-NMR, or AgD `newdata` with integration points created by
#' [add_integration()]), or by averaging predictions for a sample of individuals
#' (for IPD studies in the network for IPD NMA/ML-NMR, or IPD `newdata`).
#'
#' For example, with a binary outcome, the population-average event
#' probabilities on treatment \eqn{k} in study/population \eqn{j} are
#' \deqn{\bar{p}_{jk} = \int_\mathfrak{X} p_{jk}(\mathbf{x}) f_{jk}(\mathbf{x})
#' d\mathbf{x}}
#' for a joint covariate distribution \eqn{f_{jk}(\mathbf{x})} with
#' support \eqn{\mathfrak{X}} or
#' \deqn{\bar{p}_{jk} = \sum_i p_{jk}(\mathbf{x}_i)}
#' for a sample of individuals with covariates \eqn{\mathbf{x}_i}.
#'
#' Population-average absolute predictions follow similarly for other types of
#' outcomes, however for survival outcomes there are specific considerations.
#'
#' ## Standardised survival predictions
#' Different types of population-average survival predictions, often called
#' standardised survival predictions, follow from the **standardised survival
#' function** created by integrating (or equivalently averaging) the
#' individual-level survival functions at each time \eqn{t}:
#' \deqn{\bar{S}_{jk}(t) = \int_\mathfrak{X} S_{jk}(t | \mathbf{x}) f_{jk}(\mathbf{x})
#' d\mathbf{x}}
#' which is itself produced using `type = "survival"`.
#'
#' The **standardised hazard function** corresponding to this standardised
#' survival function is a weighted average of the individual-level hazard
#' functions
#' \deqn{\bar{h}_{jk}(t) = \frac{\int_\mathfrak{X} S_{jk}(t | \mathbf{x}) h_{jk}(t | \mathbf{x}) f_{jk}(\mathbf{x})
#' d\mathbf{x} }{\bar{S}_{jk}(t)}}
#' weighted by the probability of surviving to time \eqn{t}. This is produced
#' using `type = "hazard"`.
#'
#' The corresponding **standardised cumulative hazard function** is
#' \deqn{\bar{H}_{jk}(t) = -\log(\bar{S}_{jk}(t))}
#' and is produced using `type = "cumhaz"`.
#'
#' **Quantiles and medians** of the standardised survival times are found by
#' solving
#' \deqn{\bar{S}_{jk}(t) = 1-\alpha}
#' for the \eqn{\alpha\%} quantile, using numerical root finding. These are
#' produced using `type = "quantile"` or `"median"`.
#'
#' **(Restricted) means** of the standardised survival times are found by
#' integrating
#' \deqn{\mathrm{RMST}_{jk}(t^*) = \int_0^{t^*} \bar{S}_{jk}(t) dt}
#' up to the restricted time horizon \eqn{t^*}, with \eqn{t^*=\infty} for mean
#' standardised survival time. These are produced using `type = "rmst"` or
#' `"mean"`.
#'
#' @return A [nma_summary] object if `summary = TRUE`, otherwise a list
#'   containing a 3D MCMC array of samples and (for regression models) a data
#'   frame of study information.
#' @export
#'
#' @seealso [plot.nma_summary()] for plotting the predictions.
#'
#' @examples ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Predicted log odds of success in each study in the network
#' predict(smk_fit_RE)
#'
#' # Predicted probabilities of success in each study in the network
#' predict(smk_fit_RE, type = "response")
#'
#' # Predicted probabilities in a population with 67 observed events out of 566
#' # individuals on No Intervention, corresponding to a Beta(67, 566 - 67)
#' # distribution on the baseline probability of response, using
#' # `baseline_type = "response"`
#' (smk_pred_RE <- predict(smk_fit_RE,
#'                         baseline = distr(qbeta, 67, 566 - 67),
#'                         baseline_type = "response",
#'                         type = "response"))
#' plot(smk_pred_RE, ref_line = c(0, 1))
#'
#' # Predicted probabilities in a population with a baseline log odds of
#' # response on No Intervention given a Normal distribution with mean -2
#' # and SD 0.13, using `baseline_type = "link"` (the default)
#' # Note: this is approximately equivalent to the above Beta distribution on
#' # the baseline probability
#' (smk_pred_RE2 <- predict(smk_fit_RE,
#'                          baseline = distr(qnorm, mean = -2, sd = 0.13),
#'                          type = "response"))
#' plot(smk_pred_RE2, ref_line = c(0, 1))
#' }
#'
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_mlnmr_example
#' @examples \donttest{
#' # Predicted probabilities of response in each study in the network
#' (pso_pred <- predict(pso_fit, type = "response"))
#' plot(pso_pred, ref_line = c(0, 1))
#'
#' # Predicted probabilites of response in a new target population, with means
#' # and SDs or proportions given by
#' new_agd_int <- data.frame(
#'   bsa_mean = 0.6,
#'   bsa_sd = 0.3,
#'   prevsys = 0.1,
#'   psa = 0.2,
#'   weight_mean = 10,
#'   weight_sd = 1,
#'   durnpso_mean = 3,
#'   durnpso_sd = 1
#' )
#'
#' # We need to add integration points to this data frame of new data
#' # We use the weighted mean correlation matrix computed from the IPD studies
#' new_agd_int <- add_integration(new_agd_int,
#'                                durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
#'                                prevsys = distr(qbern, prob = prevsys),
#'                                bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
#'                                weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
#'                                psa = distr(qbern, prob = psa),
#'                                cor = pso_net$int_cor,
#'                                n_int = 64)
#'
#' # Predicted probabilities of achieving PASI 75 in this target population, given
#' # a Normal(-1.75, 0.08^2) distribution on the baseline probit-probability of
#' # response on Placebo (at the reference levels of the covariates), are given by
#' (pso_pred_new <- predict(pso_fit,
#'                          type = "response",
#'                          newdata = new_agd_int,
#'                          baseline = distr(qnorm, -1.75, 0.08)))
#' plot(pso_pred_new, ref_line = c(0, 1))
#' }
#'
#' ## Progression free survival with newly-diagnosed multiple myeloma
#' @template ex_ndmm_example
#' @examples \donttest{
#' # We can produce a range of predictions from models with survival outcomes,
#' # chosen with the type argument to predict
#'
#' # Predicted survival probabilities at 5 years
#' predict(ndmm_fit, type = "survival", times = 5)
#'
#' # Survival curves
#' plot(predict(ndmm_fit, type = "survival"))
#'
#' # Hazard curves
#' # Here we specify a vector of times to avoid attempting to plot infinite
#' # hazards for some studies at t=0
#' plot(predict(ndmm_fit, type = "hazard", times = seq(0.001, 6, length.out = 50)))
#'
#' # Cumulative hazard curves
#' plot(predict(ndmm_fit, type = "cumhaz"))
#'
#' # Survival time quantiles and median survival
#' predict(ndmm_fit, type = "quantile", quantiles = c(0.2, 0.5, 0.8))
#' plot(predict(ndmm_fit, type = "median"))
#'
#' # Mean survival time
#' predict(ndmm_fit, type = "mean")
#'
#' # Restricted mean survival time
#' # By default, the time horizon is the shortest follow-up time in the network
#' predict(ndmm_fit, type = "rmst")
#'
#' # An alternative restriction time can be set using the times argument
#' predict(ndmm_fit, type = "rmst", times = 5)  # 5-year RMST
#' }
predict.stan_nma <- function(object, ...,
                             baseline = NULL, newdata = NULL, study = NULL,
                             type = c("link", "response"),
                             level = c("aggregate", "individual"),
                             baseline_trt = NULL,
                             baseline_type = c("link", "response"),
                             baseline_level = c("individual", "aggregate"),
                             probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                             predictive_distribution = FALSE,
                             expand = TRUE,
                             summary = TRUE,
                             progress = FALSE,
                             trt_ref = NULL) {
  # Checks
  if (!inherits(object, "stan_nma")) abort("Expecting a `stan_nma` object, as returned by nma().")

  if (!rlang::is_bool(progress)) abort("`progress` must be a single logical value, TRUE/FALSE.")

  is_surv <- inherits(object, "stan_nma_surv")  # Survival flag

  if (!is_surv) type <- rlang::arg_match(type)
  level <- rlang::arg_match(level)

  baseline_type <- rlang::arg_match(baseline_type)
  baseline_level <- rlang::arg_match(baseline_level)


  # Get additional survival arguments passed from NextMethod()
  if (is_surv) {
    dlist <- list(...)
    times <- dlist$times
    aux <- dlist$aux
    quantiles <- dlist$quantiles
    times_seq <- dlist$times_seq
  }


  # Get network reference treatment
  nrt <- levels(object$network$treatments)[1]

  # Deprecated trt_ref in favour of baseline_trt
  if (is.null(baseline_trt) && !is.null(trt_ref)) baseline_trt <- trt_ref

  if (!is.null(baseline_trt)) {
    if (is.null(baseline)) {
      # warn("Ignoring `baseline_trt` since `baseline` is not given.")
      baseline_trt <- nrt
    } else {
      if (length(baseline_trt) > 1) abort("`baseline_trt` must be length 1.")
      baseline_trt <- as.character(baseline_trt)
      lvls_trt <- levels(object$network$treatments)
      if (! baseline_trt %in% lvls_trt)
        abort(sprintf("`baseline_trt` does not match a treatment in the network.\nSuitable values are: %s",
                      ifelse(length(lvls_trt) <= 5,
                             paste0(lvls_trt, collapse = ", "),
                             paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    }
  } else {
    # Set baseline_trt to network reference treatment if unset
    baseline_trt <- nrt
  }

  # Define auxiliary parameters for survival models
  if (is_surv) {
    aux_pars <- switch(object$likelihood,
                       exponential = NULL,
                       `exponential-aft` = NULL,
                       lognormal = "sdlog",
                       gengamma = c("sigma", "k"),
                       mspline = "scoef",
                       pexp = "scoef",
                       "shape")
  } else {
    aux_pars <- NULL
  }

  # Was aux_by used?
  used_aux_by <- !is.null(object$aux_by) && !(length(object$aux_by) == 1 && object$aux_by == ".study")

  if (!is.null(baseline) && ".mu" %in% all.vars(object$regression)) {
    if (is.null(newdata)) {
      newdata <- data.frame(.mu = 1)
    } else {
      newdata$.mu <- 1
    }
  }

  if (!is.null(newdata)) {
    if (!is.data.frame(newdata)) abort("`newdata` is not a data frame.")

    .study <- pull_non_null(newdata, rlang::enquo(study))
    if (is.null(.study)) {
      if (inherits(object, "integration_tbl"))
        newdata$.study <- nfactor(paste("New", seq_len(nrow(newdata))))
      else
        newdata$.study <- nfactor("New 1")
    } else {
      check_study(.study)
      newdata <- dplyr::mutate(newdata, .study = nfactor(.study))
    }
  }

  if (!rlang::is_bool(expand))
    abort("`expand` should be TRUE or FALSE.")

  if (!rlang::is_bool(summary))
    abort("`summary` should be TRUE or FALSE.")

  check_probs(probs)

  # Cannot produce predictions for inconsistency models
  if (object$consistency != "consistency")
    abort(glue::glue("Cannot produce predictions under inconsistency '{object$consistency}' model."))

  # Get NMA formula
  nma_formula <- make_nma_formula(object$regression,
                                  consistency = object$consistency,
                                  classes = !is.null(object$network$classes),
                                  class_interactions = object$class_interactions)


  # Without regression model ---------------------------------------------------
  if (is.null(object$regression) && !aux_needs_integration(aux_regression = object$aux_regression, aux_by = object$aux_by)) {

    if (is_surv && !is.null(aux_pars) && xor(is.null(baseline), is.null(aux))) {
        abort("Specify both `baseline` and `aux`, or neither")
    }

    if (is_surv) times <- rlang::eval_tidy(times)

    if (!is.null(baseline)) {
      if (!inherits(baseline, "distr") && !rlang::is_string(baseline))
        abort("Baseline response `baseline` should be specified using distr(), a character string naming a study in the network, or NULL.")
    }

    if (level == "individual")
      abort("Cannot produce individual predictions without a regression model.")

    ## Without baseline specified ----------------------------------------------
    if (is.null(baseline)) {

      if (!has_ipd(object$network) && !has_agd_arm(object$network)) {
        abort("No arm-based data (IPD or AgD) in network. Specify `baseline` to produce predictions of absolute effects.")
      } else {

        # Make design matrix of all studies with baselines, and all treatments
        if (expand && (is.null(object$aux_by) || !".trt" %in% object$aux_by)) {
          studies <- forcats::fct_unique(forcats::fct_drop(forcats::fct_c(
            if (has_ipd(object$network)) object$network$ipd$.study else factor(),
            if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor()
            )))
          preddat <- tidyr::expand_grid(.study = studies, .trt = object$network$treatments)
        } else {
          # If aux_by stratifies by .trt too, then we can only predict for observed treatment arms
          preddat <- dplyr::bind_rows(
            if (has_ipd(object$network)) dplyr::distinct(object$network$ipd, .data$.study, .data$.trt) else dplyr::tibble(),
            if (has_agd_arm(object$network)) dplyr::distinct(object$network$agd_arm, .data$.study, .data$.trt) else dplyr::tibble()
          ) %>%
            dplyr::arrange(.data$.study, .data$.trt)
        }

        # Add in .trtclass if defined in network
        if (!is.null(object$network$classes)) {
          preddat$.trtclass <- object$network$classes[as.numeric(preddat$.trt)]
        }

        # Design matrix, just treating all data as AgD arm
        X_list <- make_nma_model_matrix(nma_formula,
                                        dat_agd_arm = preddat,
                                        xbar = object$xbar,
                                        consistency = object$consistency,
                                        classes = !is.null(object$network$classes),
                                        newdata = TRUE)
        X_all <- X_list$X_agd_arm
        rownames(X_all) <- paste0("pred[", preddat$.study, ": ", preddat$.trt, "]")

        # Get posterior samples
        post <- as.array(object, pars = c("mu", "d"))
        if (predictive_distribution) {
          # For predictive distribution, use delta_new instead of d
          delta_new <- get_delta_new(object)
          post[ , , dimnames(delta_new)[[3]]] <- delta_new
        }

        # Get prediction array
        pred_array <- tcrossprod_mcmc_array(post, X_all)

        # Get aux parameters for survival models
        if (is_surv) {
          if (!is.null(aux)) abort("Specify both `aux` and `baseline` or neither.")

          if (is.null(aux_pars)) {
            aux_array <- NULL
          } else {
            aux_array <- as.array(object, pars = aux_pars)
          }
        }

        # Deal with times argument
        if (type %in% c("survival", "hazard", "cumhaz")) {
          if (is.null(times)) {
            # Take times from network
            surv_all <- dplyr::bind_rows(object$network$ipd,
                                         if (has_agd_arm(object$network)) tidyr::unnest(object$network$agd_arm, cols = ".Surv") else NULL)
            surv_all <- dplyr::mutate(surv_all, !!! get_Surv_data(surv_all$.Surv))

            # Calculate times sequence if times_seq specified
            if (!is.null(times_seq)) {
              surv_all <- dplyr::group_by(surv_all, .data$.study) %>%
                dplyr::summarise(time = list(seq(from = 0, to = max(.data$time), length.out = times_seq))) %>%
                tidyr::unnest(cols = "time") %>%
                dplyr::ungroup()
            }

            # Add times vector to preddat
            preddat <- dplyr::left_join(preddat,
                                        dplyr::group_by(surv_all, .data$.study) %>%
                                          dplyr::summarise(.time = list(.data$time)),
                                        by = ".study")

          } else {
            # Use provided vector of times
            if (!is.numeric(times))
              abort("`times` must be a numeric vector of times to predict at.")

            preddat <- dplyr::mutate(preddat, .time = list(times))
          }
        } else if (type == "rmst") {
          if (is.null(times)) {
            # Take time horizon from network
            surv_all <- dplyr::bind_rows(object$network$ipd,
                                         if (has_agd_arm(object$network)) tidyr::unnest(object$network$agd_arm, cols = ".Surv") else NULL)
            surv_all <- dplyr::mutate(surv_all, !!! get_Surv_data(surv_all$.Surv))

            # Time horizon is earliest last follow-up time amongst the studies
            last_times <- surv_all %>%
              dplyr::group_by(.data$.study) %>%
              dplyr::summarise(time = max(.data$time))

            times <- min(last_times$time)
            preddat$.time <- times

          } else {
            # Use provided time
            if (!is.numeric(times) && length(times) > 1)
              abort("`times` must be a scalar numeric value giving the restricted time horizon.")

            preddat$.time <- times
          }
        }

      }
    ## With baseline specified -------------------------------------------------
    } else {

      # Make design matrix of SINGLE study, and all treatments
      if (expand) {
        preddat <- tibble::tibble(.study = factor("..dummy.."), .trt = object$network$treatments)
      } else if (rlang::is_string(baseline)) {
        preddat <- dplyr::bind_rows(
          if (has_ipd(object$network)) dplyr::distinct(object$network$ipd, .data$.study, .data$.trt),
          if (has_agd_arm(object$network)) dplyr::distinct(object$network$agd_arm, .data$.study, .data$.trt)
        ) %>%
          dplyr::filter(.data$.study == baseline) %>%
          dplyr::mutate(.study = forcats::fct_recode(.data$.study, "..dummy.." = !! baseline))
      } else {
        if (is.null(newdata) || !rlang::has_name(newdata, ".trt"))
          abort("Provide `newdata` with a `.trt` column specifying the treatments to predict for when `expand = FALSE`.")
        if (!all(newdata$.trt %in% levels(object$network$treatments)))
          abort("Treatments in `newdata` do not match those in the network.")
        preddat <- dplyr::mutate(newdata, .study = "..dummy..", .trt = factor(.data$.trt, levels = levels(object$network$treatments)))
      }

      # Only predict for observed arms if using aux_by
      if (used_aux_by && rlang::is_string(aux)) {
        aux_by_obs <- dplyr::bind_rows(
          if (has_ipd(object$network)) dplyr::distinct(object$network$ipd, .data$.study, !!! rlang::syms(object$aux_by)),
          if (has_agd_arm(object$network)) dplyr::distinct(object$network$agd_arm, .data$.study, !!! rlang::syms(object$aux_by))
        ) %>%
          dplyr::mutate(.study = forcats::fct_recode(.data$.study, "..dummy.." = !! aux))

        preddat <- dplyr::inner_join(preddat, aux_by_obs, by = unique(c(".study", object$aux_by)))
      }

      # Add in .trtclass if defined in network
      if (!is.null(object$network$classes)) {
        preddat$.trtclass <- object$network$classes[as.numeric(preddat$.trt)]
      }

      # Design matrix, just treating all data as AgD arm
      X_list <- make_nma_model_matrix(nma_formula,
                                      dat_agd_arm = preddat,
                                      xbar = object$xbar,
                                      consistency = object$consistency,
                                      classes = !is.null(object$network$classes),
                                      newdata = TRUE)
      X_all <- X_list$X_agd_arm
      rownames(X_all) <- paste0("pred[", preddat$.trt, "]")

      # Get posterior samples
      d <- as.array(object, pars = "d")
      dim_d <- dim(d)

      if (rlang::is_string(baseline)) {
        # Using the baseline from a study in the network
        if (! baseline %in% unique(forcats::fct_c(if (has_ipd(object$network)) object$network$ipd$.study else factor(),
                                                  if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor())))
          abort("`baseline` must match the name of an IPD or AgD (arm-based) study in the network, or be a distr() distribution.")

        mu <- as.array(object, pars = "mu")
        mu <- mu[ , , grep(paste0("\\[\\Q", baseline, "\\E[\\:,\\]]"), dimnames(mu)[[3]], perl = TRUE), drop = FALSE]

        baseline_type <- "link"
        baseline_level <- "individual"
      } else {
        # Generate baseline samples
        dim_mu <- c(dim_d[1:2], 1)
        u <- runif(prod(dim_mu))
        mu <- array(rlang::eval_tidy(rlang::call2(baseline$qfun, p = u, !!! baseline$args)),
                    dim = dim_mu)
      }

      # Convert to linear predictor scale if baseline_type = "response"
      if (baseline_type == "response") {
        mu <- link_fun(mu, link = object$link)
      }

      # Convert to samples on network ref trt if baseline_trt given
      if (baseline_trt != nrt) {
        mu <- mu - d[ , , paste0("d[", baseline_trt, "]"), drop = FALSE]
      }

      # Combine mu and d
      dim_post <- c(dim_d[1:2], dim_d[3] + 1)
      post <- array(NA_real_, dim = dim_post, dimnames = c(dimnames(d)[1:2], list(parameters = NULL)))
      post[ , , 1] <- mu
      if (!predictive_distribution) {
        post[ , , 2:dim_post[3]] <- d
      } else {
        # For predictive distribution, use delta_new instead of d
        post[ , , 2:dim_post[3]] <- get_delta_new(object)
      }

      # Get prediction array
      pred_array <- tcrossprod_mcmc_array(post, X_all)

      # Get aux parameters for survival models
      if (is_surv) {

        if (is.null(aux_pars)) {
          aux_array <- NULL
        } else {

          n_aux <- length(aux_pars)

          if (rlang::is_string(aux)) {
            # Using the aux from a study in the network

            if (! aux %in% unique(forcats::fct_c(if (has_ipd(object$network)) object$network$ipd$.study else factor(),
                                                      if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor()))) {
              if (n_aux == 1) {
                abort("`aux` must match the name of an IPD or AgD (arm-based) study in the network, or be a distr() distribution.")
              } else {
                abort(glue::glue("`aux` must match the name of an IPD or AgD (arm-based) study in the network, or be a named list of distr() specifications for ",
                                 glue::glue_collapse(aux_pars, sep = ", ", last = " and "), "."))
              }
            }

            aux_array <- as.array(object, pars = aux_pars)
            aux_array <- aux_array[ , , grep(paste0("\\[\\Q", aux, "\\E[\\:,\\]]"), dimnames(aux_array)[[3]], perl = TRUE), drop = FALSE]

            # Set preddat .study to use this aux par (and basis, for mspline/pexp)
            preddat$.study <- aux

          } else {
            if (object$likelihood %in% c("mspline", "pexp")) {
              abort(glue::glue('Producing predictions with external `aux` spline coefficients is not currently supported for "{object$likelihood}" models.'))
            } else {
              aux_names <- paste0(aux_pars, "[..dummy..]")
            }

            if (n_aux > 1) {
              if (!rlang::is_bare_list(aux, n = n_aux) ||
                  !setequal(names(aux), aux_pars) ||
                  any(purrr::map_lgl(aux, ~!inherits(., "distr"))))
                abort(glue::glue("`aux` must be a named list of distr() specifications for ",
                                 glue::glue_collapse(aux_pars, sep = ", ", last = " and "), ", or a study name."))
            } else {
              if (!inherits(aux, "distr"))
                abort("`aux` must be specified using distr(), or the name of an IPD or AgD (arm-based) study in the network.")
            }

            dim_aux <- c(dim_mu[1:2], n_aux)
            u <- array(runif(prod(dim_aux)), dim = dim_aux)

            if (n_aux == 1) {
              aux_array <- array(rlang::eval_tidy(rlang::call2(aux$qfun, p = u, !!! aux$args)),
                                 dim = dim_aux,
                                 dimnames = list(iterations = NULL,
                                                 chains = NULL,
                                                 parameters = aux_names))
            } else {
              aux_array <- array(NA_real_,
                                 dim = dim_aux,
                                 dimnames = list(iterations = NULL,
                                                 chains = NULL,
                                                 parameters = aux_names))
              for (i in 1:n_aux) {
                aux_array[, , i] <- rlang::eval_tidy(rlang::call2(aux[[aux_pars[i]]]$qfun, p = u[ , , i, drop = TRUE], !!! aux[[aux_pars[i]]]$args))
              }
            }
          }
        }

        # Check times argument
        if (is.null(times) && type %in% c("survival", "hazard", "cumhaz", "rmst"))
          abort("`times` must be specified when `baseline` and `aux` are provided")

        if (type %in% c("survival", "hazard", "cumhaz") && !is.numeric(times))
          abort("`times` must be a numeric vector of times to predict at.")

        if (type == "rmst" && (!is.numeric(times) && length(times) > 1))
          abort("`times` must be a scalar numeric value giving the restricted time horizon.")

        # Add times vector in to preddat
        if (type %in% c("survival", "hazard", "cumhaz")) {
          preddat <- dplyr::mutate(preddat, .time = list(times))
        } else if (type == "rmst") {
          preddat$.time <- times
        }
      }

    }

    # Get predictions for each category for ordered models
    if (object$likelihood == "ordered") {
      cc <- as.array(object, pars = "cc")
      n_cc <- dim(cc)[3]
      l_cc <- stringr::str_replace(dimnames(cc)[[3]], "^cc\\[(.+)\\]$", "\\1")

      # Apply cutoffs
      d_p <- d_pt <- dim(pred_array)
      d_pt[3] <- d_p[3]*n_cc
      dn_p <- dn_pt <- dimnames(pred_array)
      dn_pt[[3]] <- rep(dn_p[[3]], each = n_cc)
      pred_temp <- array(dim = d_pt, dimnames = dn_pt)

      for (i in 1:d_p[3]) {
        pred_temp[ , , (i-1)*n_cc + 1:n_cc] <- sweep(cc, 1:2, pred_array[ , , i, drop = FALSE],
                                                     FUN = function(x, y) {y - x})
        dn_pt[[3]][(i-1)*n_cc + 1:n_cc] <- paste0(stringr::str_sub(dn_p[[3]][i], start = 1, end = -2),
                                                  ", ", l_cc, "]")
      }
      dimnames(pred_temp) <- dn_pt
      pred_array <- pred_temp
    }

    # Get predictions for survival models
    if (is_surv) {

      pred_temp <- pred_array

      d_p <- dim(pred_temp)
      dn_p <- dimnames(pred_temp)

      if (type %in% c("survival", "hazard", "cumhaz")) {
        d_p[3] <- sum(lengths(preddat$.time))
        dn_p[[3]] <- paste0(rep(stringr::str_sub(dn_p[[3]], start = 1, end = -2), times = lengths(preddat$.time)),
                            ", ", unlist(purrr::map(preddat$.time, seq_along)), "]")
      } else if (type == "quantile") {
        dn_p[[3]] <- paste0(rep(stringr::str_sub(dn_p[[3]], start = 1, end = -2), each = length(quantiles)),
                            ", ", rep(quantiles, times = d_p[3]), "]")
        d_p[3] <- d_p[3]*length(quantiles)
      }

      if (!is.null(object$aux_regression)) {
        X_aux <- make_nma_model_matrix(object$aux_regression,
                                       xbar = object$xbar,
                                       dat_ipd = preddat)$X_ipd
        beta_aux <- as.array(object, pars = "beta_aux")
      }

      pred_array <- array(NA_real_, dim = d_p, dimnames = dn_p)

      aux_names <- dimnames(aux_array)[[3]]

      j <- 0
      for (i in 1:nrow(preddat)) {
        if (type %in% c("survival", "hazard", "cumhaz", "rmst")) {
          tt <- preddat$.time[[i]]
          jinc <- length(tt)
        } else if (type == "quantile") {
          tt <- NA
          jinc <- length(quantiles)
        } else {
          tt <- NA
          jinc <- 1
        }

        s <- get_aux_labels(preddat[i, ], by = object$aux_by)

        aux_s <- grepl(paste0("[", s, if (object$likelihood %in% c("mspline", "pexp")) "," else "]"),
                       aux_names, fixed = TRUE)

        if (object$likelihood %in% c("mspline", "pexp")) {
          basis <- object$basis[[as.character(preddat$.study[i])]]
        } else {
          basis <- NULL
        }

        if (!is.null(object$aux_regression)) {
          auxi <- make_aux_predict(aux = aux_array[ , , aux_s, drop = FALSE],
                                   beta_aux = beta_aux,
                                   X_aux = X_aux[i, , drop = FALSE],
                                   likelihood = object$likelihood)
        } else {
          auxi <- aux_array[ , , aux_s, drop = FALSE]
        }

        pred_array[ , , (j+1):(j+jinc)] <-
          make_surv_predict(eta = pred_temp[ , , i, drop = FALSE],
                            aux = auxi,
                            times = tt,
                            quantiles = quantiles,
                            likelihood = object$likelihood,
                            type = type,
                            basis = basis)

        j <- j + jinc
      }
    }

    # Transform predictions if type = "response"
    if (type == "response") {
      pred_array <- inverse_link(pred_array, link = object$link)
    }

    # Produce nma_summary
    if (object$likelihood == "ordered") {
      pred_meta <- tibble::tibble(.trt = rep(preddat$.trt, each = n_cc),
                                  .category = rep(l_cc, times = nrow(preddat)))

    } else if (object$likelihood %in% valid_lhood$survival) {
      if (type %in% c("survival", "hazard", "cumhaz")) {
        preddat <- tidyr::unnest(preddat, cols = ".time")
        pred_meta <- tibble::tibble(.trt = preddat$.trt,
                                    .time = preddat$.time)
      } else if (type == "rmst") {
        pred_meta <- tibble::tibble(.trt = preddat$.trt,
                                    .time = preddat$.time)
      } else if (type == "quantile") {
        pred_meta <- tibble::tibble(.trt = rep(preddat$.trt, each = length(quantiles)),
                                    .quantile = rep(quantiles, times = nrow(preddat)))
      } else {
        pred_meta <- tibble::tibble(.trt = preddat$.trt)
      }

    } else {
      pred_meta <- tibble::tibble(.trt = preddat$.trt)
    }

    if (is.null(baseline)) {
      if (object$likelihood == "ordered") {
        pred_meta <- tibble::add_column(pred_meta,
                                        .study = rep(preddat$.study, each = n_cc),
                                        .before = 1)
      } else if (type == "quantile") {
        pred_meta <- tibble::add_column(pred_meta,
                                        .study = rep(preddat$.study, each = length(quantiles)),
                                        .before = 1)
      } else {
        pred_meta <- tibble::add_column(pred_meta,
                                        .study = preddat$.study,
                                        .before = 1)
      }
    }

    if (summary) {
      pred_summary <- summary_mcmc_array(pred_array, probs)
      pred_summary <- dplyr::bind_cols(pred_meta, pred_summary)
    } else {
      pred_summary <- pred_meta
    }

    out <- list(summary = pred_summary, sims = pred_array)

  # With regression model ------------------------------------------------------
  } else {

    if (!is_surv || is.null(aux_pars)) {
      if (xor(is.null(newdata), is.null(baseline)))
        abort("Specify both `newdata` and `baseline`, or neither.")
    } else {
      if (xor(is.null(newdata), is.null(baseline)) || xor(is.null(newdata), is.null(aux)))
        abort("Specify all of `newdata`, `baseline`, and `aux`, or none.")
    }

    if (!is.null(baseline)) {
      if (!(inherits(baseline, "distr") ||
            rlang::is_string(baseline) ||
            (rlang::is_list(baseline) && all(purrr::map_lgl(baseline, ~inherits(., what = "distr") || rlang::is_string(.))))))
        abort("Baseline response `baseline` should be a single distr() specification or character string naming a study in the network, a list of such specifications, or NULL.")
    }

    ## Without baseline and newdata specified ----------------------------------
    if (is.null(baseline) && is.null(newdata)) {

      if (is_surv) times <- rlang::eval_tidy(times)

      # Get data for prediction
      if (level == "individual") {
        if (!has_ipd(object$network))
          abort(paste("No IPD in network to produce individual predictions for.",
                      "  - Specify IPD in `newdata` for which to produce predictions, or",
                      '  - Produce aggregate predictions with level = "aggregate"',
                      sep = "\n"))

        preddat <- get_model_data_columns(object$network$ipd,
                                          regression = object$regression,
                                          aux_regression = object$aux_regression,
                                          keep = object$aux_by)

        if (is_surv) {
          # Deal with times argument
          if (type %in% c("survival", "hazard", "cumhaz")) {
            if (is.null(times)) {
              # Take times from network
              preddat$.time <- get_Surv_data(object$network$ipd$.Surv)$time
            } else {
              abort('Cannot specify `times` with `level = "individual"` and `newdata = NULL`')
            }
          } else if (type == "rmst") {
            if (is.null(times)) {
              # Take time horizon from network
              surv_all <- object$network$ipd %>%
                dplyr::mutate(!!! get_Surv_data(object$network$ipd$.Surv))

              # Time horizon is earliest last follow-up time amongst the studies
              last_times <- surv_all %>%
                dplyr::group_by(.data$.study) %>%
                dplyr::summarise(time = max(.data$time))

              times <- min(last_times$time)
              preddat$.time <- times

            } else {
              # Use provided time
              if (!is.numeric(times) && length(times) > 1)
                abort("`times` must be a scalar numeric value giving the restricted time horizon.")

              preddat$.time <- times
            }
          }

          preddat <- dplyr::group_by(preddat, .data$.study) %>%
            dplyr::mutate(.obs_id = 1:dplyr::n()) %>%
            dplyr::ungroup()
        }

      } else {

        if (!has_ipd(object$network) && !has_agd_arm(object$network)) {
          abort("No arm-based data (IPD or AgD) in network. Specify `baseline` and `newdata` to produce predictions of absolute effects.")
        }

        if ((has_agd_arm(object$network) || has_agd_contrast(object$network)) && !has_agd_sample_size(object$network))
          abort(
            paste("AgD study sample sizes not specified in network, cannot calculate aggregate predictions.",
                  "  - Specify `sample_size` in set_agd_*(), or",
                  "  - Specify covariate values using the `newdata` argument",
                  sep = "\n"))

        # Deal with times argument for type = "rmst"
        if (is_surv && type == "rmst") {
          if (is.null(times)) {
            # Take time horizon from network
            surv_all <- dplyr::bind_rows(object$network$ipd,
                                         if (has_agd_arm(object$network)) tidyr::unnest(object$network$agd_arm, cols = ".Surv") else NULL
                                         )
            surv_all <- dplyr::mutate(surv_all, !!! get_Surv_data(surv_all$.Surv))

            # Time horizon is earliest last follow-up time amongst the studies
            last_times <- surv_all %>%
              dplyr::group_by(.data$.study) %>%
              dplyr::summarise(time = max(.data$time))

            times <- min(last_times$time)
          } else {
            # Use provided time
            if (!is.numeric(times) && length(times) > 1)
              abort("`times` must be a scalar numeric value giving the restricted time horizon.")
          }
        } else if (is_surv && type %in% c("survival", "hazard", "cumhaz")) {
          # Checks for survival/hazard/cumhaz times
          if (!is.null(times) && !is.numeric(times))
            abort("`times` must be a numeric vector of times to predict at.")
        }

        if (has_agd_arm(object$network)) {
          if (!is_surv) {
            if (inherits(object$network, "mlnmr_data")) {
              dat_agd_arm <- .unnest_integration(object$network$agd_arm) %>%
                dplyr::mutate(.sample_size = .data$.sample_size / object$network$n_int)
            } else {
              dat_agd_arm <- object$network$agd_arm
            }
          } else {
            if (type %in% c("survival", "hazard", "cumhaz") && is.null(times)) {
              if (is.null(times_seq)) {
                # Use times from network, unnest
                dat_agd_arm <- object$network$agd_arm %>%
                  # Drop duplicated names in outer dataset from .data_orig before unnesting
                  dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(object$network$agd_arm))))) %>%
                  tidyr::unnest(cols = c(".Surv", ".data_orig")) %>%
                  dplyr::mutate(!!! get_Surv_data(.$.Surv),
                                .time = .data$time) %>%
                  # Add in ID variable for each observation
                  dplyr::group_by(.data$.study) %>%
                  dplyr::mutate(.obs_id = 1:dplyr::n()) %>%
                  dplyr::ungroup()

              } else {
                # Calculate times sequence if times_seq specified
                agd_times <- object$network$agd_arm %>%
                  tidyr::unnest(cols = ".Surv") %>%
                  dplyr::mutate(!!! get_Surv_data(.$.Surv), .time = .data$time) %>%
                  dplyr::group_by(.data$.study) %>%
                  dplyr::summarise(.time = list(seq(from = 0, to = max(.data$.time), length.out = times_seq)),
                                   .obs_id = list(1:times_seq))

                dat_agd_arm <- object$network$agd_arm %>%
                  # Drop duplicated names in outer dataset from .data_orig before unnesting
                  # Take only one row of .data_orig (all duplicated anyway)
                  dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(object$network$agd_arm)))[1,])) %>%
                  dplyr::left_join(agd_times, by = ".study") %>%
                  tidyr::unnest(cols = c(".time", ".obs_id", ".data_orig"))
              }
            } else {
              # Use provided times
              dat_agd_arm <- object$network$agd_arm %>%
                # Drop duplicated names in outer dataset from .data_orig before unnesting
                # Take only one row of .data_orig (all duplicated anyway)
                dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(object$network$agd_arm)))[1,]),
                              # Use provided time vector
                              .time = if (type %in% c("survival", "hazard", "cumhaz", "rmst")) list(times) else NA,
                              .obs_id = if (type %in% c("survival", "hazard", "cumhaz", "rmst")) list(1:length(times)) else NA) %>%
                tidyr::unnest(cols = c(".time", ".obs_id", ".data_orig"))
            }

            # Unnest integration points if present
            if (inherits(object, "stan_mlnmr")) {
              dat_agd_arm <- .unnest_integration(dat_agd_arm) %>%
                dplyr::mutate(.sample_size = .data$.sample_size / object$network$n_int)
            }

            # Drop .Surv column, not needed
            dat_agd_arm <- dplyr::select(dat_agd_arm, -".Surv")
          }

          # A `.mu` column is expected if baseline risk meta-regression is used in this model
          dat_agd_arm$.mu <- 1

          # Only take necessary columns
          dat_agd_arm <- get_model_data_columns(dat_agd_arm,
                                                regression = object$regression,
                                                aux_regression = object$aux_regression,
                                                keep = object$aux_by,
                                                label = "AgD (arm-based)")
        } else {
          dat_agd_arm <- tibble::tibble()
        }

        if (has_ipd(object$network)) {
          dat_ipd <- object$network$ipd
          if (is_surv) {
            # For aggregate survival predictions we average the entire survival
            # curve over the population (i.e. over every individual), so we need
            # to expand out every time for every individual
            if (type %in% c("survival", "hazard", "cumhaz") && is.null(times)) {
              if (is.null(times_seq)) {
                # Use times from network
                ipd_times <- dplyr::mutate(dat_ipd, !!! get_Surv_data(dat_ipd$.Surv), .time = .data$time) %>%
                  dplyr::select(".study", ".trt", ".time") %>%
                  # Add in ID variable for each observation, needed later for aggregating
                  dplyr::group_by(.data$.study) %>%
                  dplyr::mutate(.obs_id = 1:dplyr::n()) %>%
                  dplyr::group_by(.data$.study, .data$.trt) %>%
                  tidyr::nest(.time = ".time", .obs_id = ".obs_id")

                dat_ipd <- dplyr::left_join(dat_ipd,
                                            ipd_times,
                                            by = c(".study", ".trt")) %>%
                  tidyr::unnest(cols = c(".time", ".obs_id"))

              } else {
                # Calculate times sequence if times_seq specified
                ipd_times <- dplyr::mutate(dat_ipd, !!! get_Surv_data(dat_ipd$.Surv), .time = .data$time) %>%
                  dplyr::group_by(.data$.study) %>%
                  dplyr::summarise(.time = list(seq(from = 0, to = max(.data$.time), length.out = times_seq)),
                                   .obs_id = list(1:times_seq)) %>%
                  dplyr::ungroup()

                dat_ipd <- dplyr::left_join(dat_ipd,
                                            ipd_times,
                                            by = ".study") %>%
                  tidyr::unnest(cols = c(".time", ".obs_id"))
              }

            } else {
              # Use provided times
              if (type %in% c("survival", "hazard", "cumhaz", "rmst")) {
                dat_ipd <- dplyr::mutate(dat_ipd,
                                         .time = list(times),
                                         .obs_id = list(1:length(times))) %>%
                  tidyr::unnest(cols = c(".time", ".obs_id"))
              }
            }

            # Drop .Surv column, not needed
            dat_ipd <- dplyr::select(dat_ipd, -".Surv")
          }

          # A `.mu` column is expected if baseline risk meta-regression is used in this model
          dat_ipd$.mu <- 1

          # Only take necessary columns
          dat_ipd <- get_model_data_columns(dat_ipd,
                                            regression = object$regression,
                                            aux_regression = object$aux_regression,
                                            keep = object$aux_by,
                                            label = "IPD")

          dat_ipd$.sample_size <- 1
        } else {
          dat_ipd <- tibble::tibble()
        }

        preddat <- dplyr::bind_rows(dat_ipd, dat_agd_arm)
      }

      # Produce predictions on every treatment for each observed arm/individual
      if (expand && (is.null(object$aux_by) || ! ".trt" %in% object$aux_by)) {
        if (packageVersion("dplyr") >= "1.1.1") {
          preddat <- preddat %>%
            dplyr::rename(.trt_old = ".trt") %>%
            dplyr::left_join(tidyr::expand(., .study = .data$.study,
                                              .trt = .data$.trt_old),
                             by = ".study",
                             relationship = "many-to-many")
        } else {
          preddat <- preddat %>%
            dplyr::rename(.trt_old = ".trt") %>%
            dplyr::left_join(tidyr::expand(., .study = .data$.study,
                                           .trt = .data$.trt_old),
                             by = ".study")
        }
        preddat <- dplyr::select(preddat, -".trt_old")
      }

      # If producing aggregate-level predictions, output these in factor order
      # Individual-level predictions will be in the order of the input data
      if (level == "aggregate") {
        preddat <- dplyr::arrange(preddat, .data$.study, .data$.trt)
      }

      # Add in .trtclass if defined in network
      if (!is.null(object$network$classes)) {
        preddat$.trtclass <- object$network$classes[as.numeric(preddat$.trt)]
      }

      if (has_agd_contrast(object$network)) {
        dat_agd_contrast <- object$network$agd_contrast
      } else {
        dat_agd_contrast <- tibble::tibble()
      }

      # Design matrix, just treating all data as IPD
      # Contrast data is included just so that the correct columns are excluded
      X_list <- make_nma_model_matrix(nma_formula,
                                      dat_ipd = preddat,
                                      dat_agd_contrast = dat_agd_contrast,
                                      xbar = object$xbar[names(object$xbar) != ".mu"],
                                      consistency = object$consistency,
                                      classes = !is.null(object$network$classes),
                                      newdata = TRUE)
      X_all <- X_list$X_ipd
      rownames(X_all) <- paste0("pred[", make_data_labels(preddat$.study, preddat$.trt), "]")

      offset_all <- X_list$offset_ipd

      # Get posterior samples
      post <- as.array(object, pars = c("mu", "d", "beta"))

      # Get aux parameters for survival models
      if (is_surv) {
        if (!is.null(aux)) abort("Specify all of `aux`, `baseline`, and `newdata`, or none.")

        if (is.null(aux_pars)) {
          aux_array <- NULL
        } else {
          aux_array <- as.array(object, pars = aux_pars)
        }
      }

    ## With baseline and newdata specified -------------------------------------
    } else {

      if (is_surv) {
        has_int <- inherits(newdata, "integration_tbl")

        newdata <- dplyr::group_by(newdata, .data$.study) %>%
          dplyr::mutate(.obs_id = 1:dplyr::n()) %>%
          dplyr::ungroup()

        # Restore class
        if (has_int) class(newdata) <- c("integration_tbl", class(newdata))
      }

      if (level == "individual") {
        if (!has_ipd(object$network))
          warn("Producing individual predictions from an aggregate-level regression. Interpret with great caution!")

        preddat <- newdata
      } else {
        if (inherits(object, "stan_mlnmr")) {
          if (!inherits(newdata, "integration_tbl")) {
            inform(paste0("No integration points found in `newdata`, averaging over individuals provided in `newdata`.\n",
                          "To set up integration points, use add_integration()."))
            preddat <- newdata
          } else {
            preddat <- .unnest_integration(newdata)
          }
        } else {
          if (has_ipd(object$network) && inherits(newdata, "integration_tbl")) {
            # Allow integration of IPD model over aggregate population
            preddat <- .unnest_integration(newdata)
          } else {
            preddat <- newdata
          }
        }
      }

      preddat$.sample_size <- 1

      # Check times argument
      if (is_surv && type %in% c("survival", "hazard", "cumhaz", "rmst")) {
        if (!rlang::is_quosure(times)) times <- rlang::enquo(times)
        if (rlang::quo_is_null(times))
          abort("`times` must be specified when `newdata`, `baseline` and `aux` are provided")


        teval <- rlang::eval_tidy(times, data = preddat)

        if ((rlang::quo_is_symbol(times) || rlang::is_scalar_character(rlang::eval_tidy(times))) && rlang::has_name(preddat, rlang::as_name(times))) {
          # times is a column of newdata
          preddat$.time <- teval

          if (type %in% c("survival", "hazard", "cumhaz") && !is.numeric(teval))
            abort("`times` must be a numeric vector or column of `newdata` giving times to predict at.")

          if (type == "rmst" && !is.numeric(teval))
            abort("`times` must be a scalar numeric value or column of `newdata` giving the restricted time horizon.")

          if (level == "aggregate" && type %in% c("survival", "hazard", "cumhaz")) {
            # Need to average survival curve over all covariate values at every time
            p_times <- dplyr::group_by(preddat, .data$.study) %>%
              dplyr::select(".study", ".time", ".obs_id") %>%
              tidyr::nest(.time = ".time", .obs_id = ".obs_id")

            preddat <- dplyr::select(preddat, -".time", -".obs_id") %>%
              dplyr::left_join(p_times, by = ".study") %>%
              tidyr::unnest(cols = c(".time", ".obs_id"))
          }

        } else {
          # times is a vector in global env
          if (type %in% c("survival", "hazard", "cumhaz") && !is.numeric(teval))
            abort("`times` must be a numeric vector or column of `newdata` giving times to predict at.")

          if (type == "rmst" && (!is.numeric(teval) || length(teval) > 1))
            abort("`times` must be a scalar numeric value or column of `newdata` giving the restricted time horizon.")

          # Add times vector in to preddat
          if (type %in% c("survival", "hazard", "cumhaz")) {
            preddat <- dplyr::mutate(preddat, .time = list(teval), .obs_id = list(1:length(teval))) %>%
              tidyr::unnest(cols = c(".time", ".obs_id"))
          } else if (type == "rmst") {
            preddat$.time <- teval
          }
        }
      }

      # Check all variables are present
      preddat <- get_model_data_columns(preddat,
                                        regression = object$regression,
                                        aux_regression = object$aux_regression,
                                        keep = setdiff(object$aux_by, c(".trt", ".trtclass", ".study", ".contr", ".omega")),
                                        label = "`newdata`")

      # Check aux spec
      if (is_surv && !is.null(aux_pars)) {
        studies <- unique(preddat$.study)
        n_studies <- length(studies)
        n_aux <- length(aux_pars)

        if (n_aux == 1) {
          if (!inherits(aux, "distr") && !rlang::is_string(aux) && length(aux) != n_studies)
            abort(sprintf("`aux` must be a single distr() specification or study name, or a list of length %d (number of `newdata` studies)", n_studies))
          if (inherits(aux, "distr") || rlang::is_string(aux)) {
            aux <- rep(list(aux), times = n_studies)
            names(aux) <- studies
          } else {
            if (any(purrr::map_lgl(aux, ~!inherits(., "distr") && !rlang::is_string(.))))
              abort(sprintf("`aux` must be a single distr() specification or study name, or a list of length %d (number of `newdata` studies)", n_studies))
            if (!rlang::is_named(aux)) {
              names(aux) <- studies
            } else {
              aux_names <- names(aux)
              if (!setequal(aux_names, studies))
                abort(glue::glue("`aux` list names must match all study names from `newdata`.\n",
                                 "Unmatched list names: ",
                                 glue::glue_collapse(glue::double_quote(setdiff(aux_names, studies)), sep = ", ", width = 30),
                                 ".\n",
                                 "Unmatched `newdata` study names: ",
                                 glue::glue_collapse(glue::double_quote(setdiff(studies, aux_names)), sep = ", ", width = 30),
                                 ".\n"))
            }
          }
        } else {
          aux_names <- names(aux)
          if (!(rlang::is_string(aux) || (
            rlang::is_bare_list(aux) &&
            length(aux) %in% c(n_aux, n_studies) &&
            (setequal(aux_names, aux_pars) || setequal(aux_names, levels(studies))) &&
            all(purrr::map_lgl(purrr::list_flatten(aux), ~inherits(., "distr") || rlang::is_string(.)))))) {
            abort(glue::glue("`aux` must be a single named list of distr() specifications for {glue::glue_collapse(aux_pars, sep = ', ', last = ' and ')}, ",
                             "a study name, or a list of length {n_studies} (number of `newdata` studies) of such lists."))
          }

          if (setequal(aux_names, aux_pars) || rlang::is_string(aux)) {
            aux <- rep(list(aux), times = n_studies)
            names(aux) <- studies
          } else if (!rlang::is_named(aux)) {
            names(aux) <- studies
          }
        }

        # Check aux_by
        if (".trt" %in% object$aux_by) {
          inform(c("Note: Producing predictions for new data from a model with baseline hazard stratified by treatment arm.",
                   "Did you mean to use `aux_regression` instead?"))
        }
      }

      # Make design matrix of all studies and all treatments
      if (expand) {
        if (rlang::has_name(preddat, ".trt")) preddat <- dplyr::select(preddat, -".trt")
        if (packageVersion("dplyr") >= "1.1.1") {
          preddat <- dplyr::left_join(preddat,
                                      tidyr::expand(preddat,
                                                    .study = .data$.study,
                                                    .trt = object$network$treatments),
                                      by = ".study",
                                      relationship = "many-to-many")
        } else {
          preddat <- dplyr::left_join(preddat,
                                      tidyr::expand(preddat,
                                                    .study = .data$.study,
                                                    .trt = object$network$treatments),
                                      by = ".study")
        }
      } else {
        if (!rlang::has_name(preddat, ".trt"))
          abort("`newdata` should have a `.trt` column specifying the treatments to predict for when `expand = FALSE`.")
        if (!all(preddat$.trt %in% levels(object$network$treatments)))
          abort("Treatments in `newdata` do not match those in the network.")
        preddat <- dplyr::mutate(preddat, .trt = factor(.data$.trt, levels = levels(object$network$treatments)))
      }

      # With aux_by = .trt, only predict for observed arms
      if (used_aux_by && any(purrr::map_lgl(aux, rlang::is_string))) {
        has_aux_by <- TRUE

        aux_by_obs <- dplyr::bind_rows(
          if (has_ipd(object$network)) dplyr::distinct(object$network$ipd, .data$.study, !!! rlang::syms(object$aux_by)),
          if (has_agd_arm(object$network)) dplyr::distinct(object$network$agd_arm, .data$.study, !!! rlang::syms(object$aux_by))
        )

        aux_by_obs <- purrr::imap_dfr(aux[purrr::map_lgl(aux, rlang::is_string)],
                                      ~dplyr::filter(aux_by_obs, .study == .x) %>% dplyr::mutate(.study = .y))

        # Add in rows for any distr() studies
        if (!all(purrr::map_lgl(aux, rlang::is_string))) {
          aux_by_distr <- dplyr::filter(preddat, .study %in% names(aux[purrr::map_lgl(aux, ~!rlang::is_string(.))])) %>%
            dplyr::distinct(.data$.study, !!! rlang::syms(object$aux_by))

          aux_by_obs <- dplyr::bind_rows(aux_by_obs, aux_by_distr)
        }

        preddat <- dplyr::inner_join(preddat, aux_by_obs, by = unique(c(".study", object$aux_by)))
      } else {
        has_aux_by <- FALSE
      }

      # Make sure preddat$.study doesn't have extra levels
      preddat$.study <- forcats::fct_drop(preddat$.study)

      # Add in .trtclass if defined in network
      if (!is.null(object$network$classes)) {
        preddat$.trtclass <- object$network$classes[as.numeric(preddat$.trt)]
      }

      # Design matrix, just treating all data as IPD
      X_list <- make_nma_model_matrix(nma_formula,
                                      dat_ipd = preddat,
                                      xbar = object$xbar,
                                      consistency = object$consistency,
                                      classes = !is.null(object$network$classes),
                                      newdata = TRUE)
      X_all <- X_list$X_ipd
      rownames(X_all) <- paste0("pred[", make_data_labels(preddat$.study, preddat$.trt), "]")

      offset_all <- X_list$offset_ipd

      # Get posterior samples
      post_temp <- as.array(object, pars = c("d", "beta"))

      # Check baseline
      studies <- unique(preddat$.study)
      n_studies <- length(studies)

      if (!inherits(baseline, "distr")) {
        if (!length(baseline) %in% c(1, n_studies))
          abort(sprintf("`baseline` must be a single distr() distribution or character string, or a list of length %d (number of `newdata` studies)", n_studies))
        if (length(baseline) == 1) {
          baseline <- baseline[[1]]
        } else {
          if (!rlang::is_named(baseline)) {
            names(baseline) <- studies
          } else {
            bl_names <- names(baseline)
            if (dplyr::n_distinct(bl_names) != n_studies)
              abort("`baseline` list names must be distinct study names from `newdata`")
            if (length(bad_bl_names <- setdiff(bl_names, studies)))
              abort(glue::glue("`baseline` list names must match all study names from `newdata`.\n",
                               "Unmatched list names: ",
                               glue::glue_collapse(glue::double_quote(bad_bl_names), sep = ", ", width = 30),
                               ".\n",
                               "Unmatched `newdata` study names: ",
                               glue::glue_collapse(glue::double_quote(setdiff(studies, bl_names)), sep = ", ", width = 30),
                               ".\n"))
          }
        }
      }

      if (rlang::is_string(baseline)) {
        # Using the baseline from a study in the network
        if (! baseline %in% unique(forcats::fct_c(if (has_ipd(object$network)) object$network$ipd$.study else factor(),
                                                  if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor())))
          abort("`baseline` must match the name of an IPD or AgD (arm-based) study in the network, or be a distr() distribution.")
      }

      if (inherits(baseline, "distr") || rlang::is_string(baseline)) {
        baseline <- rep(list(baseline), times = n_studies)
        names(baseline) <- studies
      }

      # Generate baseline samples
      dim_post_temp <- dim(post_temp)
      dim_mu <- c(dim_post_temp[1:2], n_studies)
      dimnames_mu <- c(dimnames(post_temp)[1:2], list(parameters = paste0("mu[", levels(studies), "]")))

      u <- array(runif(prod(dim_mu)), dim = dim_mu)
      mu <- array(NA_real_, dim = dim_mu, dimnames = dimnames_mu)

      if (any(purrr::map_lgl(baseline, rlang::is_string))) mu_temp <- as.array(object, pars = "mu")

      baseline_type <- rep_len(baseline_type, n_studies)
      baseline_level <- rep_len(baseline_level, n_studies)

      for (s in 1:n_studies) {
        # NOTE: mu must be in *factor order* for later multiplication with design matrix, not observation order
        ss <- levels(studies)[s]

        if (inherits(baseline[[ss]], "distr")) {
          mu[ , , s] <- array(rlang::eval_tidy(rlang::call2(baseline[[ss]]$qfun, p = u[ , , s], !!! baseline[[ss]]$args)),
                              dim = c(dim_mu[1:2], 1))
        } else if (rlang::is_string(baseline[[ss]])) {
          # Using the baseline from a study in the network
          if (! baseline[[ss]] %in% unique(forcats::fct_c(if (has_ipd(object$network)) object$network$ipd$.study else factor(),
                                                    if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor())))
            abort("All elements of `baseline` must be strings matching the name of an IPD or AgD (arm-based) study in the network, or be a distr() distribution.")

          mu[ , , s] <- mu_temp[ , , grep(paste0("\\[\\Q", baseline[[ss]], "\\E[\\:,\\]]"), dimnames(mu_temp)[[3]], perl = TRUE), drop = FALSE]

          baseline_type[s] <- "link"
          baseline_level[s] <- "individual"
        }
      }

      # Convert baseline samples as necessary

      if (length(baseline_type) == 1) baseline_type <- rep_len(baseline_type, n_studies)
      if (length(baseline_level) == 1) baseline_level <- rep_len(baseline_level, n_studies)

      if (!inherits(object, "stan_mlnmr") && !has_ipd(object$network)) {
        # AgD-only regression, ignore baseline_level = "individual"
        # if (baseline_level == "individual")
        #   inform('Setting baseline_level = "aggregate", model intercepts are aggregate level for AgD meta-regression.')

        # Convert to linear predictor scale if baseline_type = "response"
        if (any(baseline_type == "response")) {
          mu[ , , baseline_type == "response"] <- link_fun(mu[ , , baseline_type == "response"], link = object$link)
        }

        # Convert to samples on network ref trt if baseline_trt given
        if (baseline_trt != nrt) {
          mu <- sweep(mu, 1:2, post_temp[ , , paste0("d[", baseline_trt, "]"), drop = FALSE], FUN = "-")
        }
      } else { # ML-NMR or IPD NMR
        if (any(baseline_level == "individual")) {

          # Convert to linear predictor scale if baseline_type = "response"
          if (any(baseline_type == "response")) {
            mu[ , , baseline_level == "individual" & baseline_type == "response"] <- link_fun(mu[ , , baseline_level == "individual" & baseline_type == "response"], link = object$link)
          }

          # Convert to samples on network ref trt if baseline_trt given
          if (baseline_trt != nrt) {
            mu[ , , baseline_level == "individual" & baseline_level == "individual"] <- sweep(mu[ , , baseline_level == "individual" & baseline_level == "individual", drop = FALSE], 1:2, post_temp[ , , paste0("d[", baseline_trt, "]"), drop = FALSE], FUN = "-")
          }

        }

        if (any(baseline_level == "aggregate")) {

          # Assume that aggregate baselines are *unadjusted*, ie. are crude poolings over reference arm outcomes
          # In this case, we need to marginalise over the natural outcome scale

          mu0 <- mu
          if (any(baseline_type == "link")) {
            mu0[ , , baseline_level == "aggregate" & baseline_type == "link"] <- inverse_link(mu[ , , baseline_level == "aggregate" & baseline_type == "link"], link = object$link)
          }

          preddat_trt_ref <- dplyr::filter(preddat, .data$.trt == baseline_trt)

          # Get posterior samples of betas and d[baseline_trt]
          post_beta <- as.array(object, pars = "beta")
          if (baseline_trt == nrt) {
            post_d <- 0
          } else {
            post_d <- as.array(object, pars = paste0("d[", baseline_trt, "]"))
          }

          # Get design matrix for regression for baseline_trt
          X_trt_ref <- X_all[preddat$.trt == baseline_trt, , drop = FALSE]
          X_beta_trt_ref <- X_trt_ref[ , !grepl("^(\\.study|\\.trt|\\.contr)[^:]+$", colnames(X_trt_ref)), drop = FALSE]

          if (!is.null(offset_all)) offset_trt_ref <- offset_all[preddat$.trt == baseline_trt]

          range_mu <- range(as.array(object, pars = "mu"))

          # Aggregate response on natural scale, use numerical solver
          # Define function to solve for mu
          mu_solve <- function(mu, mu0, post_beta, post_d, X_beta, offset, link) {
            lp <- mu + X_beta %*% post_beta + post_d + offset
            ginv_lp <- inverse_link(lp, link = link)
            return(mu0 - mean(ginv_lp))
          }

          for (s in 1:n_studies) {
            # NOTE: mu must be in *factor order* for later multiplication with design matrix, not observation order

            if (baseline_level[s] != "aggregate") next;

            # Study select
            ss <- preddat_trt_ref$.study == levels(studies)[s]

            s_X_beta <- X_beta_trt_ref[ss, , drop = FALSE]
            if (!is.null(offset_all)) s_offset <- offset_trt_ref[ss]

            for (i_iter in 1:dim_post_temp[1]) {
              for (i_chain in 1:dim_post_temp[2]) {
                rtsolve <- uniroot(mu_solve, interval = range_mu, extendInt = "yes", ...,
                                   mu0 = mu0[i_iter, i_chain, s, drop = TRUE],
                                   post_beta = post_beta[i_iter, i_chain, , drop = TRUE],
                                   post_d = if (baseline_trt == nrt) 0 else post_d[i_iter, i_chain, , drop = TRUE],
                                   X_beta = s_X_beta,
                                   offset = if (!is.null(offset_all)) s_offset else 0,
                                   link = object$link)
                mu[i_iter, i_chain, s] <- rtsolve$root
              }
            }
          }
        }
      }

      # Combine mu, d, and beta
      dim_post <- c(dim_post_temp[1:2], dim_mu[3] + dim_post_temp[3])
      dimnames_post <- c(dimnames(post_temp)[1:2], list(parameters = c(dimnames(mu)[[3]], dimnames(post_temp)[[3]])))
      post <- array(NA_real_, dim = dim_post, dimnames = dimnames_post)
      post[ , , 1:dim_mu[3]] <- mu
      post[ , , dim_mu[3] + 1:dim_post_temp[3]] <- post_temp


      # Get aux parameters for survival models
      if (is_surv) {

        if (is.null(aux_pars)) {
          aux_array <- NULL
        } else {

          aux_ex_labels <- unlist(purrr::imap(aux, ~dplyr::filter(preddat, .study == .y) %>% get_aux_labels(by = object$aux_by)))

          if (object$likelihood %in% c("mspline", "pexp")) {
            if (!all(purrr::map_lgl(aux, rlang::is_string)))
              abort(glue::glue('Producing predictions with external `aux` spline coefficients is not currently supported for "{object$likelihood}" models.'))

            n_aux <- length(object$basis[[1]])
            aux_names <- paste0(rep(aux_pars, times = length(aux_ex_labels)), "[", rep(aux_ex_labels, each = n_aux), ", ", rep(1:n_aux, times = length(aux_ex_labels)), "]")
          } else {
            n_aux <- length(aux_pars)
            aux_names <- paste0(rep(aux_pars, times = length(aux_ex_labels)), "[", rep(aux_ex_labels, each = n_aux) , "]")
          }

          dim_aux <- c(dim_mu[1:2], length(aux_names))
          u <- array(runif(prod(dim_aux)), dim = dim_aux)
          aux_array <- array(NA_real_,
                             dim = dim_aux,
                             dimnames = list(iterations = NULL,
                                             chains = NULL,
                                             parameters = aux_names))

          if (any(purrr::map_lgl(aux, rlang::is_string))) aux_temp <- as.array(object, pars = aux_pars)

          for (s in 1:n_studies) {
            ss <- as.character(studies[s])
            se <- grep(paste0("\\[\\Q", ss, "\\E[\\:,\\]]"), aux_names, perl = TRUE)
            if (!rlang::is_string(aux[[ss]])) {
              if (n_aux == 1) {
                aux_array[, , se] <- rlang::eval_tidy(rlang::call2(aux[[ss]]$qfun, p = u[ , , se, drop = TRUE], !!! aux[[ss]]$args))
              } else for (i in 1:n_aux) {
                if (!setequal(names(aux[[ss]]), aux_pars) || !all(purrr::map_lgl(aux[[ss]], inherits, "distr")))
                  abort(glue::glue("`aux` must be a single named list of distr() specifications for {glue::glue_collapse(aux_pars, sep = ', ', last = ' and ')}, ",
                                   "a study name, or a list of length {n_studies} (number of `newdata` studies) of such lists."))
                aux_array[, , se[i]] <- rlang::eval_tidy(rlang::call2(aux[[ss]][[aux_pars[i]]]$qfun, p = u[ , , se[i], drop = TRUE], !!! aux[[ss]][[aux_pars[i]]]$args))
              }
            } else {
              if (! aux[[ss]] %in% unique(forcats::fct_c(if (has_ipd(object$network)) object$network$ipd$.study else factor(),
                                                   if (has_agd_arm(object$network)) object$network$agd_arm$.study else factor()))) {
                if (n_aux == 1) abort("All elements of `aux` must match the name of an IPD or AgD (arm-based) study in the network, or be a distr() distribution.")
                else abort("All elements of `aux` must match the name of an IPD or AgD (arm-based) study in the network, or be a list of distr() distributions.")
              }

              aux_array[ , , se] <- aux_temp[ , , grep(paste0("\\[\\Q", aux[[ss]], "\\E[\\:,\\]]"), dimnames(aux_temp)[[3]], perl = TRUE), drop = FALSE]
            }
          }
        }
      }
    }

    # For predictive distribution, use delta_new instead of d
    if (predictive_distribution) {
      delta_new <- get_delta_new(object)
      post[ , , dimnames(delta_new)[[3]]] <- delta_new
    }

    # Get cutoffs for ordered models
    if (object$likelihood == "ordered") {
      cc <- as.array(object, pars = "cc")
      n_cc <- dim(cc)[3]
      l_cc <- stringr::str_replace(dimnames(cc)[[3]], "^cc\\[(.+)\\]$", "\\1")
    }

    studies <- levels(forcats::fct_drop(preddat$.study))
    n_studies <- length(studies)
    treatments <- levels(forcats::fct_drop(preddat$.trt))
    n_trt <- length(treatments)

    if (progress) {
      pb <- utils::txtProgressBar(max = n_studies * n_trt, style = 3, width = min(100, getOption("width")))
      on.exit(close(pb))
    }

    # Make prediction arrays
    if (is_surv) {
      # Handle survival models separately - produce predictions study by study

      if (level == "individual") {
        if (type %in% c("survival", "hazard", "cumhaz", "rmst")) {
          outdat <- dplyr::select(preddat, ".study", ".trt", ".obs_id", ".time")
        } else {
          outdat <- dplyr::select(preddat, ".study", ".trt", ".obs_id")
        }
      } else {
        if (type %in% c("survival", "hazard", "cumhaz")) {
          outdat <- dplyr::distinct(preddat, .data$.study, .data$.trt, .data$.obs_id, .data$.time)
        } else if (type == "rmst") {
          outdat <- dplyr::distinct(preddat, .data$.study, .data$.trt, .data$.time)
        } else {
          outdat <- dplyr::distinct(preddat, .data$.study, .data$.trt)
        }
      }

      if (type == "quantile") {
        outdat <- dplyr::cross_join(outdat,
                                    data.frame(.quantile = quantiles))
      }

      studies <- levels(forcats::fct_drop(outdat$.study))
      n_studies <- length(studies)
      treatments <- levels(forcats::fct_drop(outdat$.trt))
      n_trt <- length(treatments)

      if (!is.null(object$aux_regression)) {
        X_aux <- make_nma_model_matrix(object$aux_regression,
                                       xbar = object$xbar,
                                       dat_ipd = preddat)$X_ipd
        beta_aux <- as.array(object, pars = "beta_aux")
      }

      aux_int <- aux_needs_integration(aux_regression = object$aux_regression, aux_by = object$aux_by)

      dim_pred_array <- dim(post)
      dim_pred_array[3] <- nrow(outdat)
      dimnames_pred_array <- dimnames(post)

      if (level == "individual") {
        if (type == "quantile") {
          dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, ", ", outdat$.obs_id, ", ", outdat$.quantile, "]")
        } else {
          dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, ", ", outdat$.obs_id, "]")
        }
      } else if (type %in% c("survival", "hazard", "cumhaz")) {
        dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, ", ", outdat$.obs_id, "]")
      } else if (type == "quantile") {
        dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, ", ", outdat$.quantile, "]")
      } else {
        dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, "]")
      }

      pred_array <- array(NA_real_,
                          dim = dim_pred_array,
                          dimnames = dimnames_pred_array)

      for (s in 1:n_studies) {

        # Basis for mspline models
        if (object$likelihood %in% c("mspline", "pexp")) {
          if (!is.null(aux)) {
            basis <- object$basis[[aux[[studies[s]]]]]
          } else {
            basis <- object$basis[[studies[s]]]
          }
        } else {
          basis <- NULL
        }

        for (trt in 1:n_trt) {
          #if (progress) cglue("\rCalculating predictions for {studies[s]}, {treatments[trt]} [{(s-1)*n_trt + trt} / {n_studies * n_trt}]", sep = "")

          # Collapse preddat by unique rows for efficiency
          collapse_by <- setdiff(colnames(preddat), c(".time", ".obs_id", ".sample_size"))

          if (packageVersion("dplyr") > "1.1.0") {
            pd_undups <- dplyr::filter(preddat, .data$.study == studies[s], .data$.trt == treatments[trt]) %>%
              dplyr::distinct(dplyr::pick(dplyr::all_of(collapse_by))) %>%
              dplyr::mutate(.dup_id = seq_len(dplyr::n()))

            if (nrow(pd_undups) == 0) next

            pd_col <- dplyr::filter(preddat, .data$.study == studies[s], .data$.trt == treatments[trt]) %>%
              dplyr::left_join(pd_undups, by = collapse_by) %>%
              dplyr::group_by(dplyr::pick(dplyr::all_of(collapse_by))) %>%
              dplyr::mutate(.ndup = dplyr::n(),
                            .undup = dplyr::if_else(.data$.ndup > 1, 1 == 1:dplyr::n(), rep(TRUE, times = dplyr::n())))
          } else {
            pd_undups <- dplyr::filter(preddat, .data$.study == studies[s], .data$.trt == treatments[trt]) %>%
              dplyr::distinct(dplyr::across(dplyr::all_of(collapse_by))) %>%
              dplyr::mutate(.dup_id = seq_len(dplyr::n()))

            if (nrow(pd_undups) == 0) next

            pd_col <- dplyr::filter(preddat, .data$.study == studies[s], .data$.trt == treatments[trt]) %>%
              dplyr::left_join(pd_undups, by = collapse_by) %>%
              dplyr::group_by(dplyr::across(dplyr::all_of(collapse_by))) %>%
              dplyr::mutate(.ndup = dplyr::n(),
                            .undup = dplyr::if_else(.data$.ndup > 1, 1 == 1:dplyr::n(), rep(TRUE, times = dplyr::n())))
          }


          # Select corresponding rows
          ss_uncol <- which(preddat$.study == studies[s] & preddat$.trt == treatments[trt])
          ss <- ss_uncol[pd_col$.undup]

          # Linear predictor array for this study
          eta_pred_array <- tcrossprod_mcmc_array(post, X_all[ss, , drop = FALSE])

          if (!is.null(offset_all))
            eta_pred_array <- sweep(eta_pred_array, 3, offset_all[ss], FUN = "+")

          # Aux select
          aux_l <- get_aux_labels(preddat[ss, ], by = object$aux_by)
          aux_id <- get_aux_id(preddat[ss, ], by = object$aux_by)

          if (!aux_int) { #(length(setdiff(object$aux_by, c(".study", ".trt"))) == 0) {
            aux_s <- grepl(paste0("\\[(\\Q", paste(aux_l, collapse = "\\E|\\Q"), if (object$likelihood %in% c("mspline", "pexp")) "\\E)," else "\\E)\\]"),
                           dimnames(aux_array)[[3]])

            # Add in arm-level aux regression terms, if present
            if (!is.null(object$aux_regression)) {
              aux_array_s <- make_aux_predict(aux = aux_array[ , , aux_s, drop = FALSE],
                                              beta_aux = beta_aux,
                                              X_aux = X_aux[ss[1], , drop = FALSE],
                                              likelihood = object$likelihood)
            } else {
              aux_array_s <- aux_array[ , , aux_s, drop = FALSE]
            }

          } else {
            # Aux regression or stratified aux pars within arm, need to expand these out over the individuals/integration points

            if (object$likelihood %in% c("mspline", "pexp")) {
              aux_array_s <- array(NA_real_, dim = c(dim(eta_pred_array), length(basis)))
              for (i in 1:length(basis)) {
                aux_s <- grep(paste0("\\[(\\Q", paste(aux_l, collapse = "\\E|\\Q"), "\\E), ", i, "\\]"),
                              dimnames(aux_array)[[3]])
                aux_array_s[ , , , i] <- aux_array[ , , aux_s[aux_id]]
              }
            } else if (object$likelihood == "gengamma") {
              aux_array_s <- array(NA_real_, dim = c(dim(eta_pred_array), 2),
                                   dimnames = list(iterations = NULL, chains = NULL, parameters = NULL, aux = c("sigma[]", "k[]")))
              sigma_s <- grep(paste0("^sigma\\[(\\Q", paste(aux_l, collapse = "\\E|\\Q"), "\\E)", "\\]"),
                            dimnames(aux_array)[[3]])
              k_s <- grep(paste0("^k\\[(\\Q", paste(aux_l, collapse = "\\E|\\Q"), "\\E)", "\\]"),
                              dimnames(aux_array)[[3]])
              aux_array_s[ , , , "sigma[]"] <- aux_array[ , , sigma_s[aux_id]]
              aux_array_s[ , , , "k[]"] <- aux_array[ , , k_s[aux_id]]

            } else {
              aux_s <- grep(paste0("\\[(\\Q", paste(aux_l, collapse = "\\E|\\Q"), "\\E)\\]"),
                            dimnames(aux_array)[[3]])
              aux_array_s <- aux_array[ , , aux_s[aux_id], drop = FALSE]
            }

            # Deal with aux regression
            if (!is.null(object$aux_regression)) {
              if (object$likelihood %in% c("mspline", "pexp", "gengamma")) for (i in 1:length(ss)) {
                aux_array_s[ , , i, ] <- make_aux_predict(aux = aux_array_s[ , , i, , drop = TRUE],
                                                              beta_aux = beta_aux,
                                                              X_aux = X_aux[ss[i], , drop = FALSE],
                                                              likelihood = object$likelihood)
              } else for (i in 1:length(ss)) {
                aux_array_s[ , , i] <- make_aux_predict(aux = aux_array_s[ , , i, drop = FALSE],
                                                            beta_aux = beta_aux,
                                                            X_aux = X_aux[ss[i], , drop = FALSE],
                                                            likelihood = object$likelihood)
              }
            }
          }


          if (type %in% c("survival", "hazard", "cumhaz")) {
            s_time <- preddat$.time[ss_uncol]
          } else if (type == "rmst") {
            s_time <- preddat$.time[ss_uncol]
            if (length(unique(s_time)) == 1) {
              # Same restriction time for all obs, just take the first (faster)
              s_time <- s_time[1]
            }
          } else {
            s_time <- NULL
          }

          # Aggregate predictions when level = "aggregate"
          if (level == "aggregate") {
            s_preddat <-
              dplyr::select(preddat[ss_uncol, ], ".study", ".trt", ".sample_size", if (type %in% c("survival", "hazard", "cumhaz")) ".obs_id" else NULL) %>%
              dplyr::mutate(.study = forcats::fct_inorder(forcats::fct_drop(.data$.study)),
                            .trt = forcats::fct_inorder(forcats::fct_drop(.data$.trt)))

            if (type %in% c("survival", "hazard", "cumhaz")) {
              s_preddat <- dplyr::group_by(s_preddat, .data$.study, .data$.trt, .data$.obs_id)
            } else {
              s_preddat <- dplyr::group_by(s_preddat, .data$.study, .data$.trt)
            }

            s_preddat <- dplyr::mutate(s_preddat, .weights = .data$.sample_size / sum(.data$.sample_size))

            pred_array[ , , outdat$.study == studies[s] & outdat$.trt == treatments[trt]] <-
              make_agsurv_predict(eta = eta_pred_array[, , pd_col$.dup_id, drop = FALSE],
                                  aux = if (!aux_int) aux_array_s
                                        else if (object$likelihood %in% c("mspline", "pexp", "gengamma")) aux_array_s[ , , pd_col$.dup_id, , drop = FALSE]
                                        else aux_array_s[ , , pd_col$.dup_id, drop = FALSE],
                                  times = s_time,
                                  quantiles = quantiles,
                                  likelihood = object$likelihood,
                                  type = type,
                                  basis = basis,
                                  weights = s_preddat$.weights,
                                  id = dplyr::group_indices(s_preddat))


          } else { # Individual predictions
            s_pred_array <-
              make_surv_predict(eta = eta_pred_array[, , pd_col$.dup_id, drop = FALSE],
                                aux = if (!aux_int) aux_array_s
                                      else if (object$likelihood %in% c("mspline", "pexp", "gengamma")) aux_array_s[ , , pd_col$.dup_id, , drop = FALSE]
                                      else aux_array_s[ , , pd_col$.dup_id, drop = FALSE],
                                times = s_time,
                                quantiles = quantiles,
                                likelihood = object$likelihood,
                                type = type,
                                basis = basis)

            pred_array[ , , outdat$.study == studies[s] & outdat$.trt == treatments[trt]] <- s_pred_array
          }

          if (progress) utils::setTxtProgressBar(pb, (s-1)*n_trt + trt)
        }
      }

    } else if (level == "individual") {

      # Get prediction array
      pred_array <- tcrossprod_mcmc_array(post, X_all)

      if (!is.null(offset_all))
        pred_array <- sweep(pred_array, 3, offset_all, FUN = "+")

      # Get predictions for each category for ordered models
      if (object$likelihood == "ordered") {
        # Apply cutoffs
        d_p <- d_pt <- dim(pred_array)
        d_pt[3] <- d_p[3]*n_cc
        dn_p <- dn_pt <- dimnames(pred_array)
        dn_pt[[3]] <- rep(dn_p[[3]], each = n_cc)
        pred_temp <- array(dim = d_pt, dimnames = dn_pt)

        for (i in 1:d_p[3]) {
          pred_temp[ , , (i-1)*n_cc + 1:n_cc] <- sweep(cc, 1:2, pred_array[ , , i, drop = FALSE],
                                                       FUN = function(x, y) {y - x})
          dn_pt[[3]][(i-1)*n_cc + 1:n_cc] <- paste0(stringr::str_sub(dn_p[[3]][i], start = 1, end = -2),
                                                    ", ", l_cc, "]")
        }
        dimnames(pred_temp) <- dn_pt
        pred_array <- pred_temp
      }

      # Transform predictions if type = "response"
      if (type == "response") {
        pred_array <- inverse_link(pred_array, link = object$link)
      }

      if (progress) utils::setTxtProgressBar(pb, n_studies * n_trt)

    } else { # Predictions aggregated over each population

      # Produce aggregated predictions study by study - more memory efficient
      outdat <- dplyr::distinct(preddat, .data$.study, .data$.trt)

      if (object$likelihood == "ordered") {
        outdat <- tibble::tibble(.study = rep(outdat$.study, each = n_cc),
                                 .trt = rep(outdat$.trt, each = n_cc),
                                 .cc = rep(l_cc, times = nrow(outdat)))
      }

      studies <- levels(forcats::fct_drop(outdat$.study))
      n_studies <- length(studies)
      treatments <- levels(forcats::fct_drop(outdat$.trt))
      n_trt <- length(treatments)

      dim_pred_array <- dim(post)
      dim_pred_array[3] <- nrow(outdat)
      dimnames_pred_array <- dimnames(post)
      if (object$likelihood == "ordered") {
        dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, ", ", outdat$.cc, "]")
      } else {
        dimnames_pred_array[[3]] <- paste0("pred[", outdat$.study, ": ", outdat$.trt, "]")
      }

      pred_array <- array(NA_real_,
                          dim = dim_pred_array,
                          dimnames = dimnames_pred_array)

      brmr <- grepl("(^\\.mu\\:)|(\\:\\.mu$)", colnames(X_all))

      for (s in 1:n_studies) {

        # if (progress) cglue("\rCalculating predictions for {studies[s]} [{s} / {n_studies}]", sep = "")

        # Study select
        ss <- preddat$.study == studies[s]

        # Get prediction array for this study
        s_pred_array <- tcrossprod_mcmc_array(
          post[, , !brmr, drop = FALSE],
          X_all[ss, !brmr, drop = FALSE]
        )

        if (any(brmr)) {
          s_pred_array <- s_pred_array +
            tcrossprod_mcmc_array(
              post[, , brmr, drop = FALSE],
              X_all[ss, brmr, drop = FALSE]
            ) * (
              tcrossprod_mcmc_array(
                post[, , grepl("^mu\\[", colnames(X_all)), drop = FALSE],
                X_all[ss, grepl("^mu\\[", colnames(X_all)), drop = FALSE]
              ) - object$xbar[[".mu"]]
            )

        }

        if (!is.null(offset_all))
          s_pred_array <- sweep(s_pred_array, 3, offset_all[ss], FUN = "+")

        # Get predictions for each category for ordered models
        if (object$likelihood == "ordered") {
          # Apply cutoffs
          d_p <- d_pt <- dim(s_pred_array)
          d_pt[3] <- d_p[3]*n_cc
          dn_p <- dn_pt <- dimnames(s_pred_array)
          dn_pt[[3]] <- rep(dn_p[[3]], each = n_cc)
          pred_temp <- array(dim = d_pt, dimnames = dn_pt)

          for (i in 1:d_p[3]) {
            pred_temp[ , , (i-1)*n_cc + 1:n_cc] <- sweep(cc, 1:2, s_pred_array[ , , i, drop = FALSE],
                                                         FUN = function(x, y) {y - x})
            dn_pt[[3]][(i-1)*n_cc + 1:n_cc] <- paste0(stringr::str_sub(dn_p[[3]][i], start = 1, end = -2),
                                                      ", ", l_cc, "]")
          }
          dimnames(pred_temp) <- dn_pt
          s_pred_array <- pred_temp
        }

        # Transform predictions if type = "response"
        if (type == "response") {
          s_pred_array <- inverse_link(s_pred_array, link = object$link)
        }

        # Aggregate predictions since level = "aggregate"
        if (object$likelihood == "ordered") {
          s_preddat <- preddat[ss, c(".study", ".trt", ".sample_size")] %>%
            dplyr::slice(rep(seq_len(nrow(.)), each = n_cc)) %>%
            dplyr::mutate(.study = forcats::fct_inorder(forcats::fct_drop(.data$.study)),
                          .trt = forcats::fct_inorder(.data$.trt),
                          .cc = forcats::fct_inorder(rep_len(l_cc, nrow(.)))) %>%
            dplyr::group_by(.data$.study, .data$.trt, .data$.cc) %>%
            dplyr::mutate(.weights = .data$.sample_size / sum(.data$.sample_size))

          X_weighted_mean <- Matrix::Matrix(0, ncol = dim(s_pred_array)[3], nrow = dplyr::n_distinct(s_preddat$.trt) * n_cc)

          X_weighted_mean[cbind(dplyr::group_indices(s_preddat),
                                1:dim(s_pred_array)[3])] <- s_preddat$.weights
        } else {
          s_preddat <- preddat[ss, c(".study", ".trt", ".sample_size")] %>%
            dplyr::mutate(.study = forcats::fct_inorder(forcats::fct_drop(.data$.study)),
                          .trt = forcats::fct_inorder(.data$.trt)) %>%
            dplyr::group_by(.data$.study, .data$.trt) %>%
            dplyr::mutate(.weights = .data$.sample_size / sum(.data$.sample_size))

          X_weighted_mean <- Matrix::Matrix(0, ncol = dim(s_pred_array)[3], nrow = dplyr::n_distinct(s_preddat$.trt))

          X_weighted_mean[cbind(dplyr::group_indices(s_preddat),
                                1:dim(s_pred_array)[3])] <- s_preddat$.weights
        }

        pred_array[ , , outdat$.study == studies[s]] <- tcrossprod_mcmc_array(s_pred_array, X_weighted_mean)

        if (progress) utils::setTxtProgressBar(pb, s * n_trt)

      }

      preddat <- dplyr::distinct(preddat, .data$.study, .data$.trt)
    }

    # if (progress) cat("\rDone!")

    # Produce nma_summary
    if (object$likelihood == "ordered") {
      pred_meta <- tibble::tibble(.study = rep(preddat$.study, each = n_cc),
                                  .trt = rep(preddat$.trt, each = n_cc),
                                  .category = rep(l_cc, times = nrow(preddat)))
    } else if (is_surv) {
      pred_meta <- tibble::tibble(.study = outdat$.study,
                                  .trt = outdat$.trt)
      if (type %in% c("survival", "hazard", "cumhaz", "rmst")) {
        pred_meta <- tibble::add_column(pred_meta, .time = outdat$.time, .after = ".trt")
      } else if (type == "quantile") {
        pred_meta <- tibble::add_column(pred_meta, .quantile = outdat$.quantile, .after = ".trt")
      }
    } else {
      pred_meta <- tibble::tibble(.study = preddat$.study,
                                  .trt = preddat$.trt)
    }

    if (summary) {
      pred_summary <- summary_mcmc_array(pred_array, probs)
      pred_summary <- dplyr::bind_cols(pred_meta, pred_summary)
    } else {
      pred_summary <- pred_meta
    }

    out <- list(summary = pred_summary, sims = pred_array)

  }

  if (summary) {
    attr(out, "xlab") <- "Treatment"
    attr(out, "ylab") <- get_scale_name(likelihood = object$likelihood,
                                        link = object$link,
                                        measure = "absolute",
                                        type = type)
    if (object$likelihood == "ordered") {
      class(out) <- c("ordered_nma_summary", "nma_summary")
    } else if (type %in% c("survival", "hazard", "cumhaz")) {
      class(out) <- c("surv_nma_summary", "nma_summary")
      attr(out, "xlab") <- "Time"
    } else {
      class(out) <- "nma_summary"
    }
  }

  if (progress) cat("\r")

  return(out)
}


#' @param times A numeric vector of times to evaluate predictions at.
#'   Alternatively, if `newdata` is specified, `times` can be the name of a
#'   column in `newdata` which contains the times. If `NULL` (the default) then
#'   predictions are made at the event/censoring times from the studies included
#'   in the network (or according to `times_seq`). Only used if `type` is
#'   `"survival"`, `"hazard"`, `"cumhaz"` or `"rmst"`.
#' @param aux An optional [distr()] distribution for the auxiliary parameter(s)
#'   in the baseline hazard (e.g. shapes). Can also be a character string naming
#'   a study in the network to take the estimated auxiliary parameter
#'   distribution from. If `NULL`, predictions are produced using the parameter
#'   estimates for each study in the network with IPD or arm-based AgD.
#'
#'   For regression models, this may be a list of [distr()] distributions (or
#'   study names in the network to use the auxiliary parameters from) of the
#'   same length as the number of studies in `newdata`, possibly named by the
#'   study names or otherwise in order of appearance in `newdata`.
#' @param quantiles A numeric vector of quantiles of the survival time
#'   distribution to produce estimates for when `type = "quantile"`.
#' @param times_seq A positive integer, when specified evaluate predictions at
#'   this many evenly-spaced event times between 0 and the latest follow-up time
#'   in each study, instead of every observed event/censoring time. Only used if
#'   `newdata = NULL` and `type` is `"survival"`, `"hazard"` or `"cumhaz"`. This
#'   can be useful for plotting survival or (cumulative) hazard curves, where
#'   prediction at every observed even/censoring time is unnecessary and can be
#'   slow. When a call from within `plot()` is detected, e.g. like
#'   `plot(predict(fit, type = "survival"))`, `times_seq` will default to 50.
#'
#' @export
#' @rdname predict.stan_nma
predict.stan_nma_surv <- function(object, times = NULL,
                                  ...,
                                  baseline_trt = NULL,
                                  baseline = NULL,
                                  aux = NULL,
                                  newdata = NULL, study = NULL,
                                  type = c("survival", "hazard", "cumhaz", "mean", "median", "quantile", "rmst", "link"),
                                  quantiles = c(0.25, 0.5, 0.75),
                                  level = c("aggregate", "individual"),
                                  times_seq = NULL,
                                  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                                  predictive_distribution = FALSE,
                                  expand = TRUE,
                                  summary = TRUE,
                                  progress = interactive(),
                                  trt_ref = NULL) {

  type <- rlang::arg_match(type)
  times <- rlang::enquo(times)

  if (!is.null(newdata)) study <- pull_non_null(newdata, rlang::enquo(study))

  if (!rlang::is_double(quantiles, finite = TRUE) || any(quantiles < 0) || any(quantiles > 1))
    abort("`quantiles` must be a numeric vector of quantiles between 0 and 1.")

  if (!is.null(times_seq) && (!rlang::is_integerish(times_seq, n = 1, finite = TRUE) || times_seq <= 0))
    abort("`times_seq` must be a single positive integer.")

  # Set times_seq by default if called within plot()
  if (is.null(times_seq) && type %in% c("survival", "hazard", "cumhaz") &&
      "plot" %in% unlist(lapply(sys.calls(), function(x) deparse(x[[1]]))))
    times_seq <- 50

  # Other checks (including times, aux) in predict.stan_nma()
  # Need to pass stan_nma_surv-specific args directly, otherwise these aren't picked up by NextMethod()
  NextMethod(times = times,
             aux = aux,
             type = type,
             quantiles = quantiles,
             times_seq = times_seq,
             baseline_level = "individual",
             baseline_type = "link",
             progress = progress)
}

#' Produce survival predictions from arrays of linear predictors and auxiliary parameters
#'
#' Designed to work with a single arm at a time
#'
#' @param eta Array of samples from the linear predictor
#' @param aux Array of samples of auxiliary parameters
#' @param times Vector of evaluation times, or time horizon for type = "rmst"
#' @param likelihood Likelihood function to use
#' @param type Type of prediction to create
#' @param quantiles Quantiles for type = "quantile"
#' @param basis M-spline basis
#'
#' @noRd
make_surv_predict <- function(eta, aux, times, likelihood,
                              type = c("survival", "hazard", "cumhaz", "mean", "median", "quantile", "rmst", "link"),
                              quantiles = c(0.25, 0.5, 0.75),
                              basis = NULL) {

  if (type == "link") return(eta)

  # Check for times beyond mspline boundary knots
  if (likelihood == "mspline") {
    lower <- attr(basis, "Boundary.knots")[1]
    upper <- attr(basis, "Boundary.knots")[2]
    if (! type %in% c("median", "quantile") && (type == "mean" || any(times < lower) || any(times > upper))) warn("Evaluating M-spline at times beyond the boundary knots.")
  }

  d_out <- dim(eta)
  dn_out <- dimnames(eta)

  n_eta <- dim(eta)[3]

  if (type %in% c("survival", "hazard", "cumhaz") && n_eta == 1) {
    dn_out[[3]] <- paste0(rep(stringr::str_sub(dn_out[[3]], start = 1, end = -2), each = length(times)),
                          ", ", rep(seq_along(times), times = d_out[3]), "]")
    d_out[3] <- d_out[3] * length(times)
  }

  if (type == "quantile") {
    dn_out[[3]] <- paste0(rep(stringr::str_sub(dn_out[[3]], start = 1, end = -2), each = length(quantiles)),
                          ", ", rep(quantiles, times = d_out[3]), "]")
    d_out[3] <- d_out[3] * length(quantiles)
  }

  out <- array(NA_real_, dim = d_out, dimnames = dn_out)

  if (type %in% c("survival", "hazard", "cumhaz") && n_eta == 1) { # Multiple times, single linear predictor
    if (!is.null(aux)) aux <- matrix(aux, ncol = dim(aux)[3], dimnames = list(NULL, dimnames(aux)[[3]]))
    for (i in 1:length(times)) {
      out[, , i] <- surv_predfuns[[likelihood]][[type]](times = times[i],
                                                        eta = as.vector(eta),
                                                        aux = aux,
                                                        quantiles = quantiles,
                                                        basis = basis)
    }
  } else if (n_eta > 1) { # Single/multiple times, multiple linear predictors
    if (type == "quantile") {
      iinc <- length(quantiles)
      quantiles <- rep(quantiles, each = length(eta[ , , 1]))
    } else {
      iinc <- 1
    }
    for (i in 1:n_eta) {
      ti <- if (length(times) == 1) times else times[i]
      auxi <- NULL
      if (!is.null(aux)) {
        if (length(dim(aux)) == 4) {
          auxi <- matrix(aux[ , , i, ], ncol = dim(aux)[4], dimnames = list(NULL, dimnames(aux)[[4]]))
        } else if (likelihood %in% c("gengamma", "mspline", "pexp")) {
          auxi <- matrix(aux, ncol = dim(aux)[3], dimnames = list(NULL, dimnames(aux)[[3]]))
        } else if (dim(aux)[3] == 1) {
          auxi <- as.vector(aux)
        } else {
          auxi <- as.vector(aux[ , , i])
        }
      }
      out[ , , ((i-1)*iinc+1):(i*iinc)] <-
        surv_predfuns[[likelihood]][[type]](times = ti,
                                            eta = as.vector(eta[ , , i]),
                                            aux = auxi,
                                            quantiles = quantiles,
                                            basis = basis)
    }
  } else { # Single time, single linear predictor
    if (!is.null(aux)) aux <- matrix(aux, ncol = dim(aux)[3], dimnames = list(NULL, dimnames(aux)[[3]]))
    if (type == "quantile") quantiles <- rep(quantiles, each = length(eta))
    out <- array(surv_predfuns[[likelihood]][[type]](times = times,
                                                     eta = as.vector(eta),
                                                     aux = aux,
                                                     quantiles = quantiles,
                                                     basis = basis),
                 dim = d_out, dimnames = dn_out)
  }

  # Check for times beyond mspline boundary knots
  if (likelihood == "mspline" && type %in% c("median", "quantile")) {
    if (any(out < lower) || any(out > upper)) warn("Evaluating M-spline at times beyond the boundary knots.")
  }

  return(out)
}

#' Produce aggregate (standardised) survival predictions from arrays of linear predictors and auxiliary parameters
#'
#' Designed to work with a single arm at a time
#'
#' @param eta Array of samples from the linear predictor
#' @param aux Array of samples of auxiliary parameters
#' @param times Vector of evaluation times, or time horizon for type = "rmst"
#' @param likelihood Likelihood function to use
#' @param type Type of prediction to create
#' @param quantiles Quantiles for type = "quantile"
#' @param basis M-spline basis
#' @param weights Weights for weighted mean over the population
#' @param id Time ID for survival/hazard predictions
#'
#' @noRd
make_agsurv_predict <- function(eta, aux, times, likelihood,
                                type = c("survival", "hazard", "cumhaz", "mean", "median", "quantile", "rmst", "link"),
                                quantiles = c(0.25, 0.5, 0.75),
                                basis = NULL, weights, id) {

  if (type %in% c("survival", "cumhaz", "link")) {

    X_weighted_mean <- Matrix::Matrix(0, ncol = dim(eta)[3], nrow = max(id))

    X_weighted_mean[cbind(id, 1:dim(eta)[3])] <- weights

    pred_array <-
      make_surv_predict(eta = eta,
                        aux = aux,
                        times = times,
                        quantiles = quantiles,
                        likelihood = likelihood,
                        type = if (type == "cumhaz") "survival" else type,
                        basis = basis)

    out <- tcrossprod_mcmc_array(pred_array, X_weighted_mean)

    if (type == "cumhaz") out <- -log(out)

  } else if (type == "hazard") {

    X_weighted_mean <- Matrix::Matrix(0, ncol = dim(eta)[3], nrow = max(id))

    X_weighted_mean[cbind(id, 1:dim(eta)[3])] <- weights

    S_array <-
      make_surv_predict(eta = eta,
                        aux = aux,
                        times = times,
                        quantiles = quantiles,
                        likelihood = likelihood,
                        type = "survival",
                        basis = basis)

    h_array <-
      make_surv_predict(eta = eta,
                        aux = aux,
                        times = times,
                        quantiles = quantiles,
                        likelihood = likelihood,
                        type = "hazard",
                        basis = basis)

    Shbar <- tcrossprod_mcmc_array(S_array * h_array, X_weighted_mean)
    Sbar <- tcrossprod_mcmc_array(S_array, X_weighted_mean)
    out <- Shbar / Sbar

    # Fix up edge case with all S=0
    S0 <- which(Shbar == 0 & Sbar == 0)
    out[S0] <- 0

  } else if (type %in% c("median", "quantile")) {

    if (type == "median") quantiles <- 0.5

    out <- array(NA_real_, dim = c(dim(eta)[1:2], length(quantiles)))

    for (i in 1:length(quantiles)) {
      out[ , , i] <- quantile_Sbar(p = quantiles[i],
                           eta = eta,
                           weights = weights,
                           aux = aux,
                           likelihood = likelihood,
                           basis = basis)
    }

  } else if (type %in% c("mean", "rmst")) {

    if (type == "mean") times <- Inf

    out <- array(NA_real_, dim = c(dim(eta)[1:2], length(times)))

    for (i in 1:dim(out)[1]) for (j in 1:dim(out)[2]) {
      if (is.null(aux)) {
        auxi <- NULL
      } else if (length(dim(aux)) == 4) {
        auxi <- aux[i, j, , ]
      } else if (dim(aux)[[3]] > 1) {
        auxi <- matrix(aux[i, j, ], nrow = 1, dimnames = list(NULL, dimnames(aux)[[3]]))
      } else {
        auxi <- aux[i, j, ]
      }
      out[i, j, ] <- rmst_Sbar(times = times,
                               eta = eta[i, j, ],
                               weights = weights,
                               aux = auxi,
                               likelihood = likelihood,
                               basis = basis)
    }
  }

  # Check for times beyond mspline boundary knots
  if (likelihood == "mspline") {
    lower <- attr(basis, "Boundary.knots")[1]
    upper <- attr(basis, "Boundary.knots")[2]
    if (type == "mean" ||
        (type %in% c("survival", "hazard", "cumhaz", "rmst") && (any(times < lower) || any(times > upper))) ||
        (type %in% c("median", "quantile") && (any(out < lower) || any(out > upper))))
      warn("Evaluating M-spline at times beyond the boundary knots.")
  }

  return(out)
}

#' Restricted mean survival times for marginal survival curves
#' Given linear predictor vector evaluated over the population and associated weights
#' @noRd
rmst_Sbar <- function(times, eta, weights, aux, likelihood, basis, start = 0) {
  require_pkg("flexsurv")

  flexsurv::rmst_generic(pSbar, t = times, start = start,
                         eta = eta, weights = weights, aux = aux,
                         likelihood = likelihood, basis = basis,
                         scalarargs = c("basis", "likelihood", "eta", "aux", "weights"))
}

pSbar <- function(times, eta, weights, aux, likelihood, basis) {
  Sbar <- numeric(length(times))
  for (i in 1:length(times)) {
    S <- surv_predfuns[[likelihood]][["survival"]](times = times[i],
                                                   eta = eta,
                                                   aux = aux,
                                                   basis = basis)

    Sbar[i] <- weighted.mean(S, weights)
  }
  return(1 - Sbar)
}


#' Quantile function for marginal survival curves
#' Given linear predictor array evaluated over the population and associated weights
#' @noRd
quantile_Sbar <- function(p, eta, weights, aux, likelihood, basis) {
  # require_pkg("flexsurv")
  #
  # q <- flexsurv::qgeneric(Sbar, p = rep(p, each = prod(dim(eta)[1:2])),
  #                         eta = eta, weights = weights, aux = aux,
  #                         likelihood = likelihood, basis = basis,
  #                         scalarargs = c("basis", "likelihood", "eta", "aux", "weights"),
  #                         lbound = 0)
  #
  # return(q)

  # Faster to call rstpm2::vuniroot directly

  require_pkg("rstpm2")

  upper <- if (!is.null(basis)) attr(basis, "Boundary.knots")[2] else 1

  rstpm2::vuniroot(qSbar,
                   lower = 0,
                   upper = upper,
                   extendInt = "downX",
                   n = prod(dim(eta)[1:2]),
                   p = 1 - p,
                   eta = eta,
                   weights = weights,
                   aux = aux,
                   likelihood = likelihood,
                   basis = basis,
                   tol = .Machine$double.eps)$root

}

qSbar <- function(times, p, eta, weights, aux, likelihood, basis) {
  S <- array(NA_real_, dim = dim(eta))
  auxi <- NULL
  for (i in 1:dim(eta)[3]) {
    if (!is.null(aux)) {
      if (length(dim(aux)) == 4) {
        auxi <- matrix(aux[ , , i, ], ncol = dim(aux)[4], dimnames = list(NULL, dimnames(aux)[[4]]))
      } else if (likelihood %in% c("gengamma", "mspline", "pexp")) {
        auxi <- matrix(aux, ncol = dim(aux)[3], dimnames = list(NULL, dimnames(aux)[[3]]))
      } else if (dim(aux)[3] == 1) {
        auxi <- as.vector(aux)
      } else {
        auxi <- as.vector(aux[ , , i])
      }
    }
    S[ , , i] <- surv_predfuns[[likelihood]][["survival"]](times = times,
                                                           eta = as.vector(eta[ , , i]),
                                                           aux = auxi,
                                                           basis = basis)
  }

  Sbar <- apply(S, MARGIN = 1:2, FUN = weighted.mean, w = weights)

  Sbar - p
}


#' Construct prediction functions programmatically
#' @noRd
make_predfun <- function(base, type, ..., .ns = list()) {

  dots <- rlang::enexprs(...)

  if (!is.null(.ns)) {
    .ns <- purrr::list_modify(list(survival = "flexsurv", hazard = "flexsurv", cumhaz = "flexsurv",
                                   mean = "flexsurv", quantile = "flexsurv", rmst = "flexsurv"),
                              !!! .ns)
  }

  list(
    survival = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$survival == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("p", base), q = quote(times), lower.tail = FALSE, !!! dots, .ns = .ns$survival)
      })),
    hazard = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$hazard == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("h", base), x = quote(times), !!! dots, .ns = .ns$hazard)
      })),
    cumhaz = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$cumhaz == "flexsurv") quote(require_pkg("flexsurv"))
        !! rlang::call2(paste0("H", base), x = quote(times), !!! dots, .ns = .ns$cumhaz)
      })),
    mean = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$mean == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("mean_", base), !!! dots, .ns = .ns$mean)
      })),
    median = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$quantile == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("q", base), p = 0.5, !!! dots, .ns = .ns$quantile)
      })),
    quantile = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$quantile == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("q", base), p = quote(quantiles), !!! dots, .ns = .ns$quantile)
      })),
    rmst = rlang::new_function(rlang::pairlist2(times=, eta=, aux=, quantiles=, basis=), rlang::expr({
      !! if (!is.null(.ns) && .ns$rmst == "flexsurv") quote(require_pkg("flexsurv"))
      !! rlang::call2(paste0("rmst_", base), t = quote(times), !!! dots, .ns = .ns$rmst)
      }))
  )

}

surv_predfuns <- list(
  exponential = make_predfun(base = "exp",
                             rate = exp(eta),
                             .ns = list(survival = "stats", median = "stats", quantile = "stats")),

  weibull = make_predfun(base = "weibullPH",
                         shape = aux, scale = exp(eta)),

  gompertz = make_predfun(base = "gompertz",
                          shape = aux, rate = exp(eta)),

  `exponential-aft` = make_predfun(base = "exp",
                                   rate = exp(-eta),
                                   .ns = list(survival = "stats", median = "stats", quantile = "stats")),

  `weibull-aft` = make_predfun(base = "weibullPH",
                               shape = aux, scale = exp(-aux * eta)),

  lognormal = make_predfun(base = "lnorm",
                           meanlog = eta, sdlog = aux,
                           .ns = list(survival = "stats", median = "stats", quantile = "stats")),

  loglogistic = make_predfun(base = "llogis",
                             shape = aux, scale = exp(eta)),

  gamma = make_predfun(base = "gamma",
                       rate = exp(-eta), shape = aux,
                       .ns = list(survival = "stats", median = "stats", quantile = "stats")),

  gengamma = make_predfun(base = "gengamma",
                          mu = eta,
                          sigma = aux[, grepl("^sigma\\[", colnames(aux))],
                          Q = 1 / sqrt(aux[, grepl("^k\\[", colnames(aux))])),

  mspline = make_predfun(base = "mspline",
                         rate = exp(eta), scoef = aux, basis = basis,
                         .ns = NULL),

  pexp = make_predfun(base = "mspline",
                      rate = exp(eta), scoef = aux, basis = basis,
                      .ns = NULL)
)

#' Produce auxiliary parameters when aux_regression is used
#' @noRd
make_aux_predict <- function(aux, beta_aux, X_aux, likelihood) {

  if (likelihood %in% c("mspline", "pexp")) {

    nX <- ncol(X_aux)
    lscoef <- aperm(apply(aux, 1:2, inv_softmax), c(2, 3, 1))
    lscoef_reg <- lscoef

    for (i in 1:dim(lscoef_reg)[[3]]) {
      lscoef_reg[ , , i] <- lscoef[ , , i, drop = FALSE] + tcrossprod_mcmc_array(beta_aux[ , , ((i-1)*nX + 1):(i*nX), drop = FALSE], X_aux)
    }

    out <- aperm(apply(lscoef_reg, 1:2, softmax), c(2, 3, 1))

  } else if (likelihood == "gengamma") {

    nX <- ncol(X_aux)
    out <- aux
    out[ , , 1] <- exp(log(aux[ , , 1, drop = FALSE]) + tcrossprod_mcmc_array(beta_aux[ , , 1:nX, drop = FALSE], X_aux))
    out[ , , 2] <- exp(log(aux[ , , 2, drop = FALSE]) + tcrossprod_mcmc_array(beta_aux[ , , (nX+1):(2*nX), drop = FALSE], X_aux))

  } else {
    out <- exp(log(aux) + tcrossprod_mcmc_array(beta_aux, X_aux))
  }

  dimnames(out) <- dimnames(aux)
  return(out)
}

#' The nma_nodesplit class
#'
#' The `nma_nodesplit` and `nma_nodesplit_df` classes contains the results from
#' running a node-splitting model with the function [nma()].
#'
#' @rdname nma_nodesplit-class
#' @name nma_nodesplit-class
#' @aliases nma_nodesplit nma_nodesplit_df nma_nodesplit_df-class
#'
#' @details Objects of class `nma_nodesplit` inherit from the [stan_nma] class,
#'   and contain the results of fitting a single node-split model. They have one
#'   additional component, `nodesplit`, which gives the comparison that was
#'   node-split as a length 2 vector.
#'
#'   Objects of class `nma_nodesplit_df` are tibble data frames with one row
#'   for each node-split comparison and columns:
#'   \describe{
#'   \item{`trt1`, `trt2`}{Treatments forming the comparison}
#'   \item{`model`}{A list column containing the results of each model as a
#'   `nma_nodesplit` object}
#'   }
#'   Optionally, there will be an additional row for the consistency model if
#'   this was fitted (e.g. by `get_nodesplits(., include_consistency = TRUE)`)
#'   with `trt1` and `trt2` both `NA`.
#'
NULL

#' Print `nma_nodesplit_df` objects
#'
#' @param x A [nma_nodesplit_df] object
#' @param ... Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}
#'
#' @seealso The summary method [summary.nma_nodesplit_df()] summarises the
#'   node-splitting results.
#'
#' @return `x` is returned invisibly.
#'
#' @export
print.nma_nodesplit_df <- function(x, ...) {
  n_ns <- nrow(dplyr::filter(x, !is.na(.data$trt1) & !is.na(.data$trt2)))

  cglue("Node-splitting model{if (n_ns > 1) 's' else ''} fitted for {n_ns} comparison{if (n_ns > 1) 's' else ''}.")
  cglue("To summarise these results, use `summary()`.")

  for (i in 1:nrow(x)) {
    cglue("")
    if (n_ns > 1) {
      if (is.na(x$trt1[i]) && is.na(x$trt2[i])) { # Consistency model
        sec_header("Consistency model")
      } else {
        sec_header(glue::glue("Node-split {x$trt2[i]} vs. {x$trt1[i]}"))
      }
      cglue("")
    }
    print(x$model[[i]], ...)
  }

  invisible(x)
}

#' @export
#' @rdname print.nma_nodesplit_df
print.nma_nodesplit <- function(x, ...) {
  NextMethod(...)
}

#' Summarise the results of node-splitting models
#'
#' Posterior summaries of node-splitting models (`nma_nodesplit` and
#' `nma_nodesplit_df` objects) can be produced using the `summary()` method, and
#' plotted using the `plot()` method.
#'
#' @param x,object A `nma_nodesplit` or `nma_nodesplit_df` object
#' @param consistency Optional, a `stan_nma` object for the corresponding fitted
#'   consistency model, to display the network estimates alongside the direct
#'   and indirect estimates. The fitted consistency model present in the
#'   `nma_nodesplit_df` object will be used if this is present (see
#'   [get_nodesplits()]).
#' @param ... Additional arguments passed on to other methods
#' @param probs Numeric vector of specifying quantiles of interest, default
#'   `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#'
#' @details The `plot()` method is a shortcut for `plot(summary(nma_nodesplit))`. For
#'   details of plotting options, see [plot.nodesplit_summary()].
#'
#' @return A [nodesplit_summary] object
#' @export
#'
#' @seealso [plot.nodesplit_summary()]
#'
#' @template ex_smoking_nma_re_nodesplit_example
#' @examples \donttest{
#' # Summarise the node-splitting results
#' summary(smk_fit_RE_nodesplit)
#'
#' # Plot the node-splitting results
#' plot(smk_fit_RE_nodesplit)
#' }
summary.nma_nodesplit_df <- function(object, consistency = NULL, ...,
                                     probs = c(0.025, 0.25, 0.5, 0.75, 0.975)) {

  if (any(is.na(object$trt1) & is.na(object$trt2))) { # Consistency model present in object
    consistency <- dplyr::filter(object, is.na(.data$trt1) & is.na(.data$trt2))$model[[1]]

    object <- dplyr::filter(object, !is.na(.data$trt1) | !is.na(.data$trt2))

  } else if (!is.null(consistency)) { # Consistency model provided

    # Check provided consistency object
    if (!inherits(consistency, "stan_nma"))
      abort("`consistency` should be a fitted consistency model (object of class `stan_nma`).")

    if (consistency$consistency != "consistency")
      abort(glue::glue("`consistency` should be a fitted consistency model, not a ",
                       switch(consistency$consistency,
                              nodesplit = "node-splitting",
                              ume = "UME inconsistency"),
                       " model."))

    if (!identical(consistency$stanfit@sim$fnames_oi, setdiff(object$model[[1]]$stanfit@sim$fnames_oi, "omega")))
      abort("The fitted consistency model `consistency` does not match the node-splitting model.")

  }

  out <- object
  out$summary <- purrr::map(out$model, .summarise_ns,
                            cons_mod = consistency, probs = probs)
  out$p_value <- purrr::map_dbl(purrr::map(out$model, as.array, pars = "omega"),
                                ~2 * ifelse(median(.) > 0, mean(. < 0), mean(. > 0)))
  out$dic <- purrr::map(out$model, dic)

  # Drop the model objects from the output
  out$model <- NULL

  class(out) <- c("nodesplit_summary", class(out))

  return(out)
}

#' @export
#' @rdname summary.nma_nodesplit_df
summary.nma_nodesplit <- function(object, consistency = NULL, ...,
                                  probs = c(0.025, 0.25, 0.5, 0.75, 0.975)) {

  # Turn nma_nodesplit into nma_nodesplit_df
  df <- tibble::tibble(trt1 = object$nodesplit[1],
                       trt2 = object$nodesplit[2],
                       model = list(object))

  class(df) <- c("nma_nodesplit_df", class(df))

  summary(df, consistency = consistency, ..., probs = probs)
}

#' @export
#' @rdname summary.nma_nodesplit_df
plot.nma_nodesplit <- function(x, consistency = NULL, ...) {

  # Turn nma_nodesplit into nma_nodesplit_df
  df <- tibble::tibble(trt1 = x$nodesplit[1],
                       trt2 = x$nodesplit[2],
                       model = list(x))

  class(df) <- c("nma_nodesplit_df", class(df))

  plot(summary(df, consistency = consistency), ...)
}

#' @export
#' @rdname summary.nma_nodesplit_df
plot.nma_nodesplit_df <- function(x, consistency = NULL, ...) {
  plot(summary(x, consistency = consistency), ...)
}

# Function to summarise direct, indirect, network effects
.summarise_ns <- function(ns_mod, cons_mod = NULL, ..., probs = c(0.025, 0.25, 0.5, 0.75, 0.975)) {
  omega <- as.array(ns_mod, pars = "omega")
  nodesplit <- ns_mod$nodesplit
  ns_text <- paste0(nodesplit[2], " vs. ", nodesplit[1])

  s_dim <- dim(omega)
  s_dim[3] <- 3 # direct, indirect, omega

  s_dimnames <- dimnames(omega)
  s_dimnames[[3]] <- c(paste0("d_dir[", ns_text, "]"),
                       paste0("d_ind[", ns_text, "]"),
                       "omega")

  if (!is.null(cons_mod)) {
    s_dim[3] <- s_dim[3] + 1
    s_dimnames[[3]] <- c(paste0("d_net[", ns_text, "]"),
                         s_dimnames[[3]])
  }
  if (ns_mod$trt_effects == "random") {
    s_dim[3] <- s_dim[3] + 1
    s_dimnames[[3]] <- c(s_dimnames[[3]], "tau")

    if (!is.null(cons_mod)) {
      s_dim[3] <- s_dim[3] + 1
      s_dimnames[[3]] <- c(s_dimnames[[3]], "tau_consistency")
    }
  }

  sims <- array(dim = s_dim, dimnames = s_dimnames)

  if (!is.null(cons_mod)) {
    if (nodesplit[1] == levels(cons_mod$network$treatments)[1]) {
      sims[ , , paste0("d_net[", ns_text, "]")] <-
        as.array(cons_mod, pars = paste0("d[", nodesplit[2], "]"))
    } else {
      sims[ , , paste0("d_net[", ns_text, "]")] <-
        as.array(cons_mod, pars = paste0("d[", nodesplit[2], "]")) -
        as.array(cons_mod, pars = paste0("d[", nodesplit[1], "]"))
    }
  }

  if (nodesplit[1] == levels(ns_mod$network$treatments)[1]) {
    sims[ , , paste0("d_ind[", ns_text, "]")] <-
      as.array(ns_mod, pars = paste0("d[", nodesplit[2], "]"))
  } else {
    sims[ , , paste0("d_ind[", ns_text, "]")] <-
      as.array(ns_mod, pars = paste0("d[", nodesplit[2], "]")) -
      as.array(ns_mod, pars = paste0("d[", nodesplit[1], "]"))
  }

  # Direct estimate = indirect estimate + inconsistency factor omega
  sims[ , , paste0("d_dir[", ns_text, "]")] <- sims[ , , paste0("d_ind[", ns_text, "]"), drop = FALSE] + omega

  sims[ , , "omega"] <- omega

  if(ns_mod$trt_effects == "random") {
    sims[ , , "tau"] <- as.array(ns_mod, pars = "tau")

    if (!is.null(cons_mod)) {
      sims[ , , "tau_consistency"] <- as.array(cons_mod, pars = "tau")
    }
  }

  out <- list(summary = summary_mcmc_array(sims, probs = probs),
              sims = sims)
  class(out) <- "nma_summary"

  return(out)
}

#' Prior distributions
#'
#' These functions are used to specify prior distributions for the model parameters.
#'
#' @param location Prior location. Typically prior mean (see details).
#' @param scale Prior scale. Typically prior standard deviation (see details).
#' @param df Prior degrees of freedom.
#' @param rate Prior rate.
# #' @param lower,upper Lower and upper bounds for a uniform prior distribution.
#'
#' @rdname priors
#' @name priors
#' @aliases priors normal
#'
#' @details The `location` and `scale` parameters are typically the prior mean
#'   and standard deviation, with the following exceptions:
#'   \itemize{
#'   \item For the Cauchy distribution `location` is the prior median and
#'   `scale` is the prior scale.
#'   \item For the log-Normal distribution, `location` and `scale` are the prior
#'   mean and standard deviation of the logarithm.
#'   }
#'
#' ## Compatibility with model parameters
#' The following table summarises which prior distributions may be used with
#' which model parameters. Essentially, priors that take only non-negative
#' values (e.g. half-Normal) may only be used for non-negative parameters
#' (heterogeneity SD/variance/precision, and any auxiliary parameter). If a
#' real-valued prior distribution is specified for a non-negative parameter, it
#' will be truncated at 0 to be non-negative.
#'
#' |       | \strong{Intercept} `prior_intercept` | \strong{Treatment effects} `prior_trt` | \strong{Heterogeneity} `prior_het` | \strong{Regression coefficients} `prior_reg` | \strong{Auxiliary parameter} `prior_aux` |
#' | ----- | :---: | :---: | :---: | :---: | :---: |
#' | \strong{Normal} `normal()` | Yes | Yes | Yes | Yes | Yes |
#' | \strong{half-Normal} `half_normal()` | - | - | Yes | - | Yes |
#' | \strong{log-Normal} `log_normal()` | - | - | Yes | - | Yes |
#' | \strong{Cauchy }`cauchy()` | Yes | Yes | Yes | Yes | Yes |
#' | \strong{half-Cauchy} `half_cauchy()` | - | - | Yes | - | Yes |
#' | \strong{Student t} `student_t()` | Yes | Yes | Yes | Yes | Yes |
#' | \strong{half-Student t} `half_student_t()` | - | - | Yes | - | Yes |
#' | \strong{log-Student t} `log_student_t()` | - | - | Yes | - | Yes |
#' | \strong{Exponential} `exponential()` | - | - | Yes | - | Yes |
#' | \strong{Flat} `flat()` | Yes | Yes | Yes | Yes | Yes |
#'
#' The `flat()` prior is a special case where no prior information is added to
#' the model, resulting in an implicit flat uniform prior distribution over the
#' entire support for a parameter. This will be an improper prior if the
#' parameter is unbounded, and is not generally advised. See the
#' [Stan user's guide](https://mc-stan.org/docs/stan-users-guide/some-differences-in-the-statistical-models-that-are-allowed.html)
#' for more details.
#'
#' @return Object of class [nma_prior].
#' @export
#'
#' @seealso [summary.nma_prior()] for summarising details of prior
#'   distributions. [plot_prior_posterior()] for plots comparing the prior and
#'   posterior distributions of model parameters.
normal <- function(location = 0, scale) {
  check_prior_location(location)
  check_prior_scale(scale)

  return(new_nma_prior("Normal", location = location, scale = scale))
}

#' @rdname priors
#' @export
half_normal <- function(scale) {
  check_prior_scale(scale)

  return(new_nma_prior("half-Normal", location = 0, scale = scale))
}

#' @rdname priors
#' @export
log_normal <- function(location, scale) {
  check_prior_location(location)
  check_prior_scale(scale)

  return(new_nma_prior("log-Normal", location = location, scale = scale))
}

#' @rdname priors
#' @export
cauchy <- function(location = 0, scale) {
  check_prior_location(location, type = "location (median)")
  check_prior_scale(scale, type = "scale")

  return(new_nma_prior("Cauchy", location = location, scale = scale))
}

#' @rdname priors
#' @export
half_cauchy <- function(scale) {
  check_prior_scale(scale, type = "scale")

  return(new_nma_prior("half-Cauchy", location = 0, scale = scale))
}

#' @rdname priors
#' @export
student_t <- function(location = 0, scale, df) {
  check_prior_location(location)
  check_prior_scale(scale)
  check_prior_scale(df, type = "degrees of freedom")

  return(new_nma_prior("Student t", location = location, scale = scale, df = df))
}

#' @rdname priors
#' @export
half_student_t <- function(scale, df) {
  check_prior_scale(scale)
  check_prior_scale(df, type = "degrees of freedom")

  return(new_nma_prior("half-Student t", location = 0, scale = scale, df = df))
}

#' @rdname priors
#' @export
log_student_t <- function(location, scale, df) {
  check_prior_location(location)
  check_prior_scale(scale)
  check_prior_scale(df, type = "degrees of freedom")

  return(new_nma_prior("log-Student t", location = location, scale = scale, df = df))
}

#' @rdname priors
#' @export
exponential <- function(scale = 1/rate, rate = 1/scale) {
  if (missing(scale) && missing(rate))
    abort("Missing argument. Specify either `rate` or `scale`.")
  if (!missing(rate) && !missing(scale))
    warn("Both `rate` and `scale` provided, only `scale` will be used")

  check_prior_scale(scale, type = "scale or rate")

  return(new_nma_prior("Exponential", scale = scale))
}

#' @rdname priors
#' @export
flat <- function() {
  return(new_nma_prior("flat (implicit)"))
}

# #' @rdname priors
# #' @export
# uniform <- function(lower, upper) {
#   check_prior_location(lower, type = "lower limit")
#   check_prior_location(lower, type = "upper limit")
#
#   return(new_nma_prior("Uniform", lower = lower, upper = upper))
# }

# Check functions
check_prior_location <- function(x, type = "location (mean)") {
  if (!(.is_default(x) && rlang::is_empty(x))) {
    if (!is.numeric(x)) abort(glue::glue("Prior {type} must be numeric."))
    if (length(x) > 1) abort(glue::glue("Prior {type} must be numeric, length 1."))
  }
}

check_prior_scale <- function(x, type = "scale (standard deviation)") {
  if (!(.is_default(x) && rlang::is_empty(x))) {
    if (!is.numeric(x)) abort(glue::glue("Prior {type} must be numeric."))
    if (length(x) > 1) abort(glue::glue("Prior {type} must be numeric, length 1."))
    if (x <= 0) abort(glue::glue("Prior {type} must be strictly positive."))
  }
}

new_nma_prior <- function(dist, location = NA_real_, scale = NA_real_, df = NA_real_, ...) {
  o <- list(dist = dist,
            fun = deparse(sys.call(sys.parent())[[1]]),
            location = location,
            scale = scale,
            df = df,
            ...)
  class(o) <- "nma_prior"
  return(o)
}


#' Produce tidy prior details
#'
#' Produces prior details in a data frame, in a suitable format for
#' ggdist::stat_dist_*
#'
#' @param prior A nma_prior object
#' @param trunc Optional vector of truncation limits
#' @param ... Not used
#'
#' @return A data frame
#' @noRd
get_tidy_prior <- function(prior, trunc = NULL, ...) {
  if (!inherits(prior, "nma_prior"))
    abort("Not a `nma_prior` object.")
  if (!is.null(trunc) && (!rlang::is_double(trunc, n = 2) || trunc[2] <= trunc[1]))
    abort("`trunc` should be a numeric length 2 vector of lower and upper truncation limits.")

  d <- prior$dist
  is_trunc <- !is.null(trunc)

  if (d == "Normal") {
    out <- tibble::tibble(dist_label = d,
                          dist = "norm",
                          args = list(list(mean = prior$location, sd = prior$scale)))
  } else if (d == "half-Normal") {
    out <- tibble::tibble(dist_label = d,
                          dist = "trunc",
                          args = list(list(spec = "norm",
                                           a = if (is_trunc) max(0, trunc[1]) else 0,
                                           b = if (is_trunc) trunc[2] else Inf,
                                           mean = prior$location, sd = prior$scale)))
  } else if (d == "log-Normal") {
    out <- tibble::tibble(dist_label = d,
                          dist = "lnorm",
                          args = list(list(meanlog = prior$location, sdlog = prior$scale)))
  } else if (d == "Cauchy") {
    out <- tibble::tibble(dist_label = d,
                          dist = "cauchy",
                          args = list(list(location = prior$location, scale = prior$scale)))
  } else if (d == "half-Cauchy") {
    out <- tibble::tibble(dist_label = d,
                          dist = "trunc",
                          args = list(list(spec = "cauchy",
                                           a = if (is_trunc) max(0, trunc[1]) else 0,
                                           b = if (is_trunc) trunc[2] else Inf,
                                           location = prior$location, scale = prior$scale)))
  } else if (d == "Student t") {
    out <- tibble::tibble(dist_label = d,
                          dist = "gent",
                          args = list(list(location = prior$location, scale = prior$scale, df = prior$df)))
  } else if (d == "half-Student t") {
    out <- tibble::tibble(dist_label = d,
                          dist = "trunc",
                          args = list(list(spec = "gent",
                                           a = if (is_trunc) max(0, trunc[1]) else 0,
                                           b = if (is_trunc) trunc[2] else Inf,
                                           location = prior$location, scale = prior$scale, df = prior$df)))
  } else if (d == "log-Student t") {
    out <- tibble::tibble(dist_label = d,
                          dist = "logt",
                          args = list(list(location = prior$location, scale = prior$scale, df = prior$df)))
  } else if (d == "Exponential") {
    out <- tibble::tibble(dist_label = d,
                          dist = "exp",
                          args = list(list(rate = 1 / prior$scale)))
  } else if (d == "flat (implicit)") {
    out <- tibble::tibble(dist_label = d,
                          dist = "unif",
                          args = list(list(min = if (is_trunc) trunc[1] else -Inf,
                                           max = if (is_trunc) trunc[2] else Inf)))
  }

  if (is_trunc && d %in% c("Normal", "Cauchy", "Student t", "Exponential", "log-Normal", "log-Student t")) {
    out$args <- list(rlang::list2(spec = out$dist, a = trunc[1], b = trunc[2], !!! out$args[[1]]))
    out$dist <- "trunc"
  }

  return(out)
}


#' Get prior distribution call
#'
#' @param x A nma_prior object
#'
#' @return String giving call to construct x
#' @noRd
get_prior_call <- function(x) {
  # Deal with lists of priors
  if (all(purrr::map_lgl(x, ~inherits(., "nma_prior")))) {
    out <- purrr::imap_chr(x, ~paste0(.y, " = ", get_prior_call(.x)))
    out <- paste0("list(", paste(out, collapse = ", "), ")")
    return(out)
  }

  if (!inherits(x, "nma_prior"))
    abort("Not a `nma_prior` object.")

  prior_args <- purrr::list_modify(x, dist = purrr::zap(), fun = purrr::zap())
  if (stringr::str_starts(x$dist, "half-")) prior_args <- purrr::list_modify(prior_args, location = purrr::zap())
  prior_args <- prior_args[!is.na(prior_args)]
  if (length(prior_args)) {
    out <- paste0(x$fun, "(",
                  paste(names(prior_args), prior_args, sep = ' = ', collapse = ', '),
                  ")")
  } else {
    out <- paste0(x$fun, "()")
  }
  return(out)
}

# # Density for general truncated distribution
# dtrunc <- function(x, dist, trunc, ...) {
#   a <- list(...)
#   dfun <- paste0("d", dist)
#   pfun <- paste0("p", dist)
#
#   out <- ifelse(x < trunc[1] | x > trunc[2],
#                 0,
#                 do.call(dfun, args = rlang::list2(x = x, !!! a)) / (do.call(pfun, args = rlang::list2(q = trunc[2], !!! a)) - do.call(pfun, args = rlang::list2(q = trunc[1], !!! a))))
#
#   return(out)
# }
#
# # CDF for general truncated distribution
# ptrunc <- function(q, dist, trunc, ...) {
#   a <- list(...)
#   pfun <- paste0("p", dist)
#
#   out <- (do.call(pfun, args = rlang::list2(q = q, !!! a)) - do.call(pfun, args = rlang::list2(q = trunc[1], !!! a))) /
#     (do.call(pfun, args = rlang::list2(q = trunc[2], !!! a)) - do.call(pfun, args = rlang::list2(q = trunc[1], !!! a)))
#
#   return(out)
# }
#
# # Inverse CDF for general truncated distribution
# qtrunc <- function(p, dist, trunc, ...) {
#   a <- list(...)
#   qfun <- paste0("q", dist)
#   pfun <- paste0("p", dist)
#
#   pU <- do.call(pfun, args = rlang::list2(q = trunc[2], !!! a))
#   pL <- do.call(pfun, args = rlang::list2(q = trunc[1], !!! a))
#
#   pt <- pL + p * (pU - pL)
#
#   out <- do.call(qfun, args = rlang::list2(p = pt, !!! a))
#   out <- pmin(pmax(trunc[1], out), trunc[2])
#   return(out)
# }

#' Generalised Student's t distribution (with location and scale)
#'
#' Density, distribution, and quantile function for the generalised t
#' distribution with degrees of freedom `df`, shifted by `location` and scaled
#' by `scale`.
#'
#' @param x,q Vector of quantiles
#' @param p Vector of probabilities
#' @param df Degrees of freedom, greater than zero
#' @param location Location parameter
#' @param scale Scale parameter, greater than zero
#'
#' @return `dgent()` gives the density, `pgent()` gives the distribution
#'   function, `qgent()` gives the quantile function.
#' @export
#' @rdname generalised_t
#'
dgent <- function(x, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")

  return(stats::dt((x - location) / scale, df) / scale)
}

#' @export
#' @rdname generalised_t
pgent <- function(q, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")

  return(stats::pt((q - location) / scale, df))
}

#' @export
#' @rdname generalised_t
qgent <- function(p, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")

  return(location + scale * stats::qt(p, df))
}

#' Log Student's t distribution
#'
#' Density, distribution, and quantile function for the log t distribution,
#' whose logarithm has degrees of freedom `df`, mean `location`, and standard
#' deviation `scale`.
#'
#' @param x,q Vector of quantiles
#' @param p Vector of probabilities
#' @param df Degrees of freedom, greater than zero
#' @param location Location parameter
#' @param scale Scale parameter, greater than zero
#'
#' @details If \eqn{\log(Y) \sim t_\nu(\mu, \sigma^2)}, then \eqn{Y} has a log t
#'   distribution with `location` \eqn{\mu}, `scale` \eqn{\sigma}, and `df`
#'   \eqn{\nu}.
#'
#'   The mean and all higher moments of the log t distribution are undefined or
#'   infinite.
#'
#'   If `df = 1` then the distribution is a log Cauchy distribution. As `df`
#'   tends to infinity, this approaches a log Normal distribution.
#'
#' @return `dlogt()` gives the density, `plogt()` gives the distribution
#'   function, `qlogt()` gives the quantile function.
#' @export
#' @rdname log_t
#'
dlogt <- function(x, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")
  if (!is.numeric(df) || any(df <= 0))
    abort("`df` must be numeric, greater than zero.")

  return(dgent(log(x), df = df, location = location, scale = scale) / x)
}

#' @export
#' @rdname log_t
plogt <- function(q, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")

  return(pgent(q = log(q), df = df, location = location, scale = scale))
}

#' @export
#' @rdname log_t
qlogt <- function(p, df, location = 0, scale = 1) {
  if (!is.numeric(location))
    abort("`location` must be numeric.")
  if (!is.numeric(scale) || any(scale <= 0))
    abort("`scale` must be numeric, greater than zero.")

  return(exp(qgent(p = p, df = df, location = location, scale = scale)))
}

# Generated by vignette example_statins.Rmd: do not edit by hand
# Instead edit example_statins.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(statins)


## -------------------------------------------------------------------------------------------------
statin_net <- set_agd_arm(statins, 
                          study = studyc,
                          trt = trtc,
                          r = r, 
                          n = n,
                          trt_ref = "Placebo")
statin_net


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## statin_fit_FE <- nma(statin_net,
##                      trt_effects = "fixed",
##                      regression = ~.trt:prevention,
##                      prior_intercept = normal(scale = 100),
##                      prior_trt = normal(scale = 100),
##                      prior_reg = normal(scale = 100))

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
statin_fit_FE <- nma(statin_net, 
                     trt_effects = "fixed",
                     regression = ~.trt:prevention,
                     prior_intercept = normal(scale = 100),
                     prior_trt = normal(scale = 100),
                     prior_reg = normal(scale = 100),
                     iter = 5000)


## -------------------------------------------------------------------------------------------------
statin_fit_FE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(statin_fit_FE, pars = c("d", "beta", "mu"))


## ----statin_FE_pp_plot----------------------------------------------------------------------------
plot_prior_posterior(statin_fit_FE, prior = c("trt", "reg"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## statin_fit_RE <- nma(statin_net,
##                      trt_effects = "random",
##                      regression = ~.trt:prevention,
##                      prior_intercept = normal(scale = 100),
##                      prior_trt = normal(scale = 100),
##                      prior_reg = normal(scale = 100),
##                      prior_het = half_normal(scale = 5),
##                      adapt_delta = 0.99)

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
statin_fit_RE <- nowarn_on_ci(nma(statin_net, 
                     trt_effects = "random",
                     regression = ~.trt:prevention,
                     prior_intercept = normal(scale = 100),
                     prior_trt = normal(scale = 100),
                     prior_reg = normal(scale = 100),
                     prior_het = half_normal(scale = 5),
                     adapt_delta = 0.99,
                     iter = 5000))


## -------------------------------------------------------------------------------------------------
statin_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(statin_fit_RE, pars = c("d", "beta", "mu", "delta"))


## ----statin_RE_pp_plot----------------------------------------------------------------------------
plot_prior_posterior(statin_fit_RE, prior = c("trt", "reg", "het"))


## -------------------------------------------------------------------------------------------------
(statin_dic_FE <- dic(statin_fit_FE))

## -------------------------------------------------------------------------------------------------
(statin_dic_RE <- dic(statin_fit_RE))


## ----statin_FE_resdev_plot------------------------------------------------------------------------
plot(statin_dic_FE)


## ----statin_RE_resdev_plot------------------------------------------------------------------------
plot(statin_dic_RE)


## -------------------------------------------------------------------------------------------------
statin_releff_FE <- relative_effects(statin_fit_FE,
                                     newdata = data.frame(prevention = c("Primary", "Secondary")),
                                     study = prevention)

statin_releff_FE


## ----statins_releff_FE, fig.height = 2------------------------------------------------------------
plot(statin_releff_FE, 
     ref_line = 0)


## ----statins_beta_FE, fig.height = 4--------------------------------------------------------------
plot(statin_fit_FE, 
     pars = "beta", 
     ref_line = 0,
     stat = "halfeye")


## ----statins_tests, include=FALSE, eval=params$run_tests------------------------------------------
#--- Test against TSD 3 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# Relative effects
statin_FE_releff <- as.data.frame(summary(statin_fit_FE, pars = "d"))

test_that("FE relative effects", {
  expect_equivalent(statin_FE_releff$mean, -0.11, tolerance = tol)
  expect_equivalent(statin_FE_releff$sd, 0.10, tolerance = tol)
  expect_equivalent(statin_FE_releff$`2.5%`, -0.30, tolerance = tol)
  expect_equivalent(statin_FE_releff$`97.5%`, 0.09, tolerance = tol)
})

statin_RE_releff <- as.data.frame(summary(statin_fit_RE, pars = "d"))

test_that("RE relative effects", {
  expect_equivalent(statin_RE_releff$mean, -0.07, tolerance = tol)
  expect_equivalent(statin_RE_releff$sd, 0.20, tolerance = tol)
  skip_on_ci()
  expect_equivalent(statin_RE_releff$`2.5%`, -0.48, tolerance = tol)
  expect_equivalent(statin_RE_releff$`97.5%`, 0.36, tolerance = tol)
})

# Regression coefficients
statin_FE_beta <- as.data.frame(summary(statin_fit_FE, pars = "beta"))

test_that("FE regression beta", {
  expect_equivalent(statin_FE_beta$mean, -0.21, tolerance = tol)
  expect_equivalent(statin_FE_beta$sd, 0.11, tolerance = tol)
  expect_equivalent(statin_FE_beta$`2.5%`, -0.42, tolerance = tol)
  expect_equivalent(statin_FE_beta$`97.5%`, 0.01, tolerance = tol)
})

statin_RE_beta <- as.data.frame(summary(statin_fit_RE, pars = "beta"))

test_that("RE regression beta", {
  expect_equivalent(statin_RE_beta$mean, -0.29, tolerance = tol)
  expect_equivalent(statin_RE_beta$sd, 0.26, tolerance = tol)
  skip_on_ci()
  expect_equivalent(statin_RE_beta$`2.5%`, -0.86, tolerance = tol)
  expect_equivalent(statin_RE_beta$`97.5%`, 0.20, tolerance = tol)
})

# RE heterogeneity SD
statin_RE_sd <- as.data.frame(summary(statin_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(statin_RE_sd$`50%`, 0.19, tolerance = tol)
  expect_equivalent(statin_RE_sd$sd, 0.20, tolerance = tol)
  skip_on_ci()
  expect_equivalent(statin_RE_sd$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(statin_RE_sd$`97.5%`, 0.76, tolerance = tol)
})

# DIC
test_that("FE DIC", {
  expect_equivalent(statin_dic_FE$resdev, 45.9, tolerance = tol_dic)
  expect_equivalent(statin_dic_FE$pd, 21.0, tolerance = tol_dic)
  expect_equivalent(statin_dic_FE$dic, 66.9, tolerance = tol_dic)
})

test_that("RE DIC", {
  expect_equivalent(statin_dic_RE$resdev, 42.6, tolerance = tol_dic)
  expect_equivalent(statin_dic_RE$pd, 24.2, tolerance = tol_dic)
  expect_equivalent(statin_dic_RE$dic, 66.8, tolerance = tol_dic)
})

test_that("Robust to custom options(contrasts) settings", {
  withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                         unordered = "contr.SAS")), {
    statin_fit_FE_SAS <- nma(statin_net, 
                         trt_effects = "fixed",
                         regression = ~.trt:prevention,
                         prior_intercept = normal(scale = 100),
                         prior_trt = normal(scale = 100),
                         prior_reg = normal(scale = 100),
                         iter = 5000)
    
    # Model pars are different (reference level of prevention is different) but
    # relative effects should still be calculated correctly
    statin_fit_FE_SAS_releff <- as_tibble(relative_effects(statin_fit_FE_SAS))[, c("parameter", "mean", "sd")]
  })

  expect_equal(statin_fit_FE_SAS_releff,
               as_tibble(relative_effects(statin_fit_FE))[, c("parameter", "mean", "sd")],
               tolerance = tol)
})



# Force clean up
rm(list = ls())
gc()

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Only test gradients, no sampling
smk_fit_RE <- nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5),
                  test_grad = TRUE)

test_that("Error if not ML-NMR model", {
  expect_error(plot_integration_error(smk_fit_RE),
               "Expecting a `stan_mlnmr` object, created by fitting a ML-NMR model with numerical integration")
})

# Generated by vignette example_smoking.Rmd: do not edit by hand
# Instead edit example_smoking.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(smoking)


## -------------------------------------------------------------------------------------------------
smknet <- set_agd_arm(smoking, 
                      study = studyn,
                      trt = trtc,
                      r = r, 
                      n = n,
                      trt_ref = "No intervention")
smknet


## ----eval=FALSE-----------------------------------------------------------------------------------
## plot(smknet, weight_edges = TRUE, weight_nodes = TRUE)

## ----smoking_network_plot, echo=FALSE, fig.width=8, fig.height=6, out.width="100%"----------------
plot(smknet, weight_edges = TRUE, weight_nodes = TRUE) + ggplot2::theme(plot.margin = ggplot2::unit(c(1, 1, 1, 6), "lines"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))

## -------------------------------------------------------------------------------------------------
summary(half_normal(scale = 5))


## -------------------------------------------------------------------------------------------------
smkfit <- nma(smknet, 
              trt_effects = "random",
              prior_intercept = normal(scale = 100),
              prior_trt = normal(scale = 100),
              prior_het = normal(scale = 5))


## -------------------------------------------------------------------------------------------------
smkfit


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(smkfit, pars = c("d", "tau", "mu", "delta"))


## ----smoking_pp_plot, fig.width=8, fig.height=6, out.width="100%"---------------------------------
plot_prior_posterior(smkfit)


## ----smoking_pp_plot_tau--------------------------------------------------------------------------
plot_prior_posterior(smkfit, prior = "het")


## -------------------------------------------------------------------------------------------------
(dic_consistency <- dic(smkfit))


## ----smoking_resdev_plot, fig.width=8-------------------------------------------------------------
plot(dic_consistency)


## -------------------------------------------------------------------------------------------------
smoking[smoking$r == 0, ]


## -------------------------------------------------------------------------------------------------
smkfit_ume <- nma(smknet, 
                  consistency = "ume",
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5))
smkfit_ume


## -------------------------------------------------------------------------------------------------
dic_consistency
(dic_ume <- dic(smkfit_ume))


## ----smoking_devdev_plot--------------------------------------------------------------------------
plot(dic_consistency, dic_ume, point_alpha = 0.5, interval_alpha = 0.2)


## -------------------------------------------------------------------------------------------------
smk_nodesplit <- nma(smknet, 
                     consistency = "nodesplit",
                     trt_effects = "random",
                     prior_intercept = normal(scale = 100),
                     prior_trt = normal(scale = 100),
                     prior_het = normal(scale = 5))


## -------------------------------------------------------------------------------------------------
summary(smk_nodesplit)


## ----smk_nodesplit, fig.width = 7-----------------------------------------------------------------
plot(smk_nodesplit) +
  ggplot2::theme(legend.position = "bottom", legend.direction = "horizontal")


## ----smoking_releff, fig.height=4.5---------------------------------------------------------------
(smk_releff <- relative_effects(smkfit, all_contrasts = TRUE))
plot(smk_releff, ref_line = 0)


## ----smoking_ranks, fig.height=3------------------------------------------------------------------
(smk_ranks <- posterior_ranks(smkfit, lower_better = FALSE))
plot(smk_ranks)

## ----smoking_rankprobs----------------------------------------------------------------------------
(smk_rankprobs <- posterior_rank_probs(smkfit, lower_better = FALSE))
plot(smk_rankprobs)

## ----smoking_cumrankprobs-------------------------------------------------------------------------
(smk_cumrankprobs <- posterior_rank_probs(smkfit, lower_better = FALSE, cumulative = TRUE))
plot(smk_cumrankprobs)


## ----smoking_tests, include=FALSE, eval=params$run_tests------------------------------------------
#--- Test against TSD 4 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# A = No intervention
# B = Self-help
# C = Individual counselling
# D = Group counselling

trt_codes <- c(A = "No intervention",
               B = "Self-help",
               C = "Individual counselling",
               D = "Group counselling")

# Relative effects
tsd_re <- tribble(
  ~contrast, ~est, ~sd, ~lower, ~upper,
  "dAB", 0.49, 0.40, -0.29, 1.31,
  "dAC", 0.84, 0.24,  0.39, 1.34,
  "dAD", 1.10, 0.44,  0.26, 2.00,
  "dBC", 0.35, 0.41, -0.46, 1.18,
  "dBD", 0.61, 0.49, -0.34, 1.59,
  "dCD", 0.26, 0.41, -0.55, 1.09) %>% 
  mutate(trt_b = recode(substr(contrast, 2, 2),
                        !!! trt_codes),
         trt = recode(substr(contrast, 3, 3),
                        !!! trt_codes),
         trt_b = ordered(trt_b, levels = levels(smknet$treatments)),
         trt = ordered(trt, levels = levels(smknet$treatments)),
         rev = if_else(trt_b > trt, -1, 1),
         .l = lower, .u = upper,
         lower = if_else(trt_b > trt, .u, .l),
         upper = if_else(trt_b > trt, .l, .u)) %>% 
  arrange(trt_b, trt) %>% 
  mutate_at(vars(est, lower, upper), ~.*rev)

test_that("RE relative effects", {
  expect_equivalent(smk_releff$summary$mean, tsd_re$est, tolerance = tol)
  expect_equivalent(smk_releff$summary$sd, tsd_re$sd, tolerance = tol)
  expect_equivalent(smk_releff$summary$`2.5%`, tsd_re$lower, tolerance = tol)
  expect_equivalent(smk_releff$summary$`97.5%`, tsd_re$upper, tolerance = tol)
})

# Heterogeneity SD
smk_tau <- summary(smkfit, pars = "tau")

test_that("RE heterogeneity SD", {
  expect_equivalent(smk_tau$summary$`50%`, 0.82, tolerance = tol)
  expect_equivalent(smk_tau$summary$sd, 0.19, tolerance = tol)
  expect_equivalent(smk_tau$summary$`2.5%`, 0.55, tolerance = tol)
  expect_equivalent(smk_tau$summary$`97.5%`, 1.27, tolerance = tol)
})

# DIC
test_that("DIC", {
  expect_equivalent(dic_consistency$resdev, 54.0, tolerance = tol_dic)
  expect_equivalent(dic_consistency$pd, 45.0, tolerance = tol_dic)
  expect_equivalent(dic_consistency$dic, 99.0, tolerance = tol_dic)
})

# Relative effects (UME)

# Treatment ordering is different in TSD 4 - use trtn instead which reflects this
smknet2 <- set_agd_arm(smoking, studyn, trtn, r = r, n = n, trt_ref = 1)
smkfit_ume2 <- nma(smknet2, 
                   consistency = "ume",
                   trt_effects = "random",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100),
                   prior_het = normal(scale = 5),
                   iter = 4000)

# Results of modified TSD 4 model including multi-arm correction
tsd_ume <- tribble(
  ~contrast, ~est, ~sd, ~lower, ~upper,
  # "dAB",  0.34, 0.58, -0.81, 1.50,
  # "dAC",  0.86, 0.27,  0.34, 1.43,
  # "dAD",  1.43, 0.88, -0.21, 3.29,
  # "dBC", -0.05, 0.74, -1.53, 1.42,
  # "dBD",  0.65, 0.73, -0.80, 2.12,
  # "dCD",  0.20, 0.78, -1.37, 1.73)  %>% 
  "dAB",  0.34, 0.59, -0.82, 1.51,
  "dAC",  0.90, 0.28,  0.37, 1.49,
  "dAD",  1.12, 0.80, -0.36, 2.80,
  "dBC",  0.04, 0.73, -1.39, 1.50,
  "dBD",  0.61, 0.72, -0.80, 2.05,
  "dCD",  0.20, 0.79, -1.40, 1.75)
  # mutate(trt_b = recode(substr(contrast, 2, 2),
  #                       !!! trt_codes),
  #        trt = recode(substr(contrast, 3, 3),
  #                       !!! trt_codes),
  #        trt_b = ordered(trt_b, levels = levels(smknet$treatments)),
  #        trt = ordered(trt, levels = levels(smknet$treatments)),
  #        rev = if_else(trt_b > trt, -1, 1),
  #        .l = lower, .u = upper,
  #        lower = if_else(trt_b > trt, .u, .l),
  #        upper = if_else(trt_b > trt, .l, .u)) %>% 
  # arrange(trt_b, trt) %>% 
  # mutate_at(vars(est:upper), ~.*rev)

smk_ume_releff <- summary(smkfit_ume2, pars = "d")

test_that("UME relative effects", {
  # skip("Different model parameterisation")
  expect_equivalent(smk_ume_releff$summary$mean, tsd_ume$est, tolerance = tol)
  expect_equivalent(smk_ume_releff$summary$sd, tsd_ume$sd, tolerance = tol)
  expect_equivalent(smk_ume_releff$summary$`2.5%`, tsd_ume$lower, tolerance = tol)
  expect_equivalent(smk_ume_releff$summary$`97.5%`, tsd_ume$upper, tolerance = tol)
})

# Heterogeneity SD (UME)
smk_ume_tau <- summary(smkfit_ume2, pars = "tau")

test_that("UME heterogeneity SD", {
  # skip("Different model parameterisation")
  # expect_equivalent(smk_ume_tau$summary$`50%`, 0.89, tolerance = tol)
  # expect_equivalent(smk_ume_tau$summary$sd, 0.22, tolerance = tol)
  # expect_equivalent(smk_ume_tau$summary$`2.5%`, 0.58, tolerance = tol)
  # expect_equivalent(smk_ume_tau$summary$`97.5%`, 1.45, tolerance = tol)
  expect_equivalent(smk_ume_tau$summary$`50%`, 0.91, tolerance = tol)
  expect_equivalent(smk_ume_tau$summary$sd, 0.21, tolerance = tol)
  expect_equivalent(smk_ume_tau$summary$`2.5%`, 0.59, tolerance = tol)
  expect_equivalent(smk_ume_tau$summary$`97.5%`, 1.48, tolerance = tol)
})

# DIC (UME)
test_that("UME DIC", {
  expect_equivalent(dic_ume$resdev, 53.4, tolerance = tol_dic)
  expect_equivalent(dic_ume$pd, 46.1, tolerance = tol_dic)
  expect_equivalent(dic_ume$dic, 99.5, tolerance = tol_dic)
})

dic_ume2 <- dic(smkfit_ume2)
test_that("UME DIC", {
  expect_equivalent(dic_ume2$resdev, 53.4, tolerance = tol_dic)
  expect_equivalent(dic_ume2$pd, 46.1, tolerance = tol_dic)
  expect_equivalent(dic_ume2$dic, 99.5, tolerance = tol_dic)
})

# Check that multinomial ordered model produces same results
smknet3 <- set_agd_arm(smoking,
                       studyn,
                       trtc,
                       r = multi(nonevent = n, event = r, inclusive = TRUE),
                       trt_ref = "No intervention")
smkfit_ord <- nma(smknet3,
              trt_effects = "random",
              link = "logit",
              prior_intercept = normal(scale = 100),
              prior_trt = normal(scale = 100),
              prior_het = normal(scale = 5),
              prior_aux = flat())


smk_ord_releff <- relative_effects(smkfit_ord, all_contrasts = TRUE)
test_that("Equivalent ordered multinomial RE relative effects", {
  expect_equivalent(smk_ord_releff$summary$mean, tsd_re$est, tolerance = tol)
  expect_equivalent(smk_ord_releff$summary$sd, tsd_re$sd, tolerance = tol)
  expect_equivalent(smk_ord_releff$summary$`2.5%`, tsd_re$lower, tolerance = tol)
  expect_equivalent(smk_ord_releff$summary$`97.5%`, tsd_re$upper, tolerance = tol)
})

smk_ord_tau <- summary(smkfit_ord, pars = "tau")
test_that("Equivalent ordered multinomial RE heterogeneity SD", {
  expect_equivalent(smk_ord_tau$summary$`50%`, 0.82, tolerance = tol)
  expect_equivalent(smk_ord_tau$summary$sd, 0.19, tolerance = tol)
  expect_equivalent(smk_ord_tau$summary$`2.5%`, 0.55, tolerance = tol)
  expect_equivalent(smk_ord_tau$summary$`97.5%`, 1.27, tolerance = tol)
})

# DIC
dic_ord <- dic(smkfit_ord)
test_that("Equivalent ordered multinomial DIC", {
  expect_equivalent(dic_ord$resdev, 54.0, tolerance = tol_dic)
  expect_equivalent(dic_ord$pd, 45.0, tolerance = tol_dic)
  expect_equivalent(dic_ord$dic, 99.0, tolerance = tol_dic)
})

test_that("Robust to custom options(contrasts) settings", {
  skip_on_cran()
  
  withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                                       unordered = "contr.SAS")), {
    smkfit_SAS <- nma(smknet,
                      trt_effects = "random",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_het = normal(scale = 5))
    
    smkfit_SAS_summary <- as_tibble(summary(smkfit_SAS))[, c("parameter", "mean", "sd")]
    smkfit_SAS_releff <- as_tibble(relative_effects(smkfit_SAS))[, c("parameter", "mean", "sd")]
    smkfit_SAS_pred <- as_tibble(predict(smkfit_SAS))[, c("parameter", "mean", "sd")]
    
    smkfit_ume_SAS <- nma(smknet,
                      trt_effects = "random",
                      consistency = "ume",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_het = normal(scale = 5))
    
    smkfit_ume_SAS_summary <- as_tibble(summary(smkfit_ume_SAS))[, c("parameter", "mean", "sd")]
  })

  expect_equal(smkfit_SAS_summary,
               as_tibble(summary(smkfit))[, c("parameter", "mean", "sd")],
               tolerance = tol)
  expect_equal(smkfit_SAS_releff,
               as_tibble(relative_effects(smkfit))[, c("parameter", "mean", "sd")],
               tolerance = tol)
  expect_equal(smkfit_SAS_pred,
               as_tibble(predict(smkfit))[, c("parameter", "mean", "sd")],
               tolerance = tol)
  
  expect_equal(smkfit_ume_SAS_summary,
               as_tibble(summary(smkfit_ume))[, c("parameter", "mean", "sd")],
               tolerance = tol)
})


# Force clean up
rm(list = ls())
gc()

test_that("check_prior_location gives suitable errors",{
  m <- "Prior location \\(mean\\) must be numeric"
  expect_error(check_prior_location("a"), m)
  expect_error(check_prior_location(list()), m)
  expect_error(check_prior_location(1:2), m)
  expect_error(check_prior_location("a", type = "median"), "Prior median must be numeric")
})

test_that("check_prior_scale gives suitable errors",{
  m <- "Prior scale \\(standard deviation\\) must be numeric"
  expect_error(check_prior_scale("a"), m)
  expect_error(check_prior_scale(list()), m)
  expect_error(check_prior_scale(1:2), m)
  expect_error(check_prior_scale("a", type = "variance"), "Prior variance must be numeric")
  expect_error(check_prior_scale(0), "Prior scale \\(standard deviation\\) must be strictly positive")
  expect_error(check_prior_scale(-1), "Prior scale \\(standard deviation\\) must be strictly positive")
})

p <- normal(0, 1)

test_that("summary.nma_prior digits argument", {
  m <- "must be a single integer"
  expect_error(summary(p, digits = "a"), m)
  expect_error(summary(p, digits = list()), m)
  expect_error(summary(p, digits = NA), m)
  expect_error(summary(p, digits = NULL), m)
  expect_error(summary(p, digits = 1:2), m)
  expect_error(summary(p, digits = 1.5), m)
  expect_error(summary(p, digits = Inf), m)
})

test_that("summary.nma_prior probs argument", {
  m <- "numeric vector of probabilities"
  expect_error(summary(p, probs = "a"), m)
  expect_error(summary(p, probs = -1), m)
  expect_error(summary(p, probs = 1.5), m)
  expect_error(summary(p, probs = Inf), m)
  expect_error(summary(p, probs = list()), m)
  expect_error(summary(p, probs = NA), m)
  expect_error(summary(p, probs = NULL), m)
})

test_that("summary.nma_prior trunc argument", {
  m <- "length 2 numeric vector"
  expect_error(summary(p, trunc = "a"), m)
  expect_error(summary(p, trunc = -1), m)
  expect_error(summary(p, trunc = 1.5), m)
  expect_error(summary(p, trunc = Inf), m)
  expect_error(summary(p, trunc = list()), m)
  expect_error(summary(p, trunc = NA), m)
  expect_error(summary(p, trunc = 1:3), m)
  expect_error(summary(p, trunc = c(1, NA)), m)
})

test_that("summary.nma_prior sense check", {
  expect_output(summary(p, probs = 0.95, digits = 2), "95%.+between -1\\.96 and 1\\.96")
})

# Generated by vignette example_bcg_vaccine.Rmd: do not edit by hand
# Instead edit example_bcg_vaccine.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------

## ----include=FALSE--------------------------------------------------------------------------------
set.seed(18284729)


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(bcg_vaccine)


## -------------------------------------------------------------------------------------------------
bcg_net <- set_agd_arm(bcg_vaccine, 
                       study = studyn,
                       trt = trtc,
                       r = r, 
                       n = n,
                       trt_ref = "Unvaccinated")
bcg_net


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval = FALSE---------------------------------------------------------------------------------
## bcg_fit_unadj <- nma(bcg_net,
##                      trt_effects = "random",
##                      prior_intercept = normal(scale = 100),
##                      prior_trt = normal(scale = 100),
##                      prior_het = half_normal(scale = 5))

## ----echo = FALSE---------------------------------------------------------------------------------
bcg_fit_unadj <- nowarn_on_ci(
  nma(bcg_net, 
      seed = 14308133,
      iter = 10000,
      trt_effects = "random",
      prior_intercept = normal(scale = 100),
      prior_trt = normal(scale = 100),
      prior_het = half_normal(scale = 5))
  )


## -------------------------------------------------------------------------------------------------
bcg_fit_unadj


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(bcg_fit_unadj, pars = c("d", "mu", "delta", "tau"))


## ----bcg_unadj_pp_plot----------------------------------------------------------------------------
plot_prior_posterior(bcg_fit_unadj, prior = c("trt", "het"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval = FALSE---------------------------------------------------------------------------------
## bcg_fit_lat <- nma(bcg_net,
##                    trt_effects = "random",
##                    regression = ~.trt:latitude,
##                    prior_intercept = normal(scale = 100),
##                    prior_trt = normal(scale = 100),
##                    prior_reg = normal(scale = 100),
##                    prior_het = half_normal(scale = 5),
##                    adapt_delta = 0.99)

## ----echo = FALSE---------------------------------------------------------------------------------
bcg_fit_lat <- nowarn_on_ci(
                 nma(bcg_net, 
                     seed = 1932599147,
                     iter = 5000,
                     trt_effects = "random",
                     regression = ~.trt:latitude,
                     prior_intercept = normal(scale = 100),
                     prior_trt = normal(scale = 100),
                     prior_reg = normal(scale = 100),
                     prior_het = half_normal(scale = 5),
                     adapt_delta = 0.99)
                 )


## -------------------------------------------------------------------------------------------------
bcg_fit_lat


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(bcg_fit_lat, pars = c("d", "beta", "mu", "delta", "tau"))


## ----bcg_lat_pp_plot------------------------------------------------------------------------------
plot_prior_posterior(bcg_fit_lat, prior = c("trt", "reg", "het"))


## -------------------------------------------------------------------------------------------------
(bcg_dic_unadj <- dic(bcg_fit_unadj))

## -------------------------------------------------------------------------------------------------
(bcg_dic_lat <- dic(bcg_fit_lat))


## -------------------------------------------------------------------------------------------------
summary(bcg_fit_unadj, pars = "tau")
summary(bcg_fit_lat, pars = "tau")


## ----bcg_vaccine_beta_lat, fig.height = 4---------------------------------------------------------
summary(bcg_fit_lat, pars = "beta")

plot(bcg_fit_lat, 
     pars = "beta", 
     ref_line = 0,
     stat = "halfeye")


## -------------------------------------------------------------------------------------------------
bcg_releff_lat <- relative_effects(bcg_fit_lat,
                                   newdata = tibble::tibble(latitude = seq(10, 50, by = 10),
                                                            label = paste0(latitude, "\u00B0 latitude")),
                                   study = label)

bcg_releff_lat


## ----bcg_vaccine_releff_lat, fig.height = 5-------------------------------------------------------
plot(bcg_releff_lat, 
     ref_line = 0)


## ----bcg_vaccine_reg_plot-------------------------------------------------------------------------
library(dplyr)
library(ggplot2)

# Get data for regression line
lat_range <- range(bcg_vaccine$latitude)
lat_dat <- tibble(latitude = seq(lat_range[1], lat_range[2], by = 1))

bcg_lat_reg <- relative_effects(bcg_fit_lat, 
                                newdata = lat_dat) %>% 
  as_tibble() %>% 
  bind_cols(lat_dat)

# Get study log odds ratios
bcg_lor <- bcg_vaccine %>% 
  group_by(studyn) %>% 
  mutate(lor = log(r / (n - r)) - log(first(r) / (first(n) - first(r))),
         sample_size = sum(n)) %>% 
  slice(-1)

# Plot
ggplot(aes(x = latitude), data = bcg_lor) +
  geom_hline(yintercept = 0, colour = "grey60") +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), data = bcg_lat_reg,
              fill = "darkred", alpha = 0.3) +
  geom_line(aes(y = mean), data = bcg_lat_reg,
            colour = "darkred") +
  geom_point(aes(y = lor, size = sample_size), alpha = 0.6) +
  coord_cartesian(xlim = c(0, 60)) +
  xlab("Degrees Latitude") + ylab("log Odds Ratio") +
  scale_size("Sample Size") +
  theme_multinma()


## ----bcg_vaccine_predictive_unadj-----------------------------------------------------------------
(bcg_predeff_unadj <- relative_effects(bcg_fit_unadj, predictive_distribution = TRUE))


## -------------------------------------------------------------------------------------------------
mean(as.matrix(bcg_predeff_unadj) > 0)


## -------------------------------------------------------------------------------------------------
bcg_predeff_lat <- relative_effects(bcg_fit_lat,
                                   newdata = tibble::tibble(latitude = seq(0, 50, by = 10),
                                                            label = paste0(latitude, "\u00B0 latitude")),
                                   study = label,
                                   predictive_distribution = TRUE)

bcg_predeff_lat


## -------------------------------------------------------------------------------------------------
colMeans(as.matrix(bcg_predeff_lat) > 0)


## ----bcg_vaccine_tests, include=FALSE, eval=params$run_tests--------------------------------------
#--- Test against TSD 3 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# Relative effects
bcg_unadj_releff <- as.data.frame(summary(bcg_fit_unadj, pars = "d"))

test_that("Unadjusted relative effects", {
  expect_equivalent(bcg_unadj_releff$mean, -0.762, tolerance = tol)
  expect_equivalent(bcg_unadj_releff$sd, 0.22, tolerance = tol)
  expect_equivalent(bcg_unadj_releff$`2.5%`, -1.21, tolerance = tol)
  expect_equivalent(bcg_unadj_releff$`97.5%`, -0.34, tolerance = tol)
})

test_that("Unadjusted predictive distribution", {
  bcg_unadj_releff_pred <- as.data.frame(relative_effects(bcg_fit_unadj, predictive_distribution = TRUE))
  expect_equivalent(bcg_unadj_releff_pred$mean, -0.762, tolerance = tol)
  expect_equivalent(bcg_unadj_releff_pred$`2.5%`, -2.27, tolerance = tol)
  expect_equivalent(bcg_unadj_releff_pred$`97.5%`, 0.72, tolerance = tol)
})

bcg_lat_releff <- as.data.frame(summary(bcg_fit_lat, pars = "d"))

test_that("Regression relative effects", {
  expect_equivalent(bcg_lat_releff$mean, -0.763, tolerance = tol)
  expect_equivalent(bcg_lat_releff$sd, 0.126, tolerance = tol)
  expect_equivalent(bcg_lat_releff$`2.5%`, -1.04, tolerance = tol)
  expect_equivalent(bcg_lat_releff$`97.5%`, -0.52, tolerance = tol)
})


test_that("Regression predictive distribution", {
  bcg_lat_releff_pred <- relative_effects(bcg_fit_lat,
                                          newdata = data.frame(latitude = c(0, 13, 50)),
                                          predictive_distribution = TRUE)
  expect_equivalent(colMeans(as.matrix(bcg_lat_releff_pred) > 0), c(0.8, 0.35, 0.006), tolerance = tol)
})

# Regression coefficients
bcg_lat_beta <- as.data.frame(summary(bcg_fit_lat, pars = "beta"))

test_that("Regression beta", {
  expect_equivalent(bcg_lat_beta$mean, -0.032, tolerance = tol)
  expect_equivalent(bcg_lat_beta$sd, 0.009, tolerance = tol)
  expect_equivalent(bcg_lat_beta$`2.5%`, -0.05, tolerance = tol)
  expect_equivalent(bcg_lat_beta$`97.5%`, -0.01, tolerance = tol)
})

# RE heterogeneity SD
bcg_unadj_sd <- as.data.frame(summary(bcg_fit_unadj, pars = "tau"))

test_that("Unadjusted heterogeneity SD", {
  expect_equivalent(bcg_unadj_sd$`50%`, 0.649, tolerance = tol)
  expect_equivalent(bcg_unadj_sd$sd, 0.202, tolerance = tol)
  expect_equivalent(bcg_unadj_sd$`2.5%`, 0.39, tolerance = tol)
  expect_equivalent(bcg_unadj_sd$`97.5%`, 1.17, tolerance = tol)
})

bcg_lat_sd <- as.data.frame(summary(bcg_fit_lat, pars = "tau"))

test_that("Regression heterogeneity SD", {
  expect_equivalent(bcg_lat_sd$`50%`, 0.272, tolerance = tol)
  expect_equivalent(bcg_lat_sd$sd, 0.188, tolerance = tol)
  expect_equivalent(bcg_lat_sd$`2.5%`, 0.03, tolerance = tol)
  expect_equivalent(bcg_lat_sd$`97.5%`, 0.75, tolerance = tol)
})

# DIC
test_that("Unadjusted DIC", {
  expect_equivalent(bcg_dic_unadj$resdev, 26.1, tolerance = tol_dic)
  expect_equivalent(bcg_dic_unadj$pd, 23.5, tolerance = tol_dic)
  expect_equivalent(bcg_dic_unadj$dic, 49.6, tolerance = tol_dic)
})

test_that("Regression DIC", {
  expect_equivalent(bcg_dic_lat$resdev, 30.4, tolerance = tol_dic)
  expect_equivalent(bcg_dic_lat$pd, 21.1, tolerance = tol_dic)
  expect_equivalent(bcg_dic_lat$dic, 51.5, tolerance = tol_dic)
})

test_that("Relative effects and predict work with data.frame", {
  new <- tibble::tibble(latitude = seq(10, 50, by = 10), label = paste0(latitude, "\u00B0 latitude"))
  expect_identical(relative_effects(bcg_fit_lat, newdata = new, study = label),
                   relative_effects(bcg_fit_lat, newdata = as.data.frame(new), study = label))
  
  # For predict() we need to account for the random baseline sample
  qcons <- function(p, cons = 0) {cons}
  
  expect_identical(predict(bcg_fit_lat, newdata = new, study = label,
                           baseline = distr(qcons, -2)),
                   predict(bcg_fit_lat, newdata = as.data.frame(new), study = label,
                           baseline = distr(qcons, -2)))
  
  expect_identical(predict(bcg_fit_lat, newdata = new, study = label,
                           baseline = distr(qcons, 0.5),
                           baseline_type = "response"),
                   predict(bcg_fit_lat, newdata = as.data.frame(new), study = label,
                           baseline = distr(qcons, 0.5),
                           baseline_type = "response"))
})

test_that("Predictions using network baselines are correct", {
  pred_all <- predict(bcg_fit_lat, type = "response")
  pred_12 <- predict(bcg_fit_lat, type = "response", 
                     baseline = list("1" = "1", "2" = "2"),
                     newdata = subset(bcg_vaccine, studyn %in% 1:2),
                     study = studyn)
  
  expect_equal(unclass(as.array(pred_12)),
               as.array(pred_all)[ , , 1:4])
})


# Force clean up
rm(list = ls())
gc()

sa_net <- set_agd_contrast(social_anxiety, studyc,
                           trtc,
                           y = y,
                           se = se,
                           trt_class = classc,
                           trt_ref = "Waitlist")

sa_fit_EXclass_RE <- suppressWarnings(nma(sa_net, trt_effects = "random",
                                          prior_trt = normal(0, 100),
                                          prior_het = half_normal(5),
                                          class_effects = "exchangeable",
                                          iter = 10,
                                          prior_class_sd = normal(0.33,0.1),
                                          class_sd =
                                            list(`Exercise and SH no support` =
                                                   c("Exercise promotion", "Self-help no support"),
                                                 `SSRIs and NSSA` =
                                                   c("SSRI/SNRI", "NSSA"),
                                                 `Psychodynamic & Other psychological therapies` =
                                                   c("Psychodynamic psychotherapy", "Other psychological therapies"))))

test_that("class_mean and class_sd parameters are named correctly", {
  # Extract summaries for class_mean
  cm_summary <- tibble::as_tibble(summary(sa_fit_EXclass_RE, pars = "class_mean"))
  cm_params <- cm_summary$parameter

  # Expected class means - shouldn't have sole-occupancy classes, except if these have shared SDs
  expected_cm <- setdiff(levels(sa_fit_EXclass_RE$network$classes),
                         c("Waitlist", "Pill placebo", "Psychological placebo"))

  expect_equal(cm_params, paste0("class_mean[", expected_cm, "]"))

  # Extract summaries for class_sd
  cs_summary <- tibble::as_tibble(summary(sa_fit_EXclass_RE, pars = "class_sd"))
  cs_params <- cs_summary$parameter

  # Expected class SDs - shared and named correctly, no sole-occupancy classes
  expected_cs <- forcats::fct_collapse(sa_fit_EXclass_RE$network$classes,
                                       `Exercise and SH no support` =
                                         c("Exercise promotion", "Self-help no support"),
                                       `SSRIs and NSSA` =
                                         c("SSRI/SNRI", "NSSA"),
                                       `Psychodynamic & Other psychological therapies` =
                                         c("Psychodynamic psychotherapy", "Other psychological therapies")) %>%
    levels() %>% setdiff(c("Waitlist", "Pill placebo", "Psychological placebo"))
  expect_equal(cs_params, paste0("class_sd[", expected_cs, "]"))
})

# Generated by vignette example_dietary_fat.Rmd: do not edit by hand
# Instead edit example_dietary_fat.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE------------------


## ----eval = FALSE-------------------------------------------------------------
# library(multinma)
# options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -----------------------------------------------------------------------------
head(dietary_fat)


## -----------------------------------------------------------------------------
diet_net <- set_agd_arm(dietary_fat, 
                        study = studyc,
                        trt = trtc,
                        r = r, 
                        E = E,
                        trt_ref = "Control",
                        sample_size = n)
diet_net


## -----------------------------------------------------------------------------
summary(normal(scale = 100))


## -----------------------------------------------------------------------------
diet_fit_FE <- nma(diet_net, 
                   trt_effects = "fixed",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100))


## -----------------------------------------------------------------------------
diet_fit_FE


## ----eval=FALSE---------------------------------------------------------------
# # Not run
# print(diet_fit_FE, pars = c("d", "mu"))


## ----diet_FE_pp_plot----------------------------------------------------------
plot_prior_posterior(diet_fit_FE)


## -----------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=FALSE---------------------------------------------------------------
# diet_fit_RE <- nma(diet_net,
#                    trt_effects = "random",
#                    prior_intercept = normal(scale = 100),
#                    prior_trt = normal(scale = 100),
#                    prior_het = half_normal(scale = 5))

## ----echo=FALSE, warning=FALSE------------------------------------------------
diet_fit_RE <- nowarn_on_ci(
                 nma(diet_net, 
                     trt_effects = "random",
                     prior_intercept = normal(scale = 100),
                     prior_trt = normal(scale = 100),
                     prior_het = half_normal(scale = 5))
                 )


## -----------------------------------------------------------------------------
diet_fit_RE


## ----eval=FALSE---------------------------------------------------------------
# # Not run
# print(diet_fit_RE, pars = c("d", "mu", "delta"))


## ----diet_RE_pp_plot----------------------------------------------------------
plot_prior_posterior(diet_fit_RE, prior = c("trt", "het"))


## -----------------------------------------------------------------------------
(dic_FE <- dic(diet_fit_FE))

## -----------------------------------------------------------------------------
(dic_RE <- dic(diet_fit_RE))


## ----diet_FE_resdev_plot------------------------------------------------------
plot(dic_FE)


## ----diet_RE_resdev_plot------------------------------------------------------
plot(dic_RE)


## ----diet_pred_FE, fig.height = 2---------------------------------------------
pred_FE <- predict(diet_fit_FE, 
                   baseline = distr(qnorm, mean = -3, sd = 1.77^-0.5), 
                   type = "response")
pred_FE
plot(pred_FE)

## ----diet_pred_RE, fig.height = 2---------------------------------------------
pred_RE <- predict(diet_fit_RE, 
                   baseline = distr(qnorm, mean = -3, sd = 1.77^-0.5), 
                   type = "response")
pred_RE
plot(pred_RE)


## ----diet_pred_FE_all, fig.height=10------------------------------------------
pred_FE_studies <- predict(diet_fit_FE, type = "response")
pred_FE_studies
plot(pred_FE_studies) + ggplot2::facet_grid(Study~., labeller = ggplot2::label_wrap_gen(width = 10))


## ----diet_tests_tests, include=FALSE, eval=params$run_tests-------------------
#--- Test against TSD 2 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# Relative effects
diet_FE_releff <- as.data.frame(relative_effects(diet_fit_FE))

test_that("FE relative effects", {
  expect_equivalent(diet_FE_releff$mean, -0.01, tolerance = tol)
  expect_equivalent(diet_FE_releff$sd, 0.054, tolerance = tol)
  expect_equivalent(diet_FE_releff$`2.5%`, -0.11, tolerance = tol)
  expect_equivalent(diet_FE_releff$`50%`, -0.01, tolerance = tol)
  expect_equivalent(diet_FE_releff$`97.5%`, 0.10, tolerance = tol)
})

diet_RE_releff <- as.data.frame(relative_effects(diet_fit_RE))

test_that("RE relative effects", {
  expect_equivalent(diet_RE_releff$mean, -0.02, tolerance = tol)
  expect_equivalent(diet_RE_releff$sd, 0.09, tolerance = tol)
  expect_equivalent(diet_RE_releff$`2.5%`, -0.19, tolerance = tol)
  expect_equivalent(diet_RE_releff$`50%`, -0.01, tolerance = tol)
  expect_equivalent(diet_RE_releff$`97.5%`, 0.16, tolerance = tol)
})

# RE heterogeneity SD
diet_RE_sd <- as.data.frame(summary(diet_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(diet_RE_sd$mean, 0.13, tolerance = tol)
  expect_equivalent(diet_RE_sd$sd, 0.12, tolerance = tol)
  expect_equivalent(diet_RE_sd$`2.5%`, 0.00, tolerance = tol)
  expect_equivalent(diet_RE_sd$`50%`, 0.10, tolerance = tol)
  expect_equivalent(diet_RE_sd$`97.5%`, 0.43, tolerance = tol)
})

# DIC
test_that("FE DIC", {
  expect_equivalent(dic_FE$resdev, 23.32, tolerance = tol_dic)
  expect_equivalent(dic_FE$pd, 10.9, tolerance = tol_dic)
  expect_equivalent(dic_FE$dic, 33.2, tolerance = tol_dic)
})

test_that("RE DIC", {
  expect_equivalent(dic_RE$resdev, 21.5, tolerance = tol_dic)
  expect_equivalent(dic_RE$pd, 13.3, tolerance = tol_dic)
  expect_equivalent(dic_RE$dic, 34.8, tolerance = tol_dic)
})

# Predictions
diet_pred_FE <- as.data.frame(pred_FE)

test_that("FE predicted probabilities", {
  expect_equivalent(diet_pred_FE$mean, c(0.06, 0.06), tolerance = tol_dic)
  expect_equivalent(diet_pred_FE$sd, c(0.04, 0.04), tolerance = tol_dic)
  expect_equivalent(diet_pred_FE$`2.5%`, c(0.01, 0.01), tolerance = tol_dic)
  expect_equivalent(diet_pred_FE$`50%`, c(0.05, 0.05), tolerance = tol_dic)
  expect_equivalent(diet_pred_FE$`97.5%`, c(0.18, 0.18), tolerance = tol_dic)
})

diet_pred_RE <- as.data.frame(pred_RE)

test_that("RE predicted probabilities", {
  expect_equivalent(diet_pred_RE$mean, c(0.06, 0.06), tolerance = tol_dic)
  expect_equivalent(diet_pred_RE$sd, c(0.04, 0.04), tolerance = tol_dic)
  expect_equivalent(diet_pred_RE$`2.5%`, c(0.01, 0.01), tolerance = tol_dic)
  expect_equivalent(diet_pred_RE$`50%`, c(0.05, 0.05), tolerance = tol_dic)
  expect_equivalent(diet_pred_RE$`97.5%`, c(0.18, 0.18), tolerance = tol_dic)
})

# Predictions specifying study for baseline
test_that("Specifying study for baseline gives correct result", {
  pred_FE_DART <- predict(diet_fit_FE, type = "response", baseline = "DART")
  expect_identical(c(as.array(pred_FE_DART)),
                   c(as.array(pred_FE_studies)[ , , 1:2]))
})

# Predictions escape study names correctly
test_that("Predictions escaping baseline study names", {
  diet_net2 <- set_agd_arm(dietary_fat, 
                        study = paste0(studyc, "(", studyn, ")$"),
                        trt = trtc,
                        r = r, 
                        E = E,
                        trt_ref = "Control",
                        sample_size = n)
  
  diet_fit_FE2 <- nma(diet_net2, 
                   trt_effects = "fixed",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100))
  
  pred_FE_DART <- predict(diet_fit_FE, type = "response", baseline = "DART")
  pred_FE_DART2 <- predict(diet_fit_FE2, type = "response", baseline = "DART(1)$")
  expect_equivalent(as.data.frame(pred_FE_DART2)[, c("mean", "sd")],
                    as.data.frame(pred_FE_DART)[, c("mean", "sd")],
                    tol = tol)
})

# Test identical model treating data as IPD
diet_net_ipd <- set_ipd(dietary_fat, 
                        study = studyc,
                        trt = trtc,
                        r = r, 
                        E = E,
                        trt_ref = "Control")

diet_fit_FE_ipd <- nma(diet_net_ipd, 
                       trt_effects = "fixed",
                       prior_intercept = normal(scale = 100),
                       prior_trt = normal(scale = 100))

diet_fit_RE_ipd <- nowarn_on_ci(
                     nma(diet_net_ipd, 
                         trt_effects = "random",
                         prior_intercept = normal(scale = 100),
                         prior_trt = normal(scale = 100),
                         prior_het = half_normal(scale = 5),
                         iter = 4000)
                   )

# Relative effects
diet_FE_ipd_releff <- as.data.frame(relative_effects(diet_fit_FE_ipd))

test_that("IPD FE relative effects", {
  expect_equivalent(diet_FE_ipd_releff$mean, -0.01, tolerance = tol)
  expect_equivalent(diet_FE_ipd_releff$sd, 0.054, tolerance = tol)
  expect_equivalent(diet_FE_ipd_releff$`2.5%`, -0.11, tolerance = tol)
  expect_equivalent(diet_FE_ipd_releff$`50%`, -0.01, tolerance = tol)
  expect_equivalent(diet_FE_ipd_releff$`97.5%`, 0.10, tolerance = tol)
})

diet_RE_ipd_releff <- as.data.frame(relative_effects(diet_fit_RE_ipd))

test_that("IPD RE relative effects", {
  expect_equivalent(diet_RE_ipd_releff$mean, -0.02, tolerance = tol)
  expect_equivalent(diet_RE_ipd_releff$sd, 0.09, tolerance = tol)
  expect_equivalent(diet_RE_ipd_releff$`2.5%`, -0.19, tolerance = tol)
  expect_equivalent(diet_RE_ipd_releff$`50%`, -0.01, tolerance = tol)
  expect_equivalent(diet_RE_ipd_releff$`97.5%`, 0.16, tolerance = tol)
})

# RE heterogeneity SD
diet_RE_ipd_sd <- as.data.frame(summary(diet_fit_RE_ipd, pars = "tau"))

test_that("IPD RE heterogeneity SD", {
  expect_equivalent(diet_RE_ipd_sd$mean, 0.13, tolerance = tol)
  expect_equivalent(diet_RE_ipd_sd$sd, 0.12, tolerance = tol)
  expect_equivalent(diet_RE_ipd_sd$`2.5%`, 0.00, tolerance = tol)
  expect_equivalent(diet_RE_ipd_sd$`50%`, 0.10, tolerance = tol)
  expect_equivalent(diet_RE_ipd_sd$`97.5%`, 0.43, tolerance = tol)
})

# DIC
dic_FE_ipd <- dic(diet_fit_FE_ipd)
test_that("IPD FE DIC", {
  expect_equivalent(dic_FE_ipd$resdev, 23.32, tolerance = tol_dic)
  expect_equivalent(dic_FE_ipd$pd, 10.9, tolerance = tol_dic)
  expect_equivalent(dic_FE_ipd$dic, 33.2, tolerance = tol_dic)
})

dic_RE_ipd <- dic(diet_fit_RE_ipd)
test_that("IPD RE DIC", {
  expect_equivalent(dic_RE_ipd$resdev, 21.5, tolerance = tol_dic)
  expect_equivalent(dic_RE_ipd$pd, 13.3, tolerance = tol_dic)
  expect_equivalent(dic_RE_ipd$dic, 34.8, tolerance = tol_dic)
})


# Force clean up
rm(list = ls())
gc()

# Generated by vignette example_blocker.Rmd: do not edit by hand
# Instead edit example_blocker.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----eval = FALSE---------------------------------------------------------------------------------
# library(multinma)
# options(mc.cores = parallel::detectCores())
# library(ggplot2)

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)
library(ggplot2)


## -------------------------------------------------------------------------------------------------
head(blocker)


## -------------------------------------------------------------------------------------------------
blocker_net <- set_agd_arm(blocker, 
                           study = studyn,
                           trt = trtc,
                           r = r, 
                           n = n,
                           trt_ref = "Control")
blocker_net


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## -------------------------------------------------------------------------------------------------
blocker_fit_FE <- nma(blocker_net, 
                   trt_effects = "fixed",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100))


## -------------------------------------------------------------------------------------------------
blocker_fit_FE


## ----eval=FALSE-----------------------------------------------------------------------------------
# # Not run
# print(blocker_fit_FE, pars = c("d", "mu"))


## ----blocker_FE_pp_plot---------------------------------------------------------------------------
plot_prior_posterior(blocker_fit_FE, prior = "trt")


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## -------------------------------------------------------------------------------------------------
blocker_fit_RE <- nma(blocker_net, 
                   trt_effects = "random",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100),
                   prior_het = half_normal(scale = 5))


## -------------------------------------------------------------------------------------------------
blocker_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
# # Not run
# print(blocker_fit_RE, pars = c("d", "mu", "delta"))


## ----blocker_RE_pp_plot---------------------------------------------------------------------------
plot_prior_posterior(blocker_fit_RE, prior = c("trt", "het"))


## -------------------------------------------------------------------------------------------------
(dic_FE <- dic(blocker_fit_FE))

## -------------------------------------------------------------------------------------------------
(dic_RE <- dic(blocker_fit_RE))


## ----blocker_FE_resdev_plot-----------------------------------------------------------------------
plot(dic_FE)


## ----blocker_RE_resdev_plot-----------------------------------------------------------------------
plot(dic_RE)


## ----blocker_leverage_FE--------------------------------------------------------------------------
plot(dic_FE, type = "leverage") + 
  # Add labels for points outside DIC=3
  geom_text(aes(label = parameter), data = ~subset(., dic > 3), vjust = -0.5)


## ----blocker_leverage_RE--------------------------------------------------------------------------
plot(dic_RE, type = "leverage") + 
  # Add labels for points outside DIC=3
  geom_text(aes(label = parameter), data = ~subset(., dic > 3), vjust = -0.5)


## ----blocker_pred_FE, fig.height = 2--------------------------------------------------------------
pred_FE <- predict(blocker_fit_FE, 
                   baseline = distr(qnorm, mean = -2.2, sd = 3.3^-0.5), 
                   type = "response")
pred_FE
plot(pred_FE)

## ----blocker_pred_RE, fig.height = 2--------------------------------------------------------------
pred_RE <- predict(blocker_fit_RE, 
                   baseline = distr(qnorm, mean = -2.2, sd = 3.3^-0.5), 
                   type = "response")
pred_RE
plot(pred_RE)


## ----blocker_pred_FE_beta, fig.height = 2---------------------------------------------------------
pred_FE_beta <- predict(blocker_fit_FE, 
                        baseline = distr(qbeta, 4, 36-4),
                        baseline_type = "response",
                        type = "response")
pred_FE_beta
plot(pred_FE_beta)


## ----blocker_pred_RE_beta, fig.height = 2---------------------------------------------------------
pred_RE_beta <- predict(blocker_fit_RE, 
                        baseline = distr(qbeta, 4, 36-4),
                        baseline_type = "response",
                        type = "response")
pred_RE_beta
plot(pred_RE_beta)


## ----blocker_tests, include=FALSE, eval=params$run_tests------------------------------------------
#--- Test against TSD 2 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# Relative effects
blocker_FE_releff <- as.data.frame(relative_effects(blocker_fit_FE))

test_that("FE relative effects", {
  expect_equivalent(blocker_FE_releff$mean, -0.26, tolerance = tol)
  expect_equivalent(blocker_FE_releff$sd, 0.050, tolerance = tol)
  expect_equivalent(blocker_FE_releff$`2.5%`, -0.36, tolerance = tol)
  expect_equivalent(blocker_FE_releff$`50%`, -0.26, tolerance = tol)
  expect_equivalent(blocker_FE_releff$`97.5%`, -0.16, tolerance = tol)
})

blocker_RE_releff <- as.data.frame(relative_effects(blocker_fit_RE))

test_that("RE relative effects", {
  expect_equivalent(blocker_RE_releff$mean, -0.25, tolerance = tol)
  expect_equivalent(blocker_RE_releff$sd, 0.066, tolerance = tol)
  expect_equivalent(blocker_RE_releff$`2.5%`, -0.38, tolerance = tol)
  expect_equivalent(blocker_RE_releff$`50%`, -0.25, tolerance = tol)
  expect_equivalent(blocker_RE_releff$`97.5%`, -0.12, tolerance = tol)
})

# RE heterogeneity SD
blocker_RE_sd <- as.data.frame(summary(blocker_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(blocker_RE_sd$mean, 0.14, tolerance = tol)
  expect_equivalent(blocker_RE_sd$sd, 0.082, tolerance = tol)
  expect_equivalent(blocker_RE_sd$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(blocker_RE_sd$`50%`, 0.13, tolerance = tol)
  expect_equivalent(blocker_RE_sd$`97.5%`, 0.32, tolerance = tol)
})

# DIC
test_that("FE DIC", {
  expect_equivalent(dic_FE$resdev, 46.8, tolerance = tol_dic)
  expect_equivalent(dic_FE$pd, 23.0, tolerance = tol_dic)
  expect_equivalent(dic_FE$dic, 69.8, tolerance = tol_dic)
})

test_that("RE DIC", {
  expect_equivalent(dic_RE$resdev, 41.9, tolerance = tol_dic)
  expect_equivalent(dic_RE$pd, 28.1, tolerance = tol_dic)
  expect_equivalent(dic_RE$dic, 70.0, tolerance = tol_dic)
})

# Predictions
blocker_pred_FE <- as.data.frame(pred_FE)

test_that("FE predicted probabilities", {
  expect_equivalent(blocker_pred_FE$mean, c(0.11, 0.09), tolerance = tol_dic)
  expect_equivalent(blocker_pred_FE$sd, c(0.055, 0.045), tolerance = tol_dic)
  expect_equivalent(blocker_pred_FE$`2.5%`, c(0.04, 0.03), tolerance = tol_dic)
  expect_equivalent(blocker_pred_FE$`50%`, c(0.10, 0.08), tolerance = tol_dic)
  expect_equivalent(blocker_pred_FE$`97.5%`, c(0.25, 0.20), tolerance = tol_dic)
})

blocker_pred_RE <- as.data.frame(pred_RE)

test_that("RE predicted probabilities", {
  expect_equivalent(blocker_pred_RE$mean, c(0.11, 0.09), tolerance = tol_dic)
  expect_equivalent(blocker_pred_RE$sd, c(0.055, 0.046), tolerance = tol_dic)
  expect_equivalent(blocker_pred_RE$`2.5%`, c(0.04, 0.03), tolerance = tol_dic)
  expect_equivalent(blocker_pred_RE$`50%`, c(0.10, 0.08), tolerance = tol_dic)
  expect_equivalent(blocker_pred_RE$`97.5%`, c(0.25, 0.20), tolerance = tol_dic)
})

# Check predictions with Beta distribution on baseline probability
blocker_predbeta_FE <- as.data.frame(pred_FE_beta)

test_that("FE predicted probabilities (Beta distribution)", {
  expect_equal(blocker_pred_FE$mean, blocker_predbeta_FE$mean, tolerance = tol)
  expect_equal(blocker_pred_FE$sd, blocker_predbeta_FE$sd, tolerance = tol)
  expect_equal(blocker_pred_FE$`2.5%`, blocker_predbeta_FE$`2.5%`, tolerance = tol)
  expect_equal(blocker_pred_FE$`50%`, blocker_predbeta_FE$`50%`, tolerance = tol)
  expect_equal(blocker_pred_FE$`97.5%`, blocker_predbeta_FE$`97.5%`, tolerance = tol)
})

blocker_predbeta_RE <- as.data.frame(pred_RE_beta)

test_that("RE predicted probabilities (Beta distribution)", {
  expect_equal(blocker_pred_RE$mean, blocker_predbeta_RE$mean, tolerance = tol)
  expect_equal(blocker_pred_RE$sd, blocker_predbeta_RE$sd, tolerance = tol)
  expect_equal(blocker_pred_RE$`2.5%`, blocker_predbeta_RE$`2.5%`, tolerance = tol)
  expect_equal(blocker_pred_RE$`50%`, blocker_predbeta_RE$`50%`, tolerance = tol)
  expect_equal(blocker_pred_RE$`97.5%`, blocker_predbeta_RE$`97.5%`, tolerance = tol)
})

# Test that ordered multinomial model is equivalent
blocker_ord_net <- set_agd_arm(blocker, 
                           study = studyn,
                           trt = trtc,
                           r = multi(nonevents = n, events = r, inclusive = TRUE),
                           trt_ref = "Control")

blocker_ord_fit_FE <-  nma(blocker_ord_net, 
                       trt_effects = "fixed",
                       link = "logit",
                       prior_intercept = normal(scale = 100),
                       prior_trt = normal(scale = 100),
                       prior_aux = flat())

blocker_ord_fit_RE <-  nma(blocker_ord_net, 
                       trt_effects = "random",
                       link = "logit",
                       prior_intercept = normal(scale = 100),
                       prior_trt = normal(scale = 100),
                       prior_het = half_normal(scale = 5),
                       prior_aux = flat())

blocker_ord_FE_releff <- as.data.frame(relative_effects(blocker_ord_fit_FE))

test_that("Equivalent ordered multinomial FE relative effects", {
  expect_equivalent(blocker_ord_FE_releff$mean, -0.26, tolerance = tol)
  expect_equivalent(blocker_ord_FE_releff$sd, 0.050, tolerance = tol)
  expect_equivalent(blocker_ord_FE_releff$`2.5%`, -0.36, tolerance = tol)
  expect_equivalent(blocker_ord_FE_releff$`50%`, -0.26, tolerance = tol)
  expect_equivalent(blocker_ord_FE_releff$`97.5%`, -0.16, tolerance = tol)
})

blocker_ord_RE_releff <- as.data.frame(relative_effects(blocker_ord_fit_RE))

test_that("Equivalent ordered multinomial RE relative effects", {
  expect_equivalent(blocker_ord_RE_releff$mean, -0.25, tolerance = tol)
  expect_equivalent(blocker_ord_RE_releff$sd, 0.066, tolerance = tol)
  expect_equivalent(blocker_ord_RE_releff$`2.5%`, -0.38, tolerance = tol)
  expect_equivalent(blocker_ord_RE_releff$`50%`, -0.25, tolerance = tol)
  expect_equivalent(blocker_ord_RE_releff$`97.5%`, -0.12, tolerance = tol)
})

blocker_ord_RE_sd <- as.data.frame(summary(blocker_ord_fit_RE, pars = "tau"))

test_that("Equivalent ordered multinomial RE heterogeneity SD", {
  expect_equivalent(blocker_ord_RE_sd$mean, 0.14, tolerance = tol)
  expect_equivalent(blocker_ord_RE_sd$sd, 0.082, tolerance = tol)
  expect_equivalent(blocker_ord_RE_sd$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(blocker_ord_RE_sd$`50%`, 0.13, tolerance = tol)
  expect_equivalent(blocker_ord_RE_sd$`97.5%`, 0.32, tolerance = tol)
})

test_that("Equivalent ordered multinomial FE DIC", {
  expect_equivalent(dic_FE$resdev, 46.8, tolerance = tol_dic)
  expect_equivalent(dic_FE$pd, 23.0, tolerance = tol_dic)
  expect_equivalent(dic_FE$dic, 69.8, tolerance = tol_dic)
})

test_that("Equivalent ordered multinomial RE DIC", {
  expect_equivalent(dic_RE$resdev, 41.9, tolerance = tol_dic)
  expect_equivalent(dic_RE$pd, 28.1, tolerance = tol_dic)
  expect_equivalent(dic_RE$dic, 70.0, tolerance = tol_dic)
})

blocker_ord_pred_FE <- as.data.frame(predict(blocker_ord_fit_FE, 
                                             baseline = distr(qnorm, mean = -2.2, sd = 3.3^-0.5),
                                             type = "response"))

test_that("Equivalent ordered multinomial FE predicted probabilities", {
  expect_equivalent(blocker_ord_pred_FE$mean, c(0.11, 0.09), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_FE$sd, c(0.055, 0.045), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_FE$`2.5%`, c(0.04, 0.03), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_FE$`50%`, c(0.10, 0.08), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_FE$`97.5%`, c(0.25, 0.20), tolerance = tol_dic)
})

blocker_ord_pred_RE <- as.data.frame(predict(blocker_ord_fit_RE, 
                                             baseline = distr(qnorm, mean = -2.2, sd = 3.3^-0.5),
                                             type = "response"))

test_that("Equivalent ordered multinomial RE predicted probabilities", {
  expect_equivalent(blocker_ord_pred_RE$mean, c(0.11, 0.09), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_RE$sd, c(0.055, 0.046), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_RE$`2.5%`, c(0.04, 0.03), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_RE$`50%`, c(0.10, 0.08), tolerance = tol_dic)
  expect_equivalent(blocker_ord_pred_RE$`97.5%`, c(0.25, 0.20), tolerance = tol_dic)
})

# Check predictions with Beta distribution on baseline probability
blocker_ord_predbeta_FE <- predict(blocker_ord_fit_FE, 
                               baseline = distr(qbeta, 4, 36 - 4), #3.66565, 36.74819 - 3.66565), 
                               baseline_type = "response",
                               type = "response") %>% 
  as.data.frame()

test_that("FE ordered multinomial predicted probabilities (Beta distribution)", {
  expect_equal(blocker_ord_pred_FE$mean, blocker_ord_predbeta_FE$mean, tolerance = tol)
  expect_equal(blocker_ord_pred_FE$sd, blocker_ord_predbeta_FE$sd, tolerance = tol)
  expect_equal(blocker_ord_pred_FE$`2.5%`, blocker_ord_predbeta_FE$`2.5%`, tolerance = tol)
  expect_equal(blocker_ord_pred_FE$`50%`, blocker_ord_predbeta_FE$`50%`, tolerance = tol)
  expect_equal(blocker_ord_pred_FE$`97.5%`, blocker_ord_predbeta_FE$`97.5%`, tolerance = tol)
})

blocker_ord_predbeta_RE <- predict(blocker_ord_fit_RE, 
                               baseline = distr(qbeta, 4, 36 - 4), #3.66565, 36.74819 - 3.66565), 
                               baseline_type = "response",
                               type = "response") %>% 
  as.data.frame()

test_that("RE ordered multinomial predicted probabilities (Beta distribution)", {
  expect_equal(blocker_ord_pred_RE$mean, blocker_ord_predbeta_RE$mean, tolerance = tol)
  expect_equal(blocker_ord_pred_RE$sd, blocker_ord_predbeta_RE$sd, tolerance = tol)
  expect_equal(blocker_ord_pred_RE$`2.5%`, blocker_ord_predbeta_RE$`2.5%`, tolerance = tol)
  expect_equal(blocker_ord_pred_RE$`50%`, blocker_ord_predbeta_RE$`50%`, tolerance = tol)
  expect_equal(blocker_ord_pred_RE$`97.5%`, blocker_ord_predbeta_RE$`97.5%`, tolerance = tol)
})


# Force clean up
rm(list = ls())
gc()

# Generated by vignette example_social_anxiety.Rmd: do not edit by hand
# Instead edit example_social_anxiety.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE, eval_multinomial = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----include=FALSE, setup-------------------------------------------------------------------------
library(multinma)
library(dplyr)      # dplyr and tidyr for data manipulation
library(tidyr)
library(ggplot2)    # ggplot2 for plotting

## ----include=FALSE--------------------------------------------------------------------------------
options(mc.cores = parallel::detectCores())

## ----include=FALSE--------------------------------------------------------------------------------
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(social_anxiety)


## -------------------------------------------------------------------------------------------------
sa_net <- set_agd_contrast(social_anxiety,
                           study = studyc, 
                           trt = trtc,
                           y = y,
                           se = se,
                           trt_class = classc,
                           trt_ref = "Waitlist")

sa_net


## -------------------------------------------------------------------------------------------------
plot(sa_net, level = "class") + 
  theme(legend.position = "bottom", legend.box = "vertical")


## ----echo=TRUE, results='hide'--------------------------------------------------------------------
set.seed(951)
sa_fit_FE <- nma(sa_net,
                 trt_effects = "fixed",
                 prior_trt = normal(0, 100),
                 prior_het = half_normal(5),
)

sa_fit_RE <- nma(sa_net,
                 trt_effects = "random",
                 prior_trt = normal(0, 100),
                 prior_het = half_normal(5),
)


## -------------------------------------------------------------------------------------------------
(sa_dic_FE <- dic(sa_fit_FE))
(sa_dic_RE <- dic(sa_fit_RE))


## ----echo=TRUE, results='hide'--------------------------------------------------------------------
sa_UME_RE <- nma(sa_net,
                 trt_effects = "random",
                 consistency = "ume",
                 prior_trt = normal(0, 100),
                 prior_het = half_normal(5))


## -------------------------------------------------------------------------------------------------
(sa_dic_RE <- dic(sa_fit_RE))
(sa_dic_ume_RE <- dic(sa_UME_RE))

summary(sa_UME_RE, pars = "tau")
summary(sa_fit_RE, pars = "tau")


## -------------------------------------------------------------------------------------------------
plot(sa_dic_RE, sa_dic_ume_RE, show_uncertainty = FALSE) +
  xlab("Residual deviance - No Class model") +
  ylab("Residual deviance - UME model")


## -------------------------------------------------------------------------------------------------
as.data.frame(sa_dic_RE) %>%
  arrange(desc(resdev)) %>%
  head(5)


## ----echo=TRUE, results='hide', eval=!params$run_tests--------------------------------------------
# EMMELKAMP2006 <- data.frame(
#   Treatment_1 = c("CBT individual", "Waitlist", "Waitlist"),
#   Treatment_2 = c("Psychodynamic psychotherapy", "Psychodynamic psychotherapy", "CBT individual")
# )
# 
# ALDEN2011 <- data.frame(
#   Treatment_1 = c("Waitlist"),
#   Treatment_2 = c("CBT group")
# )
# 
# sa_fit_RE_nodesplit_EMMELKAMP <- nma(sa_net,
#                            consistency = "nodesplit",
#                            nodesplit = EMMELKAMP2006,
#                            trt_effects = "random",
#                            prior_trt = normal(0, 100),
#                            prior_het = half_normal(5),
# )
# 
# sa_fit_RE_nodesplit_ALDEN <- nma(sa_net,
#                              consistency = "nodesplit",
#                              nodesplit = ALDEN2011,
#                              trt_effects = "random",
#                              prior_trt = normal(0, 100),
#                              prior_het = half_normal(5),
# )


## ----eval=!params$run_tests-----------------------------------------------------------------------
# summary(sa_fit_RE_nodesplit_ALDEN)
# summary(sa_fit_RE_nodesplit_EMMELKAMP)


## ----echo=TRUE, results='hide'--------------------------------------------------------------------
sa_fit_EXclass_RE <- nma(sa_net,
                         trt_effects = "random",
                         prior_trt = normal(0, 100),
                         prior_het = half_normal(5),
                         class_effects = "exchangeable",
                         prior_class_mean = normal(0, 10),
                         prior_class_sd = normal(0.33,0.1),
                         class_sd = 
                           list(`Exercise and SH no support` = 
                                  c("Exercise promotion", "Self-help no support"),
                                `SSRIs and NSSA` = 
                                  c("SSRI/SNRI", "NSSA"),
                                `Psychodynamic & Other psychological therapies` =
                                  c("Psychodynamic psychotherapy", "Other psychological therapies"))
                         )


## -------------------------------------------------------------------------------------------------
(sa_dic_EXclass_RE <- dic(sa_fit_EXclass_RE))
(sa_dic_RE <- dic(sa_fit_RE))

summary(sa_fit_RE, pars = "tau")
summary(sa_fit_EXclass_RE, pars = "tau")


## -------------------------------------------------------------------------------------------------
plot(sa_dic_EXclass_RE, sa_dic_RE, show_uncertainty = FALSE) +
  xlab("Residual deviance - Exchangeable Class model") +
  ylab("Residual deviance - No Class model")


## ----echo=TRUE, results='hide'--------------------------------------------------------------------
sa_fit_COclass_RE <- nma(sa_net,
                         trt_effects = "random",
                         prior_trt = normal(0, 100),
                         prior_het = half_normal(5),
                         class_effects = "common")

sa_fit_EXclass_FE <- nma(sa_net,
                         trt_effects = "fixed",
                         prior_trt = normal(0, 100),
                         prior_het = half_normal(5),
                         class_effects = "exchangeable",
                         prior_class_mean = normal(0, 10),
                         prior_class_sd = normal(0.33,0.1),
                         class_sd = 
                           list(`Exercise and SH no support` = 
                                  c("Exercise promotion", "Self-help no support"),
                                `SSRIs and NSSA` = 
                                  c("SSRI/SNRI", "NSSA"),
                                `Psychodynamic & Other psychological therapies` = 
                                  c("Psychodynamic psychotherapy", "Other psychological therapies"))
                         )


## -------------------------------------------------------------------------------------------------
(sa_dic_COclass_RE <- dic(sa_fit_COclass_RE))
(sa_dic_EXclass_FE <- dic(sa_fit_EXclass_FE))
(sa_dic_EXclass_RE <- dic(sa_fit_EXclass_RE))

summary(sa_fit_COclass_RE, pars = "tau")
summary(sa_fit_EXclass_RE, pars = "tau")


## -------------------------------------------------------------------------------------------------
plot(sa_dic_COclass_RE, sa_dic_EXclass_RE, show_uncertainty = FALSE) +
  xlab("Residual deviance - Common Class model") +
  ylab("Residual deviance - Exchangeable Class model")


## -------------------------------------------------------------------------------------------------
plot(relative_effects(sa_fit_EXclass_RE), ref_line = 0)


## -------------------------------------------------------------------------------------------------
plot(sa_fit_EXclass_RE,
     pars = "class_mean",
     ref_line = 0)


## ----fig.height=8---------------------------------------------------------------------------------
# Relative treatment effects
trt_eff <- as_tibble(relative_effects(sa_fit_EXclass_RE)) %>% 
  # Add in class details
  mutate(Class = sa_net$classes[as.numeric(.trtb)],
         level = "treatment")

# Class effects
class_eff <- as_tibble(summary(sa_fit_EXclass_RE, pars = "class_mean")) %>% 
  # Extract class details
  mutate(Class = factor(gsub(".*\\[(.+)\\]", "\\1", parameter), levels = levels(sa_net$classes)),
         level = "class",
         .trtb = factor("Class Mean", levels = c(levels(sa_net$classes), "Class Mean")))

# Combine and plot
bind_rows(trt_eff, class_eff) %>% 
  ggplot(aes(y = .trtb, 
             x = mean, xmin = `2.5%`, xmax = `97.5%`,
             colour = level, shape = level)) +
  geom_vline(xintercept = 0, colour = "grey60") +
  geom_pointrange() +
  facet_grid(rows = "Class", scales = "free", space = "free", labeller = label_wrap_gen(22)) +
  scale_shape_manual(values = c(15, 16), guide = guide_none()) +
  scale_colour_manual(values = c("#113259", "#55A480"), guide = guide_none()) +
  xlab("SMD") + ylab("") +
  theme_multinma() +
  theme(strip.text.y = element_text(angle = 0))


## -------------------------------------------------------------------------------------------------
# Class means
EXclass_mean <- as.matrix(sa_fit_EXclass_RE, pars = "class_mean")

EXclass_mean <- cbind(`d[Reference]` = 0, EXclass_mean)

# Take ranks at each iteration
EXranks <- t(apply(EXclass_mean, 1, rank))

# Get median rank and 95% credible interval
EXresults <- t(apply(EXranks, 2, quantile, probs = c(0.025, 0.5, 0.975)))

# Convert to data frame
EXresults_df <- as.data.frame(EXresults)
EXresults_df$class <- rownames(EXresults_df)

EXresults_df$class <- factor(EXresults_df$class, 
                             levels = sort(unique(EXresults_df$class), decreasing = TRUE))

ggplot(EXresults_df, aes(x=class, y=`50%`, ymin=`2.5%`, ymax=`97.5%`)) +
geom_pointrange(size = 0.5) +
coord_flip() +
xlab("Class") + ylab("Posterior Ranks") +
theme_multinma()


## -------------------------------------------------------------------------------------------------
EXranks_df <- as.data.frame(EXranks)

# Rank probabilities for class
rank_probs_EX <- apply(EXranks_df, 2, function(x) table(factor(x, levels = 1:ncol(EXranks_df))) / nrow(EXranks_df))

# Convert to data frame
rank_probs_df_EX <- as.data.frame(rank_probs_EX)

# Convert the data frame to a long format
rank_probs_long_EX <- rank_probs_df_EX %>%
  mutate(Rank = row_number()) %>%
  pivot_longer(
    cols = -Rank,
    names_to = "Class",
    values_to = "Probability"
  )

# Plot density
ggplot(rank_probs_long_EX, aes(x = Rank, y = Probability)) +
  geom_line() +
  facet_wrap(~ Class) +
  theme_multinma() +
  labs(x = "Rank",
       y = "Probability")


## ----socialanxiety_tests, include=FALSE, eval=params$run_tests------------------------------------
#--- Test against TSD 4 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# Relative effects
sa_EX_RE_releff <- as.data.frame(relative_effects(sa_fit_EXclass_RE))

# EX RE model parameters
test_ex_re <- tribble(
  ~parameter, ~mean, ~sd, ~`2.5%`, ~`50%`, ~`97.5%`,
  "d[Alprazolam]", -0.8314958, 0.28125207, -1.3755076, -0.8307162, -0.27711275,
  "d[CBT group]", -0.8142867, 0.09621211, -1.0065441, -0.8131570, -0.63148837,
  "d[CBT group + Fluoxetine]", -0.9211830, 0.19446183, -1.3156687, -0.9204697, -0.54765137,
  "d[CBT group + Moclobemide]", -1.2066027, 0.25000952, -1.6962705, -1.213031, -0.70271794,
  "d[CBT group + Phenelzine]", -1.6973883, 0.22347273, -2.1374866, -1.6956205, -1.26489286,
  "d[CBT group Enhanced]", -1.0473793, 0.21208307, -1.4793471, -1.0441864, -0.64037100,
  "d[CBT group Heimberg]", -0.7791976, 0.11644941, -1.0081920, -0.7790447, -0.55037257,
  "d[CBT individual]", -1.1730443, 0.15200737, -1.4664337, -1.1747683, -0.87424063,
  "d[CBT individual Heimberg]", -1.0210418, 0.19936795, -1.4083739, -1.0193569, -0.62629645,
  "d[Citalopram]", -0.8467242, 0.19163580, -1.2173637, -0.8569810, -0.43683285,
  "d[Clonazapam]", -1.0556030, 0.19780158, -1.4377446, -1.0581377, -0.67179776,
  "d[Cognitive therapy]", -1.5262932, 0.15985534, -1.8486260, -1.5249683, -1.21370893,
  "d[Cognitive therapy shortened sessions]", -0.9947664, 0.12715601, -1.2402119, -0.9965832, -0.73904361,
  "d[Escitalopram]", -0.8607573, 0.15488520, -1.1585547, -0.8654861, -0.54829102,
  "d[Exercise promotion]", -0.1805154, 0.40643886, -0.9872490, -0.1819663, 0.60912243,
  "d[Exposure in vivo]", -0.8190751, 0.13069686, -1.0765767, -0.8175697, -0.56103203,
  "d[Fluoxetine]", -0.8576191, 0.14304817, -1.1261241, -0.8605516, -0.57023950,
  "d[Fluvoxamine]", -0.9064442, 0.15317111, -1.2144374, -0.9042166, -0.60930856,
  "d[Gabapentin]", -0.8511784, 0.26831004, -1.3791457, -0.8477352, -0.32055234,
  "d[Interpersonal psychotherapy]", -0.4179080, 0.20660264, -0.8173726, -0.4137257, -0.01862694,
  "d[Levetiracetam]", -0.7959018, 0.32429393, -1.4507038, -0.7888249, -0.16125339,
  "d[Mindfulness]", -0.3321746, 0.21466120, -0.7685138, -0.3299451, 0.09519575,
  "d[Mirtazapine]", -0.7650427, 0.33297935, -1.4222047, -0.7676135, -0.11613886,
  "d[Moclobemide]", -0.7108082, 0.15076437, -1.0025218, -0.7098811, -0.41162089,
  "d[Paroxetine]", -0.9518310, 0.13803974, -1.2327792, -0.9530097, -0.68226256,
  "d[Paroxetine + Clonazapam]", -1.3265332, 0.29791193, -1.9052410, -1.3210094, -0.74984083,
  "d[Phenelzine]", -1.2715405, 0.15435898, -1.5720287, -1.2731792, -0.95776860,
  "d[Pill placebo]", -0.4342747, 0.12462281, -0.6844952, -0.4348165, -0.18907267,
  "d[Pregabalin]", -0.6868730, 0.18355833, -1.0606219, -0.6856703, -0.33094991,
  "d[Psychodynamic + Clonazepam]", -1.2161722, 0.28910209, -1.7912116, -1.2176852, -0.65840233,
  "d[Psychodynamic psychotherapy]", -0.6235235, 0.16406580, -0.9554527, -0.6227220, -0.29926675,
  "d[Psychological placebo]", -0.6467276, 0.15004879, -0.9335005, -0.6508577, -0.35282347,
  "d[Self-help book no support]", -0.7338743, 0.12963766, -0.9942650, -0.7345158, -0.48176798,
  "d[Self-help book with support]", -0.8347549, 0.17192892, -1.1769224, -0.8370861, -0.49302079,
  "d[Self-help internet no support]", -0.6028340, 0.14218310, -0.8756893, -0.6037919, -0.32938047,
  "d[Self-help internet with support]", -0.8435101, 0.08756184, -1.0161868, -0.8422352, -0.67559595,
  "d[Sertraline]", -0.8870449, 0.15228323, -1.1909221, -0.8867806, -0.57684247,
  "d[Social skills training]", -0.8263210, 0.27080686, -1.3600444, -0.8290302, -0.29391996,
  "d[Supportive therapy]", -0.2475428, 0.23241985, -0.6929505, -0.2487845, 0.21323466,
  "d[Venlafaxine]", -0.9027961, 0.14619543, -1.1883951, -0.9010945, -0.61143773)

test_that("FE relative effects", {
  expect_equivalent(sa_EX_RE_releff$mean, test_ex_re$mean, tolerance = tol)
  expect_equivalent(sa_EX_RE_releff$sd, test_ex_re$sd, tolerance = tol)
  expect_equivalent(sa_EX_RE_releff$`2.5%`, test_ex_re$`2.5%`, tolerance = tol)
  expect_equivalent(sa_EX_RE_releff$`50%`, test_ex_re$`50%`, tolerance = tol)
  expect_equivalent(sa_EX_RE_releff$`97.5%`, test_ex_re$`97.5%`, tolerance = tol)
})

# DIC checks
test_that("FE DIC", {
  expect_equivalent(sa_dic_FE$resdev, 288.3, tolerance = tol_dic)
  expect_equivalent(sa_dic_FE$pd, 40.1, tolerance = tol_dic)
  expect_equivalent(sa_dic_FE$dic, 328.4, tolerance = tol_dic)
})

test_that("RE DIC", {
  expect_equivalent(sa_dic_RE$resdev, 162.6, tolerance = tol_dic)
  expect_equivalent(sa_dic_RE$pd, 94.8, tolerance = tol_dic)
  expect_equivalent(sa_dic_RE$dic, 257.4, tolerance = tol_dic)
})

test_that("UME RE DIC", {
  expect_equivalent(sa_dic_ume_RE$resdev, 160.7, tolerance = tol_dic)
  expect_equivalent(sa_dic_ume_RE$pd, 108.3, tolerance = tol_dic)
  expect_equivalent(sa_dic_ume_RE$dic, 269, tolerance = tol_dic)
})

test_that("EX RE DIC", {
  expect_equivalent(sa_dic_EXclass_RE$resdev, 162.5, tolerance = tol_dic)
  expect_equivalent(sa_dic_EXclass_RE$pd, 87.6, tolerance = tol_dic)
  expect_equivalent(sa_dic_EXclass_RE$dic, 250.1, tolerance = tol_dic)
})

test_that("CO RE DIC", {
  expect_equivalent(sa_dic_COclass_RE$resdev, 158.4, tolerance = tol_dic)
  expect_equivalent(sa_dic_COclass_RE$pd, 93.1, tolerance = tol_dic)
  expect_equivalent(sa_dic_COclass_RE$dic, 251.5, tolerance = tol_dic)
})

test_that("EX FE DIC", {
  expect_equivalent(sa_dic_EXclass_FE$resdev, 285, tolerance = tol_dic)
  expect_equivalent(sa_dic_EXclass_FE$pd, 34.5, tolerance = tol_dic)
  expect_equivalent(sa_dic_EXclass_FE$dic, 319.4, tolerance = tol_dic)
})

#--- Tau checks ---
CO_RE_tau <- summary(sa_fit_COclass_RE, pars = "tau")
EX_RE_tau <- summary(sa_fit_EXclass_RE, pars = "tau")
RE_tau <- summary(sa_fit_RE, pars = "tau")
UME_RE_tau <- summary(sa_UME_RE, pars = "tau")

test_that("CO RE Tau", {
  expect_equivalent(CO_RE_tau$summary$mean, 0.252, tolerance = tol)
  expect_equivalent(CO_RE_tau$summary$sd, 0.0291, tolerance = tol)
  expect_equivalent(CO_RE_tau$summary$`2.5%`, 0.198, tolerance = tol)
  expect_equivalent(CO_RE_tau$summary$`50%`, 0.251, tolerance = tol)
  expect_equivalent(CO_RE_tau$summary$`97.5%`, 0.312, tolerance = tol)
})

test_that("EX RE Tau", {
  expect_equivalent(EX_RE_tau$summary$mean, 0.196, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$sd, 0.0298, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`2.5%`, 0.142, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`50%`, 0.195, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`97.5%`, 0.257, tolerance = tol)
})

test_that("RE Tau", {
  expect_equivalent(RE_tau$summary$mean, 0.207, tolerance = tol)
  expect_equivalent(RE_tau$summary$sd, 0.0308, tolerance = tol)
  expect_equivalent(RE_tau$summary$`2.5%`, 0.149, tolerance = tol)
  expect_equivalent(RE_tau$summary$`50%`, 0.207, tolerance = tol)
  expect_equivalent(RE_tau$summary$`97.5%`, 0.269, tolerance = tol)
})

test_that("UME RE Tau", {
  expect_equivalent(EX_RE_tau$summary$mean, 0.221, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$sd, 0.0333, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`2.5%`, 0.158, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`50%`, 0.221, tolerance = tol)
  expect_equivalent(EX_RE_tau$summary$`97.5%`, 0.289, tolerance = tol)
})

# nodesplit_ALDEN <- summary(sa_fit_RE_nodesplit_ALDEN)
# nodesplit_EMMELKAMP <- summary(sa_fit_RE_nodesplit_EMMELKAMP)
# 
# ALDEN_test <- tribble(
#   ~parameter, ~mean, ~sd, ~`2.5%`, ~`50%`, ~`97.5%`,
#   "d_dir[CBT group vs. Waitlist]", -0.90, 0.13, -1.16, -0.90, -0.64,
#   "d_ind[CBT group vs. Waitlist]", -0.73, 0.13, -0.99, -0.73, -0.46,
#   "omega", -0.17, 0.17, -0.51, -0.17, 0.18,
#   "tau", 0.21, 0.03, 0.15, 0.21, 0.27
# )
# 
# 
# EMMELKAMP_test1 <- tribble(
#   ~parameter, ~mean, ~sd, ~`2.5%`, ~`25%`, ~`50%`, ~`75%`, ~`97.5%`, ~Bulk_ESS, ~Tail_ESS, ~Rhat,
#   "d_dir", 0.71, 0.33, 0.06, 0.49, 0.71, 0.94, 1.35, 5354, 2759, 1.00,
#   "d_ind", 0.45, 0.26, -0.07, 0.27, 0.45, 0.63, 0.96, 4159, 3058, 1.00,
#   "omega", 0.26, 0.39, -0.51, 0.00, 0.25, 0.53, 1.02, 4849, 3147, 1.00,
#   "tau", 0.21, 0.03, 0.15, 0.19, 0.21, 0.23, 0.27, 1291, 2516, 1.01
# )
# 
# EMMELKAMP_test2 <- tribble(
#   ~parameter, ~mean, ~sd, ~`2.5%`, ~`25%`, ~`50%`, ~`75%`, ~`97.5%`, ~Bulk_ESS, ~Tail_ESS, ~Rhat,
#   "d_dir", -0.61, 0.19, -0.96, -0.73, -0.61, -0.48, -0.22, 5103, 3262, 1.00,
#   "d_ind", -0.68, 0.42, -1.49, -0.96, -0.68, -0.40, 0.15, 4157, 3138, 1.00,
#   "omega", 0.08, 0.46, -0.84, -0.23, 0.07, 0.39, 0.98, 4293, 2877, 1.00,
#   "tau", 0.21, 0.03, 0.15, 0.19, 0.21, 0.23, 0.27, 959, 1921, 1.00
# )
# 
# EMMELKAMP_test3 <- tribble(
#   ~parameter, ~mean, ~sd, ~`2.5%`, ~`25%`, ~`50%`, ~`75%`, ~`97.5%`, ~Bulk_ESS, ~Tail_ESS, ~Rhat,
#   "d_dir", -0.84, 0.25, -1.32, -1.00, -0.84, -0.67, -0.36, 4676, 3392, 1,
#   "d_ind", -1.46, 0.24, -1.93, -1.62, -1.46, -1.30, -0.99, 1466, 2413, 1,
#   "omega", 0.62, 0.34, -0.04, 0.39, 0.62, 0.85, 1.31, 1999, 2802, 1,
#   "tau", 0.20, 0.03, 0.15, 0.18, 0.20, 0.22, 0.26, 1238, 1834, 1
# )
# 
# test_that("Node-splitting estimates_ALDEN", {
#   expect_equal(ALDEN_test$mean, as.numeric(nodesplit_ALDEN$summary[[1]]$summary$mean), tolerance = tol)
#   expect_equal(ALDEN_test$sd, as.numeric(nodesplit_ALDEN$summary[[1]]$summary$sd), tolerance = tol)
#   expect_equal(ALDEN_test$`2.5%`, as.numeric(nodesplit_ALDEN$summary[[1]]$summary$`2.5%`), tolerance = tol)
#   expect_equal(ALDEN_test$`50%`, as.numeric(nodesplit_ALDEN$summary[[1]]$summary$`50%`), tolerance = tol)
#   expect_equal(ALDEN_test$`97.5%`, as.numeric(nodesplit_ALDEN$summary[[1]]$summary$`97.5%`), tolerance = tol)
# })
# 
# test_that("Node-splitting estimates_EMMEL1", {
#   expect_equal(EMMELKAMP_test1$mean, as.numeric(nodesplit_EMMELKAMP$summary[[1]]$summary$mean), tolerance = tol)
#   expect_equal(EMMELKAMP_test1$sd, as.numeric(nodesplit_EMMELKAMP$summary[[1]]$summary$sd), tolerance = tol)
#   expect_equal(EMMELKAMP_test1$`2.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[1]]$summary$`2.5%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test1$`50%`, as.numeric(nodesplit_EMMELKAMP$summary[[1]]$summary$`50%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test1$`97.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[1]]$summary$`97.5%`), tolerance = tol)
# })
# 
# test_that("Node-splitting estimates_EMMEL2", {
#   expect_equal(EMMELKAMP_test2$mean, as.numeric(nodesplit_EMMELKAMP$summary[[2]]$summary$mean), tolerance = tol)
#   expect_equal(EMMELKAMP_test2$sd, as.numeric(nodesplit_EMMELKAMP$summary[[2]]$summary$sd), tolerance = tol)
#   expect_equal(EMMELKAMP_test2$`2.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[2]]$summary$`2.5%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test2$`50%`, as.numeric(nodesplit_EMMELKAMP$summary[[2]]$summary$`50%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test2$`97.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[2]]$summary$`97.5%`), tolerance = tol)
# })
# 
# test_that("Node-splitting estimates_EMMEL3", {
#   expect_equal(EMMELKAMP_test3$mean, as.numeric(nodesplit_EMMELKAMP$summary[[3]]$summary$mean), tolerance = tol)
#   expect_equal(EMMELKAMP_test3$sd, as.numeric(nodesplit_EMMELKAMP$summary[[3]]$summary$sd), tolerance = tol)
#   expect_equal(EMMELKAMP_test3$`2.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[3]]$summary$`2.5%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test3$`50%`, as.numeric(nodesplit_EMMELKAMP$summary[[3]]$summary$`50%`), tolerance = tol)
#   expect_equal(EMMELKAMP_test3$`97.5%`, as.numeric(nodesplit_EMMELKAMP$summary[[3]]$summary$`97.5%`), tolerance = tol)
# })


# Force clean up
rm(list = ls())
gc()

#' Example smoking FE NMA
#'
#' Smoking FE NMA for use in examples.
#'
#' @name example_smk_fe
#' @rdname aa_example_smk_fe
#' @description Calling `example("example_smk_fe")` will run a fixed effects
#'   NMA model with the smoking cessation data, using the code in the Examples
#'   section below. The resulting `stan_nma` object `smk_fit_FE` will then be
#'   available in the global environment.
#'
#' @keywords examples
#' @template ex_smoking_network
#' @template ex_smoking_nma_fe
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("smk_net", smk_net, .GlobalEnv)
#'   assign("smk_fit_FE", smk_fit_FE, .GlobalEnv)
#' }
#' }
NULL

#' Example smoking RE NMA
#'
#' Smoking RE NMA for use in examples.
#'
#' @name example_smk_re
#' @rdname aa_example_smk_re
#' @description Calling `example("example_smk_re")` will run a random effects
#'   NMA model with the smoking cessation data, using the code in the Examples
#'   section below. The resulting `stan_nma` object `smk_fit_RE` will then be
#'   available in the global environment.
#'
#' @keywords examples
#' @template ex_smoking_network
#' @template ex_smoking_nma_re
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("smk_net", smk_net, .GlobalEnv)
#'   assign("smk_fit_RE", smk_fit_RE, .GlobalEnv)
#' }
#' }
NULL

#' Example smoking UME NMA
#'
#' Smoking UME NMA for use in examples.
#'
#' @name example_smk_ume
#' @rdname aa_example_smk_ume
#' @description Calling `example("example_smk_ume")` will run an unrelated mean
#'   effects (inconsistency) NMA model with the smoking cessation data, using the
#'   code in the Examples section below. The resulting `stan_nma` object
#'   `smk_fit_RE_UME` will then be available in the global environment.
#'
#' @keywords examples
#' @template ex_smoking_network
#' @template ex_smoking_nma_re_ume
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("smk_net", smk_net, .GlobalEnv)
#'   assign("smk_fit_RE_UME", smk_fit_RE_UME, .GlobalEnv)
#' }
#' }
NULL

#' Example smoking node-splitting
#'
#' Smoking node-splitting for use in examples.
#'
#' @name example_smk_nodesplit
#' @rdname aa_example_smk_nodesplit
#' @description Calling `example("example_smk_nodesplit")` will run
#'   node-splitting models with the smoking cessation data, using the code in
#'   the Examples section below. The resulting `nma_nodesplit_df` object
#'   `smk_fit_RE_nodesplit` will then be available in the global environment.
#'
#' @keywords examples
#' @template ex_smoking_network
#' @template ex_smoking_nma_re_nodesplit
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("smk_net", smk_net, .GlobalEnv)
#'   assign("smk_fit_RE_nodesplit", smk_fit_RE_nodesplit, .GlobalEnv)
#' }
#' }
NULL

#' Example plaque psoriasis ML-NMR
#'
#' Plaque psoriasis ML-NMR for use in examples.
#'
#' @name example_pso_mlnmr
#' @rdname aa_example_pso_mlnmr
#' @description Calling `example("example_pso_mlnmr")` will run a ML-NMR model
#'   with the plaque psoriasis IPD and AgD, using the code in the Examples
#'   section below. The resulting `stan_nma` object `pso_fit` will then be
#'   available in the global environment.
#'
#' @keywords examples
#' @template ex_plaque_psoriasis_network
#' @template ex_plaque_psoriasis_integration
#' @template ex_plaque_psoriasis_mlnmr
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("pso_net", pso_net, .GlobalEnv)
#'   assign("pso_fit", pso_fit, .GlobalEnv)
#' }
#' }
NULL

#' Example newly-diagnosed multiple myeloma
#'
# Newly-diagnosed multiple myeloma progression-free survival in a proportional
# hazards Weibull NMA for use in examples.
#'
#' @name example_ndmm
#' @rdname aa_example_ndmm
#' @description Calling `example("example_ndmm")` will run a proportional
#'   hazards Weibull NMA model on the newly-diagnosed multiple myeloma data,
#'   using the code in the Examples section below. The resulting `stan_nma`
#'   object `ndmm_fit` will then be available in the global environment.
#'
#' @keywords examples
#' @template ex_ndmm_network
#' @template ex_ndmm
#' @examples \dontshow{
#' if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
#'   assign("ndmm_net", ndmm_net, .GlobalEnv)
#'   assign("ndmm_fit", ndmm_fit, .GlobalEnv)
#' }
#' }
NULL

#' Distribution functions for M-spline baseline hazards
#'
#' Density, distribution, quantile, hazard, cumulative hazard, and restricted
#' mean survival time functions for the M-spline baseline hazards model.
#'
#' @param x,q Vector of quantiles
#' @param basis M-spline basis produced by [splines2::mSpline()]
#' @param scoef Vector (or matrix) of spline coefficients with length (or number
#'   of columns) equal to the dimension of `basis`
#' @param rate Vector of rate parameters
#' @param log,log.p Logical; if `TRUE`, probabilities `p` are given as
#'   \eqn{\log(p)}
#' @param lower.tail Logical; if `TRUE` (the default), probabilities are
#'   \eqn{P(X \le x)}, otherwise \eqn{P(X > x)}
#' @param p Vector of probabilities
#' @param t Vector of times to which the restricted mean survival time is
#'   calculated
#' @param start Optional left-truncation time or times. The returned restricted
#'   mean survival will be conditioned on survival up to this time
#'
#' @details Survival models with a flexible M-spline on the baseline hazard are
#'   described by \insertCite{Phillippo_mspline;textual}{multinma}.
#'   Piecewise-exponential baseline hazards are a special case where the degree
#'   of the M-spline polynomial is 0.
#'
#'   The d/p/h/H functions are calculated from their definitions. `qmspline()`
#'   uses numerical inversion via [flexsurv::qgeneric()]. `rmst_mspline()`uses
#'   numerical integration via [flexsurv::rmst_generic()], except for the
#'   special case of the piecewise-exponential hazard (i.e. degree 0 M-splines)
#'   which uses the explicit formula from
#'   \insertCite{Royston2013;textual}{multinma}.
#'
#'   Beyond the boundary knots, the hazard is assumed to be constant. (This
#'   differs from the approach in [splines2::mSpline()] that extrapolates the
#'   polynomial basis functions, which is numerically unstable and highly
#'   dependent on the data just before the boundary knots.) As with all
#'   extrapolation, care should be taken when evaluating the splines at times
#'   beyond the boundary knots (either directly through the d/p/h/H/rmst
#'   functions, or indirectly by requesting quantiles with `qmspline()` that
#'   correspond to times beyond the boundary knots). For this reason evaluating
#'   the (unrestricted) mean survival time is not generally recommended as this
#'   requires integrating over an infinite time horizon (i.e. `rmst_mspline()`
#'   with `t = Inf`).
#'
#' @return `dmspline()` gives the density, `pmspline()` gives the distribution
#'   function (CDF), `qmspline()` gives the quantile function (inverse-CDF),
#'   `hmspline()` gives the hazard function, `Hmspline()` gives the cumulative
#'   hazard function, and `rmst_mspline()` gives restricted mean survival times.
#'
#' @rdname mspline
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
dmspline <- function(x, basis, scoef, rate, log = FALSE) {
  if (!is_mspline(basis)) abort("`basis` must be an M-spline basis produced by splines2::mSpline()")
  if (!rlang::is_bool(log)) abort("`log` must be a logical value (TRUE or FALSE).")

  if (missing(x)) out <- hmspline(basis = basis, scoef = scoef, rate = rate) * pmspline(basis = basis, scoef = scoef, rate = rate, lower.tail = FALSE)
  else out <- hmspline(x, basis = basis, scoef = scoef, rate = rate) * pmspline(q = x, basis = basis, scoef = scoef, rate = rate, lower.tail = FALSE)

  if (log) out <- log(out)

  return(out)
}

#' @rdname mspline
#' @export
pmspline <- function(q, basis, scoef, rate, lower.tail = TRUE, log.p = FALSE) {
  if (!is_mspline(basis)) abort("`basis` must be an M-spline basis produced by splines2::mSpline()")
  if (!rlang::is_bool(lower.tail)) abort("`lower.tail` must be a logical value (TRUE or FALSE).")
  if (!rlang::is_bool(log.p)) abort("`log.p` must be a logical value (TRUE or FALSE).")

  if (missing(q)) out <- exp(-Hmspline(basis = basis, scoef = scoef, rate = rate))
  else  out <- exp(-Hmspline(x = q, basis = basis, scoef = scoef, rate = rate))

  if (lower.tail) out <- 1 - out
  if (log.p) out <- log(out)

  return(out)
}

#' @rdname mspline
#' @export
qmspline <- function(p, basis, scoef, rate, lower.tail = TRUE, log.p = FALSE) {
  if (!is.numeric(p)) abort("`p` must be a numeric vector of quantiles.")
  require_pkg("flexsurv")

  flexsurv::qgeneric(pmspline, p = p,
                     scalarargs = "basis", matargs = "scoef",
                     basis = basis, scoef = scoef, rate = rate,
                     lower.tail = lower.tail, log.p = log.p,
                     lbound = 0)
}

#' @rdname mspline
#' @export
hmspline <- function(x, basis, scoef, rate, log = FALSE) {
  require_pkg("splines2")
  if (!is_mspline(basis)) abort("`basis` must be an M-spline basis produced by splines2::mSpline()")
  if (!rlang::is_bool(log)) abort("`log` must be a logical value (TRUE or FALSE).")

  # Extrapolate with constant hazard beyond boundary knots
  if (!missing(x)) {
    x <- pmax(x, attr(basis, "Boundary.knots")[1])
    x <- pmin(x, attr(basis, "Boundary.knots")[2])
  }

  xb <- if (missing(x)) update(basis, integral = FALSE)
  else update(basis, x = x, integral = FALSE)

  if (!is.matrix(scoef)) scoef <- matrix(scoef, nrow = 1)

  if (is.matrix(scoef) && nrow(scoef) == nrow(xb)) {
    out <- rowSums(xb * scoef) * rate
  } else if (nrow(xb) == 1) {
    out <- scoef %*% xb[1,] * rate
  } else if (is.matrix(scoef) && nrow(scoef) == 1) {
    out <- xb %*% scoef[1,] * rate
  } else {
    out <- xb %*% scoef * rate
  }

  # Return 0 for x < 0
  lt0 <- x < 0
  out[lt0] <- 0

  if (log) out <- log(out)

  return(drop(out))
}

#' @rdname mspline
#' @export
Hmspline <- function(x, basis, scoef, rate, log = FALSE) {
  require_pkg("splines2")
  if (!is_mspline(basis)) abort("`basis` must be an M-spline basis produced by splines2::mSpline()")
  if (!rlang::is_bool(log)) abort("`log` must be a logical value (TRUE or FALSE).")

  # Extrapolate with constant hazard beyond boundary knots
  if (!missing(x)) {
    lower <- attr(basis, "Boundary.knots")[1]
    upper <- attr(basis, "Boundary.knots")[2]

    ex_lo <- x < lower & x > 0
    ex_up <- x > upper

    ex_time_lo <- x[ex_lo]
    ex_time_up <- x[ex_up] - upper

    xorig <- x
    x <- pmax(x, lower)
    x <- pmin(x, upper)
  } else {
    ex_lo <- ex_up <- FALSE
  }

  xb <- if (missing(x)) update(basis, integral = TRUE)
        else update(basis, x = x, integral = TRUE)

  if (!is.matrix(scoef)) scoef <- matrix(scoef, nrow = 1)

  if (is.matrix(scoef) && nrow(scoef) == nrow(xb)) {
    out <- rowSums(xb * scoef) * rate
  } else if (nrow(xb) == 1) {
    out <- scoef %*% xb[1,] * rate
  } else if (is.matrix(scoef) && nrow(scoef) == 1) {
    out <- xb %*% scoef[1,] * rate
  } else {
    out <- xb %*% scoef * rate
  }

  if (any(ex_up)) {
    ex_up_rate <- if (length(rate) == 1) rate else rate[ex_up]
    ex_up_scoef <- if (nrow(scoef) == 1) scoef else scoef[ex_up, , drop = FALSE]
    h_up <- hmspline(xorig[ex_up], basis = basis, scoef = ex_up_scoef, rate = ex_up_rate)
    out[ex_up] <- out[ex_up] + h_up * ex_time_up
  }

  if (any(ex_lo)) {
    ex_lo_rate <- if (length(rate) == 1) rate else rate[ex_lo]
    ex_lo_scoef <- if (nrow(scoef) == 1) scoef else scoef[ex_lo, , drop = FALSE]
    h_lo <- hmspline(xorig[ex_lo], basis = basis, scoef = ex_lo_scoef, rate = ex_lo_rate)
    out[ex_lo] <- h_lo * ex_time_lo
  }

  # Return 0 for x < 0
  lt0 <- x < 0
  out[lt0] <- 0

  if (log) out <- log(out)

  return(drop(out))
}

#' @rdname mspline
#' @export
rmst_mspline <- function(t, basis, scoef, rate, start = 0) {
  if (attr(basis, "degree") == 0) {
    # Piecewise exponential has explicit formula (Royston and Parmar 2013)
    nr <- max(length(t), if (is.matrix(scoef)) nrow(scoef) else 1, length(rate))

    knots <- c(attr(basis, "Boundary.knots")[1], attr(basis, "knots"))
    knots <- matrix(knots, nrow = nr, ncol = length(knots), byrow = TRUE)

    h <- apply(knots, 2, hmspline, basis = basis, scoef = scoef, rate = rate)

    delta <- t(apply(pmax(pmin(cbind(knots, Inf), t) - start, 0), 1, diff))

    hd <- h * delta

    H <- t(apply(cbind(0, hd[, -ncol(knots), drop = FALSE]), 1, cumsum))

    rowSums(exp(-H) / h * (1 - exp(-hd)))

  } else {
    # General M-splines require numerical integration
    require_pkg("flexsurv")
    flexsurv::rmst_generic(pmspline, t, start = start,
                           basis = basis, scoef = scoef, rate = rate,
                           scalarargs = "basis", matargs = "scoef")
  }
}

# Don't export mean_mspline - this is a bad idea in general
#' @noRd
mean_mspline <- function(basis, scoef, rate, ...) {
  rmst_mspline(t = Inf, basis, scoef, rate)
}

# Check for mspline/ispline objects
#' @param x Object to check is MSpline class
#' @return logical
#' @noRd
is_mspline <- function(x) {
  inherits(x, "MSpline")
}

# Create a (logit scale) coefficient vector corresponding to a constant hazard
#' @param basis A M-spline basis created using splines2::mSpline()
#' @return Coefficient vector
#' @noRd
mspline_constant_hazard <- function(basis) {
  if (!is_mspline(basis)) abort("`basis` must be an M-spline basis created using splines2::mSpline()")

  df <- ncol(basis)
  ord <- attr(basis, "degree") + 1
  iknots <- attr(basis, "knots")
  bknots <- attr(basis, "Boundary.knots")

  # Using approach of Jackson arXiv:2306.03957
  knots <- c(rep(bknots[1], ord), iknots, rep(bknots[2], ord))
  coefs <- (knots[(1:df) + ord] - knots[1:df]) / (ord * (diff(bknots)))

  # inverse softmax transform
  inv_softmax(coefs)
}

#' Create a vector of (normalised) weights for a RW(1) prior on an M-spline with
#' unequally spaced knots
#' @param basis M-spline basis created using splines2::mSpline()
#' @return Vector of normalised weights
#' @noRd
rw1_prior_weights <- function(basis) {
  nscoef <- ncol(basis)
  ord <- attr(basis, "degree") + 1
  iknots <- attr(basis, "knots")
  bknots <- attr(basis, "Boundary.knots")
  knots <- c(rep(bknots[1], ord), iknots, rep(bknots[2], ord))
  if (ord == 1) {
    wts <- (knots[2:nscoef] - knots[1:(nscoef - 1)]) / (knots[nscoef] - bknots[1])
  } else {
    wts <- (knots[(ord + 1):(nscoef + ord - 1)] - knots[2:nscoef]) / ((ord - 1) * (bknots[2] - bknots[1]))
  }
  return(sqrt(wts))
}

#' Softmax transform
#'
#' The softmax transform is a multivariate generalisation of the logit
#' transform. `softmax()` maps a vector of \eqn{K-1} values on the real line to a
#' \eqn{K}-simplex (i.e. values between 0 and 1, that sum to 1). `inv_softmax()`
#' provides the inverse transform, mapping a \eqn{K}-simplex vector to a vector of
#' \eqn{K-1} real values.
#'
#' @param x \eqn{K-1} vector of reals
#' @return `softmax()` returns a vector of length \eqn{K} that is a simplex.
#'   `inv_softmax()` returns a vector of reals of length \eqn{K-1}.
#' @export
#'
#' @examples
#' x <- c(-1, 3, -0.5, 2)
#' (p <- softmax(x))
#' sum(p)
#' inv_softmax(p)
#'
softmax <- function(x) {
  x0 <- c(0, x)
  exp(x0 - logsumexp(x0))
}

logsumexp <- function(x) {
  maxx <- max(x)
  max(x) + log(sum(exp(x - maxx)))
}

#' inverse softmax transform
#' @param p \eqn{K} vector simplex
#' @export
#' @rdname softmax
inv_softmax <- function(p) {
  log(p[-1]) - log(p[1])
}


#' Knot locations for M-spline baseline hazard models
#'
#' Several different algorithms are provided to calculate knot locations for
#' M-spline baseline hazard models. This function is called internally within
#' the [nma()] function, but may be called directly by the user for more
#' control.
#'
#' @param network A network object, containing survival outcomes
#' @param n_knots Non-negative integer giving the number of internal knots
#'   (default `7`)
# #' @param degree Non-negative integer giving the degree of the M-spline
# #'   polynomial (default `3`)
#' @param type String specifying the knot location algorithm to use (see
#'   details). The default used by [nma()] is `"quantile"`, except when a
#'   regression model is specified (using `aux_regression`) in which case the
#'   default is `"quantile_common"`.
#'
#' @details The `type` argument can be used to choose between different
#'   algorithms for placing the knots:
#'
#' \describe{
#'   \item{`"quantile"`}{Creates separate knot locations for each study,
#'   internal knots are placed at evenly-spaced quantiles of the observed event
#'   times within each study.}
#'   \item{`"quantile_lumped"`}{Creates a common set of knots for all studies,
#'   calculated as evenly-spaced quantiles of the observed event times from all
#'   studies lumped together.}
#'   \item{`"quantile_common"`}{Creates a common set of knots for all studies,
#'   taking quantiles of the quantiles of the observed event times within each
#'   study. This often seems to result in a more even knot spacing than
#'   `"quantile_lumped"`, particularly when follow-up is uneven across studies,
#'   and may handle differing behaviour in the baseline hazard across studies
#'   better than `"quantile_longest"`.}
#'   \item{`"quantile_longest"`}{Creates a common set of knots for all studies,
#'   using evenly-spaced quantiles of the observed event times in the longest
#'   study.}
#'   \item{`"equal"`}{Creates separate knot locations for each study, at
#'   evenly-spaced times between the boundary knots in each study.}
#'   \item{`"equal_common"`}{Creates a common set of knots for all studies, at
#'   evenly-spaced times between the earliest entry time and last
#'   event/censoring time in the network.}
#' }
#'
#' Boundary knots are calculated as follows:
#'
#' * For separate knot locations in each study, boundary knots are placed at the
#' earliest entry time and last event/censoring time in each study.
#' * For a common set of knots across all studies, boundary knots are placed at
#' the earliest entry time and last event/censoring time across all studies.
#'
#' Models with regression on the spline coefficients (i.e. with `aux_regression`
#' specified) require a common set of knots across all studies.
#'
#' Provided that a sufficient number of knots are used, model fit should be
#' largely unaffected by the knot locations. However, sampling difficulties can
#' sometimes occur if knot placement is poor, for example if a knot is placed
#' just before the last follow-up time in a study.
#'
#' @return A named list of vectors giving the knot locations in each study.
#' @seealso [knots.stan_nma()] for obtaining the knots from a fitted model
#' object.
#' @export
#'
#' @template ex_ndmm_network
#' @examples
#'
#' # The default knot locations
#' make_knots(ndmm_net, type = "quantile")
#'
#' # Increasing the number of knots
#' make_knots(ndmm_net, n_knots = 10)
#'
#' # Comparing alternative knot positioning algorithms
#' # Visualise these with a quick function
#' plot_knots <- function(network, knots) {
#'   ggplot2::ggplot() +
#'     geom_km(network) +
#'     ggplot2::geom_vline(ggplot2::aes(xintercept = .data$knot),
#'                         data = tidyr::pivot_longer(as.data.frame(knots), cols = dplyr::everything(),
#'                                                    names_to = "Study", values_to = "knot"),
#'                         linetype = 2, colour = "grey60") +
#'     ggplot2::facet_wrap(~Study) +
#'     theme_multinma()
#' }
#'
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile"))
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_common"))
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_lumped"))
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_longest"))
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "equal"))
#' plot_knots(ndmm_net, make_knots(ndmm_net, type = "equal_common"))
#'
make_knots <- function(network,
                       n_knots = 7,
                       type = c("quantile",
                                "quantile_common",
                                "quantile_lumped",
                                "quantile_longest",
                                "equal",
                                "equal_common")) {

  # Argument checks
  if (!inherits(network, "nma_data"))
    abort("`network` must be an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")

  if (!(identical(network$outcome$ipd, "survival") || identical(network$outcome$agd_arm, "survival")))
    abort("`network` does not contain survival outcomes.")

  if (!rlang::is_scalar_integerish(n_knots, finite = TRUE) || n_knots < 1)
    abort("`n_knots` must be a positive integer giving the number of internal knots.")

  type <- rlang::arg_match(type)

  # Get survival data
  survdat <- dplyr::bind_rows(
    if (has_ipd(network)) dplyr::select(network$ipd, ".study", ".trt", ".Surv") else NULL,
    if (has_agd_arm(network)) dplyr::select(network$agd_arm, ".study", ".trt", ".Surv") %>% tidyr::unnest(cols = ".Surv") else NULL
  )

  survdat <- dplyr::mutate(survdat,
                           !!! get_Surv_data(survdat$.Surv),
                           observed = .data$status == 1,
                           .study = forcats::fct_drop(.data$.study))

  observed_survdat <- dplyr::filter(survdat, .data$observed)

  studies <- levels(survdat$.study)
  n_studies <- length(studies)

  # Calculate knots
  if (type == "quantile") {

    # Boundary knots
    b_knots <- by(survdat, survdat$.study, function(x) c(min(x$delay_time), max(x$time)),
                  simplify = FALSE)

    # Internal knots
    i_knots <- by(observed_survdat, observed_survdat$.study,
                  function(x) quantile(x$time, probs = seq(1, n_knots) / (n_knots + 1)),
                  simplify = FALSE)

  } else if (type == "quantile_common") {

    # Boundary knots
    b_knots <- c(min(survdat$delay_time), max(survdat$time))
    b_knots <- rep_len(list(b_knots), n_studies)
    names(b_knots) <- studies

    # Internal knots
    # Take quantiles of knots and boundary knots in individual studies
    temp_b_knots <- by(survdat, survdat$.study, function(x) c(min(x$delay_time), max(x$time)),
                       simplify = FALSE)

    temp_i_knots <- by(observed_survdat, observed_survdat$.study,
                       function(x) quantile(x$time, probs = seq(1, n_knots) / (n_knots + 1)),
                       simplify = FALSE)

    i_knots <- quantile(c(unlist(temp_i_knots), unlist(temp_b_knots)), probs = seq(0, 1, length.out = n_knots+2))[2:(n_knots+1)]
    i_knots <- rep_len(list(i_knots), n_studies)
    names(i_knots) <- studies

  } else if (type == "quantile_lumped") {

    # Boundary knots
    b_knots <- c(min(survdat$delay_time), max(survdat$time))
    b_knots <- rep_len(list(b_knots), n_studies)
    names(b_knots) <- studies

    # Internal knots
    i_knots <- quantile(observed_survdat$time, probs = seq(1, n_knots) / (n_knots + 1))
    i_knots <- rep_len(list(i_knots), n_studies)
    names(i_knots) <- studies

  } else if (type == "quantile_longest") {

    # Boundary knots
    b_knots <- c(min(survdat$delay_time), max(survdat$time))
    b_knots <- rep_len(list(b_knots), n_studies)
    names(b_knots) <- studies

    # Find study with longest follow-up
    fu <- by(survdat, survdat$.study, function(x) max(x$time) - min(x$delay_time), simplify = TRUE)
    max_fu <- names(which.max(fu))

    # Internal knots
    i_knots <- quantile(dplyr::filter(observed_survdat, .data$.study == max_fu)$time, probs = seq(1, n_knots) / (n_knots + 1))
    i_knots <- rep_len(list(i_knots), n_studies)
    names(i_knots) <- studies

  } else if (type == "equal") {

    # Boundary knots
    b_knots <- by(survdat, survdat$.study, function(x) c(min(x$delay_time), max(x$time)),
                  simplify = FALSE)

    # Internal knots
    i_knots <- by(survdat, survdat$.study,
                  function(x) seq(from = min(x$delay_time), to = max(x$time), length.out = n_knots+2)[2:(n_knots+1)],
                  simplify = FALSE)

  } else if (type == "equal_common") {

    # Boundary knots
    b_knots <- c(min(survdat$delay_time), max(survdat$time))
    b_knots <- rep_len(list(b_knots), n_studies)
    names(b_knots) <- studies

    # Internal knots
    i_knots <- seq(from = min(survdat$delay_time), to = max(survdat$time), length.out = n_knots+2)[2:(n_knots+1)]
    i_knots <- rep_len(list(i_knots), n_studies)
    names(i_knots) <- studies
  }

  # Combine boundary and internal knots
  out <- purrr::map2(b_knots, i_knots, ~ unname(c(.x[1], .y, .x[2])))

  return(out)
}


#' Knot locations for a fitted model
#'
#' Obtain the knot locations from a fitted M-spline or piecewise exponential
#' model.
#'
#' @param Fn A fitted [stan_nma] object
#' @param type String, indicating whether to return all knots (`"all"`, the
#'   default), or only the internal knots (`"internal"`) or boundary knots
#'   (`"boundary"`).
#' @param ... Other arguments, passed on to [splines2::knots()]
#'
#' @returns A list of vectors of knot locations, for each study in the network.
#' @seealso [make_knots()] for constructing knots for a network object.
#' @exportS3Method stats::knots
knots.stan_nma <- function(Fn, type = c("all", "internal", "boundary"), ...) {
  type <- rlang::arg_match(type)
  if (!Fn$likelihood %in% c("mspline", "pexp"))
    abort("No knots present. Not an M-spline or piecewise exponential model.")

  basis <- Fn$basis

  if (type == "all") {
    .knots <- function(x, ...) {
      ik <- stats::knots(x, type = "internal", ...)
      bk <- stats::knots(x, type = "boundary", ...)
      c(bk[1], ik, bk[2])
    }
    return(purrr::map(basis, .knots, ...))
  } else {
    return(purrr::map(basis, stats::knots, type = type, ...))
  }
}

#' Add numerical integration points to aggregate data
#'
#' The `add_integration()` generic creates Quasi-Monte Carlo numerical
#' integration points using a Gaussian copula and Sobol' sequences, as described
#' in \insertCite{methods_paper;textual}{multinma}. Methods are available for
#' networks stored in `nma_data` objects, and for data frames. The function
#' `unnest_integration()` unnests integration points stored in a data frame, to
#' aid plotting or other exploration.
#'
#' @param x An `nma_data` object, as created by the `set_*()` functions or
#'   `combine_network()`, or data frame
#' @param ... Distributions for covariates, see "Details"
#' @param cor Correlation matrix to use for generating the integration points.
#'   By default, this takes a weighted correlation matrix from all IPD studies.
#'   Rows and columns should match the order of covariates specified in `...`.
#' @param cor_adjust Adjustment to apply to the correlation matrix given by
#'   `cor` (or computed from the IPD if `cor = NULL`) to obtain the Gaussian
#'   copula correlations, either `"spearman"`, `"pearson"`, or `"none"`, see
#'   "Details". The default when `cor = NULL` is `"spearman"`, otherwise the
#'   default is `"pearson"`.
#' @param n_int Number of integration points to generate, default 64. Powers of
#'   2 are recommended, as these are expected to be particularly efficient for
#'   QMC integration.
#' @param int_args A named list of arguments to pass to
#'   \code{\link[randtoolbox:quasiRNG]{sobol()}}
#'
#' @return For the `nma_data` method, an object of class [nma_data]. For the
#'   `data.frame` method, the input data frame is returned (as a `tibble`) with
#'   an added column for each covariate (prefixed with ".int_"), containing the
#'   numerical integration points nested as length-`n_int` vectors within each
#'   row. For `unnest_integration()`, a data frame with integration points
#'   unnested.
#' @export
#'
#' @details The arguments passed to `...` specify distributions for the
#'   covariates. Argument names specify the name of the covariate, which should
#'   match a covariate name in the IPD (if IPD are present). The required
#'   marginal distribution is then specified using the function [distr()].
#'
#'   The argument `cor_adjust` specifies how the correlation matrix given by
#'   `cor` (or computed from the IPD if `cor = NULL`) is adjusted to obtain the
#'   correlation matrix for the Gaussian copula, using the formulae in
#'   \insertCite{Xiao2018;textual}{multinma}.
#'
#'   * `cor_adjust = "spearman"` should be used when the correlations `cor` have
#'   been computed using Spearman's rank correlation. Correlations between
#'   continuous covariates will be reproduced exactly by the integration points.
#'   Correlations between discrete covariates will be reproduced approximately.
#'   This is the default when `cor = NULL` and correlations are calculated from
#'   the IPD studies.
#'
#'   * `cor_adjust = "pearson"` should be used when the correlations `cor` have
#'   been computed using Pearson's product-moment correlation. Correlations between
#'   Normal covariates will be reproduced exactly by the integration points, all
#'   others will be reproduced approximately. Correlations between discrete
#'   covariates will be reproduced approximately (and identically to `cor_adjust
#'   = "spearman"`). This is the default when `cor` is provided by the user,
#'   since [cor()] defaults to `method = "pearson"` and Pearson correlations are
#'   most likely reported in published data. However, we recommend providing
#'   Spearman correlations (e.g. from `cor(., method = "spearman")`) and using
#'   `cor_adjust = "spearman"` where possible.
#'
#'   * `cor_adjust = "none"` allows the user to specify the correlation matrix
#'   for the Gaussian copula directly; no adjustment is applied.
#'
#'   * `cor_adjust = "legacy"` is also available, which reproduces exactly the
#'   behaviour from version 0.3.0 and earlier. This is similar to `cor_adjust =
#'   "none"`, but unadjusted Spearman correlations are used if `cor = NULL`.
#'
#'   When adding integration points to a network object the correlation matrix
#'   used is stored in `$int_cor`, and the copula correlation matrix and
#'   adjustment used are stored as attributes of `$int_cor`. If this correlation
#'   matrix is passed again to `add_integration()` (e.g. to reuse the
#'   correlations for an external target population) this will be detected, and
#'   the correct setting for `cor_adjust` will automatically be applied.
#'
#' @references \insertAllCited{}
#'
#' @examples ## Plaque psoriasis ML-NMR - network setup and adding integration points
#' @template ex_plaque_psoriasis_network
#' @template ex_plaque_psoriasis_integration
#' @examples
#'
#' ## Adding integration points to a data frame, e.g. for prediction
#' # Define a data frame of covariate summaries
#' new_agd_int <- data.frame(
#'   bsa_mean = 0.6,
#'   bsa_sd = 0.3,
#'   prevsys = 0.1,
#'   psa = 0.2,
#'   weight_mean = 10,
#'   weight_sd = 1,
#'   durnpso_mean = 3,
#'   durnpso_sd = 1)
#'
#' # Adding integration points, using the weighted average correlation matrix
#' # computed for the plaque psoriasis network
#' new_agd_int <- add_integration(new_agd_int,
#'   durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
#'   prevsys = distr(qbern, prob = prevsys),
#'   bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
#'   weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
#'   psa = distr(qbern, prob = psa),
#'   cor = pso_net$int_cor,
#'   n_int = 64)
#'
#' # Here, since we reused the correlation matrix pso_net$int_cor from the
#' # network, the correct setting of cor_adjust = "spearman" is automatically
#' # applied
#'
#' new_agd_int
#'
add_integration <- function(x, ...) {
  UseMethod("add_integration")
}

#' @export
#' @rdname add_integration
add_integration.default <- function(x, ...) {
  abort(glue::glue("No add_integration method defined for class '{class(x)}'."))
}

#' @export
#' @rdname add_integration
add_integration.data.frame <- function(x, ...,
                                       cor = NULL, cor_adjust = NULL,
                                       n_int = 64L, int_args = list()) {

  x <- tibble::as_tibble(x)

  # Check n_int
  if (!rlang::is_scalar_integerish(n_int) || n_int <= 0) {
    abort("`n_int` should be a positive integer.")
  }

  # Check int_args
  if (!is.list(int_args) || (!rlang::is_empty(int_args) && !rlang::is_named(int_args))) {
    abort("`int_args` should be a named list.")
  }

  # Check covariate arguments
  ds <- list(...)

  if (length(ds) == 0) {
    abort("No covariate distributions specified. Covariate distributions should be specified as named arguments using the function `distr`.")
  }

  if (any(purrr::map_lgl(ds, ~!inherits(., "distr"))) || !rlang::is_named(ds)) {
    abort("Covariate distributions should be specified as named arguments using the function `distr`.")
  }

  x_names <- names(ds)  # Covariate names
  nx <- length(ds)      # Number of covariates

  if (nx > 1) {
    # Check cor
    if (!is.null(cor)) {
      tryCatch(cor <- as.matrix(cor),
               error = function(e) abort("`cor` should be a correlation matrix or NULL"))

      if (!is.numeric(cor) ||
          !isSymmetric(cor) ||
          !isTRUE(all.equal(diag(cor), rep(1, nrow(cor)), check.names = FALSE)) ||
          !all(eigen(cor, symmetric = TRUE)$values > 0))
        abort("`cor` should be a correlation matrix or NULL")

      if (ncol(cor) != nx)
        abort("Dimensions of correlation matrix `cor` and number of covariates specified in `...` do not match.")
    } else {
      abort("Specify a correlation matrix using the `cor` argument.")
    }

    # Check cor_adjust
    # Use copula cor if present
    if (!is.null(attr(cor, "copula_cor", exact = TRUE)))  {
      cor_adjust <- attr(cor, "cor_adjust", exact = TRUE)
    } else if (!is.null(cor_adjust)) {
      cor_adjust <- rlang::arg_match(cor_adjust, values = c("spearman", "pearson", "none", "legacy"))
    } else {
      # Otherwise assume pearson cor (default for stats::cor)
      cor_adjust <- "pearson"
    }
  }

  # Generate Sobol points
  u <- do.call(randtoolbox::sobol, purrr::list_modify(list(n = n_int, dim = nx), !!! int_args))

  # Correlate Sobol points with Gaussian copula
  if (nx > 1) {
    # Use copula cor if present
    if (!is.null(attr(cor, "copula_cor", exact = TRUE)))  {
      copula_cor <- attr(cor, "copula_cor", exact = TRUE)
    } else if (cor_adjust %in% c("none", "legacy")) {
      copula_cor <- cor
    } else {
      # Apply adjustment to sample correlations to obtain copula correlations

      dtypes <- get_distribution_type(..., data = head(x))

      if (cor_adjust == "spearman") {
        copula_cor <- cor_adjust_spearman(cor, types = dtypes)
      } else if (cor_adjust == "pearson") {
        copula_cor <- cor_adjust_pearson(cor, types = dtypes)
      }

      # Check that adjustments still give a correlation matrix
      if (!all(eigen(copula_cor, symmetric = TRUE)$values > 0)) {
        warn("Adjusted correlation matrix not positive definite; using Matrix::nearPD().")
        copula_cor <- as.matrix(Matrix::nearPD(copula_cor, corr = TRUE)$mat)
      }
    }

    cop <- copula::normalCopula(copula::P2p(copula_cor), dim = nx, dispstr = "un")
    u_cor <- copula::cCopula(u, copula = cop, inverse = TRUE)
    # columns to list
    u_cor_l <- purrr::array_branch(u_cor, 2)
  } else {
    u_cor <- u
    u_cor_l <- list(u_cor)
  }

  # Use inverse CDFs to get integration points for specified marginals
  out <- x
  x_int_names <- paste0(".int_", x_names)

  # If integration points already present, remove and warn
  int_cols <- stringr::str_detect(colnames(x), "^\\.int_")
  if (any(int_cols))
    warn("Replacing integration points already present in data frame.", "int_col_present")

  # Bind in generated integration points
  out <-
    dplyr::bind_cols(
      x[, !int_cols],
      purrr::pmap_dfc(list(x_int_names, ds, u_cor_l),
                      ~ dplyr::rowwise(x) %>%
                        dplyr::transmute(!! ..1 := list(rlang::eval_tidy(rlang::call2(..2$qfun, p = ..3, !!! ..2$args)))))
    )

  # Check valid values produced
  invalid_rows <- out %>%
    dplyr::mutate_at(x_int_names,
                     .funs = ~purrr::map_lgl(.,
                                             ~any(is.na(.) | is.infinite(.) |
                                                    is.null(.) | is.nan(.)))
    ) %>%
    dplyr::filter_at(x_int_names, dplyr::any_vars(.))

  if (nrow(invalid_rows) > 0)
    abort("Invalid integration points were generated (either NA, NaN, Inf, or NULL).",
          "invalid_int_generated",
          invalid_rows = invalid_rows)

  class(out) <- c("integration_tbl", class(out))
  if (nx > 1) {
    attr(out, "cor_adjust") <- cor_adjust
    attr(out, "copula_cor") <- copula_cor
  }
  return(out)
}

#' @export
#' @rdname add_integration
add_integration.nma_data <- function(x, ...,
                                     cor = NULL, cor_adjust = NULL,
                                     n_int = 64L, int_args = list()) {

  network <- x

  # Checks for n_int and int_args performed in data.frame method

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("Expecting an `nma_data` object, as created by the `set_*` or `combine_network` functions.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  if (!has_agd_arm(network) && !has_agd_contrast(network)) {
    abort("No aggregate data found in network.")
  }

  # Covariate arguments
  ds <- list(...)

  if (length(ds) == 0) {
    abort("No covariate distributions specified. Covariate distributions should be specified as named arguments using the function `distr`.")
  }

  if (any(purrr::map_lgl(ds, ~!inherits(., "distr"))) || !rlang::is_named(ds)) {
    abort("Covariate distributions should be specified as named arguments using the function `distr`.")
  }

  x_names <- names(ds)
  nx <- length(ds)

  # Check cor
  if (!is.null(cor)) {
    tryCatch(cor <- as.matrix(cor),
             error = function(e) abort("`cor` should be a correlation matrix or NULL"))

    if (!is.numeric(cor) ||
        !isSymmetric(cor) ||
        !isTRUE(all.equal(diag(cor), rep(1, nrow(cor)), check.names = FALSE)) ||
        !all(eigen(cor, symmetric = TRUE)$values > 0)) {
      abort("`cor` should be a correlation matrix or NULL")
    }
  } else if (nx > 1) {
    if (!has_ipd(network)) abort("Specify a correlation matrix using the `cor` argument, or provide IPD studies in the network.")
  }

  # Check cor_adjust

  # If user provided cor
  if (!is.null(cor)) {
    if (!is.null(attr(cor, "copula_cor", exact = TRUE))) {
      # Use copula cor if present
      cor_adjust <- attr(cor, "cor_adjust", exact = TRUE)
    } else if (!is.null(cor_adjust)) {
      # Check cor_adjust when given
      cor_adjust <- rlang::arg_match(cor_adjust, values = c("spearman", "pearson", "none", "legacy"))
    } else {
      # Otherwise assume pearson cor (default for stats::cor)
      cor_adjust <- "pearson"
    }
  } else {
    if (!is.null(cor_adjust)) {
      # Check cor_adjust when given
      cor_adjust <- rlang::arg_match(cor_adjust, values = c("spearman", "pearson", "none", "legacy"))
    } else {
      # No cor given, calculate from IPD with spearman
      cor_adjust <- "spearman"
    }
  }

  # If IPD is provided, check that covariate names match
  if (has_ipd(network) && any(! x_names %in% colnames(network$ipd))) {
    abort(paste0("Covariate name(s) not found in IPD: ",
                 paste0(setdiff(x_names, colnames(network$ipd)), collapse = ", ")))
  }

  # Use weighted average correlation matrix from IPD, if cor = NULL
  if (is.null(cor) && nx > 1) {
    inform("Using weighted average correlation matrix computed from IPD studies.")
    if (cor_adjust == "none") abort('Cannot specify cor_adjust = "none" when calculating correlations from IPD.')


    # Check for any missing covariates
    if (!all(complete.cases(dplyr::select(network$ipd, dplyr::all_of(x_names))))) {
      warn("Missing values found for some covariates in IPD. Calculating correlations using complete cases.")
    }

    # Weighted z-transform average
    ipd_cors <-
      network$ipd %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::group_modify(~tibble::tibble(
        w = nrow(.) - 3,
        r = list(cor(dplyr::select(., dplyr::all_of(x_names)),
                     method = if (cor_adjust == "legacy") "spearman" else cor_adjust,
                     use = "complete.obs"))
        )) %>%
      dplyr::mutate(z = purrr::map2(.data$w, .data$r, ~.x * log((1 + .y) / (1 - .y)) / 2))

    w_z <- Reduce(`+`, ipd_cors$z) / sum(ipd_cors$w)
    ipd_cor <- (exp(2 * w_z) - 1) / (exp(2 * w_z) + 1)

    diag(ipd_cor) <- 1

    cor <- ipd_cor
  }

  out <- network

  # Establish handlers for more specific errors / warnings when possible
  int_col_present <- function(wrn) {
    warn("Replacing integration points already present in network.")
    rlang::cnd_muffle(wrn)
  }

  invalid_int_generated <- function(err) {
    invalid_rows <- err$invalid_rows
    abort(
      glue::glue("Invalid integration points were generated (either NA, NaN, Inf, or NULL).\n",
                 "Check the input parameters for the following (study, treatment):\n",
                 glue::glue_collapse(glue::glue(" {invalid_rows$.study}, {invalid_rows$.trt}"),
                                     sep = "\n"))
    )
  }

  if (has_agd_arm(network)) {
    out$agd_arm <- withCallingHandlers(
      add_integration.data.frame(network$agd_arm, ...,
                                 cor = cor, cor_adjust = cor_adjust, n_int = n_int, int_args = int_args),
      int_col_present = int_col_present,
      invalid_int_generated = invalid_int_generated)

    copula_cor <- attr(out$agd_arm, "copula_cor")
  }

  if (has_agd_contrast(network)) {
    out$agd_contrast <- withCallingHandlers(
      add_integration.data.frame(network$agd_contrast, ...,
                                 cor = cor, cor_adjust = cor_adjust, n_int = n_int, int_args = int_args),
      int_col_present = int_col_present,
      invalid_int_generated = invalid_int_generated)

    copula_cor <- attr(out$agd_contrast, "copula_cor")
  }

  # Set as mlnmr_data class
  out$n_int <- n_int
  out$int_names <- x_names

  if (nx == 1) cor <- matrix(1)
  colnames(cor) <- x_names
  rownames(cor) <- x_names
  out$int_cor <- cor
  attr(out$int_cor, "cor_adjust") <- cor_adjust
  attr(out$int_cor, "copula_cor") <- copula_cor

  class(out) <- c("mlnmr_data", "nma_data")
  return(out)
}


#' @param data Data frame with nested integration points, stored in list
#'   columns as `.int_<variable name>`
#'
#' @export
#' @rdname add_integration
unnest_integration <- function(data) {
  if (!inherits(data, "data.frame")) abort("Expecting a data frame.")

  # Get integration variable names
  data_names <- colnames(data)
  x_int_names <- stringr::str_subset(data_names, "^\\.int_")
  x_names <- stringr::str_remove(x_int_names, "^\\.int_")

  if (length(x_int_names) == 0)
    abort("No integration points present in data frame.")

  name_conflicts <- rlang::has_name(data, x_names)
  if (any(name_conflicts)) {
    warn(glue::glue("Columns will be overwritten when unnesting integration points: ",
                    glue::glue_collapse(x_names[name_conflicts], sep = ", ")),
         "unnest_name_conflict")
  }

  out <- data %>%
    dplyr::select(-dplyr::one_of(x_names[name_conflicts])) %>%
    dplyr::rename_at(x_int_names, ~stringr::str_remove(., "^\\.int_")) %>%
    tidyr::unnest(dplyr::all_of(x_names))

  return(out)
}

#' Internal version of unnest_integration, doesn't throw warnings when
#' overwriting columns
#' @noRd
.unnest_integration <- function(data) {
  ignore <- function(wrn) rlang::cnd_muffle(wrn)
  out <- withCallingHandlers(unnest_integration(data),
                              unnest_name_conflict = ignore)
  return(out)
}

#' Specify a general marginal distribution
#'
#' `distr()` is used within the function [add_integration()] to specify marginal
#' distributions for the covariates, via a corresponding inverse CDF. It is also
#' used in [predict.stan_nma()] to specify a distribution for the baseline
#' response (intercept) when predicting absolute outcomes.
#'
#' @param qfun an inverse CDF, either as a function name or a string
#' @param ... parameters of the distribution as arguments to `qfun`, these will
#'   be quoted and evaluated later in the context of the aggregate data sources
#'
#' @return An object of class [distr].
#' @export
#'
#' @details The function `qfun` should have a formal argument called `p`. This
#'   restriction serves as a crude check for inverse CDFs (e.g. an error will be
#'   given if `dnorm` is used instead of `qnorm`). If a user-written CDF is
#'   supplied, it must have an argument `p` which takes a vector of
#'   probabilities.
#'
#' @seealso [add_integration()] where `distr()` is used to specify marginal
#'   distributions for covariates to integrate over, and [predict.stan_nma()]
#'   where `distr()` is used to specify a distribution on the baseline response.
#' @examples
#' ## Specifying marginal distributions for integration
#'
#' df <- data.frame(x1_mean = 2, x1_sd = 0.5, x2 = 0.8)
#'
#' # Distribution parameters are evaluated in the context of the data frame
#' add_integration(df,
#'                 x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
#'                 x2 = distr(qbern, prob = x2),
#'                 cor = diag(2))
#'
distr <- function(qfun, ...) {
  qfun_quo <- rlang::enquo(qfun)
  d <- list(qfun = match.fun(qfun),
            args = rlang::enquos(...),
            qfun_name = tryCatch(rlang::as_name(qfun_quo), error = function(e) "user_function"))
  if (! "p" %in% rlang::fn_fmls_names(qfun)) {
    abort("`qfun` should be an inverse CDF function (for example `qnorm`, `qgamma`, `qbinom`, ...) but does not appear to be (no formal argument `p`)")
  }
  class(d) <- "distr"
  return(d)
}

#' The Bernoulli Distribution
#'
#' The density function `dbern()`, distribution function `pbern()`, and quantile
#' function `qbern()` for a Bernoulli distribution, with success probability
#' `prob`. These are equivalent to `dbinom(p, 1, prob)`, `pbinom(p, 1, prob)`
#' and `qbinom(p, 1, prob)`.
#'
#' @param x,q vector of quantiles
#' @param p vector of probabilities
#' @param prob probability of success
#' @param lower.tail,log.p,log see [stats::Binomial]
#'
#' @return Numeric vector of length equal to the maximum of the lengths of the
#'   input arguments.
#'
#' @export
#' @rdname Bernoulli
#' @aliases qbern
qbern <- function(p, prob, lower.tail = TRUE, log.p = FALSE) {
  return(qbinom(p, size = 1, prob = prob, lower.tail = lower.tail, log.p = log.p))
}

#' @export
#' @rdname Bernoulli
#' @aliases pbern
pbern <- function(q, prob, lower.tail = TRUE, log.p = FALSE) {
  return(pbinom(q, size = 1, prob = prob, lower.tail = lower.tail, log.p = log.p))
}

#' @export
#' @rdname Bernoulli
#' @aliases dbern
dbern <- function(x, prob, log = FALSE) {
  return(dbinom(x, size = 1, prob = prob, log = log))
}

#' The Gamma distribution
#'
#' We provide convenient extensions of the `[dpq]gamma` functions, which allow
#' the distribution to be specified in terms of its mean and standard deviation,
#' instead of shape and rate/scale.
#'
#' @param x,q vector of quantiles
#' @param p vector of probabilities
#' @param shape,rate,scale,log,lower.tail,log.p see [stats::GammaDist]
#' @param mean,sd mean and standard deviation, overriding `shape` and
#'   `rate` or `scale` if specified
#'
#' @return Numeric vector of length equal to the maximum of the lengths of the
#'   input arguments.
#'
#' @export
#' @rdname GammaDist
#' @aliases qgamma
qgamma <- function(p, shape, rate = 1, scale = 1/rate, lower.tail = TRUE,
                   log.p = FALSE, mean, sd) {
  if (!missing(mean) && !missing(sd))
    return(stats::qgamma(p, shape = (mean / sd)^2, rate = mean / sd^2,
                        lower.tail = lower.tail, log.p = log.p))
  else
    return(stats::qgamma(p, shape = shape, rate = rate,
                        lower.tail = lower.tail, log.p = log.p))
}

#' @export
#' @rdname GammaDist
#' @aliases dgamma
dgamma <- function(x, shape, rate = 1, scale = 1/rate, log = FALSE, mean, sd) {
  if (!missing(mean) && !missing(sd))
    return(stats::dgamma(x, shape = (mean / sd)^2, rate = mean / sd^2, log = log))
  else
    return(stats::dgamma(x, shape = shape, rate = rate, log = log))
}

#' @export
#' @rdname GammaDist
#' @aliases pgamma
pgamma <- function(q, shape, rate = 1, scale = 1/rate, lower.tail = TRUE,
                   log.p = FALSE, mean, sd) {
  if (!missing(mean) && !missing(sd))
    return(stats::pgamma(q, shape = (mean / sd)^2, rate = mean / sd^2,
                         lower.tail = lower.tail, log.p = log.p))
  else
    return(stats::pgamma(q, shape = shape, rate = rate,
                         lower.tail = lower.tail, log.p = log.p))
}


#' The logit Normal distribution
#'
#' Density, distribution function, and quantile function for the logit Normal
#' distribution. The location and scale parameters of the distribution are `mu`
#' and `sigma`, which are the mean and standard deviation on the logit scale.
#' For convenience, the distribution may also be specified in terms of its mean
#' and standard deviation, instead of its logit-mean and logit-sd.
#'
#' @param x,q vector of quantiles
#' @param p vector of probabilities
#' @param mu,sigma location and scale parameters, on the logit scale
#' @param ... additional arguments, passed to `[dpq]norm()`
#' @param mean,sd mean and standard deviation, overriding `mu` and `sigma` if
#'   specified
#'
#' @return Numeric vector of length equal to the maximum of the lengths of the
#'   input arguments.
#'
#' @export
#' @rdname logitNormal
#' @aliases dlogitnorm
#' @importFrom stats integrate dnorm
dlogitnorm <- function(x, mu = 0, sigma = 1, ..., mean, sd) {
  islog <- isTRUE(rlang::list2(...)$log)

  if (!missing(mean) && !missing(sd)) pars <- pars_logitnorm(mean, sd)
  else pars <- list(mu = mu, sigma = sigma)

  out <- dnorm(qlogis(x), mean = pars[["mu"]], sd = pars[["sigma"]], ...)
  if (islog) {
    out <- out - log(x) - log1p(-x)
  } else {
    out <- out / (x * (1 - x))
  }
  return(out)
}

#' @export
#' @rdname logitNormal
#' @aliases plogitnorm
plogitnorm <- function(q, mu = 0, sigma = 1, ..., mean, sd) {
  if (!missing(mean) && !missing(sd)) pars <- pars_logitnorm(mean, sd)
  else pars <- list(mu = mu, sigma = sigma)
  return(pnorm(qlogis(q), mean = pars[["mu"]], sd = pars[["sigma"]], ...))
}

#' @export
#' @rdname logitNormal
#' @aliases qlogitnorm
qlogitnorm <- function(p, mu = 0, sigma = 1, ..., mean, sd){
  if (!missing(mean) && !missing(sd)) pars <- pars_logitnorm(mean, sd)
  else pars <- list(mu = mu, sigma = sigma)
  return(plogis(qnorm(p, mean = pars[["mu"]], sd = pars[["sigma"]], ...)))
}


# Internal functions for *logitnorm()
.lndiff <- function(est, m, s){
  smean <- integrate(function(x) x * dlogitnorm(x, mu = est[1], sigma = est[2]), 0, 1)$value
  ssd <- sqrt(integrate(function(x) (x - smean)^2 * dlogitnorm(x, mu = est[1], sigma = est[2]), 0, 1)$value)
  return((smean - m)^2 + (ssd - s)^2)
}

.lnopt <- function(m, s) {
  opt <- optim(c(m, s), .lndiff, m = m, s = s)

  if (opt$convergence != 0) {
    warn("Optimisation did not converge, NAs produced.")
    return(c(mu = NA, sigma = NA))
  } else {
    return(c(mu = opt$par[1], sigma = opt$par[2]))
  }
}

# Estimate mu and sigma parameters of logit-normal from sample mean and sd
pars_logitnorm <- function(m, s) {
  if (length(m) != length(s) && length(m) > 1 && length(s) > 1) {
    abort("Parameters not same length.")
  }
  if (any(m > 1 | m < 0)) abort("Mean not in [0,1]. Have you rescaled?")

  return(as.data.frame(do.call(rbind, mapply(.lnopt, m, s, SIMPLIFY = FALSE))))

}


#' Get type of distribution
#'
#' @param ... distr() distributions
#' @param data List-like sample data for the distr() distribution parameters
#'
#' @return Named string vector with elements "continuous", "discrete" or "binary"
#'
#' @noRd
get_distribution_type <- function(..., data = list()) {
  ds <- list(...)
  dnames <- names(ds)

  out <- vector("character", length  = length(ds))
  names(out) <- dnames

  # List of known continuous distributions
  known_continuous <- c("qbeta", "qcauchy", "qchisq", "qexp", "qf", "qgamma",
                        "qlnorm", "qnorm", "qt", "qunif", "qweibull", "qlogitnorm")

  # List of known discrete distributions
  known_discrete <- c("qgeom", "qnbinom", "qpois")
  # Will handle qbinom and qhyper separately, since they have binary special cases

  # List of known binary distributions
  known_binary <- "qbern"

  for (i in 1:length(ds)) {
    di <- ds[[i]]
    # Known distributions
    if (di$qfun_name %in% known_continuous) {
      out[i] <- "continuous"
    } else if (di$qfun_name %in% known_discrete) {
      out[i] <- "discrete"
    } else if (di$qfun_name %in% known_binary) {
      out[i] <- "binary"

    # Handle discrete distributions with special binary cases
    } else if (di$qfun_name == "qbinom") {
      if (all(rlang::eval_tidy(di$args$size, data = data) == 1)) out[i] <- "binary"
      else out[i] <- "discrete"
    } else if (di$qfun_name == "qhyper") {
      if (all(rlang::eval_tidy(di$args$k, data = data) == 1)) out[i] <- "binary"
      else out[i] <- "discrete"

    # Otherwise run qfun on grid of values and check with rlang::is_integerish
    } else {
      ps <- 1:99 / 100
      support <- rlang::eval_tidy(rlang::call2(di$qfun, p = ps, !!! di$args),
                                  data = data)

      if (rlang::is_integerish(support)) {
        if (all(dplyr::near(support, 0) | dplyr::near(support, 1))) out[i] <- "binary"
        else out[i] <- "discrete"
      } else {
        out[i] <- "continuous"
      }
    }
  }

  return(out)
}

# Convert Spearman correlations to underlying Gaussian copula correlations
# using formulae in Xiao and Zhou (2018)
cor_adjust_spearman <- function(X, types) {
  bin <- types == "binary"
  cont <- !bin   # Treat discrete as continuous

  X[cont, cont] <- 2 * sin(pi * X[cont, cont] / 6)
  X[bin, bin] <- sin(pi * X[bin, bin] / 2)
  X[cont, bin] <- sqrt(2) * sin(pi * X[cont, bin] / (2 * sqrt(3)))
  X[bin, cont] <- sqrt(2) * sin(pi * X[bin, cont] / (2 * sqrt(3)))

  diag(X) <- 1

  return(X)
}

# Convert Pearson correlations to underlying Gaussian copula correlations
# using formulae in Xiao and Zhou (2018)
cor_adjust_pearson <- function(X, types) {
  bin <- types == "binary"
  cont <- !bin   # Treat discrete as continuous

  # No adjustment to cont-cont, assume that these are Normal
  X[bin, bin] <- sin(pi * X[bin, bin] / 2)
  # Use bin-cont formula assuming cont is Normal
  X[cont, bin] <- sqrt(pi/2) * X[cont, bin]
  X[bin, cont] <- sqrt(pi/2) * X[bin, cont]

  diag(X) <- 1

  return(X)
}

skip_on_cran()

test_that("qlogitnorm", {
  p <- runif(30000)

  mu <- c(0.1, 1, 3)
  sigma <- c(0.8, 0.5, 1)
  q1 <- qlogitnorm(p, mu = mu, sigma = sigma)
  expect_true(all(q1 <= 1 & q1 >= 0))
  expect_equal(plogitnorm(q1, mu = mu, sigma = sigma), p)

  m <- c(0.2, 0.5, 0.8)
  s <- c(0.1, 0.2, 0.05)
  q2 <- matrix(qlogitnorm(p, mean = m, sd = s), nrow = 3)
  expect_true(all(q2 <= 1 & q2 >= 0))
  expect_equal(c(plogitnorm(q2, mean = m, sd = s)), p)

  expect_equal(rowMeans(q2), m, tol = 0.01)
  expect_equal(apply(q2, 1, sd), s, tol = 0.01)
})

test_that("plogitnorm", {
  q <- runif(1000, 0.05, 0.95)

  mu <- c(0.1, 1, 3)
  sigma <- c(0.8, 0.5, 1)
  p1 <- plogitnorm(q, mu = mu, sigma = sigma)
  expect_true(all(p1 <= 1 & p1 >= 0))
  expect_equal(qlogitnorm(p1, mu = mu, sigma = sigma), q)

  m <- c(0.2, 0.5, 0.8)
  s <- c(0.1, 0.2, 0.05)
  p2 <- plogitnorm(q, mean = m, sd = s)
  expect_true(all(p2 <= 1 & p2 >= 0))
  expect_equal(qlogitnorm(p2, mean = m, sd = s), q)
})

library(multinma)
library(dplyr)
library(purrr)

test_that("expects nma_data object", {
  expect_error(add_integration("uh oh"), "No add_integration method defined")
})

test_that("error on empty network", {
  expect_error(add_integration(set_ipd(data.frame())), "Empty network")
})

test_that("error if no AgD", {
  expect_error(add_integration(set_ipd(smoking, studyn, trtc, y = r)), "No aggregate data")
})

# Make dummy covariate data for smoking network
ns_agd <- max(smoking$studyn)
smkdummy <-
  smoking %>%
  group_by(studyn) %>%
  mutate(x1_mean = rnorm(1), x1_sd = runif(1, 0.5, 2),
         x2 = rbeta(1, 5, 10),
         x3_mean = rnorm(1), x3_sd = runif(1, 0.5, 2),
         x4 = rbeta(1, 10, 5)) %>%
  ungroup()

smkdummy_contrast <- smkdummy %>%
  group_by(studyn) %>%
  mutate(arm = 1:n(),
         cc = if_else(any(r == 0), 0.5, 0),
         r = r + cc,
         n = n + cc,
         lor = if_else(arm == 1, NA_real_, log(r * (first(n) - first(r)) / ((n - r) * first(r)))),
         se = sqrt(if_else(arm == 1, 1 / r + 1 / (n - r), 1 / r + 1 / (n - r) + 1 / first(r) + 1 / (first(n) - first(r))))) %>%
  ungroup()

ns_ipd <- 2

xall_cor <- 0.25
n_i <- 200
cormat <- matrix(xall_cor, nrow = 4, ncol = 4)
diag(cormat) <- 1

cop <- copula::normalCopula(copula::P2p(cormat), dim = 4, dispstr = "un")
u <- matrix(runif(n_i * 4 * ns_ipd), ncol = 4)
u_cor <- as.data.frame(copula::cCopula(u, cop, inverse = TRUE))

ipddummy <-
  tibble(studyn = rep(ns_agd, n_i * ns_ipd) + rep(1:ns_ipd, each = n_i),
         trtn = sample(1:2, n_i * ns_ipd, TRUE)) %>%
  mutate(x1 = qnorm(u_cor[,1]),
         x2 = qbinom(u_cor[,2], 1, 0.6),
         x3 = qnorm(u_cor[,3], 1, 0.5),
         x4 = qbinom(u_cor[,4], 1, 0.4),
         r = rbinom(n(), 1, 0.2))

smknet_agd <- set_agd_arm(smkdummy, studyn, trtn, r = r, n = n)
smknet_ipd <- set_ipd(ipddummy, studyn, trtn, r = r)
smknet <- combine_network(smknet_agd, smknet_ipd)

test_that("n_int should be a positive integer", {
  m <- "should be a positive integer"

  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = "oh dear"), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 1.1), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = -5), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 0), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 1:2), m)

  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = "oh dear"), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 1.1), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = -5), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 0), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = 1:2), m)
})

test_that("int_args is named list", {
  m <- "should be a named list"

  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = "oh dear"), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = 1.1), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = NULL), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = list(a = 1, 2)), m)

  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = "oh dear"), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = 1.1), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = NULL), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), int_args = list(a = 1, 2)), m)
})

test_that("cor should be correlation matrix or NULL", {
  m <- "should be a correlation matrix or NULL"

  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = "a"), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = list()), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = 2), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = matrix(1:4)), m)
  expect_error(add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = matrix(1:4, nrow = 2)), m)

  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = "a"), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = list()), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = 2), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = matrix(1:4)), m)
  expect_error(add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbinom, 1, x2),
                               cor = matrix(1:4, nrow = 2)), m)
})

test_that("cor must be specified if no IPD", {
  expect_error(add_integration(smknet_agd,
                               x1 = distr(qnorm, mean = 1, sd = 1),
                               x2 = distr(qbinom, 1, x2),
                               cor = NULL),
               "Specify a correlation matrix")

  expect_error(add_integration(smkdummy,
                               x1 = distr(qnorm, mean = 1, sd = 1),
                               x2 = distr(qbinom, 1, x2),
                               cor = NULL),
               "Specify a correlation matrix")
})

test_that("cor is not required if only one covariate", {
  expect_s3_class(add_integration(smknet_agd, x1 = distr(qnorm, mean = 1, sd = 1), cor = NULL),
                  "mlnmr_data")

  s <- add_integration(smkdummy, x1 = distr(qnorm, mean = 1, sd = 1), cor = NULL)
  expect_s3_class(s, "data.frame")
  expect_true(rlang::has_name(s, ".int_x1"))
})

test_that("covariate arguments should be named distr", {
  m <- "should be specified as named arguments using the function `distr`"

  expect_error(add_integration(smknet, cor = cormat, x1 = "a"), m)
  expect_error(add_integration(smknet, cor = cormat, x1 = list()), m)
  expect_error(add_integration(smknet, cor = cormat, "a"), m)
  expect_error(add_integration(smknet, cor = cormat,
                               x1 = distr(qnorm, mean = 1, sd = 1), x2 = list(), m))
  expect_error(add_integration(smknet, cor = cormat,
                               x1 = distr(qnorm, mean = 1, sd = 1), distr(qnorm, mean = 1, sd = 1), m))
  expect_error(add_integration(smknet, cor = cormat), paste("No covariate distributions specified.+", m))

  expect_error(add_integration(smkdummy, cor = cormat, x1 = "a"), m)
  expect_error(add_integration(smkdummy, cor = cormat, x1 = list()), m)
  expect_error(add_integration(smkdummy, cor = cormat, "a"), m)
  expect_error(add_integration(smkdummy, cor = cormat,
                               x1 = distr(qnorm, mean = 1, sd = 1), x2 = list(), m))
  expect_error(add_integration(smkdummy, cor = cormat,
                               x1 = distr(qnorm, mean = 1, sd = 1), distr(qnorm, mean = 1, sd = 1), m))
  expect_error(add_integration(smkdummy, cor = cormat), paste("No covariate distributions specified.+", m))
})

test_that("covariate names must match IPD if provided", {
  m <- "Covariate name\\(s\\) not found in IPD: aaa"

  expect_error(add_integration(smknet, aaa = distr(qnorm, mean = x1_mean, sd = x1_sd)), m)
  expect_error(add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               aaa = distr(qbinom, size = 1, prob = x2)), m)
  expect_error(add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               aaa = distr(qbinom, size = 1, prob = x2),
                               cor = cormat), m)
})

test_that("warning if missing covariate values when calculating correlations from IPD", {
  smknet_miss <- combine_network(smknet_ipd,
                                 set_agd_arm(smkdummy, studyn, trtn, r = r, n = n))
  smknet_miss$ipd[5, "x1"] <- NA
  expect_warning(add_integration(smknet_miss,
                                 x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                 x2 = distr(qbinom, size = 1, prob = x2)),
                 "Missing values.+complete cases")
})

test_that("replaces already present integration points with warning", {
  smknet_int <- add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd))
  m <- "Replacing integration points already present in network"

  expect_warning(add_integration(smknet_int, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd)), m)
  expect_warning(add_integration(smknet_int, x3 = distr(qnorm, mean = x3_mean, sd = x3_sd)), m)

  smkdummy_int <- add_integration(smkdummy, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd))
  m2 <- "Replacing integration points already present in data frame"

  expect_warning(add_integration(smkdummy_int, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd)), m2)
  expect_warning(add_integration(smkdummy_int, x3 = distr(qnorm, mean = x3_mean, sd = x3_sd)), m2)
})

make_sub <- function(x, n, sub) {
  x[sample.int(length(x), n)] <- sub
  return(x)
}

test_that("error if qfun produces NaN, NA, NULL, Inf", {
  m <- "Invalid integration points were generated"

  smkdummy_miss <- smkdummy %>% mutate(x1_mean = make_sub(x1_mean, 5, NA),
                                       x3_mean = make_sub(x3_mean, 5, NA))

  smkdummy_miss_contrast <- smkdummy_contrast %>%
    mutate(x1_mean = make_sub(x1_mean, 5, NA), x3_mean = make_sub(x3_mean, 5, NA))

  smknet_miss <- set_agd_arm(smkdummy_miss, studyn, trtn, r = r, n = n) %>%
    combine_network(smknet_ipd)
  smknet_missc <- set_agd_contrast(smkdummy_miss_contrast, studyn, trtn, y = lor, se = se) %>%
    combine_network(smknet_ipd)

  expect_error(suppressWarnings(
    add_integration(smknet_miss,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x2 = distr(qbinom, 1, x2),
                    x3 = distr(qnorm, x3_mean, x3_sd))),
               m)

  expect_error(suppressWarnings(
    add_integration(smknet_missc,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x2 = distr(qbinom, 1, x2),
                    x3 = distr(qnorm, x3_mean, x3_sd))),
               m)

  smkdummy_bad <- smkdummy %>% mutate(x1_sd = make_sub(x1_sd, 5, -1),
                                      x3_sd = make_sub(x3_sd, 5, -1))

  smkdummy_bad_contrast <- smkdummy_contrast %>%
    mutate(x1_sd = make_sub(x1_sd, 5, -1), x3_sd = make_sub(x3_sd, 5, -1))

  smknet_bad <- set_agd_arm(smkdummy_bad, studyn, trtn, r = r, n = n) %>%
    combine_network(smknet_ipd)
  smknet_badc <- set_agd_contrast(smkdummy_bad_contrast, studyn, trtn, y = lor, se = se) %>%
    combine_network(smknet_ipd)

  expect_error(suppressWarnings(
    add_integration(smknet_bad,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x3 = distr(qnorm, mean = x3_mean, sd = x3_sd))),
               m)

  expect_error(suppressWarnings(
    add_integration(smknet_badc,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x3 = distr(qnorm, mean = x3_mean, sd = x3_sd))),
               m)

})

test_that("correctly assess type of marginal distributions", {
  dtype <- get_distribution_type(
             x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
             x2 = distr(qgamma, mean = x3_mean, sd = x3_sd),
             x3 = distr(qbern, prob = x2),
             x4 = distr(qbinom, size = 1, prob = x2),
             x5 = distr(qbinom, size = n, prob = x2),
             x6 = distr(qbinom, 1, x2),
             x7 = distr(qpois, lambda = x3_sd),
             x8 = distr(function(p, ...) qnorm(p, ...), mean = x1_mean, sd = x1_sd),
             x9 = distr(function(p, ...) qpois(p, ...), lambda = 0.5),
             x10 = distr(function(p, ...) qbern(p, ...), prob = x2),
             data = smkdummy)
  expect_identical(dtype,
                   c(x1 = "continuous",
                     x2 = "continuous",
                     x3 = "binary",
                     x4 = "binary",
                     x5 = "discrete",
                     x6 = "binary",
                     x7 = "discrete",
                     x8 = "continuous",
                     x9 = "discrete",
                     x10 = "binary"))
})

test_that("cor_adjust logic is correct", {
  expect_error(add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "bad"),
               "`cor_adjust` must be one of")

  # Spearman should be the default for cor = NULL
  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2))$int_cor, "cor_adjust"),
                   "spearman")
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "spearman")$int_cor, "copula_cor"),
               cor_adjust_spearman(
                 add_integration(smknet,
                                 x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                 x2 = distr(qbern, x2),
                                 cor_adjust = "spearman")$int_cor,
                 types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "pearson")$int_cor, "cor_adjust"),
                   "pearson")
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "pearson")$int_cor, "copula_cor"),
               cor_adjust_pearson(
                 add_integration(smknet,
                                 x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                 x2 = distr(qbern, x2),
                                 cor_adjust = "pearson")$int_cor,
                 types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "legacy")$int_cor, "cor_adjust"),
                   "legacy")
  # Check that "legacy" does the right (wrong) thing (Spearman cor with no adjustment)
  expect_equal(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "legacy")$int_cor, "copula_cor"),
               add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "spearman")$int_cor,
               check.attributes = FALSE,
               tolerance = 0)
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "legacy")$int_cor, "copula_cor"),
               add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "legacy")$int_cor,
               check.attributes = FALSE,
               tolerance = 0)

  expect_error(add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "none"),
               'Cannot specify cor_adjust = "none"')

  # Pearson should be the default for user-provided cor
  cmat <- matrix(c(1, 0.4, 0.4, 1), nrow = 2)
  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor = cmat)$int_cor, "cor_adjust"),
                   "pearson")
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor = cmat)$int_cor, "copula_cor"),
               cor_adjust_pearson(cmat, types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "spearman",
                                        cor = cmat)$int_cor, "cor_adjust"),
                   "spearman")
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "spearman",
                                    cor = cmat)$int_cor, "copula_cor"),
               cor_adjust_spearman(cmat, types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "none",
                                        cor = cmat)$int_cor, "cor_adjust"),
                   "none")
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "none",
                                    cor = cmat)$int_cor, "copula_cor"),
               add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "none",
                               cor = cmat)$int_cor,
               check.attributes = FALSE,
               tolerance = 0)
  expect_equal(add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "none",
                               cor = cmat)$int_cor,
               cmat,
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smknet,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "legacy",
                                        cor = cmat)$int_cor, "cor_adjust"),
                   "legacy")

  # Check that "legacy" does the right thing - same as "none" with user cor
  expect_equal(attr(add_integration(smknet,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "legacy",
                                    cor = cmat)$int_cor, "copula_cor"),
               add_integration(smknet,
                               x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                               x2 = distr(qbern, x2),
                               cor_adjust = "legacy",
                               cor = cmat)$int_cor,
               check.attributes = FALSE,
               tolerance = 0)

  # Pearson should be the default for user-provided cor
  expect_identical(attr(add_integration(smkdummy,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor = cmat), "cor_adjust"),
                   "pearson")
  expect_equal(attr(add_integration(smkdummy,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor = cmat), "copula_cor"),
               cor_adjust_pearson(cmat, types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smkdummy,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "spearman",
                                        cor = cmat), "cor_adjust"),
                   "spearman")
  expect_equal(attr(add_integration(smkdummy,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor = cmat,
                                    cor_adjust = "spearman"), "copula_cor"),
               cor_adjust_spearman(cmat, types = c("continuous", "binary")),
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smkdummy,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "none",
                                        cor = cmat), "cor_adjust"),
                   "none")
  expect_equal(attr(add_integration(smkdummy,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "none",
                                    cor = cmat), "copula_cor"),
               cmat,
               check.attributes = FALSE,
               tolerance = 0)

  expect_identical(attr(add_integration(smkdummy,
                                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                        x2 = distr(qbern, x2),
                                        cor_adjust = "legacy",
                                        cor = cmat), "cor_adjust"),
                   "legacy")

  # Check that "legacy" does the right thing - same as "none" with user cor
  expect_equal(attr(add_integration(smkdummy,
                                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                    x2 = distr(qbern, x2),
                                    cor_adjust = "legacy",
                                    cor = cmat), "copula_cor"),
               cmat,
               check.attributes = FALSE,
               tolerance = 0)

  # Check that reusing $int_cor from a network works automatically
  s1 <- add_integration(smknet,
                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                        x2 = distr(qbinom, size = 1, prob = x2),
                        x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                        x4 = distr(qbern, prob = x4))

  expect_identical(add_integration(smknet,
                                   x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                   x2 = distr(qbinom, size = 1, prob = x2),
                                   x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                                   x4 = distr(qbern, prob = x4),
                                   cor = s1$int_cor),
                   s1)

  s2 <- add_integration(smknet,
                        x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                        x2 = distr(qbinom, size = 1, prob = x2),
                        x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                        x4 = distr(qbern, prob = x4),
                        cor_adjust = "pearson")

  expect_identical(add_integration(smknet,
                                   x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                   x2 = distr(qbinom, size = 1, prob = x2),
                                   x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                                   x4 = distr(qbern, prob = x4),
                                   cor = s2$int_cor),
                   s2)
})

test_that("integration point marginals and correlations are correct", {
  skip_on_cran()
  tol <- 0.005
  cor_tol <- 0.05
  n_int <- 2^14

  # 1 covariate
  s1 <- add_integration(smknet, x1 = distr(qnorm, mean = x1_mean, sd = x1_sd), n_int = n_int)
  expect_equal(map_dbl(s1$agd_arm$.int_x1, mean), s1$agd_arm$x1_mean, tolerance = tol)
  expect_equal(map_dbl(s1$agd_arm$.int_x1, sd), s1$agd_arm$x1_sd, tolerance = tol)

  # Multiple covariates, IPD cor matrix
  s2_spearman <- add_integration(smknet,
                       x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                       x2 = distr(qbinom, size = 1, prob = x2),
                       x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                       x4 = distr(qbern, prob = x4),
                       n_int = n_int)

  # Mean, SD, proportion should match
  expect_equal(map_dbl(s2_spearman$agd_arm$.int_x1, mean), s2_spearman$agd_arm$x1_mean, tolerance = tol)
  expect_equal(map_dbl(s2_spearman$agd_arm$.int_x1, sd), s2_spearman$agd_arm$x1_sd, tolerance = tol)
  expect_equal(map_dbl(s2_spearman$agd_arm$.int_x2, mean), s2_spearman$agd_arm$x2, tolerance = tol)

  # Cont-cont correlations should match exactly for Spearman correlation
  expect_equal(map2_dbl(s2_spearman$agd_arm$.int_x1, s2_spearman$agd_arm$.int_x3, cor, method = "spearman"),
               rep(s2_spearman$int_cor[1, 3], nrow(s2_spearman$agd_arm)), tolerance = tol)

  # Other correlations should be approximate
  int_cors <- s2_spearman$agd_arm %>% rowwise() %>%
    mutate(int_cors = list(cor(cbind(.int_x1, .int_x2, .int_x3, .int_x4), method = "spearman"))) %>%
    pull(int_cors)
  # expect_equal(unlist(map(int_cors, ~.[upper.tri(.)])),
  #              rep(s2_spearman$int_cor[upper.tri(s2_spearman$int_cor)], times = nrow(s2_spearman$agd_arm)),
  #              tolerance = cor_tol)
  expect_lt(median(abs(unlist(map(int_cors, ~.[upper.tri(.)])) -
                   rep(s2_spearman$int_cor[upper.tri(s2_spearman$int_cor)], times = nrow(s2_spearman$agd_arm)))),
            cor_tol)

  s2_pearson <- add_integration(smknet,
                                 x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                 x2 = distr(qbinom, size = 1, prob = x2),
                                 x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                                x4 = distr(qbern, prob = x4),
                                cor_adjust = "pearson",
                                 n_int = n_int)
  expect_equal(map_dbl(s2_pearson$agd_arm$.int_x1, mean), s2_pearson$agd_arm$x1_mean, tolerance = tol)
  expect_equal(map_dbl(s2_pearson$agd_arm$.int_x1, sd), s2_pearson$agd_arm$x1_sd, tolerance = tol)
  expect_equal(map_dbl(s2_pearson$agd_arm$.int_x2, mean), s2_pearson$agd_arm$x2, tolerance = tol)

  # Normal-Normal correlations should match exactly for Pearson correlation
  expect_equal(map2_dbl(s2_pearson$agd_arm$.int_x1, s2_pearson$agd_arm$.int_x3, cor, method = "pearson"),
               rep(s2_pearson$int_cor[1, 3], nrow(s2_pearson$agd_arm)), tolerance = tol)

  # Other correlations should be approximate
  int_cors <- s2_pearson$agd_arm %>% rowwise() %>%
    mutate(int_cors = list(cor(cbind(.int_x1, .int_x2, .int_x3, .int_x4), method = "pearson"))) %>%
    pull(int_cors)
  # expect_equal(unlist(map(int_cors, ~.[upper.tri(.)])),
  #              rep(s2_pearson$int_cor[upper.tri(s2_pearson$int_cor)], times = nrow(s2_pearson$agd_arm)),
  #              tolerance = cor_tol)
  expect_lt(median(abs(unlist(map(int_cors, ~.[upper.tri(.)])) -
                         rep(s2_pearson$int_cor[upper.tri(s2_pearson$int_cor)], times = nrow(s2_pearson$agd_arm)))),
            cor_tol)

  # Multiple covariates, user cor matrix
  s3_pearson <- add_integration(smknet,
                       x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                       x2 = distr(qbinom, size = 1, prob = x2),
                       x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                       x4 = distr(qbern, prob = x4),
                       cor = cormat,
                       n_int = n_int)
  expect_equal(map_dbl(s3_pearson$agd_arm$.int_x1, mean), s3_pearson$agd_arm$x1_mean, tolerance = tol)
  expect_equal(map_dbl(s3_pearson$agd_arm$.int_x1, sd), s3_pearson$agd_arm$x1_sd, tolerance = tol)
  expect_equal(map_dbl(s3_pearson$agd_arm$.int_x2, mean), s3_pearson$agd_arm$x2, tolerance = tol)

  # Normal-Normal correlations should match exactly for Pearson correlation
  expect_equal(map2_dbl(s3_pearson$agd_arm$.int_x1, s3_pearson$agd_arm$.int_x3, cor, method = "pearson"),
               rep(cormat[1, 3], nrow(s3_pearson$agd_arm)), tolerance = tol)

  # Other correlations should be approximate
  int_cors <- s3_pearson$agd_arm %>% rowwise() %>%
    mutate(int_cors = list(cor(cbind(.int_x1, .int_x2, .int_x3, .int_x4), method = "pearson"))) %>%
    pull(int_cors)
  # expect_equal(unlist(map(int_cors, ~.[upper.tri(.)])),
  #              rep(s3_pearson$int_cor[upper.tri(s3_pearson$int_cor)], times = nrow(s3_pearson$agd_arm)),
  #              tolerance = cor_tol)
  expect_lt(median(abs(unlist(map(int_cors, ~.[upper.tri(.)])) -
                         rep(s3_pearson$int_cor[upper.tri(s3_pearson$int_cor)], times = nrow(s3_pearson$agd_arm)))),
            cor_tol)


  s3_spearman <- add_integration(smknet,
                                x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                                x2 = distr(qbinom, size = 1, prob = x2),
                                x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                                x4 = distr(qbern, prob = x4),
                                cor = cormat, cor_adjust = "spearman",
                                n_int = n_int)
  expect_equal(map_dbl(s3_spearman$agd_arm$.int_x1, mean), s3_spearman$agd_arm$x1_mean, tolerance = tol)
  expect_equal(map_dbl(s3_spearman$agd_arm$.int_x1, sd), s3_spearman$agd_arm$x1_sd, tolerance = tol)
  expect_equal(map_dbl(s3_spearman$agd_arm$.int_x2, mean), s3_spearman$agd_arm$x2, tolerance = tol)

  # Cont-cont correlations should match exactly for Spearman correlation
  expect_equal(map2_dbl(s3_spearman$agd_arm$.int_x1, s3_spearman$agd_arm$.int_x3, cor, method = "spearman"),
               rep(cormat[1, 3], nrow(s3_spearman$agd_arm)), tolerance = tol)

  # Other correlations should be approximate
  int_cors <- s3_spearman$agd_arm %>% rowwise() %>%
    mutate(int_cors = list(cor(cbind(.int_x1, .int_x2, .int_x3, .int_x4), method = "spearman"))) %>%
    pull(int_cors)
  # expect_equal(unlist(map(int_cors, ~.[upper.tri(.)])),
  #              rep(s3_spearman$int_cor[upper.tri(s3_spearman$int_cor)], times = nrow(s3_spearman$agd_arm)),
  #              tolerance = cor_tol)
  expect_lt(median(abs(unlist(map(int_cors, ~.[upper.tri(.)])) -
                         rep(s3_spearman$int_cor[upper.tri(s3_spearman$int_cor)], times = nrow(s3_spearman$agd_arm)))),
            cor_tol)
})

test_that("non positive definite cor matrices are fixed up", {
  badcor <- matrix(0.99, nrow = 4, ncol = 4)
  diag(badcor) <- 1

  expect_warning(
    add_integration(smknet,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x2 = distr(qbinom, size = 1, prob = x2),
                    x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                    x4 = distr(qbern, prob = x4),
                    cor = badcor,
                    n_int = 10),
    "Adjusted correlation matrix not positive definite; using Matrix::nearPD()."
  )

  expect_warning(
    add_integration(smknet,
                    x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                    x2 = distr(qbinom, size = 1, prob = x2),
                    x3 = distr(qnorm, mean = x3_mean, sd = x3_sd),
                    x4 = distr(qbern, prob = x4),
                    cor = badcor, cor_adjust = "spearman",
                    n_int = 10),
    "Adjusted correlation matrix not positive definite; using Matrix::nearPD()."
  )
})

#' multinma: A Package for Network Meta-Analysis of Individual and Aggregate
#' Data in Stan
#'
#' @description
#'   \if{html}{\figure{logo.svg}{options: width="120" style="float: right;" alt="multinma logo"}}
#'   An R package for performing network meta-analysis and network meta-regression
#'   with aggregate data, individual patient data, or mixtures of both.
#'
#' @details Network meta-analysis (NMA) combines (aggregate) data from multiple
#'   studies on multiple treatments in order to produce consistent estimates of
#'   relative treatment effects between each pair of treatments in the network
#'   \insertCite{TSD2}{multinma}.
#'
#'   Network meta-regression (NMR) extends NMA to include covariates, allowing
#'   adjustment for differences in effect-modifying variables between studies
#'   \insertCite{TSD3}{multinma}. NMR is typically performed using aggregate
#'   data (AgD), which lacks power and is prone to ecological bias. NMR with
#'   individual patient data (IPD) is the gold standard, if data are available.
#'
#'   Multilevel network meta-regression (ML-NMR) allows IPD and AgD to be
#'   incorporated together in a network meta-regression
#'   \insertCite{methods_paper,Phillippo_thesis}{multinma}. As in IPD NMR, an
#'   individual-level regression model is defined. AgD studies are then fitted
#'   by integrating the individual-level model over the respective covariate
#'   distributions. This correctly links the two levels of the model (instead of
#'   "plugging in" mean covariate values), avoiding aggregation bias.
#'   Population-adjusted treatment effects \insertCite{TSD18}{multinma} can be
#'   produced for any study population in the network, or for an external target
#'   population.
#'
#'   Models are estimated in a Bayesian framework using Stan
#'   \insertCite{Carpenter2017}{multinma}. Quasi-Monte Carlo numerical
#'   integration based on Sobol' sequences is used for the integration in ML-NMR
#'   models, with a Gaussian copula to account for correlations between
#'   covariates \insertCite{methods_paper,Phillippo_thesis}{multinma}.
#'
#' @section Getting Started:
#'   A good place to start is with the package vignettes which walk through
#'   example analyses, see `vignette("vignette_overview")` for an overview.
#'   The series of NICE Technical Support Documents on evidence synthesis gives
#'   a detailed introduction to network meta-analysis:
#'
#'   \insertRef{TSD_evsynth}{multinma}
#'
#'   Multilevel network meta-regression is set out in the following methods paper:
#'
#'   \insertRef{methods_paper}{multinma}
#'
#' @name multinma-package
#' @aliases multinma
#' @useDynLib multinma, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom dplyr %>%
#' @importFrom rlang abort warn inform enquo .data := %||%
#' @importFrom rstan sampling
#' @importFrom Rdpack reprompt
#' @importFrom graphics plot pairs
#' @importFrom grDevices nclass.Sturges
#' @importFrom stats complete.cases sd median quantile model.frame model.matrix
#'   model.offset terms optim pbinom dbinom qbinom as.formula update.formula
#'   weighted.mean runif dunif plogis pnorm qlogis qnorm uniroot update var
#'   setNames
#' @importFrom utils packageVersion head
#' @importFrom RcppParallel CxxFlags
#' @importFrom rstantools rstan_config
#'
#' @references
#'   \insertAllCited{}
#'
"_PACKAGE"

# Stop R CMD check thinking . used in pipes is an undeclared global variable
if (getRversion() >= "2.15.1")  utils::globalVariables(c("."))

# Reexport survival::Surv()
#' @importFrom survival Surv
#' @export
survival::Surv

#' Set up individual patient data
#'
#' Set up a network containing individual patient data (IPD). Multiple data
#' sources may be combined once created using [combine_network()].
#'
#' @template args-data_common
#' @template args-data_y
# #' @template args-data_rE
#' @param r column of `data` specifying a binary outcome or Poisson outcome count
#' @param E column of `data` specifying the total time at risk for Poisson
#'   outcomes
#' @template args-data_Surv
#'
#' @return An object of class [nma_data]
#' @export
#'
#' @template args-details_trt_ref
#' @template args-details_mutate
#'
#' @seealso [set_agd_arm()] for arm-based aggregate data, [set_agd_contrast()]
#'   for contrast-based aggregate data, and [combine_network()] for combining
#'   several data sources in one network.
#' @template seealso_nma_data
#' @examples
#' # Set up network of plaque psoriasis IPD
#' head(plaque_psoriasis_ipd)
#'
#' pso_net <- set_ipd(plaque_psoriasis_ipd,
#'                    study = studyc,
#'                    trt = trtc,
#'                    r = pasi75)
#'
#' # Print network details
#' pso_net
#'
#' # Plot network
#' plot(pso_net)
#'
#' # Setting a different reference treatment
#' set_ipd(plaque_psoriasis_ipd,
#'         study = studyc,
#'         trt = trtc,
#'         r = pasi75,
#'         trt_ref = "PBO")

set_ipd <- function(data,
                    study,
                    trt,
                    y = NULL,
                    r = NULL, E = NULL,
                    Surv = NULL,
                    trt_ref = NULL,
                    trt_class = NULL) {

  # Check data is data frame
  if (!inherits(data, "data.frame")) abort("Argument `data` should be a data frame")
  if (nrow(data) == 0) {
    return(
      structure(
        list(agd_arm = NULL,
             agd_contrast = NULL,
             ipd = NULL,
             treatments = NULL,
             classes = NULL,
             studies = NULL),
        class = "nma_data")
    )
  }

  # Pull study and treatment columns
  if (missing(study)) abort("Specify `study`")
  .study <- pull_non_null(data, enquo(study))
  if (is.null(.study)) abort("`study` cannot be NULL")
  check_study(.study)

  if (is.factor(.study)) {
    study_original_levels <- levels(.study)
    .study <- forcats::fct_drop(.study)
  } else {
    study_original_levels <- NULL
  }

  if (missing(trt)) abort("Specify `trt`")
  .trt <- pull_non_null(data, enquo(trt))
  if (is.null(.trt)) abort("`trt` cannot be NULL")
  check_trt(.trt)

  if (is.factor(.trt)) {
    trt_original_levels <- levels(.trt)
    .trt <- forcats::fct_drop(.trt)
  } else {
    trt_original_levels <- NULL
  }

  # Treatment classes
  .trtclass <- pull_non_null(data, enquo(trt_class))
  if (!is.null(.trtclass)) {
    check_trt_class(.trtclass, .trt)

    if (is.factor(.trtclass)) {
      trtclass_original_levels <- levels(.trtclass)
      .trtclass <- forcats::fct_drop(.trtclass)
    } else {
      trtclass_original_levels <- NULL
    }
  }

  if (!is.null(trt_ref) && length(trt_ref) > 1) abort("`trt_ref` must be length 1.")

  # Pull and check outcomes
  .y <- pull_non_null(data, enquo(y))
  .r <- pull_non_null(data, enquo(r))
  .E <- pull_non_null(data, enquo(E))
  .Surv <- pull_non_null(data, enquo(Surv))

  check_outcome_continuous(.y, with_se = FALSE)

  if (!is.null(.r) && inherits(.r, c("multi_ordered", "multi_competing"))) {
    if (inherits(.r, "multi_competing")) abort("Competing multinomial outcomes are not yet supported.")

    # Most checks are carried out by multi(), some additional checks here specific to IPD
    if (any(! .r[!is.na(.r)] %in% c(0, 1))) abort("Multinomial outcome `r` must equal 0 or 1")
    if (any(r_zero_rows <- rowSums(.r, na.rm = TRUE) == 0))
      abort(glue::glue("Individual{if (sum(r_zero_rows) > 1) 's' else ''} without outcomes in any category, ",
                       "row{if (sum(r_zero_rows) > 1) 's' else ''} ",
                       glue::glue_collapse(which(r_zero_rows), sep = ", ", last = " and "), "."))
    if (any(r_multi_rows <- rowSums(.r, na.rm = TRUE) > 1))
      abort(glue::glue("Individual{if (sum(r_multi_rows) > 1) 's' else ''} with outcomes in more than one category, ",
                       "row{if (sum(r_multi_rows) > 1) 's' else ''} ",
                       glue::glue_collapse(which(r_multi_rows), sep = ", ", last = " and "), "."))
  } else {
    check_outcome_binary(.r, .E)
  }

  check_outcome_survival(.Surv)

  o_type <- get_outcome_type(y = .y, se = NULL,
                             r = .r, n = NULL, E = .E,
                             Surv = .Surv)

  # Check for single-arm studies
  single_arm_studies <- tibble::tibble(.study, .trt) %>%
    dplyr::distinct(.data$.study, .data$.trt) %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::filter(dplyr::n() == 1) %>%
    dplyr::pull(.data$.study)

  if (length(single_arm_studies)) {
    if (o_type == "survival") {
      inform(glue::glue("Single-arm stud{if (length(single_arm_studies) > 1) 'ies' else 'y'} present in the network: ",
                        glue::glue_collapse(glue::double_quote(as.character(single_arm_studies)), sep = ", ", last = " and "), "."))
    } else {
      abort(glue::glue("Single-arm studies are not supported: issue with stud{if (length(single_arm_studies) > 1) 'ies' else 'y'} ",
                       glue::glue_collapse(glue::double_quote(single_arm_studies), sep = ", ", last = " and "), "."))
    }
  }

  # Create tibble in standard format
  d <- tibble::tibble(
    .study = nfactor(.study),
    .trt = nfactor(.trt)
  )

  if (!is.null(trt_ref)) {
    trt_ref <- as.character(trt_ref)
    lvls_trt <- levels(d$.trt)
    if (! trt_ref %in% lvls_trt)
      abort(sprintf("`trt_ref` does not match a treatment in the data.\nSuitable values are: %s",
                    ifelse(length(lvls_trt) <= 5,
                           paste0(lvls_trt, collapse = ", "),
                           paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    d$.trt <- forcats::fct_relevel(d$.trt, trt_ref)
  }

  if (!is.null(.trtclass)) {
    d <- tibble::add_column(d, .trtclass = nfactor(.trtclass))
    class_lookup <- d %>%
      dplyr::distinct(.data$.trt, .data$.trtclass) %>%
      dplyr::arrange(.data$.trt)
    class_ref <- as.character(class_lookup[[1, ".trtclass"]])
    d$.trtclass <- forcats::fct_relevel(d$.trtclass, class_ref)
    classes <- forcats::fct_relevel(nfactor(class_lookup$.trtclass), class_ref)
  } else {
    classes <- NULL
  }

  if (o_type == "continuous") {
    d <- tibble::add_column(d, .y = .y)
  } else if (o_type == "binary") {
    d <- tibble::add_column(d, .r = .r)
  } else if (o_type == "rate") {
    d <- tibble::add_column(d, .r = .r, .E = .E)
  } else if (o_type %in% c("ordered", "competing")) {
    # Store internally as a standard matrix (strip class attribute) as a simple
    # work-around for dplyr/vctrs not using s3 inheritance to find matrix methods
    .r <- unclass(.r)

    d <- tibble::add_column(d, .r = .r)
  } else if (o_type == "survival") {
    d <- tibble::add_column(d, .Surv = .Surv)
  }

  drop_reserved <- setdiff(colnames(data), colnames(d))
  d <- dplyr::bind_cols(d, data[, drop_reserved, drop = FALSE])

  # Drop original study and treatment columns
  d <- drop_original(d, data, enquo(study))
  d <- drop_original(d, data, enquo(trt))
  if (!is.null(.trtclass)) d <- drop_original(d, data, enquo(trt_class))

  # Produce nma_data object
  out <- structure(
    list(agd_arm = NULL,
         agd_contrast = NULL,
         ipd = d,
         treatments = forcats::fct_unique(d$.trt),
         classes = classes,
         studies = forcats::fct_unique(d$.study),
         outcome = list(agd_arm = NA, agd_contrast = NA, ipd = o_type)),
    class = "nma_data")

  # If trt_ref not specified, mark treatments factor as default, calculate
  # current reference trt
  if (is.null(trt_ref)) {
    trt_ref <- get_default_trt_ref(out)
    trt_sort <- order(forcats::fct_relevel(out$treatments, trt_ref))
    out$treatments <- .default(forcats::fct_relevel(out$treatments, trt_ref)[trt_sort])
    out$ipd$.trt <- forcats::fct_relevel(out$ipd$.trt, trt_ref)
    if (!is.null(.trtclass)) {
      class_ref <- as.character(out$classes[trt_sort[1]])
      if (!is.null(trtclass_original_levels))
        class_ref <- c(class_ref,
                       setdiff(intersect(trtclass_original_levels, levels(out$classes)),
                               class_ref))
      out$ipd$.trtclass <- forcats::fct_relevel(out$ipd$.trtclass, class_ref)
      out$classes <- forcats::fct_relevel(out$classes, class_ref)[trt_sort]
    }
  }

  # Add original_levels attributes (if not NULL)
  attr(out$treatments, "original_levels") <- trt_original_levels
  attr(out$studies, "original_levels") <- study_original_levels
  if (!is.null(.trtclass)) attr(out$classes, "original_levels") <- trtclass_original_levels

  return(out)
}


#' Set up arm-based aggregate data
#'
#' Set up a network containing arm-based aggregate data (AgD), such as event
#' counts or mean outcomes on each arm. Multiple data sources may be combined
#' once created using [combine_network()].
#'
#' @template args-data_common
#' @template args-data_y
#' @template args-data_se
#' @template args-data_rE
#' @param n column of `data` specifying Binomial outcome numerator
#' @param sample_size column of `data` giving the sample size in each arm.
#'   Optional, see details.
#'
#' @return An object of class [nma_data]
#' @export

#' @template args-details_trt_ref
#' @template args-details_sample_size
#' @details
#' If a Binomial outcome is specified and `sample_size` is omitted, `n` will be
#' used as the sample size by default. If a Multinomial outcome is specified and
#' `sample_size` is omitted, the sample size will be determined automatically
#' from the supplied counts by default.
#'
#' @template args-details_mutate
#'
#' @seealso [set_ipd()] for individual patient data, [set_agd_contrast()] for
#'   contrast-based aggregate data, and [combine_network()] for combining
#'   several data sources in one network.
#' @template seealso_nma_data
#' @template ex_smoking_network
#' @examples
#'
#' # Plot network
#' plot(smk_net)
set_agd_arm <- function(data,
                        study,
                        trt,
                        y = NULL, se = NULL,
                        r = NULL, n = NULL, E = NULL,
                        sample_size = NULL,
                        trt_ref = NULL,
                        trt_class = NULL) {

  # Check data is data frame
  if (!inherits(data, "data.frame")) abort("Argument `data` should be a data frame")
  if (nrow(data) == 0) {
    return(
      structure(
        list(agd_arm = NULL,
             agd_contrast = NULL,
             ipd = NULL,
             treatments = NULL,
             classes = NULL,
             studies = NULL),
        class = "nma_data")
    )
  }

  # Pull study and treatment columns
  if (missing(study)) abort("Specify `study`")
  .study <- pull_non_null(data, enquo(study))
  if (is.null(.study)) abort("`study` cannot be NULL")
  check_study(.study)

  if (is.factor(.study)) {
    study_original_levels <- levels(.study)
    .study <- forcats::fct_drop(.study)
  } else {
    study_original_levels <- NULL
  }

  if (missing(trt)) abort("Specify `trt`")
  .trt <- pull_non_null(data, enquo(trt))
  if (is.null(.trt)) abort("`trt` cannot be NULL")
  check_trt(.trt)

  if (is.factor(.trt)) {
    trt_original_levels <- levels(.trt)
    .trt <- forcats::fct_drop(.trt)
  } else {
    trt_original_levels <- NULL
  }

  # Check for single-arm studies
  single_arm_studies <- tibble::tibble(.study, .trt) %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::filter(dplyr::n() == 1) %>%
    dplyr::pull(.data$.study)

  if (length(single_arm_studies)) {
    abort(glue::glue("Single-arm studies are not supported: issue with stud{if (length(single_arm_studies) > 1) 'ies' else 'y'} ",
                     glue::glue_collapse(glue::double_quote(single_arm_studies), sep = ", ", last = " and "), "."))
  }

  # Treatment classes
  .trtclass <- pull_non_null(data, enquo(trt_class))
  if (!is.null(.trtclass)) {
    check_trt_class(.trtclass, .trt)

    if (is.factor(.trtclass)) {
      trtclass_original_levels <- levels(.trtclass)
      .trtclass <- forcats::fct_drop(.trtclass)
    } else {
      trtclass_original_levels <- NULL
    }
  }

  if (!is.null(trt_ref) && length(trt_ref) > 1) abort("`trt_ref` must be length 1.")

  # Pull and check outcomes
  .y <- pull_non_null(data, enquo(y))
  .se <- pull_non_null(data, enquo(se))
  .r <- pull_non_null(data, enquo(r))
  .n <- pull_non_null(data, enquo(n))
  .E <- pull_non_null(data, enquo(E))

  check_outcome_continuous(.y, .se, with_se = TRUE)

  if (!is.null(.r) && inherits(.r, c("multi_ordered", "multi_competing"))) {
    # Checks are carried out by multi()
    if (inherits(.r, "multi_competing")) abort("Competing multinomial outcomes are not yet supported.")
  } else {
    check_outcome_count(.r, .n, .E)
  }

  o_type <- get_outcome_type(y = .y, se = .se,
                             r = .r, n = .n, E = .E,
                             Surv = NULL)

  # Pull and check sample size
  .sample_size <- pull_non_null(data, enquo(sample_size))
  if (!is.null(.sample_size)) {
    check_sample_size(.sample_size)
  } else if (o_type == "count") {
    .sample_size <- .n
  } else if (o_type %in% c("ordered", "competing")) {
    # Always stored as exclusive counts
    .sample_size <- rowSums(.r, na.rm = TRUE)
  }
  else inform("Note: Optional argument `sample_size` not provided, some features may not be available (see ?set_agd_arm).")

  # Create tibble in standard format
  d <- tibble::tibble(
    .study = nfactor(.study),
    .trt = nfactor(.trt)
  )

  if (!is.null(trt_ref)) {
    trt_ref <- as.character(trt_ref)
    lvls_trt <- levels(d$.trt)
    if (! trt_ref %in% lvls_trt)
      abort(sprintf("`trt_ref` does not match a treatment in the data.\nSuitable values are: %s",
                    ifelse(length(lvls_trt) <= 5,
                           paste0(lvls_trt, collapse = ", "),
                           paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    d$.trt <- forcats::fct_relevel(d$.trt, trt_ref)
  }

  if (!is.null(.trtclass)) {
    d <- tibble::add_column(d, .trtclass = nfactor(.trtclass))
    class_lookup <- d %>%
      dplyr::distinct(.data$.trt, .data$.trtclass) %>%
      dplyr::arrange(.data$.trt)
    class_ref <- as.character(class_lookup[[1, ".trtclass"]])
    d$.trtclass <- forcats::fct_relevel(d$.trtclass, class_ref)
    classes <- forcats::fct_relevel(nfactor(class_lookup$.trtclass), class_ref)
  } else {
    classes <- NULL
  }

  if (o_type == "continuous") {
    d <- tibble::add_column(d, .y = .y, .se = .se)
  } else if (o_type == "count") {
    d <- tibble::add_column(d, .r = .r, .n = .n)
  } else if (o_type == "rate") {
    d <- tibble::add_column(d, .r = .r, .E = .E)
  } else if (o_type %in% c("ordered", "competing")) {
    # Store internally as a standard matrix (strip class attribute) as a simple
    # work-around for dplyr/vctrs not using s3 inheritance to find matrix methods
    .r <- unclass(.r)

    d <- tibble::add_column(d, .r = .r)
  }

  if (!is.null(.sample_size)) d <- tibble::add_column(d, .sample_size = .sample_size)

  # Bind in original data
  drop_reserved <- setdiff(colnames(data), colnames(d))
  d <- dplyr::bind_cols(d, data[, drop_reserved, drop = FALSE])

  # Drop original study and treatment columns
  d <- drop_original(d, data, enquo(study))
  d <- drop_original(d, data, enquo(trt))
  if (!is.null(.trtclass)) d <- drop_original(d, data, enquo(trt_class))

  # Produce nma_data object
  out <- structure(
    list(agd_arm = d,
         agd_contrast = NULL,
         ipd = NULL,
         treatments = forcats::fct_unique(d$.trt),
         classes = classes,
         studies = forcats::fct_unique(d$.study),
         outcome = list(agd_arm = o_type, agd_contrast = NA, ipd = NA)),
    class = "nma_data")

  # If trt_ref not specified, mark treatments factor as default, calculate
  # current reference trt
  if (is.null(trt_ref)) {
    trt_ref <- get_default_trt_ref(out)
    trt_sort <- order(forcats::fct_relevel(out$treatments, trt_ref))
    out$treatments <- .default(forcats::fct_relevel(out$treatments, trt_ref)[trt_sort])
    out$agd_arm$.trt <- forcats::fct_relevel(out$agd_arm$.trt, trt_ref)
    if (!is.null(.trtclass)) {
      class_ref <- as.character(out$classes[trt_sort[1]])
      if (!is.null(trtclass_original_levels))
        class_ref <- c(class_ref,
                       setdiff(intersect(trtclass_original_levels, levels(out$classes)),
                               class_ref))
      out$agd_arm$.trtclass <- forcats::fct_relevel(out$agd_arm$.trtclass, class_ref)
      out$classes <- forcats::fct_relevel(out$classes, class_ref)[trt_sort]
    }
  }

  # Add original_levels attributes (if not NULL)
  attr(out$treatments, "original_levels") <- trt_original_levels
  attr(out$studies, "original_levels") <- study_original_levels
  if (!is.null(.trtclass)) attr(out$classes, "original_levels") <- trtclass_original_levels

  return(out)
}


#' Set up contrast-based aggregate data
#'
#' Set up a network containing contrast-based aggregate data (AgD), i.e.
#' summaries of relative effects between treatments such as log Odds Ratios.
#' Multiple data sources may be combined once created using [combine_network()].
#'
#' @template args-data_common
#' @template args-data_y
#' @template args-data_se
#' @param sample_size column of `data` giving the sample size in each arm.
#'   Optional, see details.
#'
#' @details Each study should have a single reference/baseline treatment,
#'   against which relative effects in the other arm(s) are given. For the
#'   reference arm, include a data row with continuous outcome `y` equal to
#'   `NA`. If a study has three or more arms (so two or more relative effects),
#'   set the standard error `se` for the reference arm data row equal to the
#'   standard error of the mean outcome on the reference arm (this determines
#'   the covariance of the relative effects, when expressed as differences in
#'   mean outcomes between arms).
#'
#' @template args-details_mutate
#'
#' @template args-details_trt_ref
#' @template args-details_sample_size
#'
#' @return An object of class [nma_data]
#' @export
#'
#' @seealso [set_ipd()] for individual patient data, [set_agd_arm()] for
#'   arm-based aggregate data, and [combine_network()] for combining several
#'   data sources in one network.
#' @template seealso_nma_data
#' @examples
#' # Set up network of Parkinson's contrast data
#' head(parkinsons)
#'
#' park_net <- set_agd_contrast(parkinsons,
#'                              study = studyn,
#'                              trt = trtn,
#'                              y = diff,
#'                              se = se_diff,
#'                              sample_size = n)
#'
#' # Print details
#' park_net
#'
#' # Plot network
#' plot(park_net)

set_agd_contrast <- function(data,
                             study,
                             trt,
                             y = NULL, se = NULL,
                             sample_size = NULL,
                             trt_ref = NULL,
                             trt_class = NULL) {

  # Check data is data frame
  if (!inherits(data, "data.frame")) abort("Argument `data` should be a data frame")
  if (nrow(data) == 0) {
    return(
      structure(
        list(agd_arm = NULL,
             agd_contrast = NULL,
             ipd = NULL,
             treatments = NULL,
             classes = NULL,
             studies = NULL),
        class = "nma_data")
    )
  }


  # Pull study and treatment columns
  if (missing(study)) abort("Specify `study`")
  .study <- pull_non_null(data, enquo(study))
  if (is.null(.study)) abort("`study` cannot be NULL")
  check_study(.study)

  if (is.factor(.study)) {
    study_original_levels <- levels(.study)
    .study <- forcats::fct_drop(.study)
  } else {
    study_original_levels <- NULL
  }

  if (missing(trt)) abort("Specify `trt`")
  .trt <- pull_non_null(data, enquo(trt))
  if (is.null(.trt)) abort("`trt` cannot be NULL")
  check_trt(.trt)

  if (is.factor(.trt)) {
    trt_original_levels <- levels(.trt)
    .trt <- forcats::fct_drop(.trt)
  } else {
    trt_original_levels <- NULL
  }

  # Check for single-arm studies
  single_arm_studies <- tibble::tibble(.study, .trt) %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::filter(dplyr::n() == 1) %>%
    dplyr::pull(.data$.study)

  if (length(single_arm_studies)) {
    abort(glue::glue("Single-arm studies are not supported: issue with stud{if (length(single_arm_studies) > 1) 'ies' else 'y'} ",
                     glue::glue_collapse(glue::double_quote(single_arm_studies), sep = ", ", last = " and "), "."))
  }

  # Treatment classes
  .trtclass <- pull_non_null(data, enquo(trt_class))
  if (!is.null(.trtclass)) {
    check_trt_class(.trtclass, .trt)

    if (is.factor(.trtclass)) {
      trtclass_original_levels <- levels(.trtclass)
      .trtclass <- forcats::fct_drop(.trtclass)
    } else {
      trtclass_original_levels <- NULL
    }
  }

  if (!is.null(trt_ref) && length(trt_ref) > 1) abort("`trt_ref` must be length 1.")

  # Pull and check outcomes
  .y <- pull_non_null(data, enquo(y))
  .se <- pull_non_null(data, enquo(se))

  if (is.null(.y)) abort("Specify continuous outcome `y`")
  if (rlang::is_list(.y) || !is.null(dim(.y)))
    abort("Continuous outcome `y` must be a regular column (not a list or matrix column)")

  if (is.null(.se)) abort("Specify standard error `se`")
  if (rlang::is_list(.se) || !is.null(dim(.se)))
    abort("Standard error `se` must be a regular column (not a list or matrix column)")

  # Pull and check sample size
  .sample_size <- pull_non_null(data, enquo(sample_size))
  if (!is.null(.sample_size)) {
    check_sample_size(.sample_size)
  } else {
    inform("Note: Optional argument `sample_size` not provided, some features may not be available (see ?set_agd_contrast).")
  }

  # Determine baseline arms by .y = NA
  bl <- is.na(.y)

  # if (anyDuplicated(.study[bl])) abort("Multiple baseline arms (where y = NA) for a study.")

  tibble::tibble(.study, .trt, bl, .se) %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::mutate(n_arms = dplyr::n(),
                  n_bl = sum(.data$bl)) %>%
    {
      if (any(.$n_bl > 1))
        abort("Multiple baseline arms (where y = NA) in a study or studies.")
      else if (any(.$n_bl == 0))
        abort("Study or studies without a specified baseline arm (where y = NA).")
      else .
    } %>%
    dplyr::filter(.data$bl, .data$n_arms > 2) %>%
    {
      check_outcome_continuous(1, .$.se, with_se = TRUE,
                               append = " on baseline arms in studies with >2 arms.")
    }

  check_outcome_continuous(.y[!bl], .se[!bl], with_se = TRUE,
    append = " for non-baseline rows (i.e. those specifying contrasts against baseline).")

  o_type <- get_outcome_type(y = .y[!bl], se = .se[!bl],
                             r = NULL, n = NULL, E = NULL, Surv = NULL)

  # Create tibble in standard format
  d <- tibble::tibble(
    .study = nfactor(.study),
    .trt = nfactor(.trt),
    .y = .y,
    .se = .se)

  if (!is.null(trt_ref)) {
    trt_ref <- as.character(trt_ref)
    lvls_trt <- levels(d$.trt)
    if (! trt_ref %in% lvls_trt)
      abort(sprintf("`trt_ref` does not match a treatment in the data.\nSuitable values are: %s",
                    ifelse(length(lvls_trt) <= 5,
                           paste0(lvls_trt, collapse = ", "),
                           paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    d$.trt <- forcats::fct_relevel(d$.trt, trt_ref)
  }

  if (!is.null(.trtclass)) {
    d <- tibble::add_column(d, .trtclass = nfactor(.trtclass))
    class_lookup <- d %>%
      dplyr::distinct(.data$.trt, .data$.trtclass) %>%
      dplyr::arrange(.data$.trt)
    class_ref <- as.character(class_lookup[[1, ".trtclass"]])
    d$.trtclass <- forcats::fct_relevel(d$.trtclass, class_ref)
    classes <- forcats::fct_relevel(nfactor(class_lookup$.trtclass), class_ref)
  } else {
    classes <- NULL
  }

  if (!is.null(.sample_size)) {
    d <- tibble::add_column(d, .sample_size = .sample_size)
  }

  # Bind in original data
  drop_reserved <- setdiff(colnames(data), colnames(d))
  d <- dplyr::bind_cols(d, data[, drop_reserved, drop = FALSE])

  # Drop original study and treatment columns
  d <- drop_original(d, data, enquo(study))
  d <- drop_original(d, data, enquo(trt))
  if (!is.null(.trtclass)) d <- drop_original(d, data, enquo(trt_class))

  # Make sure rows from each study are next to each other (required for Stan resdev/log_lik code)
  d <- dplyr::mutate(d, .study_inorder = forcats::fct_inorder(.data$.study)) %>%
    dplyr::arrange(.data$.study_inorder) %>%
    dplyr::select(-".study_inorder")

  # Check covariance matrices are positive definite
  agd_Sigma <- make_Sigma(d)
  posdef <- purrr::map_lgl(agd_Sigma, ~all(eigen(., only.values = TRUE)$values > sqrt(.Machine$double.eps)))

  if (any(!posdef)) {
    abort(glue::glue("Contrast covariance matrix not positive definite for stud{if (sum(!posdef) > 1) 'ies' else 'y'} ",
                     glue::glue_collapse(glue::double_quote(names(posdef[!posdef])), sep = ", ", last = " and "), ".\n",
                     "Check the `se` column."))
  }

  # Produce nma_data object
  out <- structure(
    list(agd_arm = NULL,
         agd_contrast = d,
         ipd = NULL,
         treatments = forcats::fct_unique(d$.trt),
         classes = classes,
         studies = forcats::fct_unique(d$.study),
         outcome = list(agd_arm = NA, agd_contrast = o_type, ipd = NA)),
    class = "nma_data")

  # If trt_ref not specified, mark treatments factor as default, calculate
  # current reference trt
  if (is.null(trt_ref)) {
    trt_ref <- get_default_trt_ref(out)
    trt_sort <- order(forcats::fct_relevel(out$treatments, trt_ref))
    out$treatments <- .default(forcats::fct_relevel(out$treatments, trt_ref)[trt_sort])
    out$agd_contrast$.trt <- forcats::fct_relevel(out$agd_contrast$.trt, trt_ref)
    if (!is.null(.trtclass)) {
      class_ref <- as.character(out$classes[trt_sort[1]])
      if (!is.null(trtclass_original_levels))
        class_ref <- c(class_ref,
                       setdiff(intersect(trtclass_original_levels, levels(out$classes)),
                               class_ref))
      out$agd_contrast$.trtclass <- forcats::fct_relevel(out$agd_contrast$.trtclass, class_ref)
      out$classes <- forcats::fct_relevel(out$classes, class_ref)[trt_sort]
    }
  }

  # Add original_levels attributes (if not NULL)
  attr(out$treatments, "original_levels") <- trt_original_levels
  attr(out$studies, "original_levels") <- study_original_levels
  if (!is.null(.trtclass)) attr(out$classes, "original_levels") <- trtclass_original_levels

  return(out)
}


#' Set up aggregate survival data
#'
#' Set up a network containing aggregate survival data (AgD) in the form of
#' event/censoring times (e.g. reconstructed from digitized Kaplan-Meier curves)
#' and covariate summary statistics from each study. Multiple data sources may be
#' combined once created using [combine_network()].
#'
#' @template args-data_common
#' @template args-data_Surv
#' @param covariates data frame of covariate summary statistics for each study
#'   or study arm, with corresponding `study` and `trt` columns to match to
#'   those in `data`
#'
#' @return An object of class [nma_data]
#' @export
#'
#' @template args-details_trt_ref
#' @template args-details_mutate
#'
#' @seealso [set_ipd()] for individual patient data, [set_agd_contrast()] for
#'   contrast-based aggregate data, and [combine_network()] for combining
#'   several data sources in one network.
#' @template seealso_nma_data
#' @examples
#' ## Newly diagnosed multiple myeloma
#'
#' head(ndmm_agd)  # Reconstructed Kaplan-Meier data
#' ndmm_agd_covs   # Summary covariate information on each arm
#'
#' set_agd_surv(ndmm_agd,
#'              study = studyf,
#'              trt = trtf,
#'              Surv = Surv(eventtime, status),
#'              covariates = ndmm_agd_covs)
#'
set_agd_surv <- function(data,
                         study,
                         trt,
                         Surv,
                         covariates = NULL,
                         trt_ref = NULL,
                         trt_class = NULL) {

  # Check data is data frame
  if (!inherits(data, "data.frame")) abort("Argument `data` should be a data frame")
  if (nrow(data) == 0) {
    return(
      structure(
        list(agd_arm = NULL,
             agd_contrast = NULL,
             ipd = NULL,
             treatments = NULL,
             classes = NULL,
             studies = NULL),
        class = "nma_data")
    )
  }

  if (!is.null(covariates)) {
    if (!inherits(covariates, "data.frame")) abort("Argument `covariates` should be a data frame")
  }

  # Pull study and treatment columns
  if (missing(study)) abort("Specify `study`")
  .study <- pull_non_null(data, enquo(study))
  if (is.null(.study)) abort("`study` cannot be NULL")
  check_study(.study)

  if (is.factor(.study)) {
    study_original_levels <- levels(.study)
    .study <- forcats::fct_drop(.study)
  } else {
    study_original_levels <- NULL
  }

  if (missing(trt)) abort("Specify `trt`")
  .trt <- pull_non_null(data, enquo(trt))
  if (is.null(.trt)) abort("`trt` cannot be NULL")
  check_trt(.trt)

  if (is.factor(.trt)) {
    trt_original_levels <- levels(.trt)
    .trt <- forcats::fct_drop(.trt)
  } else {
    trt_original_levels <- NULL
  }

  # Check for single-arm studies
  single_arm_studies <- tibble::tibble(.study, .trt) %>%
    dplyr::distinct(.data$.study, .data$.trt) %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::filter(dplyr::n() == 1) %>%
    dplyr::pull(.data$.study)

  if (length(single_arm_studies)) {
    inform(glue::glue("Single-arm stud{if (length(single_arm_studies) > 1) 'ies' else 'y'} present in the network: ",
                      glue::glue_collapse(glue::double_quote(as.character(single_arm_studies)), sep = ", ", last = " and "), "."))
  }

  # Treatment classes
  .trtclass <- pull_non_null(data, enquo(trt_class))
  if (!is.null(.trtclass)) {
    check_trt_class(.trtclass, .trt)

    if (is.factor(.trtclass)) {
      trtclass_original_levels <- levels(.trtclass)
      .trtclass <- forcats::fct_drop(.trtclass)
    } else {
      trtclass_original_levels <- NULL
    }
  }

  if (!is.null(trt_ref) && length(trt_ref) > 1) abort("`trt_ref` must be length 1.")

  # Pull and check outcomes
  .Surv <- pull_non_null(data, enquo(Surv))

  check_outcome_survival(.Surv)

  o_type <- get_outcome_type(y = NULL, se = NULL,
                             r = NULL, n = NULL, E = NULL,
                             Surv = .Surv)

  # Create tibble in standard format
  d <- tibble::tibble(
    .study = nfactor(.study),
    .trt = nfactor(.trt)
  )

  if (!is.null(trt_ref)) {
    trt_ref <- as.character(trt_ref)
    lvls_trt <- levels(d$.trt)
    if (! trt_ref %in% lvls_trt)
      abort(sprintf("`trt_ref` does not match a treatment in the data.\nSuitable values are: %s",
                    ifelse(length(lvls_trt) <= 5,
                           paste0(lvls_trt, collapse = ", "),
                           paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    d$.trt <- forcats::fct_relevel(d$.trt, trt_ref)
  }

  if (!is.null(.trtclass)) {
    d <- tibble::add_column(d, .trtclass = nfactor(.trtclass))
    class_lookup <- d %>%
      dplyr::distinct(.data$.trt, .data$.trtclass) %>%
      dplyr::arrange(.data$.trt)
    class_ref <- as.character(class_lookup[[1, ".trtclass"]])
    d$.trtclass <- forcats::fct_relevel(d$.trtclass, class_ref)
    classes <- forcats::fct_relevel(nfactor(class_lookup$.trtclass), class_ref)
  } else {
    classes <- NULL
  }

  d <- tibble::add_column(d, .Surv = .Surv)

  # Add in sample size
  d <- dplyr::group_by(d, .data$.study, .data$.trt) %>%
    dplyr::mutate(.sample_size = dplyr::n()) %>%
    dplyr::ungroup()

  # Bind in original data
  drop_reserved <- setdiff(colnames(data), colnames(d))
  d <- dplyr::bind_cols(d, data[, drop_reserved, drop = FALSE])

  # Drop original study and treatment columns
  d <- drop_original(d, data, enquo(study))
  d <- drop_original(d, data, enquo(trt))
  if (!is.null(.trtclass)) d <- drop_original(d, data, enquo(trt_class))

  # Nest survival data, keeping original data nested for later too
  d <- dplyr::group_by(d, dplyr::across(dplyr::any_of(c(".study", ".trt", ".trtclass", ".sample_size")))) %>%
    tidyr::nest(.Surv = ".Surv",
                .data_orig = !dplyr::any_of(c(".study", ".trt", ".trtclass", ".sample_size", ".Surv"))) %>%
    dplyr::ungroup()

  # Join covariate details
  if (!is.null(covariates)) {
    covariates <- dplyr::ungroup(covariates)

    .cov_study <- pull_non_null(covariates, enquo(study))
    if (is.null(.cov_study)) abort("`study` cannot be NULL")
    check_study(.cov_study)
    .cov_study <- forcats::fct_drop(nfactor(.cov_study))

    .cov_trt <- pull_non_null(covariates, enquo(trt))
    if (is.null(.cov_trt)) abort("`trt` cannot be NULL")
    check_trt(.cov_trt)
    .cov_trt <- forcats::fct_drop(nfactor(.cov_trt))

    if (!is.null(trt_ref)) {
      .cov_trt <- forcats::fct_relevel(.cov_trt, trt_ref)
    }

    covs <- dplyr::mutate(covariates, .study = .cov_study, .trt = .cov_trt)
    covs <- drop_original(covs, data, enquo(study))
    covs <- drop_original(covs, data, enquo(trt))

    d <- tryCatch(
          dplyr::inner_join(d, covs, by = c(".study", ".trt"),
                            multiple = "first",
                            unmatched = c("error", "drop")),
          error = function(e) abort("Not all study arms in `data` have matching rows in `covariates`", parent = e))

    # Re-drop factors, in case extra unneeded trt/study rows included in covariate data
    d$.trt <- forcats::fct_drop(d$.trt)
    d$.study <- forcats::fct_drop(d$.study)
  }

  # Produce nma_data object
  out <- structure(
    list(agd_arm = d,
         agd_contrast = NULL,
         ipd = NULL,
         treatments = forcats::fct_unique(d$.trt),
         classes = classes,
         studies = forcats::fct_unique(d$.study),
         outcome = list(agd_arm = o_type, agd_contrast = NA, ipd = NA)),
    class = "nma_data")

  # If trt_ref not specified, mark treatments factor as default, calculate
  # current reference trt
  if (is.null(trt_ref)) {
    trt_ref <- get_default_trt_ref(out)
    trt_sort <- order(forcats::fct_relevel(out$treatments, trt_ref))
    out$treatments <- .default(forcats::fct_relevel(out$treatments, trt_ref)[trt_sort])
    out$agd_arm$.trt <- forcats::fct_relevel(out$agd_arm$.trt, trt_ref)
    if (!is.null(.trtclass)) {
      class_ref <- as.character(out$classes[trt_sort[1]])
      if (!is.null(trtclass_original_levels))
        class_ref <- c(class_ref,
                       setdiff(intersect(trtclass_original_levels, levels(out$classes)),
                               class_ref))
      out$agd_arm$.trtclass <- forcats::fct_relevel(out$agd_arm$.trtclass, class_ref)
      out$classes <- forcats::fct_relevel(out$classes, class_ref)[trt_sort]
    }
  }

  # Add original_levels attributes (if not NULL)
  attr(out$treatments, "original_levels") <- trt_original_levels
  attr(out$studies, "original_levels") <- study_original_levels
  if (!is.null(.trtclass)) attr(out$classes, "original_levels") <- trtclass_original_levels

  return(out)

}


#' Combine multiple data sources into one network
#'
#' Multiple data sources created using [set_ipd()], [set_agd_arm()], or
#' [set_agd_contrast()] can be combined into a single network for analysis.
#'
#' @param ... multiple data sources, as defined using the `set_*` functions
#' @param trt_ref reference treatment for the entire network, as a string (or
#'   coerced as such) referring to the levels of the treatment factor variable
#'
#' @return An object of class [nma_data]
#' @export
#'
#' @seealso [set_ipd()], [set_agd_arm()], and [set_agd_contrast()] for defining
#'   different data sources.
#' @template seealso_nma_data
#'
#' @examples ## Parkinson's - combining contrast- and arm-based data
#' studies <- parkinsons$studyn
#' (parkinsons_arm <- parkinsons[studies %in% 1:3, ])
#' (parkinsons_contr <- parkinsons[studies %in% 4:7, ])
#'
#' park_arm_net <- set_agd_arm(parkinsons_arm,
#'                             study = studyn,
#'                             trt = trtn,
#'                             y = y,
#'                             se = se,
#'                             sample_size = n)
#'
#' park_contr_net <- set_agd_contrast(parkinsons_contr,
#'                                    study = studyn,
#'                                    trt = trtn,
#'                                    y = diff,
#'                                    se = se_diff,
#'                                    sample_size = n)
#'
#' park_net <- combine_network(park_arm_net, park_contr_net)
#'
#' # Print network details
#' park_net
#'
#' # Plot network
#' plot(park_net, weight_edges = TRUE, weight_nodes = TRUE)
#'
#' @examples ## Plaque Psoriasis - combining IPD and AgD in a network
#' @template ex_plaque_psoriasis_network
#' @examples
#'
#' # Plot network
#' plot(pso_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE)
combine_network <- function(..., trt_ref) {
  s <- list(...)

  # Check that arguments all inherit from nma_data class
  if (!purrr::every(s, inherits, what = "nma_data")) {
    abort("Expecting to combine objects of class `nma_data`, created using set_* functions")
  }

  # Combine treatment code factor
  trts <- stringr::str_sort(forcats::lvls_union(purrr::map(s, "treatments")), numeric = TRUE)

  # Use original factor order (possibly with more levels) if available
  trt_original_levels <- purrr::map(purrr::map(s, "treatments"), attr, which = "original_levels")
  if (!any(purrr::map_lgl(trt_original_levels, is.null)) &&
      all(purrr::map_lgl(trt_original_levels, ~identical(., trt_original_levels[[1]])))) {

    trt_original_levels <- trt_original_levels[[1]]
    trts <- intersect(trt_original_levels, trts)
  } else {
    trt_original_levels <- NULL
  }

  if (!missing(trt_ref)) {
    if (! trt_ref %in% trts) {
      abort(sprintf("`trt_ref` does not match a treatment in the network.\nSuitable values are: %s",
                      ifelse(length(trts) <= 5,
                             paste0(trts, collapse = ", "),
                             paste0(paste0(trts[1:5], collapse = ", "), ", ..."))))
    }
    trts <- c(trt_ref, setdiff(trts, trt_ref))
  }

  # Combine classes factor
  has_classes <- purrr::map_lgl(purrr::map(s, "classes"), ~!is.null(.))

  if (all(has_classes)) {
    class_lookup <- tibble::tibble(.trt = forcats::fct_c(!!! purrr::map(s, "treatments")),
                                   .trtclass = forcats::fct_c(!!! purrr::map(s, "classes"))) %>%
      dplyr::mutate(.trt = forcats::fct_relevel(.data$.trt, trts)) %>%
      dplyr::distinct(.data$.trt, .data$.trtclass) %>%
      dplyr::arrange(.data$.trt)

    check_trt_class(class_lookup$.trtclass, class_lookup$.trt)

    class_lvls <- stringr::str_sort(levels(class_lookup$.trtclass), numeric = TRUE)

    # Use original factor order (possibly with more levels) if available
    class_original_levels <- purrr::map(purrr::map(s, "classes"), attr, which = "original_levels")
    if (!any(purrr::map_lgl(class_original_levels, is.null)) &&
        all(purrr::map_lgl(class_original_levels, ~identical(., class_original_levels[[1]])))) {

      class_original_levels <- class_original_levels[[1]]
      class_lvls <- intersect(class_original_levels, class_lvls)
    } else {
      class_original_levels <- NULL
    }

    class_ref <- as.character(class_lookup[[1, ".trtclass"]])
    class_lvls <- c(class_ref, setdiff(class_lvls, class_ref))

    class_lookup$.trtclass <- forcats::fct_relevel(class_lookup$.trtclass, class_lvls)

    classes <- class_lookup$.trtclass
  } else if (any(has_classes)) {
    warn("Not all data sources have defined treatment classes. Removing treatment class information.")
    classes <- NULL
  } else {
    classes <- NULL
  }

  # Check that no studies are duplicated between data sources
  all_studs <- purrr::flatten_chr(purrr::map(s, ~levels(.$studies)))
  if (anyDuplicated(all_studs)) {
    abort(sprintf("Studies with same label found in multiple data sources: %s",
                  paste0(unique(all_studs[duplicated(all_studs)]), collapse = ", ")))
  }

  # Combine study code factor
  studs <- stringr::str_sort(forcats::lvls_union(purrr::map(s, "studies")), numeric = TRUE)

  # Use original factor order (possibly with more levels) if available
  study_original_levels <- purrr::map(purrr::map(s, "studies"), attr, which = "original_levels")
  if (!any(purrr::map_lgl(study_original_levels, is.null)) &&
      all(purrr::map_lgl(study_original_levels, ~identical(., study_original_levels[[1]])))) {

    study_original_levels <- study_original_levels[[1]]
    studs <- intersect(study_original_levels, studs)
  } else {
    study_original_levels <- NULL
  }

  # Get ipd
  ipd <- purrr::map(s, "ipd")
  if (!rlang::is_empty(ipd)) {
    for (j in 1:length(ipd)) {
      if (rlang::is_empty(ipd[[j]])) next
      ipd[[j]]$.trt <- forcats::lvls_expand(ipd[[j]]$.trt, trts)
      ipd[[j]]$.study <- forcats::lvls_expand(ipd[[j]]$.study, studs)
      if (!is.null(classes)) ipd[[j]]$.trtclass <- forcats::lvls_expand(ipd[[j]]$.trtclass, class_lvls)
    }
  }

  # Get agd_arm
  agd_arm <- purrr::map(s, "agd_arm")
  if (!rlang::is_empty(agd_arm)) {
    for (j in 1:length(agd_arm)) {
      if (rlang::is_empty(agd_arm[[j]])) next
      agd_arm[[j]]$.trt <- forcats::lvls_expand(agd_arm[[j]]$.trt, trts)
      agd_arm[[j]]$.study <- forcats::lvls_expand(agd_arm[[j]]$.study, studs)
      if (!is.null(classes))
        agd_arm[[j]]$.trtclass <- forcats::lvls_expand(agd_arm[[j]]$.trtclass, class_lvls)
    }
  }

  # Get agd_contrast
  agd_contrast <- purrr::map(s, "agd_contrast")
  if (!rlang::is_empty(agd_contrast)) {
    for (j in 1:length(agd_contrast)) {
      if (rlang::is_empty(agd_contrast[[j]])) next
      agd_contrast[[j]]$.trt <- forcats::lvls_expand(agd_contrast[[j]]$.trt, trts)
      agd_contrast[[j]]$.study <- forcats::lvls_expand(agd_contrast[[j]]$.study, studs)
      if (!is.null(classes))
        agd_contrast[[j]]$.trtclass <- forcats::lvls_expand(agd_contrast[[j]]$.trtclass, class_lvls)
    }
  }

  # Get outcome type
  o_ipd <- unique(purrr::map_chr(purrr::map(s, "outcome"), "ipd"))
  o_ipd <- o_ipd[!is.na(o_ipd)]
  if (length(o_ipd) > 1) abort("Multiple outcome types present in IPD.")
  if (length(o_ipd) == 0) o_ipd <- NA

  o_agd_arm <- unique(purrr::map_chr(purrr::map(s, "outcome"), "agd_arm"))
  o_agd_arm <- o_agd_arm[!is.na(o_agd_arm)]
  if (length(o_agd_arm) > 1) abort("Multiple outcome types present in AgD (arm-based).")
  if (length(o_agd_arm) == 0) o_agd_arm <- NA

  o_agd_contrast <- unique(purrr::map_chr(purrr::map(s, "outcome"), "agd_contrast"))
  o_agd_contrast <- o_agd_contrast[!is.na(o_agd_contrast)]
  if (length(o_agd_contrast) > 1) abort("Multiple outcome types present in AgD (contrast-based).")
  if (length(o_agd_contrast) == 0) o_agd_contrast <- NA

  outcome <- list(agd_arm = o_agd_arm,
                  agd_contrast = o_agd_contrast,
                  ipd = o_ipd)

  # Check outcome combination
  check_outcome_combination(outcome)

  # Additional checks when combining multinomial outcomes
  if (o_ipd %in% c("ordered", "competing"))
    check_multi_combine(purrr::map(ipd, ".r"))
  if (o_agd_arm %in% c("ordered", "competing"))
    check_multi_combine(purrr::map(agd_arm, ".r"))
  if (o_ipd %in% c("ordered", "competing") && o_agd_arm %in% c("ordered", "competing"))
    check_multi_combine(purrr::map(c(ipd, agd_arm), ".r"))

  # Check integration points
  if (any(purrr::map_lgl(s, inherits, what = "mlnmr_data")) &&
      all(purrr::map_lgl(s, inherits, what = "mlnmr_data") |
        (purrr::map_lgl(s, has_ipd) & !purrr::map_lgl(s, has_agd_arm) & !purrr::map_lgl(s, has_agd_contrast)))) {

    has_int <- TRUE

    s_int <- s[purrr::map_lgl(s, inherits, what = "mlnmr_data")]

    # Check int_names
    l_int_names <- purrr::map(s_int, "int_names")
    if (!all(purrr::map_lgl(l_int_names, ~identical(., l_int_names[[1]])))) {
      abort("Cannot combine AgD sources with different sets of covariates with integration points.")
    } else {
      int_names <- l_int_names[[1]]
    }

    # Check n_int
    l_n_int <- purrr::map_dbl(s_int, "n_int")
    if (!all(l_n_int == l_n_int[1])) {
      abort("Cannot combine AgD sources with different numbers of integration points.")
    } else {
      n_int <- l_n_int[[1]]
    }

  } else if (any(purrr::map_lgl(s, inherits, what = "mlnmr_data"))) {
    message("Dropping integration points from combined network, some AgD sources do not have integration points.")
    has_int <- FALSE
  } else {
    has_int <- FALSE
  }

  # Produce nma_data object
  ipd <- dplyr::bind_rows(ipd)
  agd_arm <- dplyr::bind_rows(agd_arm)
  agd_contrast <- dplyr::bind_rows(agd_contrast)

  out <- structure(
    list(agd_arm = agd_arm,
         agd_contrast = agd_contrast,
         ipd = ipd,
         treatments = factor(trts, levels = trts),
         classes = classes,
         studies = factor(studs, levels = studs),
         outcome = outcome),
    class = "nma_data")

  # Integration data setup
  if (has_int) {
    out$n_int <- n_int
    out$int_names <- int_names

    class(out) <- c("mlnmr_data", class(out))
  }

  # If trt_ref not specified, mark treatments factor as default, calculate
  # current reference trt
  if (missing(trt_ref)) {
    trt_ref <- get_default_trt_ref(out)
    trt_sort <- order(forcats::fct_relevel(out$treatments, trt_ref))
    out$treatments <- .default(forcats::fct_relevel(out$treatments, trt_ref)[trt_sort])

    if (has_ipd(out))
      out$ipd$.trt <- forcats::fct_relevel(out$ipd$.trt, trt_ref)
    if (has_agd_arm(out))
      out$agd_arm$.trt <- forcats::fct_relevel(out$agd_arm$.trt, trt_ref)
    if (has_agd_contrast(out))
      out$agd_contrast$.trt <- forcats::fct_relevel(out$agd_contrast$.trt, trt_ref)

    if (!is.null(classes)) {
      class_ref <- as.character(out$classes[trt_sort[1]])
      out$classes <- forcats::fct_relevel(out$classes, class_ref)[trt_sort]

      if (has_ipd(out))
        out$ipd$.trtclass <- forcats::fct_relevel(out$ipd$.trtclass, class_ref)
      if (has_agd_arm(out))
        out$agd_arm$.trtclass <- forcats::fct_relevel(out$agd_arm$.trtclass, class_ref)
      if (has_agd_contrast(out))
        out$agd_contrast$.trtclass <- forcats::fct_relevel(out$agd_contrast$.trtclass, class_ref)
    }
  }

  # Add original_levels attributes (if not NULL)
  attr(out$treatments, "original_levels") <- trt_original_levels
  attr(out$studies, "original_levels") <- study_original_levels
  if (!is.null(classes)) attr(out$classes, "original_levels") <- class_original_levels

  return(out)
}

#' Multinomial outcome data
#'
#' This function aids the specification of multinomial outcome data when setting
#' up a network with [set_agd_arm()] or [set_ipd()]. It takes a set of columns
#' (or, more generally, numeric vectors of the same length) of outcome counts in
#' each category, and binds these together to produce a matrix.
#'
#' @param ... Two or more numeric columns (or vectors) of category counts.
#'   Argument names (optional) will be used to label the categories.
#' @param inclusive Logical, are ordered category counts inclusive (`TRUE`) or
#'   exclusive (`FALSE`)? Default `FALSE`. Only used when `type = "ordered"`.
#'   See details.
#' @param type String, indicating whether categories are `"ordered"` or
#'   `"competing"`. Currently only ordered categorical outcomes are supported by
#'   the modelling functions in this package.
#'
#' @details When specifying ordered categorical counts, these can either be
#'   given as *exclusive* counts (`inclusive = FALSE`, the default) where
#'   individuals are only counted in the highest category they achieve, or
#'   *inclusive* counts (`inclusive = TRUE`) where individuals are counted in
#'   every category up to and including the highest category achieved.
#'   (Competing outcomes, by nature, are always specified as exclusive counts.)
#'
#'   `NA` values can be used to indicate categories/cutpoints that were not
#'   measured.
#'
#' @return A matrix of (exclusive) category counts
#' @export
#'
#' @examples
#' # These two data sets specify the same ordered categorical data for outcomes
#' # r0 < r1 < r2, but the first uses the "inclusive" format and the second the
#' # "exclusive" format.
#' df_inclusive <- tibble::tribble(~r0, ~r1, ~r2,
#'                                 1, 1, 1,
#'                                 5, 4, 1,
#'                                 5, 2, 2,
#'                                 10, 5, 0,
#'                                 5, 5, 0,
#'                                 7, NA, 6,   # Achieved r2 or not (no r1)
#'                                 10, 4, NA)  # Achieved r1 or not (no r2)
#'
#' df_exclusive <- tibble::tribble(~r0, ~r1, ~r2,
#'                                 0, 0, 1,
#'                                 1, 3, 1,
#'                                 3, 0, 2,
#'                                 5, 5, 0,
#'                                 0, 5, 0,
#'                                 1, NA, 6,   # Achieved r2 or not (no r1)
#'                                 6, 4, NA)   # Achieved r1 or not (no r2)
#'
#' (r_inclusive <- with(df_inclusive, multi(r0, r1, r2, inclusive = TRUE)))
#' (r_exclusive <- with(df_exclusive, multi(r0, r1, r2, inclusive = FALSE)))
#'
#' # Counts are always stored in exclusive format
#' stopifnot(isTRUE(all.equal(r_inclusive, r_exclusive)))
#'
#'
#' ## HTA Plaque Psoriasis
#' library(dplyr)
#'
#' # Ordered outcomes here are given as "exclusive" counts
#' head(hta_psoriasis)
#'
#' # Calculate lowest category count (failure to achieve PASI 50)
#' pso_dat <- hta_psoriasis %>%
#'   mutate(`PASI<50` = sample_size - rowSums(cbind(PASI50, PASI75, PASI90), na.rm = TRUE))
#'
#' # Set up network
#' pso_net <- set_agd_arm(pso_dat,
#'                        study = paste(studyc, year),
#'                        trt = trtc,
#'                        r = multi(`PASI<50`, PASI50, PASI75, PASI90,
#'                                  inclusive = FALSE,
#'                                  type = "ordered"))
#'
#' pso_net
#'
multi <- function(..., inclusive = FALSE, type = c("ordered", "competing")) {
  # Argument checks
  if (!rlang::is_bool(inclusive)) abort("`inclusive` must be a logical value TRUE/FALSE")
  type <- rlang::arg_match(type)

  if (type == "competing" && inclusive) {
    warn("Ignoring inclusive = TRUE, competing outcomes are always given by exclusive counts.")
    inclusive <- FALSE
  }

  # Collect dots
  # We take this route via quosures to simplify automatic naming
  q_dots <- rlang::enquos(..., .named = TRUE)

  if (length(q_dots) < 2) abort("At least 2 outcomes must be specified in `...`")

  if (anyDuplicated(names(q_dots))) {
    dups <- unique(names(q_dots)[duplicated(names(q_dots))])
    abort(glue::glue("Duplicate outcome category labels ",
                     glue::glue_collapse(glue::double_quote(dups), sep = ", ", last = " and "),
                     "."))
  }

  # Construct outcome matrix
  dots <- purrr::map(q_dots, rlang::eval_tidy)

  # Input vectors must be same length (or length 1 for recycling)
  dots_lengths <- lengths(dots)
  if (length(unique(dots_lengths[dots_lengths > 1])) > 1) abort("Input vectors in `...` must be the same length (or length 1).")

  out <- do.call(cbind, dots)

  # Check counts
  if (!is.numeric(out)) abort("Categorical outcome count must be numeric")
  if (any(is.nan(out))) abort("Categorical outcome count cannot be NaN")
  if (any(is.infinite(out))) abort("Categorical outcome count cannot be Inf")
  if (!rlang::is_integerish(out)) abort("Categorical outcome count must be integer-valued")
  if (any(out < 0, na.rm = TRUE)) abort("Categorical outcome count must be non-negative")

  if (type == "ordered") {
    if (any(c1_na <- is.na(out[, 1]))) {
      abort(glue::glue("Ordered outcome counts cannot be missing in the lowest category.\n",
                       "NAs found in row{if (sum(c1_na) > 1) 's' else ''} ",
                       glue::glue_collapse(which(c1_na), sep = ", ", width = 30, last = " and "), "."))
    }
  }


  if (any(only1 <- apply(out, 1, function(x) sum(!is.na(x))) < 2)) {
    abort(glue::glue("Outcome counts must be present for at least 2 categories.\n",
                     "Issues in row{if (sum(only1) > 1) 's' else ''} ",
                     glue::glue_collapse(which(only1), sep = ", ", width = 30, last = " and "), "."))
  }

  if (inclusive) {
    if (any(non_decreasing <- apply(out, 1, function(x) max(diff(x[!is.na(x)]))) > 0)) {
      abort(glue::glue("Inclusive ordered outcome counts must be decreasing or constant across increasing categories.\n",
                       "Increasing counts found in row{if (sum(non_decreasing) > 1) 's' else ''} ",
                       glue::glue_collapse(which(non_decreasing), sep = ", ", width = 30, last = " and "), "."))
    }

    # Store internally as exclusive counts
    ncat <- ncol(out)
    for (i in 1:nrow(out)) {
      j <- 1L
      k <- 2L
      while (k <= ncat) {
        if (is.na(out[i, k])) {
          k <- k + 1L
        } else {
          out[i, j] <- out[i, j] - out[i, k]
          j <- k
          k <- k + 1L
        }
      }
    }
  }

  class(out) <- c(switch(type,
                         ordered = "multi_ordered",
                         competing = "multi_competing"),
                  class(out))
  return(out)
}

#' Pull non-null variables from data
#'
#' Allows mutate syntax, e.g. factor(study) or sqrt(var)
#' Uses pull for variable names as strings or integer positions
#'
#' @param data data frame
#' @param var quosure (possibly NULL) for variable to pull
#'
#' @noRd
pull_non_null <- function(data, var) {
  var_null <- rlang::quo_is_missing(var) | rlang::quo_is_null(var)
  if (!var_null) {
    if (rlang::is_symbolic(rlang::quo_get_expr(var))) return(dplyr::pull(dplyr::transmute(data, {{ var }})))
    else return(dplyr::pull(data, {{ var }}))
  }
  else return(NULL)
}

#' Drop variables from original data set
#'
#' Allows for combination of pull and mutate semantics - doesn't drop mutated
#' results, handles integer column references
#'
#' @param data data frame after transformation
#' @param orig_data original data frame
#' @param var quosure for variable to drop
#'
#' @noRd
drop_original <- function(data, orig_data, var) {
  e_var <- rlang::quo_get_expr(var)
  if (rlang::is_symbol(e_var)) {
    # Character/bare column name, drop unless protected name (starts with dot)
    if (stringr::str_starts(rlang::as_name(e_var), "\\.")) return(data)
    else return(dplyr::select(data, - {{ var }}))
  } else if (rlang::is_integerish(e_var)) {
    # Integer column number, drop based on original location
    orig_var <- colnames(orig_data)[e_var]
    if (stringr::str_starts(orig_var, "\\.")) return(data)
    else return(dplyr::select(data, - {{ orig_var }}))
  } else {
    # Mutate expression, don't drop
    return(data)
  }
}

#' Get outcome type
#'
#' Determines outcome type based on which inputs are NA
#'
#' @noRd
get_outcome_type <- function(y, se, r, n, E, Surv) {
  o <- c()
  if (!is.null(y)) o <- c(o, "continuous")
  if (!is.null(r)) {
    if (inherits(r, "multi_ordered")) o <- c(o, "ordered")
    if (inherits(r, "multi_competing")) o <- c(o, "competing")
    if (!is.null(E)) o <- c(o, "rate")
    if (!is.null(n)) o <- c(o, "count")
    if (!inherits(r, c("multi_ordered", "multi_competing")) && is.null(n) && is.null(E)) o <- c(o, "binary")
  }
  if (!is.null(Surv)) o <- c(o, "survival")
  if (length(o) == 0) abort("Please specify one and only one outcome.")
  if (length(o) > 1) abort(glue::glue("Please specify one and only one outcome, instead of ",
                                      glue::glue_collapse(o, sep = ", ", last = " and "), "."))

  return(o)
}

#' Check continuous outcomes
#'
#' @param y vector
#' @param se vector
#' @param with_se continuous outcome with SE?
#' @param append text to append to error message
#'
#' @noRd
check_outcome_continuous <- function(y, se = NULL, with_se = TRUE, append = NULL) {
  null_y <- is.null(y)
  null_se <- is.null(se)

  if (with_se) {
    if (!null_y && !null_se) {
      if (rlang::is_list(y) || !is.null(dim(y)))
        abort("Continuous outcome `y` must be a regular column (not a list or matrix column)")
      if (rlang::is_list(se) || !is.null(dim(se)))
        abort("Standard error `se` must be a regular column (not a list or matrix column)")
      if (!is.numeric(y)) abort(paste0("Continuous outcome `y` must be numeric", append))
      if (!is.numeric(se)) abort(paste0("Standard error `se` must be numeric", append))
      if (any(is.nan(se))) abort(paste0("Standard error `se` cannot be NaN", append))
      if (any(is.na(y))) abort(paste0("Continuous outcome `y` contains missing values", append))
      if (any(is.na(se))) abort(paste0("Standard error `se` contains missing values", append))
      if (any(is.infinite(se))) abort(paste0("Standard error `se` cannot be infinite", append))
      if (any(se <= 0)) abort(paste0("Standard errors must be positive", append))
    } else {
      if (!null_y) abort(paste0("Specify standard error `se` for continuous outcome `y`", append))
      if (!null_se) abort(paste0("Specify continuous outcome `y`", append))
    }
    invisible(list(y = y, se = se))
  } else {
    if (!null_y) {
      if (rlang::is_list(y) || !is.null(dim(y)))
        abort("Continuous outcome `y` must be a regular column (not a list or matrix column)")
      if (any(is.na(y))) abort(paste0("Continuous outcome `y` contains missing values", append))
      if (!is.numeric(y)) abort(paste0("Continuous outcome `y` must be numeric", append))
    }
    invisible(list(y = y))
  }
}

#' Check count outcomes
#'
#' @param r vector
#' @param n vector
#' @param E vector
#'
#' @noRd
check_outcome_count <- function(r, n, E) {
  null_r <- is.null(r)
  null_n <- is.null(n)
  null_E <- is.null(E)

  if (!null_n) {
    if (rlang::is_list(n) || !is.null(dim(n)))
      abort("Denominator `n` must be a regular column (not a list or matrix column)")
    if (!is.numeric(n)) abort("Denominator `n` must be numeric")
    if (any(is.na(n))) abort("Denominator `n` contains missing values")
    if (any(n != trunc(n))) abort("Denominator `n` must be integer-valued")
    if (any(n <= 0)) abort("Denominator `n` must be greater than zero")
    if (null_r) abort("Specify outcome count `r`.")
  }

  if (!null_E) {
    if (rlang::is_list(E) || !is.null(dim(E)))
      abort("Time at risk `E` must be a regular column (not a list or matrix column)")
    if (!is.numeric(E)) abort("Time at risk `E` must be numeric")
    if (any(is.na(E))) abort("Time at risk `E` contains missing values")
    if (any(E <= 0)) abort("Time at risk `E` must be positive")
    if (null_r) abort("Specify outcome count `r`.")
  }

  if (!null_r) {
    if (rlang::is_list(r) || !is.null(dim(r)))
      abort("Outcome count `r` must be a regular column (not a list or matrix column)")
    if (null_n && null_E) abort("Specify denominator `n` (count outcome) or time at risk `E` (rate outcome)")
    if (!is.numeric(r)) abort("Outcome count `r` must be numeric")
    if (any(is.na(r))) abort("Outcome count `r` contains missing values")
    if (any(r != trunc(r))) abort("Outcome count `r` must be integer-valued")
    if (!null_n && any(n < r | r < 0)) abort("Count outcome `r` must be between 0 and `n`")
    if (!null_E && any(r < 0)) abort("Rate outcome count `r` must be non-negative")
  }

  invisible(list(r = r, n = n, E = E))
}

#' Check binary outcomes
#'
#' @param r vector
#' @param E vector
#'
#' @noRd
check_outcome_binary <- function(r, E) {
  null_r <- is.null(r)
  null_E <- is.null(E)

  if (!null_E) {
    if (null_r) {
      abort("Specify count `r` for rate outcome")
    } else {
      if (rlang::is_list(r) || !is.null(dim(r)))
        abort("Rate outcome count `r` must be a regular column (not a list or matrix column)")
      if (rlang::is_list(E) || !is.null(dim(E)))
        abort("Time at risk `E` must be a regular column (not a list or matrix column)")
      if (!is.numeric(E)) abort("Time at risk `E` must be numeric")
      if (any(is.na(E))) abort("Time at risk `E` contains missing values")
      if (any(E <= 0)) abort("Time at risk `E` must be positive")
      if (!is.numeric(r)) abort("Rate outcome count `r` must be numeric")
      if (any(is.na(r))) abort("Rate outcome count `r` contains missing values")
      if (any(r != trunc(r))) abort("Rate outcome count `r` must be non-negative integer")
      if (any(r < 0)) abort("Rate outcome count `r` must be non-negative integer")
    }
  } else if (!null_r) {
    if (rlang::is_list(r) || !is.null(dim(r)))
      abort("Binary outcome `r` must be a regular column (not a list or matrix column)")
    if (!is.numeric(r)) abort("Binary outcome `r` must be numeric")
    if (any(is.na(r))) abort("Binary outcome `r` contains missing values")
    if (any(! r %in% c(0, 1))) abort("Binary outcome `r` must equal 0 or 1")
  }

  invisible(list(r = r, E = E))
}

#' Check survival outcomes
#'
#' @param Surv vector
#'
#' @noRd
check_outcome_survival <- function(Surv) {
  if (!is.null(Surv)) {

    if (!survival::is.Surv(Surv)) abort("Survival outcome `Surv` must be a `Surv` object created using `Surv()`")

    stype <- attr(Surv, "type")
    allowed_stypes <- c("right", "left", "interval", "interval2", "counting")
    if (!stype %in% allowed_stypes)
      abort(glue::glue('Survival outcome `Surv` of type "{stype}" is not supported.\n',
                       "Supported types are ",
                       glue::glue_collapse(allowed_stypes, sep = ", ", last = " and "), "."))

    status <- Surv[, "status"]
    if (any(is.na(status))) abort("Survival outcome `Surv` contains missing event status values")
    if (!all(status %in% 0:3)) abort("Survival outcome `Surv` event status values must be 0, 1, 2, or 3")

    S <- get_Surv_data(Surv)
    if (any(is.na(S$time), is.na(S$start_time), is.na(S$delay_time))) abort("Survival outcome `Surv` contains missing times")
    if (any(is.infinite(S$time), is.infinite(S$start_time), is.infinite(S$delay_time))) abort("Survival outcome `Surv` contains infinite times")
    if (any(S$time <= 0)) abort("Survival outcome `Surv` must have strictly positive outcome times")
    if (any(S$start_time < 0, S$delay_time < 0)) abort("Survival outcome `Surv` must have non-negative start times")

  }

  invisible(list(Surv = Surv))
}

#' Check valid outcome combination across data sources
#'
#' @param outcomes outcome list, see nma_data-class
#'
#' @noRd
check_outcome_combination <- function(outcomes) {
  valid <- list(
    list(agd_arm = c("count", NA),
         agd_contrast = c("continuous", NA),
         ipd = c("binary", NA)),
    list(agd_arm = c("rate", NA),
         agd_contrast = c("continuous", NA),
         ipd = c("rate", NA)),
    list(agd_arm = c("continuous", NA),
         agd_contrast = c("continuous", NA),
         ipd = c("continuous", NA)),
    list(agd_arm = c("ordered", NA),
         agd_contrast = c("continuous", NA),
         ipd = c("ordered", NA)),
    list(agd_arm = c("survival", NA),
         agd_contrast = c("continuous", NA),
         ipd = c("survival", NA))
  )

  if (!any(purrr::map_lgl(valid,
                 ~all(c(outcomes$agd_arm %in% .$agd_arm,
                        outcomes$agd_contrast %in% .$agd_contrast,
                        outcomes$ipd %in% .$ipd))))) {
    rlang::abort(glue::glue("Combining ",
                     glue::glue_collapse(outcomes[!is.na(outcomes)], sep = ', ', last = ' and '),
                     " outcomes is not supported."))
  }
}

#' Check sample size
#'
#' @param sample_size vector
#'
#' @noRd
check_sample_size <- function(sample_size) {
  if (rlang::is_list(sample_size) || !is.null(dim(sample_size)))
    abort("Sample size `sample_size` must be a regular column (not a list or matrix column)")
  if (!is.numeric(sample_size))
    abort("Sample size `sample_size` must be numeric")
  if (any(is.nan(sample_size)))
    abort("Sample size `sample_size` cannot be NaN")
  if (any(is.na(sample_size)))
    abort("Sample size `sample_size` contains missing values")
  if (any(sample_size != trunc(sample_size)))
    abort("Sample size `sample_size` must be integer-valued")
  if (any(sample_size <= 0))
    abort("Sample size `sample_size` must be greater than zero")
  if (any(is.infinite(sample_size)))
    abort("Sample size `sample_size` cannot be infinite")
}

#' Check treatment column
#'
#' @param trt Treatment vector
#'
#' @noRd
check_trt <- function(trt) {
  if (any(is.na(trt)))
    abort("`trt` cannot contain missing values")
  if (rlang::is_list(trt) || !is.null(dim(trt)))
    abort("`trt` must be a regular column (not a list or matrix column)")
}

#' Check study column
#'
#' @param study Treatment vector
#'
#' @noRd
check_study <- function(study) {
  if (any(is.na(study)))
    abort("`study` cannot contain missing values")
  if (rlang::is_list(study) || !is.null(dim(study)))
    abort("`study` must be a regular column (not a list or matrix column)")
}


#' Check treatment class coding
#'
#' @param trt_class Class vector
#' @param trt Treatment vector
#'
#' @noRd
check_trt_class <- function(trt_class, trt) {
  if (rlang::is_list(trt_class) || !is.null(dim(trt_class)))
    abort("`trt_class` must be a regular column (not a list or matrix column)")
  if (any(is.na(trt)))
    abort("`trt` cannot contain missing values")
  if (any(is.na(trt_class)))
    abort("`trt_class` cannot contain missing values")
  if (anyDuplicated(unique(cbind(trt, trt_class))[, "trt"]))
    abort("Treatment present in more than one class (check `trt` and `trt_class`)")
}

#' Checks for combining multinomial outcomes
#'
#' @param x List containing class `multi_*` outcome objects or the unclassed
#'   matrices within
#'
#' @noRd
check_multi_combine <- function(x) {
  # Remove any NULL elements
  x <- x[!purrr::map_lgl(x, is.null)]

  is_ordered <- purrr::map_lgl(x, ~inherits(., "multi_ordered"))
  is_competing <- purrr::map_lgl(x, ~inherits(., "multi_competing"))
  if (any(is_ordered) && any(is_competing))
    abort("Cannot combine ordered and competing multinomial outcomes.")

  x_u <- purrr::map(x, unclass)
  n_cat <- purrr::map_int(x_u, ncol)
  if (any(n_cat != n_cat[1]))
    abort("Cannot combine multinomial outcomes with different numbers of categories.")

  l_cat <- purrr::map(x_u, colnames)
  if (any(purrr::map_lgl(l_cat, ~any(. != l_cat[[1]]))))
    abort("Cannot combine multinomial outcomes with different category labels.")
}

#' Check for IPD and AgD in network
#'
#' @param network nma_data object
#'
#' @return logical TRUE/FALSE
#' @noRd
has_ipd <- function(network) {
  if (!inherits(network, "nma_data")) abort("Not nma_data object.")
  return(!rlang::is_empty(network$ipd))
}

has_agd_arm <- function(network) {
  if (!inherits(network, "nma_data")) abort("Not nma_data object.")
  return(!rlang::is_empty(network$agd_arm))
}

has_agd_contrast <- function(network) {
  if (!inherits(network, "nma_data")) abort("Not nma_data object.")
  return(!rlang::is_empty(network$agd_contrast))
}

#' Check whether AgD sample size columns are available
#'
#' @param network nma_data object
#'
#' @return logical TRUE/FALSE
#' @noRd
has_agd_sample_size <- function(network) {
  if (!inherits(network, "nma_data")) abort("Not nma_data object.")
  ss_a <- !has_agd_arm(network) || tibble::has_name(network$agd_arm, ".sample_size")
  ss_c <- !has_agd_contrast(network) || tibble::has_name(network$agd_contrast, ".sample_size")
  return(ss_a && ss_c)
}

#' Natural-order factors
#'
#' Produces factors with levels in natural sort order (i.e. 1 5 10 not 1 10 5)
#'
#' @noRd
nfactor <- function(x, ..., numeric = TRUE, resort = FALSE) {
  if (is.factor(x) && !resort) {
    return(x)
  } else {
    return(factor(x, levels = stringr::str_sort(unique(x), numeric = numeric), ...))
  }
}

#' Calculate baseline risk
#'
#' @param network An `nma_data` object.
#' @param link The name of the link function to use.
#'
#' @return The baseline risk as a single numeric value.
#' @noRd
calculate_baseline_risk <- function(network, link) {
  if (!inherits(network, "nma_data")) abort("Not nma_data object.")

  likelihood <- check_likelihood(NULL, network$outcome)
  ref_trt <- levels(network$treatments)[1L]

  if (likelihood %in% unlist(valid_lhood[c("binary", "count")])) {

    ref_dat <- dplyr::bind_rows(
      if (has_ipd(network)) dplyr::group_by(network$ipd, .data$.study, .data$.trt) %>%
        dplyr::summarise(.r = sum(.data$.r), .n = dplyr::n()),
      if (has_agd_arm(network)) dplyr::select(network$agd_arm, ".study", ".trt", ".r", ".n")
    ) %>%
      dplyr::filter(.data$.trt == ref_trt) %>%
      dplyr::mutate(.n = ifelse(.data$.r == 0, .data$.n + 0.5, .data$.n),
                    .r = ifelse(.data$.r == 0, 0.5, .data$.r))

    bl <- stats::coef(suppressWarnings(stats::glm(cbind(.r, .n - .r) ~ .study -1, data = ref_dat, family = stats::binomial(link = link))))
    out <- mean(bl)

  } else if (likelihood %in% valid_lhood$ordered) {

    # Take lowest outcome and treat as binary

    ref_dat <- dplyr::bind_rows(
      if (has_ipd(network)) dplyr::group_by(network$ipd, ".study", ".trt") %>%
        dplyr::summarise(.r = rowSums(.data$.r[, -1], na.rm = TRUE), .n = dplyr::n()),
      if (has_agd_arm(network)) dplyr::select(network$agd_arm, ".study", ".trt", ".r") %>%
        dplyr::mutate(.n = rowSums(.data$.r, na.rm = TRUE), .r = rowSums(.data$.r[, -1], na.rm = TRUE))
    ) %>%
      dplyr::filter(.data$.trt == ref_trt) %>%
      dplyr::mutate(.n = ifelse(.data$.r == 0, .data$.n + 0.5, .data$.n),
                    .r = ifelse(.data$.r == 0, 0.5, .data$.r))

    bl <- stats::coef(suppressWarnings(stats::glm(cbind(.r, .n - .r) ~ .study -1, data = ref_dat, family = stats::binomial(link = link))))
    out <- mean(bl)

  } else if (likelihood %in% valid_lhood$rate) {

    ref_dat <- dplyr::bind_rows(
      if (has_ipd(network)) dplyr::group_by(network$ipd, .data$.study, .data$.trt) %>%
        dplyr::summarise(.r = sum(.data$.r), .E = sum(.data$.E)),
      if (has_agd_arm(network)) dplyr::select(network$agd_arm, ".study", ".trt", ".r", ".E")
    ) %>%
      dplyr::filter(.data$.trt == ref_trt) %>%
      dplyr::mutate(.r = ifelse(.data$.r == 0, 0.5, .data$.r))

    bl <- stats::coef(suppressWarnings(stats::glm(.r ~ offset(log(.E)) + .study -1, data = ref_dat, family = stats::poisson(link = link))))
    out <- mean(bl)

  } else if (likelihood %in% valid_lhood$continuous) {

    out <- mean(link_fun(c(network$ipd$.y, network$agd_arm$.y), link))

  } else {
    inform(glue::glue("NOTE: Automated centering of baseline risk not yet implemented for survival outcomes."))
    out <- 0
  }

  return(out)
}

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Only test gradients, no sampling
smk_fit_RE <- nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5),
                  test_grad = TRUE)

test_that("all_contrasts argument", {
  m <- "should be TRUE or FALSE"
  expect_error(marginal_effects(smk_fit_RE, all_contrasts = "a"), m)
  expect_error(marginal_effects(smk_fit_RE, all_contrasts = 1), m)
  expect_error(marginal_effects(smk_fit_RE, all_contrasts = list()), m)
  expect_error(marginal_effects(smk_fit_RE, all_contrasts = NA), m)
  expect_error(marginal_effects(smk_fit_RE, all_contrasts = NULL), m)
})

test_that("trt_ref argument", {
  m <- "does not match a treatment in the network"
  expect_error(marginal_effects(smk_fit_RE, trt_ref = "a"), m)
  expect_error(marginal_effects(smk_fit_RE, trt_ref = 1), m)
  expect_error(marginal_effects(smk_fit_RE, trt_ref = list("a")), m)
  expect_error(marginal_effects(smk_fit_RE, trt_ref = NA), m)

  # expect_warning(marginal_effects(smk_fit_RE, all_contrasts = TRUE, trt_ref = "Self-help"), "Ignoring `trt_ref`")
})

test_that("probs argument", {
  m <- "numeric vector of probabilities"
  expect_error(marginal_effects(smk_fit_RE, probs = "a"), m)
  expect_error(marginal_effects(smk_fit_RE, probs = -1), m)
  expect_error(marginal_effects(smk_fit_RE, probs = 1.5), m)
  expect_error(marginal_effects(smk_fit_RE, probs = Inf), m)
  expect_error(marginal_effects(smk_fit_RE, probs = list()), m)
  expect_error(marginal_effects(smk_fit_RE, probs = NA), m)
  expect_error(marginal_effects(smk_fit_RE, probs = NULL), m)
})

test_that("summary argument", {
  m <- "should be TRUE or FALSE"
  expect_error(marginal_effects(smk_fit_RE, summary = "a"), m)
  expect_error(marginal_effects(smk_fit_RE, summary = 1), m)
  expect_error(marginal_effects(smk_fit_RE, summary = list()), m)
  expect_error(marginal_effects(smk_fit_RE, summary = NA), m)
  expect_error(marginal_effects(smk_fit_RE, summary = NULL), m)
})

test_that("newdata argument", {
  m <- "not a data frame"
  expect_error(marginal_effects(smk_fit_RE, newdata = "a"), m)
  expect_error(marginal_effects(smk_fit_RE, newdata = 1), m)
  expect_error(marginal_effects(smk_fit_RE, newdata = list()), m)
  expect_error(marginal_effects(smk_fit_RE, newdata = NA), m)
})


skip_on_cran()  # Reduce CRAN check time


# Binary AgD only --------------------------------------------------------------

# Only small number of samples to test
smk_fit_RE <- suppressWarnings(nma(smk_net,
                                   trt_effects = "random",
                                   prior_intercept = normal(scale = 100),
                                   prior_trt = normal(scale = 100),
                                   prior_het = normal(scale = 5),
                                   iter = 10))

test_that("Binary AgD: .trta, .trtb columns are correct", {
  re1.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re1.1$.trtb, "]"),
                   re1.1$parameter)
  expect_identical(as.character(re1.1$.trta), rep("No intervention", times = nrow(re1.1)))

  re1.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, mtype = "ratio"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re1.2$.trtb, "]"),
                   re1.2$parameter)
  expect_identical(as.character(re1.2$.trta), rep("No intervention", times = nrow(re1.2)))

  re1.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, mtype = "link"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re1.3$.trtb, "]"),
                   re1.3$parameter)
  expect_identical(as.character(re1.3$.trta), rep("No intervention", times = nrow(re1.3)))


  re2.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, all_contrasts = TRUE, mtype = "difference"))
  expect_identical(paste0("marg[", re2.1$.study, ": ", re2.1$.trtb, " vs. ", re2.1$.trta, "]"),
                   re2.1$parameter)

  re2.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, all_contrasts = TRUE, mtype = "ratio"))
  expect_identical(paste0("marg[", re2.1$.study, ": ", re2.2$.trtb, " vs. ", re2.2$.trta, "]"),
                   re2.2$parameter)

  re2.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, all_contrasts = TRUE, mtype = "link"))
  expect_identical(paste0("marg[", re2.1$.study, ": ", re2.3$.trtb, " vs. ", re2.3$.trta, "]"),
                   re2.3$parameter)


  re3.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, trt_ref = "Self-help", mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re3.1$.trtb, "]"),
                   re3.1$parameter)
  expect_identical(as.character(re3.1$.trta), rep("Self-help", times = nrow(re3.1)))

  re3.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, trt_ref = "Self-help", mtype = "ratio"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re3.2$.trtb, "]"),
                   re3.2$parameter)
  expect_identical(as.character(re3.2$.trta), rep("Self-help", times = nrow(re3.2)))

  re3.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, trt_ref = "Self-help", mtype = "link"))
  expect_identical(paste0("marg[", re3.1$.study, ": ", re3.3$.trtb, "]"),
                   re3.3$parameter)
  expect_identical(as.character(re3.3$.trta), rep("Self-help", times = nrow(re3.3)))
})


test_that("Binary AgD: .trta, .trtb columns are correct in new populations", {
  re1.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.trtb, "]"),
                   re1.1$parameter)
  expect_identical(as.character(re1.1$.trta), rep("No intervention", times = nrow(re1.1)))

  re1.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), mtype = "ratio"))
  expect_identical(paste0("marg[", re1.2$.trtb, "]"),
                   re1.2$parameter)
  expect_identical(as.character(re1.2$.trta), rep("No intervention", times = nrow(re1.2)))

  re1.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), mtype = "link"))
  expect_identical(paste0("marg[", re1.3$.trtb, "]"),
                   re1.3$parameter)
  expect_identical(as.character(re1.3$.trta), rep("No intervention", times = nrow(re1.3)))


  re2.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "difference"))
  expect_identical(paste0("marg[", re2.1$.trtb, " vs. ", re2.1$.trta, "]"),
                   re2.1$parameter)

  re2.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "ratio"))
  expect_identical(paste0("marg[", re2.2$.trtb, " vs. ", re2.2$.trta, "]"),
                   re2.2$parameter)

  re2.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "link"))
  expect_identical(paste0("marg[", re2.3$.trtb, " vs. ", re2.3$.trta, "]"),
                   re2.3$parameter)


  re3.1 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), trt_ref = "Self-help", mtype = "difference"))
  expect_identical(paste0("marg[", re3.1$.trtb, "]"),
                   re3.1$parameter)
  expect_identical(as.character(re3.1$.trta), rep("Self-help", times = nrow(re3.1)))

  re3.2 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), trt_ref = "Self-help", mtype = "ratio"))
  expect_identical(paste0("marg[", re3.2$.trtb, "]"),
                   re3.2$parameter)
  expect_identical(as.character(re3.2$.trta), rep("Self-help", times = nrow(re3.2)))

  re3.3 <- tibble::as_tibble(marginal_effects(smk_fit_RE, baseline = distr(qnorm, 0, 0.1), trt_ref = "Self-help", mtype = "link"))
  expect_identical(paste0("marg[", re3.3$.trtb, "]"),
                   re3.3$parameter)
  expect_identical(as.character(re3.3$.trta), rep("Self-help", times = nrow(re3.3)))
})

blocker_net <- set_agd_arm(blocker, studyn, trtc, r = r, n = n)
blocker_fit <- suppressWarnings(nma(blocker_net,
                                    prior_intercept = normal(scale = 100),
                                    prior_trt = normal(scale = 100),
                                    iter = 10))

test_that("all_contrasts works with only two treatments", {
  expect_equal(
    dplyr::select(as.data.frame(marginal_effects(blocker_fit)), -"parameter"),
    dplyr::select(as.data.frame(marginal_effects(blocker_fit, all_contrasts = TRUE)) ,-"parameter"),
    check.attributes = FALSE
  )
  expect_identical(
    unname(as.array(marginal_effects(blocker_fit))),
    unname(as.array(marginal_effects(blocker_fit, all_contrasts = TRUE)))
  )
})


# Binary with IPD regression ---------------------------------------------------

library(dplyr)

pso_ipd <- mutate(plaque_psoriasis_ipd,
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn %in% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- mutate(plaque_psoriasis_agd,
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn %in% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

pso_net <- combine_network(
  set_ipd(filter(pso_ipd, complete),
          studyc, trtc,
          r = pasi75),
  set_agd_arm(pso_agd,
              study = studyc, trt = trtc,
              r = pasi75_r, n = pasi75_n))

pso_net <- add_integration(pso_net,
                           durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                           prevsys = distr(qbern, prob = prevsys),
                           bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
                           weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
                           psa = distr(qbern, prob = psa),
                           n_int = 4)

# Only small number of samples to test
pso_fit <- suppressWarnings(nma(pso_net,
                                trt_effects = "fixed",
                                regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
                                prior_intercept = normal(scale = 10),
                                prior_trt = normal(scale = 10),
                                prior_reg = normal(scale = 10),
                                init_r = 0.1,
                                iter = 10))

pso_new <- tibble(
  bsa_mean = 0.6,
  bsa_sd = 0.3,
  prevsys = 0.1,
  psa = 0.2,
  weight_mean = 10,
  weight_sd = 1,
  durnpso_mean = 3,
  durnpso_sd = 1,
  studyc = "NEW"
)

pso_new <- add_integration(pso_new,
                           durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                           prevsys = distr(qbern, prob = prevsys),
                           bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
                           weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
                           psa = distr(qbern, prob = psa),
                           cor = pso_net$int_cor,
                           n_int = 4)


test_that("newdata validation", {
  expect_error(marginal_effects(pso_fit, newdata = dplyr::select(pso_new, -".int_durnpso"), baseline = distr(qnorm, 0, 0.1)),
               'Regression variable "durnpso" not found in `newdata`')
  expect_error(marginal_effects(pso_fit, newdata = dplyr::select(pso_new, -".int_durnpso", -".int_weight"), baseline = distr(qnorm, 0, 0.1)),
               'Regression variables "durnpso" and "weight" not found in `newdata`')
})

test_that("baseline required when newdata specified", {
  m <- "Specify both `newdata` and `baseline`, or neither"
  expect_error(marginal_effects(pso_fit, newdata = pso_new), m)
  expect_error(marginal_effects(pso_fit, baseline = distr(qnorm, 0, 0.1)), m)
})

test_that("Binary IPD: .study, .trta, .trtb columns are correct", {
  re1 <- tibble::as_tibble(marginal_effects(pso_fit))
  expect_identical(paste0("marg[", re1$.study, ": ", re1$.trtb, "]"),
                   re1$parameter)
  expect_identical(as.character(re1$.trta), rep(levels(pso_net$treatments)[1], times = nrow(re1)))

  re2 <- tibble::as_tibble(marginal_effects(pso_fit, all_contrasts = TRUE))
  expect_identical(paste0("marg[", re2$.study, ": ", re2$.trtb, " vs. ", re2$.trta, "]"),
                   re2$parameter)

  re3 <- tibble::as_tibble(marginal_effects(pso_fit, trt_ref = "ETN"))
  expect_identical(paste0("marg[", re3$.study, ": ", re3$.trtb, "]"),
                   re3$parameter)
  expect_identical(as.character(re3$.trta), rep("ETN", times = nrow(re3)))

  re4 <- tibble::as_tibble(marginal_effects(pso_fit, baseline = distr(qnorm, 0, 0.1), newdata = pso_new, study = studyc))
  expect_identical(paste0("marg[", re4$.study, ": ", re4$.trtb, "]"),
                   re4$parameter)
  expect_identical(as.character(re4$.trta), rep(levels(pso_net$treatments)[1], times = nrow(re4)))

  re5 <- tibble::as_tibble(marginal_effects(pso_fit, all_contrasts = TRUE, baseline = distr(qnorm, 0, 0.1), newdata = pso_new, study = studyc))
  expect_identical(paste0("marg[", re5$.study, ": ", re5$.trtb, " vs. ", re5$.trta, "]"),
                   re5$parameter)

  re6 <- tibble::as_tibble(marginal_effects(pso_fit, trt_ref = "ETN", baseline = distr(qnorm, 0, 0.1), newdata = pso_new, study = studyc))
  expect_identical(paste0("marg[", re6$.study, ": ", re6$.trtb, "]"),
                   re6$parameter)
  expect_identical(as.character(re6$.trta), rep("ETN", times = nrow(re6)))
})


# Ordered categorical -----------------------------------------------------

ord_net <- set_agd_arm(plaque_psoriasis_agd,
                       studyc, trtc,
                       r = multi(r0 = pasi75_n,
                                 PASI75 = pasi75_r,
                                 PASI90 = pasi90_r,
                                 PASI100 = pasi100_r,
                                 type = "ordered", inclusive = TRUE))


# Only small number of samples to test
ord_fit <- suppressWarnings(nma(ord_net,
                                trt_effects = "random",
                                prior_intercept = normal(scale = 100),
                                prior_trt = normal(scale = 100),
                                prior_het = normal(scale = 5),
                                iter = 10))

test_that("Ordered categorical: .study, .trta, .trtb, .category columns are correct", {
  re1.1 <- tibble::as_tibble(marginal_effects(ord_fit, mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re1.1$.trtb, ", ", re1.1$.category, "]"),
                   re1.1$parameter)
  expect_identical(as.character(re1.1$.trta), rep("SEC_300", times = nrow(re1.1)))

  re1.2 <- tibble::as_tibble(marginal_effects(ord_fit, mtype = "ratio"))
  expect_identical(paste0("marg[", re1.2$.study, ": ", re1.2$.trtb, ", ", re1.2$.category, "]"),
                   re1.2$parameter)
  expect_identical(as.character(re1.2$.trta), rep("SEC_300", times = nrow(re1.2)))

  re1.3 <- tibble::as_tibble(marginal_effects(ord_fit, mtype = "link"))
  expect_identical(paste0("marg[", re1.3$.study, ": ", re1.3$.trtb, ", ", re1.3$.category, "]"),
                   re1.3$parameter)
  expect_identical(as.character(re1.3$.trta), rep("SEC_300", times = nrow(re1.3)))


  re2.1 <- tibble::as_tibble(marginal_effects(ord_fit, all_contrasts = TRUE, mtype = "difference"))
  expect_identical(paste0("marg[", re2.1$.study, ": ", re2.1$.trtb, " vs. ", re2.1$.trta, ", ", re2.1$.category, "]"),
                   re2.1$parameter)

  re2.2 <- tibble::as_tibble(marginal_effects(ord_fit, all_contrasts = TRUE, mtype = "ratio"))
  expect_identical(paste0("marg[", re2.2$.study, ": ", re2.2$.trtb, " vs. ", re2.2$.trta, ", ", re2.2$.category, "]"),
                   re2.2$parameter)

  re2.3 <- tibble::as_tibble(marginal_effects(ord_fit, all_contrasts = TRUE, mtype = "link"))
  expect_identical(paste0("marg[", re2.3$.study, ": ", re2.3$.trtb, " vs. ", re2.3$.trta, ", ", re2.3$.category, "]"),
                   re2.3$parameter)


  re3.1 <- tibble::as_tibble(marginal_effects(ord_fit, trt_ref = "PBO", mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re3.1$.trtb, ", ", re3.1$.category, "]"),
                   re3.1$parameter)
  expect_identical(as.character(re3.1$.trta), rep("PBO", times = nrow(re3.1)))

  re3.2 <- tibble::as_tibble(marginal_effects(ord_fit, trt_ref = "PBO", mtype = "ratio"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re3.2$.trtb, ", ", re3.2$.category, "]"),
                   re3.2$parameter)
  expect_identical(as.character(re3.2$.trta), rep("PBO", times = nrow(re3.2)))

  re3.3 <- tibble::as_tibble(marginal_effects(ord_fit, trt_ref = "PBO", mtype = "link"))
  expect_identical(paste0("marg[", re3.1$.study, ": ", re3.3$.trtb, ", ", re3.3$.category, "]"),
                   re3.3$parameter)
  expect_identical(as.character(re3.3$.trta), rep("PBO", times = nrow(re3.3)))
})


test_that("Ordered categorical: .trta, .trtb, .category columns are correct in new populations", {
  re1.1 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.trtb, ", ", re1.1$.category, "]"),
                   re1.1$parameter)
  expect_identical(as.character(re1.1$.trta), rep("SEC_300", times = nrow(re1.1)))

  re1.2 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), mtype = "ratio"))
  expect_identical(paste0("marg[", re1.2$.trtb, ", ", re1.2$.category, "]"),
                   re1.2$parameter)
  expect_identical(as.character(re1.2$.trta), rep("SEC_300", times = nrow(re1.2)))

  re1.3 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), mtype = "link"))
  expect_identical(paste0("marg[", re1.3$.trtb, ", ", re1.3$.category, "]"),
                   re1.3$parameter)
  expect_identical(as.character(re1.3$.trta), rep("SEC_300", times = nrow(re1.3)))


  re2.1 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "difference"))
  expect_identical(paste0("marg[", re2.1$.trtb, " vs. ", re2.1$.trta, ", ", re2.1$.category, "]"),
                   re2.1$parameter)

  re2.2 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "ratio"))
  expect_identical(paste0("marg[", re2.2$.trtb, " vs. ", re2.2$.trta, ", ", re2.2$.category, "]"),
                   re2.2$parameter)

  re2.3 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), all_contrasts = TRUE, mtype = "link"))
  expect_identical(paste0("marg[", re2.3$.trtb, " vs. ", re2.3$.trta, ", ", re2.3$.category, "]"),
                   re2.3$parameter)


  re3.1 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), trt_ref = "PBO", mtype = "difference"))
  expect_identical(paste0("marg[", re3.1$.trtb, ", ", re3.1$.category, "]"),
                   re3.1$parameter)
  expect_identical(as.character(re3.1$.trta), rep("PBO", times = nrow(re3.1)))

  re3.2 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), trt_ref = "PBO", mtype = "ratio"))
  expect_identical(paste0("marg[", re3.2$.trtb, ", ", re3.2$.category, "]"),
                   re3.2$parameter)
  expect_identical(as.character(re3.2$.trta), rep("PBO", times = nrow(re3.2)))

  re3.3 <- tibble::as_tibble(marginal_effects(ord_fit, baseline = distr(qnorm, 0, 0.1), trt_ref = "PBO", mtype = "link"))
  expect_identical(paste0("marg[", re3.3$.trtb, ", ", re3.3$.category, "]"),
                   re3.3$parameter)
  expect_identical(as.character(re3.3$.trta), rep("PBO", times = nrow(re3.3)))
})


# Survival ----------------------------------------------------------------

surv_net <- set_ipd(ndmm_agd, study, trt, Surv = Surv(eventtime, status))

# Only small number of samples to test
surv_fit <- suppressWarnings(nma(surv_net,
                                 likelihood = "weibull",
                                 prior_intercept = normal(scale = 100),
                                 prior_trt = normal(scale = 100),
                                 prior_aux = normal(scale = 5),
                                 iter = 10))

test_that("Survival: .study, .trta, .trtb columns are correct", {
  re1.1 <- tibble::as_tibble(marginal_effects(surv_fit, type = "median", mtype = "difference"))
  expect_identical(paste0("marg[", re1.1$.study, ": ", re1.1$.trtb, "]"),
                   re1.1$parameter)
  expect_identical(as.character(re1.1$.trta), rep("Pbo", times = nrow(re1.1)))

  re1.2 <- tibble::as_tibble(marginal_effects(surv_fit, type = "hazard", mtype = "ratio")) %>%
    group_by(.study, .trtb, .trta) %>%
    mutate(.id = 1:n())
  expect_identical(paste0("marg[", re1.2$.study, ": ", re1.2$.trtb, ", ", re1.2$.id, "]"),
                   re1.2$parameter)
  expect_identical(as.character(re1.2$.trta), rep("Pbo", times = nrow(re1.2)))

  re1.3 <- tibble::as_tibble(marginal_effects(surv_fit, type = "quantile", mtype = "difference"))
  expect_identical(paste0("marg[", re1.3$.study, ": ", re1.3$.trtb, ", ", re1.3$.quantile, "]"),
                   re1.3$parameter)
  expect_identical(as.character(re1.3$.trta), rep("Pbo", times = nrow(re1.3)))


  re2.1 <- tibble::as_tibble(marginal_effects(surv_fit, all_contrasts = TRUE, type = "median", mtype = "difference"))
  expect_identical(paste0("marg[", re2.1$.study, ": ", re2.1$.trtb, " vs. ", re2.1$.trta, "]"),
                   re2.1$parameter)

  re2.2 <- tibble::as_tibble(marginal_effects(surv_fit, all_contrasts = TRUE, type = "hazard", mtype = "ratio")) %>%
    group_by(.study, .trtb, .trta) %>%
    mutate(.id = 1:n())
  expect_identical(paste0("marg[", re2.2$.study, ": ", re2.2$.trtb, " vs. ", re2.2$.trta, ", ", re2.2$.id, "]"),
                   re2.2$parameter)

  re2.3 <- tibble::as_tibble(marginal_effects(surv_fit, all_contrasts = TRUE, type = "quantile", mtype = "difference"))
  expect_identical(paste0("marg[", re2.3$.study, ": ", re2.3$.trtb, " vs. ", re2.3$.trta, ", ", re2.3$.quantile, "]"),
                   re2.3$parameter)


  re3.1 <- tibble::as_tibble(marginal_effects(surv_fit, trt_ref = "Len", type = "median", mtype = "difference"))
  expect_identical(paste0("marg[", re3.1$.study, ": ", re3.1$.trtb, "]"),
                   re3.1$parameter)
  expect_identical(as.character(re3.1$.trta), rep("Len", times = nrow(re3.1)))

  re3.2 <- tibble::as_tibble(marginal_effects(surv_fit, trt_ref = "Len", type = "hazard", mtype = "ratio")) %>%
    group_by(.study, .trtb, .trta) %>%
    mutate(.id = 1:n())
  expect_identical(paste0("marg[", re3.2$.study, ": ", re3.2$.trtb, ", ", re3.2$.id, "]"),
                   re3.2$parameter)
  expect_identical(as.character(re3.2$.trta), rep("Len", times = nrow(re3.2)))

  re3.3 <- tibble::as_tibble(marginal_effects(surv_fit, trt_ref = "Len", type = "quantile", mtype = "difference"))
  expect_identical(paste0("marg[", re3.3$.study, ": ", re3.3$.trtb, ", ", re3.3$.quantile, "]"),
                   re3.3$parameter)
  expect_identical(as.character(re3.3$.trta), rep("Len", times = nrow(re3.3)))
})

ndmm_net <- combine_network(
  set_ipd(ndmm_ipd, study = study, trt = trt,
          Surv = Surv(eventtime, status)),
  set_agd_surv(ndmm_agd, study = study, trt = trt,
               Surv = Surv(eventtime, status)),
  set_agd_contrast(data.frame(study = "A", trt = c("Pbo", "Len"), y = c(NA, 0), se = c(NA, 1), ss = 10),
                   study = study, trt = trt, y = y, se = se, sample_size = ss)
)

test_that("make_knots()", {

  expect_error(make_knots("test"), "must be an `nma_data` object")
  expect_error(make_knots(ndmm_net, n_knots = "1"), "`n_knots` must be a positive integer")
  expect_error(make_knots(ndmm_net, type = "wrong"), "`type` must be one of")

  # defaults
  k <- make_knots(ndmm_net)
  expect_named(k, setdiff(levels(ndmm_net$studies), "A"))

  attal <- dplyr::filter(ndmm_ipd, study == "Attal2012")
  attal_obs <- dplyr::filter(attal, status == 1)

  expect_identical(k$Attal2012,
                   c(0,
                     quantile(attal_obs$eventtime, probs = (1:7)/8, names = FALSE),
                     max(attal$eventtime)))

  # setting n_knots
  k2 <- make_knots(ndmm_net, n_knots = 3)

  expect_named(k2, setdiff(levels(ndmm_net$studies), "A"))
  expect_identical(k2$Attal2012,
                   c(0,
                     quantile(attal_obs$eventtime, probs = (1:3)/4, names = FALSE),
                     max(attal$eventtime)))

  # common knots
  k3 <- make_knots(ndmm_net, type = "quantile_common")
  expect_named(k3, setdiff(levels(ndmm_net$studies), "A"))
  expect_all_equal(k3, k3[1])
})

test_that("knots()", {
  ndmm_fit_mspline <- suppressWarnings(nma(ndmm_net,
                                           likelihood = "mspline",
                                           prior_intercept = normal(0, 100),
                                           prior_trt = normal(0, 10),
                                           prior_aux = half_normal(1),
                                           test_grad = TRUE))

  expect_identical(knots(ndmm_fit_mspline), make_knots(ndmm_net))
  expect_identical(knots(ndmm_fit_mspline, type = "internal"),
                   purrr::map(make_knots(ndmm_net), ~.[-c(1, 9)]))
  expect_identical(knots(ndmm_fit_mspline, type = "boundary"),
                   purrr::map(make_knots(ndmm_net), ~.[c(1, 9)]))

  ndmm_fit_mspline2 <- suppressWarnings(nma(ndmm_net,
                                           likelihood = "mspline",
                                           n_knots = 3,
                                           prior_intercept = normal(0, 100),
                                           prior_trt = normal(0, 10),
                                           prior_aux = half_normal(1),
                                           test_grad = TRUE))

  expect_identical(knots(ndmm_fit_mspline2), make_knots(ndmm_net, n_knots = 3))

  ndmm_fit_mspline_nphr <- suppressWarnings(nma(ndmm_net,
                                                likelihood = "mspline",
                                                aux_regression = ~.trt,
                                                prior_intercept = normal(0, 100),
                                                prior_trt = normal(0, 10),
                                                prior_aux = half_normal(1),
                                                prior_aux_reg = normal(0, 10),
                                                test_grad = TRUE))

  expect_identical(knots(ndmm_fit_mspline_nphr), make_knots(ndmm_net, type = "quantile_common"))

  ndmm_fit_pexp <- suppressWarnings(nma(ndmm_net,
                                           likelihood = "pexp",
                                           prior_intercept = normal(0, 100),
                                           prior_trt = normal(0, 10),
                                           prior_aux = half_normal(1),
                                           test_grad = TRUE))

  expect_identical(knots(ndmm_fit_pexp), make_knots(ndmm_net))

  ndmm_fit_weib <- suppressWarnings(nma(ndmm_net,
                                           likelihood = "weibull",
                                           prior_intercept = normal(0, 100),
                                           prior_trt = normal(0, 10),
                                           prior_aux = half_normal(1),
                                           test_grad = TRUE))

  expect_error(knots(ndmm_fit_weib), "No knots present")
})

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Only test gradients, no sampling
smk_fit_RE <- nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5),
                  test_grad = TRUE)

test_that("all_contrasts argument", {
  m <- "should be TRUE or FALSE"
  expect_error(relative_effects(smk_fit_RE, all_contrasts = "a"), m)
  expect_error(relative_effects(smk_fit_RE, all_contrasts = 1), m)
  expect_error(relative_effects(smk_fit_RE, all_contrasts = list()), m)
  expect_error(relative_effects(smk_fit_RE, all_contrasts = NA), m)
  expect_error(relative_effects(smk_fit_RE, all_contrasts = NULL), m)
})

test_that("trt_ref argument", {
  m <- "does not match a treatment in the network"
  expect_error(relative_effects(smk_fit_RE, trt_ref = "a"), m)
  expect_error(relative_effects(smk_fit_RE, trt_ref = 1), m)
  expect_error(relative_effects(smk_fit_RE, trt_ref = list("a")), m)
  expect_error(relative_effects(smk_fit_RE, trt_ref = NA), m)

  # expect_warning(relative_effects(smk_fit_RE, all_contrasts = TRUE, trt_ref = "Self-help"), "Ignoring `trt_ref`")
})

test_that("probs argument", {
  m <- "numeric vector of probabilities"
  expect_error(relative_effects(smk_fit_RE, probs = "a"), m)
  expect_error(relative_effects(smk_fit_RE, probs = -1), m)
  expect_error(relative_effects(smk_fit_RE, probs = 1.5), m)
  expect_error(relative_effects(smk_fit_RE, probs = Inf), m)
  expect_error(relative_effects(smk_fit_RE, probs = list()), m)
  expect_error(relative_effects(smk_fit_RE, probs = NA), m)
  expect_error(relative_effects(smk_fit_RE, probs = NULL), m)
})

test_that("summary argument", {
  m <- "should be TRUE or FALSE"
  expect_error(relative_effects(smk_fit_RE, summary = "a"), m)
  expect_error(relative_effects(smk_fit_RE, summary = 1), m)
  expect_error(relative_effects(smk_fit_RE, summary = list()), m)
  expect_error(relative_effects(smk_fit_RE, summary = NA), m)
  expect_error(relative_effects(smk_fit_RE, summary = NULL), m)
})

test_that("newdata argument", {
  m <- "not a data frame"
  expect_error(relative_effects(smk_fit_RE, newdata = "a"), m)
  expect_error(relative_effects(smk_fit_RE, newdata = 1), m)
  expect_error(relative_effects(smk_fit_RE, newdata = list()), m)
  expect_error(relative_effects(smk_fit_RE, newdata = NA), m)
})


skip_on_cran()  # Reduce CRAN check time

# Only small number of samples to test
smk_fit_RE <- suppressWarnings(nma(smk_net,
                                   trt_effects = "random",
                                   prior_intercept = normal(scale = 100),
                                   prior_trt = normal(scale = 100),
                                   prior_het = normal(scale = 5),
                                   iter = 10))

test_that(".trta, .trtb columns are correct", {
  re1 <- tibble::as_tibble(relative_effects(smk_fit_RE))
  expect_identical(paste0("d[", re1$.trtb, "]"),
                   re1$parameter)
  expect_identical(as.character(re1$.trta), rep("No intervention", times = nrow(re1)))

  re2 <- tibble::as_tibble(relative_effects(smk_fit_RE, all_contrasts = TRUE))
  expect_identical(paste0("d[", re2$.trtb, " vs. ", re2$.trta, "]"),
                   re2$parameter)

  re3 <- tibble::as_tibble(relative_effects(smk_fit_RE, trt_ref = "Self-help"))
  expect_identical(paste0("d[", re3$.trtb, "]"),
                   re3$parameter)
  expect_identical(as.character(re3$.trta), rep("Self-help", times = nrow(re3)))
})

pso_net <- set_ipd(plaque_psoriasis_ipd[complete.cases(plaque_psoriasis_ipd), ],
                   studyc, trtc,
                   r = pasi75)

# Only small number of samples to test
pso_fit <- suppressWarnings(nma(pso_net,
                                trt_effects = "fixed",
                                regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
                                prior_intercept = normal(scale = 10),
                                prior_trt = normal(scale = 10),
                                prior_reg = normal(scale = 10),
                                init_r = 0.1,
                                iter = 10))

pso_new <- data.frame(durnpso = 10, prevsys = TRUE, bsa = 20, weight = 75, psa = FALSE, study = c("One", "Two"))

test_that("newdata validation", {
  expect_error(relative_effects(pso_fit, newdata = pso_new[-1], study = study),
               'Regression variable "durnpso" not found in `newdata`')
  expect_error(relative_effects(pso_fit, newdata = pso_new[-(1:2)], study = study),
               'Regression variables "durnpso" and "prevsys" not found in `newdata`')

  make_bad <- function(x, vars = "durnpso") {
    bad <- pso_new
    bad[vars] <- x
    return(bad)
  }

  expect_error(relative_effects(pso_fit, newdata = make_bad(NA), study = study),
               "missing or infinite values in `newdata`: durnpso")
  expect_error(relative_effects(pso_fit, newdata = make_bad(NaN), study = study),
               "missing or infinite values in `newdata`: durnpso")
  expect_error(relative_effects(pso_fit, newdata = make_bad(Inf), study = study),
               "missing or infinite values in `newdata`: durnpso")

  expect_error(relative_effects(pso_fit, newdata = make_bad(NA, c("durnpso", "psa")), study = study),
               "missing or infinite values in `newdata`: durnpso, psa")
  expect_error(relative_effects(pso_fit, newdata = make_bad(NaN, c("durnpso", "psa")), study = study),
               "missing or infinite values in `newdata`: durnpso, psa")
  expect_error(relative_effects(pso_fit, newdata = make_bad(Inf, c("durnpso", "psa")), study = study),
               "missing or infinite values in `newdata`: durnpso, psa")
})

test_that(".study, .trta, .trtb columns are correct", {
  re1 <- tibble::as_tibble(relative_effects(pso_fit))
  expect_identical(paste0("d[", re1$.study, ": ", re1$.trtb, "]"),
                   re1$parameter)
  expect_identical(as.character(re1$.trta), rep(levels(pso_net$treatments)[1], times = nrow(re1)))

  re2 <- tibble::as_tibble(relative_effects(pso_fit, all_contrasts = TRUE))
  expect_identical(paste0("d[", re2$.study, ": ", re2$.trtb, " vs. ", re2$.trta, "]"),
                   re2$parameter)

  re3 <- tibble::as_tibble(relative_effects(pso_fit, trt_ref = "ETN"))
  expect_identical(paste0("d[", re3$.study, ": ", re3$.trtb, "]"),
                   re3$parameter)
  expect_identical(as.character(re3$.trta), rep("ETN", times = nrow(re3)))

  re4 <- tibble::as_tibble(relative_effects(pso_fit, newdata = pso_new, study = study))
  expect_identical(paste0("d[", re4$.study, ": ", re4$.trtb, "]"),
                   re4$parameter)
  expect_identical(as.character(re4$.trta), rep(levels(pso_net$treatments)[1], times = nrow(re4)))

  re5 <- tibble::as_tibble(relative_effects(pso_fit, all_contrasts = TRUE, newdata = pso_new, study = study))
  expect_identical(paste0("d[", re5$.study, ": ", re5$.trtb, " vs. ", re5$.trta, "]"),
                   re5$parameter)

  re6 <- tibble::as_tibble(relative_effects(pso_fit, trt_ref = "ETN", newdata = pso_new, study = study))
  expect_identical(paste0("d[", re6$.study, ": ", re6$.trtb, "]"),
                   re6$parameter)
  expect_identical(as.character(re6$.trta), rep("ETN", times = nrow(re6)))
})

test_that("`inclusive` argument is validated", {
  m <- "`inclusive` must be a logical value"

  expect_error(multi(a = 1:4, b = 1:4, inclusive = "a"), m)
  expect_error(multi(a = 1:4, b = 1:4, inclusive = 1), m)
  expect_error(multi(a = 1:4, b = 1:4, inclusive = list()), m)
  expect_error(multi(a = 1:4, b = 1:4, inclusive = NA), m)
  expect_error(multi(a = 1:4, b = 1:4, inclusive = NULL), m)

  expect_warning(multi(1, 2 , inclusive = TRUE, type = "competing"),
                 "Ignoring inclusive = TRUE")
})

test_that("`type` argument is validated", {
  m <- "`type` must be"

  expect_error(multi(a = 1:4, b = 1:4, type = "a"), m)
  expect_error(multi(a = 1:4, b = 1:4, type = 1), m)
  expect_error(multi(a = 1:4, b = 1:4, type = list()), m)
  expect_error(multi(a = 1:4, b = 1:4, type = NA), m)
  expect_error(multi(a = 1:4, b = 1:4, type = NULL), m)
})

test_that("basic validation of `...`", {
  expect_error(multi(), "At least 2 outcomes")
  expect_error(multi(1:3), "At least 2 outcomes")
  expect_error(multi(a = 1:4, a = 1:4), "Duplicate outcome category labels")
  a <- 1:4
  expect_error(multi(a, a), "Duplicate outcome category labels")
  expect_error(multi(1:2, 1:4), "must be the same length")
  expect_error(multi(a = 1, b = Inf), "cannot be Inf")
  expect_error(multi(a = 1, b = NaN), "cannot be NaN")
  expect_error(multi(a = 1, b = "a"), "must be numeric")
  expect_error(multi(a = 1, b = 1.5), "must be integer")
  expect_error(multi(a = 1, b = -1), "must be non-negative")
})

df_inclusive <- tibble::tribble(~a, ~b, ~c,
                                1, 1, 1,
                                5, 4, 1,
                                5, 2, 2,
                                10, 5, 0,
                                5, 5, 0,    # y > -Inf, y > b, y > c
                                7, NA, 6,   # y > -Inf, y > c
                                10, 4, NA)  # y > -Inf, y > b

# internally stored as exclusive counts
df_exclusive <- tibble::tribble(~a, ~b, ~c,
                                0, 0, 1,
                                1, 3, 1,
                                3, 0, 2,
                                5, 5, 0,
                                0, 5, 0,    # -Inf < y < b, b < y < c, y > c
                                1, NA, 6,   # -Inf < y < c, y > c
                                6, 4, NA)   # -Inf < y < b, y > b

test_that("ordered inclusive outcome validation", {
  m <- "must be decreasing or constant across increasing categories"

  expect_error(multi(1, 2, 3, inclusive = TRUE, type = "ordered"), paste0(m, ".+row 1"))
  expect_error(multi(5, 4, 6, inclusive = TRUE, type = "ordered"), paste0(m, ".+row 1"))
  expect_error(multi(5, NA, 6, inclusive = TRUE, type = "ordered"), paste0(m, ".+row 1"))
  expect_error(with(df_inclusive[-7, ], multi(c, b, a, inclusive = TRUE, type = "ordered")), paste0(m, ".+rows 2, 3, 4, 5 and 6"))

  expect_error(multi(a = NA, b = 10, c = 5, inclusive = TRUE, type = "ordered"),
               "cannot be missing in the lowest category.+row 1")

  expect_error(multi(a = 10, b = NA, inclusive = TRUE, type = "ordered"),
               "must be present for at least 2 categories.+row 1")

  o <- with(df_inclusive, multi(a, b, c, inclusive = TRUE, type = "ordered"))
  expect_s3_class(o, "multi_ordered")
  expect_equivalent(unclass(o), as.matrix(df_exclusive))
  expect_equal(colnames(o), colnames(df_inclusive))

  o2 <- with(df_inclusive, multi(A = a, B = b, C = c, inclusive = TRUE, type = "ordered"))
  expect_equal(colnames(o2), c("A", "B", "C"))
})

test_that("exclusive outcome validation", {
  expect_error(multi(a = NA, b = 10, c = 5, inclusive = FALSE, type = "ordered"),
               "cannot be missing in the lowest category.+row 1")

  expect_error(multi(a = 10, b = NA, inclusive = FALSE, type = "ordered"),
               "must be present for at least 2 categories.+row 1")
  expect_error(multi(a = 10, b = NA, inclusive = FALSE, type = "competing"),
               "must be present for at least 2 categories.+row 1")

  o <- with(df_exclusive, multi(a, b, c, inclusive = FALSE, type = "ordered"))
  expect_s3_class(o, "multi_ordered")
  expect_equivalent(unclass(o), as.matrix(df_exclusive))
  expect_equal(colnames(o), colnames(df_exclusive))

  o2 <- with(df_exclusive, multi(a, b, c, inclusive = FALSE, type = "competing"))
  expect_s3_class(o2, "multi_competing")
  expect_equivalent(unclass(o2), as.matrix(df_exclusive))
  expect_equal(colnames(o2), colnames(df_exclusive))

  o3 <- with(df_exclusive, multi(A = a, B = b, C = c, inclusive = FALSE, type = "ordered"))
  expect_equal(colnames(o3), c("A", "B", "C"))

  o4 <- with(df_exclusive, multi(A = a, B = b, C = c, inclusive = FALSE, type = "competing"))
  expect_equal(colnames(o4), c("A", "B", "C"))

  # NA should be allowed in leftmost category for competing outcomes
  df_e2 <- dplyr::add_row(df_exclusive, a = NA, b = 5, c = 10)
  o5 <- with(df_e2, multi(a, b, c, inclusive = FALSE, type = "competing"))
  expect_equivalent(unclass(o5), as.matrix(df_e2))
})

skip_on_cran()

library(vdiffr)
library(ggplot2)
library(dplyr)

test_that("Atrial fibrillation", {
  af_net <- set_agd_arm(atrial_fibrillation[atrial_fibrillation$studyc != "WASPO", ],
                        study = studyc,
                        trt = trtc,
                        r = r,
                        n = n,
                        trt_class = trt_class)

  expect_doppelganger("Atrial fibrillation network",
    plot(af_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE) +
      ggplot2::theme(legend.position = "bottom", legend.box = "vertical")
  )

  expect_doppelganger("Atrial fibrillation network star",
                      plot(af_net, layout = "star")
  )
})

test_that("BCG vaccine", {
  bcg_net <- set_agd_arm(bcg_vaccine,
                         study = studyn,
                         trt = trtc,
                         r = r,
                         n = n,
                         trt_ref = "Unvaccinated")

  expect_doppelganger("BCG vaccine network", plot(bcg_net))
})

test_that("Blocker", {
  blocker_net <- set_agd_arm(blocker,
                             study = studyn,
                             trt = trtc,
                             r = r,
                             n = n,
                             trt_ref = "Control")

  expect_doppelganger("Blocker network", plot(blocker_net))
})

test_that("Diabetes", {
  db_net <- set_agd_arm(diabetes,
                        study = studyc,
                        trt = trtc,
                        r = r,
                        n = n)
  expect_doppelganger("Diabetes network",
    plot(db_net, weight_edges = TRUE, weight_nodes = TRUE) +
      ggplot2::guides(size = ggplot2::guide_legend(order = 1), edge_width = ggplot2::guide_legend(order = 2)) +
      ggplot2::theme(legend.box.margin = ggplot2::unit(c(0, 0, 0, 4), "lines"))
  )
})

test_that("Dietary fat", {
  diet_net <- set_agd_arm(dietary_fat,
                          study = studyc,
                          trt = trtc,
                          r = r,
                          E = E,
                          trt_ref = "Control",
                          sample_size = n)

  expect_doppelganger("Dietary fat network", plot(diet_net))
})

test_that("HTA psoriasis", {
  pso_net <- set_agd_arm(hta_psoriasis,
                         study = paste(studyc, year),
                         trt = trtc,
                         r = multi(r0 = sample_size - rowSums(cbind(PASI50, PASI75, PASI90), na.rm = TRUE),
                                   PASI50, PASI75, PASI90,
                                   inclusive = FALSE,
                                   type = "ordered"))

  expect_doppelganger("HTA psoriasis network",
    plot(pso_net, weight_edges = TRUE, weight_nodes = TRUE) +
      ggplot2::guides(size = ggplot2::guide_legend(order = 1), edge_width = ggplot2::guide_legend(order = 2)) +
      # Nudge the legend over
      ggplot2::theme(legend.box.spacing = ggplot2::unit(0.75, "in"),
                     plot.margin = ggplot2::margin(0.1, 0, 0.1, 0.75, "in"))
  )
})

test_that("Parkinsons", {

  arm_net <- set_agd_arm(parkinsons,
                         study = studyn,
                         trt = trtn,
                         y = y,
                         se = se,
                         sample_size = n)

  expect_doppelganger("Parkinsons arm network",
    plot(arm_net, weight_edges = TRUE, weight_nodes = TRUE) +
    ggplot2::guides(size = ggplot2::guide_legend(order = 1), edge_width = ggplot2::guide_legend(order = 2))
  )

  contr_net <- set_agd_contrast(parkinsons,
                                study = studyn,
                                trt = trtn,
                                y = diff,
                                se = se_diff,
                                sample_size = n)

  expect_doppelganger("Parkinsons contrast network",
    plot(contr_net, weight_edges = TRUE, weight_nodes = TRUE) +
      ggplot2::guides(size = ggplot2::guide_legend(order = 1), edge_width = ggplot2::guide_legend(order = 2))
  )

  studies <- parkinsons$studyn
  (parkinsons_arm <- parkinsons[studies %in% 1:3, ])
  (parkinsons_contr <- parkinsons[studies %in% 4:7, ])

  mix_arm_net <- set_agd_arm(parkinsons_arm,
                             study = studyn,
                             trt = trtn,
                             y = y,
                             se = se,
                             sample_size = n)

  mix_contr_net <- set_agd_contrast(parkinsons_contr,
                                    study = studyn,
                                    trt = trtn,
                                    y = diff,
                                    se = se_diff,
                                    sample_size = n)

  mix_net <- combine_network(mix_arm_net, mix_contr_net)

  expect_doppelganger("Parkinsons mixed network",
    plot(mix_net, weight_edges = TRUE, weight_nodes = TRUE) +
      ggplot2::guides(size = ggplot2::guide_legend(order = 1), edge_width = ggplot2::guide_legend(order = 2))
  )
})

test_that("Plaque psoriasis", {

  # IPD studies
  pso_ipd <- plaque_psoriasis_ipd %>%
    mutate(
      # Variable transformations
      bsa = bsa / 100,
      weight = weight / 10,
      durnpso = durnpso / 10,
      prevsys = as.numeric(prevsys),
      psa = as.numeric(psa),
      # Treatment classes
      trtclass = case_when(trtn == 1 ~ "Placebo",
                           trtn %in% c(2, 3, 5, 6) ~ "IL-17 blocker",
                           trtn == 4 ~ "TNFa blocker",
                           trtn == 7 ~ "IL-12/23 blocker"),
      # Check complete cases for covariates of interest
      is_complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
    ) %>%
    arrange(studyc, trtn) %>%
    filter(is_complete)

  # AgD studies
  pso_agd <- plaque_psoriasis_agd %>%
    mutate(
      # Variable transformations
      bsa_mean = bsa_mean / 100,
      bsa_sd = bsa_sd / 100,
      weight_mean = weight_mean / 10,
      weight_sd = weight_sd / 10,
      durnpso_mean = durnpso_mean / 10,
      durnpso_sd = durnpso_sd / 10,
      prevsys = prevsys / 100,
      psa = psa / 100,
      # Treatment classes
      trtclass = case_when(trtn == 1 ~ "Placebo",
                           trtn %in% c(2, 3, 5, 6) ~ "IL-17 blocker",
                           trtn == 4 ~ "TNFa blocker",
                           trtn == 7 ~ "IL-12/23 blocker")
    ) %>%
    arrange(studyc, trtn)

  pso_net <- combine_network(
    set_ipd(pso_ipd,
            study = studyc,
            trt = trtc,
            r = multi(r0 = 1,
                      PASI75 = pasi75,
                      PASI90 = pasi90,
                      PASI100 = pasi100,
                      type = "ordered", inclusive = TRUE),
            trt_class = trtclass),
    set_agd_arm(pso_agd,
                study = studyc,
                trt = trtc,
                r = multi(r0 = pasi75_n,
                          PASI75 = pasi75_r,
                          PASI90 = pasi90_r,
                          PASI100 = pasi100_r,
                          type = "ordered", inclusive = TRUE),
                trt_class = trtclass)
  )


  class_pal <- c(`IL-12/23 blocker` = "#D95F02",
                 `IL-17 blocker` = "#7570B3",
                 `Placebo` = "#E7298A",
                 `TNFa blocker` = "#E6AB02")


  expect_doppelganger("Plaque psoriasis network",
    plot(pso_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE, nudge = 0.1) +
      ggraph::scale_edge_colour_manual("Data",
                                       values = c(AgD = "#113259", IPD = "#55A480")) +
      scale_fill_manual("Treatment class",
                        values = class_pal,
                        aesthetics = c("fill", "colour")) +
      guides(size = guide_legend(order = 1),
             edge_colour = guide_legend(override.aes = list(edge_width = 2), order = 2),
             fill = guide_legend(override.aes = list(size = 2), order = 3),
             color = guide_legend(order = 3),
             edge_width = guide_legend(order = 4))
  )
})

test_that("Smoking", {

  smknet <- set_agd_arm(smoking,
                        study = studyn,
                        trt = trtc,
                        r = r,
                        n = n,
                        trt_ref = "No intervention")

  expect_doppelganger("Smoking network",
    plot(smknet, weight_edges = TRUE, weight_nodes = TRUE) + ggplot2::theme(plot.margin = ggplot2::unit(c(1, 1, 1, 6), "lines"))
  )
})

test_that("Statins", {

  statin_net <- set_agd_arm(statins,
                            study = studyc,
                            trt = trtc,
                            r = r,
                            n = n,
                            trt_ref = "Placebo")

  expect_doppelganger("Statins network",
    plot(statin_net)
  )
})

test_that("Thrombolytics", {

  thrombo_net <- set_agd_arm(thrombolytics,
                             study = studyn,
                             trt = trtc,
                             r = r,
                             n = n)

  expect_doppelganger("Thrombolytics network",
    plot(thrombo_net, weight_edges = TRUE, weight_nodes = TRUE) +
      ggplot2::guides(size = ggplot2::guide_legend(order = 2), edge_width = ggplot2::guide_legend(order = 1))
  )

  # Make trtf factor to order treatments in same way as Dias analysis
  trts <- dplyr::distinct(thrombolytics, trtn, trtc)
  trts <- dplyr::arrange(trts, trtn)
  thrombolytics$trtf <- factor(thrombolytics$trtn, levels = trts$trtn, labels = trts$trtc)
  thrombo_net2 <- set_agd_arm(thrombolytics,
                              study = studyn,
                              trt = trtf,
                              r = r,
                              n = n)

  expect_doppelganger("Thrombolytics Dias network",
                      plot(thrombo_net2, weight_edges = TRUE, weight_nodes = TRUE) +
                        ggplot2::guides(size = ggplot2::guide_legend(order = 2), edge_width = ggplot2::guide_legend(order = 1))
  )
})

test_that("Transfusion", {

  tr_net <- set_agd_arm(transfusion,
                        study = studyc,
                        trt = trtc,
                        r = r,
                        n = n,
                        trt_ref = "Control")

  expect_doppelganger("Transfusion network",
    plot(tr_net)
  )
})

test_that("NDMM", {

  ndmm_ipd$trtclass <- forcats::fct_collapse(ndmm_ipd$trtf,
                                             Placebo = "Pbo",
                                             Active = c("Len", "Thal"))

  ndmm_agd$trtclass <- forcats::fct_collapse(ndmm_agd$trtf,
                                             Placebo = "Pbo",
                                             Active = c("Len", "Thal"))

  ndmm_net <- combine_network(
    set_ipd(ndmm_ipd,
            study = studyf,
            trt = trtf,
            trt_class = trtclass,
            Surv = Surv(eventtime, status)),
    set_agd_surv(ndmm_agd,
                 study = studyf,
                 trt = trtf,
                 trt_class = trtclass,
                 Surv = Surv(eventtime, status),
                 covariates = ndmm_agd_covs)
  )

  ndmm_net <- add_integration(ndmm_net,
                              age = distr(qgamma, mean = age_mean, sd = age_sd),
                              iss_stage3 = distr(qbern, iss_stage3),
                              response_cr_vgpr = distr(qbern, response_cr_vgpr),
                              male = distr(qbern, male))

  expect_doppelganger("NDMM network",
    plot(ndmm_net,
         weight_nodes = TRUE,
         weight_edges = TRUE,
         # Nudge treatment labels away from nodes
         nudge = 0.1,
         # Manual layout
         layout = data.frame(x = c(0, -1, 1),
                             y = c(-0.5, 0, 0))) +
      guides(edge_colour = guide_legend(override.aes = list(edge_width = 2))) +
      theme(legend.position = "bottom", legend.direction = "vertical")
  )
})

test_that("Duplicated study arms plotted correctly", {

  dat <- data.frame(study = "a",
                    trt = c(1, 2, 2, 2),
                    r = 1, n = 1)

  net1 <- set_agd_arm(dat, study, trt, r = r, n = n)
  expect_doppelganger("Duplicated AgD arms network", plot(net1))


  net2 <- set_ipd(dat, study, trt, r = r)
  expect_doppelganger("Duplicated IPD arms network", plot(net2))

  dat_c <- data.frame(study = "a",
                      trt = c(1, 2, 2, 2),
                      y = c(NA, 1, 1, 1),
                      se = c(0.5, 1, 1, 1))
  net3 <- set_agd_contrast(dat_c, study, trt, y = y, se = se)
  expect_doppelganger("Duplicated AgD contrasts network", plot(net3))
})

# Generated by vignette example_plaque_psoriasis.Rmd: do not edit by hand
# Instead edit example_plaque_psoriasis.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE, eval_multinomial = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------


## ----setup----------------------------------------------------------------------------------------
library(multinma)
library(dplyr)      # dplyr and tidyr for data manipulation
library(tidyr)
library(ggplot2)    # ggplot2 for plotting covariate distributions

## ----eval = FALSE---------------------------------------------------------------------------------
## options(mc.cores = parallel::detectCores())

## ----echo = FALSE---------------------------------------------------------------------------------
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc %in% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)


## -------------------------------------------------------------------------------------------------
pso_ipd <- pso_ipd %>% 
  mutate(# Variable transformations
         bsa = bsa / 100,
         prevsys = as.numeric(prevsys),
         psa = as.numeric(psa),
         weight = weight / 10,
         durnpso = durnpso / 10,
         # Treatment classes
         trtclass = case_when(trtn == 1 ~ "Placebo",
                              trtn %in% c(2, 3, 5, 6) ~ "IL blocker",
                              trtn == 4 ~ "TNFa blocker"),
         # Check complete cases for covariates of interest
         complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd %>% 
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                              trtn %in% c(2, 3, 5, 6) ~ "IL blocker",
                              trtn == 4 ~ "TNFa blocker")
  )


## -------------------------------------------------------------------------------------------------
sum(!pso_ipd$complete)
mean(!pso_ipd$complete)


## -------------------------------------------------------------------------------------------------
pso_ipd <- filter(pso_ipd, complete)


## -------------------------------------------------------------------------------------------------
pso_net <- combine_network(
  set_ipd(pso_ipd, 
          study = studyc, 
          trt = trtc, 
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd, 
              study = studyc, 
              trt = trtc, 
              r = pasi75_r, 
              n = pasi75_n,
              trt_class = trtclass)
)

pso_net


## ----pso_network_plot, fig.width=8, fig.height=6--------------------------------------------------
plot(pso_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE) + 
  ggplot2::theme(legend.position = "bottom", legend.box = "vertical")


## ----pso_covariate_plot---------------------------------------------------------------------------
# Get mean and sd of covariates in each study
ipd_summary <- pso_ipd %>% 
  group_by(studyc) %>% 
  summarise_at(vars(weight, durnpso, bsa), list(mean = mean, sd = sd, min = min, max = max)) %>% 
  pivot_longer(weight_mean:bsa_max, names_sep = "_", names_to = c("covariate", ".value")) %>% 
  # Assign distributions
  mutate(dist = recode(covariate,
                       bsa = "dlogitnorm",
                       durnpso = "dgamma",
                       weight = "dgamma")) %>% 
  # Compute density curves
  group_by(studyc, covariate) %>% 
  mutate(value = if_else(dist == "dlogitnorm",
                         list(seq(0, 1, length.out = 101)),
                         list(seq(min*0.8, max*1.2, length.out = 101)))) %>% 
  unnest(cols = value) %>% 
  mutate(dens = eval(call(first(dist), x = value, mean = first(mean), sd = first(sd))))

# Plot histograms and assumed densities
pso_ipd %>% 
  pivot_longer(c(weight, durnpso, bsa), names_to = "covariate", values_to = "value") %>% 
ggplot(aes(x = value)) +
  geom_histogram(aes(y = after_stat(density)), 
                 binwidth = function(x) diff(range(x)) / nclass.Sturges(x),
                 boundary = 0,
                 fill = "grey50") +
  geom_line(aes(y = dens), data = ipd_summary,
            colour = "darkred", linewidth = 0.5) +
  facet_wrap(~studyc + covariate, scales = "free", ncol = 3) +
  theme_multinma()


## -------------------------------------------------------------------------------------------------
pso_net <- add_integration(pso_net,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  n_int = 64
)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 10))


## -------------------------------------------------------------------------------------------------
pso_fit_FE <- nma(pso_net, 
                  trt_effects = "fixed",
                  link = "probit", 
                  likelihood = "bernoulli2",
                  regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
                  class_interactions = "common",
                  prior_intercept = normal(scale = 10),
                  prior_trt = normal(scale = 10),
                  prior_reg = normal(scale = 10),
                  init_r = 0.1,
                  QR = TRUE)


## -------------------------------------------------------------------------------------------------
print(pso_fit_FE)


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(pso_fit_FE, pars = c("d", "beta", "mu"))


## ----pso_FE_pp_plot, fig.width=8, fig.height=6----------------------------------------------------
plot_prior_posterior(pso_fit_FE, prior = c("intercept", "trt", "reg"))


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 10))
summary(half_normal(scale = 2.5))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## pso_fit_RE <- nma(pso_net,
##                   trt_effects = "random",
##                   link = "probit",
##                   likelihood = "bernoulli2",
##                   regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
##                   class_interactions = "common",
##                   prior_intercept = normal(scale = 10),
##                   prior_trt = normal(scale = 10),
##                   prior_reg = normal(scale = 10),
##                   prior_het = half_normal(scale = 2.5),
##                   init_r = 0.1,
##                   QR = TRUE)


## ----eval=!params$run_tests-----------------------------------------------------------------------
## print(pso_fit_RE)


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(pso_fit_RE, pars = c("d", "beta", "tau", "mu", "delta"))


## ----pso_RE_pairs, eval=!params$run_tests---------------------------------------------------------
## pairs(pso_fit_RE, pars = c("delta[UNCOVER-2: ETN]", "d[ETN]", "tau", "lp__"))


## ----pso_RE_pp_plot, eval=!params$run_tests, fig.width=8, fig.height=6----------------------------
## plot_prior_posterior(pso_fit_RE, prior = c("intercept", "trt", "reg", "het"))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## (pso_dic_FE <- dic(pso_fit_FE))
## (pso_dic_RE <- dic(pso_fit_RE))

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
(pso_dic_FE <- dic(pso_fit_FE))


## -------------------------------------------------------------------------------------------------
plot(pso_fit_FE,
     pars = "beta",
     stat = "halfeye",
     ref_line = 0)


## ----pso_releff_FE--------------------------------------------------------------------------------
(pso_releff_FE <- relative_effects(pso_fit_FE))
plot(pso_releff_FE, ref_line = 0)


## ----pso_pred_FE----------------------------------------------------------------------------------
(pso_pred_FE <- predict(pso_fit_FE, type = "response"))
plot(pso_pred_FE, ref_line = c(0, 1))


## ----pso_ranks_FE---------------------------------------------------------------------------------
(pso_ranks_FE <- posterior_ranks(pso_fit_FE, lower_better = FALSE))
plot(pso_ranks_FE)


## ----pso_rankprobs_FE-----------------------------------------------------------------------------
(pso_rankprobs_FE <- posterior_rank_probs(pso_fit_FE, lower_better = FALSE))
plot(pso_rankprobs_FE)


## ----pso_cumrankprobs_FE--------------------------------------------------------------------------
(pso_cumrankprobs_FE <- posterior_rank_probs(pso_fit_FE, lower_better = FALSE, cumulative = TRUE))
plot(pso_cumrankprobs_FE)


## -------------------------------------------------------------------------------------------------
new_agd_means <- tibble(
  bsa = 0.6,
  prevsys = 0.1,
  psa = 0.2,
  weight = 10,
  durnpso = 3)


## ----pso_releff_FE_new----------------------------------------------------------------------------
(pso_releff_FE_new <- relative_effects(pso_fit_FE, newdata = new_agd_means))
plot(pso_releff_FE_new, ref_line = 0)


## -------------------------------------------------------------------------------------------------
new_agd_int <- tibble(
  bsa_mean = 0.6,
  bsa_sd = 0.3,
  prevsys = 0.1,
  psa = 0.2,
  weight_mean = 10,
  weight_sd = 1,
  durnpso_mean = 3,
  durnpso_sd = 1
)


## -------------------------------------------------------------------------------------------------
new_agd_int <- add_integration(new_agd_int,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  cor = pso_net$int_cor,
  n_int = 64)


## ----pso_pred_FE_new------------------------------------------------------------------------------
(pso_pred_FE_new <- predict(pso_fit_FE, 
                            type = "response",
                            newdata = new_agd_int,
                            baseline = distr(qnorm, -1.75, 0.08)))
plot(pso_pred_FE_new, ref_line = c(0, 1))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## # IPD studies
## pso_ipd <- plaque_psoriasis_ipd %>%
##   mutate(
##     # Variable transformations
##     bsa = bsa / 100,
##     weight = weight / 10,
##     durnpso = durnpso / 10,
##     prevsys = as.numeric(prevsys),
##     psa = as.numeric(psa),
##     # Treatment classes
##     trtclass = case_when(trtn == 1 ~ "Placebo",
##                          trtn %in% c(2, 3, 5, 6) ~ "IL-17 blocker",
##                          trtn == 4 ~ "TNFa blocker",
##                          trtn == 7 ~ "IL-12/23 blocker"),
##     # Check complete cases for covariates of interest
##     is_complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
##   ) %>%
##   arrange(studyc, trtn)
## 
## # AgD studies
## pso_agd <- plaque_psoriasis_agd %>%
##   mutate(
##     # Variable transformations
##     bsa_mean = bsa_mean / 100,
##     bsa_sd = bsa_sd / 100,
##     weight_mean = weight_mean / 10,
##     weight_sd = weight_sd / 10,
##     durnpso_mean = durnpso_mean / 10,
##     durnpso_sd = durnpso_sd / 10,
##     prevsys = prevsys / 100,
##     psa = psa / 100,
##     # Treatment classes
##     trtclass = case_when(trtn == 1 ~ "Placebo",
##                          trtn %in% c(2, 3, 5, 6) ~ "IL-17 blocker",
##                          trtn == 4 ~ "TNFa blocker",
##                          trtn == 7 ~ "IL-12/23 blocker")
##     ) %>%
##   arrange(studyc, trtn)


## ----eval=!params$run_tests-----------------------------------------------------------------------
## pso_ipd %>%
##   group_by(studyc) %>%
##   summarise(n_total = n(),
##             n_missing = sum(!is_complete),
##             pct_missing = mean(!is_complete) * 100)
## 
## pso_ipd <- filter(pso_ipd, is_complete)


## ----eval=!params$run_tests-----------------------------------------------------------------------
## pso_net <- combine_network(
##   set_ipd(pso_ipd,
##     study = studyc,
##     trt = trtc,
##     r = multi(r0 = 1,
##               PASI75 = pasi75,
##               PASI90 = pasi90,
##               PASI100 = pasi100,
##               type = "ordered", inclusive = TRUE),
##     trt_class = trtclass),
##   set_agd_arm(pso_agd,
##     study = studyc,
##     trt = trtc,
##     r = multi(r0 = pasi75_n,
##               PASI75 = pasi75_r,
##               PASI90 = pasi90_r,
##               PASI100 = pasi100_r,
##               type = "ordered", inclusive = TRUE),
##     trt_class = trtclass)
## )
## 
## pso_net


## ----pso-full-network, eval=!params$run_tests, fig.width=8, fig.height=6--------------------------
## class_pal <- c("#D95F02", "#7570B3", "#E7298A", "#E6AB02")
## 
## plot(pso_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE, nudge = 0.1) +
##   ggraph::scale_edge_colour_manual("Data",
##                                    values = c(AgD = "#113259", IPD = "#55A480")) +
##   scale_fill_manual("Treatment class",
##                     values = class_pal,
##                     aesthetics = c("fill", "colour")) +
##   guides(edge_colour = guide_legend(override.aes = list(edge_width = 2)),
##          fill = guide_legend(override.aes = list(size = 2)))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## pso_net <- add_integration(pso_net,
##   durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
##   prevsys = distr(qbern, prob = prevsys),
##   bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
##   weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
##   psa = distr(qbern, prob = psa),
##   n_int = 64)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_FE <- nma(pso_net,
##                   trt_effects = "fixed",
##                   link = "probit",
##                   regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
##                   class_interactions = "common",
##                   prior_intercept = normal(scale = 10),
##                   prior_trt = normal(scale = 10),
##                   prior_reg = normal(scale = 10),
##                   prior_aux = flat(),
##                   QR = TRUE,
##                   init_r = 0.5)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_FE


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_RE <- nma(pso_net,
##                   trt_effects = "random",
##                   link = "probit",
##                   regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
##                   class_interactions = "common",
##                   prior_intercept = normal(scale = 10),
##                   prior_trt = normal(scale = 10),
##                   prior_reg = normal(scale = 10),
##                   prior_aux = flat(),
##                   prior_het = half_normal(scale = 2.5),
##                   QR = TRUE,
##                   init_r = 0.5)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_RE


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_dic_FE <- dic(pso_fit_FE))


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_dic_RE <- dic(pso_fit_RE))


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_UME <- nma(pso_net,
##                    trt_effects = "fixed",
##                    consistency = "ume",
##                    link = "probit",
##                    regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
##                    class_interactions = "common",
##                    prior_intercept = normal(scale = 10),
##                    prior_trt = normal(scale = 10),
##                    prior_reg = normal(scale = 10),
##                    prior_aux = flat(),
##                    QR = TRUE,
##                    init_r = 0.5)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_fit_UME


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_dic_FE


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_dic_UME <- dic(pso_fit_UME))


## ----pso-full-dev-dev, eval=!params$run_tests && params$eval_multinomial--------------------------
## plot(pso_dic_FE, pso_dic_UME, show_uncertainty = FALSE) +
##   xlab("Residual deviance - consistency model") +
##   ylab("Residual deviance - inconsistency (UME) model")


## ----eval=!params$run_tests-----------------------------------------------------------------------
## data.frame(classes = pso_net$classes, treatments = pso_net$treatments)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## noSEM_mods <- list(
##   durnpso = ~(prevsys + bsa + weight + psa)*.trtclass + durnpso*.trt,
##   prevsys = ~(durnpso + bsa + weight + psa)*.trtclass + prevsys*.trt,
##   bsa = ~(durnpso + prevsys + weight + psa)*.trtclass + bsa*.trt,
##   weight = ~(durnpso + prevsys + bsa + psa)*.trtclass + weight*.trt,
##   psa = ~(durnpso + prevsys + bsa + weight)*.trtclass + psa*.trt
##   )
## 
## noSEM_fits <- noSEM_mods
## 
## for (m in 1:length(noSEM_mods)) {
##   cat("Fitting model with independent interactions for", names(noSEM_mods)[m], "\n")
## 
##   noSEM_fits[[m]] <-
##     nma(pso_net,
##         trt_effects = "fixed",
##         link = "probit",
##         regression = noSEM_mods[[m]],
##         class_interactions = "independent",
##         prior_intercept = normal(scale = 10),
##         prior_trt = normal(scale = 10),
##         prior_reg = normal(scale = 10),
##         prior_aux = flat(),
##         QR = TRUE,
##         init_r = 0.5,
##         # Using save_warmup = FALSE reduces memory footprint when
##         # fitting many models in one session
##         save_warmup = FALSE)
## }


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## pso_dic_FE


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## lapply(noSEM_fits, dic)


## ----pso-full-relaxing-SEM, eval=!params$run_tests && params$eval_multinomial, fig.height=6, fig.width=8----
## library(purrr)
## library(stringr)
## library(forcats)
## 
## # Extract draws from relaxed models
## imap_dfr(noSEM_fits,
##         ~as_tibble(as.matrix(.x, pars = "beta")) %>%
##            pivot_longer(cols = everything(), names_to = "parameter", values_to = "value") %>%
##            filter(str_detect(parameter, paste0("(IXE|SEC).+:", .y))) %>%
##            mutate(model = .y)) %>%
## 
##   # Add in draws from the original model
##   bind_rows(
##     as_tibble(as.matrix(pso_fit_FE, pars = "beta")) %>%
##     pivot_longer(cols = everything(), names_to = "parameter", values_to = "value") %>%
##     filter(str_detect(parameter, ":.+IL\\-17 blocker")) %>%
##     mutate(model = "all")
##   ) %>%
## 
##   mutate(
##     # Rescale BSA to per 10%
##     value = if_else(str_detect(parameter, "bsa"), value / 10, value),
##     # Create labels
##     covariate = str_extract(parameter, "durnpso|prevsys|bsa|weight|psa"),
##     covariatef = recode_factor(covariate,
##                                durnpso = "Duration of psoriasis, per 10 years",
##                                prevsys = "Previous systemic use",
##                                bsa = "Body surface area, per 10%",
##                                weight = "Weight, per 10 kg",
##                                psa = "Psoriatic arthritis"),
##     treatment = str_remove(str_extract(parameter, "\\.trt(class)?.+?(?=[\\]:])"),
##                            "\\.trt(class)?"),
##     Interactions = fct_collapse(factor(model),
##                                 Common = "all",
##                                 other_level = "Independent")) %>%
## 
## # Plot
## ggplot(aes(x = value, y = fct_rev(treatment), colour = Interactions, fill = Interactions)) +
##   geom_vline(xintercept = 0, colour = "grey70") +
##   ggdist::stat_halfeye(normalize = "panels", slab_alpha = 0.3, .width = c(0, 0.95)) +
##   facet_wrap("covariatef", scales = "free") +
##   xlab("Interaction effect (SMD)") +
##   ylab("Treatment / Class") +
##   scale_colour_manual(values = c(Common = "#7B3294", Independent = "#91D388"),
##                       aesthetics = c("colour", "fill")) +
##   theme_multinma() +
##   theme(legend.position = c(0.85, 0.2))


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_releff_FE <- relative_effects(pso_fit_FE))


## ----pso-full-releff, eval=!params$run_tests && params$eval_multinomial, fig.height=8, fig.width=6----
## plot(pso_releff_FE, ref_line = 0)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_pred_FE <- predict(pso_fit_FE, type = "response"))


## ----pso-full-pred, eval=!params$run_tests && params$eval_multinomial, fig.width=6, fig.height=8----
## plot(pso_pred_FE, ref_line = c(0, 1))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## new_agd_means <- tibble::tribble(
##              ~study, ~covariate,  ~mean,   ~sd,
##           "PsoBest",      "bsa",     24,  20.5,
##           "PsoBest",  "durnpso",   18.2,  14.1,
##           "PsoBest",  "prevsys",   0.54,    NA,
##           "PsoBest",      "psa",  0.207,    NA,
##           "PsoBest",   "weight",     85,  19.1,
##          "PROSPECT",      "bsa",   18.7,  18.4,
##          "PROSPECT",  "durnpso",   19.6,  13.5,
##          "PROSPECT",  "prevsys", 0.9095,    NA,
##          "PROSPECT",      "psa",  0.202,    NA,
##          "PROSPECT",   "weight",   87.5,  20.3,
##   "Chiricozzi 2019",      "bsa",     23, 16.79,
##   "Chiricozzi 2019",  "durnpso",  16.93, 10.82,
##   "Chiricozzi 2019",  "prevsys", 0.9061,    NA,
##   "Chiricozzi 2019",      "psa", 0.2152,    NA,
##   "Chiricozzi 2019",   "weight",   78.3, 15.87
##   ) %>%
##   # Tidy up
##   pivot_wider(id_cols = study,
##               names_from = covariate,
##               values_from = c(mean, sd),
##               names_glue = "{covariate}_{.value}") %>%
##   # Rescale as per analysis
##   transmute(study,
##             bsa_mean = bsa_mean / 100,
##             bsa_sd = bsa_sd / 100,
##             weight_mean = weight_mean / 10,
##             weight_sd = weight_sd / 10,
##             durnpso_mean = durnpso_mean / 10,
##             durnpso_sd = durnpso_sd / 10,
##             prevsys = prevsys_mean,
##             psa = psa_mean)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_releff_FE_new <- relative_effects(pso_fit_FE,
##                                        newdata = transmute(new_agd_means,
##                                                            study,
##                                                            bsa = bsa_mean,
##                                                            weight = weight_mean,
##                                                            durnpso = durnpso_mean,
##                                                            prevsys,
##                                                            psa),
##                                        study = study))


## ----pso-full-releff-new, eval=!params$run_tests && params$eval_multinomial-----------------------
## plot(pso_releff_FE_new, ref_line = 0) + facet_wrap("Study")


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## new_agd_int <- add_integration(filter(new_agd_means, study != "PsoBest"),
##                                durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
##                                prevsys = distr(qbern, prob = prevsys),
##                                bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
##                                weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
##                                psa = distr(qbern, prob = psa),
##                                n_int = 64,
##                                cor = pso_net$int_cor)


## ----eval=!params$run_tests && params$eval_multinomial--------------------------------------------
## (pso_pred_FE_new <- predict(pso_fit_FE,
##         type = "response",
##         newdata = new_agd_int,
##         study = study,
##         baseline = list(PROSPECT = distr(qbeta, 1156, 1509-1156),
##                         "Chiricozzi 2019" = distr(qbeta, 243, 330-243)),
##         baseline_type = "response",
##         baseline_level = "aggregate",
##         baseline_trt = "SEC_300"))


## ----pso-full-pred-new, eval=!params$run_tests && params$eval_multinomial-------------------------
## plot(pso_pred_FE_new, ref_line = c(0, 1)) +
##   facet_grid(rows = "Study") +
##   aes(colour = Category) +
##   scale_colour_brewer(palette = "Blues")


## ----pso_tests, include=FALSE, eval=params$run_tests----------------------------------------------
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1

# FE model parameters
test_fe <- tribble(
  ~parameter                            , ~mean, ~sd , ~`2.5%`, ~`50%`, ~`97.5%`,
  "beta[durnpso]"                       , 0.05 , 0.06, -0.08  , 0.05  , 0.17    ,
  "beta[prevsys]"                       , -0.13, 0.16, -0.44  , -0.13 , 0.17    ,
  "beta[bsa]"                           , -0.06, 0.45, -0.98  , -0.05 , 0.78    ,
  "beta[weight]"                        , 0.04 , 0.03, -0.02  , 0.04  , 0.10    ,
  "beta[psa]"                           , -0.08, 0.17, -0.42  , -0.08 , 0.25    ,
  "beta[durnpso:.trtclassTNFa blocker]", -0.03, 0.08, -0.18  , -0.03 , 0.12    ,
  "beta[durnpso:.trtclassIL blocker]"  , -0.01, 0.07, -0.15  , -0.01 , 0.12    ,
  "beta[prevsys:.trtclassTNFa blocker]", 0.19 , 0.19, -0.19  , 0.19  , 0.56    ,
  "beta[prevsys:.trtclassIL blocker]"  , 0.06 , 0.17, -0.28  , 0.06  , 0.40    ,
  "beta[bsa:.trtclassTNFa blocker]"    , 0.05 , 0.52, -0.95  , 0.04  , 1.10    ,
  "beta[bsa:.trtclassIL blocker]"      , 0.29 , 0.49, -0.64  , 0.29  , 1.27    ,
  "beta[weight:.trtclassTNFa blocker]" , -0.17, 0.04, -0.23  , -0.17 , -0.10   ,
  "beta[weight:.trtclassIL blocker]"   , -0.10, 0.03, -0.16  , -0.10 , -0.03   ,
  "beta[psa:.trtclassTNFa blocker]"    , -0.06, 0.21, -0.46  , -0.06 , 0.37    ,
  "beta[psa:.trtclassIL blocker]"      , 0.00 , 0.18, -0.35  , 0.00  , 0.37    ,
  "d[ETN]"                              , 1.55 , 0.08, 1.39   , 1.55  , 1.72    ,
  "d[IXE_Q2W]"                          , 2.95 , 0.09, 2.79   , 2.95  , 3.13    ,
  "d[IXE_Q4W]"                          , 2.54 , 0.08, 2.38   , 2.54  , 2.71    ,
  "d[SEC_150]"                          , 2.14 , 0.11, 1.93   , 2.14  , 2.37    ,
  "d[SEC_300]"                          , 2.45 , 0.12, 2.22   , 2.45  , 2.69    )

summary_fe <- summary(pso_fit_FE, pars = c("beta", "d")) %>% 
  as_tibble() %>% 
  select(parameter, mean, sd, `2.5%`, `50%`, `97.5%`)

test_that("FE model parameters", {
  expect_equal(summary_fe, test_fe, tolerance = tol, check.attributes = FALSE)
})

# FE DIC
test_that("FE DIC", {
  expect_equivalent(pso_dic_FE$resdev, 3129.4, tolerance = tol_dic)
  expect_equivalent(pso_dic_FE$pd, 24.1, tolerance = tol_dic)
  expect_equivalent(pso_dic_FE$dic, 3153.5, tolerance = tol_dic)
})

# Population average relative effects in target population
test_releff_fe_new <- tribble(
  ~parameter         , ~mean, ~sd , ~`2.5%`, ~`50%`, ~`97.5%`,
  "d[New 1: ETN]"    , 1.25 , 0.24, 0.81 , 1.24, 1.73  ,
  "d[New 1: IXE_Q2W]", 2.89 , 0.23, 2.46 , 2.88, 3.36  ,
  "d[New 1: IXE_Q4W]", 2.48 , 0.23, 2.05 , 2.47, 2.94  ,
  "d[New 1: SEC_150]", 2.08 , 0.23, 1.64 , 2.07, 2.55  ,
  "d[New 1: SEC_300]", 2.39 , 0.23, 1.95 , 2.38, 2.86  )

summary_releff_fe_new <- pso_releff_FE_new %>% 
  as_tibble() %>% 
  select(parameter, mean, sd, `2.5%`, `50%`, `97.5%`)

test_that("FE PATE in target population", {
  expect_equal(summary_releff_fe_new, test_releff_fe_new, tolerance = tol, check.attributes = FALSE)
})

# Check construction of all contrasts
pso_releff_FE_all_contr <- relative_effects(pso_fit_FE, all_contrasts = TRUE)

# Reconstruct from basic contrasts in each study
dk <- function(study, trt, sims = pso_releff_FE$sims) {
  if (trt == "PBO") return(0)
  else return(sims[ , , paste0("d[", study, ": ", trt, "]"), drop = FALSE])
}

test_all_contr <- tibble(
  contr = pso_releff_FE_all_contr$summary$parameter,
  .study = factor(stringr::str_extract(contr, "(?<=\\[)(.+)(?=:)")),
  .trtb = factor(stringr::str_extract(contr, "(?<=\\: )(.+)(?= vs\\.)"), levels = levels(pso_net$treatments)),
  .trta = factor(stringr::str_extract(contr, "(?<=vs\\. )(.+)(?=\\])"), levels = levels(pso_net$treatments))
) %>% 
  rowwise() %>% 
  mutate(as_tibble(multinma:::summary.mcmc_array(dk(.study, .trtb) - dk(.study, .trta)))) %>% 
  select(.study, .trtb, .trta, parameter = contr, mean:Rhat)

test_that("Construction of all contrasts is correct", {
  ntrt <- nlevels(pso_net$treatments)
  nstudy <- nlevels(test_all_contr$.study)
  expect_equal(nrow(pso_releff_FE_all_contr$summary), nstudy * ntrt * (ntrt - 1) / 2)
  expect_equal(pso_releff_FE_all_contr$summary, test_all_contr, check.attributes = FALSE)
})

# Check all contrasts in target population
pso_releff_FE_all_contr_new <- relative_effects(pso_fit_FE, newdata = new_agd_means, all_contrasts = TRUE)

test_all_contr_new <- tibble(
  contr = pso_releff_FE_all_contr_new$summary$parameter,
  .study = factor(stringr::str_extract(contr, "(?<=\\[)(.+)(?=:)")),
  .trtb = factor(stringr::str_extract(contr, "(?<=\\: )(.+)(?= vs\\.)"), levels = levels(pso_net$treatments)),
  .trta = factor(stringr::str_extract(contr, "(?<=vs\\. )(.+)(?=\\])"), levels = levels(pso_net$treatments))
) %>% 
  rowwise() %>% 
  mutate(as_tibble(multinma:::summary.mcmc_array(dk(.study, .trtb, pso_releff_FE_new$sims) - dk(.study, .trta, pso_releff_FE_new$sims)))) %>% 
  select(.study, .trtb, .trta, parameter = contr, mean:Rhat)

test_that("Construction of all contrasts in target population is correct", {
  ntrt <- nlevels(pso_net$treatments)
  nstudy <- nlevels(test_all_contr_new$.study)
  expect_equal(nrow(pso_releff_FE_all_contr_new$summary), nstudy * ntrt * (ntrt - 1) / 2)
  expect_equal(pso_releff_FE_all_contr_new$summary, test_all_contr_new, check.attributes = FALSE)
})

# Population average response probabilities in target population
test_pred_fe_new <- tribble(
  ~parameter            , ~mean, ~sd , ~`2.5%`, ~`50%`, ~`97.5%`,
  "pred[New 1: PBO]"    , 0.06 , 0.03, 0.02   , 0.06  , 0.12    ,
  "pred[New 1: ETN]"    , 0.37 , 0.06, 0.26   , 0.37  , 0.48    ,
  "pred[New 1: IXE_Q2W]", 0.90 , 0.03, 0.84   , 0.90  , 0.94    , 
  "pred[New 1: IXE_Q4W]", 0.81 , 0.04, 0.72   , 0.81  , 0.88    ,
  "pred[New 1: SEC_150]", 0.68 , 0.06, 0.57   , 0.68  , 0.78    ,
  "pred[New 1: SEC_300]", 0.78 , 0.05, 0.68   , 0.78  , 0.86    )

summary_pred_fe_new <- pso_pred_FE_new %>% 
  as_tibble() %>% 
  select(parameter, mean, sd, `2.5%`, `50%`, `97.5%`)

test_that("FE average response probabilities in target population", {
  expect_equal(summary_pred_fe_new, test_pred_fe_new, tolerance = tol, check.attributes = FALSE)
})

test_that("Robust to custom options(contrasts) settings", {
  withr::with_options(list(contrasts = c(ordered = "contr.SAS", unordered = "contr.SAS")), {
    pso_fit_FE_SAS <- nma(pso_net, 
                          trt_effects = "fixed",
                          link = "probit", 
                          likelihood = "bernoulli2",
                          regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
                          class_interactions = "common",
                          prior_intercept = normal(scale = 10),
                          prior_trt = normal(scale = 10),
                          prior_reg = normal(scale = 10),
                          init_r = 0.1,
                          QR = TRUE)
    
    pso_fit_FE_SAS_releff <- as_tibble(relative_effects(pso_fit_FE_SAS))[, c("parameter", "mean", "sd")]
  })
  
  expect_equal(pso_fit_FE_SAS_releff,
               as_tibble(relative_effects(pso_fit_FE))[, c("parameter", "mean", "sd")],
               tolerance = tol)
})

test_that("Integration checks with int_check throw expected warnings", {
  local_edition(3)
  
  pso_net_lownint <- 
    suppressWarnings(add_integration(pso_net,
                    durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                    prevsys = distr(qbern, prob = prevsys),
                    bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
                    weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
                    psa = distr(qbern, prob = psa),
                    n_int = 2))

  expect_warning(
  expect_warning(
  expect_warning(
    nma(pso_net_lownint,
        trt_effects = "fixed",
        link = "probit", 
        likelihood = "bernoulli2",
        regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
        class_interactions = "common",
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10),
        prior_reg = normal(scale = 10),
        init_r = 0.1,
        QR = TRUE,
        ),
    class = "int_check_rhat"),
    class = "int_check_essb"),
    class = "int_check_esst")
})


# Force clean up
rm(list = ls())
gc()

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Only test gradients, no sampling
smk_fit_RE <- suppressWarnings(nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5),
                  iter = 10))

test_that("baseline argument", {
  m <- "should be specified using distr"
  expect_error(predict(smk_fit_RE, baseline = 1), m)
  expect_error(predict(smk_fit_RE, baseline = list("a")), m)
  expect_error(predict(smk_fit_RE, baseline = NA), m)

  expect_error(predict(smk_fit_RE, baseline = "a"), "`baseline` must match the name of an IPD or AgD \\(arm-based\\) study in the network")
})

test_that("trt_ref argument", {
  m <- "does not match a treatment in the network"
  expect_error(predict(smk_fit_RE, baseline = distr(qnorm, mean = -1, sd = 0.01), trt_ref = "a"), m)
  expect_error(predict(smk_fit_RE, baseline = distr(qnorm, mean = -1, sd = 0.01), trt_ref = 1), m)
  expect_error(predict(smk_fit_RE, baseline = distr(qnorm, mean = -1, sd = 0.01), trt_ref = list("a")), m)
  expect_error(predict(smk_fit_RE, baseline = distr(qnorm, mean = -1, sd = 0.01), trt_ref = NA), m)
})

test_that("probs argument", {
  m <- "numeric vector of probabilities"
  expect_error(predict(smk_fit_RE, probs = "a"), m)
  expect_error(predict(smk_fit_RE, probs = -1), m)
  expect_error(predict(smk_fit_RE, probs = 1.5), m)
  expect_error(predict(smk_fit_RE, probs = Inf), m)
  expect_error(predict(smk_fit_RE, probs = list()), m)
  expect_error(predict(smk_fit_RE, probs = NA), m)
  expect_error(predict(smk_fit_RE, probs = NULL), m)
})

test_that("summary argument", {
  m <- "should be TRUE or FALSE"
  expect_error(predict(smk_fit_RE, summary = "a"), m)
  expect_error(predict(smk_fit_RE, summary = 1), m)
  expect_error(predict(smk_fit_RE, summary = list()), m)
  expect_error(predict(smk_fit_RE, summary = NA), m)
  expect_error(predict(smk_fit_RE, summary = NULL), m)
})

test_that("expand argument", {
  m <- "should be TRUE or FALSE"
  expect_error(predict(smk_fit_RE, expand = "a"), m)
  expect_error(predict(smk_fit_RE, expand = 1), m)
  expect_error(predict(smk_fit_RE, expand = list()), m)
  expect_error(predict(smk_fit_RE, expand = NA), m)
  expect_error(predict(smk_fit_RE, expand = NULL), m)
})

test_that("newdata argument", {
  m <- "not a data frame"
  expect_error(predict(smk_fit_RE, newdata = "a"), m)
  expect_error(predict(smk_fit_RE, newdata = 1), m)
  expect_error(predict(smk_fit_RE, newdata = list()), m)
  expect_error(predict(smk_fit_RE, newdata = NA), m)
})

test_that("type argument", {
  m <- "must be one of"
  expect_error(predict(smk_fit_RE, type = "a"), m)
  expect_error(predict(smk_fit_RE, type = "lin"), m)

  m2 <- "must be a character vector"
  expect_error(predict(smk_fit_RE, type = 1), m2)
  expect_error(predict(smk_fit_RE, type = list("a")), m2)
  expect_error(predict(smk_fit_RE, type = NA), m2)
})

test_that("level argument", {
  m <- "must be one of"
  expect_error(predict(smk_fit_RE, level = "a"), m)
  expect_error(predict(smk_fit_RE, level = "agg"), m)

  m2 <- "must be a character vector"
  expect_error(predict(smk_fit_RE, level = 1), m2)
  expect_error(predict(smk_fit_RE, level = list("a")), m2)
  expect_error(predict(smk_fit_RE, level = NA), m2)

  expect_error(predict(smk_fit_RE, level = "individual"),
               "Cannot produce individual predictions without a regression model.")
})

test_that("baseline_type argument", {
  m <- "must be one of"
  expect_error(predict(smk_fit_RE, baseline_type = "a"), m)
  expect_error(predict(smk_fit_RE, baseline_type = "lin"), m)

  m2 <- "must be a character vector"
  expect_error(predict(smk_fit_RE, baseline_type = 1), m2)
  expect_error(predict(smk_fit_RE, baseline_type = list("a")), m2)
  expect_error(predict(smk_fit_RE, baseline_type = NA), m2)
})

test_that("baseline_level argument", {
  m <- "must be one of"
  expect_error(predict(smk_fit_RE, baseline_level = "a"), m)
  expect_error(predict(smk_fit_RE, baseline_level = "agg"), m)

  m2 <- "must be a character vector"
  expect_error(predict(smk_fit_RE, baseline_level = 1), m2)
  expect_error(predict(smk_fit_RE, baseline_level = list("a")), m2)
  expect_error(predict(smk_fit_RE, baseline_level = NA), m2)
})


skip_on_cran()  # Reduce CRAN check time

test_that(".study, .trt columns are correct", {
  pred1 <- tibble::as_tibble(predict(smk_fit_RE))
  expect_identical(paste0("pred[", pred1$.study, ": ", pred1$.trt, "]"),
                   pred1$parameter)

  pred2 <- tibble::as_tibble(predict(smk_fit_RE, baseline = distr(qnorm, 0, 1)))
  expect_identical(paste0("pred[", pred2$.trt, "]"),
                   pred2$parameter)
})

test_that("expand TRUE and FALSE", {
  pred1 <- tibble::as_tibble(predict(smk_fit_RE, expand = TRUE))
  expect_identical(pred1$parameter,
                   paste0("pred[",
                          tidyr::expand_grid(s = smk_net$studies, t = smk_net$treatments) %>%
                            dplyr::mutate(l = paste(s, t, sep = ": ")) %>%
                            dplyr::pull(l), "]"))

  pred2 <- tibble::as_tibble(predict(smk_fit_RE, expand = FALSE))
  expect_identical(pred2$parameter,
                   paste0("pred[",
                          dplyr::arrange(smk_net$agd_arm, .study, .trt) %>%
                            dplyr::mutate(l = paste(.study, .trt, sep = ": ")) %>%
                            dplyr::pull(l), "]"))

  pred3 <- tibble::as_tibble(predict(smk_fit_RE, expand = FALSE, baseline = "1"))
  expect_identical(pred3$parameter,
                   paste0("pred[",
                          dplyr::filter(smk_net$agd_arm, .study == "1")$.trt, "]"))

  expect_error(predict(smk_fit_RE, expand = FALSE, baseline = distr(qnorm, 0, 1)),
               "Provide `newdata` with a `\\.trt` column")

  pred4 <- tibble::as_tibble(predict(smk_fit_RE, expand = FALSE,
                                     baseline = distr(qnorm, 0, 1),
                                     newdata = data.frame(.trt = c("No intervention", "Self-help"))))
  expect_identical(pred4$parameter,
                   paste0("pred[", c("No intervention", "Self-help"), "]"))

  expect_error(predict(smk_fit_RE, expand = FALSE,
                       baseline = distr(qnorm, 0, 1),
                       newdata = data.frame(.trt = c("a", "b"))),
               "Treatments in `newdata` do not match those in the network")
})

pso_net <- set_ipd(plaque_psoriasis_ipd[complete.cases(plaque_psoriasis_ipd), ],
                   studyc, trtc,
                   r = pasi75)

# Only small number of samples to test
pso_fit <- suppressWarnings(nma(pso_net,
               trt_effects = "fixed",
               regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
               prior_intercept = normal(scale = 10),
               prior_trt = normal(scale = 10),
               prior_reg = normal(scale = 10),
               init_r = 0.1,
               iter = 10))

pso_new <- data.frame(durnpso = 10, prevsys = TRUE, bsa = 20, weight = 75, psa = FALSE, study = c("One", "Two"))

test_that("baseline and newdata for regression models", {
  m <- "Specify both `newdata` and `baseline`, or neither"
  expect_error(predict(pso_fit, newdata = pso_new), m)
  expect_error(predict(pso_fit, baseline = distr(qnorm, 1, 1)), m)
})

test_that("baseline argument", {
  m <- "should be a single distr\\(\\) specification or character string naming a study in the network, a list of such specifications, or NULL"
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = 1), m)
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = NA), m)

  m2 <- "or a list of length 2"
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = list(distr(qnorm, 1, 1),
                                                                                  distr(qnorm, 2, 1),
                                                                                  distr(qnorm, 3, 1))), m2)
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = list(One = distr(qnorm, 1, 1),
                                                                                  Two = distr(qnorm, 2, 1),
                                                                                  Three = distr(qnorm, 3, 1))), m2)

  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = list(One = distr(qnorm, 1, 1),
                                                                                  Three = distr(qnorm, 3, 1))),
               "must match all study names")

  m3 <- "must match the name of an IPD or AgD \\(arm-based\\) study in the network"
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = "a"), m3)
  expect_error(predict(pso_fit, study = study, newdata = pso_new, baseline = list("a")), m3)
})

test_that("newdata validation", {
  expect_error(predict(pso_fit, newdata = pso_new[-1], study = study, baseline = distr(qnorm, 1, 1)),
               'Regression variable "durnpso" not found in `newdata`')
  expect_error(predict(pso_fit, newdata = pso_new[-(1:2)], study = study, baseline = distr(qnorm, 1, 1)),
               'Regression variables "durnpso" and "prevsys" not found in `newdata`')

  make_bad <- function(x, vars = "durnpso") {
    bad <- pso_new
    bad[vars] <- x
    return(bad)
  }

  expect_error(predict(pso_fit, newdata = make_bad(NA), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso")
  expect_error(predict(pso_fit, newdata = make_bad(NaN), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso")
  expect_error(predict(pso_fit, newdata = make_bad(Inf), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso")

  expect_error(predict(pso_fit, newdata = make_bad(NA, c("durnpso", "psa")), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso, psa")
  expect_error(predict(pso_fit, newdata = make_bad(NaN, c("durnpso", "psa")), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso, psa")
  expect_error(predict(pso_fit, newdata = make_bad(Inf, c("durnpso", "psa")), study = study, baseline = distr(qnorm, 1, 1)),
               "missing or infinite values in `newdata`: durnpso, psa")
})

test_that(".study, .trt columns are correct", {
  pred1 <- tibble::as_tibble(predict(pso_fit))
  expect_identical(paste0("pred[", pred1$.study, ": ", pred1$.trt, "]"),
                   pred1$parameter)

  pred2 <- tibble::as_tibble(predict(pso_fit, newdata = pso_new, study = study, baseline = distr(qnorm, 1, 1)))
  expect_identical(paste0("pred[", pred2$.study, ": ", pred2$.trt, "]"),
                   pred2$parameter)
})


test_that("expand TRUE and FALSE", {
  pred1 <- tibble::as_tibble(predict(pso_fit, expand = TRUE))
  expect_identical(pred1$parameter,
                   paste0("pred[",
                          tidyr::expand_grid(s = pso_net$studies, t = pso_net$treatments) %>%
                            dplyr::mutate(l = paste(s, t, sep = ": ")) %>%
                            dplyr::pull(l), "]"))

  pred2 <- tibble::as_tibble(predict(pso_fit, expand = FALSE))
  expect_identical(pred2$parameter,
                   paste0("pred[",
                          dplyr::distinct(pso_net$ipd, .study, .trt) %>%
                            dplyr::arrange(.study, .trt) %>%
                            dplyr::mutate(l = paste(.study, .trt, sep = ": ")) %>%
                            dplyr::pull(l), "]"))

  pred3 <- tibble::as_tibble(predict(pso_fit, expand = FALSE, baseline = "UNCOVER-1",
                                     newdata = dplyr::mutate(pso_new, .trt = "IXE_Q2W")))
  expect_identical(pred3$parameter, "pred[New 1: IXE_Q2W]")

  expect_error(predict(pso_fit, expand = FALSE, baseline = distr(qnorm, 0, 1), newdata = pso_new),
               "`newdata` should have a `\\.trt` column")

  pred4 <- tibble::as_tibble(predict(pso_fit, expand = FALSE,
                                     baseline = distr(qnorm, 0, 1),
                                     study = study,
                                     newdata = dplyr::mutate(pso_new, .trt = "IXE_Q2W")))
  expect_identical(pred4$parameter,
                   paste0("pred[", c("One", "Two"), ": ", "IXE_Q2W", "]"))

  expect_error(predict(pso_fit, expand = FALSE,
                       baseline = distr(qnorm, 0, 1),
                       newdata = dplyr::mutate(pso_new, .trt = "a")),
               "Treatments in `newdata` do not match those in the network")

  pred5 <- tibble::as_tibble(predict(pso_fit, expand = TRUE, level = "individual"))
  expect_identical(pred5$parameter,
                   paste0("pred[",
                          dplyr::group_by(pso_net$ipd, .study) %>%
                            dplyr::mutate(id = 1:dplyr::n()) %>%
                            dplyr::select(-.trt) %>%
                            tidyr::expand(id, .trt = pso_net$treatments) %>%
                            dplyr::mutate(l = paste0(.study, ": ", .trt, ", ", id)) %>%
                            dplyr::pull(l), "]"))

  pred6 <- tibble::as_tibble(predict(pso_fit, expand = FALSE, level = "individual"))
  expect_identical(pred6$parameter,
                   paste0("pred[", dplyr::group_by(pso_net$ipd, .study, .trt) %>%
                            dplyr::mutate(l = paste0(.study, ": ", .trt, ", ", 1:dplyr::n())) %>%
                            dplyr::pull(l), "]"))
})


hta_net <- set_agd_arm(hta_psoriasis,
                       study = paste(studyc, year),
                       trt = trtc,
                       r = multi(r0 = sample_size - rowSums(cbind(PASI50, PASI75, PASI90), na.rm = TRUE),
                                 PASI50, PASI75, PASI90,
                                 inclusive = FALSE,
                                 type = "ordered"))

# Only small number of samples to test
hta_fit_FE <- suppressWarnings(nma(hta_net,
                               trt_effects = "fixed",
                               link = "probit",
                               prior_intercept = normal(scale = 100),
                               prior_trt = normal(scale = 10),
                               prior_aux = flat(),
                               iter = 10))

test_that(".study, .trt, .category columns are correct", {
  pred1 <- tibble::as_tibble(predict(hta_fit_FE))
  expect_identical(paste0("pred[", pred1$.study, ": ", pred1$.trt, ", ", pred1$.category, "]"),
                   pred1$parameter)

  pred2 <- tibble::as_tibble(predict(hta_fit_FE, baseline = distr(qnorm, 0, 1)))
  expect_identical(paste0("pred[", pred2$.trt, ", ", pred2$.category, "]"),
                   pred2$parameter)
})

ndmm_net <- combine_network(
  set_ipd(dplyr::slice_sample(ndmm_ipd, n = 10, by = c("study", "trt")),
          study, trt,
          Surv = Surv(eventtime/7, status),
          trt_class = trt != "Pbo"),
  set_agd_surv(dplyr::slice_sample(ndmm_agd, n= 10, by = c("study", "trt")),
               study, trt,
               Surv = Surv(eventtime/7, status),
               covariates = ndmm_agd_covs,
               trt_class = trt != "Pbo")
) %>%
  add_integration(age = distr(qnorm, age_mean, age_sd),
                  n_int = 5)

# Only small number of samples to test
ndmm_fit_weib <- suppressWarnings(nma(ndmm_net,
                                  likelihood = "weibull-aft",
                                  prior_intercept = normal(0, 100),
                                  prior_trt = normal(0, 10),
                                  prior_aux = half_normal(10),
                                  iter = 10))

ndmm_fit_weib_nphr <- suppressWarnings(nma(ndmm_net,
                                      likelihood = "weibull-aft",
                                      aux_regression = ~.trt,
                                      prior_intercept = normal(0, 100),
                                      prior_trt = normal(0, 10),
                                      prior_aux = half_normal(10),
                                      prior_aux_reg = normal(0, 10),
                                      iter = 10))

ndmm_fit_weib_nphs <- suppressWarnings(nma(ndmm_net,
                                           likelihood = "weibull-aft",
                                           aux_by = ".trt",
                                           prior_intercept = normal(0, 100),
                                           prior_trt = normal(0, 10),
                                           prior_aux = half_normal(10),
                                           iter = 10))

ndmm_fit_exp <- suppressWarnings(nma(ndmm_net,
                                     likelihood = "exponential",
                                     prior_intercept = normal(0, 100),
                                     prior_trt = normal(0, 10),
                                     iter = 10))

ndmm_fit_gengamma <- suppressWarnings(nma(ndmm_net,
                                          likelihood = "gengamma",
                                          prior_intercept = normal(0, 100),
                                          prior_trt = normal(0, 10),
                                          prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                          iter = 10))

ndmm_fit_gengamma_nphr <- suppressWarnings(nma(ndmm_net,
                                          likelihood = "gengamma",
                                          aux_regression = ~.trt,
                                          prior_intercept = normal(0, 100),
                                          prior_trt = normal(0, 10),
                                          prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                          prior_aux_reg = normal(0, 10),
                                          iter = 10))

ndmm_fit_gengamma_nphs <- suppressWarnings(nma(ndmm_net,
                                               likelihood = "gengamma",
                                               aux_by = ".trt",
                                               prior_intercept = normal(0, 100),
                                               prior_trt = normal(0, 10),
                                               prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                               iter = 10))

ndmm_fit_mspline <- suppressWarnings(nma(ndmm_net,
                                         likelihood = "mspline",
                                         prior_intercept = normal(0, 100),
                                         prior_trt = normal(0, 10),
                                         prior_aux = half_normal(1),
                                         iter = 10))

ndmm_fit_mspline_nphr <- suppressWarnings(nma(ndmm_net,
                                         likelihood = "mspline",
                                         aux_regression = ~.trt,
                                         prior_intercept = normal(0, 100),
                                         prior_trt = normal(0, 10),
                                         prior_aux = half_normal(1),
                                         prior_aux_reg = normal(0, 10),
                                         iter = 10))

ndmm_fit_mspline_nphs <- suppressWarnings(nma(ndmm_net,
                                              likelihood = "mspline",
                                              aux_by = ".trt",
                                              prior_intercept = normal(0, 100),
                                              prior_trt = normal(0, 10),
                                              prior_aux = half_normal(1),
                                              iter = 10))

ndmm_preddat <- dplyr::bind_rows(
  dplyr::transmute(ndmm_net$ipd, .study, .trt, .time = .Surv[, "time"]),
  tidyr::unnest(ndmm_net$agd_arm, cols = ".Surv") %>% dplyr::transmute(.study, .trt, .time = .Surv[, "time"]),
)

test_that(".study, .trt, .time columns are correct (weibull, no regression, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))

  # Check non-PH
  pred1.1_nphr <- tibble::as_tibble(predict(ndmm_fit_weib_nphr, type = "survival"))
  expect_equivalent(pred1.1_nphr[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphr$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::inner_join(preddat1,
                                     dplyr::distinct(ndmm_preddat, .study, .trt),
                                     by = c(".study", ".trt")) %>%
    dplyr::arrange(.study, .trt)

  pred1.1_nphs <- tibble::as_tibble(predict(ndmm_fit_weib_nphs, type = "survival"))
  expect_equivalent(pred1.1_nphs[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphs$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

})


test_that(".study, .trt, .time columns are correct (exponential, no regression, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (gengamma, no regression, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))

  # Check non-PH
  pred1.1_nphr <- tibble::as_tibble(predict(ndmm_fit_gengamma_nphr, type = "survival"))
  expect_equivalent(pred1.1_nphr[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphr$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::inner_join(preddat1,
                                     dplyr::distinct(ndmm_preddat, .study, .trt),
                                     by = c(".study", ".trt")) %>%
    dplyr::arrange(.study, .trt)

  pred1.1_nphs <- tibble::as_tibble(predict(ndmm_fit_gengamma_nphs, type = "survival"))
  expect_equivalent(pred1.1_nphs[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphs$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

})


test_that(".study, .trt, .time columns are correct (mspline, no regression, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # expect_warning(
  #   pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "mean")),
  #   "Evaluating M-spline at times beyond the boundary knots")
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  expect_warning(
    pred3.2 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "median")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  expect_warning(
    pred4.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "quantile", quantiles = qs)),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))

  # Check non-PH
  pred1.1_nphr <- tibble::as_tibble(predict(ndmm_fit_mspline_nphr, type = "survival"))
  expect_equivalent(pred1.1_nphr[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphr$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::inner_join(preddat1,
                                     dplyr::distinct(ndmm_preddat, .study, .trt),
                                     by = c(".study", ".trt")) %>%
    dplyr::arrange(.study, .trt)

  pred1.1_nphs <- tibble::as_tibble(predict(ndmm_fit_mspline_nphs, type = "survival"))
  expect_equivalent(pred1.1_nphs[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1_nphs$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

})

test_that("Survival predictions for new studies require correct args", {
  m <- "Specify both `baseline` and `aux`, or neither"

  expect_error(predict(ndmm_fit_weib, type = "survival",
                       times = 0:5,
                       baseline = distr(qnorm, 0, 1)),
               m)
  expect_error(predict(ndmm_fit_weib, type = "survival",
                       times = 0:5,
                       aux = distr(qnorm, 0, 1)),
               m)
  expect_error(predict(ndmm_fit_weib, type = "survival",
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qnorm, 0, 1)),
               "`times` must be specified")
})

test_that(".study, .trt, .time columns are correct (weibull, no regression, new data)", {

  time <- 0:5

  # Prediction format new times
  preddat1 <- tidyr::expand_grid(.study = "New 1",
                                 .trt = unique(ndmm_preddat$.trt),
                                 .time = time) %>%
    dplyr::group_by(.trt) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "survival",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.1[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_weib_nphr, type = "survival",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.1r[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1s <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_weib_nphs, type = "survival",
                                        times = time,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".trt", ".time")],
                    preddat1s[, c(".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1s$.trt, ", ", preddat1s$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "hazard",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.2[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "cumhaz",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.3[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tibble::tibble(.trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "mean",
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = distr(qlnorm, 0, 0.01)))
  # expect_equivalent(pred3.1[, ".trt"],
  #                   preddat3)
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "median",
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.2[, ".trt"],
                    preddat3)
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "rmst",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.3[, c(".trt", ".time")],
                    dplyr::mutate(preddat3, .time = 3))
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "link",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.4[, ".trt"],
                    preddat3)
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- tidyr::expand_grid(.trt = unique(ndmm_preddat$.trt),
                                 .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib, type = "quantile", quantiles = qs,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (exponential, no regression, new data)", {

  time <- 0:5

  # Prediction format new times
  preddat1 <- tidyr::expand_grid(.study = "New 1",
                                 .trt = unique(ndmm_preddat$.trt),
                                 .time = time) %>%
    dplyr::group_by(.trt) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "survival",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.1[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "hazard",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.2[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "cumhaz",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.3[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tibble::tibble(.trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "mean",
  #                                      baseline = distr(qnorm, 0, 1)))
  # expect_equivalent(pred3.1[, ".trt"],
  #                   preddat3)
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "median",
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.2[, ".trt"],
                    preddat3)
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "rmst",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.3[, c(".trt", ".time")],
                    dplyr::mutate(preddat3, .time = 3))
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "link",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.4[, ".trt"],
                    preddat3)
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- tidyr::expand_grid(.trt = unique(ndmm_preddat$.trt),
                                 .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp, type = "quantile", quantiles = qs,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (gengamma, no regression, new data)", {

  time <- 0:5

  # Prediction format new times
  preddat1 <- tidyr::expand_grid(.study = "New 1",
                                 .trt = unique(ndmm_preddat$.trt),
                                 .time = time) %>%
    dplyr::group_by(.trt) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "survival",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.1[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_gengamma_nphr, type = "survival",
                                        times = time,
                                        baseline = distr(qnorm, 0, 1),
                                        aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                   k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.1r[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1s <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_gengamma_nphs, type = "survival",
                                        times = time,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".trt", ".time")],
                    preddat1s[, c(".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1s$.trt, ", ", preddat1s$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "hazard",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.2[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "cumhaz",
                                       times = time,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.3[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tibble::tibble(.trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "mean",
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = list(sigma = distr(qlnorm, 0, 0.1),
  #                                                 k = distr(qlnorm, 0, 0.1))))
  # expect_equivalent(pred3.1[, ".trt"],
  #                   preddat3)
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "median",
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.2[, ".trt"],
                    preddat3)
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "rmst",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.3[, c(".trt", ".time")],
                    dplyr::mutate(preddat3, .time = 3))
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "link",
                                       time = 3,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.4[, ".trt"],
                    preddat3)
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- tidyr::expand_grid(.trt = unique(ndmm_preddat$.trt),
                                 .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma, type = "quantile", quantiles = qs,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that("errors for aux lists", {
  expect_error(predict(ndmm_fit_gengamma, type = "survival",
                       times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = list(k = distr(qlnorm, 0, 0.1))),
               "`aux` must be a named list of distr\\(\\) specifications for sigma and k")
  expect_error(predict(ndmm_fit_gengamma, type = "survival",
                       times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 0.1))),
               "`aux` must be a named list of distr\\(\\) specifications for sigma and k")
  expect_error(predict(ndmm_fit_gengamma, type = "survival",
                       times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qlnorm, 0, 0.1)),
               "`aux` must be a named list of distr\\(\\) specifications for sigma and k")
})

test_that(".study, .trt, .time columns are correct (mspline, no regression, new data)", {

  expect_error(predict(ndmm_fit_mspline, type = "survival",
                       times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qnorm, 0, 1)),
               'Producing predictions with external `aux` spline coefficients is not currently supported for "mspline" models.')

  time <- 0:5

  # Prediction format new times
  preddat1 <- tidyr::expand_grid(.study = "New 1",
                                 .trt = unique(ndmm_preddat$.trt),
                                 .time = time) %>%
    dplyr::group_by(.trt) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "survival",
                                       times = time,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred1.1[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_mspline_nphr, type = "survival",
                                        times = time,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1r[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1s <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_mspline_nphs, type = "survival",
                                        times = time,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".trt", ".time")],
                    preddat1s[, c(".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1s$.trt, ", ", preddat1s$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "hazard",
                                       times = time,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred1.2[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "cumhaz",
                                       times = time,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred1.3[, c(".trt", ".time")],
                    preddat1[, c(".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tibble::tibble(.trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "mean",
  #                                      baseline = "Attal2012,
  #                                      aux = "Attal2012"))
  # expect_equivalent(pred3.1[, ".trt"],
  #                   preddat3)
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.trt, "]"))

  pred3.2 <- suppressWarnings(tibble::as_tibble(predict(ndmm_fit_mspline, type = "median",
                                       baseline = "Attal2012",
                                       aux = "Attal2012")))
  expect_equivalent(pred3.2[, ".trt"],
                    preddat3)
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "rmst",
                                       time = 3,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred3.3[, c(".trt", ".time")],
                    dplyr::mutate(preddat3, .time = 3))
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline, type = "link",
                                       time = 3,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred3.4[, ".trt"],
                    preddat3)
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- tidyr::expand_grid(.trt = unique(ndmm_preddat$.trt),
                                 .quantile = qs)

  pred4.1 <- suppressWarnings(tibble::as_tibble(predict(ndmm_fit_mspline, type = "quantile", quantiles = qs,
                                       baseline = "Attal2012",
                                       aux = "Attal2012")))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})


ndmm_fit_weib_reg <- suppressWarnings(nma(ndmm_net,
                                          likelihood = "weibull-aft",
                                          regression = ~I(age/10)*.trt,
                                          class_interactions= "common",
                                          prior_intercept = normal(0, 100),
                                          prior_trt = normal(0, 10),
                                          prior_reg = normal(0, 10),
                                          prior_aux = half_normal(10),
                                          iter = 10,
                                          seed = 42))

ndmm_fit_weib_reg_nphr <- suppressWarnings(nma(ndmm_net,
                                          likelihood = "weibull-aft",
                                          regression = ~I(age/10)*.trt,
                                          aux_regression = ~.trt,
                                          class_interactions= "common",
                                          prior_intercept = normal(0, 100),
                                          prior_trt = normal(0, 10),
                                          prior_reg = normal(0, 10),
                                          prior_aux = half_normal(10),
                                          iter = 10,
                                          seed = 42))

ndmm_fit_weib_reg_nphs <- suppressWarnings(nma(ndmm_net,
                                          likelihood = "weibull-aft",
                                          regression = ~I(age/10)*.trt,
                                          aux_by = ".trt",
                                          class_interactions= "common",
                                          prior_intercept = normal(0, 100),
                                          prior_trt = normal(0, 10),
                                          prior_reg = normal(0, 10),
                                          prior_aux = half_normal(10),
                                          iter = 10,
                                          seed = 42))

ndmm_fit_exp_reg <- suppressWarnings(nma(ndmm_net,
                                         likelihood = "exponential",
                                         regression = ~I(age/10)*.trt,
                                         class_interactions = "common",
                                         prior_intercept = normal(0, 100),
                                         prior_trt = normal(0, 10),
                                         prior_reg = normal(0, 10),
                                         iter = 10))

ndmm_fit_gengamma_reg <- suppressWarnings(nma(ndmm_net,
                                              likelihood = "gengamma",
                                              regression = ~I(age/10)*.trt,
                                              class_interactions= "common",
                                              prior_intercept = normal(0, 100),
                                              prior_trt = normal(0, 10),
                                              prior_reg = normal(0, 10),
                                              prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                              init_r = 0.1,
                                              iter = 10))

ndmm_fit_gengamma_reg_nphr <- suppressWarnings(nma(ndmm_net,
                                              likelihood = "gengamma",
                                              regression = ~I(age/10)*.trt,
                                              aux_regression = ~.trt,
                                              class_interactions= "common",
                                              prior_intercept = normal(0, 100),
                                              prior_trt = normal(0, 10),
                                              prior_reg = normal(0, 10),
                                              prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                              init_r = 0.1,
                                              iter = 10))

ndmm_fit_gengamma_reg_nphs <- suppressWarnings(nma(ndmm_net,
                                              likelihood = "gengamma",
                                              regression = ~I(age/10)*.trt,
                                              aux_by = ".trt",
                                              class_interactions= "common",
                                              prior_intercept = normal(0, 100),
                                              prior_trt = normal(0, 10),
                                              prior_reg = normal(0, 10),
                                              prior_aux = list(sigma = half_normal(5), k = half_normal(5)),
                                              init_r = 0.1,
                                              iter = 10))

ndmm_fit_mspline_reg <- suppressWarnings(nma(ndmm_net,
                                             likelihood = "mspline",
                                             regression = ~I(age/10)*.trt,
                                             class_interactions = "common",
                                             prior_intercept = normal(0, 100),
                                             prior_trt = normal(0, 10),
                                             prior_reg = normal(0, 10),
                                             prior_aux = half_normal(1),
                                             iter = 10))

ndmm_fit_mspline_reg_nphr <- suppressWarnings(nma(ndmm_net,
                                             likelihood = "mspline",
                                             regression = ~I(age/10)*.trt,
                                             aux_regression = ~.trt,
                                             class_interactions = "common",
                                             prior_intercept = normal(0, 100),
                                             prior_trt = normal(0, 10),
                                             prior_reg = normal(0, 10),
                                             prior_aux = half_normal(1),
                                             iter = 10))

ndmm_fit_mspline_reg_nphs <- suppressWarnings(nma(ndmm_net,
                                             likelihood = "mspline",
                                             regression = ~I(age/10)*.trt,
                                             aux_by = ".trt",
                                             class_interactions = "common",
                                             prior_intercept = normal(0, 100),
                                             prior_trt = normal(0, 10),
                                             prior_reg = normal(0, 10),
                                             prior_aux = half_normal(1),
                                             iter = 10))


test_that("Survival predictions for new studies require correct args", {
  m <- "Specify all of `newdata`, `baseline`, and `aux`, or none"

  expect_error(predict(ndmm_fit_weib_reg, type = "survival",
                       baseline = distr(qnorm, 0, 1)),
               m)
  expect_error(predict(ndmm_fit_weib_reg, type = "survival",
                       times = 0:5,
                       aux = distr(qnorm, 0, 1)),
               m)
  expect_error(predict(ndmm_fit_weib_reg, type = "survival",
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qnorm, 0, 1)),
               m)
})

test_that(".study, .trt, .time columns are correct (weibull, regression, individual, network data)", {
  # Prediction format for level = "individual", observed times
  preddat1 <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt))

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", level = "individual"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphr, type = "survival", level = "individual"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphs, type = "survival", level = "individual"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", level = "individual"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", level = "individual"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Single summaries, also at individual times
  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "mean", level = "individual"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "median", level = "individual"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst", level = "individual"))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "link", level = "individual"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat1, tibble::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "quantile", quantiles = qs,
                                       level = "individual"))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (exponential, regression, individual, network data)", {
  # Prediction format for level = "individual", observed times
  preddat1 <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt))

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", level = "individual"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", level = "individual"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", level = "individual"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Single summaries, also at individual times
  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "mean", level = "individual"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "median", level = "individual"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst", level = "individual"))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "link", level = "individual"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat1, tibble::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "quantile", quantiles = qs,
                                       level = "individual"))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (gengamma, regression, individual, network data)", {
  # Prediction format for level = "individual", observed times
  preddat1 <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt))

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", level = "individual"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphr, type = "survival", level = "individual"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphs, type = "survival", level = "individual"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", level = "individual"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", level = "individual"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Single summaries, also at individual times
  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "mean", level = "individual"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "median", level = "individual"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst", level = "individual"))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "link", level = "individual"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat1, tibble::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "quantile", quantiles = qs,
                                       level = "individual"))
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (mspline, regression, individual, network data)", {
  # Prediction format for level = "individual", observed times
  preddat1 <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt))

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", level = "individual"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphr, type = "survival", level = "individual"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::filter(ndmm_preddat, .study %in% unique(ndmm_ipd$study)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup()

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphs, type = "survival", level = "individual"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", level = "individual"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", level = "individual"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Single summaries, also at individual times
  # expect_warning(
  #   pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "mean", level = "individual")),
  #   "Evaluating M-spline at times beyond the boundary knots")
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  expect_warning(
    pred3.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "median", level = "individual")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst", level = "individual"))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat1[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "link", level = "individual"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat1[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat1, tibble::tibble(.quantile = qs))

  expect_warning(
    pred4.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "quantile", quantiles = qs,
                                        level = "individual")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred4.1[, c(".trt", ".quantile")],
                    preddat4[, c(".trt", ".quantile")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (weibull, regression, aggregate, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphr, type = "survival"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::arrange(.study, .trt, id)

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphs, type = "survival"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (exponential, regression, aggregate, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (gengamma, regression, aggregate, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphr, type = "survival"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::arrange(.study, .trt, id)

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphs, type = "survival"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "median"))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "quantile", quantiles = qs))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (mspline, regression, aggregate, network data)", {
  # Prediction format for observed times
  preddat1 <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::select(-.trt) %>%
    dplyr::cross_join(dplyr::distinct(ndmm_preddat, .trt)) %>%
    dplyr::arrange(.study, .trt)

  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphr, type = "survival"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- ndmm_preddat %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::arrange(.study, .trt, id)

  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphs, type = "survival"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for provided times
  times <- 0:5
  preddat2 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt) %>%
    dplyr::mutate(.time = list(times), id = list(seq_along(times))) %>%
    tidyr::unnest(cols = c(".time", "id"))

  pred2.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", times = times))
  expect_equivalent(pred2.1[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.1$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", times = times))
  expect_equivalent(pred2.2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", times = times))
  expect_equivalent(pred2.3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt)

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "mean"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  expect_warning(
    pred3.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "median")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "rmst"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "link"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::distinct(ndmm_preddat, .study, .trt) %>%
    tidyr::expand(.study, .trt, .quantile = qs)

  expect_warning(
    pred4.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "quantile", quantiles = qs)),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (weibull, regression, aggregate, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm) %>%
    add_integration(age = distr(qlnorm, meanlog = log(4.5), sdlog = 1), n_int = 5)

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphr, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  newdata2 <- dplyr::bind_rows(
    dplyr::mutate(newdata, study = "Test"),
    dplyr::mutate(newdata, study = "B"),
    dplyr::mutate(newdata, study = "C")
  )
  preddat2 <- dplyr::mutate(newdata2, .study = factor(study), .time = rep(tm, times = 3)) %>%
    dplyr::group_by(.study) %>%
    dplyr::mutate(id = 1:dplyr::n()) %>%
    dplyr::ungroup() %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  pred1.1r2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphr, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata2,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1r2[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r2$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred1.1r3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphr, type = "survival", time = time,
                                         study = study,
                                         newdata = newdata2,
                                         baseline = list(Test = "Attal2012", B = "Morgan2012", C = distr(qlnorm, 0, 1)),
                                         aux = list(Test = "Attal2012", B = "Morgan2012", C = distr(qlnorm, 0, 0.01))))
  expect_equivalent(pred1.1r3[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r3$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  preddat1_nphs <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphs, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  preddat2_nphs <- dplyr::filter(preddat2, .trt %in% c("Pbo", "Len"))
  pred1.1s2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphs, type = "survival", time = time,
                                         study = study,
                                         newdata = newdata2,
                                         baseline = "Attal2012",
                                         aux = "Attal2012"))
  expect_equivalent(pred1.1s2[, c(".study", ".trt", ".time")],
                    preddat2_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s2$parameter,
                   paste0("pred[", preddat2_nphs$.study, ": ", preddat2_nphs$.trt, ", ", preddat2_nphs$id, "]"))

  preddat3_nphs <- dplyr::filter(preddat2, .study == "Test" & .trt %in% c("Pbo", "Len") | .study == "B" & .trt %in% c("Pbo", "Thal") | .study == "C")
  pred1.1s3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg_nphs, type = "survival", time = time,
                                         study = study,
                                         newdata = newdata2,
                                         baseline = list(Test = "Attal2012", B = "Morgan2012", C = distr(qlnorm, 0, 1)),
                                         aux = list(Test = "Attal2012", B = "Morgan2012", C = distr(qlnorm, 0, 0.01))))
  expect_equivalent(pred1.1s3[, c(".study", ".trt", ".time")],
                    preddat3_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s3$parameter,
                   paste0("pred[", preddat3_nphs$.study, ": ", preddat3_nphs$.trt, ", ", preddat3_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  pred1.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.4[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.5 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.5[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.5$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.6 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.6[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.6$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tidyr::expand_grid(.study = unique(factor(newdata$study)),
                                 .trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = distr(qlnorm, 0, 0.01)))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "rmst", time = 3,
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = distr(qlnorm, 0, 0.01)))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- preddat3 %>% tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "quantile", quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})


test_that(".study, .trt, .time columns are correct (exponential, regression, aggregate, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm) %>%
    add_integration(age = distr(qlnorm, meanlog = log(4.5), sdlog = 1), n_int = 5)

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  pred1.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.4[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.5 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.5[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.5$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.6 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.6[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.6$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tidyr::expand_grid(.study = unique(factor(newdata$study)),
                                 .trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1)))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "rmst", time = 3,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- preddat3 %>% tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "quantile", quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (gengamma, regression, aggregate, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm) %>%
    add_integration(age = distr(qlnorm, meanlog = log(4.5), sdlog = 1), n_int = 5)

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphr, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg_nphs, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))


  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  pred1.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.4[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.5 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.5[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.5$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.6 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.6[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.6$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tidyr::expand_grid(.study = unique(factor(newdata$study)),
                                 .trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = list(sigma = distr(qlnorm, 0, 0.1),
  #                                                 k = distr(qlnorm, 0, 0.1))))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  #pred3.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "rmst", time = 3,
  #                                     study = study,
  #                                     newdata = newdata,
  #                                     baseline = distr(qnorm, 0, 1),
  #                                     aux = list(sigma = distr(qlnorm, 0, 0.1),
  #                                                k = distr(qlnorm, 0, 0.1))))
  #expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                  preddat3[, c(".study", ".trt")])
  #expect_identical(pred3.3$parameter,
  #                 paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- preddat3 %>% tidyr::expand(.study, .trt, .quantile = qs)

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "quantile", quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (mspline, regression, aggregate, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm) %>%
    add_integration(age = distr(qlnorm, meanlog = log(4.5), sdlog = 1), n_int = 5)

  expect_error(predict(ndmm_fit_mspline_reg, type = "survival", time = time,
                       study = study,
                       newdata = newdata,
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qlnorm, 0, 0.01)),
               'Producing predictions with external `aux` spline coefficients is not currently supported for "mspline" models.')

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1b <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred1.1b[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1b$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1r <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphr, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1r[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1r$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  preddat1_nphs <- dplyr::filter(preddat1, .trt %in% c("Pbo", "Len"))
  pred1.1s <- tibble::as_tibble(predict(ndmm_fit_mspline_reg_nphs, type = "survival", time = time,
                                        study = study,
                                        newdata = newdata,
                                        baseline = "Attal2012",
                                        aux = "Attal2012"))
  expect_equivalent(pred1.1s[, c(".study", ".trt", ".time")],
                    preddat1_nphs[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1s$parameter,
                   paste0("pred[", preddat1_nphs$.study, ": ", preddat1_nphs$.trt, ", ", preddat1_nphs$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  pred1.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.4[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.4$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.5 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.5[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.5$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.6 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.6[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.6$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Prediction format for single summaries
  preddat3 <- tidyr::expand_grid(.study = unique(factor(newdata$study)),
                                 .trt = unique(ndmm_preddat$.trt))

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = "Attal2012"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  expect_warning(
    pred3.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "median",
                                         study = study,
                                         newdata = newdata,
                                         baseline = distr(qnorm, 0, 1),
                                         aux = "Attal2012")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # pred3.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "rmst", time = 3,
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = "Attal2012"))
  # expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.3$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- preddat3 %>% tidyr::expand(.study, .trt, .quantile = qs)

  expect_warning(
    pred4.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "quantile", quantiles = qs,
                                         study = study,
                                         newdata = newdata,
                                         baseline = distr(qnorm, 0, 1),
                                         aux = "Attal2012")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (weibull, regression, individual, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm, age = rlnorm(length(tm), log(4.5), 0.25))

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  # With external times vector, evaluate every time and every treatment for every new individual
  preddat2 <- dplyr::mutate(newdata, .study = factor(study)) %>%
    dplyr::cross_join(
      dplyr::cross_join(dplyr::tibble(.time = tm, id = 1:length(tm)),
                        dplyr::tibble(.trt = unique(ndmm_preddat$.trt))))
  pred2.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "survival", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred2.4[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.4$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.5 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "hazard", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred2.5[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.5$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.6 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "cumhaz", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred2.6[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.6$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- preddat1

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, level = "individual",
  #                                      type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = distr(qlnorm, 0, 0.01)))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, level = "individual",
                                       type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, level = "individual",
                                       type = "rmst", time = 3,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, level = "individual",
                                       type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat3, dplyr::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_weib_reg, type = "quantile", level = "individual",
                                       quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = distr(qlnorm, 0, 0.01)))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (exponential, regression, individual, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm, age = rlnorm(length(tm), log(4.5), 0.25))

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  # With external times vector, evaluate every time and every treatment for every new individual
  preddat2 <- dplyr::mutate(newdata, .study = factor(study)) %>%
    dplyr::cross_join(
      dplyr::cross_join(dplyr::tibble(.time = tm, id = 1:length(tm)),
                        dplyr::tibble(.trt = unique(ndmm_preddat$.trt))))
  pred2.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "survival", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred2.4[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.4$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.5 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "hazard", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred2.5[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.5$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.6 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "cumhaz", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred2.6[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.6$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- preddat1

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, level = "individual",
  #                                      type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1)))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, level = "individual",
                                       type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, level = "individual",
                                       type = "rmst", time = 3,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, level = "individual",
                                       type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat3, dplyr::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_exp_reg, type = "quantile", level = "individual",
                                       quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1)))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (gengamma, regression, individual, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm, age = rlnorm(length(tm), log(4.5), 0.25))

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  # With external times vector, evaluate every time and every treatment for every new individual
  preddat2 <- dplyr::mutate(newdata, .study = factor(study)) %>%
    dplyr::cross_join(
      dplyr::cross_join(dplyr::tibble(.time = tm, id = 1:length(tm)),
                        dplyr::tibble(.trt = unique(ndmm_preddat$.trt))))
  pred2.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "survival", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred2.4[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.4$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.5 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "hazard", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred2.5[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.5$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.6 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "cumhaz", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred2.6[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.6$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- preddat1

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, level = "individual",
  #                                      type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = list(sigma = distr(qlnorm, 0, 0.1),
  #                                                 k = distr(qlnorm, 0, 0.1))))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.2 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, level = "individual",
                                       type = "median",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  #pred3.3 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, level = "individual",
  #                                     type = "rmst", time = 3,
  #                                     study = study,
  #                                     newdata = newdata,
  #                                     baseline = distr(qnorm, 0, 1),
  #                                     aux = list(sigma = distr(qlnorm, 0, 0.1),
  #                                                k = distr(qlnorm, 0, 0.1))))
  #expect_equivalent(pred3.3[, c(".study", ".trt")],
  #                  preddat3[, c(".study", ".trt")])
  #expect_identical(pred3.3$parameter,
  #                 paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, level = "individual",
                                       type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat3, dplyr::tibble(.quantile = qs))

  pred4.1 <- tibble::as_tibble(predict(ndmm_fit_gengamma_reg, type = "quantile", level = "individual",
                                       quantiles = qs,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = list(sigma = distr(qlnorm, 0, 0.1),
                                                  k = distr(qlnorm, 0, 0.1))))
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})

test_that(".study, .trt, .time columns are correct (mspline, regression, individual, new data)", {

  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm, age = rlnorm(length(tm), log(4.5), 0.25))

  expect_error(predict(ndmm_fit_mspline_reg, type = "survival", time = time, level = "individual",
                       study = study,
                       newdata = newdata,
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qlnorm, 0, 0.01)),
               'Producing predictions with external `aux` spline coefficients is not currently supported for "mspline" models.')

  # Prediction format new times
  preddat1 <- dplyr::mutate(newdata, .study = factor(study), .time = tm, id = 1:dplyr::n()) %>%
    dplyr::cross_join(dplyr::tibble(.trt = unique(ndmm_preddat$.trt)))

  # Times from newdata column
  pred1.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.1[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.1b <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = "Attal2012",
                                       aux = "Attal2012"))
  expect_equivalent(pred1.1b[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.1b$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.2[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.2$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  pred1.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", level = "individual",
                                       time = time,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred1.3[, c(".study", ".trt", ".time")],
                    preddat1[, c(".study", ".trt", ".time")])
  expect_identical(pred1.3$parameter,
                   paste0("pred[", preddat1$.study, ": ", preddat1$.trt, ", ", preddat1$id, "]"))

  # Times from global env
  # With external times vector, evaluate every time and every treatment for every new individual
  preddat2 <- dplyr::mutate(newdata, .study = factor(study)) %>%
    dplyr::cross_join(
      dplyr::cross_join(dplyr::tibble(.time = tm, id = 1:length(tm)),
                        dplyr::tibble(.trt = unique(ndmm_preddat$.trt))))
  pred2.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "survival", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred2.4[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.4$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.5 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "hazard", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred2.5[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.5$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  pred2.6 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "cumhaz", level = "individual",
                                       time = tm,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred2.6[, c(".study", ".trt", ".time")],
                    preddat2[, c(".study", ".trt", ".time")])
  expect_identical(pred2.6$parameter,
                   paste0("pred[", preddat2$.study, ": ", preddat2$.trt, ", ", preddat2$id, "]"))

  # Prediction format for single summaries
  preddat3 <- preddat1

  # pred3.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, level = "individual",
  #                                      type = "mean",
  #                                      study = study,
  #                                      newdata = newdata,
  #                                      baseline = distr(qnorm, 0, 1),
  #                                      aux = "Attal2012"))
  # expect_equivalent(pred3.1[, c(".study", ".trt")],
  #                   preddat3[, c(".study", ".trt")])
  # expect_identical(pred3.1$parameter,
  #                  paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  expect_warning(
    pred3.2 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, level = "individual",
                                         type = "median",
                                         study = study,
                                         newdata = newdata,
                                         baseline = distr(qnorm, 0, 1),
                                         aux = "Attal2012")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred3.2[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.2$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.3 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, level = "individual",
                                       type = "rmst", time = 3,
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred3.3[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.3$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  pred3.4 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, level = "individual",
                                       type = "link",
                                       study = study,
                                       newdata = newdata,
                                       baseline = distr(qnorm, 0, 1),
                                       aux = "Attal2012"))
  expect_equivalent(pred3.4[, c(".study", ".trt")],
                    preddat3[, c(".study", ".trt")])
  expect_identical(pred3.4$parameter,
                   paste0("pred[", preddat3$.study, ": ", preddat3$.trt, ", ", preddat3$id, "]"))

  # Prediction format for quantiles
  qs <- c(0.2, 0.4, 0.6, 0.8)
  preddat4 <- dplyr::cross_join(preddat3, dplyr::tibble(.quantile = qs))

  expect_warning(
    pred4.1 <- tibble::as_tibble(predict(ndmm_fit_mspline_reg, type = "quantile", level = "individual",
                                         quantiles = qs,
                                         study = study,
                                         newdata = newdata,
                                         baseline = distr(qnorm, 0, 1),
                                         aux = "Attal2012")),
    "Evaluating M-spline at times beyond the boundary knots")
  expect_equivalent(pred4.1[, c(".study", ".trt")],
                    preddat4[, c(".study", ".trt")])
  expect_identical(pred4.1$parameter,
                   paste0("pred[", preddat4$.study, ": ", preddat4$.trt, ", ", preddat4$id, ", ", preddat4$.quantile, "]"))
})

test_that("errors for aux lists (regression, new data)", {
  tm <- 0:5

  newdata <- dplyr::tibble(study = "Test", time = tm) %>%
    add_integration(age = distr(qlnorm, meanlog = log(4.5), sdlog = 1), n_int = 5)

  expect_error(predict(ndmm_fit_gengamma_reg, type = "survival",
                       newdata = newdata, times = time, study = study,
                       baseline = distr(qnorm, 0, 1),
                       aux = list(k = distr(qlnorm, 0, 0.1))),
               "`aux` must be a single named list of distr\\(\\) specifications for sigma and k")
  expect_error(predict(ndmm_fit_gengamma_reg, type = "survival",
                       newdata = newdata, times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 0.1))),
               "`aux` must be a single named list of distr\\(\\) specifications for sigma and k")
  expect_error(predict(ndmm_fit_gengamma_reg, type = "survival",
                       newdata = newdata, times = time,
                       baseline = distr(qnorm, 0, 1),
                       aux = distr(qlnorm, 0, 0.1)),
               "`aux` must be a single named list of distr\\(\\) specifications for sigma and k")
})

test_that("aux argument", {
  newdata <- tibble::tibble(study = c("a", "b"), age = 4, time = 1)

  # Single aux parameter, no regression
  m <- "`aux` must be specified using distr\\(\\), or the name of an IPD or AgD \\(arm-based\\) study in the network"
  expect_error(predict(ndmm_fit_weib, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = 1), m)
  expect_error(predict(ndmm_fit_weib, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = list("a")), m)
  expect_error(predict(ndmm_fit_weib, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = NA), m)

  expect_error(predict(ndmm_fit_weib, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = "a"),
               "`aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a distr\\(\\) distribution")

  # Single aux parameter, regression
  m2 <- "`aux` must be a single distr\\(\\) specification or study name, or a list of length 2 \\(number of `newdata` studies\\)"
  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = 1), m2)
  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = list("a")), m2)
  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = NA), m2)

  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = "a"),
               "All elements of `aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a distr\\(\\) distribution")

  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(a = distr(qnorm, 1, 1),
                                  c = distr(qnorm, 1, 1))),
               "must match all study names from `newdata`")
  expect_error(predict(ndmm_fit_weib_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(a = distr(qnorm, 1, 1),
                                  b = "bad")),
               "All elements of `aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a distr\\(\\) distribution")

  # Multiple aux parameters, no regression
  m3 <- "`aux` must be a named list of distr\\(\\) specifications for sigma and k, or a study name"

  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = 1), m3)
  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = list("a")), m3)
  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = NA), m3)

  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = "a"),
               "`aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a named list of distr\\(\\) specifications for sigma and k")

  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 1))), m3)
  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(k = distr(qlnorm, 0, 1))), m3)
  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(k = distr(qlnorm, 0, 1),
                                  sigma = "bad")), m3)
  expect_error(predict(ndmm_fit_gengamma, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 1),
                                  k = distr(qlnorm, 0, 1),
                                  bad = distr(qlnorm, 0, 1))), m3)

  # Multiple aux parameters, regression
  m4 <- "`aux` must be a single named list of distr\\(\\) specifications for sigma and k, a study name, or a list of length 2 \\(number of `newdata` studies\\) of such lists"
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = 1), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = list("a")), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = NA), m4)

  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1), aux = "a"),
               "All elements of `aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a list of distr\\(\\) distributions")

  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 1))), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(k = distr(qlnorm, 0, 1))), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 1),
                                  k = distr(qlnorm, 0, 1),
                                  bad = distr(qlnorm, 0, 1))), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(
                                  a = list(sigma = distr(qlnorm, 0, 1),
                                           k = distr(qlnorm, 0, 1))
                                 )), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(
                                  a = list(sigma = distr(qlnorm, 0, 1),
                                           k = distr(qlnorm, 0, 1)),
                                  b = list(sigma = distr(qlnorm, 0, 1))
                                 )), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(
                                  a = list(sigma = distr(qlnorm, 0, 1),
                                           k = distr(qlnorm, 0, 1)),
                                  b = list(sigma = distr(qlnorm, 0, 1),
                                           k = distr(qlnorm, 0, 1)),
                                  c = list(sigma = distr(qlnorm, 0, 1),
                                           k = distr(qlnorm, 0, 1))
                       )), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(sigma = distr(qlnorm, 0, 1),
                                  k = "bad")), m4)
  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(
                         a = list(sigma = distr(qlnorm, 0, 1),
                                  k = distr(qlnorm, 0, 1)),
                         b = list(sigma = distr(qlnorm, 0, 1),
                                  k = "bad")
                       )), m4)

  expect_error(predict(ndmm_fit_gengamma_reg, times = 1, study = study, newdata = newdata, baseline = distr(qnorm, 0, 1),
                       aux = list(
                         a = list(sigma = distr(qlnorm, 0, 1),
                                  k = distr(qlnorm, 0, 1)),
                         b = "bad")
                       ), "All elements of `aux` must match the name of an IPD or AgD \\(arm-based\\) study in the network, or be a list of distr\\(\\) distributions")

})

test_that("baseline and aux are escaped correctly", {
  ndmm_net <- set_ipd(ndmm_ipd,
                      study = gsub("(.+)([0-9]{4})", "\\1 (\\2)$", study),
                      trt = trt,
                      Surv = Surv(eventtime/7, status))

  ndmm_fit_gg <- suppressWarnings(nma(ndmm_net,
                  likelihood = "gengamma",
                  aux_regression = ~.trt,
                  init_r = 0.1,
                  iter = 10))

  par <- c("pred[Len, 1]", "pred[Pbo, 1]")

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit_gg, times = 10, type = "survival",
            aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
    par)

  ndmm_fit_weib <- suppressWarnings(nma(ndmm_net,
                                      likelihood = "weibull",
                                      aux_regression = ~.trt,
                                      iter = 10))
  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit_weib, times = 10, type = "survival",
                             aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
    par)

  ndmm_fit_ms <- suppressWarnings(nma(ndmm_net,
                                        likelihood = "mspline",
                                        aux_regression = ~.trt,
                                        iter = 10))
  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit_ms, times = 10, type = "survival",
                             aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
    par)

  par <- c("pred[New 1: Len, 1]", "pred[New 1: Pbo, 1]")
  par2 <- c("pred[(a): Len, 1]", "pred[(a): Pbo, 1]", "pred[(b): Len, 1]", "pred[(b): Pbo, 1]")

  ndmm_fit2_gg <- suppressWarnings(nma(ndmm_net,
                   likelihood = "gengamma",
                   regression = ~I(age/10)*.trt,
                   aux_regression = ~.trt,
                   init_r = 0.1,
                   iter = 10))

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_gg, times = 10, type = "survival",
                             newdata = data.frame(age = 5),
                             aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
    par)

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_gg, times = 10, type = "survival",
                             newdata = data.frame(age = 5, s = c("(a)", "(b)")), study = s,
                             aux = list("(a)" = "Attal (2012)$",
                                        "(b)" = "McCarthy (2012)$"),
                             baseline = list("(a)" = "Attal (2012)$",
                                             "(b)" = "McCarthy (2012)$")))$parameter,
    par2)

  ndmm_fit2_weib <- suppressWarnings(nma(ndmm_net,
                                       likelihood = "weibull",
                                       regression = ~I(age/10)*.trt,
                                       aux_regression = ~.trt,
                                       iter = 10))

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_weib, times = 10, type = "survival",
                             newdata = data.frame(age = 5),
                             aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
    par)

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_weib, times = 10, type = "survival",
                             newdata = data.frame(age = 5, s = c("(a)", "(b)")), study = s,
                             aux = list("(a)" = "Attal (2012)$",
                                        "(b)" = "McCarthy (2012)$"),
                             baseline = list("(a)" = "Attal (2012)$",
                                             "(b)" = "McCarthy (2012)$")))$parameter,
    par2)

  ndmm_fit2_ms <- suppressWarnings(nma(ndmm_net,
                                       likelihood = "mspline",
                                       regression = ~I(age/10)*.trt,
                                       aux_regression = ~.trt,
                                       iter = 10))

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_ms, times = 10, type = "survival",
                             newdata = data.frame(age = 5),
                             aux = "Attal (2012)$", baseline = "Attal (2012)$"))$parameter,
  par)

  expect_identical(
    dplyr::as_tibble(predict(ndmm_fit2_ms, times = 10, type = "survival",
                             newdata = data.frame(age = 5, s = c("(a)", "(b)")), study = s,
                             aux = list("(a)" = "Attal (2012)$",
                                        "(b)" = "McCarthy (2012)$"),
                             baseline = list("(a)" = "Attal (2012)$",
                                             "(b)" = "McCarthy (2012)$")))$parameter,
    par2)

})

library(multinma)
library(dplyr)

test_that("set_* produces empty nma_data objects", {
  empty_nma_data <- structure(
    list(agd_arm = NULL,
         agd_contrast = NULL,
         ipd = NULL,
         treatments = NULL,
         classes = NULL,
         studies = NULL), class = "nma_data")

  expect_equal(set_ipd(smoking[0, ], "studyn", "trtc"), empty_nma_data)
  expect_equal(set_agd_arm(smoking[0, ], "studyn", "trtc"), empty_nma_data)
  expect_equal(set_agd_contrast(smoking[0, ], "studyn", "trtc"), empty_nma_data)
  expect_equal(set_agd_surv(smoking[0, ], "studyn", "trtc"), empty_nma_data)
})

test_that("set_* error if data does not inherit data.frame", {
  vec <- 1:5
  msg <- "Argument `data` should be a data frame"

  expect_error(set_ipd(vec), msg)
  expect_error(set_agd_arm(vec), msg)
  expect_error(set_agd_contrast(vec), msg)
  expect_error(set_agd_surv(vec), msg)
})

test_that("set_* error if study not given, missing values, or not regular 1D column", {
  expect_error(set_ipd(smoking), "Specify `study`")
  expect_error(set_agd_arm(smoking), "Specify `study`")
  expect_error(set_agd_contrast(smoking), "Specify `study`")
  expect_error(set_agd_surv(smoking), "Specify `study`")

  smk_miss <- smoking
  smk_miss[1, "studyn"] <- NA
  expect_error(set_ipd(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_arm(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_contrast(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_surv(smk_miss, "studyn", "trtc"), "cannot contain missing values")

  expect_error(set_ipd(smoking, cbind(studyn, studyn)), "must be a regular column")
  expect_error(set_ipd(smoking, list(studyn)), "must be a regular column")
  expect_error(set_agd_arm(smoking, cbind(studyn, studyn)), "must be a regular column")
  expect_error(set_agd_arm(smoking, list(studyn)), "must be a regular column")
  expect_error(set_agd_contrast(smoking, cbind(studyn, studyn)), "must be a regular column")
  expect_error(set_agd_contrast(smoking, list(studyn)), "must be a regular column")
  expect_error(set_agd_surv(smoking, cbind(studyn, studyn)), "must be a regular column")
  expect_error(set_agd_surv(smoking, list(studyn)), "must be a regular column")
})

test_that("set_* error if single-arm studies included", {
  m <- "Single-arm studies are not supported"

  s <- tibble(study = c("a", "b", "b", "c"), trt = c("A", "A", "A", "B"), r = 1, n = 2, time = 1, status = 1)
  expect_error(set_ipd(s, study, trt, r = r), paste0(m, '.+studies "a", "b" and "c"'))
  expect_error(set_agd_arm(s, study, trt, r = r, n = n), paste0(m, '.+studies "a" and "c"'))
  expect_error(set_agd_contrast(s, study, trt, y = r, se = n), paste0(m, '.+studies "a" and "c"'))

  # Allowed with message for survival outcomes
  expect_message(set_ipd(s, study, trt, Surv = Surv(time, status)), 'Single-arm studies present in the network: "a", "b" and "c"')
  expect_message(set_agd_surv(s, study, trt, Surv = Surv(time, status)), 'Single-arm studies present in the network: "a", "b" and "c"')
})

test_that("set_* error if trt not given, missing values, or not regular 1D column", {
  expect_error(set_ipd(smoking, "studyn"), "Specify `trt`")
  expect_error(set_agd_arm(smoking, "studyn"), "Specify `trt`")
  expect_error(set_agd_contrast(smoking, "studyn"), "Specify `trt`")
  expect_error(set_agd_surv(smoking, "studyn"), "Specify `trt`")

  smk_miss <- smoking
  smk_miss[1, "trtc"] <- NA
  expect_error(set_ipd(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_arm(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_contrast(smk_miss, "studyn", "trtc"), "cannot contain missing values")
  expect_error(set_agd_surv(smk_miss, "studyn", "trtc"), "cannot contain missing values")

  expect_error(set_ipd(smoking, studyn, cbind(trtc, trtc)), "must be a regular column")
  expect_error(set_ipd(smoking, studyn, list(trtc)), "must be a regular column")
  expect_error(set_agd_arm(smoking, studyn, cbind(trtc, trtc)), "must be a regular column")
  expect_error(set_agd_arm(smoking, studyn, list(trtc)), "must be a regular column")
  expect_error(set_agd_contrast(smoking, studyn, cbind(trtc, trtc)), "must be a regular column")
  expect_error(set_agd_contrast(smoking, studyn, list(trtc)), "must be a regular column")
  expect_error(set_agd_surv(smoking, studyn, cbind(trtc, trtc)), "must be a regular column")
  expect_error(set_agd_surv(smoking, studyn, list(trtc)), "must be a regular column")
})

# Dummy data
agd_arm <- tibble(
  studyn = c(1, 1, 2, 2, 2),
  studyc = letters[studyn],
  studyf = factor(studyc),
  studyf2 = forcats::fct_rev(studyf),
  trtn = c(1, 2, 1, 2, 3),
  trtc = LETTERS[trtn],
  trtf = factor(trtc),
  trtf2 = forcats::fct_rev(trtf),
  tclassn = c(1, 2, 1, 2, 2),
  tclassc = letters[tclassn],
  tclassf = factor(tclassc),
  tclassf2 = forcats::fct_rev(tclassf),
  cont = rnorm(5),
  cont_pos = abs(cont),
  cont_neg = -cont_pos,
  cont_inf = c(cont_pos[1:4], Inf),
  cont_nan = c(cont_pos[1:4], NaN),
  disc = rbinom(5, 20, 0.5) + 1,
  disc_p1 = disc + 1,
  disc_m1 = disc - 1,
  disc_neg = -disc,
  disc_inf = c(disc[1:4], Inf),
  disc_nan = c(disc[1:4], NaN),
  disc_na = c(disc[1:4], NA),
  bin = sample(c(0,0,0,0,1,1,1,1), 5)
  #Surv =
)

test_that("set_agd_arm - continuous outcome checks work", {
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont), "Specify standard error")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", se = cont_pos), "Specify continuous outcome `y`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = trtc, se = cont_pos), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = as.character(trtn), se = cont_pos), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = trtc), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = as.character(trtn)), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = cont_neg), "must be positive")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = -cont_pos), "must be positive")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = cont_inf), "cannot be infinite")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = cont_nan), "cannot be NaN")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cbind(cont, cont), se = cont_pos),
               "must be a regular column")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cbind(cont_pos, cont_pos)),
               "must be a regular column")
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", y = cont, se = cont_pos)$agd_arm[, c(".y", ".se")],
    transmute(agd_arm, .y = cont, .se = cont_pos))
})

test_that("set_agd_arm - count outcome checks work", {
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc), "Specify denominator")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", n = disc_p1), "Specify outcome count `r`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = trtc, n = disc_p1), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = trtc), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = cont, n = disc), "must be integer")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = cont), "must be integer")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc_neg, n = disc), "must be between 0 and `n`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = -disc, n = disc), "must be between 0 and `n`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc_p1, n = disc), "must be between 0 and `n`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_neg), "greater than zero")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = -disc), "greater than zero")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", E = cont_pos), "Specify outcome count `r`")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1, E = cont_neg), "must be positive")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1, E = -cont_pos), "must be positive")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1, E = trtc), "must be numeric")
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1, E = as.character(trtn)), "must be numeric")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = cbind(disc, disc), n = disc_p1), "must be a regular column")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = disc, n = cbind(disc_p1, disc_p1)), "must be a regular column")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = disc, E = cbind(cont_pos, cont_pos)), "must be a regular column")
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1)$agd_arm[, c(".r", ".n")],
    transmute(agd_arm, .r = disc, .n = disc_p1))
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = "disc", n = "disc_p1")$agd_arm[, c(".r", ".n")],
    transmute(agd_arm, .r = disc, .n = disc_p1))
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc + 1)$agd_arm[, c(".r", ".n")],
    transmute(agd_arm, .r = disc, .n = disc_p1))
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = floor(disc/2), n = disc)$agd_arm[, c(".r", ".n")],
    transmute(agd_arm, .r = floor(disc/2), .n = disc))
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = disc, E = cont_pos)$agd_arm[, c(".r", ".E")],
    transmute(agd_arm, .r = disc, .E = cont_pos))
  expect_equivalent(
    set_agd_arm(agd_arm, "studyn", "trtc", r = "disc", E = "cont_pos")$agd_arm[, c(".r", ".E")],
    transmute(agd_arm, .r = disc, .E = cont_pos))
})


multi_inclusive <- tribble(~r_a, ~r_b, ~r_c,
                           1, 1, 1,
                           5, 4, 1,
                           5, 2, 2,
                           10, 5, 0,
                           6, 0, 0)
agd_arm_multi_i <- bind_cols(agd_arm, multi_inclusive)

multi_exclusive <- tribble(~r_a, ~r_b, ~r_c,
                           0, 0, 1,
                           1, 3, 1,
                           3, 0, 2,
                           5, 5, 0,
                           6, 0, 0)
agd_arm_multi_e <- bind_cols(agd_arm, multi_exclusive)

test_that("set_agd_arm - multinomial outcome checks work", {
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc)),
               "At least 2 outcomes", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, disc)),
               "Duplicate outcome category labels", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, 1:2)),
               "must be the same length", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, disc_inf)),
               "cannot be Inf", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, disc_nan)),
               "cannot be NaN", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, trtc)),
               "must be numeric", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, cont_pos)),
               "must be integer", class = "error")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, r = multi(disc, disc_neg)),
               "must be non-negative", class = "error")

  expect_error(set_agd_arm(agd_arm_multi_i, studyn, trtc, r = multi(r_c, r_b, r_a, inclusive = TRUE)),
               "must be decreasing or constant", class = "error")
  expect_equivalent(unclass(set_agd_arm(agd_arm_multi_i, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = TRUE))$agd_arm$.r),
                    as.matrix(multi_exclusive))

  expect_equivalent(unclass(set_agd_arm(agd_arm_multi_e, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = FALSE))$agd_arm$.r),
                    as.matrix(multi_exclusive))
})

test_that("set_agd_arm - sample size checks work", {
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = trtc),
               "must be numeric")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = as.character(trtn)),
               "must be numeric")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = cont),
               "must be integer")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = disc_neg),
               "must be greater than zero")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = -disc),
               "must be greater than zero")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = disc_inf),
               "cannot be infinite")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = disc_nan),
               "cannot be NaN")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = cbind(disc, disc)),
               "must be a regular column")
  expect_error(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos, sample_size = list(disc)),
               "must be a regular column")
  expect_message(set_agd_arm(agd_arm, studyn, trtc, y = cont, se = cont_pos), "`sample_size` not provided")

  expect_equal(set_agd_arm(agd_arm, studyn, trtc, r = disc, n = disc_p1)$agd_arm$.sample_size, agd_arm$disc_p1)
  expect_equal(set_agd_arm(agd_arm, studyn, trtc, r = floor(disc/2), n = disc, sample_size = disc + 1)$agd_arm$.sample_size, agd_arm$disc_p1)
  expect_equal(set_agd_arm(agd_arm_multi_i, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = TRUE))$agd_arm$.sample_size, multi_inclusive$r_a)
  expect_equal(set_agd_arm(agd_arm_multi_e, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = FALSE))$agd_arm$.sample_size, multi_inclusive$r_a)
})

test_that("set_ipd - continuous outcome checks work", {
  expect_error(set_ipd(agd_arm, "studyn", "trtc", y = trtc), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", y = as.character(trtn)), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", y = cbind(cont, cont)), "must be a regular column")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", y = list(cont)), "must be a regular column")
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", y = cont)$ipd[, ".y"],
    transmute(agd_arm, .y = cont))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", y = "cont")$ipd[, ".y"],
    transmute(agd_arm, .y = cont))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", y = cont/2)$ipd[, ".y"],
    transmute(agd_arm, .y = cont/2))
})

test_that("set_ipd - binary outcome checks work", {
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = trtc), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = as.character(trtn)), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = cont), "must equal 0 or 1")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = disc_neg), "must equal 0 or 1")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin + 1), "must equal 0 or 1")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", E = cont_pos), "Specify count `r`")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, E = cont_neg), "must be positive")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, E = -cont_pos), "must be positive")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, E = trtc), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, E = as.character(trtn)), "must be numeric")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = cbind(bin, bin)), "must be a regular column")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = cbind(bin, bin), E = cont_pos), "must be a regular column")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, E = cbind(cont_pos, cont_pos)), "must be a regular column")
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", r = bin, E = cont_pos)$ipd[, c(".r", ".E")],
    transmute(agd_arm, .r = bin, .E = cont_pos))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", r = "bin", E = "cont_pos")$ipd[, c(".r", ".E")],
    transmute(agd_arm, .r = bin, .E = cont_pos))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", r = bin, E = cont_pos/2)$ipd[, c(".r", ".E")],
    transmute(agd_arm, .r = bin, .E = cont_pos/2))
})

test_that("set_ipd - multinomial outcome checks work", {
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(bin)),
               "At least 2 outcomes", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(bin, bin)),
               "Duplicate outcome category labels", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(bin, 1:2)),
               "must be the same length", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(disc, disc_inf)),
               "cannot be Inf", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(disc, disc_nan)),
               "cannot be NaN", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(disc, trtc)),
               "must be numeric", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(bin, bin + 1)),
               "must equal 0 or 1", class = "error")
  expect_error(set_ipd(agd_arm, studyn, trtc, r = multi(bin, -bin)),
               "must be non-negative", class = "error")

  i_multi_inclusive <- tribble(~r_a, ~r_b, ~r_c,
                               1, 1, 1,
                               1, 1, 0,
                               1, 0, 0,
                               1, 1, NA,
                               1, NA, 0)
  ipd_multi_i <- bind_cols(agd_arm, i_multi_inclusive)

  i_multi_exclusive <- tribble(~r_a, ~r_b, ~r_c,
                               0, 0, 1,
                               0, 1, 0,
                               1, 0, 0,
                               0, 1, NA,
                               1, NA, 0)
  ipd_multi_e <- bind_cols(agd_arm, i_multi_exclusive)

  expect_error(set_ipd(ipd_multi_i[-4,], study = I(1), trtc, r = multi(r_c, r_b, r_a, inclusive = TRUE)),
               "must be decreasing or constant", class = "error")
  expect_error(set_ipd(ipd_multi_i[-c(4,5),], study = I(1), trtc, r = multi(r_b, r_c, inclusive = TRUE)),
               "Individual without outcomes in any category, row 3", class = "error")
  expect_equivalent(unclass(set_ipd(ipd_multi_i, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = TRUE))$ipd$.r),
                    as.matrix(i_multi_exclusive))

  expect_error(set_ipd(ipd_multi_e[-c(4,5),], study = I(1), trtc, r = multi(r_b, r_c, inclusive = FALSE)),
               "Individual without outcomes in any category, row 3", class = "error")
  expect_error(set_ipd(ipd_multi_e, studyn, trtc, r = multi(c = 1, r_a, r_b, inclusive = FALSE)),
               "Individuals with outcomes in more than one category, rows 2, 3, 4 and 5", class = "error")
  expect_equivalent(unclass(set_ipd(ipd_multi_e, studyn, trtc, r = multi(r_a, r_b, r_c, inclusive = FALSE))$ipd$.r),
                    as.matrix(i_multi_exclusive))
})

test_that("set_ipd - survival outcome checks work", {
  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = trtc), "must be a `Surv` object")
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, bin))$ipd[[".Surv"]],
    with(agd_arm, Surv(cont_pos, bin)))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos + 1, bin))$ipd[[".Surv"]],
    with(agd_arm, Surv(cont_pos, cont_pos + 1, bin)))
  expect_equivalent(
    set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos + 1, c(0, 1, 3, 3, 3), type = "interval"))$ipd[[".Surv"]],
    with(agd_arm, Surv(cont_pos, cont_pos + 1, c(0, 1, 3, 3, 3), type = "interval")))

  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, factor(bin))), 'type "mright" is not supported')

  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, bin)), "must have strictly positive outcome times")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_inf, bin)), "infinite times")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_nan, bin)), "missing times")

  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, cont_pos, bin)), "must have non-negative start times")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, cont_pos, c(3, 3, 3, 3, 3), type = "interval")), "must have non-negative start times")

  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, rep_len(c(0, 1, NA), nrow(agd_arm)))), "missing event status values")
  expect_error(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv("a", bin)), "not numeric")
  expect_error(suppressWarnings(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos/2, bin))), "missing times")
  expect_error(suppressWarnings(set_ipd(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos*2, bin*5, type = "interval"))), "missing event status values")
})

test_that("set_agd_surv - survival outcome checks work", {
  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = trtc), "must be a `Surv` object")
  expect_equivalent(
    tidyr::unnest(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, bin))$agd_arm, ".Surv")$.Surv,
    with(agd_arm, Surv(cont_pos, bin)))

  expect_equivalent(
    tidyr::unnest(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos + 1, bin))$agd_arm, ".Surv")$.Surv,
    with(agd_arm, Surv(cont_pos, cont_pos + 1, bin)))
  expect_equivalent(
    tidyr::unnest(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos + 1, c(0, 1, 3, 3, 3), type = "interval"))$agd_arm, ".Surv")$.Surv,
    with(agd_arm, Surv(cont_pos, cont_pos + 1, c(0, 1, 3, 3, 3), type = "interval")))

  expect_equivalent(
    tidyr::unnest(set_agd_surv(ndmm_agd, studyf, trtf, Surv = Surv(eventtime, status))$agd_arm, ".Surv")$.Surv,
    with(ndmm_agd, Surv(eventtime, status)))
  expect_equivalent(
    tidyr::unnest(set_agd_surv(ndmm_agd, studyf, trtf,
                               Surv = Surv(eventtime, status),
                               covariates = ndmm_agd_covs)$agd_arm, ".Surv")$.Surv,
    with(ndmm_agd, Surv(eventtime, status)))

  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, factor(bin))), 'type "mright" is not supported')

  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, bin)), "must have strictly positive outcome times")
  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_inf, bin)), "infinite times")
  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_nan, bin)), "missing times")

  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, cont_pos, bin)), "must have non-negative start times")
  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_neg, cont_pos, c(3, 3, 3, 3, 3), type = "interval")), "must have non-negative start times")

  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, rep_len(c(0, 1, NA), nrow(agd_arm)))), "missing event status values")
  expect_error(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv("a", bin)), "not numeric")
  expect_error(suppressWarnings(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos/2, bin))), "missing times")
  expect_error(suppressWarnings(set_agd_surv(agd_arm, "studyn", "trtc", Surv = Surv(cont_pos, cont_pos*2, bin*5, type = "interval"))), "missing event status values")
})

test_that("set_agd_surv - covariate checks work", {
  expect_error(set_agd_surv(ndmm_agd, studyf, trtf, Surv = Surv(eventtime, status), covariates = list()), "should be a data frame")

  expect_equivalent(
    dplyr::select(set_agd_surv(ndmm_agd, studyf, trtf, Surv = Surv(eventtime, status),
                                  covariates = ndmm_agd_covs)$agd_arm,
                     study, trt, sample_size = .sample_size,
                     age_min:male) %>%
      dplyr::arrange(study, trt),
    dplyr::select(ndmm_agd_covs, -studyf, -trtf))

  expect_error(set_agd_surv(ndmm_agd, studyf, trtf, Surv = Surv(eventtime, status),
                            covariates = ndmm_agd_covs[-1,]),
               "Not all study arms in `data` have matching rows in `covariates`")
})

# Dummy contrast data
agd_contrast <- agd_arm %>%
  group_by(studyn) %>%
  mutate(arm = 1:n(),
         ydiff = if_else(arm == 1, NA_real_, cont - first(cont)),
         ydiff_chr = if_else(arm == 1, NA_character_, trtc),
         ydiff_multi = if_else(arm %in% 1:2, NA_real_, cont - first(cont)),
         sediff = if_else(arm == 1, cont_pos, sqrt(cont_pos^2 + first(cont_pos)^2)),
         sediff_miss = if_else(arm == 1, NA_real_, sqrt(cont_pos^2 + first(cont_pos)^2))) %>%
  ungroup()

test_that("set_agd_contrast - continuous outcome checks work", {
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff), "Specify standard error")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", se = sediff), "Specify continuous outcome `y`")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff_chr, se = sediff), "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = as.character(ydiff), se = sediff), "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = trtc), "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = as.character(sediff)), "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = cont_neg), "must be positive")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = -cont_pos), "must be positive")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = cont_inf), "cannot be infinite")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = cont_nan), "cannot be NaN")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = trtn, se = sediff), "without a specified baseline arm")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff_multi, se = sediff), "Multiple baseline arms")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = sediff_miss), "Standard error.+missing values on baseline arms")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = cbind(ydiff, ydiff), se = sediff), "must be a regular column")
  expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = cbind(sediff, sediff)), "must be a regular column")
  expect_equivalent(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff, se = sediff)$agd_contrast[, c(".y", ".se")],
                    transmute(agd_contrast, .y = ydiff, .se = sediff))
  expect_equivalent(set_agd_contrast(agd_contrast, "studyn", "trtc", y = "ydiff", se = "sediff")$agd_contrast[, c(".y", ".se")],
                    transmute(agd_contrast, .y = ydiff, .se = sediff))
  expect_equivalent(set_agd_contrast(agd_contrast, "studyn", "trtc", y = ydiff/2, se = sediff)$agd_contrast[, c(".y", ".se")],
                    transmute(agd_contrast, .y = ydiff/2, .se = sediff))
})

test_that("set_agd_contrast - sample size checks work", {
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = trtc),
               "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = as.character(trtn)),
               "must be numeric")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = cont),
               "must be integer")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = cont/2),
               "must be integer")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = disc_neg),
               "must be greater than zero")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = -disc),
               "must be greater than zero")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = disc_inf),
               "cannot be infinite")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = disc_nan),
               "cannot be NaN")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = cbind(disc, disc)),
               "must be a regular column")
  expect_error(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff, sample_size = list(disc)),
               "must be a regular column")
  expect_message(set_agd_contrast(agd_contrast, studyn, trtc, y = ydiff, se = sediff), "`sample_size` not provided")
})

test_that("set_agd_contrast - positive definite check", {
  agd_contrast_nonpd <- agd_contrast %>%
    group_by(studyn) %>%
    mutate(sediff = if_else(arm == 1, max(cont_pos)*2, sqrt(cont_pos^2 + first(cont_pos)^2))) %>%
    ungroup()

  expect_error(set_agd_contrast(agd_contrast_nonpd, studyn, trtc, y = ydiff, se = sediff),
               'not positive definite for study "2"')
  expect_error(set_agd_contrast(agd_contrast_nonpd, studyc, trtc, y = ydiff, se = sediff),
               'not positive definite for study "b"')

  agd_contrast_nonpd2 <- bind_rows(agd_contrast_nonpd,
                                   filter(agd_contrast_nonpd, studyn == 2) %>% mutate(studyn = 3, studyc = "c"))

  expect_error(set_agd_contrast(agd_contrast_nonpd2, studyn, trtc, y = ydiff, se = sediff),
               'not positive definite for studies "2" and "3"')
  expect_error(set_agd_contrast(agd_contrast_nonpd2, studyc, trtc, y = ydiff, se = sediff),
               'not positive definite for studies "b" and "c"')
})

test_that("set_* - take one and only one outcome", {
  m <- "specify one and only one outcome"
  expect_error(set_ipd(agd_arm, "studyn", "trtc", r = bin, y = cont), m)
  expect_error(set_agd_arm(agd_arm, "studyn", "trtc", r = disc, n = disc_p1, y = cont, se = cont_pos), m)
  # expect_error(set_agd_contrast(agd_contrast, "studyn", "trtc"), m)
})

# Reference treatment for survival outcomes tie-breaks by longest follow-up
bmax <- max(subset(agd_arm$cont_pos, agd_arm$trtf == "B")) > max(subset(agd_arm$cont_pos, agd_arm$trtf == "A"))

test_that("set_* `.trt` column is correct", {
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)

  expect_equal(set_ipd(agd_arm, studyc, 6, y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, 6, y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, 6, y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, 6, Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)

  expect_equal(set_ipd(agd_arm, studyc, "trtc", y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, "trtc", y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, "trtc", y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, "trtc", Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)

  expect_equal(set_ipd(agd_arm, studyc, factor(trtc), y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, factor(trtc), y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, factor(trtc), y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, factor(trtc), Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)


  expect_equal(set_ipd(agd_arm, studyc, trtf, y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf, y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyf, trtf, y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf, Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)

  expect_equal(set_ipd(agd_arm, studyc, trtf2, y = cont, trt_ref = "C")$ipd$.trt,
               agd_arm$trtf2)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf2, y = cont, se = cont_pos, trt_ref = "C")$agd_arm$.trt,
               agd_arm$trtf2)
  expect_equal(set_agd_contrast(agd_contrast, studyf, trtf2, y = ydiff, se = sediff, trt_ref = "C")$agd_contrast$.trt,
               agd_contrast$trtf2)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf2, Surv = Surv(cont_pos, bin), trt_ref = "C")$agd_arm$.trt,
               agd_arm$trtf2)

  # Check that unused factor levels are dropped
  expect_equal(set_ipd(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), y = cont)$ipd$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_arm(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), y = cont, se = cont_pos)$agd_arm$.trt,
               agd_arm$trtf)
  expect_equal(set_agd_contrast(agd_contrast, studyf, forcats::fct_expand(trtf, "zzz"), y = ydiff, se = sediff)$agd_contrast$.trt,
               agd_contrast$trtf)
  expect_equal(set_agd_surv(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), Surv = Surv(cont_pos, bin))$agd_arm$.trt,
               if (bmax) forcats::fct_relevel(agd_arm$trtf, "B", "A", "C") else agd_arm$trtf)

  expect_equal(set_ipd(agd_arm, studyc, forcats::fct_expand(trtf2, "zzz"), y = cont, trt_ref = "C")$ipd$.trt,
               agd_arm$trtf2)
  expect_equal(set_agd_arm(agd_arm, studyc, forcats::fct_expand(trtf2, "zzz"), y = cont, se = cont_pos, trt_ref = "C")$agd_arm$.trt,
               agd_arm$trtf2)
  expect_equal(set_agd_contrast(agd_contrast, studyf, forcats::fct_expand(trtf2, "zzz"), y = ydiff, se = sediff, trt_ref = "C")$agd_contrast$.trt,
               agd_contrast$trtf2)
  expect_equal(set_agd_surv(agd_arm, studyc, forcats::fct_expand(trtf2, "zzz"), Surv = Surv(cont_pos, bin), trt_ref = "C")$agd_arm$.trt,
               agd_arm$trtf2)
})

test_that("set_* `.study` column is correct", {
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, 2, trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, 2, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, 2, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, 2, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, "studyc", trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, "studyc", trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, "studyc", trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, "studyc", trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, factor(studyc), trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, factor(studyc), trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, factor(studyc), trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, factor(studyc), trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, studyf, trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, studyf, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, studyf, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, studyf, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, studyf2, trtc, y = cont)$ipd$.study,
               agd_arm$studyf2)
  expect_equal(set_agd_arm(agd_arm, studyf2, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf2)
  expect_equal(set_agd_contrast(agd_contrast, studyf2, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf2)
  expect_equal(set_agd_surv(agd_arm, studyf2, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf2)

  # Check that unused levels are dropped
  expect_equal(set_ipd(agd_arm, forcats::fct_expand(studyf, "zzz"), trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(agd_arm, forcats::fct_expand(studyf, "zzz"), trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(agd_contrast, forcats::fct_expand(studyf, "zzz"), trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(agd_arm, forcats::fct_expand(studyf, "zzz"), trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(agd_arm, forcats::fct_expand(studyf2, "zzz"), trtc, y = cont)$ipd$.study,
               agd_arm$studyf2)
  expect_equal(set_agd_arm(agd_arm, forcats::fct_expand(studyf2, "zzz"), trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf2)
  expect_equal(set_agd_contrast(agd_contrast, forcats::fct_expand(studyf2, "zzz"), trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf2)
  expect_equal(set_agd_surv(agd_arm, forcats::fct_expand(studyf2, "zzz"), trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf2)

  # Check reserved column names handled correctly
  aa <- mutate(agd_arm, .study = studyc)
  ac <- mutate(agd_contrast, .study = studyc)
  expect_equal(set_ipd(aa, .study, trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(aa, .study, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(ac, .study, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(aa, .study, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_surv(aa, .study, trtc, Surv = Surv(cont_pos, bin), covariates = aa)$agd_arm$.study,
               agd_arm$studyf)

  expect_equal(set_ipd(aa, 26, trtc, y = cont)$ipd$.study,
               agd_arm$studyf)
  expect_equal(set_agd_arm(aa, 26, trtc, y = cont, se = cont_pos)$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_contrast(ac, 32, trtc, y = ydiff, se = sediff)$agd_contrast$.study,
               agd_contrast$studyf)
  expect_equal(set_agd_surv(aa, 26, trtc, Surv = Surv(cont_pos, bin))$agd_arm$.study,
               agd_arm$studyf)
  expect_equal(set_agd_surv(aa, 26, trtc, Surv = Surv(cont_pos, bin), covariates = aa)$agd_arm$.study,
               agd_arm$studyf)
})

test_that("set_* return default `treatments` factor", {
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont)$treatments,
               .default(factor(LETTERS[1:3])))
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos)$treatments,
               .default(factor(LETTERS[1:3])))
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff)$treatments,
               .default(factor(LETTERS[1:3])))
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin))$treatments,
               if (bmax) .default(factor(c("B", "A", "C"), levels = c("B", "A", "C"))) else .default(factor(LETTERS[1:3])))
})

test_that("set_* can set `trt_ref`", {
  f_BAC <- factor(LETTERS[c(2,1,3)], levels = LETTERS[c(2,1,3)])
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_ref = factor("B"))$treatments,
               f_BAC)

  # Using trtf sets original_levels attribute
  attr(f_BAC, "original_levels") <- LETTERS[1:3]
  expect_equal(set_ipd(agd_arm, studyc, trtf, y = cont, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf, y = cont, se = cont_pos, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtf, y = ydiff, se = sediff, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf, Surv = Surv(cont_pos, bin), trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_ipd(agd_arm, studyc, trtf, y = cont, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf, y = cont, se = cont_pos, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtf, y = ydiff, se = sediff, trt_ref = factor("B"))$treatments,
               f_BAC)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf, Surv = Surv(cont_pos, bin), trt_ref = factor("B"))$treatments,
               f_BAC)

  f_213 <- factor(c(2, 1, 3), levels = c(2, 1, 3))
  expect_equal(set_ipd(agd_arm, studyc, trtn, y = cont, trt_ref = 2)$treatments,
               f_213)
  expect_equal(set_agd_arm(agd_arm, studyc, trtn, y = cont, se = cont_pos, trt_ref = 2)$treatments,
               f_213)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtn, y = ydiff, se = sediff, trt_ref = 2)$treatments,
               f_213)
  expect_equal(set_agd_surv(agd_arm, studyc, trtn, Surv = Surv(cont_pos, bin), trt_ref = 2)$treatments,
               f_213)

  f_BCA <- factor(LETTERS[c(2,3,1)], levels = LETTERS[c(2,3,1)])
  attr(f_BCA, "original_levels") <- LETTERS[3:1]
  expect_equal(set_ipd(agd_arm, studyc, trtf2, y = cont, trt_ref = "B")$treatments,
               f_BCA)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf2, y = cont, se = cont_pos, trt_ref = "B")$treatments,
               f_BCA)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtf2, y = ydiff, se = sediff, trt_ref = "B")$treatments,
               f_BCA)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf2, Surv = Surv(cont_pos, bin), trt_ref = "B")$treatments,
               f_BCA)
  expect_equal(set_ipd(agd_arm, studyc, trtf2, y = cont, trt_ref = factor("B"))$treatments,
               f_BCA)
  expect_equal(set_agd_arm(agd_arm, studyc, trtf2, y = cont, se = cont_pos, trt_ref = factor("B"))$treatments,
               f_BCA)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtf2, y = ydiff, se = sediff, trt_ref = factor("B"))$treatments,
               f_BCA)
  expect_equal(set_agd_surv(agd_arm, studyc, trtf2, Surv = Surv(cont_pos, bin), trt_ref = factor("B"))$treatments,
               f_BCA)

  # Check that unused levels are dropped
  attr(f_BAC, "original_levels") <- c("A", "B", "C", "zzz")
  expect_equal(set_ipd(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), y = cont, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_arm(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), y = cont, se = cont_pos, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_contrast(agd_contrast, studyc, forcats::fct_expand(trtf, "zzz"), y = ydiff, se = sediff, trt_ref = "B")$treatments,
               f_BAC)
  expect_equal(set_agd_surv(agd_arm, studyc, forcats::fct_expand(trtf, "zzz"), Surv = Surv(cont_pos, bin), trt_ref = "B")$treatments,
               f_BAC)

  m <- "`trt_ref` does not match a treatment in the data.+Suitable values are:"
  expect_error(set_ipd(agd_arm, studyc, trtc, y = cont, trt_ref = 2), m)
  expect_error(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_ref = 2), m)
  expect_error(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_ref = 2), m)
  expect_error(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_ref = 2), m)
})

# Check classes when default reference treatment is not first in sort order
# Add new study to make B the default trt_ref
newstudy <- tibble(studyc = "c", trtc = c("B", "C"), tclassc = "b",
                   cont = rnorm(2), cont_pos = runif(2, 0, 1), bin = c(0, 1),
                   ydiff = c(NA, cont[2]), sediff = 1)
aa <- bind_rows(agd_arm, newstudy)
ac <- bind_rows(agd_contrast, newstudy)

test_that("set_* returns correct .trtclass column", {
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = tclassc)$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc)$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc)$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = tclassc)$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = tclassc))$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc))$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc))$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassc))$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)

  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = 11)$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = 11)$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = 11)$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = 11)$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = 11))$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = 11))$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = 11))$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = 11))$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)

  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = tclassf)$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassf)$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassf)$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = tclassf)$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = tclassf))$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = tclassf))$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = tclassf))$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassf))$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)

  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont, trt_ref = "B",
                       trt_class = tclassf2)$ipd$.trtclass,
               agd_arm$tclassf2)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_ref = "B",
                           trt_class = tclassf2)$agd_arm$.trtclass,
               agd_arm$tclassf2)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_ref = "B",
                                trt_class = tclassf2)$agd_contrast$.trtclass,
               agd_contrast$tclassf2)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_ref = "B",
                           trt_class = tclassf2)$agd_arm$.trtclass,
               agd_arm$tclassf2)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = tclassf2), trt_ref = "B")$ipd$.trtclass,
               agd_arm$tclassf2)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = tclassf2), trt_ref = "B")$agd_arm$.trtclass,
               agd_arm$tclassf2)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = tclassf2), trt_ref = "B")$agd_contrast$.trtclass,
               agd_contrast$tclassf2)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassf2), trt_ref = "B")$agd_arm$.trtclass,
               agd_arm$tclassf2)

  # Check that unused factor levels are dropped
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = forcats::fct_expand(tclassf, "zzz"))$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = forcats::fct_expand(tclassf, "zzz"))$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = forcats::fct_expand(tclassf, "zzz"))$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = forcats::fct_expand(tclassf, "zzz"))$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = forcats::fct_expand(tclassf, "zzz")))$ipd$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = forcats::fct_expand(tclassf, "zzz")))$agd_arm$.trtclass,
               agd_arm$tclassf)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = forcats::fct_expand(tclassf, "zzz")))$agd_contrast$.trtclass,
               agd_contrast$tclassf)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = forcats::fct_expand(tclassf, "zzz")))$agd_arm$.trtclass,
               if (bmax) forcats::fct_relevel(agd_arm$tclassf, "b") else agd_arm$tclassf)

  # Checks when default trt_ref not first in sort order
  expect_equal(set_ipd(aa, studyc, trtc, y = cont,
                       trt_class = tclassc)$ipd$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
  expect_equal(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc)$agd_arm$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
  expect_equal(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc)$agd_contrast$.trtclass,
               factor(ac$tclassc, levels = c("b", "a")))
  expect_equal(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = tclassc)$agd_arm$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
  expect_equal(combine_network(set_ipd(aa, studyc, trtc, y = cont,
                                       trt_class = tclassc))$ipd$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
  expect_equal(combine_network(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = tclassc))$agd_arm$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
  expect_equal(combine_network(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = tclassc))$agd_contrast$.trtclass,
               factor(ac$tclassc, levels = c("b", "a")))
  expect_equal(combine_network(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassc))$agd_arm$.trtclass,
               factor(aa$tclassc, levels = c("b", "a")))
})

test_that("set_* returns classes factor variable", {
  f_class <- factor(c("a", "b", "b"))
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont, trt_class = tclassc)$classes,
               f_class)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_class = tclassc)$classes,
               f_class)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_class = tclassc)$classes,
               f_class)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_class = tclassc)$classes,
               if (bmax) factor(c("b", "a", "b"), levels = c("b", "a")) else f_class)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont, trt_class = tclassc))$classes,
               f_class)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_class = tclassc))$classes,
               f_class)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff, trt_class = tclassc))$classes,
               f_class)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_class = tclassc))$classes,
               if (bmax) factor(c("b", "a", "b"), levels = c("b", "a")) else f_class)

  # Using tclassf sets original_levels attribute
  attr(f_class, "original_levels") <- c("a", "b")
  if (bmax) {
    f_classs <- factor(c("b", "a", "b"), levels = c("b", "a"))
    attr(f_classs, "original_levels") <- c("a", "b")
  }
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"))$classes,
               f_class)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"))$classes,
               f_class)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"))$classes,
               f_class)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"))$classes,
               if (bmax) f_classs else f_class)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")))$classes,
               if (bmax) f_classs else f_class)

  f_class2 <- factor(c("b", "a", "b"), levels = c("b", "a"))
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = tclassc, trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc, trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc, trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = tclassc, trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = tclassc), trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc), trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc), trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassc), trt_ref = "B")$classes,
               f_class2)

  # Checks when default trt_ref not first in sort order
  expect_equal(set_ipd(aa, studyc, trtc, y = cont,
                       trt_class = tclassc)$classes,
               f_class2)
  expect_equal(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = tclassc)$classes,
               f_class2)
  expect_equal(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = tclassc)$classes,
               f_class2)
  expect_equal(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = tclassc)$classes,
               f_class2)
  expect_equal(combine_network(set_ipd(aa, studyc, trtc, y = cont,
                                       trt_class = tclassc))$classes,
               f_class2)
  expect_equal(combine_network(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = tclassc))$classes,
               f_class2)
  expect_equal(combine_network(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = tclassc))$classes,
               f_class2)
  expect_equal(combine_network(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = tclassc))$classes,
               f_class2)

  attr(f_class2, "original_levels") <- c("a", "b")
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                       trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                           trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                                trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                           trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)

  # Checks when default trt_ref not first in sort order
  expect_equal(set_ipd(aa, studyc, trtc, y = cont,
                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                       trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                           trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                                trt_ref = "B")$classes,
               f_class2)
  expect_equal(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b"),
                           trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_ipd(aa, studyc, trtc, y = cont,
                                       trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_arm(aa, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_contrast(ac, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)
  expect_equal(combine_network(set_agd_surv(aa, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = recode_factor(trtc, A = "a", B = "b", C = "b")),
                               trt_ref = "B")$classes,
               f_class2)

  f_class3 <- factor(c("B", "A", "C"), levels = c("B", "C", "A"))
  attr(f_class3, "original_levels") <- c("C", "B", "A")
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = trtf2,
                       trt_ref = "B")$classes,
               f_class3)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = trtf2,
                           trt_ref = "B")$classes,
               f_class3)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = trtf2,
                                trt_ref = "B")$classes,
               f_class3)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = trtf2,
                           trt_ref = "B")$classes,
               f_class3)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = trtf2),
                               trt_ref = "B")$classes,
               f_class3)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = trtf2),
                               trt_ref = "B")$classes,
               f_class3)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = trtf2),
                               trt_ref = "B")$classes,
               f_class3)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = trtf2),
                               trt_ref = "B")$classes,
               f_class3)

  # Check that unused levels are dropped
  attr(f_class, "original_levels") <- c("a", "b", "zzz")
  if (bmax) attr(f_classs, "original_levels") <- c("a", "b", "zzz")
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont,
                       trt_class = forcats::fct_expand(tclassf, "zzz"))$classes,
               f_class)
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                           trt_class = forcats::fct_expand(tclassf, "zzz"))$classes,
               f_class)
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                trt_class = forcats::fct_expand(tclassf, "zzz"))$classes,
               f_class)
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                           trt_class = forcats::fct_expand(tclassf, "zzz"))$classes,
               if (bmax) f_classs else f_class)
  expect_equal(combine_network(set_ipd(agd_arm, studyc, trtc, y = cont,
                                       trt_class = forcats::fct_expand(tclassf, "zzz")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos,
                                           trt_class = forcats::fct_expand(tclassf, "zzz")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff,
                                                trt_class = forcats::fct_expand(tclassf, "zzz")))$classes,
               f_class)
  expect_equal(combine_network(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin),
                                           trt_class = forcats::fct_expand(tclassf, "zzz")))$classes,
               if (bmax) f_classs else f_class)
})

test_that("set_* checks for bad class variable work", {
  aa2 <- agd_arm
  aa2$tclassn <- c(1, 2, 2, 1, 2) # Trt 2 and 3 in two classes
  aa2$tclassc[1] <- NA

  ac2 <- agd_contrast
  ac2$tclassn <- c(1, 2, 2, 1, 2) # Trt 2 and 3 in two classes
  ac2$tclassc[1] <- NA

  m <- "Treatment present in more than one class"

  expect_error(set_ipd(aa2, studyc, trtc, y = cont, trt_class = tclassn), m)
  expect_error(set_agd_arm(aa2, studyc, trtc, y = cont, se = cont_pos, trt_class = tclassn), m)
  expect_error(set_agd_contrast(ac2, studyc, trtc, y = ydiff, se = sediff, trt_class = tclassn), m)
  expect_error(set_agd_surv(aa2, studyc, trtc, Surv = Surv(cont_pos, bin), trt_class = tclassn), m)

  m2 <- "cannot contain missing values"

  expect_error(set_ipd(aa2, studyc, trtc, y = cont, trt_class = tclassc), m2)
  expect_error(set_agd_arm(aa2, studyc, trtc, y = cont, se = cont_pos, trt_class = tclassc), m2)
  expect_error(set_agd_contrast(ac2, studyc, trtc, y = ydiff, se = sediff, trt_class = tclassc), m2)
  expect_error(set_agd_surv(aa2, studyc, trtc, Surv = Surv(cont_pos, bin), trt_class = tclassc), m2)

  m3 <- "must be a regular column"
  expect_error(set_ipd(agd_arm, studyc, trtc, y = cont, trt_class = cbind(trtc, trtc)), m3)
  expect_error(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos, trt_class = cbind(trtc, trtc)), m3)
  expect_error(set_agd_contrast(agd_contrast, studyc, trtc, y = cont, se = cont_pos, trt_class = cbind(trtc, trtc)), m3)
  expect_error(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin), trt_class = cbind(trtc, trtc)), m3)
})

test_that("set_* return `studies` factor", {
  expect_equal(set_ipd(agd_arm, studyc, trtc, y = cont)$studies,
               factor(letters[1:2]))
  expect_equal(set_agd_arm(agd_arm, studyc, trtc, y = cont, se = cont_pos)$studies,
               factor(letters[1:2]))
  expect_equal(set_agd_contrast(agd_contrast, studyc, trtc, y = ydiff, se = sediff)$studies,
               factor(letters[1:2]))
  expect_equal(set_agd_surv(agd_arm, studyc, trtc, Surv = Surv(cont_pos, bin))$studies,
               factor(letters[1:2]))
})

make_na <- function(x, n) {
  x[sample.int(length(x), n)] <- NA
  return(x)
}

test_that("set_* error if outcomes contain missing values", {
  agd_arm_miss <- agd_arm %>% mutate_at(vars(cont:bin), ~make_na(., 1))

  m <- "contains missing values"

  expect_error(set_agd_arm(agd_arm_miss, studyn, trtn, y = cont, se = cont_pos), m)
  expect_error(set_agd_arm(agd_arm_miss, studyn, trtn, r = disc, n = disc_p1), m)
  expect_error(set_agd_arm(agd_arm_miss, studyn, trtn, r = disc, E = cont_pos), m)

  expect_error(set_ipd(agd_arm_miss, studyn, trtn, y = cont), m)
  expect_error(set_ipd(agd_arm_miss, studyn, trtn, r = disc), m)
  expect_error(set_ipd(agd_arm_miss, studyn, trtn, r = disc, E = cont_pos), m)

  # For set_agd_contrast, multiple missing y will trigger multiple baselines error
  # Missing se on baseline arm will trigger covariance error if >2 arms (otherwise fine)
  agd_contrast_miss <- agd_contrast %>% mutate(sediff = if_else(arm > 1, NA_real_, sediff))
  expect_error(set_agd_contrast(agd_contrast_miss, studyn, trtn, y = ydiff, se = sediff), m)
})

library(multinma)
library(dplyr)

# Expose internal nma.fit()
nma.fit <- multinma:::nma.fit

test_that("nma() wants a nma_data object", {
  m <- "Expecting an `nma_data` object"
  expect_error(nma(network = 1), m)
  expect_error(nma(network = list()), m)
  expect_error(nma(network = NULL), m)
})

smknet <- set_agd_arm(smoking, studyn, trtc, r = r, n = n, trt_ref = "No intervention")

test_that("nma() consistency argument must be one listed", {
  m <- "`consistency` must be"
  expect_error(nma(smknet, consistency = "con"), m)
  expect_error(nma(smknet, consistency = "a"), m)
  expect_error(nma(smknet, consistency = 1), m)
  expect_error(nma(smknet, consistency = NA), m)
  expect_error(nma(smknet, consistency = NULL), m)

  # Not needed - rlang::arg_match always returns a single string
  # expect_error(nma(smknet, consistency = c("consistency", "ume")), m)
})

test_that("nma() trt_effects argument must be one listed", {
  m <- "`trt_effects` must be"
  expect_error(nma(smknet, trt_effects = "fix"), m)
  expect_error(nma(smknet, trt_effects = "a"), m)
  expect_error(nma(smknet, trt_effects = 1), m)
  expect_error(nma(smknet, trt_effects = NA), m)
  expect_error(nma(smknet, trt_effects = NULL), m)

  # Not needed - rlang::arg_match always returns a single string
  # expect_error(nma(smknet, trt_effects = c("fixed", "random")), m)
})

test_that("nma() likelihood must be valid", {
  m <- "`likelihood` should be"
  expect_error(nma(smknet, likelihood = 1), m)
  expect_error(nma(smknet, likelihood = "a"), m)
  expect_error(nma(smknet, likelihood = "norm"), m)
  expect_error(nma(smknet, likelihood = c("normal", "bernoulli")), m)
})

test_that("nma() link must be valid", {
  m <- "`link` should be"
  expect_error(nma(smknet, link = 1), m)
  expect_error(nma(smknet, link = "a"), m)
  expect_error(nma(smknet, link = "lo"), m)
  expect_error(nma(smknet, link = c("log", "identity")), m)
})

sa_net <- set_agd_contrast(social_anxiety, studyc, trtc, y = y, se = se, trt_class = classc, trt_ref = "Waitlist")

#Class effect parameter error checks
test_that("nma() class_effect must be valid", {
  m <- "`class_effects` must be"
  expect_error(nma(sa_net, class_effects = c("independent", "common")), m)
  expect_error(nma(sa_net, class_effects = 1), m)
  expect_error(nma(sa_net, class_effects = "random"), m)
})

test_that("nma() class_sd lists must be valid", {
  expect_error(nma(sa_net, class_effect = "exchangeable", class_sd = list(group1 = c("Exposure", "NonExistentClass"))), "Some classes listed in 'class_sd' are not present in the network", fixed = TRUE)
  expect_error(nma(sa_net, class_effect = "exchangeable", class_sd = list(group1 = c("Exposure", "NSSA"), group2 = c("NSSA", "SSRI/SNRI"))), "Some classes are listed in more than one shared standard deviation group in 'class_sd'")
})

test_that("nma() class_sd must be valid", {
  m_class_sd <- "`class_sd` must be"
  expect_error(nma(sa_net, class_effect = "exchangeable", class_sd = 1), m_class_sd)
  expect_error(nma(sa_net, class_effect = "exchangeable", class_sd = "a"), m_class_sd)
})

test_that("nma() gives an informative error if class_effect are specified without classes in the network", {
  m_no_classes <- "Setting `class_effects` requires treatment classes to be specified"
  expect_error(nma(smknet, class_effect = "common"), m_no_classes)
  expect_error(nma(smknet, class_effect = "exchangeable"), m_no_classes)
})

test_that("class_effects = 'common' updates network$treatments", {
  net_common <- nma(sa_net,
                    class_effects = "common",
                    trt_effects = "random",
                    prior_trt = normal(0, 100),
                    prior_het = half_normal(5),
                    test_grad = TRUE)

  # Check that net_common$network$treatments is updated to classes
  expect_equal(levels(net_common$network$treatments),
               levels(sa_net$classes),
               info = "network$treatments should be replaced by class factor levels")
})

# Make dummy covariate data for smoking network
ns_agd <- max(smoking$studyn)
smkdummy <-
  smoking %>%
  group_by(studyn) %>%
  mutate(x1_mean = rnorm(1), x1_sd = runif(1, 0.5, 2),
         x2 = runif(1),
         x3_mean = rnorm(1), x3_sd = runif(1, 0.5, 2))

ns_ipd <- 2

x1_x2_cor <- 0.4
n_i <- 200
cormat <- matrix(x1_x2_cor, nrow = 3, ncol = 3)
diag(cormat) <- 1

cop <- copula::normalCopula(copula::P2p(cormat), dim = 3, dispstr = "un")
u <- matrix(runif(n_i * 3 * ns_ipd), ncol = 3)
u_cor <- as.data.frame(copula::cCopula(u, cop, inverse = TRUE))

ipddummy <-
  tibble(studyn = rep(ns_agd, n_i * ns_ipd) + rep(1:ns_ipd, each = n_i),
         trtn = sample(1:2, n_i * ns_ipd, TRUE)) %>%
  mutate(x1 = qnorm(u_cor[,1]),
         x2 = qbinom(u_cor[,2], 1, 0.6),
         x3 = qnorm(u_cor[,3], 1, 0.5),
         r = rbinom(n(), 1, 0.2))

smknet_agd <- set_agd_arm(smkdummy, studyn, trtn, r = r, n = n)
smknet_ipd <- set_ipd(ipddummy, studyn, trtn, r = r)
smknet_2 <- combine_network(smknet_agd, smknet_ipd) %>%
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd),
                  x2 = distr(qbinom, 1, x2),
                  x3 = distr(qnorm, x3_mean, x3_sd))

test_that("nma() regression formula is valid", {
  expect_error(nma(smknet_2, regression = y ~ x, center = FALSE), "one-sided formula")
  expect_error(nma(smknet_2, regression = ~ a, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ a*.trt, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ a^2*.trt, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ (a + b)*.trt, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variables "a" and "b" not found')

  expect_error(nma(smknet_2, regression = y ~ x, center = TRUE), "one-sided formula")
  expect_error(nma(smknet_2, regression = ~ a, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ a*.trt, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ a^2*.trt, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variable "a" not found')
  expect_error(nma(smknet_2, regression = ~ (a + b)*.trt, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), 'Regression variables "a" and "b" not found')
})


test_that("aux_regression ignored with warning", {
  surv_net <- set_ipd(ndmm_agd, study, trt, Surv = Surv(eventtime, status))

  expect_warning(f <- nma(smknet, aux_regression = ~.trt,
                     prior_intercept = normal(0, 10),
                     prior_trt = normal(0, 10),
                     prior_aux_reg = normal(0, 5), test_grad = TRUE),
                 "Ignoring `aux_regression`, only supported for survival")

  expect_null(f$aux_regression)

  expect_warning(f <- nma(surv_net, aux_regression = ~.trt,
                          likelihood = "exponential",
                          prior_intercept = normal(0, 10),
                          prior_trt = normal(0, 10), test_grad = TRUE),
                 "Ignoring `aux_regression`, no auxiliary parameters in exponential model")

  expect_null(f$aux_regression)
})

test_that("aux_by ignored with warning", {
  surv_net <- set_ipd(ndmm_agd, study, trt, Surv = Surv(eventtime, status))

  expect_warning(f <- nma(smknet, aux_by = ".trt",
                          prior_intercept = normal(0, 10),
                          prior_trt = normal(0, 10),
                          prior_aux_reg = normal(0, 5), test_grad = TRUE),
                 "Ignoring `aux_by`, only supported for survival")

  expect_null(f$aux_by)

  expect_warning(f <- nma(surv_net, aux_by = ".trt",
                          likelihood = "exponential",
                          prior_intercept = normal(0, 10),
                          prior_trt = normal(0, 10), test_grad = TRUE),
                 "Ignoring `aux_by`, no auxiliary parameters in exponential model")

  expect_null(f$aux_by)
})

make_na <- function(x, n) {
  x[sample.int(length(x), n)] <- NA
  return(x)
}

smknet_agd_missx <- smkdummy %>%
  mutate(x1_mean = make_na(x1_mean, 2), x1 = x1_mean) %>%
  set_agd_arm(studyn, trtn, r = r, n = n)

smknet_ipd_missx <- ipddummy %>%
  mutate(x1 = make_na(x1, 10)) %>%
  set_ipd(studyn, trtn, r = r)

test_that("nma() error if missing values in outcomes or predictors", {
  m <- "missing or infinite values"
  expect_error(nma(smknet_agd_missx, regression = ~x1_mean, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), m)
  expect_error(nma(smknet_agd_missx, regression = ~x1_mean, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), m)
  expect_error(nma(smknet_ipd_missx, regression = ~x1, center = FALSE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), m)
  expect_error(nma(smknet_ipd_missx, regression = ~x1, center = TRUE,
                   prior_intercept = normal(0, 10),
                   prior_trt = normal(0, 10),
                   prior_reg = normal(0, 5)), m)
  expect_error(suppressWarnings(
    # Suppress warning about naive plug-in model
    nma(combine_network(smknet_agd_missx, smknet_ipd_missx),
        regression = ~x1, center = FALSE)), m)
  expect_error(suppressWarnings(
    # Suppress warning about naive plug-in model
    nma(combine_network(smknet_agd_missx, smknet_ipd_missx),
        regression = ~x1, center = TRUE)), m)
})

smknet_3l <- combine_network(
  set_agd_arm(smkdummy %>% mutate(extra = 0.5, tclass = if_else(trtn == 1, 0, 1)),
              studyn, trtn, r = r, n = n, trt_class = tclass),
  set_ipd(ipddummy %>% mutate(extra = TRUE, tclass = if_else(trtn == 1, 0, 1)),
          studyn, trtn, r = r, trt_class = tclass),
  trt_ref = 1) %>%
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd),
                  x2 = distr(qbinom, 1, x2),
                  x3 = distr(qnorm, x3_mean, x3_sd))

smknet_3c <- combine_network(
  set_agd_arm(smkdummy %>% mutate(extra = 0.5, tclass = if_else(trtn == 1, 0, 1)),
              studyn, trtn, r = r, n = n, trt_class = tclass),
  set_ipd(ipddummy %>% mutate(extra = "a", tclass = if_else(trtn == 1, 0, 1)),
          studyn, trtn, r = r, trt_class = tclass),
  trt_ref = 1) %>%
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd),
                  x2 = distr(qbinom, 1, x2),
                  x3 = distr(qnorm, x3_mean, x3_sd))

test_that("nma() silently drops unnecessary columns", {
  expect_error(nma(smknet_3l, regression = ~(x1 + x2 + x3):.trt,
                    prior_intercept = normal(0, 10),
                    prior_trt = normal(0, 10),
                    prior_reg = normal(0, 5),
                    test_grad = TRUE), NA)

  expect_error(nma(smknet_3c, regression = ~(x1 + x2 + x3):.trt,
                    prior_intercept = normal(0, 10),
                    prior_trt = normal(0, 10),
                    prior_reg = normal(0, 5),
                    test_grad = TRUE), NA)
})

test_that("nma.fit() error if only one of x or y provided", {
  x <- matrix(1, nrow = 3, ncol = 3)
  colnames(x) <- c("a", "b", "c")

  y <- tibble(.y = 1:3)

  m <- "both be present or both NULL"
  expect_error(nma.fit(ipd_x = x), m)
  expect_error(nma.fit(ipd_y = y), m)
  expect_error(nma.fit(agd_arm_x = x), m)
  expect_error(nma.fit(agd_arm_y = y), m)
  expect_error(nma.fit(agd_contrast_x = x), "all be present or all NULL")
  expect_error(nma.fit(agd_contrast_y = y), "all be present or all NULL")
  expect_error(nma.fit(agd_contrast_Sigma = list()), "all be present or all NULL")
})

test_that("nma.fit() error if x and y dimension mismatch", {
  x <- matrix(1, nrow = 3, ncol = 3)
  colnames(x) <- c("a", "b", "c")

  y <- tibble(.y = 1:2)

  m <- "Number of rows.+do not match"
  expect_error(nma.fit(ipd_x = x, ipd_y = y), m)
  expect_error(nma.fit(agd_arm_x = x, agd_arm_y = y, n_int = 1), m)
  expect_error(nma.fit(agd_contrast_x = x, agd_contrast_y = y, agd_contrast_Sigma = list(), n_int = 1), m)
})

test_that("nma.fit() error if x column names different", {
  x1 <- x2 <- matrix(1, nrow = 3, ncol = 3)
  colnames(x1) <- c(".study1", ".trt1", "c")
  colnames(x2) <- c(".study1", ".trt1", "D")
  x3 <- x1[, 1:2]

  y <- tibble(.y = 1:3, .se = 1:3)

  Sigma <- list(matrix(1, 3, 3))

  m <- "Non-matching columns"
  expect_error(nma.fit(ipd_x = x1, ipd_y = y,
                       agd_arm_x = x2, agd_arm_y = y,
                       n_int = 1), m)
  expect_error(nma.fit(ipd_x = x1, ipd_y = y,
                       agd_contrast_x = x2, agd_contrast_y = y, agd_contrast_Sigma = Sigma,
                       n_int = 1), m)
  expect_error(nma.fit(agd_arm_x = x1, agd_arm_y = y,
                       agd_contrast_x = x2, agd_contrast_y = y, agd_contrast_Sigma = Sigma,
                       n_int = 1), m)
  expect_error(nma.fit(ipd_x = x1, ipd_y = y,
                       agd_arm_x = x3, agd_arm_y = y,
                       n_int = 1), m)
  expect_error(nma.fit(ipd_x = x1, ipd_y = y,
                       agd_contrast_x = x3, agd_contrast_y = y, agd_contrast_Sigma = Sigma,
                       n_int = 1), m)
  expect_error(nma.fit(agd_arm_x = x2, agd_arm_y = y,
                       agd_contrast_x = x3, agd_contrast_y = y, agd_contrast_Sigma = Sigma,
                       n_int = 1), m)
})

test_that("nma.fit() error if agd_contrast_Sigma is not right dimensions", {
  x1 <- matrix(1, nrow = 3, ncol = 3)
  colnames(x1) <- c(".study1", ".trt1", "x")

  y <- tibble(.y = 1:3, .se = 1:3)

  Sigma1 <- list(matrix(1, 3, 3), matrix(1, 1, 1))
  Sigma2 <- list(matrix(1, 4, 4))

  expect_error(nma.fit(agd_contrast_x = x1, agd_contrast_y = y,
                       agd_contrast_Sigma = Sigma1, likelihood = "normal", link = "identity",
                       prior_intercept = normal(0, 10),
                       prior_trt = normal(0, 10),
                       prior_reg = normal(0, 5),
                       prior_het = normal(0, 1),
                       n_int = 1), "Dimensions of `agd_contrast_Sigma`.+do not match")
  expect_error(nma.fit(agd_contrast_x = x1, agd_contrast_y = y,
                       agd_contrast_Sigma = Sigma2, likelihood = "normal", link = "identity",
                       prior_intercept = normal(0, 10),
                       prior_trt = normal(0, 10),
                       prior_reg = normal(0, 5),
                       prior_het = normal(0, 1),
                       n_int = 1), "Dimensions of `agd_contrast_Sigma`.+do not match")
})


single_study_a <- tibble(study = "A", trt = c("a", "b"), r = 500, n = 1000)
net_ss_a <- set_agd_arm(single_study_a, study, trt, r = r, n = n)

single_study_c <- tibble(study = "A", trt = c("a", "b"), y = c(NA, 0), se = c(NA, sqrt(4/500)))
net_ss_c <- set_agd_contrast(single_study_c, study, trt, y = y, se = se)

single_study_i <- tibble(study = "A", trt = rep(c("a", "b"), each = 100), y = c(rnorm(100, 0, 0.1), rnorm(100, 1, 0.1)))
net_ss_i <- set_ipd(single_study_i, study, trt, y = y)

test_that("nma() doesn't fail with a single study", {
  skip_on_cran()

  fit_ss_a <- nma(net_ss_a, prior_intercept = normal(0, 10), prior_trt = normal(0, 10))
  expect_s3_class(fit_ss_a, "stan_nma")
  d_ss_a <- relative_effects(fit_ss_a)
  expect_equivalent(d_ss_a$summary$mean, 0, tol = 0.01)
  expect_equivalent(d_ss_a$summary$sd, sqrt(4/500), tol = 0.01)

  fit_ss_c <- nma(net_ss_c, prior_intercept = normal(0, 10), prior_trt = normal(0, 10))
  expect_s3_class(fit_ss_c, "stan_nma")
  d_ss_c <- relative_effects(fit_ss_c)
  expect_equivalent(d_ss_c$summary$mean, 0, tol = 0.01)
  expect_equivalent(d_ss_c$summary$sd, sqrt(4/500), tol = 0.01)

  fit_ss_i <- nma(net_ss_i, prior_intercept = normal(0, 10), prior_trt = normal(0, 10), prior_aux = half_normal(2))
  expect_s3_class(fit_ss_i, "stan_nma")
  d_ss_i <- relative_effects(fit_ss_i)
  expect_equivalent(d_ss_i$summary$mean, 1, tol = 0.1)
  expect_equivalent(d_ss_i$summary$sd, sqrt(0.1^2 + 0.1^2)/sqrt(200), tol = 0.01)
})

test_that("nma() gives warnings for default priors", {
  m <- "Prior distributions were left at default values:"

  # Use test_grad = TRUE to avoid running any samples, just check logic works

  expect_warning(nma(smknet, test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+"))
  expect_warning(nma(smknet, prior_intercept = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_trt.+"))
  expect_warning(nma(smknet, prior_trt = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_intercept.+"))

  expect_warning(nma(smknet, trt_effects = "random", test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+", "prior_het.+"))
  expect_warning(nma(smknet, trt_effects = "random", prior_intercept = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_trt.+", "prior_het.+"))
  expect_warning(nma(smknet, trt_effects = "random", prior_trt = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_het.+"))
  expect_warning(nma(smknet, trt_effects = "random", prior_het = half_normal(1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+"))

  smknet_yi <- set_ipd(smoking, studyn, trtc, y = r)

  expect_warning(nma(smknet_yi, test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, prior_intercept = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_trt.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, prior_trt = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, prior_aux = half_normal(1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+"))

  expect_warning(nma(smknet_yi, trt_effects = "random", test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+", "prior_het.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, trt_effects = "random", prior_intercept = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_trt.+", "prior_het.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, trt_effects = "random", prior_trt = normal(0, 1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_het.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, trt_effects = "random", prior_het = half_normal(1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+", "prior_aux.+"))
  expect_warning(nma(smknet_yi, trt_effects = "random", prior_aux = half_normal(1), test_grad = TRUE), paste0(m, ".+prior_intercept.+", "prior_trt.+", "prior_het.+"))

})

test_that("nma() error with incompatible priors", {
  m <- "Invalid `prior_.+`\\. Suitable distributions are"

  expect_error(nma(smknet, prior_intercept = half_normal(1), prior_trt = normal(0, 1)), m)
  expect_error(nma(smknet, prior_intercept = normal(0, 1), prior_trt = half_normal(1)), m)
})

test_that("nma() prior checks work", {
  expect_error(nma(smknet, prior_intercept = "bad"),
               "`prior_intercept` must be a prior distribution")
  expect_error(nma(smknet, prior_intercept = list(normal(0, 1))),
               "`prior_intercept` must be a prior distribution")

  ndmm_net <- set_ipd(ndmm_ipd, studyf, trtf, Surv = Surv(eventtime, status))

  expect_error(nma(ndmm_net,
                   likelihood = "gengamma",
                   prior_intercept = normal(0, 100),
                   prior_trt = normal(0, 100),
                   prior_aux = letters),
               "`prior_aux` must be a named list of prior distributions")

  expect_error(nma(ndmm_net,
                   likelihood = "gengamma",
                   prior_intercept = normal(0, 100),
                   prior_trt = normal(0, 100),
                   prior_aux = list(a = normal(0, 1))),
               "Missing named elements with priors for sigma and k")

  expect_error(nma(ndmm_net,
                   likelihood = "gengamma",
                   prior_intercept = normal(0, 100),
                   prior_trt = normal(0, 100),
                   prior_aux = list(k = normal(0, 1))),
               "Missing named elements with priors for sigma")
})

test_that("rstan R-hat and ESS warnings are captured correctly", {
  # Need to capture these for numerical integration checks, but fragile since warning text may change
  local_edition(3)

  expect_warning(
  expect_warning(
  expect_warning(
    nma(smknet,
        prior_intercept = normal(0, 100),
        prior_trt = normal(0, 100),
        iter = 100,
        seed = 1234,
        init = 0),
    "The largest R-hat is"),
    "Bulk Effective Sample(s?) Size \\(ESS\\) is too low"),
    "Tail Effective Sample(s?) Size \\(ESS\\) is too low")
})

library(multinma)
library(dplyr)

test_that("combine_network error if not passed nma_data objects", {
  msg <- "Expecting to combine objects of class `nma_data`, created using set_\\* functions"

  expect_error(combine_network(1), msg)
  expect_error(combine_network(1, 2), msg)
  expect_error(combine_network(1, set_agd_arm(smoking, studyn, trtc, r = r, n = n)), msg)
  expect_error(combine_network(set_agd_arm(smoking, studyn, trtc, r = r, n = n), 1), msg)
})

# Dummy data
agd_arm <- tibble(
  studyn = c(1, 1, 2, 2, 2),
  studyc = letters[studyn],
  studyf = factor(studyc),
  studyf2 = factor(studyc, levels = letters[5:1]),
  trtn = c(1, 2, 1, 2, 3),
  trtc = LETTERS[trtn],
  trtf = factor(trtc),
  trtf2 = factor(trtc, levels = LETTERS[4:1]),
  tclassn = c(1, 2, 1, 2, 2),
  tclassc = letters[tclassn],
  tclassf = factor(tclassc, levels = letters[1:3]),
  tclassf2 = factor(tclassc, levels = letters[3:1]),
  y = rnorm(5),
  se = runif(5)
)
net_a_a <- set_agd_arm(agd_arm, studyf, trtf, y = y, se = se)
net_a_a2 <- set_agd_arm(agd_arm, studyf2, trtf2, y = y, se = se)

agd_contrast <- tibble(
  studyn = c(3, 3, 3),
  studyc = letters[studyn],
  studyf = factor(studyc),
  studyf2 = factor(studyc, levels = letters[5:1]),
  trtn = c(2, 3, 4),
  trtc = LETTERS[trtn],
  trtf = factor(trtc),
  trtf2 = factor(trtc, levels = LETTERS[4:1]),
  tclassn = c(2, 2, 3),
  tclassc = letters[tclassn],
  tclassf = factor(tclassc, levels = letters[1:3]),
  tclassf2 = factor(tclassc, levels = letters[3:1]),
  tclassn_bad = c(3, 3, 3),
  y = c(NA, rnorm(2)),
  se = c(0.5, 1, 1)
)
net_a_c <- set_agd_contrast(agd_contrast, studyf, trtf, y = y, se = se)
net_a_c2 <- set_agd_contrast(agd_contrast, studyf2, trtf2, y = y, se = se)

ipd <- tibble(
  studyn = c(4, 4, 4, 5, 5),
  studyc = letters[studyn],
  studyf = factor(studyc),
  studyf2 = factor(studyc, levels = letters[5:1]),
  trtn = c(1, 2, 3, 1, 4),
  trtc = LETTERS[trtn],
  trtf = factor(trtc),
  trtf2 = factor(trtc, levels = LETTERS[4:1]),
  tclassn = c(1, 2, 2, 1, 3),
  tclassc = letters[tclassn],
  tclassf = factor(tclassc, levels = letters[1:3]),
  tclassf2 = factor(tclassc, levels = letters[3:1]),
  tclassn_bad = c(1, 3, 3, 3, 4),
  y = rnorm(5)
)
net_i <- set_ipd(ipd, studyf, trtf, y = y)
net_i2 <- set_ipd(ipd, studyf2, trtf2, y = y)

test_that("combine_network produces combined treatment, class, and study factors", {
  # Note: original_levels attribute unset because levels differ in sub-networks

  c1 <- combine_network(net_a_a, net_i)
  expect_equal(c1$treatments, .default(factor(LETTERS[1:4])))
  expect_equal(levels(c1$agd_arm$.trt), LETTERS[1:4])
  expect_equal(levels(c1$ipd$.trt), LETTERS[1:4])
  expect_equal(c1$studies, factor(letters[c(1, 2, 4, 5)]))
  expect_equal(levels(c1$agd_arm$.study), letters[c(1, 2, 4, 5)])
  expect_equal(levels(c1$ipd$.study), letters[c(1, 2, 4, 5)])

  c2 <- combine_network(net_a_a, net_i, net_a_c)
  expect_equal(c2$treatments, .default(factor(LETTERS[c(2, 1, 3, 4)],
                                              levels = LETTERS[c(2, 1, 3, 4)])))
  expect_equal(levels(c2$agd_arm$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(levels(c2$agd_contrast$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(levels(c2$ipd$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(c2$studies, factor(letters[1:5]))
  expect_equal(levels(c2$agd_arm$.study), letters[1:5])
  expect_equal(levels(c2$agd_contrast$.study), letters[1:5])
  expect_equal(levels(c2$ipd$.study), letters[1:5])

  c1_classed <- combine_network(
    set_agd_arm(agd_arm, studyf, trtf, y = y, se = se, trt_class = tclassc),
    set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassc)
  )
  # Reference trt is A
  expect_equal(c1_classed$treatments, .default(factor(LETTERS[1:4])))
  expect_equal(levels(c1_classed$agd_arm$.trt), LETTERS[1:4])
  expect_equal(levels(c1_classed$ipd$.trt), LETTERS[1:4])
  expect_equal(c1_classed$studies, factor(letters[c(1, 2, 4, 5)]))
  expect_equal(levels(c1_classed$agd_arm$.study), letters[c(1, 2, 4, 5)])
  expect_equal(levels(c1_classed$ipd$.study), letters[c(1, 2, 4, 5)])
  expect_equal(c1_classed$classes, factor(letters[c(1, 2, 2, 3)]))
  expect_equal(levels(c1_classed$agd_arm$.trtclass), letters[1:3])
  expect_equal(levels(c1_classed$ipd$.trtclass), letters[1:3])

  c2_classed <- combine_network(
    set_agd_arm(agd_arm, studyf, trtf, y = y, se = se, trt_class = tclassc),
    set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassc),
    set_agd_contrast(agd_contrast, studyf, trtf, y = y, se = se, trt_class = tclassc)
  )
  # Reference treatment is B
  expect_equal(c2_classed$treatments, .default(factor(LETTERS[c(2, 1, 3, 4)],
                                                      levels = LETTERS[c(2, 1, 3, 4)])))
  expect_equal(levels(c2_classed$agd_arm$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(levels(c2_classed$agd_contrast$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(levels(c2_classed$ipd$.trt), LETTERS[c(2, 1, 3, 4)])
  expect_equal(c2_classed$studies, factor(letters[1:5]))
  expect_equal(levels(c2_classed$agd_arm$.study), letters[1:5])
  expect_equal(levels(c2_classed$agd_contrast$.study), letters[1:5])
  expect_equal(levels(c2_classed$ipd$.study), letters[1:5])
  expect_equal(c2_classed$classes, factor(letters[c(2, 1, 2, 3)], levels = c(letters[c(2, 1, 3)])))
  expect_equal(levels(c2_classed$agd_arm$.trtclass), letters[c(2, 1, 3)])
  expect_equal(levels(c2_classed$ipd$.trtclass), letters[c(2, 1, 3)])
  expect_equal(levels(c2_classed$agd_contrast$.trtclass), letters[c(2, 1, 3)])
})

test_that("combine_network produces combined treatment, class, and study factors from explicit factors", {
  # Note: original_levels attribute now set because levels are explicitly the same in sub-networks

  c1 <- combine_network(net_a_a2, net_i2)
  c1_trtf <- .default(factor(LETTERS[c(1, 4:2)], levels = LETTERS[c(1, 4:2)]))
  attr(c1_trtf, "original_levels") <- LETTERS[4:1]
  expect_equal(c1$treatments, c1_trtf)
  expect_equal(levels(c1$agd_arm$.trt), LETTERS[c(1, 4:2)])
  expect_equal(levels(c1$ipd$.trt), LETTERS[c(1, 4:2)])
  c1_studyf <- factor(letters[c(5, 4, 2, 1)], levels = letters[c(5, 4, 2, 1)])
  attr(c1_studyf, "original_levels") <- letters[5:1]
  expect_equal(c1$studies, c1_studyf)
  expect_equal(levels(c1$agd_arm$.study), letters[c(5, 4, 2, 1)])
  expect_equal(levels(c1$ipd$.study), letters[c(5, 4, 2, 1)])

  c2 <- combine_network(net_a_a2, net_i2, net_a_c2)
  c2_trtf <- .default(factor(LETTERS[c(2, 4, 3, 1)], levels = LETTERS[c(2, 4, 3, 1)]))
  attr(c2_trtf, "original_levels") <- LETTERS[4:1]
  expect_equal(c2$treatments, c2_trtf)
  expect_equal(levels(c2$agd_arm$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c2$agd_contrast$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c2$ipd$.trt), LETTERS[c(2, 4, 3, 1)])
  c2_studyf <- factor(letters[5:1], levels = letters[5:1])
  attr(c2_studyf, "original_levels") <- letters[5:1]
  expect_equal(c2$studies, c2_studyf)
  expect_equal(levels(c2$agd_arm$.study), letters[5:1])
  expect_equal(levels(c2$agd_contrast$.study), letters[5:1])
  expect_equal(levels(c2$ipd$.study), letters[5:1])

  c1_classed <- combine_network(
    set_agd_arm(agd_arm, studyf2, trtf2, y = y, se = se, trt_class = tclassf2),
    set_ipd(ipd, studyf2, trtf2, y = y, trt_class = tclassf2)
  )
  # Reference trt is A
  expect_equal(c1_classed$treatments, c1_trtf)
  expect_equal(levels(c1_classed$agd_arm$.trt), LETTERS[c(1, 4:2)])
  expect_equal(levels(c1_classed$ipd$.trt), LETTERS[c(1, 4:2)])
  expect_equal(c1_classed$studies, c1_studyf)
  expect_equal(levels(c1_classed$agd_arm$.study), letters[c(5, 4, 2, 1)])
  expect_equal(levels(c1_classed$ipd$.study), letters[c(5, 4, 2, 1)])
  c1_classf <- factor(letters[c(1, 3, 2, 2)], levels = letters[c(1, 3, 2)])
  attr(c1_classf, "original_levels") <- letters[3:1]
  expect_equal(c1_classed$classes, c1_classf)
  expect_equal(levels(c1_classed$agd_arm$.trtclass), letters[c(1, 3, 2)])
  expect_equal(levels(c1_classed$ipd$.trtclass), letters[c(1, 3, 2)])

  c2_classed <- combine_network(
    set_agd_arm(agd_arm, studyf2, trtf2, y = y, se = se, trt_class = tclassf2),
    set_ipd(ipd, studyf2, trtf2, y = y, trt_class = tclassf2),
    set_agd_contrast(agd_contrast, studyf2, trtf2, y = y, se = se, trt_class = tclassf2)
  )
  # Reference treatment is B
  expect_equal(c2_classed$treatments, c2_trtf)
  expect_equal(levels(c2_classed$agd_arm$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c2_classed$agd_contrast$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c2_classed$ipd$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(c2_classed$studies, c2_studyf)
  expect_equal(levels(c2_classed$agd_arm$.study), letters[5:1])
  expect_equal(levels(c2_classed$agd_contrast$.study), letters[5:1])
  expect_equal(levels(c2_classed$ipd$.study), letters[5:1])
  c2_classf <- factor(letters[c(2, 3, 2, 1)], levels = c(letters[c(2, 3, 1)]))
  attr(c2_classf, "original_levels") <- letters[3:1]
  expect_equal(c2_classed$classes, c2_classf)
  expect_equal(levels(c2_classed$agd_arm$.trtclass), letters[c(2, 3, 1)])
  expect_equal(levels(c2_classed$ipd$.trtclass), letters[c(2, 3, 1)])
  expect_equal(levels(c2_classed$agd_contrast$.trtclass), letters[c(2, 3, 1)])

  # Check that unused levels are dropped
  c3_classed <- combine_network(
    set_agd_arm(agd_arm,
                forcats::fct_expand(studyf2, "xxx"),
                forcats::fct_expand(trtf2, "yyy"),
                y = y, se = se,
                trt_class = forcats::fct_expand(tclassf2, "zzz")),
    set_ipd(ipd,
            forcats::fct_expand(studyf2, "xxx"),
            forcats::fct_expand(trtf2, "yyy"),
            y = y,
            trt_class = forcats::fct_expand(tclassf2, "zzz")),
    set_agd_contrast(agd_contrast,
                     forcats::fct_expand(studyf2, "xxx"),
                     forcats::fct_expand(trtf2, "yyy"),
                     y = y, se = se,
                     trt_class = forcats::fct_expand(tclassf2, "zzz")))

  # Reference treatment is B
  c3_trtf <- c2_trtf
  attr(c3_trtf, "original_levels") <- c(attr(c3_trtf, "original_levels"), "yyy")
  expect_equal(c3_classed$treatments, c3_trtf)
  expect_equal(levels(c3_classed$agd_arm$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c3_classed$agd_contrast$.trt), LETTERS[c(2, 4, 3, 1)])
  expect_equal(levels(c3_classed$ipd$.trt), LETTERS[c(2, 4, 3, 1)])
  c3_studyf <- c2_studyf
  attr(c3_studyf, "original_levels") <- c(attr(c3_studyf, "original_levels"), "xxx")
  expect_equal(c3_classed$studies, c3_studyf)
  expect_equal(levels(c3_classed$agd_arm$.study), letters[5:1])
  expect_equal(levels(c3_classed$agd_contrast$.study), letters[5:1])
  expect_equal(levels(c3_classed$ipd$.study), letters[5:1])
  c3_classf <- factor(letters[c(2, 3, 2, 1)], levels = c(letters[c(2, 3, 1)]))
  attr(c3_classf, "original_levels") <- c(letters[3:1], "zzz")
  expect_equal(c3_classed$classes, c3_classf)
  expect_equal(levels(c3_classed$agd_arm$.trtclass), letters[c(2, 3, 1)])
  expect_equal(levels(c3_classed$ipd$.trtclass), letters[c(2, 3, 1)])
  expect_equal(levels(c3_classed$agd_contrast$.trtclass), letters[c(2, 3, 1)])
})

test_that("combine_network can set alternative trt_ref", {
  c1 <- combine_network(net_a_a, net_i, net_a_c, trt_ref = "C")
  expect_equal(c1$treatments, factor(LETTERS[c(3, 1, 2, 4)], levels = LETTERS[c(3, 1, 2, 4)]))
  expect_equal(levels(c1$agd_arm$.trt), LETTERS[c(3, 1, 2, 4)])
  expect_equal(levels(c1$agd_contrast$.trt), LETTERS[c(3, 1, 2, 4)])
  expect_equal(levels(c1$ipd$.trt), LETTERS[c(3, 1, 2, 4)])

  expect_error(combine_network(net_a_a, net_i, net_a_c, trt_ref = 2),
               "does not match a treatment.*Suitable values are: A, B, C, D")
  expect_error(combine_network(net_a_a, net_a_c,
                               set_ipd(mutate(ipd, trtf = factor(LETTERS[3:7])),
                                       studyf, trtf, y = y),
                               trt_ref = 2),
               "does not match a treatment.*Suitable values are: A, B, C, D, E, \\.\\.\\.")

  expect_error(combine_network(net_a_a, net_a_c,
                               set_ipd(mutate(ipd, studyf = factor("a")),
                                       studyf, trtf, y = y)),
               "Studies with same label found in multiple data sources: a")
  expect_error(combine_network(net_a_a, net_a_a),
               "Studies with same label found in multiple data sources: a, b")

  c1_classed <- combine_network(
    set_agd_arm(agd_arm, studyf, trtf, y = y, se = se, trt_class = tclassc),
    set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassc),
    set_agd_contrast(agd_contrast, studyf, trtf, y = y, se = se, trt_class = tclassc),
    trt_ref = "C"
  )
  expect_equal(c1_classed$treatments, factor(LETTERS[c(3, 1, 2, 4)], levels = LETTERS[c(3, 1, 2, 4)]))
  expect_equal(levels(c1_classed$agd_arm$.trt), LETTERS[c(3, 1, 2, 4)])
  expect_equal(levels(c1_classed$agd_contrast$.trt), LETTERS[c(3, 1, 2, 4)])
  expect_equal(levels(c1_classed$ipd$.trt), LETTERS[c(3, 1, 2, 4)])
  expect_equal(c1_classed$classes, factor(letters[c(2, 1, 2, 3)], levels = letters[c(2, 1, 3)]))
  expect_equal(levels(c1_classed$agd_arm$.trtclass), letters[c(2, 1, 3)])
  expect_equal(levels(c1_classed$agd_contrast$.trtclass), letters[c(2, 1, 3)])
  expect_equal(levels(c1_classed$ipd$.trtclass), letters[c(2, 1, 3)])
})

test_that("combine_network error if outcomes do not match for same data source type", {
  m <- "Multiple outcome types present"
  dat_a <- tibble(study = 1, trt = 1:2, r = 1, n = 1, y = 1, se = 1)
  dat_b <- tibble(study = 2, trt = 2:3, r = 1, n = 1, y = 1, se = 1)

  expect_error(combine_network(set_ipd(dat_a, study, trt, r = r),
                               set_ipd(dat_b, study, trt, y = y)), m)
  expect_error(combine_network(set_agd_arm(dat_a, study, trt, r = r, n = n),
                               set_agd_arm(dat_b, study, trt, y = y, se = se)), m)
})

test_that("combine_network error if mismatch outcomes across data types", {
  m <- "Combining.+not supported"
  dat_a <- tibble(study = 1, trt = 1:2, r = 1, n = 1, E = 1, y = 1, se = 1)
  dat_b <- tibble(study = 2, trt = 2:3, r = 1, n = 1, E = 1, y = 1, se = 1)

  expect_error(combine_network(set_ipd(dat_a, study, trt, r = r),
                               set_agd_arm(dat_b, study, trt, y = y, se = se)), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(n, r, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, y = y, se = se)), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(n, r, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = r, n = n)), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = r, E = E),
                               set_agd_arm(dat_b, study, trt, y = y, se = se)), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, y = y),
                               set_agd_arm(dat_b, study, trt, r = r, n = n)), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, y = y),
                               set_agd_arm(dat_b, study, trt, r = multi(n, r, inclusive = TRUE))), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = r),
                               set_agd_arm(dat_b, study, trt, r = multi(n, r, inclusive = TRUE))), m)
  expect_error(combine_network(set_ipd(dat_a, study, trt, y = y),
                               set_agd_arm(dat_b, study, trt, r = r, E = E)), m)
})

test_that("combine_network error if multinomial outcomes mismatched", {
  dat_a <- tibble(study = 1, trt = 1:2, a = 1, b = 1, c = 0)
  dat_b <- tibble(study = 2, trt = 2:3, a = 1, b = 1, c = 0)

  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(a, b, inclusive = TRUE))),
               "different numbers of categories")
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(b, a, c, inclusive = TRUE))),
               "different category labels")
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(A = a, B = b, C = c, inclusive = TRUE))),
               "different category labels")

  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_ipd(dat_b, study, trt, r = multi(a, b, inclusive = TRUE))),
               "different numbers of categories")
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_ipd(dat_b, study, trt, r = multi(b, a, c, inclusive = TRUE))),
               "different category labels")
  expect_error(combine_network(set_ipd(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_ipd(dat_b, study, trt, r = multi(A = a, B = b, C = c, inclusive = TRUE))),
               "different category labels")

  expect_error(combine_network(set_agd_arm(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(a, b, inclusive = TRUE))),
               "different numbers of categories")
  expect_error(combine_network(set_agd_arm(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(b, a, c, inclusive = TRUE))),
               "different category labels")
  expect_error(combine_network(set_agd_arm(dat_a, study, trt, r = multi(a, b, c, inclusive = TRUE)),
                               set_agd_arm(dat_b, study, trt, r = multi(A = a, B = b, C = c, inclusive = TRUE))),
               "different category labels")

  # Check combining ordered and competing outcomes is disallowed, when competing are implemented
})

test_that("combine_network error if treatment classes do not match across sources", {
  m <- "Treatment present in more than one class"

  expect_error(combine_network(
    set_agd_arm(agd_arm, studyf, trtf, y = y, se = se, trt_class = tclassn),
    set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassn_bad)), m)

  expect_error(combine_network(
    set_agd_arm(agd_arm, studyf, trtf, y = y, se = se, trt_class = tclassn),
    set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassn_bad),
    set_agd_contrast(agd_contrast, studyf, trtf, y = y, se = se, trt_class = tclassn_bad)), m)
})

test_that("combine_network warns if not all sources have treatment classes", {
  expect_warning(combine_network(net_a_a,
                                 set_ipd(ipd, studyf, trtf, y = y, trt_class = tclassc)),
                 "Not all data sources have defined treatment classes")
})

test_that("combine_network works combining survival outcomes from same type of ipd/agd source", {
  expect_equivalent(combine_network(set_agd_surv(filter(ndmm_agd, study == "Morgan2012"), studyf, trtf,
                                                 Surv = Surv(eventtime, status),
                                                 covariates = ndmm_agd_covs),
                                    set_agd_surv(filter(ndmm_agd, study != "Morgan2012"), studyf, trtf,
                                                 Surv = Surv(eventtime, status),
                                                 covariates = ndmm_agd_covs))$agd_arm,
                    set_agd_surv(ndmm_agd, studyf, trtf,
                                 Surv = Surv(eventtime, status),
                                 covariates = ndmm_agd_covs)$agd_arm)

  expect_equivalent(combine_network(set_ipd(filter(ndmm_agd, study == "Morgan2012"), studyf, trtf,
                                                 Surv = Surv(eventtime, status)),
                                    set_ipd(filter(ndmm_agd, study != "Morgan2012"), studyf, trtf,
                                                 Surv = Surv(eventtime, status)))$ipd,
                    set_ipd(ndmm_agd, studyf, trtf,
                                 Surv = Surv(eventtime, status))$ipd)
})

test_that("combine_network respects mlnma_data objects with integration points", {

  ipd_net <- set_ipd(plaque_psoriasis_ipd, study = studyc, trt = trtc, r = pasi75)

  agd_net1 <- set_agd_arm(
    filter(plaque_psoriasis_agd, studyc %in% c("FEATURE", "FIXTURE", "JUNCTURE")),
    study = studyc, trt = trtc, r = pasi75_r, n = pasi75_n
  ) %>%
    add_integration(durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                    weight = distr(qgamma, mean = weight_mean, sd = weight_sd), cor = diag(2))

  agd_net2 <- set_agd_arm(
    filter(plaque_psoriasis_agd, !studyc %in% c("FEATURE", "FIXTURE", "JUNCTURE")),
    study = studyc, trt = trtc, r = pasi75_r, n = pasi75_n
  )

  agd_net3 <- set_agd_arm(
    filter(plaque_psoriasis_agd, !studyc %in% c("FEATURE", "FIXTURE", "JUNCTURE")),
    study = studyc, trt = trtc, r = pasi75_r, n = pasi75_n
  ) %>%
    add_integration(durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                    weight = distr(qgamma, mean = weight_mean, sd = weight_sd), cor = diag(2))

  expect_s3_class(combine_network(agd_net1),
                  c("mlnmr_data", "nma_data"), exact = TRUE)

  expect_s3_class(combine_network(ipd_net, agd_net1),
                  c("mlnmr_data", "nma_data"), exact = TRUE)

  expect_s3_class(combine_network(ipd_net, agd_net1, agd_net3),
                  c("mlnmr_data", "nma_data"), exact = TRUE)

  expect_s3_class(combine_network(agd_net1, agd_net3),
                  c("mlnmr_data", "nma_data"), exact = TRUE)

  expect_s3_class(
    expect_message(combine_network(agd_net1, agd_net2),
                   "Dropping integration points"),
      "nma_data", exact = TRUE)

  expect_s3_class(
    expect_message(combine_network(ipd_net, agd_net1, agd_net2),
                   "Dropping integration points"),
    "nma_data", exact = TRUE)

  expect_s3_class(combine_network(ipd_net, agd_net2), "nma_data")

  agd_net4 <- set_agd_arm(
    filter(plaque_psoriasis_agd, !studyc %in% c("FEATURE", "FIXTURE", "JUNCTURE")),
    study = studyc, trt = trtc, r = pasi75_r, n = pasi75_n
  ) %>%
    add_integration(durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd))

  expect_error(combine_network(agd_net1, agd_net4), "different sets of covariates")
  expect_error(combine_network(ipd_net, agd_net1, agd_net4), "different sets of covariates")

  agd_net5 <- set_agd_arm(
    filter(plaque_psoriasis_agd, !studyc %in% c("FEATURE", "FIXTURE", "JUNCTURE")),
    study = studyc, trt = trtc, r = pasi75_r, n = pasi75_n
  ) %>%
    add_integration(durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                    weight = distr(qgamma, mean = weight_mean, sd = weight_sd), cor = diag(2),
                    n_int = 8)

  expect_error(combine_network(agd_net1, agd_net5), "different numbers of integration points")
  expect_error(combine_network(ipd_net, agd_net1, agd_net5), "different numbers of integration points")
})

# Generated by vignette example_parkinsons.Rmd: do not edit by hand
# Instead edit example_parkinsons.Rmd and then run precompile.R

skip_on_cran()



params <-
list(run_tests = FALSE)

## ----code=readLines("children/knitr_setup.R"), include=FALSE--------------------------------------

## ----include=FALSE--------------------------------------------------------------------------------
set.seed(1042020)


## ----eval = FALSE---------------------------------------------------------------------------------
## library(multinma)
## options(mc.cores = parallel::detectCores())

## ----setup, echo = FALSE--------------------------------------------------------------------------
library(multinma)
nc <- switch(tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_")), 
             "true" =, "warn" = 2, 
             parallel::detectCores())
options(mc.cores = nc)


## -------------------------------------------------------------------------------------------------
head(parkinsons)


## -------------------------------------------------------------------------------------------------
arm_net <- set_agd_arm(parkinsons, 
                      study = studyn,
                      trt = trtn,
                      y = y, 
                      se = se,
                      sample_size = n)
arm_net


## ----parkinsons_arm_network_plot------------------------------------------------------------------
plot(arm_net, weight_edges = TRUE, weight_nodes = TRUE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## arm_fit_FE <- nma(arm_net,
##                   trt_effects = "fixed",
##                   prior_intercept = normal(scale = 100),
##                   prior_trt = normal(scale = 10))

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
arm_fit_FE <- nma(arm_net, 
                  trt_effects = "fixed",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 10),
                  iter = 10000)


## -------------------------------------------------------------------------------------------------
arm_fit_FE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(arm_fit_FE, pars = c("d", "mu"))


## ----arm_FE_pp_plot-------------------------------------------------------------------------------
plot_prior_posterior(arm_fit_FE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## arm_fit_RE <- nma(arm_net,
##                   seed = 379394727,
##                   trt_effects = "random",
##                   prior_intercept = normal(scale = 100),
##                   prior_trt = normal(scale = 100),
##                   prior_het = half_normal(scale = 5),
##                   adapt_delta = 0.99)

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
arm_fit_RE <- nowarn_on_ci(nma(arm_net, 
                  seed = 379394727,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = half_normal(scale = 5),
                  adapt_delta = 0.99,
                  iter = 10000))


## ----plot_arm_RE_pairs, fig.width = 6-------------------------------------------------------------
pairs(arm_fit_RE, pars = c("mu[4]", "d[3]", "delta[4: 3]", "tau"))


## -------------------------------------------------------------------------------------------------
arm_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(arm_fit_RE, pars = c("d", "mu", "delta"))


## ----arm_RE_pp_plot-------------------------------------------------------------------------------
plot_prior_posterior(arm_fit_RE)


## -------------------------------------------------------------------------------------------------
(arm_dic_FE <- dic(arm_fit_FE))

## -------------------------------------------------------------------------------------------------
(arm_dic_RE <- dic(arm_fit_RE))


## ----arm_FE_resdev_plot---------------------------------------------------------------------------
plot(arm_dic_FE)


## ----arm_RE_resdev_plot---------------------------------------------------------------------------
plot(arm_dic_RE)


## ----arm_releff_FE, fig.height=3------------------------------------------------------------------
(arm_releff_FE <- relative_effects(arm_fit_FE, trt_ref = 1))
plot(arm_releff_FE, ref_line = 0)

## ----arm_releff_RE, fig.height=3------------------------------------------------------------------
(arm_releff_RE <- relative_effects(arm_fit_RE, trt_ref = 1))
plot(arm_releff_RE, ref_line = 0)


## ----arm_pred_FE, fig.height = 2------------------------------------------------------------------
arm_pred_FE <- predict(arm_fit_FE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
arm_pred_FE
plot(arm_pred_FE)

## ----arm_pred_RE, fig.height = 2------------------------------------------------------------------
arm_pred_RE <- predict(arm_fit_RE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
arm_pred_RE
plot(arm_pred_RE)


## ----arm_pred_RE_all, fig.height=8----------------------------------------------------------------
arm_pred_FE_studies <- predict(arm_fit_FE, type = "response")
arm_pred_FE_studies
plot(arm_pred_FE_studies)


## ----parkinsons_arm_ranks, fig.height=2-----------------------------------------------------------
(arm_ranks <- posterior_ranks(arm_fit_FE))
plot(arm_ranks)

## ----parkinson_arm_rankprobs----------------------------------------------------------------------
(arm_rankprobs <- posterior_rank_probs(arm_fit_FE))
plot(arm_rankprobs)

## ----parkinsons_cumrankprobs----------------------------------------------------------------------
(arm_cumrankprobs <- posterior_rank_probs(arm_fit_FE, cumulative = TRUE))
plot(arm_cumrankprobs)


## -------------------------------------------------------------------------------------------------
contr_net <- set_agd_contrast(parkinsons, 
                              study = studyn,
                              trt = trtn,
                              y = diff, 
                              se = se_diff,
                              sample_size = n)
contr_net


## ----parkinsons_contr_network_plot----------------------------------------------------------------
plot(contr_net, weight_edges = TRUE, weight_nodes = TRUE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## contr_fit_FE <- nma(contr_net,
##                     trt_effects = "fixed",
##                     prior_trt = normal(scale = 100))

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
contr_fit_FE <- nma(contr_net, 
                    trt_effects = "fixed",
                    prior_trt = normal(scale = 100),
                    iter = 10000)


## -------------------------------------------------------------------------------------------------
contr_fit_FE


## ----contr_FE_pp_plot-----------------------------------------------------------------------------
plot_prior_posterior(contr_fit_FE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## contr_fit_RE <- nma(contr_net,
##                     seed = 1150676438,
##                     trt_effects = "random",
##                     prior_trt = normal(scale = 100),
##                     prior_het = half_normal(scale = 5),
##                     adapt_delta = 0.99)

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
contr_fit_RE <- nowarn_on_ci(nma(contr_net, 
                    seed = 1150676438,
                    trt_effects = "random",
                    prior_trt = normal(scale = 100),
                    prior_het = half_normal(scale = 5),
                    adapt_delta = 0.99,
                    iter = 10000))


## ----plot_contr_RE_pairs, fig.width = 6-----------------------------------------------------------
pairs(contr_fit_RE, pars = c("d[3]", "delta[4: 4 vs. 3]", "tau"))


## -------------------------------------------------------------------------------------------------
contr_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(contr_fit_RE, pars = c("d", "delta"))


## ----contr_RE_pp_plot-----------------------------------------------------------------------------
plot_prior_posterior(contr_fit_RE)


## -------------------------------------------------------------------------------------------------
(contr_dic_FE <- dic(contr_fit_FE))

## -------------------------------------------------------------------------------------------------
(contr_dic_RE <- dic(contr_fit_RE))


## ----contr_FE_resdev_plot-------------------------------------------------------------------------
plot(contr_dic_FE)


## ----contr_RE_resdev_plot-------------------------------------------------------------------------
plot(contr_dic_RE)


## ----contr_releff_FE, fig.height=3----------------------------------------------------------------
(contr_releff_FE <- relative_effects(contr_fit_FE, trt_ref = 1))
plot(contr_releff_FE, ref_line = 0)

## ----contr_releff_RE, fig.height=3----------------------------------------------------------------
(contr_releff_RE <- relative_effects(contr_fit_RE, trt_ref = 1))
plot(contr_releff_RE, ref_line = 0)


## ----contr_pred_FE, fig.height = 2----------------------------------------------------------------
contr_pred_FE <- predict(contr_fit_FE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
contr_pred_FE
plot(contr_pred_FE)

## ----contr_pred_RE, fig.height = 2----------------------------------------------------------------
contr_pred_RE <- predict(contr_fit_RE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
contr_pred_RE
plot(contr_pred_RE)


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## predict(contr_fit_FE, type = "response")


## ----parkinsons_contr_ranks, fig.height=2---------------------------------------------------------
(contr_ranks <- posterior_ranks(contr_fit_FE))
plot(contr_ranks)

## ----parkinsons_contr_rankprobs-------------------------------------------------------------------
(contr_rankprobs <- posterior_rank_probs(contr_fit_FE))
plot(contr_rankprobs)

## ----parkinsons_contr_cumrankprobs----------------------------------------------------------------
(contr_cumrankprobs <- posterior_rank_probs(contr_fit_FE, cumulative = TRUE))
plot(contr_cumrankprobs)


## -------------------------------------------------------------------------------------------------
studies <- parkinsons$studyn
(parkinsons_arm <- parkinsons[studies %in% 1:3, ])
(parkinsons_contr <- parkinsons[studies %in% 4:7, ])


## -------------------------------------------------------------------------------------------------
mix_arm_net <- set_agd_arm(parkinsons_arm, 
                           study = studyn,
                           trt = trtn,
                           y = y, 
                           se = se,
                           sample_size = n)

mix_contr_net <- set_agd_contrast(parkinsons_contr, 
                                  study = studyn,
                                  trt = trtn,
                                  y = diff, 
                                  se = se_diff,
                                  sample_size = n)

mix_net <- combine_network(mix_arm_net, mix_contr_net)
mix_net


## ----parkinsons_mix_network_plot------------------------------------------------------------------
plot(mix_net, weight_edges = TRUE, weight_nodes = TRUE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## mix_fit_FE <- nma(mix_net,
##                   trt_effects = "fixed",
##                   prior_intercept = normal(scale = 100),
##                   prior_trt = normal(scale = 100))

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
mix_fit_FE <- nma(mix_net, 
                  trt_effects = "fixed",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  iter = 10000)


## -------------------------------------------------------------------------------------------------
mix_fit_FE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(mix_fit_FE, pars = c("d", "mu"))


## ----mix_FE_pp_plot-------------------------------------------------------------------------------
plot_prior_posterior(mix_fit_FE)


## -------------------------------------------------------------------------------------------------
summary(normal(scale = 100))
summary(half_normal(scale = 5))


## ----eval=!params$run_tests-----------------------------------------------------------------------
## mix_fit_RE <- nma(mix_net,
##                   seed = 437219664,
##                   trt_effects = "random",
##                   prior_intercept = normal(scale = 100),
##                   prior_trt = normal(scale = 100),
##                   prior_het = half_normal(scale = 5),
##                   adapt_delta = 0.99)

## ----eval=params$run_tests, echo=FALSE------------------------------------------------------------
mix_fit_RE <- nowarn_on_ci(nma(mix_net, 
                  seed = 437219664,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = half_normal(scale = 5),
                  adapt_delta = 0.99,
                  iter=10000))


## ----plot_mix_RE_pairs, fig.width = 6-------------------------------------------------------------
pairs(mix_fit_RE, pars = c("d[3]", "delta[4: 4 vs. 3]", "tau"))


## -------------------------------------------------------------------------------------------------
mix_fit_RE


## ----eval=FALSE-----------------------------------------------------------------------------------
## # Not run
## print(mix_fit_RE, pars = c("d", "mu", "delta"))


## ----mix_RE_pp_plot-------------------------------------------------------------------------------
plot_prior_posterior(mix_fit_RE)


## -------------------------------------------------------------------------------------------------
(mix_dic_FE <- dic(mix_fit_FE))

## -------------------------------------------------------------------------------------------------
(mix_dic_RE <- dic(mix_fit_RE))


## ----mix_FE_resdev_plot---------------------------------------------------------------------------
plot(mix_dic_FE)


## ----mix_RE_resdev_plot---------------------------------------------------------------------------
plot(mix_dic_RE)


## ----mix_releff_FE, fig.height=3------------------------------------------------------------------
(mix_releff_FE <- relative_effects(mix_fit_FE, trt_ref = 1))
plot(mix_releff_FE, ref_line = 0)

## ----mix_releff_RE, fig.height=3------------------------------------------------------------------
(mix_releff_RE <- relative_effects(mix_fit_RE, trt_ref = 1))
plot(mix_releff_RE, ref_line = 0)


## ----mix_pred_FE, fig.height = 2------------------------------------------------------------------
mix_pred_FE <- predict(mix_fit_FE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
mix_pred_FE
plot(mix_pred_FE)

## ----mix_pred_RE, fig.height = 2------------------------------------------------------------------
mix_pred_RE <- predict(mix_fit_RE, 
                       baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                       type = "response",
                       baseline_trt = 1)
mix_pred_RE
plot(mix_pred_RE)


## ----mix_pred_FE_all, fig.height=8----------------------------------------------------------------
mix_pred_FE_studies <- predict(mix_fit_FE, type = "response")
mix_pred_FE_studies
plot(mix_pred_FE_studies)


## ----parkinsons_mix_ranks, fig.height=2-----------------------------------------------------------
(mix_ranks <- posterior_ranks(mix_fit_FE))
plot(mix_ranks)

## ----parkinsons_mix_rankprobs---------------------------------------------------------------------
(mix_rankprobs <- posterior_rank_probs(mix_fit_FE))
plot(mix_rankprobs)

## ----parkinsons_mix_cumrankprobs------------------------------------------------------------------
(mix_cumrankprobs <- posterior_rank_probs(mix_fit_FE, cumulative = TRUE))
plot(mix_cumrankprobs)


## ----parkinsons_tests, include=FALSE, eval=params$run_tests---------------------------------------
#--- Test against TSD 2 results ---
library(testthat)
library(dplyr)

tol <- 0.05
tol_dic <- 0.1


#--- Arm-level data ---

# FE Relative effects
tsd_FE <- tribble(
~trt, ~mean, ~sd , ~median, ~lower, ~upper,
2   , -1.81, 0.33, -1.81  , -2.46 ,-1.16  ,
3   , -0.47, 0.49, -0.47  , -1.43 ,0.49   ,
4   , -0.52, 0.48, -0.52  , -1.46 ,0.43   ,
5   , -0.82, 0.52, -0.82  , -1.84 ,0.22   ,

) %>% 
  mutate(trt = ordered(trt, levels = levels(arm_net$treatments))) %>%
  arrange(trt)

arm_releff_FE <- as.data.frame(arm_releff_FE)

test_that("FE relative effects", {
  expect_equivalent(arm_releff_FE$mean, tsd_FE$mean, tolerance = tol)
  expect_equivalent(arm_releff_FE$sd, tsd_FE$sd, tolerance = tol)
  expect_equivalent(arm_releff_FE$`50%`, tsd_FE$median, tolerance = tol)
  expect_equivalent(arm_releff_FE$`2.5%`, tsd_FE$lower, tolerance = tol)
  expect_equivalent(arm_releff_FE$`97.5%`, tsd_FE$upper, tolerance = tol)
})

# RE Relative effects
tsd_RE <- tribble(
~trt, ~mean, ~sd , ~median, ~lower, ~upper,
2   , -1.85, 0.54, -1.84  , -2.91 ,-0.85  ,
3   , -0.50, 0.66, -0.50  , -1.78 ,0.75   ,
4   , -0.53, 0.65, -0.53  , -1.77 ,0.71   ,
5   , -0.83, 0.80, -0.83  , -2.35 ,0.69   ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(arm_net$treatments))) %>%
  arrange(trt)

arm_releff_RE <- as.data.frame(arm_releff_RE)

test_that("RE relative effects", {
  expect_equivalent(arm_releff_RE$mean, tsd_RE$mean, tolerance = tol)
  expect_equivalent(arm_releff_RE$`50%`, tsd_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(arm_releff_RE$sd, tsd_RE$sd, tolerance = tol)
  expect_equivalent(arm_releff_RE$`2.5%`, tsd_RE$lower, tolerance = tol)
  expect_equivalent(arm_releff_RE$`97.5%`, tsd_RE$upper, tolerance = tol)
})

# Heterogeneity SD
arm_tau <- as.data.frame(summary(arm_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(arm_tau$mean, 0.4, tolerance = tol)
  expect_equivalent(arm_tau$`50%`, 0.28, tolerance = tol)
  skip_on_ci()
  expect_equivalent(arm_tau$sd, 0.43, tolerance = tol)
  expect_equivalent(arm_tau$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(arm_tau$`97.5%`, 1.55, tolerance = tol)
})

# FE probabilities
tsd_pred_FE <- tribble(
~trt, ~mean, ~sd , ~median, ~lower, ~upper,
1   , -0.73, 0.22, -0.73  , -1.16 ,-0.30  ,
2   , -2.54, 0.40, -2.54  , -3.32 ,-1.76  ,
3   , -1.21, 0.53, -1.20  , -2.25 ,-0.15  ,
4   , -1.25, 0.53, -1.25  , -2.28 ,-0.21  ,
5   , -1.55, 0.57, -1.55  , -2.66 ,-0.43  ,
) %>% 
  mutate(trt = ordered(trt, levels = levels(arm_net$treatments))) %>%
  arrange(trt)

arm_pred_FE <- as.data.frame(arm_pred_FE)

test_that("FE predicted probabilities at 3 years", {
  expect_equivalent(arm_pred_FE$mean, tsd_pred_FE$mean, tolerance = tol)
  expect_equivalent(arm_pred_FE$sd, tsd_pred_FE$sd, tolerance = tol)
  expect_equivalent(arm_pred_FE$`50%`, tsd_pred_FE$median, tolerance = tol)
  expect_equivalent(arm_pred_FE$`2.5%`, tsd_pred_FE$lower, tolerance = tol)
  expect_equivalent(arm_pred_FE$`97.5%`, tsd_pred_FE$upper, tolerance = tol)
})

# RE probabilities
tsd_pred_RE <- tribble(
~trt, ~mean, ~sd , ~median, ~lower, ~upper,
1   , -0.73, 0.22, -0.73  , -1.16 ,-0.30  ,
2   , -2.58, 0.58, -2.57  , -3.72 ,-1.50  ,
3   , -1.23, 0.70, -1.23  , -2.57 ,0.10   ,
4   , -1.26, 0.69, -1.26  , -2.57 ,0.05   ,
5   , -1.57, 0.83, -1.56  , -3.14 ,0.02   , 
) %>% 
  mutate(trt = ordered(trt, levels = levels(arm_net$treatments))) %>%
  arrange(trt)

arm_pred_RE <- as.data.frame(arm_pred_RE)

test_that("RE predicted probabilities at 3 years", {
  expect_equivalent(arm_pred_RE$mean, tsd_pred_RE$mean, tolerance = tol)
  expect_equivalent(arm_pred_RE$`50%`, tsd_pred_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(arm_pred_RE$sd, tsd_pred_RE$sd, tolerance = tol)
  expect_equivalent(arm_pred_RE$`2.5%`, tsd_pred_RE$lower, tolerance = tol)
  expect_equivalent(arm_pred_RE$`97.5%`, tsd_pred_RE$upper, tolerance = tol)
})

# FE DIC
test_that("FE DIC", {
  expect_equivalent(arm_dic_FE$resdev, 13.3, tolerance = tol_dic)
  expect_equivalent(arm_dic_FE$pd, 11.0, tolerance = tol_dic)
  expect_equivalent(arm_dic_FE$dic, 24.3, tolerance = tol_dic)
})

# RE DIC
test_that("RE DIC", {
  expect_equivalent(arm_dic_RE$resdev, 13.6, tolerance = tol_dic)
  expect_equivalent(arm_dic_RE$pd, 12.4, tolerance = tol_dic)
  expect_equivalent(arm_dic_RE$dic, 26.0, tolerance = tol_dic)
})


#--- Contrast-level data ---

# Relative effects, heterogeneity, absolute effects are all equal across data scenarios

contr_releff_FE <- as.data.frame(contr_releff_FE)

test_that("FE relative effects", {
  expect_equivalent(contr_releff_FE$mean, tsd_FE$mean, tolerance = tol)
  expect_equivalent(contr_releff_FE$sd, tsd_FE$sd, tolerance = tol)
  expect_equivalent(contr_releff_FE$`50%`, tsd_FE$median, tolerance = tol)
  expect_equivalent(contr_releff_FE$`2.5%`, tsd_FE$lower, tolerance = tol)
  expect_equivalent(contr_releff_FE$`97.5%`, tsd_FE$upper, tolerance = tol)
})

contr_releff_RE <- as.data.frame(contr_releff_RE)

test_that("RE relative effects", {
  expect_equivalent(contr_releff_RE$mean, tsd_RE$mean, tolerance = tol)
  expect_equivalent(contr_releff_RE$`50%`, tsd_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(contr_releff_RE$sd, tsd_RE$sd, tolerance = tol)
  expect_equivalent(contr_releff_RE$`2.5%`, tsd_RE$lower, tolerance = tol)
  expect_equivalent(contr_releff_RE$`97.5%`, tsd_RE$upper, tolerance = tol)
})

# Heterogeneity SD
contr_tau <- as.data.frame(summary(contr_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(contr_tau$mean, 0.4, tolerance = tol)
  expect_equivalent(contr_tau$`50%`, 0.28, tolerance = tol)
  skip_on_ci()
  expect_equivalent(contr_tau$sd, 0.43, tolerance = tol)
  expect_equivalent(contr_tau$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(contr_tau$`97.5%`, 1.55, tolerance = tol)
})

contr_pred_FE <- as.data.frame(contr_pred_FE)

test_that("FE predicted probabilities at 3 years", {
  expect_equivalent(contr_pred_FE$mean, tsd_pred_FE$mean, tolerance = tol)
  expect_equivalent(contr_pred_FE$sd, tsd_pred_FE$sd, tolerance = tol)
  expect_equivalent(contr_pred_FE$`50%`, tsd_pred_FE$median, tolerance = tol)
  expect_equivalent(contr_pred_FE$`2.5%`, tsd_pred_FE$lower, tolerance = tol)
  expect_equivalent(contr_pred_FE$`97.5%`, tsd_pred_FE$upper, tolerance = tol)
})

contr_pred_RE <- as.data.frame(contr_pred_RE)

test_that("RE predicted probabilities at 3 years", {
  expect_equivalent(contr_pred_RE$mean, tsd_pred_RE$mean, tolerance = tol)
  expect_equivalent(contr_pred_RE$`50%`, tsd_pred_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(contr_pred_RE$sd, tsd_pred_RE$sd, tolerance = tol)
  expect_equivalent(contr_pred_RE$`2.5%`, tsd_pred_RE$lower, tolerance = tol)
  expect_equivalent(contr_pred_RE$`97.5%`, tsd_pred_RE$upper, tolerance = tol)
})

test_that("Error message when using predict() on only contrast data", {
  expect_error(predict(contr_fit_FE),
               "No arm-based data \\(IPD or AgD\\) in network")
})

# FE DIC
test_that("FE DIC", {
  expect_equivalent(contr_dic_FE$resdev, 6.3, tolerance = tol_dic)
  expect_equivalent(contr_dic_FE$pd, 4.0, tolerance = tol_dic)
  expect_equivalent(contr_dic_FE$dic, 10.3, tolerance = tol_dic)
})

# RE DIC
test_that("RE DIC", {
  expect_equivalent(contr_dic_RE$resdev, 6.6, tolerance = tol_dic)
  expect_equivalent(contr_dic_RE$pd, 5.5, tolerance = tol_dic)
  expect_equivalent(contr_dic_RE$dic, 12.1, tolerance = tol_dic)
})


# Test contrast-data rows in incorrect order
park_reorder <- sample_frac(parkinsons, size = 1, replace = FALSE)

reorder_net <- set_agd_contrast(park_reorder, 
                              study = studyn,
                              trt = trtn,
                              y = diff, 
                              se = se_diff,
                              sample_size = n)

reorder_fit_FE <- nma(reorder_net, 
                      seed = 1878819627,
                      trt_effects = "fixed",
                      prior_trt = normal(scale = 100),
                      iter = 10000)

reorder_fit_RE <- nowarn_on_ci(nma(reorder_net, 
                      seed = 1147315,
                      trt_effects = "random",
                      prior_trt = normal(scale = 100),
                      prior_het = half_normal(scale = 5),
                      adapt_delta = 0.99,
                      iter = 10000))

expect_equivalent_nma_summary <- function(object, expected, ...) {
  
  act <- quasi_label(enquo(object), arg = "object")
  exp <- quasi_label(enquo(expected), arg = "expected")
  
  comp <- compare(act$val %>% as.data.frame() %>% select(-c(Bulk_ESS, Tail_ESS, Rhat)), 
                  exp$val %>% as.data.frame() %>% select(-c(Bulk_ESS, Tail_ESS, Rhat)), 
                  ...)
  expect(comp$equal, sprintf("%s not equal to %s.\n%s", 
         act$lab, exp$lab, comp$message))
  
  invisible(act$val)
}

test_that("Analysis of reordered contrast data gives same results", {
  expect_equivalent_nma_summary(relative_effects(reorder_fit_FE, trt_ref = 1),
                           contr_releff_FE, 
                           tolerance = tol)
  
  expect_equivalent_nma_summary(predict(reorder_fit_FE, 
                                     baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                                     type = "response",
                                     baseline_trt = 1),
                           contr_pred_FE,
                           tolerance = tol)
  
  expect_equivalent(dic(reorder_fit_FE)$resdev, contr_dic_FE$resdev, tolerance = tol_dic)
  expect_equivalent(dic(reorder_fit_FE)$pd, contr_dic_FE$pd, tolerance = tol_dic)
  expect_equivalent(dic(reorder_fit_FE)$dic, contr_dic_FE$dic, tolerance = tol_dic)
  
  expect_equivalent(dic(reorder_fit_RE)$resdev, contr_dic_RE$resdev, tolerance = tol_dic)
  expect_equivalent(dic(reorder_fit_RE)$pd, contr_dic_RE$pd, tolerance = tol_dic)
  expect_equivalent(dic(reorder_fit_RE)$dic, contr_dic_RE$dic, tolerance = tol_dic)
  
  skip_on_ci()
  expect_equivalent_nma_summary(relative_effects(reorder_fit_RE, trt_ref = 1),
                           contr_releff_RE, 
                           tolerance = tol)
  
  expect_equivalent_nma_summary(predict(reorder_fit_RE, 
                                     baseline = distr(qnorm, mean = -0.73, sd = 21^-0.5),
                                     type = "response",
                                     baseline_trt = 1),
                           contr_pred_RE,
                           tolerance = tol)
  
  expect_equivalent_nma_summary(summary(reorder_fit_RE, pars = "tau"), 
                           contr_tau,
                           tolerance = tol)
})


#--- Mixed data ---

# Relative effects, heterogeneity, absolute effects are all equal across data scenarios

mix_releff_FE <- as.data.frame(mix_releff_FE)

test_that("FE relative effects", {
  expect_equivalent(mix_releff_FE$mean, tsd_FE$mean, tolerance = tol)
  expect_equivalent(mix_releff_FE$sd, tsd_FE$sd, tolerance = tol)
  expect_equivalent(mix_releff_FE$`50%`, tsd_FE$median, tolerance = tol)
  expect_equivalent(mix_releff_FE$`2.5%`, tsd_FE$lower, tolerance = tol)
  expect_equivalent(mix_releff_FE$`97.5%`, tsd_FE$upper, tolerance = tol)
})

mix_releff_RE <- as.data.frame(mix_releff_RE)

test_that("RE relative effects", {
  expect_equivalent(mix_releff_RE$mean, tsd_RE$mean, tolerance = tol)
  expect_equivalent(mix_releff_RE$`50%`, tsd_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(mix_releff_RE$sd, tsd_RE$sd, tolerance = tol)
  expect_equivalent(mix_releff_RE$`2.5%`, tsd_RE$lower, tolerance = tol)
  expect_equivalent(mix_releff_RE$`97.5%`, tsd_RE$upper, tolerance = tol)
})

# Heterogeneity SD
mix_tau <- as.data.frame(summary(mix_fit_RE, pars = "tau"))

test_that("RE heterogeneity SD", {
  expect_equivalent(mix_tau$mean, 0.4, tolerance = tol)
  expect_equivalent(mix_tau$`50%`, 0.28, tolerance = tol)
  skip_on_ci()
  expect_equivalent(mix_tau$sd, 0.43, tolerance = tol)
  expect_equivalent(mix_tau$`2.5%`, 0.01, tolerance = tol)
  expect_equivalent(mix_tau$`97.5%`, 1.55, tolerance = tol)
})

mix_pred_FE <- as.data.frame(mix_pred_FE)

test_that("FE predicted probabilities at 3 years", {
  expect_equivalent(mix_pred_FE$mean, tsd_pred_FE$mean, tolerance = tol)
  expect_equivalent(mix_pred_FE$sd, tsd_pred_FE$sd, tolerance = tol)
  expect_equivalent(mix_pred_FE$`50%`, tsd_pred_FE$median, tolerance = tol)
  expect_equivalent(mix_pred_FE$`2.5%`, tsd_pred_FE$lower, tolerance = tol)
  expect_equivalent(mix_pred_FE$`97.5%`, tsd_pred_FE$upper, tolerance = tol)
})

mix_pred_RE <- as.data.frame(mix_pred_RE)

test_that("RE predicted probabilities at 3 years", {
  expect_equivalent(mix_pred_RE$mean, tsd_pred_RE$mean, tolerance = tol)
  expect_equivalent(mix_pred_RE$`50%`, tsd_pred_RE$median, tolerance = tol)
  skip_on_ci()
  expect_equivalent(mix_pred_RE$sd, tsd_pred_RE$sd, tolerance = tol)
  expect_equivalent(mix_pred_RE$`2.5%`, tsd_pred_RE$lower, tolerance = tol)
  expect_equivalent(mix_pred_RE$`97.5%`, tsd_pred_RE$upper, tolerance = tol)
})

# FE DIC
test_that("FE DIC", {
  expect_equivalent(mix_dic_FE$resdev, 9.3, tolerance = tol_dic)
  expect_equivalent(mix_dic_FE$pd, 7.0, tolerance = tol_dic)
  expect_equivalent(mix_dic_FE$dic, 16.3, tolerance = tol_dic)
})

# RE DIC
test_that("RE DIC", {
  expect_equivalent(mix_dic_RE$resdev, 9.6, tolerance = tol_dic)
  expect_equivalent(mix_dic_RE$pd, 8.5, tolerance = tol_dic)
  expect_equivalent(mix_dic_RE$dic, 18.1, tolerance = tol_dic)
})

# --- Check UME models ---
arm_fit_FE_UME <- nma(arm_net, 
                      trt_effects = "fixed",
                      consistency = "ume",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 10),
                      iter = 10000)

# NOTE: RE UME models are not really tractable - lots of divergences - not
# enough data to estimate tau

mix_fit_FE_UME <- nma(mix_net, 
                      trt_effects = "fixed",
                      consistency = "ume",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      iter = 10000)

test_that("Arm/Mixed UME results are the same", {
  expect_equivalent_nma_summary(summary(arm_fit_FE_UME, pars = "d"), 
                                summary(mix_fit_FE_UME, pars = "d"), 
                                tolerance = tol)
})

# NOTE: Contrast (and reordered contrast) results are different, since the
# choice of independent contrasts is based on the "observed" baseline arms in
# the data rather than the treatment ordering

contr_fit_FE_UME <- nma(contr_net, 
                        trt_effects = "fixed",
                        consistency = "ume",
                        prior_trt = normal(scale = 100),
                        iter = 10000)

reorder_fit_FE_UME <- nma(reorder_net, 
                          seed = 1878819627,
                          trt_effects = "fixed",
                          consistency = "ume",
                          prior_trt = normal(scale = 100),
                          iter = 10000)

test_that("Contr/Reordered UME results are the same", {
  expect_equivalent_nma_summary(summary(contr_fit_FE_UME, pars = "d"), 
                                summary(reorder_fit_FE_UME, pars = "d"), 
                                tolerance = tol)
})

# --- Test regression models with contrast data ---
park_dummy <- dplyr::mutate(parkinsons, x1 = rnorm(dplyr::n(), 0, 1))

arm_net_reg <- set_agd_arm(park_dummy, 
                           study = studyn,
                           trt = trtn,
                           y = y, 
                           se = se,
                           sample_size = n)

arm_fit_FE_reg <- nma(arm_net_reg, 
                      trt_effects = "fixed",
                      regression = ~x1:.trt,
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 10),
                      prior_reg = normal(scale = 1),
                      iter = 10000)

contr_net_reg <- set_agd_contrast(park_dummy, 
                                  study = studyn,
                                  trt = trtn,
                                  y = diff, 
                                  se = se_diff,
                                  sample_size = n)

contr_fit_FE_reg <- nma(contr_net_reg, 
                        trt_effects = "fixed",
                        regression = ~x1:.trt,
                        prior_trt = normal(scale = 100),
                        prior_reg = normal(scale = 1),
                        iter = 10000)

mix_net_reg <- combine_network(set_agd_arm(park_dummy[studies %in% 1:3, ], 
                                       study = studyn,
                                       trt = trtn,
                                       y = y, 
                                       se = se,
                                       sample_size = n), 
                           set_agd_contrast(park_dummy[studies %in% 4:7, ], 
                                  study = studyn,
                                  trt = trtn,
                                  y = diff, 
                                  se = se_diff,
                                  sample_size = n))

mix_fit_FE_reg <- nma(mix_net_reg, 
                      trt_effects = "fixed",
                      regression = ~x1:.trt,
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_reg = normal(scale = 1),
                      iter = 10000)

reorder_net_reg  <- set_agd_contrast(sample_frac(park_dummy, size = 1, replace = FALSE), 
                                     study = studyn,
                                     trt = trtn,
                                     y = diff, 
                                     se = se_diff,
                                     sample_size = n)

reorder_fit_FE_reg <- nma(reorder_net_reg, 
                          seed = 1878819627,
                          trt_effects = "fixed",
                          regression = ~x1:.trt,
                          prior_trt = normal(scale = 100),
                          prior_reg = normal(scale = 1),
                          iter = 10000)

test_that("Regression models work with contrast data", {
  expect_equivalent_nma_summary(summary(arm_fit_FE_reg, pars = c("d", "beta")), 
                                summary(contr_fit_FE_reg, pars = c("d", "beta")), 
                                tolerance = tol)
  
  expect_equivalent_nma_summary(summary(arm_fit_FE_reg, pars = c("d", "beta")), 
                                summary(mix_fit_FE_reg, pars = c("d", "beta")), 
                                tolerance = tol)
  
  expect_equivalent_nma_summary(summary(contr_fit_FE_reg, pars = c("d", "beta")), 
                                summary(reorder_fit_FE_reg, pars = c("d", "beta")), 
                                tolerance = tol)
})

test_that("Error message when using predict() on only contrast data", {
  expect_error(predict(contr_fit_FE_reg),
               "No arm-based data \\(IPD or AgD\\) in network")
})

# ML-NMR regression models too
park_dummy_mlnmr <- dplyr::mutate(parkinsons, 
                                  x1_mean = rnorm(dplyr::n(), 0, 1),
                                  x1_sd = runif(dplyr::n(), 0.1, 1))

arm_net_mlnmr <- set_agd_arm(park_dummy_mlnmr, 
                             study = studyn,
                             trt = trtn,
                             y = y, 
                             se = se,
                             sample_size = n) %>% 
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd), n_int = 10)

arm_fit_FE_mlnmr <- nma(arm_net_mlnmr, 
                        trt_effects = "fixed",
                        regression = ~x1:.trt,
                        prior_intercept = normal(scale = 100),
                        prior_trt = normal(scale = 10),
                        prior_reg = normal(scale = 1),
                        iter = 20000, save_warmup = FALSE)

contr_net_mlnmr <- set_agd_contrast(park_dummy_mlnmr, 
                                  study = studyn,
                                  trt = trtn,
                                  y = diff, 
                                  se = se_diff,
                                  sample_size = n) %>% 
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd), n_int = 10)

contr_fit_FE_mlnmr <- nma(contr_net_mlnmr, 
                          trt_effects = "fixed",
                          regression = ~x1:.trt,
                          prior_trt = normal(scale = 100),
                          prior_reg = normal(scale = 1),
                          iter = 20000, save_warmup = FALSE)

mix_net_mlnmr <- combine_network(set_agd_arm(park_dummy_mlnmr[studies %in% 1:3, ], 
                                       study = studyn,
                                       trt = trtn,
                                       y = y, 
                                       se = se,
                                       sample_size = n), 
                           set_agd_contrast(park_dummy_mlnmr[studies %in% 4:7, ], 
                                  study = studyn,
                                  trt = trtn,
                                  y = diff, 
                                  se = se_diff,
                                  sample_size = n)) %>% 
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd), n_int = 10)

mix_fit_FE_mlnmr <- nma(mix_net_mlnmr, 
                      trt_effects = "fixed",
                      regression = ~x1:.trt,
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_reg = normal(scale = 1),
                      iter = 20000, save_warmup = FALSE)

reorder_net_mlnmr  <- set_agd_contrast(sample_frac(park_dummy_mlnmr, size = 1, replace = FALSE), 
                                     study = studyn,
                                     trt = trtn,
                                     y = diff, 
                                     se = se_diff,
                                     sample_size = n) %>% 
  add_integration(x1 = distr(qnorm, x1_mean, x1_sd), n_int = 10)

reorder_fit_FE_mlnmr <- nma(reorder_net_mlnmr, 
                          seed = 1878819627,
                          trt_effects = "fixed",
                          regression = ~x1:.trt,
                          prior_trt = normal(scale = 100),
                          prior_reg = normal(scale = 1),
                          iter = 20000, save_warmup = FALSE)

test_that("ML-NMR models work with contrast data", {
  expect_equivalent_nma_summary(summary(arm_fit_FE_mlnmr, pars = c("d", "beta")), 
                                summary(contr_fit_FE_mlnmr, pars = c("d", "beta")), 
                                tolerance = tol)
  
  expect_equivalent_nma_summary(summary(arm_fit_FE_mlnmr, pars = c("d", "beta")), 
                                summary(mix_fit_FE_mlnmr, pars = c("d", "beta")), 
                                tolerance = tol)
  
  expect_equivalent_nma_summary(summary(contr_fit_FE_mlnmr, pars = c("d", "beta")), 
                                summary(reorder_fit_FE_mlnmr, pars = c("d", "beta")), 
                                tolerance = tol)
})

test_that("Error message when using predict() on only contrast data", {
  expect_error(predict(contr_fit_FE_mlnmr),
               "No arm-based data \\(IPD or AgD\\) in network")
})

test_that("Robust to custom options(contrasts) settings", {
  arm_fit_FE_SAS <- withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                                         unordered = "contr.SAS")),
                  nma(arm_net, 
                      trt_effects = "fixed",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 10),
                      iter = 10000))
  
    expect_equivalent_nma_summary(summary(arm_fit_FE_SAS), summary(arm_fit_FE),
                                  tolerance = tol)
    expect_equivalent_nma_summary(relative_effects(arm_fit_FE_SAS), relative_effects(arm_fit_FE),
                                  tolerance = tol)
  
  contr_fit_FE_SAS <- withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                                         unordered = "contr.SAS")),
                  nma(contr_net, 
                      trt_effects = "fixed",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 10),
                      iter = 10000))
  
    expect_equivalent_nma_summary(summary(contr_fit_FE_SAS), summary(contr_fit_FE),
                                  tolerance = tol)
    expect_equivalent_nma_summary(relative_effects(contr_fit_FE_SAS), relative_effects(contr_fit_FE),
                                  tolerance = tol)
    
  mix_fit_FE_SAS <- withr::with_options(list(contrasts = c(ordered = "contr.SAS",
                                                         unordered = "contr.SAS")),
                  nma(mix_net, 
                      trt_effects = "fixed",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 10),
                      iter = 10000))
  
    expect_equivalent_nma_summary(summary(mix_fit_FE_SAS), summary(mix_fit_FE),
                                  tolerance = tol)
    expect_equivalent_nma_summary(relative_effects(mix_fit_FE_SAS), relative_effects(mix_fit_FE),
                                  tolerance = tol)

})


# Force clean up
rm(list = ls())
gc()

#' Network meta-analysis models
#'
#' The `nma` function fits network meta-analysis and (multilevel) network
#' meta-regression models in Stan.
#'
#' @param network An `nma_data` object, as created by the functions `set_*()`,
#'   `combine_network()`, or `add_integration()`
#' @param consistency Character string specifying the type of (in)consistency
#'   model to fit, either `"consistency"`, `"ume"`, or `"nodesplit"`
#' @param trt_effects Character string specifying either `"fixed"` or `"random"`
#'   effects
#' @param regression A one-sided model formula, specifying the prognostic and
#'   effect-modifying terms for a regression model. Any references to treatment
#'   should use the `.trt` special variable, for example specifying effect
#'   modifier interactions as `variable:.trt` (see details). The special
#'   variable `.mu` can be used for baseline risk meta-regression: `~.mu:.trt`.
#' @param class_interactions Character string specifying whether effect modifier
#'   interactions are specified as `"common"`, `"exchangeable"`, or
#'   `"independent"`.
#' @param class_effects Character string specifying a model for treatment class effects,
#'   either `"independent"` (the default), `"exchangeable"`, or `"common"`.
#' @param class_sd Character string specifying whether the class standard deviations in a
#'   class effects model should be `"independent"` (i.e. separate for each class, the default),
#'   or `"common"` (i.e. shared across all classes). Alternatively this can be a list of
#'   character vectors, each of which describe a set classes for which to share a common class SD;
#'   any list names will be used to name the output parameters, otherwise the name will be taken
#'   from the first class in each set.
#' @param likelihood Character string specifying a likelihood, if unspecified
#'   will be inferred from the data (see details)
#' @param link Character string specifying a link function, if unspecified will
#'   default to the canonical link (see details)
#' @param ... Further arguments passed to
#'   \code{\link[rstan:stanmodel-method-sampling]{sampling()}}, such as `iter`,
#'   `chains`, `cores`, etc.
#' @param nodesplit For `consistency = "nodesplit"`, the comparison(s) to split
#'   in the node-splitting model(s). Either a length 2 vector giving the
#'   treatments in a single comparison, or a 2 column data frame listing
#'   multiple treatment comparisons to split in turn. By default, all possible
#'   comparisons will be chosen (see [get_nodesplits()]).
#' @param prior_intercept Specification of prior distribution for the intercept
#' @param prior_trt Specification of prior distribution for the treatment
#'   effects
#' @param prior_het Specification of prior distribution for the heterogeneity
#'   (if `trt_effects = "random"`)
#' @param prior_het_type Character string specifying whether the prior
#'   distribution `prior_het` is placed on the heterogeneity standard deviation
#'   \eqn{\tau} (`"sd"`, the default), variance \eqn{\tau^2} (`"var"`), or
#'   precision \eqn{1/\tau^2} (`"prec"`).
#' @param prior_reg Specification of prior distribution for the regression
#'   coefficients (if `regression` formula specified)
#' @param prior_aux Specification of prior distribution for the auxiliary
#'   parameter, if applicable (see details). For `likelihood = "gengamma"` this
#'   should be a list of prior distributions with elements `sigma` and `k`.
#' @param prior_aux_reg Specification of prior distribution for the auxiliary
#'   regression parameters, if `aux_regression` is specified (see details).
#' @param aux_by Vector of variable names listing the variables to stratify the
#'   auxiliary parameters by. Currently only used for survival models, see
#'   details. Cannot be used with `aux_regression`.
#' @param aux_regression A one-sided model formula giving a regression model for
#'   the auxiliary parameters. Currently only used for survival models, see
#'   details. Cannot be used with `aux_by`.
#' @param prior_class_mean Specification of prior distribution for the
#'   treatment class means (if `class_effects = "exchangeable"`).
#' @param prior_class_sd Specification of prior distribution for the
#'   treatment class standard deviations (if `class_effects = "exchangeable"`).
#' @param QR Logical scalar (default `FALSE`), whether to apply a QR
#'   decomposition to the model design matrix
#' @param center Logical scalar (default `TRUE`), whether to center the
#'   (numeric) regression terms about the overall means
#' @param adapt_delta See [adapt_delta] for details
#' @param int_thin A single integer value, the thinning factor for returning
#'   cumulative estimates of integration error. Saving cumulative estimates is
#'   disabled by `int_thin = 0`, which is the default.
#' @param int_check Logical, check sufficient accuracy of numerical integration
#'   by fitting half of the chains with `n_int/2`? When `TRUE`, `Rhat` and
#'   `n_eff` diagnostic warnings will be given if numerical integration has not
#'   sufficiently converged (suggesting increasing `n_int` in
#'   [add_integration()]). Default `TRUE`, but disabled (`FALSE`) when
#'   `int_thin > 0`.
#' @param mspline_degree Non-negative integer giving the degree of the M-spline
#'   polynomial for `likelihood = "mspline"`. Piecewise exponential hazards
#'   (`likelihood = "pexp"`) are a special case with `mspline_degree = 0`.
#' @param n_knots For `mspline` and `pexp` likelihoods, a non-negative integer
#'   giving the number of internal knots for partitioning the baseline hazard
#'   into intervals. The knot locations within each study will be determined by
#'   the corresponding quantiles of the observed event times, plus boundary
#'   knots at the earliest entry time (0 with no delayed entry) and the maximum
#'   event/censoring time. For example, with `n_knots = 3`, the internal knot
#'   locations will be at the 25%, 50%, and 75% quantiles of the observed event
#'   times. The default is `n_knots = 7`; overfitting is avoided by shrinking
#'   towards a constant hazard with a random walk prior (see details). If
#'   `aux_regression` is specified then a single set of knot locations will be
#'   calculated across all studies in the network. See [make_knots()] for more
#'   details on the knot positioning algorithms. Ignored when `knots` is
#'   specified.
#' @param knots For `mspline` and `pexp` likelihoods, a named list of numeric
#'   vectors of knot locations (including boundary knots) for each of the
#'   studies in the network. Currently, each vector must have the same length
#'   (i.e. each study must use the same number of knots). Alternatively, a
#'   single numeric vector of knot locations can be provided which will be
#'   shared across all studies in the network. If unspecified (the default), the
#'   knots will be chosen based on `n_knots` as described above. If
#'   `aux_regression` is specified then `knots` should be a single numeric
#'   vector of knot locations which will be shared across all studies in the
#'   network. [make_knots()] can be used to help specify `knots` directly, or to
#'   investigate knot placement prior to model fitting.
#' @param mspline_basis Instead of specifying `mspline_degree` and `n_knots` or
#'   `knots`, a named list of M-spline bases (one for each study) can be
#'   provided with `mspline_basis` which will be used directly. In this case,
#'   all other M-spline options will be ignored.
#'
#' @details When specifying a model formula in the `regression` argument, the
#'   usual formula syntax is available (as interpreted by [model.matrix()]). The
#'   only additional requirement here is that the special variable `.trt` should
#'   be used to refer to treatment. For example, effect modifier interactions
#'   should be specified as `variable:.trt`. Prognostic (main) effects and
#'   interactions can be included together compactly as `variable*.trt`, which
#'   expands to `variable + variable:.trt` (plus `.trt`, which is already in the
#'   NMA model).
#'
#'   For the advanced user, the additional specials `.study` and `.trtclass` are
#'   also available, and refer to studies and (if specified) treatment classes
#'   respectively. When node-splitting models are fitted (`consistency =
#'   "nodesplit"`) the special `.omega` is available, indicating the arms to
#'   which the node-splitting inconsistency factor is added.
#'
#'   See \code{\link[multinma:priors]{?priors}} for details on prior
#'   specification. Default prior distributions are available, but may not be
#'   appropriate for the particular setting and will raise a warning if used. No
#'   attempt is made to tailor these defaults to the data provided. Please
#'   consider appropriate prior distributions for the particular setting,
#'   accounting for the scales of outcomes and covariates, etc. The function
#'   [plot_prior_posterior()] may be useful in examining the influence of the
#'   chosen prior distributions on the posterior distributions, and the
#'   \code{\link[multinma:summary.nma_prior]{summary()}} method for `nma_prior`
#'   objects prints prior intervals.
#'
#' @section Likelihoods and link functions:
#'   Currently, the following likelihoods and link functions are supported for
#'   each data type:
#'
#'   | \strong{Data type} | \strong{Likelihood}   | \strong{Link function} |
#'   |--------------------|-----------------------|------------------------|
#'   | \strong{Binary}    | `bernoulli`, `bernoulli2`| `logit`, `probit`, `cloglog`
#'   | \strong{Count}     | `binomial`, `binomial2`  | `logit`, `probit`, `cloglog`
#'   | \strong{Rate}      | `poisson`    | `log`
#'   | \strong{Continuous}| `normal`     | `identity`, `log`
#'   | \strong{Ordered}   | `ordered`    | `logit`, `probit`, `cloglog`
#'   | \strong{Survival}  | `exponential`, `weibull`, `gompertz`, `exponential-aft`, `weibull-aft`, `lognormal`, `loglogistic`, `gamma`, `gengamma`, `mspline`, `pexp` | `log`
#'
#'   The `bernoulli2` and `binomial2` likelihoods correspond to a two-parameter
#'   Binomial likelihood for arm-based AgD, which more closely matches the
#'   underlying Poisson Binomial distribution for the summarised aggregate
#'   outcomes in a ML-NMR model than the typical (one parameter) Binomial
#'   distribution \insertCite{@see @methods_paper}{multinma}.
#'
#'   When a `cloglog` link is used, including an offset for log follow-up time
#'   (i.e. `regression = ~offset(log(time))`) results in a model on the log
#'   hazard \insertCite{@see @TSD2}{multinma}.
#'
#'   For survival data, all accelerated failure time models (`exponential-aft`,
#'   `weibull-aft`, `lognormal`, `loglogistic`, `gamma`, `gengamma`) are
#'   parameterised so that the treatment effects and any regression parameters
#'   are log Survival Time Ratios (i.e. a coefficient of \eqn{\log(2)} means
#'   that the treatment or covariate is associated with a doubling of expected
#'   survival time). These can be converted to log Acceleration Factors using
#'   the relation \eqn{\log(\mathrm{AF}) = -\log(\mathrm{STR})} (or equivalently
#'   \eqn{\mathrm{AF} = 1/\mathrm{STR}}).
#'
#'   Further details on each likelihood and link function are given by
#'   \insertCite{TSD2;textual}{multinma}. Details of survival models are given
#'   by \insertCite{Phillippo_survival;textual}{multinma}.
#'
#'
#' @section Auxiliary parameters:
#'   Auxiliary parameters are only present in the following models.
#'
#'   ## Normal likelihood with IPD
#'   When a Normal likelihood is fitted to IPD, the auxiliary parameters are the
#'   arm-level standard deviations \eqn{\sigma_{jk}} on treatment \eqn{k} in
#'   study \eqn{j}.
#'
#'   ## Ordered multinomial likelihood
#'   When fitting a model to \eqn{M} ordered outcomes, the auxiliary parameters
#'   are the latent cutoffs between each category, \eqn{c_0 < c_1 < \dots <
#'   c_M}. Only \eqn{c_2} to \eqn{c_{M-1}} are estimated; we fix \eqn{c_0 =
#'   -\infty}, \eqn{c_1 = 0}, and \eqn{c_M = \infty}. When specifying priors for
#'   these latent cutoffs, we choose to specify priors on the *differences*
#'   \eqn{c_{m+1} - c_m}. Stan automatically truncates any priors so that the
#'   ordering constraints are satisfied.
#'
#'   ## Survival (time-to-event) likelihoods
#'   All survival likelihoods except the `exponential` and `exponential-aft`
#'   likelihoods have auxiliary parameters. These are typically study-specific
#'   shape parameters \eqn{\gamma_j>0}, except for the `lognormal` likelihood
#'   where the auxiliary parameters are study-specific standard deviations on
#'   the log scale \eqn{\sigma_j>0}.
#'
#'   The `gengamma` likelihood has two sets of auxiliary parameters,
#'   study-specific scale parameters \eqn{\sigma_j>0} and shape parameters
#'   \eqn{k_j}, following the parameterisation of
#'   \insertCite{Lawless1980;textual}{multinma}, which permits a range of
#'   behaviours for the baseline hazard including increasing, decreasing,
#'   bathtub and arc-shaped hazards. This parameterisation is related to that
#'   discussed by \insertCite{Cox2007;textual}{multinma} and implemented in the
#'   `flexsurv` package with \eqn{Q = k^{-0.5}}. The parameterisation used here
#'   effectively bounds the shape parameter \eqn{k} away from numerical
#'   instabilities as \eqn{k \rightarrow \infty} (i.e. away from \eqn{Q
#'   \rightarrow 0}, the log-Normal distribution) via the prior distribution.
#'   Implicitly, this parameterisation is restricted to \eqn{Q > 0} and so
#'   certain survival distributions like the inverse-Gamma and inverse-Weibull
#'   are not part of the parameter space; however, \eqn{Q > 0} still encompasses
#'   the other survival distributions implemented in this package.
#'
#'   For the `mspline` and `pexp` likelihoods, the auxiliary parameters are the
#'   spline coefficients for each study. These form a unit simplex (i.e. lie
#'   between 0 and 1, and sum to 1), and are given a random walk prior
#'   distribution \insertCite{Phillippo_mspline}{multinma}. `prior_aux`
#'   specifies the hyperprior on the random walk standard deviation \eqn{\sigma}
#'   which controls the level of smoothing of the baseline hazard, with
#'   \eqn{\sigma = 0} corresponding to a constant baseline hazard.
#'
#'   The auxiliary parameters can be stratified by additional factors through
#'   the `aux_by` argument. For example, to allow the shape of the baseline
#'   hazard to vary between treatment arms as well as studies, use `aux_by =
#'   c(".study", ".trt")`. (Technically, `.study` is always included in the
#'   stratification even if omitted from `aux_by`, but we choose here to make
#'   the stratification explicit.) This is a common way of relaxing the
#'   proportional hazards assumption. The default is equivalent to `aux_by =
#'   ".study"` which stratifies the auxiliary parameters by study, as described
#'   above.
#'
#'   A regression model may be specified on the auxiliary parameters using
#'   `aux_regression`. This is useful if we wish to model departures from
#'   non-proportionality, rather than allowing the baseline hazards to be
#'   completely independent using `aux_by`. This is necessary if absolute
#'   predictions (e.g. survival curves) are required in a population for
#'   unobserved combinations of covariates; for example, if `aux_by = .trt` then
#'   absolute predictions may only be produced for the observed treatment arms
#'   in each study population, whereas if `aux_regression = ~.trt` then absolute
#'   predictions can be produced for all treatments in any population. For
#'   `mspline` and `pexp` likelihoods, the regression coefficients are smoothed
#'   over time using a random walk prior to avoid overfitting
#'   \insertCite{Phillippo_mspline}{multinma}: `prior_aux_reg` specifies the
#'   hyperprior for the random walk standard deviation. For other parametric
#'   likelihoods, `prior_aux_reg` specifies the prior for the auxiliary
#'   regression coefficients.
#'
#' @return `nma()` returns a [stan_nma] object, except when `consistency =
#'   "nodesplit"` when a [nma_nodesplit] or [nma_nodesplit_df] object is
#'   returned. `nma.fit()` returns a \code{\link[rstan:stanfit-class]{stanfit}}
#'   object.
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#' ## Smoking cessation NMA
#' @template ex_smoking_network
#' @template ex_smoking_nma_fe
#' @template ex_smoking_nma_re
#' @template ex_smoking_nma_re_ume
#' @template ex_smoking_nma_re_nodesplit
#' @examples \donttest{
#' # Summarise the node-splitting results
#' summary(smk_fit_RE_nodesplit)
#' }
#'
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_network
#' @template ex_plaque_psoriasis_integration
#' @template ex_plaque_psoriasis_mlnmr
#'
#' @examples
#' ## Newly-diagnosed multiple myeloma NMA
#' @template ex_ndmm_network
#' @template ex_ndmm
nma <- function(network,
                consistency = c("consistency", "ume", "nodesplit"),
                trt_effects = c("fixed", "random"),
                regression = NULL,
                class_interactions = c("common", "exchangeable", "independent"),
                class_effects = c("independent", "common", "exchangeable"),
                class_sd =  c("independent", "common"),
                likelihood = NULL,
                link = NULL,
                ...,
                nodesplit = get_nodesplits(network, include_consistency = TRUE),
                prior_intercept = .default(normal(scale = 100)),
                prior_trt = .default(normal(scale = 10)),
                prior_het = .default(half_normal(scale = 5)),
                prior_het_type = c("sd", "var", "prec"),
                prior_reg = .default(normal(scale = 10)),
                prior_aux = .default(),
                prior_aux_reg = .default(),
                prior_class_mean = .default(normal(scale = 10)),
                prior_class_sd = .default(half_normal(scale = 5)),
                aux_by = NULL,
                aux_regression = NULL,
                QR = FALSE,
                center = TRUE,
                adapt_delta = NULL,
                int_thin = 0,
                int_check = TRUE,
                mspline_degree = 3,
                n_knots = 7,
                knots = NULL,
                mspline_basis = NULL) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("Expecting an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  # Check model arguments
  consistency <- rlang::arg_match(consistency)
  trt_effects <- rlang::arg_match(trt_effects)
  class_effects <- rlang::arg_match(class_effects)

  # Check class_effects and network classes
  if (class_effects != "independent") {
    if (is.null(network$classes)) {
      abort(paste("Setting `class_effects` requires treatment classes to be specified in the network.",
                  "See set_*() argument `trt_class`.", sep = "\n"))
    }
  }



  if (class_effects == "common") {
    # Overwrite treatments with class variables
    if (has_ipd(network)) {
      network$ipd$.trt <- network$ipd$.trtclass
    }
    if (has_agd_arm(network)) {
      network$agd_arm$.trt <- network$agd_arm$.trtclass
    }
    if (has_agd_contrast(network)) {
      network$agd_contrast$.trt <- network$agd_contrast$.trtclass
    }

    # Set the network treatments vector
    network$treatments <- if (.is_default(network$treatments)) {
        .default(factor(levels(network$classes), levels = levels(network$classes)))
      } else {
        factor(levels(network$classes), levels = levels(network$classes))
      }

    # Set network classes vector
    network$classes <- network$treatments
  }

  # Check class_sd
  if (is.list(class_sd)) {
    # Check that all classes listed in 'class_sd' are in 'network$classes'
    if (!all(unlist(class_sd) %in% network$classes)) {
      stop("Some classes listed in 'class_sd' are not present in the network.")
    }


    # Check that all the collapsed classes are distinct and don't share a class
    flattened_classes <- unlist(class_sd)
    if (length(flattened_classes) != length(unique(flattened_classes))) {
      stop("Some classes are listed in more than one shared standard deviation group in 'class_sd'")
    }
  } else {
    class_sd <- rlang::arg_match(class_sd)
  }

  if (consistency == "nodesplit") {

    lvls_trt <- levels(network$treatments)
    nodesplit_include_consistency <- FALSE

    if (is.data.frame(nodesplit)) { # Data frame listing comparisons to split
      if (ncol(nodesplit) != 2)
        abort("The data frame passed to `nodesplit` should have two columns.")

      nodesplit <- tibble::as_tibble(nodesplit)
      colnames(nodesplit) <- c("trt1", "trt2")

      # NA rows indicate include_consistency = TRUE, filter these out
      if (any(is.na(nodesplit[,1]) & is.na(nodesplit[,2]))) {
        nodesplit_include_consistency <- TRUE
        nodesplit <- dplyr::filter(nodesplit, !is.na(.data$trt1) & !is.na(.data$trt2))
      }

      if (nrow(nodesplit) == 0) {
        abort("No comparisons to node-split.")
      }

      nodesplit$trt1 <- as.character(nodesplit$trt1)
      nodesplit$trt2 <- as.character(nodesplit$trt2)

      if (!all(unlist(nodesplit) %in% lvls_trt))
        abort(sprintf("All comparisons in `nodesplit` should match two treatments in the network.\nSuitable values are: %s",
                      ifelse(length(lvls_trt) <= 5,
                             paste0(lvls_trt, collapse = ", "),
                             paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

      if (any(nodesplit[,1] == nodesplit[,2]))
        abort("`nodesplit` comparison cannot be the same treatment against itself.")

      # Check valid nodesplit - must have both direct and indirect evidence
      ns_check <- dplyr::rowwise(nodesplit) %>%
        dplyr::mutate(direct = has_direct(network, .data$trt1, .data$trt2),
                      indirect = has_indirect(network, .data$trt1, .data$trt2),
                      valid = .data$direct && .data$indirect)

      if (any(!ns_check$valid)) {
        ns_valid <- dplyr::filter(ns_check, .data$valid) %>%
          dplyr::ungroup() %>%
          dplyr::select("trt1", "trt2")

        ns_invalid <- dplyr::filter(ns_check, !.data$valid) %>%
          dplyr::mutate(comparison = paste(.data$trt1, .data$trt2, sep = " vs. "))

        if (nrow(ns_valid)) {
          warn(glue::glue(
            "Ignoring node-split comparisons without both both direct and independent indirect evidence: ",
            glue::glue_collapse(ns_invalid$comparison, sep = ", ", width = 100), "."
            ))

          nodesplit <- ns_valid
        } else {
          abort("No valid comparisons for node-splitting given in `nodesplit`.\n Comparisons must have both direct and independent indirect evidence for node-splitting.")
        }

      }

      # Store comparisons as factors, in increasing order (i.e. trt1 < trt2)
      nodesplit$trt1 <- factor(nodesplit$trt1, levels = lvls_trt)
      nodesplit$trt2 <- factor(nodesplit$trt2, levels = lvls_trt)

      for (i in 1:nrow(nodesplit)) {
        if (as.numeric(nodesplit$trt1[i]) > as.numeric(nodesplit$trt2[i])) {
          nodesplit[i, ] <- rev(nodesplit[i, ])
        }
      }

      # Iteratively call node-splitting models
      n_ns <- nrow(nodesplit) + nodesplit_include_consistency
      ns_fits <- vector("list", n_ns)

      ns_arglist <- list(network = network,
                         consistency = "nodesplit",
                         trt_effects = trt_effects,
                         class_effects = class_effects,
                         class_sd = class_sd,
                         regression = regression,
                         likelihood = likelihood,
                         link = link,
                         ...,
                         prior_intercept = prior_intercept,
                         prior_trt = prior_trt,
                         prior_het = prior_het,
                         prior_het_type = prior_het_type,
                         prior_reg = prior_reg,
                         prior_aux = prior_aux,
                         prior_aux_reg = prior_aux_reg,
                         aux_by = aux_by,
                         aux_regression = aux_regression,
                         prior_class_mean = prior_class_mean,
                         prior_class_sd = prior_class_sd,
                         QR = QR,
                         center = center,
                         adapt_delta = adapt_delta,
                         int_thin = int_thin,
                         mspline_degree = mspline_degree,
                         n_knots = n_knots,
                         knots = knots,
                         mspline_basis = mspline_basis)

      if (!missing(class_interactions)) ns_arglist$class_interactions <- class_interactions


      for (i in 1:nrow(nodesplit)) {

        inform(glue::glue("Fitting model {i} of {n_ns}, node-split: ",
                          as.character(nodesplit$trt2[i]),
                          " vs. ",
                          as.character(nodesplit$trt1[i])))

        ns_arglist$nodesplit <- forcats::fct_c(nodesplit$trt1[i], nodesplit$trt2[i])

        ns_fits[[i + nodesplit_include_consistency]] <- do.call(nma, ns_arglist)
      }

      if (nodesplit_include_consistency) {
        inform(glue::glue("Fitting model {n_ns} of {n_ns}, consistency model"))
        nodesplit <- tibble::add_row(nodesplit, .before = 1)

        ns_arglist$consistency <- "consistency"
        ns_arglist$nodesplit <- NULL

        ns_fits[[1]] <- do.call(nma, ns_arglist)
      }

      nodesplit$model <- ns_fits

      # Return a nma_nodesplit_df object
      class(nodesplit) <- c("nma_nodesplit_df", class(nodesplit))
      return(nodesplit)

    } else if (rlang::is_vector(nodesplit, n = 2)) { # Vector giving single comparison to split

      nodesplit <- as.character(nodesplit)

      if (!all(nodesplit %in% lvls_trt))
        abort(sprintf("The `nodesplit` treatment comparison should match two treatments in the network.\nSuitable values are: %s",
                      ifelse(length(lvls_trt) <= 5,
                             paste0(lvls_trt, collapse = ", "),
                             paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

      if (nodesplit[1] == nodesplit[2])
        abort("`nodesplit` comparison cannot be the same treatment against itself.")

      # Check valid nodesplit - must have both direct and indirect evidence
      if (!has_direct(network, nodesplit[1], nodesplit[2])) {
        abort(glue::glue("Cannot node-split the {nodesplit[1]} vs. {nodesplit[2]} comparison, no direct evidence."))
      }
      if (!has_indirect(network, nodesplit[1], nodesplit[2])) {
        abort(glue::glue("Cannot node-split the {nodesplit[1]} vs. {nodesplit[2]} comparison, no independent indirect evidence."))
      }

      # Store comparison as factor, in increasing order (i.e. trt1 < trt2)
      nodesplit <- factor(nodesplit, levels = lvls_trt)
      if (as.numeric(nodesplit[1]) > as.numeric(nodesplit[2])) {
        nodesplit <- rev(nodesplit)
      }

    } else {
      abort("`nodesplit` should either be a length 2 vector or a 2 column data frame, giving the comparison(s) to node-split.")
    }
  }

  if (!is.null(regression) && !rlang::is_formula(regression, lhs = FALSE)) {
    abort("`regression` should be a one-sided formula.")
  }

  if (is.null(network$classes)) {
    if (!missing(class_interactions)) {
      abort(paste("Setting `class_interactions` requires treatment classes to be specified in the network.",
                  "See set_*() argument `trt_class`.", sep = "\n"))
    } else if (!is.null(regression)) {
      inform(paste("Note: No treatment classes specified in network, any interactions in `regression` formula will be separate (independent) for each treatment.",
                   "Use set_*() argument `trt_class` and nma() argument `class_interactions` to change this.", sep = "\n"))
    }
  }
  class_interactions <- rlang::arg_match(class_interactions)

  likelihood <- check_likelihood(likelihood, network$outcome)
  link <- check_link(link, likelihood)

  # When are priors on auxiliary parameters required?
  has_aux <- (likelihood == "normal" && has_ipd(network)) ||
              likelihood %in% c("ordered", "weibull", "gompertz",
                                "weibull-aft", "lognormal", "loglogistic",
                                "gamma", "gengamma", "mspline", "pexp")

  # Are study intercepts present? Not if only contrast data
  has_intercepts <- has_agd_arm(network) || has_ipd(network)

  # Check priors
  check_prior(prior_intercept)
  check_prior(prior_trt)
  check_prior(prior_het)
  check_prior(prior_reg)
  if (!.is_default(prior_aux)) {
    if (likelihood == "gengamma") check_prior(prior_aux, c("sigma", "k"))
    else check_prior(prior_aux)
  }

  prior_het_type <- rlang::arg_match(prior_het_type)

  # Prior defaults
  prior_defaults <- list()
  if (has_intercepts && .is_default(prior_intercept))
    prior_defaults$prior_intercept <- get_prior_call(prior_intercept)
  if (.is_default(prior_trt))
    prior_defaults$prior_trt <- get_prior_call(prior_trt)
  if (trt_effects == "random" && .is_default(prior_het))
    prior_defaults$prior_het <- get_prior_call(prior_het)
  if (class_effects == "exchangeable" && .is_default(prior_class_mean))
    prior_defaults$prior_class_mean <- get_prior_call(prior_class_mean)
  if (class_effects == "exchangeable" && .is_default(prior_class_sd))
    prior_defaults$prior_class_sd <- get_prior_call(prior_class_sd)
  if (!is.null(regression) && !is_only_offset(regression) && .is_default(prior_reg))
    prior_defaults$prior_reg <- get_prior_call(prior_reg)
  if (has_aux && .is_default(prior_aux)) {
    if (likelihood == "normal" && has_ipd(network)) {
      prior_aux <- .default(half_normal(scale = 5))
    } else if (likelihood == "ordered") {
      prior_aux <- .default(flat())
    } else if (likelihood %in% c("weibull", "gompertz", "weibull-aft",
                                 "lognormal", "loglogistic", "gamma")) {
      prior_aux <- .default(half_normal(scale = 10))
    } else if (likelihood == "gengamma") {
      prior_aux <- .default(list(sigma = half_normal(scale = 10),
                                 k = half_normal(scale = 10)))
    } else if (likelihood %in% c("mspline", "pexp")) {
      prior_aux <- .default(half_normal(scale = 1))
    }
    prior_defaults$prior_aux <- get_prior_call(prior_aux)
  }
  if (has_aux && !is.null(aux_regression) && .is_default(prior_aux_reg)) {
    if (likelihood %in% c("mspline", "pexp")) {
      prior_aux_reg <- .default(half_normal(scale = 1))
    } else if (likelihood %in% valid_lhood$survival) {
      prior_aux_reg <- .default(normal(scale = 10))
    }
    prior_defaults$prior_aux_reg <- get_prior_call(prior_aux_reg)
  }

  # Warn where default priors are used
  if (!rlang::is_empty(prior_defaults)) {
    warn(glue::glue(
      "Prior distributions were left at default values:",
      paste(paste(names(prior_defaults), prior_defaults, sep = " = "), collapse = "\n"),
      .sep = "\n"
    ))
  }

  # Check other args
  if (!rlang::is_bool(QR)) abort("`QR` should be a logical scalar (TRUE or FALSE).")
  if (!rlang::is_bool(center)) abort("`center` should be a logical scalar (TRUE or FALSE).")
  if (!rlang::is_scalar_integerish(int_thin) ||
      int_thin < 0) abort("`int_thin` should be an integer >= 0.")
  if (!rlang::is_bool(int_check)) abort("`int_check` should be a logical scalar (TRUE or FALSE).")
  if (int_thin > 0) int_check <- FALSE

  if (".mu" %in% all.vars(regression)) {
    if (has_agd_contrast(network)) abort("Regression on baseline risk (`.mu` in `regression` formula) is not supported with contrast data.")
    if (QR) {
      warn("Cannot fit baseline risk meta-regression model with QR decomposition, setting QR = FALSE.")
      QR <- FALSE
    }
  }

  # Set adapt_delta
  if (is.null(adapt_delta)) {
    adapt_delta <- switch(trt_effects, fixed = 0.8, random = 0.95)
  } else if (!rlang::is_scalar_double(adapt_delta) ||
      adapt_delta <= 0 || adapt_delta >= 1) abort("`adapt_delta` should be a  numeric value in (0, 1).")

  # Check aux_by / aux_regression combination
  aux_by <- rlang::enquo(aux_by)
  if (!rlang::quo_is_null(aux_by) && !is.null(aux_regression)) {
    abort("Cannot specify both `aux_by` and `aux_regression`.")
  }

  if (!is.null(aux_regression)) {
    if (!likelihood %in% valid_lhood$survival) {
      warn("Ignoring `aux_regression`, only supported for survival likelihoods at present.")
    } else if (!has_aux) {
      warn(glue::glue("Ignoring `aux_regression`, no auxiliary parameters in {likelihood} model."))
    }
  }

  if (!rlang::quo_is_null(aux_by)) {
    if (!likelihood %in% valid_lhood$survival) {
      warn("Ignoring `aux_by`, only supported for survival likelihoods at present.")
    } else if (!has_aux) {
      warn(glue::glue("Ignoring `aux_by`, no auxiliary parameters in {likelihood} model."))
    }
  }

  # Set up aux_regression
  if (!is.null(aux_regression) && likelihood %in% valid_lhood$survival &&
      has_aux &&
      (has_ipd(network) || has_agd_arm(network))) {
    if (!rlang::is_formula(aux_regression, lhs = FALSE)) abort("`aux_regression` should be a one-sided formula.")
    if (!is.null(attr(aux_regression, "offset"))) abort("Offset terms not allowed in `aux_regression`.")

    # Remove main effect of study if specified, and make sure treatment handled properly (first, no intercept for M-spline model with symmetric RW prior)
    if (".trt" %in% colnames(attr(terms(aux_regression), "factor"))) {
      aux_regression <- update.formula(aux_regression, ~. -.study -.trt +1)
      if (likelihood %in% c("mspline", "pexp")) aux_regression <- update.formula(aux_regression, ~.trt + . -1)
      else aux_regression <- update.formula(aux_regression, ~.trt + . +1)
    } else {
      aux_regression <- update.formula(aux_regression, ~. -.study +1)
    }
    has_aux_regression <- TRUE
  } else {
    has_aux_regression <- FALSE
    aux_regression <- NULL
  }

  # Set up aux_by
  if (likelihood %in% valid_lhood$survival &&
      has_aux &&
      (has_ipd(network) || has_agd_arm(network))) {

    has_aux_by <- TRUE
    if (rlang::quo_is_null(aux_by)) aux_by <- ".study"

    aux_dat <- dplyr::bind_rows(if (has_ipd(network)) dplyr::select(network$ipd, -".Surv") else NULL,
                                if (has_agd_arm(network)) {
                                  if (inherits(network, "mlnmr_data")) {
                                    .unnest_integration(network$agd_arm) %>% dplyr::select(-".Surv")
                                  } else {
                                    dplyr::select(network$agd_arm, -".Surv")
                                  }
                                } else NULL)

    # Check specs and translate into string column names
    aux_by <- colnames(get_aux_by_data(aux_dat, by = aux_by))
  } else {
    has_aux_by <- FALSE
    aux_by <- NULL
  }


  # Use numerical integration? TRUE if class mlnmr_data and regression is not NULL
  # (Avoids unnecessary use of integration points if regression formula not specified)
  use_int <- inherits(network, "mlnmr_data") && (!is.null(regression) || aux_needs_integration(aux_regression = aux_regression, aux_by = aux_by))

  # Number of numerical integration points
  # Set to 1 if no numerical integration, so that regression on summary data is possible
  n_int <- if (use_int) network$n_int else 1

  # Warn if combining AgD and IPD in a meta-regression without using integration
  if (!is.null(regression) && !inherits(network, "mlnmr_data") && has_ipd(network) &&
      (has_agd_arm(network) || has_agd_contrast(network))) {
    warn(glue::glue("No integration points available, using naive plug-in model at aggregate level.\n",
                    "Use `add_integration()` to add integration points to the network."))
  }

  # Notify if default reference treatment is used
  if (.is_default(network$treatments))
    inform(glue::glue('Note: Setting "{levels(network$treatments)[1]}" as the network reference treatment.'))
  # Notify if network is disconnected
  if (!is_network_connected(network))
    inform("Note: Network is disconnected. See ?is_network_connected for more details.")
  # Notify if reference treatment is within a class when running the exchangeable class model
  if (class_effects == "exchangeable" && !is.null(network$classes)) {
    ref_trt <- levels(network$treatments)[1]
    ref_class <- levels(network$classes)[1]

    # Count how many treatments share this class
    n_in_class <- sum(network$classes == ref_class)

    if (n_in_class >= 2) {
      inform(glue::glue("Note: Reference treatment {ref_trt} has been removed from {ref_class}."))
    }
  }

  # Get data for design matrices and outcomes
  if (has_ipd(network)) {
    dat_ipd <- network$ipd

    if (".mu" %in% all.vars(regression)) {
      if (".mu" %in% colnames(dat_ipd)) {
        warn(c(
          "Detected `.mu` in the `regression` formula and in the data.",
          `*` = "`.mu` column in data will be ignored.",
          i = "`.mu` is a special variable referring to the modelled baseline risk."
        ))
      }

      dat_ipd$.mu <- 1L
    }

    # Only take necessary columns
    dat_ipd <- get_model_data_columns(dat_ipd,
                                      regression = regression,
                                      aux_regression = aux_regression,
                                      label = "IPD",
                                      keep = if (has_aux_by) aux_by else NULL)

    y_ipd <- get_outcome_variables(network$ipd, network$outcome$ipd)

  } else {
    dat_ipd <- tibble::tibble()
    y_ipd <- NULL
  }

  if (has_agd_arm(network)) {
    dat_agd_arm <- network$agd_arm

    y_agd_arm <- get_outcome_variables(dat_agd_arm, network$outcome$agd_arm)

    # Unnest survival data
    if (network$outcome$agd_arm == "survival") {
      y_agd_arm <- tidyr::unnest(y_agd_arm, cols = ".Surv")
    }

    # Set up integration variables if present
    if (use_int) {
      if (network$outcome$agd_arm == "survival") {
        idat_agd_arm <- dat_agd_arm %>%
          # Drop duplicated names in outer dataset from .data_orig before unnesting
          dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(dat_agd_arm))))) %>%
          # Unnest - should have n_int contiguous rows for each survival time
          tidyr::unnest(cols = c(".Surv", ".data_orig")) %>%
          .unnest_integration()
      } else {
        idat_agd_arm <- .unnest_integration(dat_agd_arm)
      }
    } else {
      if (network$outcome$agd_arm == "survival") {
        idat_agd_arm <- dat_agd_arm %>%
          # Drop duplicated names in outer dataset from .data_orig before unnesting
          dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(dat_agd_arm))))) %>%
          # Unnest - should have one row for each survival time
          tidyr::unnest(cols = c(".Surv", ".data_orig"))
      } else {
        idat_agd_arm <- dat_agd_arm
      }
    }

    if (".mu" %in% all.vars(regression)) {
      if (".mu" %in% colnames(idat_agd_arm)) {
        warn(c(
          "Detected `.mu` in the `regression` formula and in the data.",
          `*` = "`.mu` column in data will be ignored.",
          i = "`.mu` is a special variable referring to the modelled baseline risk."
        ))
      }

      idat_agd_arm$.mu <- 1L
    }

    # Only take necessary columns
    idat_agd_arm <- get_model_data_columns(idat_agd_arm,
                                           regression = regression,
                                           aux_regression = aux_regression,
                                           label = "AgD (arm-based)",
                                           keep = if (has_aux_by) aux_by else NULL)

  } else {
    dat_agd_arm <- idat_agd_arm <- tibble::tibble()
    y_agd_arm <- NULL
  }

  if (has_agd_contrast(network)) {
    dat_agd_contrast <- network$agd_contrast

    y_agd_contrast <- get_outcome_variables(dat_agd_contrast, network$outcome$agd_contrast)

    # Set up integration variables if present
    if (use_int) {
      idat_agd_contrast <-  .unnest_integration(dat_agd_contrast)
    } else {
      idat_agd_contrast <- dat_agd_contrast
    }

    # Only take necessary columns
    idat_agd_contrast <- get_model_data_columns(idat_agd_contrast,
                                                regression = regression,
                                                label = "AgD (contrast-based)")

    # Get covariance structure for relative effects, using .se on baseline arm
    Sigma_agd_contrast <- make_Sigma(dat_agd_contrast)

    # Split into baseline and non-baseline arms
    dat_agd_contrast_bl <- dplyr::filter(dat_agd_contrast, is.na(.data$.y))
    idat_agd_contrast_bl <- dplyr::filter(idat_agd_contrast, is.na(.data$.y))
    dat_agd_contrast_nonbl <- dplyr::filter(dat_agd_contrast, !is.na(.data$.y))
    idat_agd_contrast_nonbl <- dplyr::filter(idat_agd_contrast, !is.na(.data$.y))
    y_agd_contrast <- dplyr::filter(y_agd_contrast, !is.na(.data$.y))
  } else {
    dat_agd_contrast <- idat_agd_contrast <-
      dat_agd_contrast_bl <- idat_agd_contrast_bl <-
      dat_agd_contrast_nonbl <- idat_agd_contrast_nonbl <- tibble::tibble()
    y_agd_contrast <- NULL
    Sigma_agd_contrast <- NULL
  }

  # Combine
  idat_all <- dplyr::bind_rows(dat_ipd, idat_agd_arm, idat_agd_contrast_nonbl)
  idat_all_plus_bl <- dplyr::bind_rows(dat_ipd, idat_agd_arm, idat_agd_contrast)

  # Get sample sizes for centering
  if (((!is.null(regression) && !is_only_offset(regression)) || has_aux_regression) && center) {
    # Check that required variables are present in each data set, and non-missing
    if (!is.null(regression)) {
      check_regression_data(regression,
                            dat_ipd = dat_ipd,
                            dat_agd_arm = idat_agd_arm,
                            dat_agd_contrast = idat_agd_contrast)
    }

    if (has_aux_regression) {
      check_regression_data(aux_regression,
                            dat_ipd = dat_ipd,
                            dat_agd_arm = idat_agd_arm,
                            dat_agd_contrast = idat_agd_contrast)
    }

    # If IPD or IPD+AgD use weighted means for centering, otherwise with only AgD use raw mean

    if (has_ipd(network) && (has_agd_arm(network) || has_agd_contrast(network)) && !has_agd_sample_size(network))
      abort(paste("AgD study sample sizes not specified in network, cannot calculate centering values.",
                  "Specify `sample_size` in set_agd_*(), or set center = FALSE.", sep = "\n"))

    if (has_agd_arm(network)) {
      if (has_ipd(network)) {
        N_agd_arm <- network$agd_arm[[".sample_size"]]
      } else {
        N_agd_arm <- rep(n_int, nrow(network$agd_arm))
      }
    } else {
      N_agd_arm <- NULL
    }

    if (has_agd_contrast(network)) {
      if (has_ipd(network)) {
        N_agd_contrast <- network$agd_contrast[[".sample_size"]]
      } else {
        N_agd_contrast <- rep(n_int, nrow(network$agd_contrast))
      }
    } else {
      N_agd_contrast <- NULL
    }

    # Apply weights across integration points
    if (has_agd_arm(network) && network$outcome$agd_arm == "survival") {
      wts <- c(rep(1, nrow(dat_ipd)),
               rep(1 / n_int, nrow(idat_agd_arm)),
               rep(N_agd_contrast / n_int, each = n_int))
    } else {
      wts <- c(rep(1, nrow(dat_ipd)),
               rep(N_agd_arm / n_int, each = n_int),
               rep(N_agd_contrast / n_int, each = n_int))
    }

    # Center numeric columns used in regression model

    if (!is.null(regression)) {
      reg_names <- all.vars(regression)

      # Centering of the baseline risk is dealt with separately (`xbar_mu`)
      reg_names <- setdiff(reg_names, ".mu")

      # Ignore any variable(s) used as offset(s)
      reg_terms <- terms(regression)

      if (length(attr(reg_terms, "offset"))) {
        off_terms <- rownames(attr(reg_terms, "factors"))[attr(reg_terms, "offset")]
        off_names <- all.vars(as.formula(paste("~", off_terms, sep = "+")))
        reg_names <- setdiff(reg_names, off_names)
      }
    } else {
      reg_names <- character()
    }

    if (has_aux_regression) {
      reg_names <- unique(c(reg_names, all.vars(aux_regression)))
    }

    reg_numeric <- purrr::map_lgl(idat_all[, reg_names], is.numeric)

    # Take weighted mean of all rows (including baseline rows for contrast data)
    if (any(reg_numeric)) {
      xbar <- purrr::map_dbl(idat_all_plus_bl[, reg_names[reg_numeric]], weighted.mean, w = wts)
    } else {
      xbar <- NULL
    }

  } else {
    xbar <- NULL
  }

  xbar_mu <- if (".mu" %in% all.vars(regression)) calculate_baseline_risk(network, link) else NULL

  # Make NMA formula
  nma_formula <- make_nma_formula(regression,
                                  consistency = consistency,
                                  classes = !is.null(network$classes),
                                  class_interactions = class_interactions)

  # Construct model matrix
  X_list <- make_nma_model_matrix(nma_formula = nma_formula,
                                  dat_ipd = dat_ipd,
                                  dat_agd_arm = idat_agd_arm,
                                  dat_agd_contrast = idat_agd_contrast,
                                  agd_contrast_bl = is.na(idat_agd_contrast$.y),
                                  xbar = xbar,
                                  consistency = consistency,
                                  nodesplit = nodesplit,
                                  classes = !is.null(network$classes))

  X_ipd <- X_list$X_ipd
  X_agd_arm <- X_list$X_agd_arm
  X_agd_contrast <- X_list$X_agd_contrast

  offset_ipd <- X_list$offset_ipd
  offset_agd_arm <- X_list$offset_agd_arm
  offset_agd_contrast <- X_list$offset_agd_contrast

  # Construct RE correlation matrix
  if (trt_effects == "random") {

    # Get study/treatment data
    if (has_ipd(network)) {
      tdat_ipd_arm <- dplyr::distinct(dat_ipd, .data$.study, .data$.trt)
    } else {
      tdat_ipd_arm <- tibble::tibble()
    }

    if (has_agd_arm(network)) {
      tdat_agd_arm <- dplyr::select(dat_agd_arm, ".study", ".trt")
    } else {
      tdat_agd_arm <- tibble::tibble()
    }

    if (has_agd_contrast(network)) {
      tdat_agd_contrast_nonbl <- dplyr::select(dat_agd_contrast_nonbl, ".study", ".trt")
    } else {
      tdat_agd_contrast_nonbl <- tibble::tibble()
    }

    tdat_all <- dplyr::bind_rows(tdat_ipd_arm, tdat_agd_arm, tdat_agd_contrast_nonbl)

    contr <- rep(c(FALSE, FALSE, TRUE),
                 times = c(nrow(tdat_ipd_arm), nrow(tdat_agd_arm), nrow(tdat_agd_contrast_nonbl)))

    if (consistency %in% c("consistency", "nodesplit")) {
      .RE_cor <- RE_cor(tdat_all$.study, tdat_all$.trt, contrast = contr, type = "reftrt")
      .which_RE <- which_RE(tdat_all$.study, tdat_all$.trt, contrast = contr, type = "reftrt")
    } else if (consistency == "ume") {
      .RE_cor <- RE_cor(tdat_all$.study, tdat_all$.trt, contrast = contr, type = "blshift")
      .which_RE <- which_RE(tdat_all$.study, tdat_all$.trt, contrast = contr, type = "blshift")
    } else {
      abort(glue::glue("Inconsistency '{consistency}' model not yet supported."))
    }
  } else {
    .RE_cor <- NULL
    .which_RE <- NULL
  }

  # Set up spline basis for mspline and pexp models
  if (likelihood %in% c("mspline", "pexp") && (has_ipd(network) || has_agd_arm(network))) {
    require_pkg("splines2")

    survdat <-
      if (!has_agd_arm(network)) {
        dplyr::tibble(.Surv = y_ipd$.Surv,
                      .study = forcats::fct_drop(dat_ipd$.study),
                      observed = .data$.Surv[, "status"]  == 1)
      } else if (!has_ipd(network)) {
        dplyr::tibble(.Surv = y_agd_arm$.Surv,
                      .study = forcats::fct_drop(rep(dat_agd_arm$.study, times = dat_agd_arm$.sample_size)),
                      observed = .data$.Surv[, "status"]  == 1)
      } else {
        dplyr::tibble(.Surv = c(y_ipd$.Surv, y_agd_arm$.Surv),
                      .study = forcats::fct_drop(forcats::fct_c(dat_ipd$.study, rep(dat_agd_arm$.study, times = dat_agd_arm$.sample_size))),
                      observed = .data$.Surv[, "status"]  == 1)
      }

    survdat <- dplyr::mutate(survdat, !!! get_Surv_data(survdat$.Surv))


    if (!is.null(mspline_basis)) {

      if (!is.list(mspline_basis) || any(!purrr::map_lgl(mspline_basis, is_mspline)))
        abort("`mspline_basis` must be a named list of M-spline bases created using splines2::mSpline().")

      missing_names <- setdiff(levels(survdat$.study), names(mspline_basis))
      if (length(missing_names) > 0)
        abort(glue::glue("`mspline_basis` must be a named list of M-spline basis for each study.\n",
                         "Missing {if (length(missing_names) > 1) 'bases' else 'basis'} for stud{if (length(missing_names) > 1) 'ies' else 'y'} ",
                         glue::glue_collapse(glue::double_quote(missing_names), sep = ", ", last = " and ", width = 30), "."))

      if (!all(purrr::map_int(mspline_basis, ncol) == ncol(mspline_basis[[1]])))
        abort("Each basis in `mspline_basis` must currently have the same number of knots.")

      if (likelihood == "pexp" && any(purrr::map_lgl(mspline_basis, ~attr(., "degree") != 0)))
        abort("`mspline_basis` for piecewise exponential model must all have degree = 0.")

      basis <- mspline_basis

    } else {

      if (likelihood == "pexp") mspline_degree <- 0

      if (!rlang::is_scalar_integerish(mspline_degree, finite = TRUE) || mspline_degree < 0)
        abort("`mspline_degree` must be a single non-negative integer.")

      if (is.null(knots)) {

        knots <- make_knots(network, n_knots = n_knots, type = if (!is.null(aux_regression)) "quantile_common" else "quantile")

      } else {  # User-provided knots
        # Check required format
        if (!has_aux_regression) {
          if ((!is.list(knots) || any(!purrr::map_lgl(knots, is.numeric))) && !is.vector(knots, mode = "numeric"))
            abort("`knots` must be a named list of numeric vectors giving the knot locations for each study, or a single numeric vector of knot locations to use for all studies.")

          if (is.vector(knots, mode = "numeric")) {
            knots <- rep_len(list(knots), dplyr::n_distinct(survdat$.study))
            names(knots) <- unique(survdat$.study)
          }

          missing_names <- setdiff(levels(survdat$.study), names(knots))
          if (length(missing_names) > 0)
            abort(glue::glue("`knots` must be a named list of numeric vectors giving the knot locations for each study.\n",
                             "Missing knot location vector{if (length(missing_names) > 1) 's' else ''} for stud{if (length(missing_names) > 1) 'ies' else 'y'} ",
                             glue::glue_collapse(glue::double_quote(missing_names), sep = ", ", last = " and ", width = 30), "."))

          if (!all(purrr::map_int(knots, length) == length(knots[[1]])))
            abort("Each vector in `knots` must currently be the same length (each study must have the same number of knots).")

          if (length(knots[[1]]) < 3)
            abort("Each vector in `knots` must be at least length 3 (boundary knots and one internal knot).")
        } else {
          # With aux_regression, only a single vector of knot locations is allowed
          if (!is.vector(knots, mode = "numeric"))
            abort("`knots` must be a single numeric vector of knot locations, shared for all studies, when `aux_regression` is specified.")

          if (length(knots) < 3)
            abort("`knots` must be at least length 3 (boundary knots and one internal knot).")

          knots <- rep_len(list(knots), dplyr::n_distinct(survdat$.study))
          names(knots) <- unique(survdat$.study)
        }
      }

      # Set up basis
      # Only evaluate at first boundary knot for now to save time/memory
      knots <- purrr::map(knots, sort)
      knots <- tibble::as_tibble(knots)
      b_knots <- knots[c(1, nrow(knots)), ]
      i_knots <- knots[-c(1, nrow(knots)), ]
      basis <- purrr::imap(b_knots,
                           ~withCallingHandlers(splines2::mSpline(.x[1],
                                                       knots = i_knots[[.y]],
                                                       Boundary.knots = .x,
                                                       degree = mspline_degree,
                                                       intercept = TRUE),
                                     error = function(e) abort(glue::glue("Could not create spline basis for study {glue::double_quote(.y)}."),
                                                               parent = e),
                                     warning = function(w) {
                                       warn(glue::glue("Warning while creating spline basis for study {glue::double_quote(.y)}."), parent = w)
                                       rlang::cnd_muffle(w)
                                     }
                                    )
                           )

    }

    # Ensure list is in factor order
    basis <- basis[levels(survdat$.study)]

  } else {
    basis <- NULL
  }

  # Set up aux_by design vector
  if (has_aux_by) {
    # Determine whether we need to integrate over the aux regression too
    aux_int <- aux_needs_integration(aux_regression = aux_regression, aux_by = aux_by)

    if (aux_int) {
      aux_dat <- dplyr::bind_rows(dat_ipd, idat_agd_arm)
    } else {
      aux_dat <- dplyr::bind_rows(dat_ipd,
                                  if (has_agd_arm(network)) dplyr::select(tidyr::unnest(dat_agd_arm, cols = ".Surv"), -".Surv") else NULL)
    }
    aux_id <- get_aux_id(aux_dat, aux_by)
  } else {
    aux_id <- integer()
    aux_int <- FALSE
  }

  # Set up aux_regression design matrix
  if (has_aux_regression) {
    X_aux <- make_nma_model_matrix(aux_regression,
                                   dat_ipd = aux_dat,
                                   xbar = xbar)$X_ipd

    # Group common rows of X_aux for efficiency if possible
    if (aux_int && use_int && has_agd_arm(network)) {
      aux_group <- 1:nrow(X_aux)
    } else {
      X_aux_dat <- as.data.frame(X_aux)
      if (!rlang::has_name(X_aux_dat, ".study")) X_aux_dat <- dplyr::mutate(X_aux_dat, aux_dat$.study, .before = 0)  # Always group within study
      aux_group <- dplyr::group_indices(dplyr::group_by_all(X_aux_dat))
    }
  } else {
    X_aux <- NULL
    aux_group <- aux_id
  }

if (class_effects == "exchangeable") {
  # Create class design vector for class means
  class_mean_design <- which_CE(network$classes, class_sd)

  # Create class design vector for class SDs
  if (is.list(class_sd)) {
    class_sd_design <- which_CE(forcats::fct_collapse(network$classes, !!!class_sd), class_sd)
  } else if (class_sd == "common") {
    class_sd_design <- list(
      # Change non-zero class IDs to 1
      id = pmin(class_mean_design$id, 1),
      # Set common class label
      label = "All Classes"
    )
  } else if (class_sd == "independent") {
    class_sd_design <- class_mean_design
  }
}
  # Fit using nma.fit
  stanfit <- nma.fit(ipd_x = X_ipd, ipd_y = y_ipd,
    agd_arm_x = X_agd_arm, agd_arm_y = y_agd_arm,
    agd_contrast_x = X_agd_contrast, agd_contrast_y = y_agd_contrast,
    agd_contrast_Sigma = Sigma_agd_contrast,
    n_int = n_int,
    ipd_offset = offset_ipd,
    agd_arm_offset = offset_agd_arm,
    agd_contrast_offset = offset_agd_contrast,
    trt_effects = trt_effects,
    xbar_mu = xbar_mu,
    RE_cor = .RE_cor,
    which_RE = .which_RE,
    class_effects = class_effects,
    which_CE = if (class_effects == "exchangeable") class_mean_design$id else NULL,
    which_CE_sd = if (class_effects == "exchangeable") class_sd_design$id else NULL,
    likelihood = likelihood,
    link = link,
    consistency = consistency,
    ...,
    prior_intercept = prior_intercept,
    prior_trt = prior_trt,
    prior_het = prior_het,
    prior_het_type = prior_het_type,
    prior_reg = prior_reg,
    prior_aux = prior_aux,
    prior_aux_reg = prior_aux_reg,
    prior_class_mean = prior_class_mean,
    prior_class_sd = prior_class_sd,
    aux_id = aux_id,
    aux_group = aux_group,
    X_aux = X_aux,
    QR = QR,
    adapt_delta = adapt_delta,
    int_thin = int_thin,
    int_check = int_check,
    basis = basis)

  # Make readable parameter names for generated quantities
  fnames_oi <- stanfit@sim$fnames_oi

  # Labels for fitted values
  ipd_data_labels <- if (has_ipd(network)) make_data_labels(dat_ipd$.study, dat_ipd$.trt) else NULL

  if (has_agd_arm(network)) {
    if (network$outcome$agd_arm != "survival") {
      agd_arm_data_labels <-  make_data_labels(dat_agd_arm$.study, dat_agd_arm$.trt)
    } else {
      agd_arm_data_labels <-  make_data_labels(rep(dat_agd_arm$.study, times = dat_agd_arm$.sample_size),
                                               rep(dat_agd_arm$.trt, times = dat_agd_arm$.sample_size))
    }
  } else {
    agd_arm_data_labels <- NULL
  }

  if (has_agd_contrast(network)) {
    dat_agd_contrast_nonbl <-
      dplyr::left_join(dat_agd_contrast_nonbl,
                       dplyr::transmute(dat_agd_contrast_bl, .data$.study, .trt_b = .data$.trt),
                       by = ".study")

    agd_contrast_data_labels <- make_data_labels(dat_agd_contrast_nonbl$.study,
                                                 dat_agd_contrast_nonbl$.trt,
                                                 dat_agd_contrast_nonbl$.trt_b)
  } else {
    agd_contrast_data_labels <- NULL
  }

  if (has_ipd(network))
    fnames_oi[grepl("^fitted_ipd\\[[0-9]+\\]$", fnames_oi)] <- paste0("fitted_ipd[", ipd_data_labels, "]")

  if (has_agd_arm(network))
    fnames_oi[grepl("^fitted_agd_arm\\[[0-9]+\\]$", fnames_oi)] <- paste0("fitted_agd_arm[", agd_arm_data_labels, "]")

  if (has_agd_contrast(network))
    fnames_oi[grepl("^fitted_agd_contrast\\[[0-9]+\\]$", fnames_oi)] <- paste0("fitted_agd_contrast[", agd_contrast_data_labels, "]")

  # Labels for RE deltas
  if (trt_effects == "random") {

    if (consistency == "ume") { # Baseline shift models (currently only UME)
      if (has_ipd(network)) {
        ipd_arms <- dplyr::distinct(dat_ipd, .data$.study, .data$.trt) %>%
          dplyr::group_by(.data$.study) %>%
          dplyr::mutate(.trt_b = sort(.data$.trt)[1])
        ipd_delta_labels <- make_data_labels(ipd_arms$.study, ipd_arms$.trt, ipd_arms$.trt_b)
      } else {
        ipd_delta_labels <- NULL
      }

      if (has_agd_arm(network)) {
        agd_arm_trt_b <- dat_agd_arm %>%
          dplyr::group_by(.data$.study) %>%
          dplyr::mutate(.trt_b = sort(.data$.trt)[1]) %>%
          dplyr::pull(.data$.trt_b)
        agd_arm_delta_labels <- make_data_labels(dat_agd_arm$.study, dat_agd_arm$.trt, agd_arm_trt_b)
      } else {
        agd_arm_delta_labels <- NULL
      }

    } else { # Reference treatment models
      if (has_ipd(network)) {
        ipd_arms <- dplyr::distinct(dat_ipd, .data$.study, .data$.trt)
        ipd_delta_labels <- make_data_labels(ipd_arms$.study, ipd_arms$.trt)
      } else {
        ipd_delta_labels <- NULL
      }
      agd_arm_delta_labels <- if (has_agd_arm(network)) make_data_labels(dat_agd_arm$.study, dat_agd_arm$.trt) else NULL
    }

    agd_contrast_delta_labels <- agd_contrast_data_labels

    delta_labels <- c(ipd_delta_labels,
                      agd_arm_delta_labels,
                      agd_contrast_delta_labels)[.which_RE > 0]

    fnames_oi[grepl("^delta\\[[0-9]+\\]$", fnames_oi)] <- paste0("delta[", delta_labels, "]")
  }

  # Labels for log_lik, resdev (only one entry per AgD contrast study)
  dev_labels <- c(ipd_data_labels,
                  agd_arm_data_labels,
                  if (has_agd_contrast(network)) as.character(dat_agd_contrast_bl$.study) else NULL)

  fnames_oi[grepl("^log_lik\\[[0-9]+\\]$", fnames_oi)] <- paste0("log_lik[", dev_labels, "]")
  fnames_oi[grepl("^resdev\\[[0-9]+\\]$", fnames_oi)] <- paste0("resdev[", dev_labels, "]")

  # Labels for cumulative integration points
  if (inherits(network, "mlnmr_data") && (has_agd_arm(network) || has_agd_contrast(network)) && int_thin > 0) {
    n_int_thin <- n_int %/% int_thin

    if (has_agd_arm(network)) {
      agd_arm_cumint_labels <- rep(agd_arm_data_labels, each = n_int_thin)
      agd_arm_cumint_labels <- paste0(agd_arm_cumint_labels, ", ", rep_len(1:n_int_thin * int_thin, length.out = length(agd_arm_cumint_labels)))

      if (likelihood == "ordered") {
        l_cat <- colnames(y_agd_arm$.r)
        agd_arm_cumint_labels <- paste0(rep(agd_arm_cumint_labels, times = length(l_cat)), ", ", rep(l_cat, each = length(agd_arm_cumint_labels)))
      }

      fnames_oi[grepl("^theta_bar_cum_agd_arm\\[", fnames_oi)] <- paste0("theta_bar_cum_agd_arm[", agd_arm_cumint_labels, "]")
      if (likelihood %in% c("bernoulli2", "binomial2"))
        fnames_oi[grepl("^theta2_bar_cum\\[[0-9]+\\]$", fnames_oi)] <- paste0("theta2_bar_cum[", agd_arm_cumint_labels, "]")
    }

    if (has_agd_contrast(network)) {
      agd_contrast_cumint_labels <- rep(agd_contrast_data_labels, each = n_int_thin)
      agd_contrast_cumint_labels <- paste0(agd_contrast_cumint_labels, ", ", rep_len(1:n_int_thin * int_thin, length.out = length(agd_contrast_cumint_labels)))

      fnames_oi[grepl("^theta_bar_cum_agd_contrast\\[[0-9]+\\]$", fnames_oi)] <- paste0("theta_bar_cum_agd_contrast[", agd_contrast_cumint_labels, "]")
    }

  }

  # Labels for survival aux pars
  if (likelihood %in% valid_lhood$survival && has_aux) {
    aux_labels <- get_aux_labels(aux_dat, by = aux_by)

    fnames_oi[grepl("^shape\\[[0-9]+\\]$", fnames_oi)] <- paste0("shape[", aux_labels, "]")
    fnames_oi[grepl("^sdlog\\[[0-9]+\\]$", fnames_oi)] <- paste0("sdlog[", aux_labels, "]")
    fnames_oi[grepl("^sigma\\[[0-9]+\\]$", fnames_oi)] <- paste0("sigma[", aux_labels, "]")
    fnames_oi[grepl("^k\\[[0-9]+\\]$", fnames_oi)] <- paste0("k[", aux_labels, "]")

    if (likelihood %in% c("mspline", "pexp")) {
      # Number of spline coefficients
      n_scoef <- ncol(basis[[1]])

      fnames_oi[grepl("^scoef\\[[0-9]+,[0-9]+\\]$", fnames_oi)] <-
        paste0("scoef[", rep(aux_labels, times = n_scoef), ", ", rep(1:n_scoef, each = length(aux_labels)), "]")
    }
  }

  if (class_effects == "exchangeable"){
    # Label class_mean parameters
    fnames_oi[grepl("^class_mean\\[[0-9]+\\]$", fnames_oi)] <- paste0("class_mean[", class_mean_design$label, "]")

    # Label class_sd parameters
    fnames_oi[grepl("^class_sd\\[[0-9]+\\]$", fnames_oi)] <- paste0("class_sd[", class_sd_design$label, "]")
    network$class_sd <- class_sd_design$label
}
  stanfit@sim$fnames_oi <- fnames_oi

  # Create stan_nma object
  out <- list(network = network,
              stanfit = stanfit,
              trt_effects = trt_effects,
              consistency = consistency,
              regression = regression,
              aux_regression = aux_regression,
              class_interactions = if (!is.null(regression) && !is.null(network$classes)) class_interactions else NULL,
              class_effects = class_effects,
              class_sd = if (class_effects == "exchangeable") class_sd else NULL,
              xbar = c(xbar, .mu = xbar_mu),
              likelihood = likelihood,
              link = link,
              aux_by = if (has_aux_by) colnames(get_aux_by_data(aux_dat, by = aux_by)) else NULL,
              priors = list(prior_intercept = if (has_intercepts) prior_intercept else NULL,
                            prior_trt = prior_trt,
                            prior_class_mean = if (class_effects == "exchangeable") prior_class_mean else NULL,
                            prior_class_sd = if (class_effects == "exchangeable") prior_class_sd else NULL,
                            prior_het = if (trt_effects == "random") prior_het else NULL,
                            prior_het_type = if (trt_effects == "random") prior_het_type else NULL,
                            prior_reg = if (!is.null(regression) && !is_only_offset(regression)) prior_reg else NULL,
                            prior_aux = if (has_aux) prior_aux else NULL,
                            prior_aux_reg = if (has_aux_regression) prior_aux_reg else NULL))

  if (likelihood %in% c("mspline", "pexp")) out$basis <- basis

  if (inherits(network, "mlnmr_data")) class(out) <- c("stan_mlnmr", "stan_nma")
  else class(out) <- "stan_nma"

  if (likelihood %in% valid_lhood$survival) class(out) <- c("stan_nma_surv", class(out))

  if (consistency == "nodesplit" && !is.data.frame(nodesplit)) {
    class(out) <- c("nma_nodesplit", class(out))
    out$nodesplit <- nodesplit
  }

  return(out)
}


#' @param ipd_x Design matrix for IPD studies
#' @param ipd_y Outcome data frame for IPD studies
#' @param agd_arm_x  Design matrix for AgD studies (arm-based)
#' @param agd_arm_y  Outcome data frame for AgD studies (arm-based)
#' @param agd_contrast_x  Design matrix for AgD studies (contrast-based)
#' @param agd_contrast_y  Outcome data frame for AgD studies (contrast-based)
#' @param agd_contrast_Sigma List of covariance matrices for contrast-based data
#' @param n_int Number of numerical integration points used
#' @param ipd_offset Vector of offset values for IPD
#' @param agd_arm_offset Vector of offset values for AgD (arm-based)
#' @param agd_contrast_offset Vector of offset values for AgD (contrast-based)
#' @param RE_cor Random effects correlation matrix, when `trt_effects = "random"`
#' @param which_RE Random effects design vector, when `trt_effects = "random"`
#' @param which_CE Class effects means design vector (0 = no class)
#' @param which_CE_sd Class effects SDs design vector (0 = no class)
#' @param basis Spline basis for `mspline` and `pexp` models
#'
#' @noRd
nma.fit <- function(ipd_x, ipd_y,
                    agd_arm_x, agd_arm_y,
                    agd_contrast_x, agd_contrast_y, agd_contrast_Sigma,
                    n_int,
                    ipd_offset = NULL, agd_arm_offset = NULL, agd_contrast_offset = NULL,
                    trt_effects = c("fixed", "random"),
                    xbar_mu = NULL,
                    RE_cor = NULL,
                    which_RE = NULL,
                    class_effects = c("independent", "exchangeable", "common"),
                    which_CE = NULL,
                    which_CE_sd = NULL,
                    likelihood = NULL,
                    link = NULL,
                    consistency = c("consistency", "ume", "nodesplit"),
                    ...,
                    prior_intercept,
                    prior_trt,
                    prior_het,
                    prior_het_type = c("sd", "var", "prec"),
                    prior_reg,
                    prior_aux,
                    prior_aux_reg,
                    prior_class_mean,
                    prior_class_sd,
                    aux_id = integer(),
                    aux_group = integer(),
                    X_aux = NULL,
                    QR = FALSE,
                    adapt_delta = NULL,
                    int_thin = 0,
                    int_check = TRUE,
                    basis) {

  if (missing(ipd_x)) ipd_x <- NULL
  if (missing(ipd_y)) ipd_y <- NULL
  if (missing(agd_arm_x)) agd_arm_x <- NULL
  if (missing(agd_arm_y)) agd_arm_y <- NULL
  if (missing(agd_contrast_x)) agd_contrast_x <- NULL
  if (missing(agd_contrast_y)) agd_contrast_y <- NULL
  if (missing(agd_contrast_Sigma)) agd_contrast_Sigma <- NULL

  # Check available x and y
  if (xor(is.null(ipd_x), is.null(ipd_y)))
    abort("`ipd_x` and `ipd_y` should both be present or both NULL.")
  if (xor(is.null(agd_arm_x), is.null(agd_arm_y)))
    abort("`agd_arm_x` and `agd_arm_y` should both be present or both NULL.")
  if (xor(is.null(agd_contrast_x), is.null(agd_contrast_y)) || xor(is.null(agd_contrast_x), is.null(agd_contrast_Sigma)))
    abort("`agd_contrast_x`, `agd_contrast_y`, `agd_contrast_Sigma` should all be present or all NULL.")

  has_ipd <- !is.null(ipd_x) && !is.null(ipd_y)
  has_agd_arm <- !is.null(agd_arm_x) && !is.null(agd_arm_y)
  has_agd_contrast <-  !is.null(agd_contrast_x) && !is.null(agd_contrast_y) && !is.null(agd_contrast_Sigma)

  # Ignore n_int if no AgD
  if (!has_agd_arm && !has_agd_contrast) n_int <- 1
  # Check n_int
  if (!rlang::is_scalar_integerish(n_int) ||
      n_int < 1) abort("`n_int` should be an integer >= 1.")

  # Check design matrices, outcomes
  if (has_ipd) {
    if (!is.matrix(ipd_x) || !is.numeric(ipd_x))
      abort("`ipd_x` should be a numeric matrix.")
    if (any(!purrr::map_lgl(ipd_y, is.numeric)))
      abort("`ipd_y` should be numeric outcome data.")
    if (nrow(ipd_x) != nrow(ipd_y))
      abort("Number of rows in `ipd_x` and `ipd_y` do not match.")
  }
  if (has_agd_arm) {
    if (!is.matrix(agd_arm_x) || !is.numeric(agd_arm_x))
      abort("`agd_arm_x` should be a numeric matrix.")
    if (any(!purrr::map_lgl(agd_arm_y, is.numeric)))
      abort("`agd_arm_y` should be numeric outcome data.")
    if (nrow(agd_arm_x) != nrow(agd_arm_y) * n_int)
      abort("Number of rows in `agd_arm_x`, `agd_arm_y`, and `n_int` do not match.")
  }
  if (has_agd_contrast) {
    if (!is.matrix(agd_contrast_x) || !is.numeric(agd_contrast_x))
      abort("`agd_contrast_x` should be a numeric matrix.")
    if (any(!purrr::map_lgl(agd_contrast_y, is.numeric)))
      abort("`agd_contrast_y` should be numeric outcome data.")
    if (nrow(agd_contrast_x) != nrow(agd_contrast_y) * n_int)
      abort("Number of rows in `agd_contrast_x`, `agd_contrast_y`, and `n_int` do not match.")
    if (!is.list(agd_contrast_Sigma) || any(purrr::map_lgl(agd_contrast_Sigma, ~!is.numeric(.))))
      abort("`agd_contrast_Sigma` should be a list of covariance matrices, of length equal to the number of AgD (contrast-based) studies.")
  }

  # Check offsets
  has_ipd_offset <- !is.null(ipd_offset)
  has_agd_arm_offset <- !is.null(agd_arm_offset)
  has_agd_contrast_offset <- !is.null(agd_contrast_offset)

  if (has_ipd_offset && !has_ipd)
    abort("`ipd_offset` given but not `ipd_x` and `ipd_y`.")
  if (has_agd_arm_offset && !has_agd_arm)
    abort("`agd_arm_offset` given but not `agd_arm_x` and `agd_arm_y`.")
  if (has_agd_contrast_offset && !has_agd_contrast)
    abort("`agd_contrast_offset` given but not `agd_contrast_x` and `agd_contrast_y`.")

  has_offsets <- any(has_ipd_offset, has_agd_arm_offset, has_agd_contrast_offset)
  if (has_offsets &&
      !all(has_ipd_offset[has_ipd],
           has_agd_arm_offset[has_agd_arm],
           has_agd_contrast_offset[has_agd_contrast]))
    abort("Offsets provided for some data sources but not all.")

  if (has_ipd_offset && (!is.numeric(ipd_offset) || length(ipd_offset) != nrow(ipd_x)))
    abort("`ipd_offset` should be a numeric vector with length matching `ipd_x`, `ipd_y`")
  if (has_agd_arm_offset && (!is.numeric(agd_arm_offset) || length(agd_arm_offset) != nrow(agd_arm_x)))
    abort("`agd_arm_offset` should be a numeric vector with length matching `agd_arm_x`, `agd_arm_y`")
  if (has_agd_contrast_offset && (!is.numeric(agd_contrast_offset) || length(agd_contrast_offset) != nrow(agd_contrast_x)))
    abort("`agd_contrast_offset` should be a numeric vector with length matching `agd_contrast_x`, `agd_contrast_y`")

  # Check matching X column names and dimensions if more than one present
  if ((has_ipd &&
       ((has_agd_arm && !identical(colnames(ipd_x), colnames(agd_arm_x))) ||
        (has_agd_contrast && !identical(colnames(ipd_x), colnames(agd_contrast_x))))) ||
      (has_agd_arm &&
       (has_agd_contrast && !identical(colnames(agd_arm_x), colnames(agd_contrast_x)))))
    abort("Non-matching columns in *_x matrices.")

  # Check model arguments
  trt_effects <- rlang::arg_match(trt_effects)

  # Check class effect arguments
  class_effects <- rlang::arg_match(class_effects)

  if (class_effects == "exchangeable") {
    if (is.null(which_CE) || !rlang::is_integerish(which_CE) || any(which_CE < 0))
      abort("`which_CE` must be an integer design vector for class effects.")
    if (is.null(which_CE_sd) || !rlang::is_integerish(which_CE_sd) || any(which_CE_sd < 0))
      abort("`which_CE_sd` must be an integer design vector for class effect SDs.")
  }

  likelihood <- check_likelihood(likelihood)
  link <- check_link(link, likelihood)

  # When are priors on auxiliary parameters required?
  has_aux <- (likelihood == "normal" && has_ipd) ||
    (likelihood %in% c("ordered", setdiff(valid_lhood$survival, c("exponential", "exponential-aft"))) &&
       (has_ipd || has_agd_arm))

  # Check priors
  check_prior(prior_intercept)
  check_prior(prior_trt)
  if (trt_effects == "random") check_prior(prior_het)
  check_prior(prior_reg)
  if (has_aux) {
    if (likelihood == "gengamma") check_prior(prior_aux, c("sigma", "k"))
    else check_prior(prior_aux)

    if (!is.null(X_aux)) check_prior(prior_aux_reg)
  }
  if (class_effects == "exchangeable"){
  check_prior(prior_class_mean)
  check_prior(prior_class_sd)
} else {
  # Dummy class effects priors for non-CE models, not used but requested by Stan data
  prior_class_mean <- normal(0, 1)
  prior_class_sd <- half_normal(1)
}
  prior_het_type <- rlang::arg_match(prior_het_type)

  # Dummy RE prior for FE model, not used but requested by Stan data
  if (trt_effects == "fixed") prior_het <- half_normal(1)

  # Check other args
  if (!rlang::is_bool(QR)) abort("`QR` should be a logical scalar (TRUE or FALSE).")
  if (!rlang::is_scalar_integerish(int_thin) ||
      int_thin < 0) abort("`int_thin` should be an integer >= 0.")
  if (!rlang::is_bool(int_check)) abort("`int_check` should be a logical scalar (TRUE or FALSE).")
  if (int_thin > 0) int_check <- FALSE

  # Set adapt_delta
  if (is.null(adapt_delta)) {
    adapt_delta <- switch(trt_effects, fixed = 0.8, random = 0.95)
  } else if (!is.numeric(adapt_delta) ||
             length(adapt_delta) > 1 ||
             adapt_delta <= 0 || adapt_delta >= 1) abort("`adapt_delta` should be a  numeric value in (0, 1).")

  # Is this a survival outcome?
  is_survival <- likelihood %in% valid_lhood$survival

  # Pull study and treatment details from *_x
  if (has_ipd) x_names <- colnames(ipd_x)
  else if (has_agd_arm) x_names <- colnames(agd_arm_x)
  else if (has_agd_contrast) x_names <- colnames(agd_contrast_x)

  col_study <- grepl("^\\.study[^:]+$", x_names)
  col_trt <- grepl("^(\\.trt|\\.contr)[^:]+$", x_names)
  col_omega <- x_names == ".omegaTRUE"
  col_reg <- !col_study & !col_trt & !col_omega
  col_brmr <- col_reg & grepl("(^\\.mu\\:)|(\\:\\.mu$)", x_names)

  n_trt <- sum(col_trt) + 1

  get_study <- function(x) which(x == 1)
  get_trt <- function(x, v = 1) if (any(x == v)) which(x == v) + 1 else 1

  if (has_ipd) {
    ipd_s_t_all <- dplyr::tibble(.study = unname(apply(ipd_x[, col_study, drop = FALSE], 1, get_study)),
                                 .trt = unname(apply(ipd_x[, col_trt, drop = FALSE], 1, get_trt)))
    ipd_s_t <- dplyr::distinct(ipd_s_t_all) %>% dplyr::mutate(.arm = 1:dplyr::n())
    ipd_arm <-  dplyr::left_join(ipd_s_t_all, ipd_s_t, by = c(".study", ".trt")) %>% dplyr::pull(.data$.arm)
    ipd_study <- ipd_s_t$.study
    ipd_trt <- ipd_s_t$.trt
    narm_ipd <- max(ipd_arm)
    ni_ipd <- nrow(ipd_x)
  } else {
    ipd_s_t_all <- dplyr::tibble(.study = integer(), .trt = integer())
    ipd_study <- ipd_trt <- ipd_arm <- numeric()
    ni_ipd <- 0
    narm_ipd <- 0
  }

  if (has_agd_arm) {
    ni_agd_arm <- nrow(agd_arm_y)
    aa1 <- 0:(ni_agd_arm - 1)*n_int + 1
    agd_arm_study <- apply(agd_arm_x[aa1, col_study, drop = FALSE], 1, get_study)
    agd_arm_trt <- apply(agd_arm_x[aa1, col_trt, drop = FALSE], 1, get_trt)

    if (!is_survival) {
      narm_agd_arm <- ni_agd_arm
    } else {
      agd_arm_s_t_all <- dplyr::tibble(.study = agd_arm_study, .trt = agd_arm_trt)
      agd_arm_s_t <- dplyr::distinct(agd_arm_s_t_all) %>% dplyr::mutate(.arm = 1:dplyr::n())
      agd_arm_arm <-  dplyr::left_join(agd_arm_s_t_all, agd_arm_s_t, by = c(".study", ".trt")) %>% dplyr::pull(.data$.arm)
      agd_arm_study <- agd_arm_s_t$.study
      agd_arm_trt <- agd_arm_s_t$.trt
      narm_agd_arm <- max(agd_arm_arm)
    }

  } else {
    agd_arm_s_t_all <- dplyr::tibble(.study = integer(), .trt = integer())
    agd_arm_study <- agd_arm_trt <- agd_arm_arm <- numeric()
    ni_agd_arm <- narm_agd_arm <- 0
  }

  if (has_agd_contrast) {
    ni_agd_contrast <- nrow(agd_contrast_y)
    ac1 <- 0:(ni_agd_contrast - 1)*n_int + 1
    agd_contrast_trt <- apply(agd_contrast_x[ac1, col_trt, drop = FALSE], 1, get_trt)
    agd_contrast_trt_b <- apply(agd_contrast_x[ac1, col_trt, drop = FALSE], 1, get_trt, v = -1)

    # Get number of contrast-based studies from length of Sigma list
    ns_agd_contrast <- length(agd_contrast_Sigma)

    # Construct block-diagonal contrast covariance matrix
    Sigma <- as.matrix(Matrix::bdiag(agd_contrast_Sigma))

    if (nrow(Sigma) != ni_agd_contrast)
      abort("Dimensions of `agd_contrast_Sigma` covariance matrices do not match the contrast-based data.")
  } else {
    agd_contrast_trt <- agd_contrast_trt_b <- numeric()
    Sigma <- matrix(1, 1, 1)
    ni_agd_contrast <- 0
    ns_agd_contrast <- 0
  }

  # Set up random effects
  if (trt_effects == "random") {
    narm <- narm_ipd + narm_agd_arm + ni_agd_contrast
    if (!is.null(which_RE)) {
      if (!rlang::is_integerish(which_RE) ||
          any(which_RE < 0) ||
          is.matrix(which_RE))
        abort("`which_RE` should be an integer vector.")
      if (length(which_RE) != narm)
        abort(glue::glue("Length of `which_RE` does not match the number of arms/contrasts.\n",
                         "Expecting length {narm}, instead length {length(which_RE)}."))
    } else {
      abort("Specify `which_RE` when trt_effects = 'random'.")
    }

    nRE <- sum(which_RE != 0)

    if (!is.null(RE_cor)) {
      if (!is.matrix(RE_cor) || !is.numeric(RE_cor))
        abort("`RE_cor` should be a numeric matrix.")
      if (any(dim(RE_cor) != c(nRE, nRE)))
        abort(glue::glue("Dimensions of `RE_cor` do not match the number of random effects.\n",
                         "Expecting [{nRE} x {nRE}], instead [{nrow(RE_cor)} x {ncol(RE_cor)}]."))
    } else {
      abort("Specify `RE_cor` when trt_effects = 'random'.")
    }
  } else {
    RE_cor <- matrix(1, 1, 1)
    which_RE <- integer()
  }

  # Make full design matrix
  X_all <- rbind(ipd_x, agd_arm_x, agd_contrast_x)

  # Make sure columns of X_all are in correct order (study, trt, omega (if nodesplit), regression terms)
  X_all <- cbind(X_all[, col_study, drop = FALSE],
                 X_all[, col_trt, drop = FALSE],
                 X_all[, col_omega, drop = FALSE],
                 X_all[, col_reg, drop = FALSE])

  # Take thin QR decomposition if QR = TRUE
  if (QR) {
    X_all_qr <- qr(X_all)
    X_all_Q <- qr.Q(X_all_qr) * sqrt(nrow(X_all) - 1)
    X_all_R <- qr.R(X_all_qr)[, sort.list(X_all_qr$pivot)] / sqrt(nrow(X_all) - 1)
    X_all_R_inv <- solve(X_all_R)
  }

  # Handle integration points
  dots <- list(...)
  nchains <- dots$chains
  if (is.null(nchains)) nchains <- 4
  if (nchains < 2) {
    warn("At least 2 chains are required to check integration convergence with int_check = TRUE.")
    int_check <- FALSE
  }
  nint_max <- n_int
  if (int_check && n_int > 1) {
    nint_vec <- c(rep(n_int, ceiling(nchains / 2)), rep(n_int %/% 2, floor(nchains / 2)))
  } else {
    nint_vec <- rep(n_int, nchains)
  }

  # Set common Stan data
  standat <- list(
    # Constants
    ns_ipd = length(unique(ipd_study)),
    ni_ipd = ni_ipd,
    ns_agd_arm = length(unique(agd_arm_study)),
    ni_agd_arm = ni_agd_arm,
    ns_agd_contrast = ns_agd_contrast,
    ni_agd_contrast = ni_agd_contrast,
    nt = n_trt,
    nchains = nchains,
    nint_max = nint_max,
    nint_vec = nint_vec,
    nX = ncol(X_all),
    int_thin = int_thin,
    # Study and treatment details
    narm_ipd = narm_ipd,
    ipd_arm = ipd_arm,
    ipd_trt = ipd_trt,
    narm_agd_arm = narm_agd_arm,
    agd_arm_trt = agd_arm_trt,
    agd_contrast_trt = as.array(agd_contrast_trt),
    agd_contrast_trt_b = as.array(agd_contrast_trt_b),
    agd_contrast_y = if (has_agd_contrast) as.array(agd_contrast_y$.y) else numeric(),
    agd_contrast_Sigma = Sigma,
    # ipd_study = ipd_study,
    # agd_arm_study = agd_arm_study,
    # agd_contrast_study = agd_contrast_study,
    # Random effects
    RE = switch(trt_effects, fixed = 0, random = 1),
    RE_cor = RE_cor,
    which_RE = which_RE,
    # Node splitting
    nodesplit = consistency == "nodesplit",
    # Design matrix or QR decomposition
    QR = QR,
    X = if (QR) X_all_Q else X_all,
    R_inv = if (QR) X_all_R_inv else matrix(0, 0, 0),
    # Offsets
    has_offset = has_offsets,
    offsets = if (has_offsets) as.array(c(ipd_offset, agd_arm_offset, agd_contrast_offset)) else numeric(),
    # Class effects
    which_CE = if (class_effects == "exchangeable") which_CE else numeric(0),
    which_CE_sd = if (class_effects == "exchangeable") which_CE_sd else numeric(0),
    class_effects = ifelse(class_effects == "exchangeable", 1, 0),
    # Baseline risk meta-regression
    brmr_n_col = sum(col_brmr),
    brmr_col = as.array(which(col_brmr)),
    xbar_mu = xbar_mu %||% 0
  )

  # Add priors
  standat <- purrr::list_modify(standat,
    !!! prior_standat(prior_intercept, "prior_intercept",
                      valid = c("Normal", "Cauchy", "Student t", "flat (implicit)")),
    !!! prior_standat(prior_trt, "prior_trt",
                      valid = c("Normal", "Cauchy", "Student t", "flat (implicit)")),
    !!! prior_standat(prior_reg, "prior_reg",
                      valid = c("Normal", "Cauchy", "Student t", "flat (implicit)")),
    !!! prior_standat(prior_het, "prior_het",
                      valid = c("Normal", "half-Normal", "log-Normal",
                                "Cauchy",  "half-Cauchy",
                                "Student t", "half-Student t", "log-Student t",
                                "Exponential", "flat (implicit)")),
    !!! prior_standat(prior_class_mean, "prior_class_mean",
                      valid = c("Normal", "Cauchy", "Student t", "flat (implicit)")),
    !!! prior_standat(prior_class_sd, "prior_class_sd",
                      valid = c("Normal", "half-Normal", "log-Normal",
                                "Cauchy",  "half-Cauchy",
                                "Student t", "half-Student t", "log-Student t",
                                "Exponential", "flat (implicit)")),
    prior_het_type = switch(prior_het_type,
                            sd = 1, var = 2, prec = 3)
    )

  # Standard pars to monitor
  pars <- c("mu", "beta", "d",
            "log_lik", "resdev",
            "lp__")

  if (has_ipd && !is_survival) pars <- c(pars, "fitted_ipd")
  if (has_agd_arm && !is_survival) pars <- c(pars, "fitted_agd_arm")
  if (has_agd_contrast) pars <- c(pars, "fitted_agd_contrast")

  # Monitor heterogeneity SD and study deltas if RE model
  if (trt_effects == "random") {
    pars <- c(pars, "tau", "delta")
  }

  # Monitor omega for node-splitting model
  if (consistency == "nodesplit") {
    pars <- c(pars, "omega")
  }

  # Monitor cumulative integration error if using numerical integration
  if (n_int > 1 && !is_survival && int_thin > 0) {
    if (has_agd_arm) pars <- c(pars, "theta_bar_cum_agd_arm")
    if (has_agd_contrast) pars <- c(pars, "theta_bar_cum_agd_contrast")
  }

  # Monitor class effects if class effects in use
  if (class_effects == "exchangeable") {
    pars <- c(pars, "class_mean", "class_sd")
  }

  # Set adapt_delta, but respect other control arguments if passed in ...
  stanargs <- list(...)
  if ("control" %in% names(stanargs))
    stanargs$control <- purrr::list_modify(stanargs$control, adapt_delta = adapt_delta)
  else
    stanargs$control <- list(adapt_delta = adapt_delta)

  # Global option rstan_refresh for refresh
  if (!"refresh" %in% names(stanargs) && !is.null(getOption("rstan_refresh"))) {
    refresh <- getOption("rstan_refresh")
    if (!rlang::is_integerish(refresh, n = 1, finite = TRUE)) abort("Global option `rstan_refresh` must be an integer.")
    stanargs$refresh <- refresh
  }

  # Set chain_id to make CHAIN_ID available in data block
  stanargs$chain_id <- 1L

  # Call Stan model for given likelihood

  # -- Normal likelihood
  if (likelihood == "normal") {

    # Dummy prior for IPD variance when no IPD - not used, but requested by Stan data
    if (!has_ipd) prior_aux <- half_normal(1)

    standat <- purrr::list_modify(standat,
      # Add outcomes
      ipd_y = if (has_ipd) ipd_y$.y else numeric(),
      agd_arm_y = if (has_agd_arm) agd_arm_y$.y else numeric(),
      agd_arm_se = if (has_agd_arm) agd_arm_y$.se else numeric(),

      # Add prior for auxiliary parameter - individual-level variance
      !!! prior_standat(prior_aux, "prior_aux",
                        valid = c("Normal", "half-Normal", "log-Normal",
                                  "Cauchy",  "half-Cauchy",
                                  "Student t", "half-Student t", "log-Student t",
                                  "Exponential", "flat (implicit)")),

      # Specify link
      link = switch(link, identity = 1, log = 2)
    )

    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$normal,
                                   data = standat,
                                   pars = c(pars, "sigma"))

  # -- Bernoulli/binomial likelihood (one parameter)
  } else if (likelihood %in% c("bernoulli", "binomial")) {

    standat <- purrr::list_modify(standat,
      # Add outcomes
      ipd_r = if (has_ipd) ipd_y$.r else integer(),
      agd_arm_r = if (has_agd_arm) agd_arm_y$.r else integer(),
      agd_arm_n = if (has_agd_arm) agd_arm_y$.n else integer(),

      # Specify link
      link = switch(link, logit = 1, probit = 2, cloglog = 3)
    )

    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$binomial_1par,
                                   data = standat,
                                   pars = pars)

  # -- Bernoulli/binomial likelihood (two parameter)
  } else if (likelihood %in% c("bernoulli2", "binomial2")) {

    standat <- purrr::list_modify(standat,
      # Add outcomes
      ipd_r = if (has_ipd) ipd_y$.r else integer(),
      agd_arm_r = if (has_agd_arm) agd_arm_y$.r else integer(),
      agd_arm_n = if (has_agd_arm) agd_arm_y$.n else integer(),

      # Specify link
      link = switch(link, logit = 1, probit = 2, cloglog = 3)
    )

    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$binomial_2par,
                                   data = standat,
                                   pars = if (n_int > 1 && int_thin > 0) c(pars, "theta2_bar_cum") else pars)

  # -- Poisson likelihood
  } else if (likelihood == "poisson") {

    standat <- purrr::list_modify(standat,
      # Add outcomes
      ipd_r = if (has_ipd) ipd_y$.r else integer(),
      ipd_E = if (has_ipd) ipd_y$.E else numeric(),
      agd_arm_r = if (has_agd_arm) agd_arm_y$.r else integer(),
      agd_arm_E = if (has_agd_arm) agd_arm_y$.E else numeric(),

      # Specify link
      link = switch(link, log = 1)
    )

    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$poisson,
                                   data = standat,
                                   pars = pars)

  # -- Ordered multinomial likelihood
  } else if (likelihood == "ordered") {

    if (has_ipd) {
      # Determine number of categories
      ncat <- ncol(ipd_y$.r)
      # Stan model takes IPD as an integer vector of category numbers
      ipd_r_int <- apply(ipd_y$.r == 1, 1, which)
      # Determine which categories are present
      ipd_cat <- t(apply(ipd_y$.r, 1,
                         function(x) {
                           cs <- which(!is.na(x))
                           c(cs, rep(0, ncat - length(cs)))
                         }))
      ipd_ncat <- rowSums(ipd_cat > 0)
    }

    if (has_agd_arm) {
      # Determine number of categories
      if (!has_ipd) ncat <- ncol(agd_arm_y$.r)
      # Determine which categories are present
      agd_arm_cat <- t(apply(agd_arm_y$.r, 1,
                         function(x) {
                           cs <- which(!is.na(x))
                           c(cs, rep(0, ncat - length(cs)))
                         }))
      agd_arm_ncat <- rowSums(agd_arm_cat > 0)
      # Replace missing category counts with 0 (these will drop out of the likelihood)
      agd_arm_r <- agd_arm_y$.r
      agd_arm_r[is.na(agd_arm_r)] <- 0
      agd_arm_n <- rowSums(agd_arm_y$.r, na.rm = TRUE)
    }

    if (!has_ipd && !has_agd_arm) {
      abort("No IPD or AgD (arm-based) in the network. Cannot fit ordered model to contrast data only.")
    }

    standat <- purrr::list_modify(standat,
      # Add outcomes
      ncat = ncat,

      ipd_r = if (has_ipd) ipd_r_int else integer(),
      ipd_cat = if (has_ipd) ipd_cat else matrix(0, 0, ncat),
      ipd_ncat = if (has_ipd) ipd_ncat else integer(),

      agd_arm_r = if (has_agd_arm) agd_arm_r else matrix(0, 0, ncat),
      agd_arm_n = if (has_agd_arm) agd_arm_n else integer(),
      agd_arm_cat = if (has_agd_arm) agd_arm_cat else matrix(0, 0, ncat),
      agd_arm_ncat = if (has_agd_arm) agd_arm_ncat else integer(),

      # Add prior for auxiliary parameters - latent cutoffs
      !!! prior_standat(prior_aux, "prior_aux",
                        valid = c("Normal", "half-Normal", "log-Normal",
                                  "Cauchy",  "half-Cauchy",
                                  "Student t", "half-Student t", "log-Student t",
                                  "Exponential", "flat (implicit)")),

      # Specify link
      link = switch(link, logit = 1, probit = 2, cloglog = 3)
    )

    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$ordered_multinomial,
                                   data = standat,
                                   pars = c(pars, "cc"))

  # -- Parametric survival likelihoods
  } else if (likelihood %in% setdiff(valid_lhood$survival, c("mspline", "pexp"))) {

    # Pull out Surv data
    ipd_surv <- get_Surv_data(ipd_y$.Surv)
    agd_arm_surv <- get_Surv_data(agd_arm_y$.Surv)

    # Add in dummy prior_aux for exponential model - not used, but requested by Stan data
    if (likelihood %in% c("exponential", "exponential-aft")) prior_aux <- flat()

    # Add in dummy prior_aux2 if not gengamma - not used, but requested by Stan data
    if (likelihood != "gengamma") prior_aux2 <- flat()

    # Add in dummy prior_aux_reg if no aux_regression
    if (is.null(X_aux)) prior_aux_reg <- flat()

    # Check aux IDs
    if (!(has_ipd || has_agd_arm)) {
      aux_id <- integer()
    } else {
      if (likelihood %in% c("exponential", "exponential-aft")) {
        aux_id <- aux_group <- rep_len(1, ni_ipd + ni_agd_arm)  # Not used (no aux pars)
      } else if (!rlang::is_integerish(aux_id, finite = TRUE)) {
        abort("`aux_id` must be an integer vector identifying the auxiliary parameter for each observation in IPD and AgD (arm-based) data.")
      }
    }

    # Set aux_int
    aux_int <- !is.null(X_aux) && max(aux_group) == nrow(X_aux)

    # Set flag for aux regression including main treatment effects
    aux_reg_trt <- any(grepl("^\\.trt[^:]", colnames(X_aux)))

    standat <- purrr::list_modify(standat,
                                  # AgD arm IDs
                                  agd_arm_arm = agd_arm_arm,

                                  # Auxiliary IDs for shape parameters
                                  #aux_by = length(aux_id) == ni_ipd + ni_agd_arm*n_int,
                                  aux_int = aux_int,
                                  aux_id = aux_id,
                                  aux_group = aux_group,

                                  # Aux regression
                                  nX_aux = if (!is.null(X_aux)) ncol(X_aux) else 0,
                                  X_aux = if (!is.null(X_aux))  X_aux else matrix(0, length(aux_id), 0),
                                  aux_reg_trt = aux_reg_trt,

                                  # Add outcomes
                                  ipd_time = ipd_surv$time,
                                  ipd_start_time = ipd_surv$start_time,
                                  ipd_delay_time = ipd_surv$delay_time,
                                  ipd_status = ipd_surv$status,

                                  agd_arm_time = agd_arm_surv$time,
                                  agd_arm_start_time = agd_arm_surv$start_time,
                                  agd_arm_delay_time = agd_arm_surv$delay_time,
                                  agd_arm_status = agd_arm_surv$status,

                                  # Specify survival distribution
                                  dist = switch(likelihood,
                                                exponential = 1,
                                                weibull = 2,
                                                gompertz = 3,
                                                `exponential-aft` = 4,
                                                `weibull-aft` = 5,
                                                lognormal = 6,
                                                loglogistic = 7,
                                                gamma = 8,
                                                gengamma = 9),

                                  # Specify link
                                  link = switch(link, log = 1),

                                  # Add prior for aux_regression coefs
                                  !!! prior_standat(prior_aux_reg, "prior_aux_reg",
                                                    valid = c("Normal", "half-Normal", "log-Normal",
                                                              "Cauchy",  "half-Cauchy",
                                                              "Student t", "half-Student t", "log-Student t",
                                                              "Exponential", "flat (implicit)"))
    )

    # Add in priors for auxiliary parameters
    if (likelihood != "gengamma") {
      standat <- purrr::list_modify(standat,
                                     # Specify prior on shape parameters
                                     !!! prior_standat(prior_aux, "prior_aux",
                                                       valid = c("Normal", "half-Normal", "log-Normal",
                                                                 "Cauchy",  "half-Cauchy",
                                                                 "Student t", "half-Student t", "log-Student t",
                                                                 "Exponential", "flat (implicit)")),

                                    # Dummy prior details for aux2
                                    !!! prior_standat(prior_aux2, "prior_aux2",
                                                      valid = "flat (implicit)"))
    } else {
      standat <- purrr::list_modify(standat,
                                     # Specify prior on sigma parameters
                                     !!! prior_standat(prior_aux$sigma, "prior_aux",
                                                       valid = c("Normal", "half-Normal", "log-Normal",
                                                                 "Cauchy",  "half-Cauchy",
                                                                 "Student t", "half-Student t", "log-Student t",
                                                                 "Exponential", "flat (implicit)")),

                                     # Specify prior on k parameters
                                     !!! prior_standat(prior_aux$k, "prior_aux2",
                                                       valid = c("Normal", "half-Normal", "log-Normal",
                                                                 "Cauchy",  "half-Cauchy",
                                                                 "Student t", "half-Student t", "log-Student t",
                                                                 "Exponential", "flat (implicit)")))
    }



    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$survival_param,
                                   data = standat,
                                   # Monitor auxiliary parameters
                                   pars =
                                     if (likelihood %in% c("exponential", "exponential-aft")) pars
                                     else if (likelihood == "lognormal") c(pars, "sdlog", "beta_aux", "d_aux")
                                     else if (likelihood == "gengamma") c(pars, "sigma", "k", "beta_aux", "d_aux")
                                     else c(pars, "shape", "beta_aux", "d_aux")
                                   )


  # -- Flexible parametric survival likelihoods (splines, piecewise exponential)
  } else if (likelihood %in% c("mspline", "pexp")) {

    # Pull out Surv data
    ipd_surv <- get_Surv_data(ipd_y$.Surv)
    agd_arm_surv <- get_Surv_data(agd_arm_y$.Surv)

    # Number of spline coefficients
    n_scoef <- ncol(basis[[1]])

    try_update <- function(..., study) {
      withCallingHandlers(update(...),
                          error = function(e) {
                            abort(glue::glue("Could not create spline basis for study {glue::double_quote(study)}."), parent = e)
                          },
                          warning = function(w) {
                            warn(glue::glue("In study {glue::double_quote(study)}: ", conditionMessage(w)))
                            rlang::cnd_muffle(w)
                          })
    }

    # Evaluate splines
    if (has_ipd) {
      ipd_time <- ipd_itime <- ipd_start_itime <- ipd_delay_itime <- matrix(nrow = length(ipd_surv$time), ncol = n_scoef)

      for (s in unique(ipd_s_t_all$.study)) {
        s_chr <- gsub("^\\.study", "", x_names[s])
        ipd_time[ipd_s_t_all$.study == s, ] <- try_update(basis[[s]], x = ipd_surv$time[ipd_s_t_all$.study == s], study = s_chr)
        ipd_itime[ipd_s_t_all$.study == s, ] <- try_update(basis[[s]], x = ipd_surv$time[ipd_s_t_all$.study == s], integral = TRUE, study = s_chr)
        ipd_start_itime[ipd_s_t_all$.study == s, ] <- try_update(basis[[s]], x = ipd_surv$start_time[ipd_s_t_all$.study == s], integral = TRUE, study = s_chr)
        ipd_delay_itime[ipd_s_t_all$.study == s, ] <- try_update(basis[[s]], x = ipd_surv$delay_time[ipd_s_t_all$.study == s], integral = TRUE, study = s_chr)
      }

    } else {
      ipd_time <- ipd_itime <- ipd_start_itime <- ipd_delay_itime <- matrix(nrow = 0, ncol = n_scoef)
    }

    if (has_agd_arm) {
      agd_arm_time <- agd_arm_itime <- agd_arm_start_itime <- agd_arm_delay_itime <- matrix(nrow = length(agd_arm_surv$time), ncol = n_scoef)

      for (s in unique(agd_arm_s_t_all$.study)) {
        s_chr <- gsub("^\\.study", "", x_names[s])
        agd_arm_time[agd_arm_s_t_all$.study == s, ] <- try_update(basis[[s]], x = agd_arm_surv$time[agd_arm_s_t_all$.study == s], study = s_chr)
        agd_arm_itime[agd_arm_s_t_all$.study == s, ] <- try_update(basis[[s]], x = agd_arm_surv$time[agd_arm_s_t_all$.study == s], integral = TRUE, study = s_chr)
        agd_arm_start_itime[agd_arm_s_t_all$.study == s, ] <- try_update(basis[[s]], x = agd_arm_surv$start_time[agd_arm_s_t_all$.study == s], integral = TRUE, study = s_chr)
        agd_arm_delay_itime[agd_arm_s_t_all$.study == s, ] <- try_update(basis[[s]], x = agd_arm_surv$delay_time[agd_arm_s_t_all$.study == s], integral = TRUE, study = s_chr)
      }

    } else {
      agd_arm_time <- agd_arm_itime <- agd_arm_start_itime <- agd_arm_delay_itime <- matrix(nrow = 0, ncol = n_scoef)
    }

    # Check aux IDs
    if (!(has_ipd || has_agd_arm)) {
      aux_id <- integer()
    } else {
      if (!rlang::is_integerish(aux_id, finite = TRUE))
        abort("`aux_id` must be an integer vector identifying the auxiliary parameter for each observation in IPD and AgD (arm-based) data.")
    }

    # Set aux_int
    aux_int <- !is.null(X_aux) && max(aux_group) == nrow(X_aux)

    # Get scoef prior means and weights for RW(1) prior with non-equally spaced knots
    if (!is.null(X_aux)) {
      # If aux_regression specified, single spline basis shared across network
      prior_aux_location <- list(mspline_constant_hazard(basis[[1]]))

      lscoef_weight <- list(rw1_prior_weights(basis[[1]]))

      aux_reg_trt <- any(grepl("^\\.trt[^:]", colnames(X_aux)))

    } else {
      prior_aux_location <- purrr::map(basis[unique(cbind(aux_id, study = c(ipd_s_t_all$.study, rep(agd_arm_s_t_all$.study, each = if (aux_int) n_int else 1))))[, "study"]],
                                       mspline_constant_hazard)

      lscoef_weight <- purrr::map(basis[unique(cbind(aux_id, study = c(ipd_s_t_all$.study, rep(agd_arm_s_t_all$.study, each = if (aux_int) n_int else 1))))[, "study"]],
                                  rw1_prior_weights)

      # Dummy prior for aux regression smoothing sd (not used)
      prior_aux_reg <- flat()

      aux_reg_trt <- FALSE
    }

    standat <- purrr::list_modify(standat,
                                  # AgD arm IDs
                                  agd_arm_arm = agd_arm_arm,

                                  # Aux IDs
                                  #aux_by = length(aux_id) == ni_ipd + ni_agd_arm*n_int,
                                  aux_int = aux_int,
                                  aux_id = aux_id,
                                  aux_group = aux_group,

                                  # Aux regression
                                  nX_aux = if (!is.null(X_aux)) ncol(X_aux) else 0,
                                  X_aux = if (!is.null(X_aux))  X_aux else matrix(0, length(aux_id), 0),
                                  aux_reg_trt = aux_reg_trt,

                                  # Number of spline coefficients
                                  n_scoef = n_scoef,

                                  # RW1 prior weights
                                  lscoef_weight = lscoef_weight,

                                  # Add outcomes
                                  ipd_time = ipd_time,
                                  ipd_itime = ipd_itime,
                                  ipd_start_itime = ipd_start_itime,
                                  ipd_delay_itime = ipd_delay_itime,
                                  ipd_delayed = ipd_surv$delay_time > 0,
                                  ipd_status = ipd_surv$status,

                                  agd_arm_time = agd_arm_time,
                                  agd_arm_itime = agd_arm_itime,
                                  agd_arm_start_itime = agd_arm_start_itime,
                                  agd_arm_delay_itime = agd_arm_delay_itime,
                                  agd_arm_delayed = agd_arm_surv$delay_time > 0,
                                  agd_arm_status = agd_arm_surv$status,

                                  # Specify link
                                  link = switch(link, log = 1),

                                  # Add prior for spline smoothing sd
                                  !!! prior_standat(prior_aux, "prior_hyper",
                                                    valid = c("Normal", "half-Normal", "log-Normal",
                                                              "Cauchy",  "half-Cauchy",
                                                              "Student t", "half-Student t", "log-Student t",
                                                              "Exponential", "flat (implicit)")),

                                  # Add prior for aux_regression smoothing sd
                                  !!! prior_standat(prior_aux_reg, "prior_reg_hyper",
                                                    valid = c("Normal", "half-Normal", "log-Normal",
                                                              "Cauchy",  "half-Cauchy",
                                                              "Student t", "half-Student t", "log-Student t",
                                                              "Exponential", "flat (implicit)"))
    )

    # Add prior mean for spline coefficients
    standat$prior_aux_location <- prior_aux_location

    # Monitor spline coefficients
    stanargs <- purrr::list_modify(stanargs,
                                   object = stanmodels$survival_mspline,
                                   data = standat,
                                   pars = c(pars,
                                            # "lscoef", "u_aux"
                                            "scoef", "beta_aux", "d_aux", "sigma", "sigma_beta"))

  } else {
    abort(glue::glue('"{likelihood}" likelihood not supported.'))
  }

  # Call sampling, managing warnings for integration checks if required
  if (n_int > 1 && int_check) {
    rhat_warn_a <- FALSE
    bulk_ess_warn_a <- FALSE
    tail_ess_warn_a <- FALSE
    stanfit <- withCallingHandlers(do.call(rstan::sampling, stanargs),
                                   warning = function(w) {
                                     m <- conditionMessage(w)
                                     if (grepl("The largest R-hat is", w, fixed = TRUE)) {
                                       rhat_warn_a <<- TRUE
                                       rlang::cnd_muffle(w)
                                     }
                                     if (grepl("Bulk Effective Sample(s?) Size \\(ESS\\) is too low", w)) {
                                       bulk_ess_warn_a <<- TRUE
                                       rlang::cnd_muffle(w)
                                     }
                                     if (grepl("Tail Effective Sample(s?) Size \\(ESS\\) is too low", w)) {
                                       tail_ess_warn_a <<- TRUE
                                       rlang::cnd_muffle(w)
                                     }
                                   })

    # Check Rhat, neff within chains with same n_int
    if (stanfit@mode == 0) {
      if (stanfit@sim$chains < nchains) {
        warn("Cannot check integration error, some chains failed to return samples.")
      } else if (any(rhat_warn_a, bulk_ess_warn_a, tail_ess_warn_a)) {
        # Only do these checks if Rhat/ESS warnings are thrown for all chains
        # combined, since these are only needed to disambiguate between low
        # iters and low n_int, and computation takes time

        sims <- as.array(stanfit)
        sims_nint1 <- sims[ , nint_vec == unique(n_int)[1], , drop = FALSE]
        sims_nint2 <- sims[ , nint_vec == unique(n_int)[2], , drop = FALSE]

        if (rhat_warn_a) {
          rhat_1 <- apply(sims_nint1, 3, rstan::Rhat)
          rhat_2 <- apply(sims_nint2, 3, rstan::Rhat)
          rhat_w <- pmax(rhat_1, rhat_2, na.rm = TRUE)
          rhat_warn_w <- any(rhat_w > 1.05, na.rm = TRUE)

          # Warnings within chains with same n_int are due to low iter
          if (rhat_warn_w) {
            warning("The largest R-hat is ", round(max(rhat_w), digits = 2),
                    ", indicating chains have not mixed.\n", "Running the chains for more iterations may help. See\n",
                    "https://mc-stan.org/misc/warnings.html#r-hat",
                    call. = FALSE)
          } else {
            # Otherwise warnings across all chains are low n_int
            rhat_a <- apply(sims, 3, rstan::Rhat)
            warn(paste0("The largest R-hat is ", round(max(rhat_a), digits = 2),
                        ", indicating that integration has not converged.\n",
                        "Increase the number of integration points `n_int` in add_integration()."),
                 class = "int_check_rhat")
          }
        }

        if (bulk_ess_warn_a) {
          bulk_ess_1 <- apply(sims_nint1, 3, rstan::ess_bulk) / ncol(sims_nint1)
          bulk_ess_2 <- apply(sims_nint2, 3, rstan::ess_bulk) / ncol(sims_nint2)
          bulk_ess_w <- pmin(bulk_ess_1, bulk_ess_2, na.rm = TRUE)
          bulk_ess_warn_w <- any(bulk_ess_w < 100, na.rm = TRUE)

          if (bulk_ess_warn_w) {
            warning("Bulk Effective Sample Size (ESS) is too low, ",
                    "indicating posterior means and medians may be unreliable.\n",
                    "Running the chains for more iterations may help. See\n",
                    "https://mc-stan.org/misc/warnings.html#bulk-ess",
                    call. = FALSE)
          } else {
            warn(paste0("Bulk Effective Sample Size (ESS) is too low, indicating that integration may not have converged.\n",
                        "Increase the number of integration points `n_int` in add_integration()."),
                 class = "int_check_essb")
          }
        }

        if (tail_ess_warn_a) {
          tail_ess_1 <- apply(sims_nint1, 3, rstan::ess_tail) / ncol(sims_nint1)
          tail_ess_2 <- apply(sims_nint2, 3, rstan::ess_tail) / ncol(sims_nint2)
          tail_ess_w <- pmin(tail_ess_1, tail_ess_2, na.rm = TRUE)
          tail_ess_warn_w <- any(tail_ess_w < 100, na.rm = TRUE)

          if (tail_ess_warn_w) {
            warning("Tail Effective Sample Size (ESS) is too low, indicating ",
                    "posterior variances and tail quantiles may be unreliable.\n",
                    "Running the chains for more iterations may help. See\n",
                    "https://mc-stan.org/misc/warnings.html#tail-ess",
                    call. = FALSE)
          } else {
            warn(paste0("Tail Effective Sample Size (ESS) is too low, indicating that integration may not have converged.\n",
                        "Increase the number of integration points `n_int` in add_integration()."),
                 class = "int_check_esst")
          }
        }
      }
    }

  } else {
    stanfit <- do.call(rstan::sampling, stanargs)
  }

  # Set readable parameter names in the stanfit object
  fnames_oi <- stanfit@sim$fnames_oi
  x_names_sub <- gsub("^(\\.study|\\.trt|\\.trtclass|\\.contr)", "", x_names)

  fnames_oi[grepl("^mu\\[[0-9]+\\]$", fnames_oi)] <- paste0("mu[", x_names_sub[col_study], "]")
  fnames_oi[grepl("^d\\[[0-9]+\\]$", fnames_oi)] <- paste0("d[", x_names_sub[col_trt], "]")
  fnames_oi[grepl("^beta\\[[0-9]+\\]$", fnames_oi)] <- paste0("beta[", x_names[col_reg], "]")
  if (!is.null(X_aux)) {
    if (likelihood %in% c("mspline", "pexp")) {
      fnames_oi[grepl("^beta_aux\\[", fnames_oi)] <- paste0("beta_aux[", rep(colnames(X_aux), times = n_scoef-1), ", ", rep(1:(n_scoef-1), each = ncol(X_aux)), "]")
      if (aux_reg_trt) {
        fnames_oi[grepl("^sigma_beta\\[", fnames_oi)] <- c("sigma_beta[.trt]",
                                                           if (ncol(X_aux) > n_trt) paste0("sigma_beta[", colnames(X_aux[, -(1:n_trt), drop = FALSE]), "]")
                                                           else NULL)
        fnames_oi[grepl("^d_aux\\[", fnames_oi)] <- paste0("d_aux[", rep(x_names_sub[col_trt], times = n_scoef-1), ", ", rep(1:(n_scoef-1), each = n_trt-1), "]")
      } else {
        fnames_oi[grepl("^sigma_beta\\[", fnames_oi)] <- paste0("sigma_beta[", colnames(X_aux), "]")
      }
    } else if (likelihood == "gengamma") {
      fnames_oi[grepl("^beta_aux\\[", fnames_oi)] <- paste0("beta_aux[", rep(colnames(X_aux), times = 2), ", ", rep(c("sigma", "k"), each = ncol(X_aux)), "]")
      if (aux_reg_trt) fnames_oi[grepl("^d_aux\\[", fnames_oi)] <- paste0("d_aux[", rep(x_names_sub[col_trt], times = 2), ", ", rep(c("sigma", "k"), each = n_trt-1), "]")
    } else {
      fnames_oi[grepl("^beta_aux\\[", fnames_oi)] <- paste0("beta_aux[", colnames(X_aux), "]")
      if (aux_reg_trt) fnames_oi[grepl("^d_aux\\[", fnames_oi)] <- paste0("d_aux[", x_names_sub[col_trt], "]")
    }
  }
  fnames_oi <- gsub("tau[1]", "tau", fnames_oi, fixed = TRUE)
  fnames_oi <- gsub("omega[1]", "omega", fnames_oi, fixed = TRUE)


  if (likelihood == "ordered") {
    if (has_ipd) l_cat <- colnames(ipd_y$.r)[-1]
    else if (has_agd_arm) l_cat <- colnames(agd_arm_y$.r)[-1]
    fnames_oi[grepl("^cc\\[[0-9]+\\]$", fnames_oi)] <- paste0("cc[", l_cat, "]")
  }

  stanfit@sim$fnames_oi <- fnames_oi

  return(stanfit)
}

#' Random effects structure
#'
#' Use `RE_cor` to generate the random effects correlation matrix, under the
#' assumption of common heterogeneity variance (i.e. all within-study
#' correlations are 0.5). Use `which_RE` to return a vector of IDs for the RE
#' deltas (0 means no RE delta on this arm).
#'
#' @param study A vector of study IDs (integer, character, or factor)
#' @param trt A factor vector of treatment codes (or coercible as such), with
#'   first level indicating the reference treatment
#' @param contrast A logical vector, of the same length as `study` and `trt`,
#'   indicating whether the corresponding data are in contrast rather than arm
#'   format.
#' @param type Character string, whether to generate RE structure under the
#'   "reference treatment" parameterisation, or the "baseline shift"
#'   parameterisation.
#'
#' @return For `RE_cor()`, a correlation matrix of dimension equal to the number
#'   of random effects deltas (excluding those that are set equal to zero).
#'
#'   For `which_RE()`, an integer vector of IDs indexing the rows and columns of
#'   the correlation matrix returned by `RE_cor()`.
#' @export
#' @aliases RE_cor
#' @rdname random_effects
#'
#' @examples
#' RE_cor(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)))
#' RE_cor(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)), type = "blshift")
RE_cor <- function(study, trt, contrast, type = c("reftrt", "blshift")) {
  if (!is.numeric(study) && !is.character(study) && !is.factor(study) || is.matrix(study)) {
    abort("`study` must be a vector, either numeric, character, or factor.")
  }
  if (!is.factor(trt)) {
    trt <- tryCatch(as.factor(trt),
                    error = function(e) {
                      abort("`trt` must be a factor (or coercible to factor).")
                    }, finally = inform("Coerced `trt` to factor."))
  }
  if (!is.logical(contrast) || is.matrix(contrast))
    abort("`contrast` must be a logical vector.")
  if (length(study) != length(trt) || length(study) != length(contrast))
    abort("`study`, `trt`, and `contrast` must be the same length.")
  type <- rlang::arg_match(type)


  reftrt <- levels(trt)[1]
  if (type == "reftrt") {
    # Treat contrast rows as non ref trt arms (since they always have REs)
    nonref <- trt != reftrt | contrast
    nRE <- sum(nonref)  # RE for each non ref trt arm
    Rho <- matrix(0, nrow = nRE, ncol = nRE)
    study <- study[nonref]
    trt <- trt[nonref]
  } else if (type == "blshift") {
    # Consider baseline arm to be first by treatment order
    # Treat contrast rows as non baseline arms (since they always have REs)
    nonbl <- tibble::tibble(study, trt, contrast) %>%
      dplyr::group_by(.data$study) %>%
      dplyr::mutate(nonbl = .data$contrast | .data$trt != sort(.data$trt)[1] | (duplicated(.data$trt) & .data$trt != reftrt)) %>%
      dplyr::pull(.data$nonbl)
    nRE <- sum(nonbl)  # RE for each non baseline arm
    Rho <- matrix(0, nrow = nRE, ncol = nRE)
    study <- study[nonbl]
    trt <- trt[nonbl]
  }

  diag(Rho) <- 1
  for (i in 1:(nRE - 1)) {
    for (j in (i + 1):nRE) {
      if (study[i] == study[j]) {
        Rho[i, j] <- 0.5
        Rho[j, i] <- 0.5
      }
    }
  }

  return(Rho)
}

#' @rdname random_effects
#' @aliases which_RE
#' @export
#' @examples
#' which_RE(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)))
#' which_RE(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)), type = "blshift")
which_RE <- function(study, trt, contrast, type = c("reftrt", "blshift")) {
  if (!is.numeric(study) && !is.character(study) && !is.factor(study) || is.matrix(study)) {
    abort("`study` must be a vector, either numeric, character, or factor.")
  }
  if (!is.factor(trt)) {
    trt <- tryCatch(as.factor(trt),
                    error = function(e) {
                      abort("`trt` must be a factor (or coercible to factor).")
                    }, finally = inform("Coerced `trt` to factor."))
  }
  if (!is.logical(contrast) || is.matrix(contrast))
    abort("`contrast` must be a logical vector.")
  if (length(study) != length(trt) || length(study) != length(contrast))
    abort("`study`, `trt`, and `contrast` must be the same length.")
  type <- rlang::arg_match(type)

  n_i <- length(study)
  id <- rep(0, n_i)

  reftrt <- levels(trt)[1]
  if (type == "reftrt") {
    trt_nonref <- trt != reftrt | contrast
    id[trt_nonref] <- 1:sum(trt_nonref)
  } else if (type == "blshift") {
    # Consider baseline arm to be first by treatment order
    non_bl <- tibble::tibble(study, trt, contrast) %>%
      dplyr::group_by(.data$study) %>%
      dplyr::mutate(non_bl = .data$contrast | .data$trt != sort(.data$trt)[1] | (duplicated(.data$trt) & .data$trt != reftrt)) %>%
      dplyr::pull(.data$non_bl)
    id[non_bl] <- 1:sum(non_bl)
  }

  return(id)
}

#' List of valid likelihoods for different outcomes
#' @noRd
valid_lhood <- list(binary = c("bernoulli", "bernoulli2"),
                    count = c("binomial", "binomial2"),
                    rate = "poisson",
                    continuous = "normal",
                    ordered = "ordered",
                    survival = c("exponential", "weibull", "gompertz",
                                 "exponential-aft", "weibull-aft",
                                 "lognormal", "loglogistic", "gamma", "gengamma",
                                 "mspline", "pexp"))


#' Create exchangeable class effects design vector
#' @param classes Network classes factor vector
#' @return A list, with elements `id` giving the design vector (0 = no class effect), and `label` giving the corresponding class labels
#' @noRd

which_CE <- function(classes, class_sd)   {

  # Class vector, without network reference treatment
  x <- classes[-1]

  # Identify sole occupancy classes
  solo_classes <- setdiff(levels(x)[table(x) == 1] , unlist(class_sd))

  # Set sole occupancy classes to NA (no class effects) and drop unused levels
  x <- droplevels(x, exclude = solo_classes)

  # Create numeric ID vector (0 = no class effect)
  id <- as.numeric(x)
  id[is.na(id)] <- 0

  # Create class labels
  label <- levels(x)

  return(list(id = id, label = label))
}


#' Check likelihood function, or provide default value
#'
#' @param x likelihood type as string
#' @param outcome outcome types as named list (ipd, agd_arm, agd_contrast)
#'
#' @noRd
check_likelihood <- function(x, outcome) {
  if (missing(outcome)) valid_lhood <- unlist(valid_lhood)
  else if (!is.na(outcome$ipd)) {
    valid_lhood <- valid_lhood[[outcome$ipd]]
    otype <- outcome$ipd
  } else if (!is.na(outcome$agd_arm)) {
    valid_lhood <- valid_lhood[[outcome$agd_arm]]
    otype <- outcome$agd_arm
  } else if (!is.na(outcome$agd_contrast)) {
    valid_lhood <- valid_lhood[[outcome$agd_contrast]]
    otype <- outcome$agd_contrast
  }
  else abort("No outcomes specified in network data.")

  # Choose default likelihood if NULL
  if (is.null(x) && missing(outcome)) abort("Please specify a suitable likelihood.")
  else if (is.null(x)) {
    x <- valid_lhood[1]

  # Check valid option if given
  } else if (!is.character(x) || length(x) > 1 || !tolower(x) %in% valid_lhood) {
    abort(glue::glue("`likelihood` should be a character string specifying a valid likelihood.\n",
                     "Suitable options for {otype} outcomes are currently: ",
                     glue::glue_collapse(glue::double_quote(valid_lhood), sep = ", ", last = " or "),
                     "."))
  }
  return(tolower(x))
}

#' Check link function, or provide default value
#'
#' @param x link function, as string
#' @param lik likelihood, as string
#'
#' @noRd
check_link <- function(x, lik) {
  valid_link <- list(normal = c("identity", "log"),
                     bernoulli = c("logit", "probit", "cloglog"),
                     bernoulli2 = c("logit", "probit", "cloglog"),
                     binomial = c("logit", "probit", "cloglog"),
                     binomial2 = c("logit", "probit", "cloglog"),
                     poisson = "log",
                     ordered = c("logit", "probit", "cloglog"),
                     exponential = "log",
                     weibull = "log",
                     gompertz = "log",
                     `exponential-aft` = "log",
                     `weibull-aft` = "log",
                     lognormal = "log",
                     loglogistic = "log",
                     gamma = "log",
                     gengamma = "log",
                     mspline = "log",
                     pexp = "log")[[lik]]

  if (is.null(x)) {
    x <- valid_link[1]
  } else if (!is.character(x) || length(x) > 1 || !tolower(x) %in% valid_link) {
    abort(glue::glue("`link` should be a character string specifying a valid link function.\n",
                     "Suitable options for a {lik} likelihood are currently: ",
                     glue::glue_collapse(glue::double_quote(valid_link), sep = ", ", last = " or "),
                     "."))
  }
  return(tolower(x))
}

#' Inverse link functions
#'
#' @param x Linear predictor values
#' @param link Character string specifying link function
#' @param ... Other parameters passed to link function
#'
#' @noRd
inverse_link <- function(x, link = c("identity", "log", "logit", "probit", "cloglog"), ...) {
  link <- rlang::arg_match(link)

  out <-
    if (link == "identity") x
    else if (link == "log") exp(x)
    else if (link == "logit") plogis(x, ...)
    else if (link == "probit") pnorm(x, ...)
    else if (link == "cloglog") 1 - exp(-exp(x))

  return(out)
}

#' Link functions
#'
#' @param x Linear predictor values
#' @param link Character string specifying link function
#' @param ... Other parameters passed to link function
#'
#' @noRd
link_fun <- function(x, link = c("identity", "log", "logit", "probit", "cloglog"), ...) {
  link <- rlang::arg_match(link)

  out <-
    if (link == "identity") x
    else if (link == "log") log(x)
    else if (link == "logit") qlogis(x, ...)
    else if (link == "probit") qnorm(x, ...)
    else if (link == "cloglog") log(-log(1 - x))

  return(out)
}

#' Get scale of outcome / linear predictor for reporting and plotting
#'
#' @param likelihood String, giving likelihood
#' @param link String, giving link function
#' @param measure String, specifying whether relative or absolute scale required
#' @param type String, specifying whether link scale or response scale required
#'
#' @return String giving the scale name, e.g. "log Odds Ratio"
#' @noRd
get_scale_name <- function(likelihood = c("normal", "bernoulli", "bernoulli2",
                                          "binomial", "binomial2", "poisson",
                                          "ordered",
                                          "exponential", "weibull", "gompertz",
                                          "exponential-aft", "weibull-aft",
                                          "lognormal", "loglogistic", "gamma",
                                          "gengamma",
                                          "mspline", "pexp"),
                           link = c("identity", "log", "logit", "probit", "cloglog"),
                           measure = c("relative", "absolute"),
                           type = c("link", "response",
                                    "survival", "hazard", "cumhaz", "mean", "median", "quantile", "rmst")) {

  likelihood <- rlang::arg_match(likelihood)
  link <- rlang::arg_match(link)
  measure <- rlang::arg_match(measure)
  type <- rlang::arg_match(type)

  check_link(link, likelihood)

  if (likelihood == "normal") {

    if (link == "identity") {
      if (measure == "relative") {
        out <- "Relative Effect"
      } else if (measure == "absolute") {
        out <- "Absolute Effect"
      }
    } else if (link == "log") {
      if (measure == "relative") {
        if (type == "link") out <- "Relative Effect (on log scale)"
        else out <- "Relative Effect (on response scale)"
      } else if (measure == "absolute") {
        if (type == "link") out <- "Absolute Effect (on log scale)"
        else out <- "Absolute Effect (on response scale)"
      }
    }

  } else if (likelihood %in% c("bernoulli", "bernoulli2", "binomial", "binomial2", "ordered")) {

    if (link == "logit") {
      if (measure == "relative") {
        if (type == "link") out <- "log Odds Ratio"
        else out <- ""
      } else if (measure == "absolute") {
        if (type == "link") out <- "log Odds"
        else out <- "Probability"
      }
    } else if (link == "probit") {
      if (measure == "relative") {
        if (type == "link") out <- "Probit Difference"
        else out <- ""
      } else if (measure == "absolute") {
        if (type == "link") out <- "Probit Probability"
        else out <- "Probability"
      }
    } else if (link == "cloglog") {
      if (measure == "relative") {
        if (type == "link") out <- "log Hazard Ratio"
        else out <- ""
      } else if (measure == "absolute") {
        if (type == "link") out <- "cloglog Probability"
        else out <- "Probability"
      }
    }

  } else if (likelihood == "poisson") {

    if (link == "log") {
      if (measure == "relative") {
        if (type == "link") out <- "log Rate Ratio"
        else out <- ""
      } else if (measure == "absolute") {
        if (type == "link") out <- "log Rate"
        else out <- "Rate"
      }
    }

  } else if (likelihood %in% c("exponential", "weibull", "gompertz", "mspline", "pexp")) {

    if (measure == "relative") {
      if (type == "link") out <- "log Hazard Ratio"
      else out <- ""
    } else if (measure == "absolute") {
      out <- switch(type,
                    survival = "Survival Probability",
                    hazard = "Hazard",
                    cumhaz = "Cumulative Hazard",
                    mean = "Mean Survival Time",
                    median = "Median Survival Time",
                    quantile = "Survival Time",
                    rmst = "Restricted Mean Survival Time",
                    link = "Linear Predictor")
    }

  } else if (likelihood %in% c("exponential-aft", "weibull-aft", "lognormal", "loglogistic", "gamma", "gengamma")) {

    if (measure == "relative") {
      if (type == "link") out <- "log Survival Time Ratio"
      else out <- ""
    } else if (measure == "absolute") {
      out <- switch(type,
                    survival = "Survival Probability",
                    hazard = "Hazard",
                    cumhaz = "Cumulative Hazard",
                    mean = "Mean Survival Time",
                    median = "Median Survival Time",
                    quantile = "Survival Time",
                    rmst = "Restricted Mean Survival Time",
                    link = "Linear Predictor")
    }

  } else {
    out <- ""
  }
  return(out)
}

#' Get outcome variables from internal nma_data
#'
#' @param x A data frame
#' @param o_type Outcome type
#'
#' @return A data frame with outcome variables selected
#' @noRd
get_outcome_variables <- function(x, o_type) {
  o_vars <- list(
    binary = ".r",
    rate = c(".r", ".E"),
    count = c(".r", ".n"),
    continuous = c(".y", ".se"),
    ordered = ".r",
    survival = ".Surv"
  )[[o_type]]

  return(
    dplyr::select(x, dplyr::one_of(intersect(o_vars, colnames(x))))
  )
}

#' Construct NMA formula
#'
#' @param regression Regression formula, or NULL
#' @param consistency Consistency/inconsistency model (character string)
#' @param classes Classes present? TRUE / FALSE
#' @param class_interactions Class interaction specification (character string)
#'
#' @return A formula
#' @noRd
make_nma_formula <- function(regression,
                             consistency = c("consistency", "nodesplit", "ume"),
                             classes,
                             class_interactions = c("common", "exchangeable", "independent")
                             ) {

  if (!is.null(regression) && !rlang::is_formula(regression)) abort("`regression` is not a formula")
  consistency <- rlang::arg_match(consistency)
  if (!rlang::is_bool(classes)) abort("`classes` should be TRUE or FALSE")
  if (classes && !is.null(regression)) class_interactions <- rlang::arg_match(class_interactions)

  if (!is.null(regression)) {

    # Set up treatment classes
    if (classes) {

      if (class_interactions == "common") {
        nma_formula <- do.call("substitute",
                               list(regression,
                                    list(.trt = quote(.trtclass))))

      } else if (class_interactions == "exchangeable") {
        abort('Exchangeable treatment class interactions (class_interactions = "exchangeable") not yet supported.')
      } else {
        nma_formula <- regression
      }

      # Remove any main effect of .trtclass, e.g. if user specified var*.trt
      # with common interactions, or used the .trtclass special
      nma_formula <- update.formula(nma_formula, ~. - .trtclass)

    } else {
      nma_formula <- regression
    }

    # Remove any main effect of .mu if baseline risk regression used
    if (".mu" %in% all.vars(regression)) nma_formula <- update.formula(nma_formula, ~. - .mu)

    if (consistency == "ume") {
      nma_formula <- update.formula(nma_formula, ~.study + .contr + . -1)
    } else if (consistency == "nodesplit") {
      nma_formula <- update.formula(nma_formula, ~.study + .trt + .omega + . -1)
    } else {
      nma_formula <- update.formula(nma_formula, ~.study + .trt + . -1)
    }
  } else {
    if (consistency == "ume") {
      nma_formula <- ~-1 + .study + .contr
    } else if (consistency == "nodesplit") {
      nma_formula <- ~-1 + .study + .trt + .omega
    } else {
      nma_formula <- ~-1 + .study + .trt
    }
  }
}

#' Construct NMA design matrix
#'
#' @param nma_formula NMA formula, returned by `make_nma_formula()`
#' @param ipd,agd_arm,agd_contrast Data frames
#' @param agd_contrast_bl Logical vector identifying baseline rows for contrast
#'   data
#' @param xbar Named numeric vector of centering values, or NULL
#' @param consistency Consistency/inconsistency model (character string)
#' @param nodesplit Length 2 character vector giving comparison to node-split
#' @param classes Classes present? TRUE / FALSE
#' @param class_interactions Class interaction specification (character string)
#' @param newdata Providing newdata post-fitting? TRUE / FALSE
#'
#' @return A named list of three matrices: X_ipd, X_agd_arm, X_agd_contrast; and
#'   three vectors of offsets: offset_ipd, offset_agd_arm, offset_agd_contrast.
#' @noRd
make_nma_model_matrix <- function(nma_formula,
                                  dat_ipd = tibble::tibble(),
                                  dat_agd_arm = tibble::tibble(),
                                  dat_agd_contrast = tibble::tibble(),
                                  agd_contrast_bl = logical(),
                                  xbar = NULL,
                                  consistency = c("consistency", "nodesplit", "ume"),
                                  nodesplit = NULL,
                                  classes = FALSE,
                                  newdata = FALSE) {
  # Checks
  if (!rlang::is_formula(nma_formula)) abort("`nma_formula` is not a formula")
  stopifnot(is.data.frame(dat_ipd),
            is.data.frame(dat_agd_arm),
            is.data.frame(dat_agd_contrast))
  consistency <- rlang::arg_match(consistency)
  if (!rlang::is_bool(classes)) abort("`classes` should be TRUE or FALSE")
  if (nrow(dat_agd_contrast) && !rlang::is_logical(agd_contrast_bl, n = nrow(dat_agd_contrast)))
    abort("`agd_contrast_bl` should be a logical vector of length nrow(agd_contrast)")
  if (!is.null(xbar) && (
        !(rlang::is_double(xbar) || rlang::is_integer(xbar)) || !rlang::is_named(xbar)))
    abort("`xbar` should be a named numeric vector")

  if (!consistency %in% c("consistency", "ume", "nodesplit")) {
    abort(glue::glue("Inconsistency '{consistency}' model not yet supported."))
  }

  if (consistency == "nodesplit") {
    if (!rlang::is_vector(nodesplit, 2))
      abort("`nodesplit` should be a vector of length 2.")
  }

  .has_ipd <- if (nrow(dat_ipd)) TRUE else FALSE
  .has_agd_arm <- if (nrow(dat_agd_arm)) TRUE else FALSE
  .has_agd_contrast <- if (nrow(dat_agd_contrast)) TRUE else FALSE

  # Sanitise factors
  if (.has_ipd) {
    dat_ipd <- dplyr::mutate_at(dat_ipd,
      .vars = if (classes) c(".trt", ".study", ".trtclass") else c(".trt", ".study"),
      .funs = fct_sanitise)
  }
  if (.has_agd_arm) {
    dat_agd_arm <- dplyr::mutate_at(dat_agd_arm,
                            .vars = if (classes) c(".trt", ".study", ".trtclass") else c(".trt", ".study"),
                            .funs = fct_sanitise)
  }
  if (.has_agd_contrast) {
    dat_agd_contrast <- dplyr::mutate_at(dat_agd_contrast,
                            .vars = if (classes) c(".trt", ".study", ".trtclass") else c(".trt", ".study"),
                            .funs = fct_sanitise)
  }

  # Define contrasts for UME model
  if (consistency == "ume") {
    # For IPD and AgD (arm-based), take the first-ordered arm as baseline
    # So the contrast sign will always be positive
    if (.has_ipd || .has_agd_arm) {
      contrs_arm <- dplyr::bind_rows(dat_ipd, dat_agd_arm) %>%
        dplyr::distinct(.data$.study, .data$.trt) %>%
        dplyr::arrange(.data$.study, .data$.trt) %>%
        dplyr::group_by(.data$.study) %>%
        dplyr::mutate(.trt_b = dplyr::first(.data$.trt)) %>%
        dplyr::ungroup() %>%
        dplyr::mutate(.contr = dplyr::if_else(.data$.trt == .data$.trt_b,
                                              "..ref..",
                                              paste0(.data$.trt, " vs. ", .data$.trt_b)),
                      .contr_sign = 1)
    } else {
      contrs_arm <- tibble::tibble()
    }

    # For AgD (contrast-based), take the specified baseline arm (with .y = NA)
    # Need to make sure to construct contrast the correct way around (d_12 instead of d_21)
    # and then make a note to change the sign if necessary
    if (.has_agd_contrast) {
      contrs_contr <- dat_agd_contrast %>%
        dplyr::distinct(.data$.study, .data$.trt) %>%  # In case integration data passed
        dplyr::left_join(dplyr::distinct(dat_agd_contrast[agd_contrast_bl, ], .data$.study, .data$.trt) %>%
                           dplyr::transmute(.data$.study, .trt_b = .data$.trt), by = ".study") %>%
        dplyr::distinct(.data$.study, .data$.trt, .data$.trt_b) %>%
        dplyr::mutate(.contr_sign = dplyr::if_else(as.numeric(.data$.trt) < as.numeric(.data$.trt_b), -1, 1),
                      .contr = dplyr::if_else(.data$.trt == .data$.trt_b,
                                              "..ref..",
                                              dplyr::if_else(.data$.contr_sign == 1,
                                                             paste0(.data$.trt, " vs. ", .data$.trt_b),
                                                             paste0(.data$.trt_b, " vs. ", .data$.trt))))
    } else {
      contrs_contr <- tibble::tibble()
    }

    # Make contrast info
    contrs_all <- dplyr::bind_rows(contrs_arm, contrs_contr)

    # Vector of all K(K-1)/2 possible contrast levels
    nt <- nlevels(contrs_all$.trt)
    ctr <- which(lower.tri(diag(nt)), arr.ind = TRUE)
    trt_lev <- levels(contrs_all$.trt)
    c_lev <- paste(trt_lev[ctr[, "row"]], trt_lev[ctr[, "col"]], sep = " vs. ")

    contrs_all <- dplyr::transmute(contrs_all,
      .data$.study, .data$.trt,
      .contr = forcats::fct_drop(factor(.data$.contr, levels = c("..ref..", c_lev))),
      .data$.contr_sign)

    # Join contrast info on to study data
    if (.has_ipd)
      dat_ipd <- dplyr::left_join(dat_ipd, contrs_all, by = c(".study", ".trt"))
    if (.has_agd_arm) {
      dat_agd_arm <- dplyr::left_join(dat_agd_arm, contrs_all, by = c(".study", ".trt"))
    }
    if (.has_agd_contrast) {
      dat_agd_contrast <- dplyr::left_join(dat_agd_contrast, contrs_all, by = c(".study", ".trt"))
    }
  }

  # Derive .omega indicator for node-splitting model
  if (consistency == "nodesplit") {
    if (.has_ipd) {
      dat_ipd <- dplyr::group_by(dat_ipd, .data$.study) %>%
        dplyr::mutate(.omega = all(nodesplit %in% .data$.trt) & .data$.trt == nodesplit[2]) %>%
        dplyr::ungroup()
    }
    if (.has_agd_arm) {
      dat_agd_arm <- dplyr::group_by(dat_agd_arm, .data$.study) %>%
        dplyr::mutate(.omega = all(nodesplit %in% .data$.trt) & .data$.trt == nodesplit[2]) %>%
        dplyr::ungroup()
    }
    if (.has_agd_contrast) {
      dat_agd_contrast <- dplyr::group_by(dat_agd_contrast, .data$.study) %>%
        dplyr::mutate(.omega = all(nodesplit %in% .data$.trt) & .data$.trt == nodesplit[2]) %>%
        dplyr::ungroup()
    }
  }

  # Construct design matrix all together then split out, so that same dummy
  # coding is used everywhere
  dat_all <- dplyr::bind_rows(dat_ipd, dat_agd_arm, dat_agd_contrast)

  # Check that required variables are present in each data set, and non-missing
  check_regression_data(nma_formula,
                        dat_ipd = dat_ipd,
                        dat_agd_arm = dat_agd_arm,
                        dat_agd_contrast = dat_agd_contrast,
                        newdata = newdata)

  # Center
  if (!is.null(xbar)) {
    dat_all[, names(xbar)] <-
      purrr::map2(dat_all[, names(xbar), drop = FALSE], xbar, ~.x - .y)
  }

  # Explicitly set contrasts attribute for key variables
  fvars <- all.vars(nma_formula)
  if (".trt" %in% fvars) stats::contrasts(dat_all$.trt) <- "contr.treatment"
  if (".trtclass" %in% fvars) stats::contrasts(dat_all$.trtclass) <- "contr.treatment"
  if (".contr" %in% fvars) stats::contrasts(dat_all$.contr) <- "contr.treatment"
  if (".omega" %in% fvars) stats::contrasts(dat_all$.omega) <- "contr.treatment"
  # .study handled separately next (not always a factor)

  # Drop study to factor to 1L if only one study (avoid contrasts need 2 or
  # more levels error)
  if (".study" %in% fvars && dplyr::n_distinct(dat_all$.study) == 1) {

    # Save study label to restore
    single_study_label <- unique(dat_all$.study)
    dat_all$.study_temp <- dat_all$.study
    dat_all$.study <- 1L

    # Fix up model formula with an intercept (if .study is main effect, which is not usually the case for aux_regression)
    if (".study" %in% colnames(attr(terms(nma_formula), "factors"))) nma_formula <- update.formula(nma_formula, ~. + 1)
  } else {
    single_study_label <- NULL
    if (".study" %in% fvars) stats::contrasts(dat_all$.study) <- "contr.treatment"
  }

  # Apply NMA formula to get design matrix
  X_all <- model.matrix(nma_formula, data = dat_all)
  offsets <- model.offset(model.frame(nma_formula, data = dat_all))
  has_offset <- !is.null(offsets)

  disc_names <- setdiff(names(attr(X_all, "contrasts")), c(".study", ".trt", ".trtclass", ".contr", ".omega"))

  if (!is.null(single_study_label)) {
    # Restore single study label and .study column
    colnames(X_all) <- stringr::str_replace(colnames(X_all),
                                            "^\\.study$",
                                            paste0(".study", single_study_label))
    dat_all <- dat_all %>%
      dplyr::mutate(.study = .data$.study_temp) %>%
      dplyr::select(-".study_temp")

    # Drop intercept column from design matrix
    if ("(Intercept)" %in% colnames(X_all)) X_all <- X_all[, -1, drop = FALSE]
  }

  # Remove columns for reference level of .trtclass
  if (classes) {
    ref_class <- levels(dat_all$.trtclass)[1]
    col_trtclass_ref <- grepl(paste0(".trtclass\\Q", ref_class, "\\E"),
                              colnames(X_all), perl = TRUE)
    X_all <- X_all[, !col_trtclass_ref, drop = FALSE]
  }

  # Remove columns for reference levels of discrete covariates
  if (length(disc_names)) for (xvar in disc_names) {
    # Check contrast type
    ctype <- attr(dat_all[[xvar]], "contrasts")
    if (is.null(ctype)) ctype <- getOption("contrasts")[if (is.ordered(dat_all[[xvar]])) "ordered" else "unordered"]

    # Get reference level for treatment/SAS contrasts
    if (ctype == "contr.treatment") {
      x_ref <-
        if (is.factor(dat_all[[xvar]])) levels(dat_all[[xvar]])[1]
        else if (is.logical(dat_all[[xvar]])) FALSE
        else levels(as.factor(dat_all[[xvar]]))[1]
    } else if (ctype == "contr.SAS") {
      x_ref <-
        if (is.factor(dat_all[[xvar]])) rev(levels(dat_all[[xvar]]))[1]
        else if (is.logical(dat_all[[xvar]])) FALSE
        else rev(levels(as.factor(dat_all[[xvar]])))[1]
    } else {
      x_ref <- NULL
    }

    # Remove reference level columns if present
    if (!is.null(x_ref)) {
      col_x_ref <- grepl(paste0("(`?)\\Q", xvar, "\\E(`?)\\Q", x_ref, "\\E"), colnames(X_all), perl = TRUE)
      X_all <- X_all[, !col_x_ref, drop = FALSE]
    }
  }

  # Remove columns for interactions with reference level of .trt or .trtclass
  ref_trt <- levels(dat_all$.trt)[1]
  regex_int_ref <- paste0("\\:\\.trt\\Q", ref_trt, "\\E$|^\\.trt\\Q", ref_trt, "\\E\\:")
  if (classes)
    regex_int_ref <- paste0(regex_int_ref, "|",
                            "\\:\\.trtclass\\Q", ref_class, "\\E$|^\\.trtclass\\Q", ref_class, "\\E\\:")
  col_int_ref <- grepl(regex_int_ref, colnames(X_all), perl = TRUE)
  X_all <- X_all[, !col_int_ref, drop = FALSE]

  # Remove global intercept column (present for aux_regression)
  X_all <- X_all[, colnames(X_all) != "(Intercept)", drop = FALSE]

  if (consistency == "ume") {
    # Set relevant entries to +/- 1 for direction of contrast, using .contr_sign
    contr_cols <- grepl("^\\.contr", colnames(X_all))
    X_all[, contr_cols] <- sweep(X_all[, contr_cols, drop = FALSE], MARGIN = 1,
                                 STATS = dat_all$.contr_sign, FUN = "*")
  }

  if (.has_ipd) {
    X_ipd <- X_all[1:nrow(dat_ipd), , drop = FALSE]
    offset_ipd <- if (has_offset) offsets[1:nrow(dat_ipd)] else NULL
  } else {
    X_ipd <- offset_ipd <- NULL
  }

  if (.has_agd_arm) {
    X_agd_arm <- X_all[nrow(dat_ipd) + 1:nrow(dat_agd_arm), , drop = FALSE]
    offset_agd_arm <- if (has_offset) offsets[nrow(dat_ipd) + 1:nrow(dat_agd_arm)] else NULL
  } else {
    X_agd_arm <- offset_agd_arm <- NULL
  }

  if (.has_agd_contrast) {
    X_agd_contrast_all <- X_all[nrow(dat_ipd) + nrow(dat_agd_arm) + 1:nrow(dat_agd_contrast), , drop = FALSE]
    offset_agd_contrast_all <-
      if (has_offset) {
        offsets[nrow(dat_ipd) + nrow(dat_agd_arm) + 1:nrow(dat_agd_contrast)]
      } else {
        NULL
      }

    # Difference out the baseline arms
    X_bl <- X_agd_contrast_all[agd_contrast_bl, , drop = FALSE]
    if (has_offset) offset_bl <- offset_agd_contrast_all[agd_contrast_bl]

    X_agd_contrast <- X_agd_contrast_all[!agd_contrast_bl, , drop = FALSE]
    offset_agd_contrast <-
      if (has_offset) {
        offset_agd_contrast_all[!agd_contrast_bl]
      } else {
        NULL
      }

    # Match non-baseline rows with baseline rows by study
    for (s in unique(dat_agd_contrast$.study)) {
      nonbl_id <- which(dat_agd_contrast$.study[!agd_contrast_bl] == s)
      bl_id <- which(dat_agd_contrast$.study[agd_contrast_bl] == s)

      bl_id <- rep_len(bl_id, length(nonbl_id))

      X_agd_contrast[nonbl_id, ] <- X_agd_contrast[nonbl_id, , drop = FALSE] - X_bl[bl_id, , drop = FALSE]
      if (has_offset) offset_agd_contrast[nonbl_id] <- offset_agd_contrast[nonbl_id] - offset_bl[bl_id]
    }

    # Remove columns for study baselines corresponding to contrast-based studies - not used
    s_contr <- unique(dat_agd_contrast$.study)
    bl_s_reg <- paste0("^\\.study(\\Q", paste0(s_contr, collapse = "\\E|\\Q"), "\\E)$")
    bl_cols <- grepl(bl_s_reg, colnames(X_agd_contrast), perl = TRUE)

    X_agd_contrast <- X_agd_contrast[, !bl_cols, drop = FALSE]
    if (.has_ipd) X_ipd <- X_ipd[, !bl_cols, drop = FALSE]
    if (.has_agd_arm) X_agd_arm <- X_agd_arm[, !bl_cols, drop = FALSE]
  } else {
    X_agd_contrast <- offset_agd_contrast <- NULL
  }

  return(list(X_ipd = X_ipd,
              X_agd_arm = X_agd_arm,
              X_agd_contrast = X_agd_contrast,
              offset_ipd = offset_ipd,
              offset_agd_arm = offset_agd_arm,
              offset_agd_contrast = offset_agd_contrast))
}

#' Extract columns used in model from data frame
#'
#' @param data Data frame
#' @param regression Regression formula or NULL
#' @param aux_regression Regression formula or NULL
#' @param label Label for data source or NULL, used for informative errors
#' @param keep Additional variables to keep in data
#'
#' @return Data frame with required columns
#' @noRd
get_model_data_columns <- function(data, regression = NULL, aux_regression = NULL, label = NULL, keep = NULL) {
  if (!is.null(label)) label <- paste(" in", label)
  regvars <- NULL
  auxregvars <- NULL

  if (!is.null(regression)) {
    regvars <- setdiff(all.vars(regression), c(".trt", ".trtclass", ".study", ".contr", ".omega"))
    badvars <- setdiff(regvars, colnames(data))
    if (length(badvars)) {
      abort(
        glue::glue("Regression variable{if (length(badvars) > 1) 's' else ''} ",
                   glue::glue_collapse(glue::double_quote(badvars), sep = ", ", last = " and "),
                   " not found", label, ".")
      )
    }
  }

  if (!is.null(aux_regression)) {
    auxregvars <- setdiff(all.vars(aux_regression), c(".trt", ".trtclass", ".study", ".contr", ".omega"))
    badvars <- setdiff(auxregvars, colnames(data))
    if (length(badvars)) {
      abort(
        glue::glue("Auxiliary regression variable{if (length(badvars) > 1) 's' else ''} ",
                   glue::glue_collapse(glue::double_quote(badvars), sep = ", ", last = " and "),
                   " not found", label, ".")
      )
    }
  }

  out <- dplyr::select(data, dplyr::starts_with("."), !! regvars, !! auxregvars, !! keep)

  # Work around dplyr::bind_rows() bug - convert .Surv column to bare matrix if present
  if (rlang::has_name(out, ".Surv")) out$.Surv <- as.matrix(out$.Surv)

  return(out)
}

#' Check data for regression
#'
#' Validates data for regression model. Checks that model matrix can be
#' constructed, and that there are no missing values.
#'
#' @param formula Model formula
#' @param dat_ipd,dat_agd_arm,dat_agd_contrast Data frames
#' @param newdata Providing newdata post-fitting? TRUE / FALSE
#'
#' @noRd
check_regression_data <- function(formula,
                                  dat_ipd = tibble::tibble(),
                                  dat_agd_arm = tibble::tibble(),
                                  dat_agd_contrast = tibble::tibble(),
                                  newdata = FALSE) {

  .has_ipd <- if (nrow(dat_ipd)) TRUE else FALSE
  .has_agd_arm <- if (nrow(dat_agd_arm)) TRUE else FALSE
  .has_agd_contrast <- if (nrow(dat_agd_contrast)) TRUE else FALSE

  # Check that required variables are present in each data set, and non-missing
  if (.has_ipd) {
    withCallingHandlers(
      X_ipd_frame <- model.frame(formula, dat_ipd, na.action = NULL),
      error = ~abort(paste0(if (newdata) "Failed to construct design matrix for `newdata`.\n" else "Failed to construct design matrix for IPD.\n", .)))

    X_ipd_has_na <- names(which(purrr::map_lgl(X_ipd_frame, ~any(is.na(.) | is.infinite(.)))))
  } else {
    X_ipd_has_na <- character(0)
  }

  if (.has_agd_arm) {
    withCallingHandlers(
      X_agd_arm_frame <- model.frame(formula, dat_agd_arm, na.action = NULL),
      error = ~abort(paste0(if (newdata) "Failed to construct design matrix for `newdata`.\n" else "Failed to construct design matrix for AgD (arm-based).\n", .)))

    X_agd_arm_has_na <- names(which(purrr::map_lgl(X_agd_arm_frame, ~any(is.na(.) | is.infinite(.)))))
  } else {
    X_agd_arm_has_na <- character(0)
  }

  if (.has_agd_contrast) {
    withCallingHandlers(
      X_agd_contrast_frame <- model.frame(formula, dat_agd_contrast, na.action = NULL),
      error = ~abort(paste0(if (newdata) "Failed to construct design matrix for `newdata`.\n" else "Failed to construct design matrix for AgD (contrast-based).\n", .)))

    X_agd_contrast_has_na <- names(which(purrr::map_lgl(X_agd_contrast_frame, ~any(is.na(.) | is.infinite(.)))))
  } else {
    X_agd_contrast_has_na <- character(0)
  }

  dat_has_na <- c(length(X_ipd_has_na) > 0,
                  length(X_agd_arm_has_na) > 0,
                  length(X_agd_contrast_has_na) > 0)
  if (any(dat_has_na)) {
    if (newdata) {
      abort(glue::glue("Variables with missing or infinite values in `newdata`: {paste(c(X_ipd_has_na, X_agd_arm_has_na, X_agd_contrast_has_na), collapse = ', ')}."))
    } else {
      abort(glue::glue(glue::glue_collapse(
        c("Variables with missing or infinite values in IPD: {paste(X_ipd_has_na, collapse = ', ')}.",
          "Variables with missing or infinite values in AgD (arm-based): {paste(X_agd_arm_has_na, collapse = ', ')}.",
          "Variables with missing or infinite values in AgD (contrast-based): {paste(X_agd_contrast_has_na, collapse = ', ')}."
        )[dat_has_na], sep = "\n")))
    }
  }

  invisible()
}

#' Check provided prior distributions
#'
#' @param x Input to check. Usually a `nma_prior` object.
#' @param list_names If `x` can be a named list of priors, the names we expect.
#'
#' @noRd
check_prior <- function(x, list_names) {
  arg <- rlang::caller_arg(x)
  if (missing(list_names)) {
    if (!inherits(x, "nma_prior"))
      abort(glue::glue("`{arg}` must be a prior distribution, see ?priors."),
            call = rlang::caller_env())
  } else {
    if (any(purrr::map_lgl(x, ~!inherits(., "nma_prior"))))
      abort(glue::glue("`{arg}` must be a named list of prior distributions, see ?priors.\n",
                       "Expecting named elements with priors for ",
                       glue::glue_collapse(list_names, sep = ", ", last = " and ", width = 30), "."),
            call = rlang::caller_env())

    nm <- setdiff(list_names, names(x))
    if (length(nm) > 0)
      abort(glue::glue("`{arg}` must be a named list of prior distributions.\n",
                       "Missing named elements with priors for ",
                       glue::glue_collapse(nm, sep = ", ", last = " and ", width = 30), "."),
            call = rlang::caller_env())
  }
}

#' Set prior details for Stan models
#'
#' @param x a `nma_prior` object
#' @param par character string, giving the Stan root parameter name (e.g.
#'   "prior_trt")
#' @param valid character vector, giving valid distributions
#'
#' @noRd
prior_standat <- function(x, par, valid){
  if (!inherits(x, "nma_prior")) abort("Not a `nma_prior` object.")

  dist <- x$dist

  if (!dist %in% valid)
    abort(glue::glue("Invalid `{par}`. Suitable distributions are: ",
                glue::glue_collapse(valid, sep = ", ", last = ", or ")))

  distn <- switch(dist,
                  `flat (implicit)` = 0,
                  Normal = , `half-Normal` = 1,
                  Cauchy = , `half-Cauchy` = 2,
                  `Student t` = , `half-Student t` = 3,
                  Exponential = 4,
                  `log-Normal` = 5,
                  `log-Student t` = 6,
                  Gamma = 7)

  out <- purrr::list_modify(c(x), dist = distn, fun = purrr::zap())
  # Set unnecessary (NA) parameters to zero. These will be ignored by Stan, but
  # need to pass rstan checks
  out[is.na(out)] <- 0
  names(out) <- paste0(par, "_", names(out))
  return(out)
}

#' Get covariance structure contrast-based data, using se on baseline arm
#'
#' @param x A data frame of agd_contrast data
#'
#' @return A list of covariance matrices, of length equal to the number of studies
#' @noRd
make_Sigma <- function(x) {
  return(unclass(
           by(x,
              forcats::fct_inorder(forcats::fct_drop(x$.study)),
              FUN = make_Sigma_block,
              simplify = FALSE)))
}

make_Sigma_block <- function(x) {
  s_ij <- x[is.na(x$.y), ".se", drop = TRUE]^2  # Covariances
  s_ii <- x[!is.na(x$.y), ".se", drop = TRUE]^2  # Variances
  narm <- length(s_ii)
  S <- matrix(s_ij, nrow = narm, ncol = narm)
  diag(S) <- s_ii
  return(S)
}

#' Get survival Surv data in consistent format
#'
#' @param x a Surv() object
#'
#' @return a list with elements time, start_time (for interval censored),
#'   delay_time (for delayed entry), and status
#' @noRd
get_Surv_data <- function(x) {
  if (!missing(x) && !is.null(x)) {

    surv_type <- attr(x, "type")

    status <- x[, "status"]

    if (surv_type == "right") {
      # Right censored
      time <- x[, "time"]
      start_time <- delay_time <- rep(0, length(time))

    } else if (surv_type == "left") {
      # Left censored
      time <- x[, "time"]
      start_time <- delay_time <- rep(0, length(time))

      # Make status indicator consistent with other types (i.e. left censored = 2)
      status[status == 0] <- 2

    } else if (surv_type %in% c("interval", "interval2")) {
      # Interval censored
      time <- start_time <- rep(0, length(x))

      # time1 is used unless status = 3 (interval censored)
      time[status < 3] <- x[status < 3, "time1"]
      time[status == 3] <- x[status == 3, "time2"]
      start_time[status == 3] <- x[status == 3, "time1"]

      delay_time <- rep(0, length(time))

    } else if (surv_type == "counting") {
      # Delayed entry
      time <- x[, "stop"]
      delay_time <- x[, "start"]
      start_time <- rep(0, length(time))
    }

  } else {
    time <- start_time <- delay_time <- numeric()
    status <- integer()
  }

  return(list(time = time,
              start_time = start_time,
              delay_time = delay_time,
              status = status))
}

#' Sanitise factor labels
#'
#' Remove forbidden chars from factor labels, replacing with "_"
#'
#' @param f A factor
#'
#' @noRd
fct_sanitise <- function(f) {
  forcats::fct_relabel(f, ~stringr::str_replace_all(., "[:\\[\\]]", "_"))
}

#' Check if formula only contains an offset
#'
#' @param f A one-sided formula, as used for regression argument in nma()
#' @noRd
is_only_offset <- function(f) {
  if (!rlang::is_formula(f, lhs = FALSE))
    abort("`f` should be a one-sided formula.")

  fterms <- terms(f)

  out <- !is.null(attr(fterms, "offset")) && length(attr(fterms, "factors")) == 0
  return(out)
}

#' Get EM variables from a model formula
#'
#' @param f A one-sided formula, as used for regression argument / output from make_nma_formula
#' @noRd
#' @return A character vector or character(0)
get_EM_vars <- function(f) {
  if (!rlang::is_formula(f, lhs = FALSE))
    abort("`f` should be a one-sided formula.")

  fnames <- colnames(attr(terms(f), "factor"))

  EM_regex <- "(^\\.trt(class)?\\:)|(\\:\\.trt(class)?$)"

  out <- stringr::str_subset(fnames, EM_regex)
  out <- unique(stringr::str_remove(out, EM_regex))
  return(out)
}

#' Make labels for data points
#'
#' @param study Study vector, coerced to character
#' @param trt Treatment vector, coerced to character
#' @param trt_b Baseline treatment vector if contrast-based, coerced to character
#'
#' @return Character vector of labels
#'
#' @noRd
make_data_labels <- function(study, trt, trt_b = NA) {
  tibble::tibble(study = study, trt = trt, trt_b = trt_b) %>%
    dplyr::group_by_all() %>%
    dplyr::mutate(grp_n = dplyr::n(),
                  grp_id = 1:dplyr::n(),
                  vs_trt_b = dplyr::if_else(is.na(trt_b),
                                            NA_character_,
                                            paste0(" vs. ", trt_b)),
                  label = as.character(
                    dplyr::if_else(.data$grp_n > 1,
                                   glue::glue("{study}: {trt}{vs_trt_b}, {grp_id}", .na = ""),
                                   glue::glue("{study}: {trt}{vs_trt_b}", .na = "")))
    ) %>%
    dplyr::pull(.data$label)
}

#' Get design vector for auxiliary parameters
#' Currently only used for survival models, to allow the user to specify shapes
#' that vary by group (e.g. by arm) as well as study
#'
#' @param data Data frame
#' @param by Vector of variables to stratify by (character or bare column names)
#' @param add_study Add in .study factor, even if not specified?
#'
#' @noRd
get_aux_id <- function(data, by, add_study = TRUE) {
  grouped <- get_aux_by_data(data = data, by = by, add_study = add_study)
  dplyr::group_indices(grouped)
}

#' Get auxiliary parameter labels based on aux_by
#' @noRd
get_aux_labels <- function(data, by, add_study = TRUE) {
  groupdat <- get_aux_by_data(data = data, by = by, add_study = add_study) %>%
    dplyr::group_data() %>%
    dplyr::select(-".rows")

  if (ncol(groupdat) == 1) {
    paste(groupdat[[1]])
  } else if (rlang::has_name(groupdat, ".study")) {
    paste0(groupdat$.study, ": ", do.call(paste, c(dplyr::select(groupdat, -".study"), sep = ", ")))
  } else {
    do.call(paste, c(groupdat, sep = ", "))
  }
}

#' Get auxiliary group data defined by aux_by
#' @noRd
get_aux_by_data <- function(data, by, add_study = TRUE) {
  tryCatch(
    dplyr::ungroup(data) %>%
      dplyr::select(if (add_study) ".study" else NULL, !! by) %>%
      dplyr::group_by_all(),
    error = function(x) abort("`aux_by` must be a vector of variable names to stratify auxiliary parameters by.", parent = x)
  )
}

#' Determine whether auxiliary parameters need to be integrated over based on
#' aux_regression and aux_by
#' @noRd
aux_needs_integration <- function(aux_regression, aux_by) {
  (!is.null(aux_regression) && length(setdiff(colnames(attr(terms(aux_regression), "factor")), c(".study", ".trt", ".trtclass"))) > 0) ||
    (!is.null(aux_by) && length(setdiff(aux_by, c(".study", ".trt", ".trtclass"))) > 0)
}

#' Deviance Information Criterion (DIC)
#'
#' Calculate the DIC for a model fitted using the [nma()] function.
#'
#' @param x A fitted model object, inheriting class [stan_nma]
#' @param penalty The method for estimating the effective number of parameters,
#'   used to penalise model fit in the DIC. Either `"pD"` (the default), or
#'   `"pV"`. For survival likelihoods only `"pV"` is currently available.
#' @param ... Other arguments (not used)
#'
#' @return A [nma_dic] object.
#' @export
#'
#' @seealso [print.nma_dic()] for printing details, [plot.nma_dic()] for
#'   producing plots of residual deviance contributions.
#'
#' @examples ## Smoking cessation
#' @template ex_smoking_nma_fe_example
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Compare DIC of FE and RE models
#' (smk_dic_FE <- dic(smk_fit_FE))
#' (smk_dic_RE <- dic(smk_fit_RE))   # substantially better fit
#'
#' # Plot residual deviance contributions under RE model
#' plot(smk_dic_RE)
#'
#' # Check for inconsistency using UME model
#' }
#' @template ex_smoking_nma_re_ume_example
#' @examples \donttest{
#' # Compare DIC
#' smk_dic_RE
#' (smk_dic_RE_UME <- dic(smk_fit_RE_UME))  # no difference in fit
#'
#' # Compare residual deviance contributions
#' plot(smk_dic_RE, smk_dic_RE_UME, show_uncertainty = FALSE)
#' }
dic <- function(x, penalty = c("pD", "pV"), ...) {
  if (!inherits(x, "stan_nma")) abort("Not a `stan_nma` object.")

  penalty <- rlang::arg_match(penalty)

  if (x$likelihood %in% valid_lhood$survival) penalty <- "pV"

  net <- x$network

  resdev <- colMeans(as.matrix(x, pars = "resdev"))

  resdev_array <- as.array(x, pars = "resdev")

  if (has_ipd(net)) {
    n_ipd <- nrow(net$ipd)
    resdev_ipd <- resdev[1:n_ipd]
    fitted_ipd <- if (!x$likelihood %in% valid_lhood$survival) colMeans(as.matrix(x, pars = "fitted_ipd")) else NULL
  } else {
    n_ipd <- 0
  }

  if (has_agd_arm(net)) {
    if (!x$likelihood %in% valid_lhood$survival) {
      n_agd_arm <- nrow(net$agd_arm)
      resdev_agd_arm <- resdev[n_ipd + (1:n_agd_arm)]
      fitted_agd_arm <- colMeans(as.matrix(x, pars = "fitted_agd_arm"))
    } else {
      n_agd_arm <- sum(net$agd_arm$.sample_size)
      resdev_agd_arm <- resdev[n_ipd + (1:n_agd_arm)]
      fitted_agd_arm <- NULL
    }
  } else {
    n_agd_arm <- 0
  }

  if (has_agd_contrast(net)) {
    # Number of residual deviances is equal to the number of studies
    nr_agd_contrast <- length(unique(net$agd_contrast$.study))
    # Number of fitted values is equal to the number of contrasts
    nf_agd_contrast <- nrow(dplyr::filter(net$agd_contrast, !is.na(.data$.y)))

    resdev_agd_contrast <- resdev[n_ipd + n_agd_arm + (1:nr_agd_contrast)]
    fitted_agd_contrast <- colMeans(as.matrix(x, pars = "fitted_agd_contrast"))
  } else {
    nr_agd_contrast <- nf_agd_contrast <- 0
  }

  has_df <- FALSE
  df_ipd <- df_agd_arm <- df_agd_contrast <- 1

  if (x$likelihood %in% c("bernoulli", "bernoulli2", "binomial", "binomial2") && penalty != "pV") {
    if (has_ipd(net)) {
      ipd_r <- net$ipd$.r
      resdevfit_ipd <- 2 * ifelse(ipd_r == 1,
                                  ipd_r * log(ipd_r / fitted_ipd),
                                  (1 - ipd_r) * log((1 - ipd_r) / (1 - fitted_ipd)))
      leverage_ipd <- resdev_ipd - resdevfit_ipd
    } else {
      leverage_ipd <- NULL
    }

    if (has_agd_arm(net)) {
      agd_arm_r <- net$agd_arm$.r
      agd_arm_n <- net$agd_arm$.n
      resdevfit_agd_arm <- 2 * (ifelse(agd_arm_r > 0,
                                       agd_arm_r * log(agd_arm_r / fitted_agd_arm),
                                       0) +
                                ifelse(agd_arm_n - agd_arm_r > 0,
                                       (agd_arm_n - agd_arm_r) *
                                         log((agd_arm_n - agd_arm_r) / (agd_arm_n - fitted_agd_arm)),
                                       0))
      leverage_agd_arm <- resdev_agd_arm - resdevfit_agd_arm
    } else {
      leverage_agd_arm <- NULL
    }

  } else if (x$likelihood == "normal" && penalty != "pV") {
    if (has_ipd(net)) {
      ipd_y <- net$ipd$.y
      ipd_arm <-  dplyr::group_indices(net$ipd, .data$.study, .data$.trt)

      # Use posterior median for sigma
      ipd_sigma <- apply(as.matrix(x, pars = "sigma"), 2, median)

      resdevfit_ipd <- resdev_ipd
      for (i in seq_along(ipd_y)) {
        resdevfit_ipd[i] <- (ipd_y[i] - fitted_ipd[i])^2 / ipd_sigma[ipd_arm[i]]^2
      }
      leverage_ipd <- resdev_ipd - resdevfit_ipd
    } else {
      leverage_ipd <- NULL
    }

    if (has_agd_arm(net)) {
      agd_arm_y <- net$agd_arm$.y
      agd_arm_se <- net$agd_arm$.se
      resdevfit_agd_arm <- (agd_arm_y - fitted_agd_arm)^2 / agd_arm_se^2
      leverage_agd_arm <- resdev_agd_arm - resdevfit_agd_arm
    } else {
      leverage_agd_arm <- NULL
    }

  } else if (x$likelihood == "poisson" && penalty != "pV") {
    if (has_ipd(net)) {
      ipd_r <- net$ipd$.r
      resdevfit_ipd <- 2 * ((fitted_ipd - ipd_r) +
                              ifelse(ipd_r > 0, ipd_r * log(ipd_r / fitted_ipd), 0))
      leverage_ipd <- resdev_ipd - resdevfit_ipd
    } else {
      leverage_ipd <- NULL
    }

    if (has_agd_arm(net)) {
      agd_arm_r <- net$agd_arm$.r
      resdevfit_agd_arm <- 2 * ((fitted_agd_arm - agd_arm_r) +
                              ifelse(agd_arm_r > 0, agd_arm_r * log(agd_arm_r / fitted_agd_arm), 0))
      leverage_agd_arm <- resdev_agd_arm - resdevfit_agd_arm
    } else {
      leverage_agd_arm <- NULL
    }

  } else if (x$likelihood == "ordered") {
    if (has_ipd(net)) {
      ipd_r <- net$ipd$.r
      fitted_ipd <- matrix(fitted_ipd, nrow = n_ipd)

      if (penalty != "pV") {
        resdevfit_ipd <- vector("double", n_ipd)
        for (i in 1:n_ipd) {
          resdevfit_ipd[i] <- 2 * sum((ipd_r[i,] * log(ipd_r[i,] / fitted_ipd[i,]))[!is.na(ipd_r[i,]) & ipd_r[i,] > 0])
        }
        leverage_ipd <- resdev_ipd - resdevfit_ipd
      }

      # Degrees of freedom is 1 - number of categories
      df_ipd <- rowSums(!is.na(ipd_r)) - 1
      has_df <- TRUE
    } else {
      leverage_ipd <- NULL
    }

    if (has_agd_arm(net)) {
      agd_arm_r <- net$agd_arm$.r
      fitted_agd_arm <- matrix(fitted_agd_arm, nrow = n_agd_arm)

      if (penalty != "pV") {
        resdevfit_agd_arm <- vector("double", n_agd_arm)
        for (i in 1:n_agd_arm) {
          resdevfit_agd_arm[i] <- 2 * sum((agd_arm_r[i,] * log(agd_arm_r[i,] / fitted_agd_arm[i,]))[!is.na(agd_arm_r[i,]) & agd_arm_r[i,] > 0])
        }
        leverage_agd_arm <- resdev_agd_arm - resdevfit_agd_arm
      }

      # Degrees of freedom is 1 - number of categories
      df_agd_arm <- rowSums(!is.na(agd_arm_r)) - 1
      has_df <- TRUE
    } else {
      leverage_agd_arm <- NULL
    }

  } else if (x$likelihood %in% valid_lhood$survival) {
    # Nothing to do, pV only

  } else if (penalty != "pV") {
    abort(glue::glue("DIC not supported for likelihood of type '{x$likelihood}'."))
  }

  if (has_agd_contrast(net)) {
    # Get covariance structure
    Sigma <- make_Sigma(net$agd_contrast)

    ul <- function(x) if (is.list(x)) x[[1]] else x

    agd_contrast_resdev_dat <-
      net$agd_contrast %>%
        dplyr::filter(!is.na(.data$.y)) %>%
        dplyr::mutate(.fitted = fitted_agd_contrast,
                      .observed = get_outcome_variables(., net$outcome$agd_contrast)[[1]],
                      .study_inorder = forcats::fct_inorder(forcats::fct_drop(.data$.study))) %>%
        dplyr::group_by(.data$.study_inorder, .data$.study) %>%
        dplyr::summarise(.y = list(.data$.y),
                         fitted = list(.data$.fitted),
                         observed = list(.data$.observed),
                         n_contrast = dplyr::n()) %>%
        dplyr::ungroup() %>%
        dplyr::mutate(Sigma = Sigma,
                      resdev = resdev_agd_contrast) %>%
        dplyr::rowwise() %>%
        dplyr::mutate(resdevfit = drop(crossprod(.data$.y - .data$fitted,
                                            solve(ul(.data$Sigma),
                                                  .data$.y - .data$fitted))),
                      leverage = .data$resdev - .data$resdevfit)

    leverage_agd_contrast <- agd_contrast_resdev_dat$leverage

    # df for multi-arm trials
    if (any(agd_contrast_resdev_dat$n_contrast > 1)) {
      has_df <- TRUE
      df_agd_contrast <- agd_contrast_resdev_dat$n_contrast
    }
  } else {
    leverage_agd_contrast <- NULL
  }

  # Get pointwise contributions for pV
  if (penalty == "pV") {
    leverage <- colSums(var(matrix(resdev_array, ncol = dim(resdev_array)[3])))/2
    if (has_ipd(net)) leverage_ipd <- leverage[1:n_ipd]
    if (has_agd_arm(net)) leverage_agd_arm <- leverage[n_ipd + (1:n_agd_arm)]
    if (has_agd_contrast(net)) leverage_agd_contrast <- leverage[n_ipd + n_agd_arm + (1:nr_agd_contrast)]
  }

  # Set pointwise contributions
  pw <- list()
  if (has_ipd(net)) {
    pw$ipd <- tibble::tibble(
      .study = net$ipd$.study,
      .trt = net$ipd$.trt,
      resdev = resdev_ipd,
      leverage = leverage_ipd,
      dic = resdev_ipd + leverage_ipd,
      fitted = fitted_ipd,
      observed = get_outcome_variables(net$ipd, net$outcome$ipd)[[1]])

    if (has_df) pw$ipd$df <- df_ipd
  } else {
    pw$ipd <- NULL
  }

  if (has_agd_arm(net)) {
    aa <- net$agd_arm
    if (x$likelihood %in% valid_lhood$survival) aa <- tidyr::unnest(aa, cols = ".Surv")

    pw$agd_arm <- tibble::tibble(
      .study = aa$.study,
      .trt = aa$.trt,
      resdev = resdev_agd_arm,
      leverage = leverage_agd_arm,
      dic = resdev_agd_arm + leverage_agd_arm,
      fitted = fitted_agd_arm,
      observed = get_outcome_variables(aa, net$outcome$agd_arm)[[1]])

    if (has_df) pw$agd_arm$df <- df_agd_arm
  } else {
    pw$agd_arm <- NULL
  }

  if (has_agd_contrast(net)) {
    pw$agd_contrast <- tibble::tibble(
      .study = agd_contrast_resdev_dat$.study,
      n_contrast = agd_contrast_resdev_dat$n_contrast,
      resdev = resdev_agd_contrast,
      leverage = leverage_agd_contrast,
      dic = resdev_agd_contrast + leverage_agd_contrast,
      fitted = agd_contrast_resdev_dat$fitted,
      observed = agd_contrast_resdev_dat$observed)

    if (has_df) pw$agd_contrast$df <- df_agd_contrast
  } else {
    pw$agd_contrast <- NULL
  }

  # Calculate pD, DIC
  totresdev <- sum(resdev)

  if (penalty == "pD") {
    pd <- sum(leverage_ipd, leverage_agd_arm, leverage_agd_contrast)
    dic <- totresdev + pd
    pv <- NULL
  } else if (penalty == "pV") {
    pv <- var(rowSums(as.matrix(x, pars = "resdev"))) / 2
    dic <- totresdev + pv
    pd <- NULL
  }

  # Return nma_dic object
  out <- list(dic = dic, pd = pd, pv = pv, resdev = totresdev, pointwise = pw, resdev_array = resdev_array)
  class(out) <- "nma_dic"
  attr(out, "penalty") <- penalty
  return(out)
}

#' The `nodesplit_summary` class
#'
#' The `nodesplit_summary` class contains posterior summary statistics for
#' node-splitting models, as a result of calling `summary()` on a
#' `nma_nodesplit` or `nma_nodesplit_df` object.
#'
#' @rdname nodesplit_summary-class
#' @name nodesplit_summary-class
#' @aliases nodesplit_summary
#'
#' @details Objects of class `nodesplit_summary` are tibble data frames, with one row
#'   for each node-split comparison and columns:
#'   \describe{
#'   \item{`trt1`, `trt2`}{Treatments forming the comparison}
#'   \item{`summary`}{A list column containing [nma_summary] objects with the
#'   posterior summaries and draws for each of the node-splitting parameters}
#'   \item{`p_value`}{Bayesian p-value for inconsistency}
#'   \item{`dic`}{A list column containing [nma_dic] objects, giving the model
#'   fit statistics}
#'   }
#'
#'   The parameters included in `summary` are:
#'   \describe{
#'   \item{`d_net`}{Network estimate from the corresponding consistency model,
#'   if available}
#'   \item{`d_dir`}{Direct estimate from the node-splitting model}
#'   \item{`d_ind`}{Indirect estimate from the node-splitting model}
#'   \item{`omega`}{Inconsistency factor \eqn{\omega = d_\mathrm{dir} -
#'   d_\mathrm{ind}}{\omega = d_dir - d_ind}}
#'   \item{`tau`}{Heterogeneity standard deviation from the node-splitting
#'   model, if a random effects model was fitted}
#'   \item{`tau_consistency`}{Heterogeneity standard deviation from the
#'   corresponding consistency model, if available and if a random effects model
#'   was fitted}
#'   }
#'
#'
NULL

#' Methods for `nodesplit_summary` objects
#'
#' The `as.data.frame()`, `as_tibble()`, and `as.tibble()` methods return the
#' node-splitting summaries in a data frame or tibble.
#'
#' @param x A `nodesplit_summary` object
#' @param ... Additional arguments passed on to other methods
#' @param digits Integer number of digits to display
#'
#' @rdname nodesplit_summary-methods
#'
#' @return A `data.frame` for `as.data.frame()`, a `tbl_df` for `as.tibble()`
#'   and `as_tibble()`.
#'
#'   The `print()` method returns `x` invisibly.
#'
#' @seealso [plot.nodesplit_summary()]
#'
#' @export
#'
print.nodesplit_summary <- function(x, ..., digits = 2) {
  if (!rlang::is_scalar_integerish(digits)) abort("`digits` must be a single integer")

  n_ns <- nrow(x)

  if (n_ns == 1) {
    cglue("Node-splitting model fitted for 1 comparison: ",
          as.character(x$trt2[1]),
          " vs. ",
          as.character(x$trt1[1]), ".")
  } else {
    cglue("Node-splitting models fitted for {n_ns} comparisons.")
  }

  for (i in 1:nrow(x)) {
    cglue("")
    if (n_ns > 1) {
      sec_header(glue::glue("Node-split {x$trt2[i]} vs. {x$trt1[i]}"))
      cglue("")
    }

    # Omit comparison details from d[]
    xsum <- x$summary[[i]]
    xsum$summary$parameter <- stringr::str_remove(xsum$summary$parameter, "\\[.*\\]")

    print(xsum, ...)
    cglue("")
    print(x$dic[[i]])
    cglue("")
    cglue("Bayesian p-value: {format.pval(x$p_value[[i]], digits = digits, eps = 10^-digits)}")
  }

  invisible(x)
}

#' @export
#' @rdname nodesplit_summary-methods
#' @param nest Whether to return a nested tibble, with the full [nma_summary]
#'   and [nma_dic] objects, or to unnest their summaries, default `FALSE`
as_tibble.nodesplit_summary <- function(x, ..., nest = FALSE) {
  if (!rlang::is_bool(nest)) abort("`nest` must be a single logical value TRUE/FALSE.")

  if (nest) { # Return underlying nested tibble
    NextMethod(...)
  } else { # Unnest summary results
    out <- x
    out$summary <- purrr::map(out$summary, tibble::as_tibble)
    out <- tidyr::unnest(out, cols = "summary")
    out$resdev <- purrr::map_dbl(out$dic, "resdev")
    out$pd <- purrr::map_dbl(out$dic, "pd")
    out$dic <- purrr::map_dbl(out$dic, "dic")
    class(out) <- setdiff(class(out), "nodesplit_summary")
    return(out)
  }
}

#' @export
#' @rdname nodesplit_summary-methods
as.tibble.nodesplit_summary <- function(x, ..., nest = FALSE) {
  return(tibble::as_tibble(x, ..., nest = nest))
}

#' @export
#' @rdname nodesplit_summary-methods
as.data.frame.nodesplit_summary <- function(x, ...) {
  return(as.data.frame(tibble::as_tibble(x, ..., nest = FALSE)))
}


#' Plots of node-splitting models
#'
#' Produce summary plots of node-splitting models
#'
#' @param x A `nodesplit_summary` object.
#' @param ... Additional arguments passed on to the underlying `ggdist` plot
#'   stat, see Details.
#' @param pars Character vector specifying the parameters to include in the
#'   plot, choices include `"d"` for the direct, indirect, and network estimates
#'   of relative effects, `"omega"` for the inconsistency factor, and `"tau"`
#'   for heterogeneity standard deviation in random effects models. Default is
#'   `"d"`.
#' @param stat Character string specifying the `ggdist` plot stat to use. The
#'   default `"dens_overlay"` is a special case, producing an overlaid density
#'   plot.
#' @param orientation Whether the `ggdist` geom is drawn horizontally
#'   (`"horizontal"`) or vertically (`"vertical"`), default `"horizontal"`.
#' @param ref_line Numeric vector of positions for reference lines, by default
#'   no reference lines are drawn.
#'
#' @details Plotting is handled by \link[ggplot2:ggplot2-package]{ggplot2} and
#'   the stats and geoms provided in the \link[ggdist:ggdist-package]{ggdist}
#'   package. As a result, the output is very flexible. Any plotting stats
#'   provided by `ggdist` may be used, via the argument `stat`. The default
#'   `"dens_overlay"` is a special exception, which uses
#'   \code{\link[ggplot2:geom_density]{ggplot2::geom_density()}}, to plot
#'   overlaid densities. Additional arguments in `...` are passed to the
#'   `ggdist` stat, to customise the output.
#'
#'   Alternative stats can be specified to produce different summaries. For
#'   example, specify `stat = "[half]eye"` to produce (half) eye plots, or `stat
#'   = "pointinterval"` to produce point estimates and credible intervals.
#'
#'   A full list of options and examples is found in the `ggdist` vignette
#'   `vignette("slabinterval", package = "ggdist")`.
#'
#'   A `ggplot` object is returned which can be further modified through the
#'   usual \link[ggplot2:ggplot2-package]{ggplot2} functions to add further
#'   aesthetics, geoms, themes, etc.
#'
#' @return A `ggplot` object.
#' @seealso [summary.nma_nodesplit_df()]
#'
#' @export
#'
#' @template ex_smoking_nma_re_nodesplit_example
#' @examples \donttest{
#' # Summarise the node-splitting results
#' (smk_nodesplit_summary <- summary(smk_fit_RE_nodesplit))
#'
#' # Plot the node-splitting results
#' plot(smk_nodesplit_summary)
#'
#' # Plot the inconsistency factors instead, change the plot stat to half-eye,
#' # and add a reference line at 0
#' plot(smk_nodesplit_summary, pars = "omega", stat = "halfeye", ref_line = 0)
#'
#' # Plot a comparison of the heterogeneity under the node-split models vs.
#' # the consistency model
#' plot(smk_nodesplit_summary, pars = "tau")
#' }
plot.nodesplit_summary <- function(x, ...,
                                   pars = "d",
                                   stat = "dens_overlay",
                                   orientation = c("horizontal", "vertical", "y", "x"),
                                   ref_line = NA_real_) {
  # Checks
  if (!is.character(pars))
    abort("`pars` must be a character vector of parameters to include.")

  if (!rlang::is_string(stat))
    abort("`stat` should be a character string specifying the name of a ggdist stat.")

  if (stat == "dens_overlay") {
    # Special case, for the classic overlaid density plot
    tb_geom <- getExportedValue("ggdist", "stat_slab")
  } else {
    stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

    tb_geom <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
                        error = function(err) {
                          abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                                      err, sep = "\n"))
                        })
  }


  if (!is.numeric(ref_line) || !is.null(dim(ref_line)))
    abort("`ref_line` should be a numeric vector.")

  orientation <- rlang::arg_match(orientation)
  if (orientation == "x") orientation <- "vertical"
  else if (orientation == "y") orientation <- "horizontal"

  # Is a horizontal geom specified?
  horizontal <- orientation == "horizontal"

  # Get draws
  draws <- tibble::as_tibble(x, nest = TRUE)
  draws$summary <- purrr::map(draws$summary, ~tibble::as_tibble(as.matrix(.)))

  draws$summary <- purrr::map(draws$summary,
                              ~tidyr::pivot_longer(., cols = dplyr::everything(),
                                 names_to = "parameter", values_to = "value"))

  draws <- dplyr::mutate(draws, comparison = paste0(.data$trt2, " vs. ", .data$trt1)) %>%
    dplyr::select("comparison", "summary") %>%
    tidyr::unnest(cols = "summary") %>%
    dplyr::mutate(parameter = stringr::str_remove(.data$parameter, "\\[.*\\]"))

  # Filter out selected pars
  # Allow selection with d instead of d_ind, d_dir, d_net
  allpars <- c(unique(draws$parameter), "d")
  badpars <- setdiff(pars, allpars)

  if (length(badpars))
    abort(glue::glue("No parameter{if (length(badpars) > 1) 's' else ''} ",
                     glue::glue_collapse(glue::double_quote(badpars), sep = ", ", last = " or "), "."))

  # Expand d and tau to include direct/indirect/network
  if ("d" %in% pars) pars <- c(setdiff(pars, "d"), "d_dir", "d_ind", "d_net")
  if ("tau" %in% pars) pars <- c(pars, "tau_consistency")

  draws <- dplyr::filter(draws, .data$parameter %in% pars)

  draws$parameter <- nfactor(draws$parameter)

  # Construct plot
  if (horizontal) {

    p <- ggplot2::ggplot(draws, ggplot2::aes(x = .data$value)) +
      ggplot2::geom_vline(xintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::xlab("Value")

    if (stat == "dens_overlay") {
      p <- p + ggplot2::aes(colour = .data$parameter, fill = .data$parameter) +
        ggplot2::ylab("Density")
    } else {
      p <- p + ggplot2::aes(y = .data$parameter) + ggplot2::ylab("Parameter")
    }

  } else {

    p <- ggplot2::ggplot(draws, ggplot2::aes(y = .data$value)) +
      ggplot2::geom_hline(yintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::ylab("Value")

    if (stat == "dens_overlay") {
      p <- p + ggplot2::aes(colour = .data$parameter, fill = .data$parameter) +
        ggplot2::xlab("Density")
    } else {
      p <- p + ggplot2::aes(x = .data$parameter) + ggplot2::xlab("Parameter")
    }

  }

  if (stat == "dens_overlay") {
    p <- p + do.call(ggplot2::geom_density,
                     args = rlang::dots_list(orientation = if (horizontal) "x" else "y",
                                             ...,
                                             trim = TRUE,
                                             alpha = 0.25,
                                             .homonyms = "first")) +
      ggplot2::facet_wrap(~comparison, scales = "free")
  } else {
    p <- p + do.call(tb_geom, args = list(orientation = orientation, ...)) +
      ggplot2::facet_wrap(~comparison)
  }

  p <- p +
    theme_multinma()

  return(p)
}

#' The nma_data class
#'
#' The `nma_data` class contains the data for a NMA in a standard format,
#' created using the functions [set_ipd()], [set_agd_arm()],
#' [set_agd_contrast()], [set_agd_surv()], or [combine_network()]. The sub-class
#' `mlnmr_data` is created by the function [add_integration()], and further
#' contains numerical integration points for the aggregate data.
#'
#' @rdname nma_data-class
#' @name nma_data-class
#' @aliases nma_data mlnmr_data mlnmr_data-class
#'
#' @details Objects of class `nma_data` have the following components:
#'   \describe{
#'   \item{`agd_arm`}{data from studies with aggregate data (arm format)}
#'   \item{`agd_contrast`}{data from studies with aggregate data (contrast
#'   format)}
#'   \item{`ipd`}{data from studies with individual patient data}
#'   \item{`treatments`}{treatment coding factor for entire network}
#'   \item{`classes`}{treatment class coding factor (same length as `treatments`
#'   for entire network)}
#'   \item{`studies`}{study coding factor for entire network}
#'   \item{`outcome`}{outcome type for each data source, named list}
#'   }
#'
#' The `agd_arm`, `agd_contrast`, and `ipd` components are
#' tibbles with the following columns:
#'   \describe{
#'   \item{`.study`}{study (as factor)}
#'   \item{`.trt`}{treatment (as factor)}
#'   \item{`.trtclass`}{treatment class (as factor), if specified}
#'   \item{`.y`}{continuous outcome}
#'   \item{`.se`}{standard error (continuous)}
#'   \item{`.r`}{event count (discrete)}
#'   \item{`.n`}{event count denominator (discrete, `agd_arm` only)}
#'   \item{`.E`}{time at risk (discrete)}
#'   \item{`.Surv`}{survival outcome of type [`Surv`] (time-to-event), nested by
#'   study arm}
#'   \item{`.sample_size`}{sample size (`agd_*` only)}
#'   \item{`...`}{other columns (typically covariates) from the original data
#'   frame}
#'   }
#'
#' Objects of class `mlnmr_data` additionally have components:
#'   \describe{
#'   \item{`n_int`}{number of numerical integration points}
#'   \item{`int_names`}{names of covariates with numerical integration points}
#'   \item{`int_cor`}{correlation matrix for covariates used to generate
#'   numerical integration points}
#'   }
#'
#' The `agd_arm` and `agd_contrast` tibbles have additional list columns with
#' prefix `.int_`, one for each covariate, which contain the numerical
#' integration points nested as length-`n_int` vectors within each row.
#'
#' @template seealso_nma_data
NULL

#' Print `nma_data` objects
#'
#' Print details of networks stored as [nma_data] objects, as created by
#' [set_ipd()], [set_agd_arm()], [set_agd_contrast()], [set_agd_surv()], or
#' [combine_network()].
#'
#' @param x `nma_data` object
#' @param ... other options (not used)
#' @param n number of studies of each type to print
#'
#' @return `x` is returned invisibly.
#'
#' @export
print.nma_data <- function(x, ..., n = 10) {
  cwidth <- getOption("width")

  # Error if n is not an integer
  if (!rlang::is_scalar_integerish(n)) abort("Argument `n` should be an integer")

  if (has_ipd(x)) {
    s_ipd <- x$ipd %>%
      dplyr::distinct(.data$.study, .data$.trt) %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::summarise("Treatment arms" = glue::glue("{dplyr::n()}: ",
                                              glue::glue_collapse(.data$.trt, sep = " | ", width = 0.8*cwidth))) %>%
      dplyr::rename(Study = ".study") %>%
      as.data.frame()
    n_ipd <- nrow(s_ipd)
  } else {
    n_ipd <- 0
  }

  if (has_agd_arm(x)) {
    s_agd_arm <- x$agd_arm %>%
      dplyr::arrange(.data$.study, .data$.trt) %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::summarise("Treatment arms" = glue::glue("{dplyr::n()}: ",
                                              glue::glue_collapse(.data$.trt, sep = " | ", width = 0.8*cwidth))) %>%
      dplyr::rename(Study = ".study") %>%
      as.data.frame()
    n_agd_arm <- nrow(s_agd_arm)
  } else {
    n_agd_arm <- 0
  }

  if (has_agd_contrast(x)) {
    s_agd_contrast <- x$agd_contrast %>%
      dplyr::arrange(.data$.study, .data$.trt) %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::summarise("Treatment arms" = glue::glue("{dplyr::n()}: ",
                                               glue::glue_collapse(.data$.trt, sep = " | ", width = 0.8*cwidth))) %>%
      dplyr::rename(Study = ".study") %>%
      as.data.frame()
    n_agd_contrast <- nrow(s_agd_contrast)
  } else {
    n_agd_contrast <- 0
  }

  if (all(n_ipd == 0, n_agd_arm == 0, n_agd_contrast == 0)) {
    cglue("An empty network.")
  } else {
    cglue("A network with ", glue::glue_collapse(c(
      "{n_ipd} IPD stud{ifelse(n_ipd == 1, 'y', 'ies')}",
      "{n_agd_arm} AgD stud{ifelse(n_agd_arm == 1, 'y', 'ies')} (arm-based)",
      "{n_agd_contrast} AgD stud{ifelse(n_agd_contrast == 1, 'y', 'ies')} (contrast-based)"
    )[c(n_ipd > 0, n_agd_arm > 0, n_agd_contrast > 0)],
    last = ", and ", sep = ", "), ".")
  }
  cat("\n")

  if (n_ipd > 0) {
    sec_header("IPD studies")
    print(s_ipd[1:min(n_ipd, n), ], right = FALSE, row.names = FALSE, max = 9999L)
    if (n_ipd > n) cglue(subtle(" ... plus {n_ipd - n} more studies"))
    cat("\n")
    cglue(" Outcome type: {x$outcome$ipd}",
          if (x$outcome$ipd == "ordered") " ({ncol(x$ipd$.r)} categories)" else "")
    # cat("\n")
  }

  if (n_agd_arm > 0) {
    sec_header("AgD studies (arm-based)")
    print(s_agd_arm[1:min(n_agd_arm, n), ], right = FALSE, row.names = FALSE, max = 9999L)
    if (n_agd_arm > n) cglue(subtle(" ... plus {n_agd_arm - n} more studies"))
    cat("\n")
    cglue(" Outcome type: {x$outcome$agd_arm}",
          if (x$outcome$agd_arm == "ordered") " ({ncol(x$agd_arm$.r)} categories)" else "")
    # cat("\n")
  }

  if (n_agd_contrast > 0) {
    sec_header("AgD studies (contrast-based)")
    print(s_agd_contrast[1:min(n_agd_contrast, n), ], right = FALSE, row.names = FALSE, max = 9999L)
    if (n_agd_contrast > n) cglue(subtle(" ... plus {n_agd_contrast - n} more studies"))
    cat("\n")
    cglue(" Outcome type: {x$outcome$agd_contrast}")
    # cat("\n")
  }

  sec_header()
  if (!is.null(x$classes)) {
    cglue("Total number of treatments: {length(x$treatments)}, in {nlevels(x$classes)} classes")
  } else {
    cglue("Total number of treatments: {length(x$treatments)}")
  }
  cglue("Total number of studies: {length(x$studies)}")
  cglue("Reference treatment is: {levels(x$treatments)[1]}")
  cglue("Network is {if (is_network_connected(x)) green('connected') else red('disconnected')}")

  invisible(x)
}

#' @rdname print.nma_data
#' @export
print.mlnmr_data <- function(x, ..., n = 10) {
  NextMethod()
  cat("\n")
  sec_header("Numerical integration")
  cglue("Numerical integration points available for {length(x$int_names)} covariates: ")
  cat(x$int_names, fill = TRUE, labels = " ")
  cglue("Number of numerical integration points: {x$n_int}")
  invisible(x)
}

#' Make section headers for print.nma_class
#'
#' @param s section header string
#' @param width line width
#' @param sep separator between s and surrounding dashes
#'
#' @noRd
sec_header <- function(s = "",
                       width = min(80, getOption("width")),
                       sep = ifelse(nchar(s), " ", "")) {
  s <- as.character(s)
  cat(subtle(strrep('-', width - nchar(s) - 2*nchar(sep))),
      bold(s),
      subtle("----"), "\n", sep = sep)
}

#' Cat out glue objects
#'
#' @param ... arguments passed to glue
#' @param sep string separator
#'
#' @noRd
cglue <- function(..., sep = "\n") {
  cat(glue::glue_data(.x = parent.frame(), ...), sep = sep)
}

#' Define crayon styles
#'
#' @noRd
subtle <- function(...) {
  if (require_pkg("crayon", error = FALSE))
    return(crayon::silver(...))
  else
    return(paste0(...))
}
bold <- function(...) {
  if (require_pkg("crayon", error = FALSE))
    return(crayon::bold(...))
  else
    return(paste0(...))
}
red <- function(...) {
  if (require_pkg("crayon", error = FALSE))
    return(crayon::red(...))
  else
    return(paste0(...))
}
green <- function(...) {
  if (require_pkg("crayon", error = FALSE))
    return(crayon::green(...))
  else
    return(paste0(...))
}

#' Convert networks to graph objects
#'
#' The method `as.igraph()` converts `nma_data` objects into the form used by
#' the \link[igraph:igraph-package]{igraph} package. The method `as_tbl_graph()`
#' converts `nma_data` objects into the form used by the
#' \link[ggraph:ggraph-package]{ggraph} and
#' \link[tidygraph:tidygraph-package]{tidygraph} packages.
#'
#' @param x An [nma_data] object to convert
#' @param ... Additional arguments
#' @param collapse Logical, collapse edges over studies? Default `TRUE`, only
#'   one edge is produced for each comparison (by IPD or AgD study type) with a
#'   `.nstudy` attribute giving the number of studies making that comparison. If
#'   `FALSE`, repeated edges are added for each study making the comparison.
#'
#' @return An `igraph` object for `as.igraph()`, a `tbl_graph` object for
#'   `as_tbl_graph()`.
#' @export
#'
#' @rdname graph_conversion
#'
#' @importFrom igraph as.igraph
#'
#' @template ex_smoking_network
#' @examples
#' # Convert to igraph object
#' igraph::as.igraph(smk_net)  # Edges combined by default
#' igraph::as.igraph(smk_net, collapse = FALSE)  # Without combining edges
#'
#' # Convert to tbl_graph object
#' tidygraph::as_tbl_graph(smk_net)  # Edges combined by default
#' tidygraph::as_tbl_graph(smk_net, collapse = FALSE)  # Without combining edges
as.igraph.nma_data <- function(x, ..., collapse = TRUE) {

  if (!rlang::is_bool(collapse))
    abort("`collapse` must be TRUE or FALSE.")

  if (has_ipd(x)) {
    e_ipd <- x$ipd %>%
      dplyr::distinct(.data$.study, .data$.trt) %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::group_modify(~make_contrasts(.x$.trt))

    if (collapse) {
      e_ipd <- e_ipd %>%
        dplyr::group_by(.data$.trt, .data$.trt_b) %>%
        dplyr::summarise(.nstudy = dplyr::n_distinct(.data$.study), .type = "IPD")
    } else {
      e_ipd$.type <- "IPD"
    }

    v_ipd <- x$ipd %>%
      dplyr::group_by(.data$.trt) %>%
      dplyr::summarise(.sample_size = dplyr::n())
  } else {
    e_ipd <- v_ipd <- tibble::tibble()
  }

  if (has_agd_arm(x) || has_agd_contrast(x)) {
    agd_all <- dplyr::bind_rows(x$agd_arm, x$agd_contrast)
    e_agd <- agd_all %>%
      dplyr::group_by(.data$.study) %>%
      dplyr::group_modify(~make_contrasts(.x$.trt))

    if (collapse) {
      e_agd <- e_agd %>%
        dplyr::group_by(.data$.trt, .data$.trt_b) %>%
        dplyr::summarise(.nstudy = dplyr::n_distinct(.data$.study), .type = "AgD")
    } else {
      e_agd$.type <- "AgD"
    }

    if (has_agd_sample_size(x)) {
      v_agd <- agd_all %>%
        dplyr::group_by(.data$.trt) %>%
        dplyr::summarise(.sample_size = sum(.data$.sample_size))
    } else {
      v_agd <- agd_all %>% dplyr::distinct(.data$.trt)
    }
  } else {
    e_agd <- v_agd <- tibble::tibble()
  }

  e_all <- dplyr::bind_rows(e_ipd, e_agd) %>%
    dplyr::rename(from = ".trt_b", to = ".trt") %>%
    dplyr::select("from", "to", dplyr::everything())

  if (collapse) {
    e_all <- e_all %>%
      dplyr::mutate(.nstudy = dplyr::if_else(is.na(.data$.nstudy), 0L, .data$.nstudy))
  }

  if (has_agd_sample_size(x)) {
    v_all <- dplyr::bind_rows(v_ipd, v_agd) %>%
      dplyr::group_by(.data$.trt) %>%
      dplyr::summarise(.sample_size = sum(.data$.sample_size, na.rm = TRUE)) %>%
      dplyr::arrange(.data$.trt)
  } else {
    v_all <- dplyr::bind_rows(v_ipd, v_agd) %>%
      dplyr::distinct(.data$.trt) %>%
      dplyr::arrange(.data$.trt)
  }

  if (!is.null(x$classes)) v_all$.trtclass <- x$classes

  g <- igraph::graph_from_data_frame(e_all, directed = FALSE, vertices = v_all)
  return(g)
}

#' @rdname graph_conversion
#'
#' @method as_tbl_graph nma_data
# Dynamically exported, see zzz.R
as_tbl_graph.nma_data <- function(x, ...) {
  require_pkg("tidygraph")
  return(tidygraph::as_tbl_graph(igraph::as.igraph(x, ...)))
}

#' Make treatment contrasts
#'
#' @param trt Vector of treatment codes
#'
#' @noRd
make_contrasts <- function(trt) {
  if (length(trt) < 2) {
    return(dplyr::tibble(.trt = trt, .trt_b = trt))
  }

  contrs <- utils::combn(sort(trt), 2)
  return(dplyr::tibble(.trt = contrs[2,],
                       .trt_b = contrs[1,]))
}

#' Get default reference treatment
#'
#' @param network An `nma_data` object, as created by the functions `set_*()`,
#'   `combine_network()`, or `add_integration()`
#' @param ... Other arguments (unused)
#'
#' @return String
#' @noRd
get_default_trt_ref <- function(network, ...) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("Expecting an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  # Prefer longer follow-up for survival outcomes
  surv_dat <- dplyr::bind_rows(
    if (identical(network$outcome$ipd, "survival")) dplyr::select(network$ipd, ".trt", ".Surv") else NULL,
    if (identical(network$outcome$agd_arm, "survival")) tidyr::unnest(network$agd_arm, cols = ".Surv") %>% dplyr::select(".trt", ".Surv") else NULL
  )

  if (nrow(surv_dat) < 1) {
    max_fu <- 0
  } else {
    max_fu <- dplyr::mutate(surv_dat, !!! get_Surv_data(surv_dat$.Surv)) %>%
      dplyr::group_by(.data$.trt) %>%
      dplyr::summarise(max_fu = max(.data$time)) %>%
      dplyr::pull("max_fu")
  }

  g_nc <- igraph::as.igraph(network, collapse = FALSE)

  nodes <-
    tibble::tibble(
      .trt = network$treatments,
      degree = igraph::degree(g_nc),
      betweenness = igraph::betweenness(g_nc),
      max_fu = max_fu
    ) %>%
    dplyr::arrange(dplyr::desc(.data$degree),
                   dplyr::desc(.data$betweenness),
                   dplyr::desc(.data$max_fu),
                   .data$.trt)

  return(as.character(nodes[[1, ".trt"]]))
}

#' Check network connectedness
#'
#' Check whether a network is connected - whether there is a path of study
#' evidence linking every pair of treatments in the network.
#'
#' @param network An `nma_data` object, as created by the functions `set_*()` or
#'   `combine_network()`.
#'
#' @return Logical `TRUE` or `FALSE`
#' @export
#'
#' @details Models will still run with disconnected networks. However, estimated
#'   relative effects between treatments across disconnected parts of the
#'   network will be entirely based on the prior distribution (typically very
#'   uncertain), as there is no information to update the prior distribution.
#'   Relative effects within each connected sub-network will be estimated as if
#'   each sub-network had been analysed separately.
#'
#' @examples ## Smoking cessation
#' @template ex_smoking_network
#' @examples is_network_connected(smk_net)  # TRUE, network is connected
#' @examples
#' ## A disconnected network
#' disc_net <- set_agd_arm(smoking[smoking$studyn %in% c(15, 21), ],
#'                         study = studyn,
#'                         trt = trtc,
#'                         r = r,
#'                         n = n)
#' is_network_connected(disc_net)  # FALSE, network is disconnected
#' disc_net
#' plot(disc_net)
is_network_connected <- function(network) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("Expecting an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  return(igraph::is_connected(igraph::as.igraph(network)))
}

#' Direct and indirect evidence
#'
#' Determine whether two treatments in a network are connected by direct and/or
#' indirect evidence, and generate a list of comparisons with both direct and
#' indirect evidence (i.e. potential inconsistency) for node-splitting.
#'
#' @param network An `nma_data` object, as created by the functions `set_*()` or
#'   `combine_network()`.
#' @param include_consistency Logical, whether to include a row of `NA`s to
#'   indicate that a consistency model (i.e. a model with no node-splitting)
#'   should also be fitted by the [nma()] function. Default is `FALSE` when
#'   calling `get_nodesplits()` by hand, and [nma()] sets this to `TRUE` by
#'   default.
#'
#' @details The list of comparisons for node-splitting is generated following
#'   the algorithm of \insertCite{Valkenhoef2016;textual}{multinma}. A
#'   comparison between two treatments has the potential for inconsistency, and
#'   is thus considered for node-splitting, if the comparison has both direct
#'   evidence and *independent* indirect evidence.
#'
#'   The notion of independent indirect evidence is necessary when multi-arm
#'   trials are present, since by design these trials are internally consistent.
#'   A comparison between two treatments has independent indirect evidence if,
#'   after removing all studies comparing the two treatments from the network,
#'   the two treatments are still connected by a path of evidence. This is the
#'   criterion considered by the `has_indirect()` function.
#'
#' @return For `has_direct()` and `has_indirect()`, a single logical value. For
#'   `get_nodesplits()`, a data frame with two columns giving the comparisons
#'   for node-splitting.
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#' # Parkinsons example
#' park_net <- set_agd_arm(parkinsons,
#'                         study = studyn,
#'                         trt = trtn,
#'                         y = y,
#'                         se = se,
#'                         trt_ref = 1)
#'
#' # View the network plot
#' plot(park_net)
#'
#' # The 4 vs. 5 comparison is a spur on the network
#' has_direct(park_net, 4, 5)
#' has_indirect(park_net, 4, 5)
#'
#' # 1 and 5 are not directly connected
#' has_direct(park_net, 1, 5)
#' has_indirect(park_net, 1, 5)
#'
#' # The 1 vs. 2 comparison does not have independent indirect evidence, since
#' # the 1-2-4 loop is a multi-arm study
#' has_indirect(park_net, 1, 2)
#'
#' # Get a list of comparisons with potential inconsistency for node-splitting
#' get_nodesplits(park_net)
#'
#' # See van Valkenhoef (2016) for a discussion of this example

get_nodesplits <- function(network, include_consistency = FALSE) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("`network` must be an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  # Check other arguments
  if (!rlang::is_bool(include_consistency))
    abort("`include_consistency` must be a logical value (TRUE or FALSE).")

  # Determine which contrasts to split using algorithm of van Valkenhoef,
  # i.e. having both direct and *independent* indirect evidence
  comparisons <- igraph::as_edgelist(igraph::as.igraph(network))
  colnames(comparisons) <- c("trt1", "trt2")

  out <- dplyr::as_tibble(comparisons) %>%
    # Remove edges of treatment against itself (from studies with multiple arms of the same treatment)
    dplyr::filter(.data$trt1 != .data$trt2) %>%
    dplyr::mutate(trt1 = factor(.data$trt1, levels = levels(network$treatments)),
                  trt2 = factor(.data$trt2, levels = levels(network$treatments))) %>%
    dplyr::arrange(.data$trt1, .data$trt2) %>%
    dplyr::rowwise() %>%
    dplyr::filter(has_direct(network, .data$trt1, .data$trt2) &&
                    has_indirect(network, .data$trt1, .data$trt2)) %>%
    dplyr::ungroup()

  # Add an NA row for the consistency model if include_consistency = TRUE
  if (include_consistency) {
    out <- dplyr::add_row(out, trt1 = NA, trt2 = NA, .before = 1)
  }

  return(out)
}

#' @param trt1,trt2 Treatments, each as a single integer, string, or factor
#' @export
#' @rdname get_nodesplits
has_direct <- function(network, trt1, trt2) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("`network` must be an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  # Check treatments
  if (!rlang::is_scalar_atomic(trt1))
    abort("`trt1` should be a single integer, string, or factor, naming a treatment.")
  if (!rlang::is_scalar_atomic(trt2))
    abort("`trt2` should be a single integer, string, or factor, naming a treatment.")

  trt1 <- as.character(trt1)
  trt2 <- as.character(trt2)
  lvls_trt <- levels(network$treatments)

  if (! trt1 %in% lvls_trt)
    abort(sprintf("`trt1` does not match a treatment in the network.\nSuitable values are: %s",
                  ifelse(length(lvls_trt) <= 5,
                         paste0(lvls_trt, collapse = ", "),
                         paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

  if (! trt2 %in% lvls_trt)
    abort(sprintf("`trt2` does not match a treatment in the network.\nSuitable values are: %s",
                  ifelse(length(lvls_trt) <= 5,
                         paste0(lvls_trt, collapse = ", "),
                         paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

  if (trt1 == trt2)
    abort("`trt1` and `trt2` cannot be the same treatment.")

  # Convert to igraph and return whether adjacent nodes or not
  return(igraph::are_adjacent(igraph::as.igraph(network), trt1, trt2))
}

#' @rdname get_nodesplits
#' @export
has_indirect <- function(network, trt1, trt2) {

  # Check network
  if (!inherits(network, "nma_data")) {
    abort("`network` must be an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")
  }

  if (all(purrr::map_lgl(network, is.null))) {
    abort("Empty network.")
  }

  # Check treatments
  if (!rlang::is_scalar_atomic(trt1))
    abort("`trt1` should be a single integer, string, or factor, naming a treatment.")
  if (!rlang::is_scalar_atomic(trt2))
    abort("`trt2` should be a single integer, string, or factor, naming a treatment.")

  trt1 <- as.character(trt1)
  trt2 <- as.character(trt2)
  lvls_trt <- levels(network$treatments)

  if (! trt1 %in% lvls_trt)
    abort(sprintf("`trt1` does not match a treatment in the network.\nSuitable values are: %s",
                  ifelse(length(lvls_trt) <= 5,
                         paste0(lvls_trt, collapse = ", "),
                         paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

  if (! trt2 %in% lvls_trt)
    abort(sprintf("`trt2` does not match a treatment in the network.\nSuitable values are: %s",
                  ifelse(length(lvls_trt) <= 5,
                         paste0(lvls_trt, collapse = ", "),
                         paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))

  if (trt1 == trt2)
    abort("`trt1` and `trt2` cannot be the same treatment.")


  # Create network with studies on both trt1 and trt2 removed
  studies <- dplyr::bind_rows(
    if (identical(network$outcome$agd_arm, "survival")) dplyr::select(network$agd_arm, -.data$.Surv) else network$agd_arm,
    network$agd_contrast,
    if (identical(network$outcome$ipd, "survival")) dplyr::select(network$ipd, -.data$.Surv) else network$ipd
  ) %>%
    dplyr::distinct(.data$.study, .data$.trt)

  ind_studies <- studies %>%
    dplyr::group_by(.data$.study) %>%
    dplyr::filter(! all(c(trt1, trt2) %in% .data$.trt))

  # Catch case where the reduced network is empty
  if (nrow(ind_studies) == 0) return(FALSE)

  ind_e <- dplyr::group_modify(ind_studies, ~make_contrasts(.x$.trt)) %>%
    dplyr::transmute(from = .data$.trt_b,
                     to = .data$.trt,
                     .data$.study)
  ind_v <- dplyr::ungroup(studies) %>% dplyr::distinct(.data$.trt)

  g <- igraph::graph_from_data_frame(ind_e, directed = FALSE, vertices = ind_v)

  # There is indirect evidence if the treatments are still connected in this
  # reduced network
  return(!is.infinite(igraph::distances(g, trt1, trt2)[1, 1]))
}

#' Network plots
#'
#' Create a network plot from a `nma_data` network object.
#'
#' @param x A [nma_data] object to plot
#' @param ... Additional arguments passed to
#'   \code{\link[ggraph:ggraph]{ggraph()}} and on to the layout function
#' @param layout The type of layout to create. Any layout accepted by
#'   \code{\link[ggraph:ggraph]{ggraph()}} may be used, including all of the
#'   layout functions provided by \link[igraph:igraph-package]{igraph}.
#' @param circular Whether to use a circular representation. See
#'   \code{\link[ggraph:ggraph]{ggraph()}}.
#' @param weight_edges Weight edges by the number of studies? Default is `TRUE`.
#' @param weight_nodes Weight nodes by the total sample size? Default is
#'   `FALSE`.
#' @param show_trt_class Colour treatment nodes by class, if `trt_class` is set?
#'   Default is `FALSE`.
#' @param level Display network at the `"treatment"` (default) or `"class"` level.
#' @param nudge Numeric value to nudge the treatment labels away from the nodes
#'   when `weight_nodes = TRUE`. Default is `0` (no adjustment to label
#'   position). A small value like `0.1` is usually sufficient.
#'
#' @details The default is equivalent to `layout = "linear"` and `circular =
#'   TRUE`, which places the treatment nodes on a circle in the order defined by
#'   the treatment factor variable. An alternative layout which may give good
#'   results for simple networks is `"sugiyama"`, which attempts to minimise the
#'   number of edge crossings.
#'
#'   `weight_nodes = TRUE` requires that sample sizes have been specified for
#'   any aggregate data in the network, using the `sample_size` option of
#'   `set_agd_*()`.
#'
#' @return A `ggplot` object, as produced by
#'   \code{\link[ggraph:ggraph]{ggraph()}}.
#' @export
#'
#' @examples ## Stroke prevention in atrial fibrillation
#' # Setting up the network
#' af_net <- set_agd_arm(atrial_fibrillation,
#'                       study = studyc,
#'                       trt = abbreviate(trtc, minlength = 3),
#'                       r = r,
#'                       n = n,
#'                       trt_class = trt_class)
#' af_net
#'
#' # Basic plot
#' plot(af_net)
#'
#' # Turn off weighting edges by number of studies
#' plot(af_net, weight_edges = FALSE)
#'
#' # Turn on weighting nodes by sample size
#' plot(af_net, weight_nodes = TRUE)
#'
#' # Colour treatment nodes by class
#' plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE)
#'
#' # Nudge the treatment labels away from the nodes
#' plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE, nudge = 0.1)
#'
#' # Output may be customised using standard ggplot commands
#' # For example, to display the legends below the plot:
#' plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE) +
#'   ggplot2::theme(legend.position = "bottom",
#'                  legend.box = "vertical",
#'                  legend.margin = ggplot2::margin(0, 0, 0, 0),
#'                  legend.spacing = ggplot2::unit(0.5, "lines"))
#'
#' # Choosing a different ggraph layout, hiding some legends
#' plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE,
#'      layout = "star") +
#'   ggplot2::guides(edge_width = "none", size = "none")
#'
plot.nma_data <- function(x, ..., layout, circular,
                          weight_edges = TRUE,
                          weight_nodes = FALSE,
                          show_trt_class = FALSE,
                          level = c("treatment", "class"),
                          nudge = 0) {
  level <- rlang::arg_match(level)
  if (missing(layout) && missing(circular)) {
    layout <- "linear"
    circular <- TRUE
  } else if (missing(layout)) {
    layout <- "linear"
  } else if (missing(circular)) {
    circular <- FALSE
  }

  if (!rlang::is_bool(weight_edges))
    abort("`weight_edges` must be TRUE or FALSE.")

  if (!rlang::is_bool(weight_nodes))
    abort("`weight_nodes` must be TRUE or FALSE.")

  if (weight_nodes && !has_agd_sample_size(x))
    abort(paste("AgD study sample sizes not specified in network, cannot weight nodes.",
                "Specify `sample_size` in set_agd_*(), or set weight_nodes = FALSE.", sep = "\n"))

  if (!rlang::is_bool(show_trt_class))
    abort("`show_trt_class` must be TRUE or FALSE.")

  if (show_trt_class && is.null(x$classes))
    abort(paste("Treatment classes not specified in network.",
                "Specify `trt_class` in set_*(), or set show_trt_class = FALSE.", sep = "\n"))

  if(level == "class" && is.null(x$classes))
    abort(paste("Treatment classes not specified in network.",
                'Specify `trt_class` in set_*(), or use level = "treatment"', sep = "\n"))

  if (!rlang::is_double(nudge, n = 1, finite = TRUE))
    abort("`nudge` must be a single numeric value")

  # network at the class level
  if (level == "class"){
    if(has_agd_contrast(x)){
      x$agd_contrast$.trt <- x$agd_contrast$.trtclass
    }
    if(has_agd_arm(x)){
      x$agd_arm$.trt <- x$agd_arm$.trtclass
    }
    if(has_ipd(x)){
      x$ipd$.trt <- x$ipd$.trtclass
    }
    x$classes <- forcats::fct_unique(x$classes)
    x$treatments <- x$classes
  }

  dat_mixed <- has_ipd(x) && (has_agd_arm(x) || has_agd_contrast(x))
  g <- ggraph::ggraph(igraph::as.igraph(x), layout = layout, circular = circular, ...)

  if (weight_edges) {
    g <- g +
      ggraph::geom_edge_fan(ggplot2::aes(edge_width = .data$.nstudy,
                                       edge_colour = .data$.type),
                            lineend = "round") +
      ggraph::scale_edge_width_continuous("Number of studies",
                                          breaks = breaks_integer(positive = TRUE),
                                          limits = function(x) range(breaks_integer(positive = TRUE)(x)))
  } else {
    g <- g +
      ggraph::geom_edge_fan(ggplot2::aes(edge_colour = .data$.type),
                            edge_width = 1,
                            lineend = "round")
  }

  if (weight_nodes) {
    if (show_trt_class) {
      g <- g +
        ggraph::geom_node_point(ggplot2::aes(size = .data$.sample_size,
                                             fill = .data$.trtclass,
                                             colour = .data$.trtclass),
                                shape = 21)
    } else {
      g <- g +
        ggraph::geom_node_point(ggplot2::aes(size = .data$.sample_size),
                                fill = "grey90", colour = "grey60",
                                shape = 21)
    }

    # Calculate nudge positions
    if (nudge == 0) {
      pos <- ggplot2::position_identity()
    } else {
      if (circular || (rlang::is_string(layout) && layout %in% c("circle", "star"))) {
        pos <- ggplot2::position_nudge(x = nudge * g$data$x,
                                       y = nudge * g$data$y)
      } else {
        pos <- ggplot2::position_nudge(x = nudge * sign(g$data$x - mean(g$data$x)),
                                       y = nudge * sign(g$data$y - mean(g$data$y)))
      }
    }

    g <- g +
      ggraph::geom_node_text(ggplot2::aes(label = .data$name),
                             hjust = "outward", vjust = "outward",
                             position = pos) +
      ggplot2::scale_size_area("Total sample size", max_size = 12)

  } else {
    if (show_trt_class) {
      g <- g +
        ggraph::geom_node_label(ggplot2::aes(label = .data$name,
                                             fill = .data$.trtclass))
    } else {
      g <- g +
        ggraph::geom_node_label(ggplot2::aes(label = .data$name),
                                fill = "grey90")
    }
  }

  if (show_trt_class) {
    g <- g + ggplot2::scale_fill_discrete(name = "Treatment Class", aesthetics = c("fill", "colour"))
  }

  g <- g +
    ggraph::scale_edge_colour_manual("Data", values = c(AgD = "#113259", IPD = "#55A480"),
                                     guide = if (dat_mixed) "legend" else "none") +
    ggraph::theme_graph(base_family = "") +
    ggplot2::coord_fixed(clip = "off")
  return(g)
}

#' Automatic breaks for integer scales
#'
#' Compute automatic breaks for integer scales, making sure that the breaks are
#' also integers.
#'
#' @param prefer_n Set of preferred numbers of break points.
#' @param extend Extend the breaks beyond the range of the data? If `TRUE`
#'   ensures that breaks are equally spaced. Default `TRUE`.
#' @param positive Create breaks for strictly positive integers? Default `TRUE`.
#' @param ... Unused.
#'
#' @details If `extend = TRUE` (the default), breaks may lie outside of the
#'   range of the data, but are guaranteed to be equally spaced. If `extend =
#'   FALSE`, the limits of the breaks are equal to the smallest and largest
#'   values in the data, but within this range breaks may not be equally spaced.
#'
#' @return Returns a function to compute breaks, as required by the `breaks`
#'   argument to the `scale_*_continuous` functions in `ggplot2`.
#' @noRd
#'
#' @examples
#' breaks_integer()(1:9)
#' breaks_integer()(1:18) # extended, equally spaced
#' breaks_integer(extend = FALSE)(1:18) # un-extended, unequally spaced
#' breaks_integer(positive = FALSE)(-1:10) # allow negative values
breaks_integer <- function(prefer_n = c(5, 4, 3, 6), extend = TRUE, positive = TRUE, ...) {
  if (!rlang::is_integerish(prefer_n, finite = TRUE) || any(prefer_n < 2))
    abort("`prefer_n` must be a vector of integers greater than 1.")
  if (!rlang::is_bool(extend))
    abort("`extend` must be a logical value TRUE or FALSE.")
  if (!rlang::is_bool(positive))
    abort("`positive` must be a logical value TRUE or FALSE.")

  def_prefer_n <- prefer_n
  def_extend <- extend
  def_positive <- positive

  function(x, prefer_n = def_prefer_n, extend = def_extend, positive = def_positive, ...) {
    r <- range(x)
    l <- diff(r)

    if (l < max(prefer_n) - 1) {
      return(seq.int(r[1], r[2]))
    }

    n <- prefer_n[which.min(l %% (prefer_n - 1))]
    if (!extend || l %% (n - 1) == 0) {
      return(unique(round(seq.int(r[1], r[2], length.out = n))))
    } else {
      remainders <- l %% (prefer_n - 1)
      shifts <- prefer_n - 1 - remainders
      n <- prefer_n[which.min(shifts)]
      shift <- min(shifts)
      s_l <- if (positive) min(floor(shift / 2), r[1] - 1) else floor(shift / 2)
      s_u <- shift - s_l
      return(seq.int(r[1] - s_l, r[2] + s_u, length.out = n))
    }
  }
}

library(dplyr)
library(tidyr)

test_that("prefer_n argument", {
  m <- "vector of integers greater than 1"
  expect_error(breaks_integer(prefer_n = "a"), m)
  expect_error(breaks_integer(prefer_n = -1), m)
  expect_error(breaks_integer(prefer_n = 0), m)
  expect_error(breaks_integer(prefer_n = 1:2), m)
  expect_error(breaks_integer(prefer_n = 1.5), m)
  expect_error(breaks_integer(prefer_n = Inf), m)
  expect_error(breaks_integer(prefer_n = list()), m)
  expect_error(breaks_integer(prefer_n = NA), m)
  expect_error(breaks_integer(prefer_n = NULL), m)
})

test_that("extend argument", {
  m <- "logical value TRUE or FALSE"
  expect_error(breaks_integer(extend = "a"), m)
  expect_error(breaks_integer(extend = 1), m)
  expect_error(breaks_integer(extend = list()), m)
  expect_error(breaks_integer(extend = NA), m)
  expect_error(breaks_integer(extend = NULL), m)
  expect_error(breaks_integer(extend = c(TRUE, FALSE)), m)
})

test_that("positive argument", {
  m <- "logical value TRUE or FALSE"
  expect_error(breaks_integer(positive = "a"), m)
  expect_error(breaks_integer(positive = 1), m)
  expect_error(breaks_integer(positive = list()), m)
  expect_error(breaks_integer(positive = NA), m)
  expect_error(breaks_integer(positive = NULL), m)
  expect_error(breaks_integer(positive = c(TRUE, FALSE)), m)
})

bdat1 <- tibble(x0 = 1:10001) %>%
  rowwise() %>%
  mutate(x = list(1:x0),
         b_int = list(breaks_integer(extend = FALSE, positive = TRUE)(x)),
         b_int_ext = list(breaks_integer(extend = TRUE, positive = TRUE)(x))
  )

bdat2 <- tibble(x0 = 2:10001) %>%
  rowwise() %>%
  mutate(x = list(2:x0),
         b_int = list(breaks_integer(extend = FALSE, positive = TRUE)(x)),
         b_int_ext = list(breaks_integer(extend = TRUE, positive = TRUE)(x))
)

bdat5 <- tibble(x0 = 5:10001) %>%
  rowwise() %>%
  mutate(x = list(5:x0),
         b_int = list(breaks_integer(extend = FALSE, positive = TRUE)(x)),
         b_int_ext = list(breaks_integer(extend = TRUE, positive = TRUE)(x))
)

bdat_neg <- tibble(x0 = -100:100) %>%
  rowwise() %>%
  mutate(x = list(-100:x0),
         b_int = list(breaks_integer(extend = FALSE, positive = FALSE)(x)),
         b_int_ext = list(breaks_integer(extend = TRUE, positive = FALSE)(x))
  )

test_that("Breaks are equally spaced with extend = TRUE", {
  expect_true(all(purrr::map_lgl(bdat1$b_int_ext[-1], ~n_distinct(diff(.)) == 1)))
  expect_true(all(purrr::map_lgl(bdat2$b_int_ext[-1], ~n_distinct(diff(.)) == 1)))
  expect_true(all(purrr::map_lgl(bdat5$b_int_ext[-1], ~n_distinct(diff(.)) == 1)))
  expect_true(all(purrr::map_lgl(bdat_neg$b_int_ext[-1], ~n_distinct(diff(.)) == 1)))
})

test_that("Breaks are integers, and > 0 when positive = TRUE", {
  expect_true(all(purrr::map_lgl(bdat1$b_int_ext, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat2$b_int_ext, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat5$b_int_ext, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat1$b_int, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat2$b_int, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat5$b_int, ~all(rlang::is_integerish(.)) && all(. > 0))))
  expect_true(all(purrr::map_lgl(bdat_neg$b_int, ~all(rlang::is_integerish(.)))))
  expect_true(all(purrr::map_lgl(bdat_neg$b_int_ext, ~all(rlang::is_integerish(.)))))
})

library(multinma)
library(dplyr)

test_that("network must be a network", {
  m <- "`network` must be an `nma_data` object"
  expect_error(has_direct(list()), m)
  expect_error(has_indirect(list()), m)
  expect_error(get_nodesplits(list()), m)
})

thrombo_net <- set_agd_arm(thrombolytics, studyn, trtc,
                           r = r, n = n)
thrombo_net2 <- set_agd_arm(thrombolytics, studyn, trtn,
                            r = r, n = n)

test_that("treatments must be scalars", {
  m <- "should be a single integer, string, or factor, naming a treatment"

  expect_error(has_direct(thrombo_net, 1:2, "SK"), m)
  expect_error(has_direct(thrombo_net, "SK", 1:2), m)
  expect_error(has_indirect(thrombo_net, 1:2, "SK"), m)
  expect_error(has_indirect(thrombo_net, "SK", 1:2), m)
})

test_that("treatments must be in network", {
  m1 <- "`trt1` does not match a treatment in the network"
  m2 <- "`trt2` does not match a treatment in the network"

  expect_error(has_direct(thrombo_net, "bad", "SK"), m1)
  expect_error(has_direct(thrombo_net, "SK", "bad"), m2)
  expect_error(has_indirect(thrombo_net, "bad", "SK"), m1)
  expect_error(has_indirect(thrombo_net, "SK", "bad"), m2)

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = c("bad", "SK"),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    "The `nodesplit` treatment comparison should match two treatments in the network")
})

test_that("trt1 and trt2 must be different", {
  m <- "`trt1` and `trt2` cannot be the same treatment"

  expect_error(has_direct(thrombo_net, "SK", "SK"), m)
  expect_error(has_indirect(thrombo_net, "SK", "SK"), m)

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = c("SK", "SK"),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    "`nodesplit` comparison cannot be the same treatment against itself")
})

test_that("has_direct and has_indirect outputs are correct", {
  expect_identical(has_direct(thrombo_net, "TNK", "Acc t-PA"), TRUE)
  expect_identical(has_direct(thrombo_net, "TNK", "t-PA"), FALSE)
  expect_identical(has_indirect(thrombo_net, "TNK", "Acc t-PA"), FALSE)
  expect_identical(has_indirect(thrombo_net, "TNK", "t-PA"), TRUE)

  expect_identical(has_direct(thrombo_net2, 6, 3), TRUE)
  expect_identical(has_direct(thrombo_net2, 6, 2), FALSE)
  expect_identical(has_indirect(thrombo_net2, 6, 3), FALSE)
  expect_identical(has_indirect(thrombo_net2, 6, 2), TRUE)
})

# Output from gemtc::mtc.nodesplit.comparisons() on thrombo network
ns_thrombo_gemtc <- tibble::tribble(
  ~trt1, ~trt2,
  1,     2,
  1,     3,
  1,     5,
  1,     7,
  1,     8,
  1,     9,
  2,     7,
  2,     8,
  2,     9,
  3,     4,
  3,     5,
  3,     7,
  3,     8,
  3,     9
) %>%
  mutate(trt1 = factor(trt1, levels = levels(thrombo_net2$treatments)),
         trt2 = factor(trt2, levels = levels(thrombo_net2$treatments)))

test_that("get_nodesplits() produces correct output for thombolytics network", {
  expect_identical(get_nodesplits(thrombo_net2), ns_thrombo_gemtc)
})

park_net <- set_agd_arm(parkinsons, studyn, trtn, y = y, se = se, trt_ref = 1)

# Compare to results in van Valkenhoef paper
ns_park_vv <- tibble::tribble(
  ~trt1, ~trt2,
  1, 3,
  1, 4,
  2, 4,
  3, 4
) %>%
  mutate(trt1 = factor(trt1, levels = levels(park_net$treatments)),
         trt2 = factor(trt2, levels = levels(park_net$treatments)))

test_that("get_nodesplits() produces correct output for parkinsons network", {
  expect_identical(get_nodesplits(park_net), ns_park_vv)
})

test_that("get_nodesplits() handles repeated treatment arms", {
  thrombo_net_rep <- set_agd_arm(rbind(thrombolytics, thrombolytics[c(2, 5), ]),
                                 studyn, trtn,
                                 r = r, n = n)

  expect_identical(get_nodesplits(thrombo_net_rep), ns_thrombo_gemtc)
})

onestudy <- data.frame(study = 1, trt = 1:3, r = 1, n = 1)
pair_net <- set_agd_arm(onestudy[1:2, ], study, trt, r = r, n = n)
multi_net <- set_agd_arm(onestudy, study, trt, r = r, n = n)

test_that("get_nodesplits() returns an empty tibble if no splits to be done", {
  expect_identical(get_nodesplits(pair_net),
                   tibble(trt1 = factor(levels = levels(pair_net$treatments)),
                          trt2 = factor(levels = levels(pair_net$treatments))))

  expect_identical(get_nodesplits(multi_net),
                   tibble(trt1 = factor(levels = levels(multi_net$treatments)),
                          trt2 = factor(levels = levels(multi_net$treatments))))
})

test_that("has_direct and has_indirect work with one study / pairwise MA", {
  expect_identical(has_direct(pair_net, 1, 2), TRUE)
  expect_identical(has_direct(multi_net, 1, 2), TRUE)

  expect_identical(has_indirect(pair_net, 1, 2), FALSE)
  expect_identical(has_indirect(multi_net, 1, 2), FALSE)
})

test_that("nma() nodesplit error if can't split given comparison", {
  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = c("TNK", "t-PA"),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    "no direct evidence"
  )

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = c("TNK", "Acc t-PA"),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    "no independent indirect evidence"
  )
})

test_that("nma() nodesplit error if no comparisons to split", {
  m <- "No comparisons to node-split"

  expect_error(
    nma(pair_net,
        consistency = "nodesplit",
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m
  )

  expect_error(
    nma(multi_net,
        consistency = "nodesplit",
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m
  )
})

test_that("nma() nodesplit warning about ignoring comparisons", {
  expect_warning(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = data.frame(trt1 = "Acc t-PA", trt2 = c("TNK", "ASPAC")),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10),
        test_grad = TRUE),
    "Ignoring node-split comparisons.+Acc t-PA vs\\. TNK"
  )
})

test_that("nma() nodesplit argument validation", {
  m1 <- "The data frame passed to `nodesplit` should have two columns"

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = data.frame(),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m1
  )

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = data.frame(a = 1, b = 2, c = 3),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m1
  )

  m2 <- "`nodesplit` should either be a length 2 vector or a 2 column data frame"

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = list(),
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m2
  )

  expect_error(
    nma(thrombo_net,
        consistency = "nodesplit",
        nodesplit = 1:3,
        prior_intercept = normal(scale = 10),
        prior_trt = normal(scale = 10)),
    m2
  )
})

thrombo_ns1 <- nma(thrombo_net,
                   consistency = "nodesplit",
                   nodesplit = c("SK", "t-PA"),
                   prior_intercept = normal(scale = 10),
                   prior_trt = normal(scale = 10),
                   test_grad = TRUE)

thrombo_ns2<- nma(thrombo_net,
                  consistency = "nodesplit",
                  nodesplit = data.frame(trt1 = "SK", trt2 = "t-PA"),
                  prior_intercept = normal(scale = 10),
                  prior_trt = normal(scale = 10),
                  test_grad = TRUE)

test_that("nma() nodesplit produces correct classes", {
  expect_s3_class(thrombo_ns1, "nma_nodesplit")
  expect_s3_class(thrombo_ns2, "nma_nodesplit_df")
})

test_that("summary method consistency argument requires consistency model", {
  m <- "should be a fitted consistency model"
  expect_error(summary(thrombo_ns1, consistency = thrombo_ns1), m)
  expect_error(summary(thrombo_ns2, consistency = thrombo_ns1), m)
  expect_error(summary(thrombo_ns1, consistency = "bad"), m)
  expect_error(summary(thrombo_ns2, consistency = "bad"), m)
})

test_that("summary method checks nodesplit and consistency models are compatible", {
  skip_on_cran()

  m <- "does not match the node-splitting model"

  suppressWarnings({
    thrombo_ns1 <- nma(thrombo_net,
                       consistency = "nodesplit",
                       nodesplit = c("SK", "t-PA"),
                       prior_intercept = normal(scale = 10),
                       prior_trt = normal(scale = 10),
                       iter = 1)

    thrombo_ns2<- nma(thrombo_net,
                      consistency = "nodesplit",
                      nodesplit = data.frame(trt1 = "SK", trt2 = "t-PA"),
                      prior_intercept = normal(scale = 10),
                      prior_trt = normal(scale = 10),
                      iter = 1)

    thrombo_re <- nma(thrombo_net,
                      trt_effects = "random",
                      prior_intercept = normal(scale = 10),
                      prior_trt = normal(scale = 10),
                      prior_het = half_normal(2),
                      iter = 1)

    thrombo_reg <- nma(thrombo_net,
                       regression = ~I(rnorm(102)),
                       prior_intercept = normal(scale = 10),
                       prior_trt = normal(scale = 10),
                       prior_reg = normal(scale = 10),
                       iter = 1)
  })

  expect_error(summary(thrombo_ns1, consistency = thrombo_re), m)
  expect_error(summary(thrombo_ns2, consistency = thrombo_re), m)
  expect_error(summary(thrombo_ns1, consistency = thrombo_reg), m)
  expect_error(summary(thrombo_ns2, consistency = thrombo_reg), m)
})

#' Target average acceptance probability
#'
#' The Stan control argument `adapt_delta` sets the target average acceptance
#' probability for the No-U-Turn Sampler (NUTS) used by Stan.
#'
#' @details The default value of `adapt_delta` used by [nma()] is 0.8 for fixed
#'   effect models, and 0.95 for random effects models.
#'
#'   You should not need to change `adapt_delta` unless you see a warning
#'   message about divergent transitions. Increasing `adapt_delta` from the
#'   default to a value closer to 1 means that Stan will use a smaller step
#'   size, making sampling slower but more robust, and resulting in fewer
#'   divergent transitions.
#'
#'   For more details see the Stan documentation available from
#'   \url{https://mc-stan.org/users/documentation/}.
#'
#' @name adapt_delta
#'
NULL

#' The nma_dic class
#'
#' The `nma_dic` class contains details of the Deviance Information Criterion
#' (DIC), produced using the [dic()] function.
#'
#' @rdname nma_dic-class
#' @name nma_dic-class
#' @aliases nma_dic
#'
#' @details Objects of class `nma_dic` have the following components:
#'   \describe{
#'   \item{`dic`}{The DIC value}
#'   \item{`pd`, `pv`}{The effective number of parameters}
#'   \item{`resdev`}{The total residual deviance}
#'   \item{`pointwise`}{A list of data frames containing the pointwise
#'   contributions for the IPD and AgD.}
#'   \item{`resdev_array`}{A 3D MCMC array \[Iterations, Chains, Parameters\] of
#'   posterior residual deviance samples.}
#'   }
#'
#' @seealso [dic()], [print.nma_dic()], [plot.nma_dic()].
#'
NULL

#' Methods for `nma_dic` objects
#'
#' The `print()` method prints details of DIC model fit statistics, computed by
#' the  [dic()] function. The `as.data.frame()`, `as_tibble()`, and `as.tibble()` methods return the
#' pointwise contributions to the DIC and $p_D$ in a data frame or tibble. The `as.array()` and `as.matrix()`
#' methods returns a 3D MCMC array (as class [mcmc_array]) or matrix of posterior draws of the residual deviances.
#'
#'
#' @param x An object of class [nma_dic]
#' @param digits Integer number of digits to display
#' @param ... Additional arguments passed on to other methods
#'
#' @rdname nma_dic-methods
#'
#' @seealso [dic()], [plot.nma_dic()]
#'
#' @return A `data.frame` for `as.data.frame()`, a `tbl_df` for `as.tibble()`
#'   and `as_tibble()`, a `matrix` for `as.matrix()`, and an `mcmc_array` for
#'   `as.array()`.
#'
#'   The `print()` method returns `x` invisibly.
#' @export
#'
print.nma_dic <- function(x, digits = 1, ...) {
  if (!rlang::is_scalar_integerish(digits)) abort("`digits` must be a single integer.")

  penalty <- attr(x, "penalty")

  n <- sum(if (rlang::has_name(x$pointwise$ipd, "df")) x$pointwise$ipd$df else nrow(x$pointwise$ipd),
           if (rlang::has_name(x$pointwise$agd_arm, "df")) x$pointwise$agd_arm$df else nrow(x$pointwise$agd_arm),
           if (rlang::has_name(x$pointwise$agd_contrast, "df")) x$pointwise$agd_contrast$df else x$pointwise$agd_contrast$n_contrast)

  cglue("Residual deviance: {round(x$resdev, digits)}", subtle(" (on {n} data points)", sep = ""))
  if (penalty == "pD")
    cglue("               pD: {round(x$pd, digits)}")
  else if (penalty == "pV")
    cglue("               pV: {round(x$pv, digits)}")
  cglue("              DIC: {round(x$dic, digits)}")

  invisible(x)
}

#' @rdname nma_dic-methods
#' @export
as.data.frame.nma_dic <- function(x, ...) {
  return(as.data.frame(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...)))
}

#' @rdname nma_dic-methods
#' @method as.tibble nma_dic
#' @export
as.tibble.nma_dic <- function(x, ...) {
  return(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...))
}

#' @rdname nma_dic-methods
#' @export
as_tibble.nma_dic <- function(x, ...) {
  return(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...))
}

#' @rdname nma_dic-methods
#' @export
as.array.nma_dic <- function(x, ...) {
  out <- x$resdev_array
  class(out) <-  c("mcmc_array", "array")
  return(out)
}

#' @rdname nma_dic-methods
#' @export
as.matrix.nma_dic <- function(x, ...){
  # Follow approach in rstan:::as.matrix.stanfit
  a <- as.array(x)
  names_a <- dimnames(a)
  dim_a <- dim(a)
  dim(a) <- c(dim_a[1] * dim_a[2], dim_a[3])
  dimnames(a) <- names_a[-2]
  class(a) <- "matrix"
  return(a)
}

#' Plots of model fit diagnostics
#'
#' The `plot()` method for [nma_dic] objects produced by [dic()] produces
#' several useful diagnostic plots for checking model fit and model comparison.
#' Further detail on these plots and their interpretation is given by
#' \insertCite{TSD2;textual}{multinma}.
#'
#' @param x A `nma_dic` object
#' @param y (Optional) A second `nma_dic` object, to produce "dev-dev" plots for
#'   model comparison.
#' @param ... Additional arguments passed on to other methods
#' @param type With a single `nma_dic` object, whether to show the residual
#'   deviance contribution for each data point (`"resdev"`, the default), or a
#'   leverage plot (`"leverage"`).
#' @param show_uncertainty Logical, show uncertainty with a `ggdist` plot
#'   stat? Default `TRUE`. Ignored for `type = "leverage"`.
#' @param stat Character string specifying the `ggdist` plot stat to use if
#'   `show_uncertainty = TRUE`, default `"pointinterval"`. If `y` is provided,
#'   currently only `"pointinterval"` is supported.
#' @param orientation Whether the `ggdist` geom is drawn horizontally
#'   (`"horizontal"`) or vertically (`"vertical"`). Only used for residual
#'   deviance plots, default `"vertical"`.
#' @param dic_contours Numeric vector of DIC contribution contours to show on
#'   leverage plots, default `1:4`.
#'
#' @return A `ggplot` object.
#' @export
#'
#' @details When a single `nma_dic` object is given, the default plot (`type =
#'   "resdev"`) shows the residual deviance contribution for each data point.
#'   For a good fitting model, each data point is expected to have a residual
#'   deviance equal to its degrees of freedom (typically 1, except for multi-arm
#'   trials in contrast format or multinomial outcomes); larger values indicate
#'   data points that are fit poorly by the model. A leverage plot can be
#'   produced with `type = "leverage"`, plotting the leverages (contributions to
#'   \eqn{p_D}) against the signed square root residual deviance, both
#'   standardised by the degrees of freedom. Contours for different DIC
#'   contribution cutoffs are also shown; points which lie outside the DIC=3
#'   contour are generally identified as contributing to a model's poor fit.
#'
#'   When two `nma_dic` objects are given, a "dev-dev" plot comparing the
#'   residual deviance contributions under each model is produced. Data points
#'   with residual deviance contributions lying on the line of equality are fit
#'   equally well under either model. Data points lying below the line of
#'   equality indicate better fit under the second model (`y`); conversely, data
#'   points lying above the line of equality indicate better fit under the first
#'   model (`x`). A common use case is to compare a standard consistency model
#'   (fitted using [nma()] with `consistency = "consistency"`) with an unrelated
#'   mean effects (UME) inconsistency model (fitted using [nma()] with
#'   `consistency = "ume"`), to check for potential inconsistency.
#'
#'   See \insertCite{TSD2;textual}{multinma} for further details.
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples ## Smoking cessation
#' @template ex_smoking_nma_fe_example
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Compare DIC of FE and RE models
#' (smk_dic_FE <- dic(smk_fit_FE))
#' (smk_dic_RE <- dic(smk_fit_RE))   # substantially better fit
#'
#' # Plot residual deviance contributions under RE model
#' plot(smk_dic_RE)
#'
#' # Further customisation is possible using ggplot commands
#' # For example, highlighting data points with residual deviance above a certain threshold
#' plot(smk_dic_RE) +
#'   ggplot2::aes(colour = ggplot2::after_stat(ifelse(y > 1.5, "darkorange", "black"))) +
#'   ggplot2::scale_colour_identity()
#'
#' # Or by posterior probability, for example here a central probability of 0.6
#' # corresponds to a lower tail probability of (1 - 0.6)/2 = 0.2
#' plot(smk_dic_RE, .width = c(0.6, 0.95)) +
#'   ggplot2::aes(colour = ggplot2::after_stat(ifelse(ymin > 1, "darkorange", "black"))) +
#'   ggplot2::scale_colour_identity()
#'
#' # Leverage plot for RE model
#' plot(smk_dic_RE, type = "leverage")
#'
#' # Check for inconsistency using UME model
#' }
#' @template ex_smoking_nma_re_ume_example
#' @examples \donttest{
#' # Compare DIC
#' smk_dic_RE
#' (smk_dic_RE_UME <- dic(smk_fit_RE_UME))  # no difference in fit
#'
#' # Compare residual deviance contributions with a "dev-dev" plot
#' plot(smk_dic_RE, smk_dic_RE_UME)
#'
#' # By default the dev-dev plot can be a little cluttered
#' # Hiding the credible intervals
#' plot(smk_dic_RE, smk_dic_RE_UME, show_uncertainty = FALSE)
#'
#' # Changing transparency
#' plot(smk_dic_RE, smk_dic_RE_UME, point_alpha = 0.5, interval_alpha = 0.1)
#' }
plot.nma_dic <- function(x, y, ...,
                         type = c("resdev", "leverage"),
                         show_uncertainty = TRUE,
                         stat = "pointinterval",
                         orientation = c("vertical", "horizontal", "x", "y"),
                         dic_contours = 1:4) {
  # Checks
  has_y <- !missing(y)

  if (has_y && !inherits(y, "nma_dic"))
    abort("Second argument `y` should be a `nma_dic` object produced by dic(), or missing.")

  if (!rlang::is_bool(show_uncertainty))
    abort("`show_uncertainty` should be TRUE or FALSE.")

  orientation <- rlang::arg_match(orientation)
  if (orientation == "x") orientation <- "vertical"
  else if (orientation == "y") orientation <- "horizontal"

  type <- rlang::arg_match(type)
  if (type == "leverage") {
    show_uncertainty <- FALSE
    if (!rlang::is_vector(dic_contours) || !(is.numeric(dic_contours) || all(is.na(dic_contours))))
      abort("`dic_contours` must be a numeric vector.")
    if (!is.null(x$pointwise$ipd) && survival::is.Surv(x$pointwise$ipd[["observed"]]) ||
        !is.null(x$pointwise$agd_arm) && survival::is.Surv(x$pointwise$agd_arm[["observed"]]))
      abort("Leverage plots are not currently supported for survival outcomes.")
  }

  # Get resdev samples from resdev_array
  resdev_post <- as.matrix.nma_summary(x$resdev_array) %>%
    tibble::as_tibble()

  data_labels <- stringr::str_extract(colnames(resdev_post), "(?<=\\[).+(?=\\]$)")
  colnames(resdev_post) <- data_labels

  resdev_post <- tidyr::pivot_longer(resdev_post, cols = dplyr::everything(),
                                     names_to = "parameter", values_to = "resdev")

  resdev_post$.label <- forcats::fct_inorder(factor(resdev_post$parameter))

  # Make sure rows for parameters are together - behaviour change between gather() and pivot_longer()
  resdev_post$.ord <- factor(resdev_post$parameter, levels = data_labels)
  resdev_post <- dplyr::arrange(resdev_post, .data$.ord)

  Type <- c(rep("IPD", NROW(x$pointwise$ipd)),
            rep("AgD (arm-based)", NROW(x$pointwise$agd_arm)),
            rep("AgD (contrast-based)", NROW(x$pointwise$agd_contrast)))

  resdev_post$Type <- rep(Type, each = prod(dim(x$resdev_array)[1:2]))

  # Some likelihoods have data points with >1 df, these are stored in `df` column
  has_df_ipd <- rlang::has_name(x$pointwise$ipd, "df")
  has_df_agd_arm <- rlang::has_name(x$pointwise$agd_arm, "df")
  has_df_agd_contrast <- rlang::has_name(x$pointwise$agd_contrast, "df")

  has_df <- has_df_ipd || has_df_agd_arm || has_df_agd_contrast

  df <- c(if (has_df_ipd) x$pointwise$ipd$df else rep(1, NROW(x$pointwise$ipd)),
          if (has_df_agd_arm) x$pointwise$agd_arm$df else rep(1, NROW(x$pointwise$agd_arm)),
          if (has_df_agd_contrast) x$pointwise$agd_contrast$df else rep(1, NROW(x$pointwise$agd_contrast)))

  resdev_post$df <- rep(df, each = prod(dim(x$resdev_array)[1:2]))
  resdev_post$df_label <- paste0("df = ", resdev_post$df)

  if (!show_uncertainty) {
    resdev_post <- dplyr::group_by(resdev_post, .data$parameter,
                                   .data$.label, .data$Type,
                                   .data$df, .data$df_label) %>%
      dplyr::summarise(resdev = mean(.data$resdev))
  }

  if (has_y) { # Produce dev-dev plot

    # Get y resdev samples from resdev_array
    y_resdev_post <- as.matrix.nma_summary(y$resdev_array) %>%
      tibble::as_tibble()

    y_data_labels <- stringr::str_extract(colnames(y_resdev_post), "(?<=\\[).+(?=\\]$)")
    colnames(y_resdev_post) <- y_data_labels

    # Check resdev[] names match
    if (any(data_labels != y_data_labels))
      abort("Data points in `x` and `y` do not match")

    y_resdev_post <- tidyr::pivot_longer(y_resdev_post, cols = dplyr::everything(),
                                         names_to = "parameter", values_to = "resdev")

    y_resdev_post$.label <- forcats::fct_inorder(factor(y_resdev_post$parameter))

    # Make sure rows for parameters are together - behaviour change between gather() and pivot_longer()
    y_resdev_post$.ord <- factor(y_resdev_post$parameter, levels = y_data_labels)
    y_resdev_post <- dplyr::arrange(y_resdev_post, .data$.ord)

    y_resdev_post$Type <- rep(Type, each = prod(dim(x$resdev_array)[1:2]))

    if (!show_uncertainty) {
      y_resdev_post <- dplyr::group_by(y_resdev_post, .data$parameter,
                                       .data$.label, .data$Type) %>%
        dplyr::summarise(resdev_y = mean(.data$resdev))

      resdev_post <- dplyr::rename(resdev_post, resdev_x = "resdev")

      xy_resdev_post <- dplyr::left_join(resdev_post, y_resdev_post,
                                         by = c("parameter", ".label", "Type")) %>%
        # Plot IPD points underneath for better clarity
        dplyr::arrange(dplyr::desc(.data$Type), .data$parameter, .data$.label)
    } else {

      if (!rlang::is_string(stat))
        abort("`stat` should be a character string specifying the name of a ggdist geom")

      stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

      if (stat != "pointinterval")
        warn(glue::glue('Currently only stat = "pointinterval" is supported when `y` is given.',
                        'Results may be unexpected with stat = "{stat}"!',
                        .sep = "\n"))

      tb_geom <- tryCatch(getExportedValue("ggdist", paste0("geom_", stat)),
                          error = function(err) {
                            abort(paste("`stat` should be a character string specifying the name of a ggdist geom:",
                                        err, sep = "\n"))
                          })

      # Parse ... for arguments to point_interval()
      dots <- list(...)
      is_int_arg <- names(dots) %in% c(".width", ".point", ".interval")
      int_dots <- dots[is_int_arg]
      geom_dots <- dots[!is_int_arg]

      # Summarise resdev_post and y_resdev_post with ggdist::point_interval()
      resdev_post <- dplyr::group_by(resdev_post, .data$parameter, .data$.label, .data$Type)
      resdev_post <- do.call(ggdist::point_interval,
                             args = rlang::dots_list(.data = resdev_post,
                                                     rlang::quo(.data$resdev),
                                                     .width = c(0.66, 0.95),
                                                     .point = mean,
                                                     !!! int_dots,
                                                     .homonyms = "last")) %>%
        dplyr::rename(resdev_x = "resdev",
                      x_lower = ".lower",
                      x_upper = ".upper")

      y_resdev_post <- dplyr::group_by(y_resdev_post, .data$parameter, .data$.label, .data$Type)
      y_resdev_post <- do.call(ggdist::point_interval,
                               args = rlang::dots_list(.data = y_resdev_post,
                                                       rlang::quo(.data$resdev),
                                                       .width = c(0.66, 0.95),
                                                       .point = mean,
                                                       !!! int_dots,
                                                       .homonyms = "last")) %>%
        dplyr::rename(resdev_y = "resdev",
                      y_lower = ".lower",
                      y_upper = ".upper")

      xy_resdev_post <- dplyr::left_join(resdev_post, y_resdev_post,
                                         by = c("parameter", ".label", "Type",
                                                ".width", ".point", ".interval"))

    }

    if (show_uncertainty) {
      ulim <- max(xy_resdev_post$x_upper, xy_resdev_post$y_upper)

      p <- ggplot2::ggplot(xy_resdev_post,
                           ggplot2::aes(y = .data$resdev_y,
                                        x = .data$resdev_x,
                                        xmin = .data$x_lower, xmax = .data$x_upper,
                                        ymin = .data$y_lower, ymax = .data$y_upper,
                                        label = .data$.label))
    } else {
      ulim <- max(xy_resdev_post$resdev_x, xy_resdev_post$resdev_y)

      p <- ggplot2::ggplot(xy_resdev_post,
                           ggplot2::aes(y = .data$resdev_y,
                                        x = .data$resdev_x,
                                        label = .data$.label))
    }

    if (dplyr::n_distinct(xy_resdev_post$Type) > 1) {
      p <- p +
        ggplot2::aes(colour = .data$Type) +
        ggplot2::scale_colour_viridis_d("")
    }

    p <- p +
      ggplot2::labs(x = "Residual Deviance (model 1)",
                    y = "Residual Deviance (model 2)") +
      ggplot2::coord_fixed(xlim = c(0, ulim), ylim = c(0, ulim)) +
      ggplot2::geom_abline(slope = 1, intercept = 0, colour = "grey60")

    if (show_uncertainty) {
      p <- p +
        # Have to layer up by data type by hand, due to the dual vertical and horizontal geoms
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "IPD"), orientation = "vertical")) +
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "IPD"), orientation = "horizontal")) +
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "AgD (arm-based)"), orientation = "vertical")) +
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "AgD (arm-based)"), orientation = "horizontal")) +
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "AgD (contrast-based)"), orientation = "vertical")) +
        do.call(tb_geom, args = purrr::list_modify(geom_dots, data = ~dplyr::filter(., .data$Type == "AgD (contrast-based)"), orientation = "horizontal"))
    } else {
      p <- p + ggplot2::geom_point(...)
    }

    p <- p + theme_multinma()

  } else {

    if (type == "resdev") { # Produce resdev plot

      if (show_uncertainty) {
        if (!rlang::is_string(stat))
          abort("`stat` should be a character string specifying the name of a ggdist stat")

        stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

        tb_stat <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
                            error = function(err) {
                              abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                                          err, sep = "\n"))
                            })

      }

      # Is a horizontal geom specified?
      horizontal <- orientation == "horizontal"

      if (horizontal) {
        resdev_post$.label <- forcats::fct_rev(resdev_post$.label)

        p <- ggplot2::ggplot(resdev_post,
                             ggplot2::aes(y = .data$.label,
                                          x = .data$resdev)) +
          ggplot2::labs(x = "Residual Deviance", y = "Data Point")

        if (dplyr::n_distinct(resdev_post$df) > 1) {
          if (dplyr::n_distinct(resdev_post$Type) > 1) {
            p <- p + ggplot2::facet_grid(Type~df_label, space = "free", scales = "free_y") +
              ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
          } else {
            p <- p + ggplot2::facet_grid(df_label~., space = "free", scales = "free_y") +
              ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
          }
        } else {
          p <- p + ggplot2::geom_vline(xintercept = 1, colour = "grey60")
          if (dplyr::n_distinct(resdev_post$Type) > 1) {
            p <- p + ggplot2::facet_grid(Type~., space = "free", scales = "free_y")
          }
        }

      } else {
        p <- ggplot2::ggplot(resdev_post,
                             ggplot2::aes(x = .data$.label,
                                          y = .data$resdev)) +
          ggplot2::labs(y = "Residual Deviance", x = "Data Point")

        if (dplyr::n_distinct(resdev_post$df) > 1) {
          if (dplyr::n_distinct(resdev_post$Type) > 1) {
            p <- p + ggplot2::facet_grid(df_label~Type, space = "free", scales = "free_x") +
              ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
          } else {
            p <- p + ggplot2::facet_grid(.~df_label, space = "free", scales = "free_x") +
              ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
          }
        } else {
          p <- p + ggplot2::geom_hline(yintercept = 1, colour = "grey60")
          if (dplyr::n_distinct(resdev_post$Type) > 1) {
            p <- p + ggplot2::facet_grid(.~Type, space = "free", scales = "free_x")
          }
        }
      }

      if (show_uncertainty) {
        p <- p + do.call(tb_stat,
                         args = rlang::dots_list(point_interval = ggdist::mean_qi,
                                                 ...,
                                                 orientation = orientation,
                                                 .homonyms = "last"))
      } else {
        p <- p + ggplot2::geom_point(...)
      }

      p <- p + theme_multinma()

      if (!horizontal) {
        p <- p + ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5))
      } else {
        p <- p + ggplot2::theme(axis.text.y = ggplot2::element_text(hjust = 0, vjust = 0.5))
      }

    } else { # Leverage plot

      # Calculate signed sqrt resdev, add in leverages
      if (NROW(x$pointwise$ipd)) {
        resdev_post_ipd <- dplyr::right_join(resdev_post,
                                        dplyr::transmute(x$pointwise$ipd,
                                                         parameter = make_data_labels(.data$.study, .data$.trt),
                                                         .data$dic,
                                                         .data$leverage,
                                                         ssrd =
                                                           if (is.matrix(.data$observed)) {  # Ordered multinomial case
                                                            sign(rowSums(.data$observed[,-1] - .data$fitted[,-1], na.rm = TRUE)) * sqrt(.data$resdev)
                                                           } else {
                                                             sign(.data$observed - .data$fitted) * sqrt(.data$resdev)
                                                           }
                                        ),
                                        by = "parameter")
      }
      if (NROW(x$pointwise$agd_arm)) {
        resdev_post_aa <- dplyr::right_join(resdev_post,
                                        dplyr::transmute(x$pointwise$agd_arm,
                                                         parameter = make_data_labels(.data$.study, .data$.trt),
                                                         .data$dic,
                                                         .data$leverage,
                                                         ssrd =
                                                           if (is.matrix(.data$observed)) {  # Ordered multinomial case
                                                             sign(rowSums(.data$observed[,-1] - .data$fitted[,-1], na.rm = TRUE)) * sqrt(.data$resdev)
                                                           } else {
                                                             sign(.data$observed - .data$fitted) * sqrt(.data$resdev)
                                                           }
                                        ),
                                        by = "parameter")
      }
      if (NROW(x$pointwise$agd_contrast)) {
        resdev_post_ac <- dplyr::right_join(resdev_post,
                                        dplyr::transmute(x$pointwise$agd_contrast,
                                                         parameter = .data$.study,
                                                         .data$dic,
                                                         .data$leverage,
                                                         ssrd = purrr::map2_dbl(.data$observed,
                                                                                .data$fitted,
                                                                                ~sign(sum(.x - .y))) * sqrt(.data$resdev)
                                        ),
                                        by = "parameter")
      }

      resdev_post <- dplyr::bind_rows(if (NROW(x$pointwise$ipd)) resdev_post_ipd else tibble::tibble(),
                                      if (NROW(x$pointwise$agd_arm)) resdev_post_aa else tibble::tibble(),
                                      if (NROW(x$pointwise$agd_contrast)) resdev_post_ac else tibble::tibble())

      # Standardise sqrt residual deviances and leverages by df (e.g. for multi-arm trials)
      resdev_post$ssrd <- resdev_post$ssrd / sqrt(resdev_post$df)
      resdev_post$leverage <- resdev_post$leverage / resdev_post$df


      p <- ggplot2::ggplot(resdev_post,
                           ggplot2::aes(y = .data$leverage,
                                        x = .data$ssrd,
                                        label = .data$.label))

      if (dplyr::n_distinct(resdev_post$Type) > 1) {
        p <- p +
          ggplot2::aes(colour = .data$Type) +
          ggplot2::scale_colour_viridis_d("")
      }

      p <- p +
        ggplot2::labs(x = "Signed Square Root Residual Deviance",
                      y = "Leverage")

      rmax <- max(sqrt(resdev_post$resdev))

      if (!all(is.na(dic_contours))) {
        p <- p +
          purrr::map(dic_contours,
                     ~ggplot2::geom_function(fun = function(x, c) c - x^2, args = list(c = .),
                                            colour = "grey60", inherit.aes = FALSE,
                                            xlim = c(-rmax, rmax)*1.2)) +
          ggplot2::annotate("label", vjust = 0, linewidth = 0,  colour = "grey60", fill = NA,
                            x = 0, y = dic_contours, label = paste0("DIC = ", dic_contours))
      }

      p <- p +
        ggplot2::geom_point(...) +
        ggplot2::coord_cartesian(
          ylim = c(min(0, resdev_post$leverage, na.rm = TRUE),
                   max(dic_contours, resdev_post$leverage, na.rm = TRUE)),
          xlim = c(-rmax, rmax)) +
        theme_multinma()

    }
  }

  return(p)
}

#' Working with 3D MCMC arrays
#'
#' 3D MCMC arrays (Iterations, Chains, Parameters) are produced by `as.array()`
#' methods applied to `stan_nma` or `nma_summary` objects.
#'
#' @rdname mcmc_array-class
#' @name mcmc_array-class
#' @aliases mcmc_array
#'
#' @examples
#' ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Working with arrays of posterior draws (as mcmc_array objects) is
#' # convenient when transforming parameters
#'
#' # Transforming log odds ratios to odds ratios
#' LOR_array <- as.array(relative_effects(smk_fit_RE))
#' OR_array <- exp(LOR_array)
#'
#' # mcmc_array objects can be summarised to produce a nma_summary object
#' smk_OR_RE <- summary(OR_array)
#'
#' # This can then be printed or plotted
#' smk_OR_RE
#' plot(smk_OR_RE, ref_line = 1)
#'
#' # Transforming heterogeneity SD to variance
#' tau_array <- as.array(smk_fit_RE, pars = "tau")
#' tausq_array <- tau_array^2
#'
#' # Correct parameter names
#' names(tausq_array) <- "tausq"
#'
#' # Summarise
#' summary(tausq_array)
#' }
NULL

#' @param x,object A 3D MCMC array of class `mcmc_array`
#' @param probs Numeric vector of quantiles of interest
#' @param ... Further arguments passed to other methods
#'
#' @rdname mcmc_array-class
#' @return The `summary()` method returns a [nma_summary] object, the `print()`
#'   method returns `x` invisibly. The `names()` method returns a character
#'   vector of parameter names, and `names()<-` returns the object with updated
#'   parameter names. The `plot()` method is a shortcut for
#'   `plot(summary(x), ...)`, passing all arguments on to [plot.nma_summary()].
#' @export
summary.mcmc_array <- function(object, ..., probs = c(0.025, 0.25, 0.5, 0.75, 0.975)) {
  ss <- summary_mcmc_array(object, probs = probs)
  out <- list(summary = ss, sims = object)
  class(out) <- "nma_summary"
  return(out)
}

#' @rdname mcmc_array-class
#' @export
print.mcmc_array <- function(x, ...) {
  d <- dim(x)
  cglue("A MCMC array with {prod(d[1:2])} draws ({d[1]} iterations in {d[2]} chain{if (d[2] > 1) 's' else ''}) of {d[3]} parameter{if (d[3] > 1) 's' else ''}.")
  NextMethod(...)
  invisible(x)
}

#' @rdname mcmc_array-class
#' @export
plot.mcmc_array <- function(x, ...) {
  xsum <- list(summary = NULL, sims = x)
  class(xsum) <- "nma_summary"
  plot(xsum, ...)
}

#' @rdname mcmc_array-class
#' @export
names.mcmc_array <- function(x) {
  return(dimnames(x)[[3]])
}

#' @rdname mcmc_array-class
#' @param value Character vector of replacement parameter names
#' @export
`names<-.mcmc_array` <- function(x, value) {
  out <- x
  dimnames(out)[[3]] <- value
  return(out)
}

#' The `nma_summary` class
#'
#' The `nma_summary` class contains posterior summary statistics of model
#' parameters or other quantities of interest, and the draws used to obtain
#' these statistics.
#'
#' @rdname nma_summary-class
#' @name nma_summary-class
#' @aliases nma_summary nma_rank_probs
#'
#' @details Objects of class `nma_summary` have the following components:
#'   \describe{
#'   \item{summary}{A data frame containing the computed summary statistics.
#'   Column `.trt` indicates the corresponding treatment, or columns `.trta` and
#'   `.trtb` indicate the corresponding contrast (`.trtb` vs. `.trta`). If a
#'   regression model was fitted with effect modifier interactions with
#'   treatment, these summaries will be study-specific. In this case, the
#'   corresponding study population is indicated in the `.study` column. If a
#'   multinomial model was fitted, the `.category` column indicates the
#'   corresponding category.}
#'   \item{sims}{A 3D array \[Iteration, Chain, Parameter\] of MCMC
#'   simulations}
#'   \item{studies}{(Optional) A data frame containing study information,
#'   printed along with the corresponding summary statistics if `summary`
#'   contains a `.study` column. Should have a matching `.study` column.}
#'   }
#'
#' The following attributes may also be set:
#'   \describe{
#'   \item{xlab}{Label for x axis in plots, usually either `"Treatment"` or
#'   `"Contrast"`.}
#'   \item{ylab}{Label for y axis in plots, usually used for the scale e.g.
#'   `"log Odds Ratio"`.}
#'   }
#'
#' The subclass `nma_rank_probs` is used by the function
#' [posterior_rank_probs()], and contains posterior rank probabilities. This
#' subclass does not have a `sims` component, as the rank probabilities are
#' themselves posterior summaries of the ranks (i.e. they do not have a
#' posterior distribution). The posterior ranks from which the rank
#' probabilities are calculated may be obtained from [posterior_ranks()].
#'
NULL

#' Methods for `nma_summary` objects
#'
#' The `as.data.frame()`, `as_tibble()`, and `as.tibble()` methods return the
#' posterior summary statistics in a data frame or tibble. The `as.matrix()`
#' method returns a matrix of posterior draws. The `as.array()` method returns a
#' 3D array \[Iteration, Chain, Parameter\] of posterior draws (as class
#' [mcmc_array]).
#'
#'
#' @param x A `nma_summary` object
#' @param ... Additional arguments passed on to other methods
#' @param digits Integer number of digits to display
#' @param pars Character vector of parameters to display in the printed summary
#' @param include Logical, are parameters named in `pars` included (`TRUE`) or excluded (`FALSE`)
#'
#' @rdname nma_summary-methods
#'
#' @seealso [plot.nma_summary()]
#'
#' @return A `data.frame` for `as.data.frame()`, a `tbl_df` for `as.tibble()`
#'   and `as_tibble()`, a `matrix` for `as.matrix()`, and an `mcmc_array` for
#'   `as.array()`.
#'
#'   The `print()` method returns `x` invisibly.
#' @export
#'
print.nma_summary <- function(x, ..., digits = 2, pars, include = TRUE) {
  # Checks
  if (!rlang::is_bool(include)) abort("`include` should be TRUE or FALSE")

  if (!missing(pars) && !is.character(pars)) abort("`pars` should be a character vector")

  if (!rlang::is_scalar_integerish(digits)) abort("`digits` must be a single integer")

  x_sum <- tibble::as_tibble(x)

  if (!missing(pars)) {
    x_sum <-
      dplyr::filter(x_sum,
        if (include) grepl(paste0("^", pars, "(\\[.+\\])?$", collapse = "|"), .data$parameter)
        else !grepl(paste0("^", pars, "(\\[.+\\])?$", collapse = "|"), .data$parameter)
        )
  }

  # Get numeric columns for formatting, handling effective sample sizes and Rhat separately
  num_col <- setdiff(names(x_sum)[purrr::map_lgl(x_sum, is.numeric)],
                     c("n_eff", "Bulk_ESS", "Tail_ESS", "Rhat"))

  x_sum <- dplyr::mutate_at(x_sum, num_col, ~round(., digits))

  if (rlang::has_name(x_sum, "n_eff")) x_sum$n_eff <- round(x_sum$n_eff, 0)
  if (rlang::has_name(x_sum, "Bulk_ESS")) x_sum$Bulk_ESS <- round(x_sum$Bulk_ESS, 0)
  if (rlang::has_name(x_sum, "Tail_ESS")) x_sum$Tail_ESS <- round(x_sum$Tail_ESS, 0)
  if (rlang::has_name(x_sum, "Rhat")) x_sum$Rhat <- round(x_sum$Rhat, max(2, digits))

  x_sum <- tibble::column_to_rownames(x_sum, "parameter")

  # Drop dot columns (.trta, .trtb, .trt, .category) from the output, except .study and .time
  x_sum <- dplyr::select(x_sum, -dplyr::matches("^\\.(?!study|time)", perl = TRUE))

  # Format summaries nicely by study, if given
  print_study_block <- function(s, info = NULL, ...) {
    this_study <- unique(s$.study)
    sec_header(paste("Study:", this_study))
    cat("\n")
    if (!is.null(info)) {
      cat("Covariate values:\n")
      info %>%
        dplyr::filter(.data$.study == this_study) %>%
        dplyr::select(-".study") %>%
        dplyr::mutate_if(is.numeric, round, digits = digits) %>%
        as.data.frame() %>%
        print(..., row.names = FALSE)
      cat("\n")
    }
    s %>% dplyr::select(-".study") %>% print(...)
    cat("\n")
  }

  if (rlang::has_name(x_sum, ".study")) {
    by(x_sum, x_sum$.study, print_study_block, info = x$studies, ..., simplify = FALSE)
  } else {
    print(x_sum, ...)
  }
  invisible(x)
}

#' Plots of summary results
#'
#' The `plot` method for `nma_summary` objects is used to produce plots of
#' parameter estimates (when called on a `stan_nma` object or its summary),
#' relative effects (when called on the output of [relative_effects()]),
#' absolute predictions (when called on the output of [predict.stan_nma()]),
#' posterior ranks and rank probabilities (when called on the output of
#' [posterior_ranks()] or [posterior_rank_probs()]).
#'
#' @param x A `nma_summary` object
#' @param ... Additional arguments passed on to the underlying `ggdist` plot
#'   stat, see Details
#' @param stat Character string specifying the `ggdist` plot stat to use,
#'   default `"pointinterval"`, except when plotting estimated
#'   survival/hazard/cumulative hazard curves from survival models where the
#'   default is `"lineribbon"`
#' @param orientation Whether the `ggdist` geom is drawn horizontally
#'   (`"horizontal"`) or vertically (`"vertical"`), default `"horizontal"`
#' @param ref_line Numeric vector of positions for reference lines, by default
#'   no reference lines are drawn
#'
#' @details Plotting is handled by \link[ggplot2:ggplot2-package]{ggplot2} and
#'   the stats and geoms provided in the \link[ggdist:ggdist-package]{ggdist}
#'   package. As a result, the output is very flexible. Any plotting stats
#'   provided by `ggdist` may be used, via the argument `stat`.
#'
#'   The default uses
#'   \code{\link[ggdist:stat_pointinterval]{ggdist::stat_pointinterval()}}, to
#'   produce medians and 95% Credible Intervals with 66% inner bands. Additional
#'   arguments in `...` are passed to the `ggdist` stat, to customise the
#'   output. For example, to produce means and Credible Intervals, specify
#'   `point_interval = "mean_qi"`. To produce an 80% Credible Interval with no
#'   inner band, specify `.width = c(0, 0.8)`.
#'
#'   Alternative stats can be specified to produce different summaries. For
#'   example, specify `stat = "[half]eye"` to produce (half) eye plots, or `stat
#'   = "histinterval"` to produce histograms with intervals.
#'
#'   A full list of options and examples is found in the `ggdist` vignette
#'   `vignette("slabinterval", package = "ggdist")`.
#'
#'   For survival/hazard/cumulative hazard curves estimated from survival
#'   models, the default uses
#'   \code{\link[ggdist:stat_lineribbon]{ggdist::stat_lineribbon()}} which
#'   produces curves of posterior medians with 50%, 80%, and 95% Credible
#'   Interval bands. Again, additional arguments in `...` are passed to the
#'   `ggdist` stat. For example, to produce posterior means and 95% Credible
#'   bands, specify `point_interval = "mean_qi"` and `.width = 0.95`.
#'
#'   A `ggplot` object is returned which can be further modified through the
#'   usual \link[ggplot2:ggplot2-package]{ggplot2} functions to add further
#'   aesthetics, geoms, themes, etc.
#'
#' @return A `ggplot` object.
#'
#' @export
#' @examples
#' ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Produce relative effects
#' smk_releff_RE <- relative_effects(smk_fit_RE)
#' plot(smk_releff_RE, ref_line = 0)
#'
#' # Customise plot options
#' plot(smk_releff_RE, ref_line = 0, stat = "halfeye")
#'
#' # Further customisation is possible with ggplot commands
#' plot(smk_releff_RE, ref_line = 0, stat = "halfeye", slab_alpha = 0.6) +
#'   ggplot2::aes(slab_fill = ggplot2::after_stat(ifelse(x < 0, "darkred", "grey60")))
#'
#' # Produce posterior ranks
#' smk_rank_RE <- posterior_ranks(smk_fit_RE, lower_better = FALSE)
#' plot(smk_rank_RE)
#'
#' # Produce rank probabilities
#' smk_rankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE)
#' plot(smk_rankprob_RE)
#'
#' # Produce cumulative rank probabilities
#' smk_cumrankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE,
#'                                            cumulative = TRUE)
#' plot(smk_cumrankprob_RE)
#'
#' # Further customisation is possible with ggplot commands
#' plot(smk_cumrankprob_RE) +
#'   ggplot2::facet_null() +
#'   ggplot2::aes(colour = Treatment)
#' }
plot.nma_summary <- function(x, ...,
                             stat = "pointinterval",
                             orientation = c("horizontal", "vertical", "y", "x"),
                             ref_line = NA_real_) {
  # Checks
  if (!rlang::is_string(stat))
    abort("`stat` should be a character string specifying the name of a ggdist stat")

  stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

  tb_geom <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
    error = function(err) {
      abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                  err, sep = "\n"))
      })


  if (!is.numeric(ref_line) || !is.null(dim(ref_line)))
    abort("`ref_line` should be a numeric vector.")

  orientation <- rlang::arg_match(orientation)
  if (orientation == "x") orientation <- "vertical"
  else if (orientation == "y") orientation <- "horizontal"

  # Is a horizontal geom specified?
  horizontal <- orientation == "horizontal"

  # Get axis labels from attributes, if available
  p_xlab <- attr(x, "xlab", TRUE)
  if (is.null(p_xlab)) p_xlab <- ""

  p_ylab <- attr(x, "ylab", TRUE)
  if (is.null(p_ylab)) p_ylab <- ""

  # Tweak output if rank summary
  if (is_ranks <- inherits(x, "nma_ranks")) {
    ntrt <- nrow(as.data.frame(x))
  }

  # Get draws
  draws <- tibble::as_tibble(as.matrix(x))

  draws <- tidyr::pivot_longer(draws, cols = dplyr::everything(),
                               names_to = "parameter", values_to = "value")

  if (has_studies <- rlang::has_name(as.data.frame(x), ".study")) {
    draws$Study <- forcats::fct_inorder(factor(
      stringr::str_extract(draws$parameter, "(?<=\\[).+(?=\\:)")))

    if (inherits(x, "ordered_nma_summary")) {
      draws$Treatment <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=\\: ).+(?=, .+?\\])")))
      draws$Category <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=, ).+(?=\\])")))
    } else {
      draws$Treatment <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=\\: ).+(?=\\])")))
    }
  } else {
    if (inherits(x, "ordered_nma_summary")) {
      draws$Treatment <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=\\[).+(?=, .+?\\])")))
      draws$Category <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=, ).+(?=\\])")))
    } else {
      draws$Treatment <- forcats::fct_inorder(factor(
        stringr::str_extract(draws$parameter, "(?<=\\[).+(?=\\])")))
    }
  }

  if (horizontal) {

    draws$Treatment <- forcats::fct_rev(draws$Treatment)

    p <- ggplot2::ggplot(draws, ggplot2::aes(y = .data$Treatment, x = .data$value)) +
      ggplot2::geom_vline(xintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::ylab(p_xlab)

    if (has_studies) {
      if (inherits(x, "ordered_nma_summary")) {
        p <- p + ggplot2::facet_grid(Study~Category)
      } else {
        p <- p + ggplot2::facet_grid(Study~.)
      }
    } else if (inherits(x, "ordered_nma_summary")) {
      p <- p + ggplot2::facet_grid(.~Category)
    }

    if (is_ranks) {
      p <- p + ggplot2::scale_x_continuous(p_ylab, breaks = 1:ntrt, minor_breaks = NULL)
    } else {
      p <- p + ggplot2::xlab(p_ylab)
    }

  } else {

    p <- ggplot2::ggplot(draws, ggplot2::aes(x = .data$Treatment, y = .data$value)) +
      ggplot2::geom_hline(yintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::xlab(p_xlab)

    if (has_studies) {
      if (inherits(x, "ordered_nma_summary")) {
        p <- p + ggplot2::facet_grid(Category~Study)
      } else {
        p <- p + ggplot2::facet_grid(.~Study)
      }
    } else if (inherits(x, "ordered_nma_summary")) {
      p <- p + ggplot2::facet_grid(Category~.)
    }


    if (is_ranks) {
      p <- p + ggplot2::scale_y_continuous(p_ylab, breaks = 1:ntrt, minor_breaks = NULL)
    } else {
      p <- p + ggplot2::ylab(p_ylab)
    }

  }

  p <- p +
    do.call(tb_geom, args = list(orientation = orientation, ...)) +
    theme_multinma()

  return(p)
}

#' @rdname plot.nma_summary
#' @export
plot.nma_parameter_summary <- function(x, ...,
                                       stat = "pointinterval",
                                       orientation = c("horizontal", "vertical", "y", "x"),
                                       ref_line = NA_real_) {
  # Checks
  if (!rlang::is_string(stat))
    abort("`stat` should be a character string specifying the name of a ggdist stat.")

  stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

  tb_geom <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
                      error = function(err) {
                        abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                                    err, sep = "\n"))
                      })


  if (!is.numeric(ref_line) || !is.null(dim(ref_line)))
    abort("`ref_line` should be a numeric vector.")

  orientation <- rlang::arg_match(orientation)
  if (orientation == "x") orientation <- "vertical"
  else if (orientation == "y") orientation <- "horizontal"

  # Is a horizontal geom specified?
  horizontal <- orientation == "horizontal"

  # Get axis labels from attributes, if available
  p_xlab <- attr(x, "xlab", TRUE)
  if (is.null(p_xlab)) p_xlab <- ""

  p_ylab <- attr(x, "ylab", TRUE)
  if (is.null(p_ylab)) p_ylab <- ""

  # Get draws
  draws <- tibble::as_tibble(as.matrix(x))

  draws <- tidyr::pivot_longer(draws, cols = dplyr::everything(),
                               names_to = "parameter", values_to = "value")

  draws$par_base <- forcats::fct_inorder(factor(
    stringr::str_remove(draws$parameter, "\\[.*\\]")))
  draws$parameter <- forcats::fct_inorder(factor(draws$parameter))

  if (horizontal) {

    draws$parameter <- forcats::fct_rev(draws$parameter)

    p <- ggplot2::ggplot(draws, ggplot2::aes(y = .data$parameter, x = .data$value)) +
      ggplot2::geom_vline(xintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::ylab(p_xlab) + ggplot2::xlab(p_ylab) +
      ggplot2::facet_grid(par_base~., scales = "free", space = "free")

  } else {

    p <- ggplot2::ggplot(draws, ggplot2::aes(x = .data$parameter, y = .data$value)) +
      ggplot2::geom_hline(yintercept = ref_line, na.rm = TRUE, colour = "grey60") +
      ggplot2::xlab(p_xlab) + ggplot2::ylab(p_ylab) +
      ggplot2::facet_grid(.~par_base, scales = "free", space = "free")

  }

  p <- p +
    do.call(tb_geom, args = list(orientation = orientation, ...)) +
    theme_multinma()

  return(p)
}

#' @rdname plot.nma_summary
#' @export
plot.nma_rank_probs <- function(x, ...) {
  # Get axis labels from attributes, if available
  p_xlab <- attr(x, "xlab", TRUE)
  if (is.null(p_xlab)) p_xlab <- ""

  p_ylab <- attr(x, "ylab", TRUE)
  if (is.null(p_ylab)) p_ylab <- ""

  dat <- as.data.frame(x)

  ntrt <- nrow(dat)

  if (has_studies <- rlang::has_name(dat, ".study")) {
    dat$Study <- forcats::fct_inorder(factor(dat$.study))
    dat$Treatment <- forcats::fct_inorder(factor(
      stringr::str_extract(dat$parameter, "(?<=\\: ).+(?=\\])")))
  } else {
    dat$Treatment <- forcats::fct_inorder(factor(
      stringr::str_extract(dat$parameter, "(?<=\\[).+(?=\\])")))
  }

  dat <- tidyr::pivot_longer(dat, cols = dplyr::starts_with("p_rank"),
                               names_to = "rank", values_to = "probability",
                               names_pattern = "^p_rank\\[([0-9]+)\\]$",
                               names_transform = list(rank = as.integer))


  p <- ggplot2::ggplot(dat,
                       ggplot2::aes(x = .data$rank, y = .data$probability)) +
    ggplot2::geom_line(...) +
    ggplot2::ylab(p_ylab) +
    ggplot2::scale_x_continuous(p_xlab, breaks = 1:ntrt, minor_breaks = NULL) +
    ggplot2::coord_cartesian(ylim = c(0, 1)) +
    theme_multinma()

  if (has_studies) {
    p <- p + ggplot2::facet_grid(Study~Treatment)
  } else {
    p <- p + ggplot2::facet_wrap(~Treatment)
  }

  return(p)
}

#' @rdname plot.nma_summary
#' @export
plot.surv_nma_summary <- function(x, ..., stat = "lineribbon") {
  # Checks
  if (!rlang::is_string(stat))
    abort("`stat` should be a character string specifying the name of a ggdist stat")

  stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

  tb_geom <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
                      error = function(err) {
                        abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                                    err, sep = "\n"))
                      })

  # Get axis labels from attributes, if available
  p_xlab <- attr(x, "xlab", TRUE)
  if (is.null(p_xlab)) p_xlab <- ""

  p_ylab <- attr(x, "ylab", TRUE)
  if (is.null(p_ylab)) p_ylab <- ""

  # Get draws
  draws <- as.data.frame(x) %>%
    dplyr::select(dplyr::starts_with(".")) %>%
    dplyr::mutate(value = purrr::array_branch(as.matrix(x), 2)) %>%
    tidyr::unnest(cols = "value")

  if (has_studies <- rlang::has_name(draws, ".study")) {
    draws$Study <- draws$.study
  }

  draws$Time <- draws$.time

  if (rlang::has_name(draws, ".trt") || dplyr::n_distinct(draws$.trta) == 1) {
    draws$Treatment <- if (rlang::has_name(draws, ".trt")) draws$.trt else draws$.trtb
    p <- ggplot2::ggplot(draws, ggplot2::aes(x = .data$Time, y = .data$value,
                                             colour = .data$Treatment,
                                             fill = .data$Treatment))
  } else {
    draws$Contrast <- paste0(draws$.trtb, " vs. ", draws$.trta)
    p <- ggplot2::ggplot(draws, ggplot2::aes(x = .data$Time, y = .data$value,
                                             colour = .data$Contrast,
                                             fill = .data$Contrast))
  }

  p <- p +
    ggplot2::ylab(p_ylab) +
    ggplot2::xlab(p_xlab)

  if (has_studies) {
      p <- p + ggplot2::facet_grid(.~Study)
  }

  p <- p +
    # Draw lines and intervals separately for better clarity
    do.call(tb_geom, args = rlang::dots_list(linewidth = 0, ..., alpha = 0.15, .homonyms = "first")) +
    do.call(tb_geom, args = rlang::dots_list(.width = 0, alpha = 1, ..., linewidth = 0.5, .homonyms = "first")) +
    theme_multinma()

  return(p)
}

#' @rdname nma_summary-methods
#' @export
as.data.frame.nma_summary <- function(x, ...) {
  return(as.data.frame(x$summary, ...))
}

#' @rdname nma_summary-methods
#' @importFrom tibble as.tibble
#' @method as.tibble nma_summary
#' @export
as.tibble.nma_summary <- function(x, ...) {
  return(tibble::as.tibble(x$summary, ...))
}

#' @rdname nma_summary-methods
#' @importFrom tibble as_tibble
#' @export
as_tibble.nma_summary <- function(x, ...) {
  return(tibble::as_tibble(x$summary, ...))
}

#' @rdname nma_summary-methods
#' @export
as.array.nma_summary <- function(x, ...) {
  out <- x$sims
  class(out) <-  c("mcmc_array", "array")
  return(out)
}

#' @rdname nma_summary-methods
#' @export
as.matrix.nma_summary <- function(x, ...){
  # Follow approach in rstan:::as.matrix.stanfit
  a <- as.array(x)
  names_a <- dimnames(a)
  dim_a <- dim(a)
  dim(a) <- c(dim_a[1] * dim_a[2], dim_a[3])
  dimnames(a) <- names_a[-2]
  class(a) <- "matrix"
  return(a)
}

#' @rdname nma_summary-methods
#' @export
as.array.nma_rank_probs <- function(x, ...) {
  abort(paste("Objects of class `nma_rank_probs` do not contain a 3D MCMC array of simulations, see ?nma_rank_probs.",
              "  - Use as.matrix(), as.data.frame(), or as_tibble() to access the summary rank probabilities",
              "  - Use posterior_ranks() to obtain the posterior ranks themselves", sep = "\n"))
}

#' @rdname nma_summary-methods
#' @export
as.matrix.nma_rank_probs <- function(x, ...){
  df_summary <- tibble::as_tibble(x, ...)
  if (rlang::has_name(df_summary, ".study")) df_summary <- dplyr::select(df_summary, -".study")
  m <- as.matrix(tibble::column_to_rownames(df_summary, "parameter"))
  return(m)
}

#' Compute summary statistics from a 3D MCMC array
#'
#' @param x A 3D MCMC array
#' @param probs Numeric vector of quantiles of interest
#'
#' @return A data frame of computed summaries, one row per parameter
#' @noRd
summary_mcmc_array <- function(x, probs = c(0.025, 0.25, 0.5, 0.75, 0.975)) {
  if (!is.array(x) || length(dim(x)) != 3) abort("Not a 3D MCMC array, [Iterations, Chains, Parameters]")
  check_probs(probs)

  pars <- dimnames(x)[[3]]
  p_mean <- apply(x, 3, mean)
  p_sd <- apply(x, 3, sd)
  p_ess_bulk <- apply(x, 3, rstan::ess_bulk)
  p_ess_tail <- apply(x, 3, rstan::ess_tail)
  p_rhat <- apply(x, 3, rstan::Rhat)
  # p_se_mean <- p_sd / sqrt(apply(x, 3, rstan:::ess_mean))

  qt <- function(x, probs, ...) {
    if (all(is.na(x))) setNames(rlang::rep_along(probs, NA_real_), paste0(probs*100, "%"))
    else quantile(x, probs = probs, ...)
  }

  p_quan <- apply(x, 3, qt, probs = probs)
  if (length(probs) == 1) {
    p_quan <- tibble::tibble(!! paste0(probs*100, "%") := p_quan)
  } else {
    p_quan <- as.data.frame(t(p_quan))
  }

  ss <- tibble::tibble(
    parameter = pars,
    mean = p_mean,
    # se_mean = p_se_mean,
    sd = p_sd,
    !!! p_quan,
    Bulk_ESS = p_ess_bulk, Tail_ESS = p_ess_tail, Rhat = p_rhat)

  return(ss)
}

#' Validate probs argument
#' @noRd
check_probs <- function(probs) {
  if (!rlang::is_double(probs, finite = TRUE) || any(probs < 0) || any(probs > 1))
    rlang::abort("`probs` must be a numeric vector of probabilities between 0 and 1.")
}

# For dynamic export of S3 methods
# Copied from `vctrs` https://vctrs.r-lib.org/ under GPL-3
s3_register <- function(generic, class, method = NULL) {
  stopifnot(is.character(generic), length(generic) == 1)
  stopifnot(is.character(class), length(class) == 1)

  pieces <- strsplit(generic, "::")[[1]]
  stopifnot(length(pieces) == 2)
  package <- pieces[[1]]
  generic <- pieces[[2]]

  caller <- parent.frame()

  get_method_env <- function() {
    top <- topenv(caller)
    if (isNamespace(top)) {
      asNamespace(environmentName(top))
    } else {
      caller
    }
  }
  get_method <- function(method, env) {
    if (is.null(method)) {
      get(paste0(generic, ".", class), envir = get_method_env())
    } else {
      method
    }
  }

  method_fn <- get_method(method)
  stopifnot(is.function(method_fn))

  # Always register hook in case package is later unloaded & reloaded
  setHook(
    packageEvent(package, "onLoad"),
    function(...) {
      ns <- asNamespace(package)

      # Refresh the method, it might have been updated by `devtools::load_all()`
      method_fn <- get_method(method)

      registerS3method(generic, class, method_fn, envir = ns)
    }
  )

  # Avoid registration failures during loading (pkgload or regular)
  if (!isNamespaceLoaded(package)) {
    return(invisible())
  }

  envir <- asNamespace(package)

  # Only register if generic can be accessed
  if (exists(generic, envir)) {
    registerS3method(generic, class, method_fn, envir = envir)
  }

  invisible()
}

#' Relative treatment effects
#'
#' Generate (population-average) relative treatment effects. If a ML-NMR or
#' meta-regression model was fitted, these are specific to each study
#' population.
#'
#' @param x A `stan_nma` object created by [nma()]
#' @param newdata Only used if a regression model is fitted. A data frame of
#'   study details, one row per study, giving the covariate values at which to
#'   produce relative effects. Column names must match variables in the
#'   regression model. If `NULL`, relative effects are produced for all studies
#'   in the network.
#' @param study Column of `newdata` which specifies study names, otherwise
#'   studies will be labelled by row number.
#' @param all_contrasts Logical, generate estimates for all contrasts (`TRUE`),
#'   or just the "basic" contrasts against the network reference treatment
#'   (`FALSE`)? Default `FALSE`.
#' @param trt_ref Reference treatment to construct relative effects against, if
#'   `all_contrasts = FALSE`. By default, relative effects will be against the
#'   network reference treatment. Coerced to character string.
#' @param probs Numeric vector of quantiles of interest to present in computed
#'   summary, default `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#' @param predictive_distribution Logical, when a random effects model has been
#'   fitted, should the predictive distribution for relative effects in a new
#'   study be returned? Default `FALSE`.
#' @param summary Logical, calculate posterior summaries? Default `TRUE`.
#'
#' @return A [nma_summary] object if `summary = TRUE`, otherwise a list
#'   containing a 3D MCMC array of samples and (for regression models) a data
#'   frame of study information.
#' @export
#'
#' @seealso [plot.nma_summary()] for plotting the relative effects.
#'
#' @examples
#' ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Produce relative effects
#' smk_releff_RE <- relative_effects(smk_fit_RE)
#' smk_releff_RE
#' plot(smk_releff_RE, ref_line = 0)
#'
#' # Relative effects for all pairwise comparisons
#' relative_effects(smk_fit_RE, all_contrasts = TRUE)
#'
#' # Relative effects against a different reference treatment
#' relative_effects(smk_fit_RE, trt_ref = "Self-help")
#'
#' # Transforming to odds ratios
#' # We work with the array of relative effects samples
#' LOR_array <- as.array(smk_releff_RE)
#' OR_array <- exp(LOR_array)
#'
#' # mcmc_array objects can be summarised to produce a nma_summary object
#' smk_OR_RE <- summary(OR_array)
#'
#' # This can then be printed or plotted
#' smk_OR_RE
#' plot(smk_OR_RE, ref_line = 1)
#' }
#'
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_mlnmr_example
#' @examples \donttest{
#' # Produce population-adjusted relative effects for all study populations in
#' # the network
#' pso_releff <- relative_effects(pso_fit)
#' pso_releff
#' plot(pso_releff, ref_line = 0)
#'
#' # Produce population-adjusted relative effects for a different target
#' # population
#' new_agd_means <- data.frame(
#'   bsa = 0.6,
#'   prevsys = 0.1,
#'   psa = 0.2,
#'   weight = 10,
#'   durnpso = 3)
#'
#' relative_effects(pso_fit, newdata = new_agd_means)
#' }
relative_effects <- function(x, newdata = NULL, study = NULL,
                             all_contrasts = FALSE, trt_ref = NULL,
                             probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                             predictive_distribution = FALSE,
                             summary = TRUE) {

  # Checks
  if (!inherits(x, "stan_nma")) abort("Expecting a `stan_nma` object, as returned by nma().")

  if (!is.null(newdata)) {
    if (!is.data.frame(newdata)) abort("`newdata` is not a data frame.")

    .study <- pull_non_null(newdata, enquo(study))
    if (is.null(.study)) {
      newdata$.study <- nfactor(paste("New", seq_len(nrow(newdata))))
    } else {
      check_study(.study)
      newdata <- dplyr::mutate(newdata, .study = nfactor(.study))

      if (anyDuplicated(newdata$.study))
        abort("Duplicate values in `study` column. Expecting one row per study.")
    }
  }

  if (!rlang::is_bool(all_contrasts))
    abort("`all_contrasts` should be TRUE or FALSE.")

  if (!is.null(trt_ref)) {
    if (all_contrasts) {
      warn("Ignoring `trt_ref` when all_contrasts = TRUE.")
      trt_ref <- NULL
    } else {
      if (length(trt_ref) > 1) abort("`trt_ref` must be length 1.")
      trt_ref <- as.character(trt_ref)
      lvls_trt <- levels(x$network$treatments)
      if (! trt_ref %in% lvls_trt)
        abort(sprintf("`trt_ref` does not match a treatment in the network.\nSuitable values are: %s",
                      ifelse(length(lvls_trt) <= 5,
                             paste0(lvls_trt, collapse = ", "),
                             paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    }
  }

  if (!rlang::is_bool(summary))
    abort("`summary` should be TRUE or FALSE.")

  check_probs(probs)

  if(!rlang::is_bool(predictive_distribution))
    abort("`predictive_distribution` should be TRUE or FALSE")
  if (predictive_distribution && x$trt_effects != "random") predictive_distribution <- FALSE

  # Cannot produce relative effects for inconsistency models
  if (x$consistency != "consistency")
    abort(glue::glue("Cannot produce relative effects under inconsistency '{x$consistency}' model."))

  # Get network reference treatment
  nrt <- levels(x$network$treatments)[1]

  # Produce relative effects
  if (is.null(x$regression) || is_only_offset(x$regression)) {
    # If no regression model, relative effects are just the d's

    if (!predictive_distribution) {
      re_array <- as.array(as.stanfit(x), pars = "d")
    } else {
      # For predictive distribution, use delta_new instead of d
      re_array <- get_delta_new(x)
    }

    # Set treatments vector
    trtb <- x$network$treatments[-1]
    trta <- x$network$treatments[1]

    if (all_contrasts) {
      re_array <- make_all_contrasts(re_array, trt_ref = nrt)

      contrs <- utils::combn(x$network$treatments, 2)
      trtb <- contrs[2, ]
      trta <- contrs[1, ]

    } else if (!is.null(trt_ref) && trt_ref != nrt) {
      d_ref <- re_array[ , , paste0("d[", trt_ref, "]"), drop = FALSE]
      re_array <- sweep(re_array, 1:2, d_ref, FUN = "-")

      # Add in parameter for network ref trt in place of trt_ref
      re_array[ , , paste0("d[", trt_ref, "]")] <- -d_ref
      d_names <- dimnames(re_array)[[3]]
      d_names[d_names == paste0("d[", trt_ref, "]")] <- paste0("d[", nrt, "]")
      dimnames(re_array)[[3]] <- d_names

      # Reorder parameters
      d_names <- c(paste0("d[", nrt, "]"), d_names[d_names != paste0("d[", nrt, "]")])
      re_array <- re_array[ , , d_names, drop = FALSE]

      trt_reorder <- sort(forcats::fct_relevel(x$network$treatments, trt_ref))
      trtb <- trt_reorder[-1]
      trta <- trt_reorder[1]
    }

    # Fix up parameter names for predictive_distribution = TRUE
    if (predictive_distribution) {
      dimnames(re_array)[[3]] <- gsub("^d\\[", "delta_new[", dimnames(re_array)[[3]])
    }

    if (summary) {
      re_summary <- summary_mcmc_array(re_array, probs = probs) %>%
        tibble::add_column(.trtb = trtb, .trta = trta, .before = 1)

      out <- list(summary = re_summary, sims = re_array)
    } else {
      out <- list(sims = re_array)
    }

  } else {
    # If regression model, relative effects are study-specific

    if (is.null(newdata)) {
      # Produce relative effects for all studies in network

      # Make data frame of study covariate means
      if ((has_agd_arm(x$network) || has_agd_contrast(x$network)) && !has_agd_sample_size(x$network))
        abort(
          paste("AgD study sample sizes not specified in network, cannot calculate mean covariate values.",
                "  - Specify `sample_size` in set_agd_*(), or",
                "  - Specify covariate values for relative effects using the `newdata` argument",
                sep = "\n"))

      if (has_agd_arm(x$network)) {
        if (x$network$outcome$agd_arm != "survival") {
          if (inherits(x$network, "mlnmr_data")) {
            dat_agd_arm <- .unnest_integration(x$network$agd_arm) %>%
              dplyr::mutate(.sample_size = .data$.sample_size / x$network$n_int)
          } else {
            dat_agd_arm <- x$network$agd_arm
          }
        } else {
          # For survival outcomes, check if we need to access .data_orig
          if (any(rlang::has_name(x$agd_arm$.data_orig, all.vars(x$regression)))) {
            dat_agd_arm <- x$network$agd_arm %>%
              # Drop duplicated names in outer dataset from .data_orig before unnesting
              dplyr::mutate(.data_orig = purrr::map(.data$.data_orig, ~ dplyr::select(., -dplyr::any_of(names(x$network$agd_arm)))),
                            # Reset sample size for weighted mean later
                            .sample_size = 1) %>%
              # Unnest .data_orig
              tidyr::unnest(cols = c(".Surv", ".data_orig"))
          } else {
            # Drop nested .Surv column, breaks join with ipd
            dat_agd_arm <- dplyr::select(x$network$agd_arm, -.data$.Surv)
          }

          # Unnest integration points if present
          if (inherits(x$network, "mlnmr_data")) {
            dat_agd_arm <- .unnest_integration(dat_agd_arm) %>%
              dplyr::mutate(.sample_size = .data$.sample_size / x$network$n_int)
          }
        }

        # A `.mu` column is expected if baseline risk meta-regression is used in this model
        dat_agd_arm$.mu <- 1

        # Only take necessary columns
        dat_agd_arm <- get_model_data_columns(dat_agd_arm, regression = x$regression, label = "AgD (arm-based)")
      } else {
        dat_agd_arm <- tibble::tibble()
      }

      if (has_agd_contrast(x$network)) {
        if (inherits(x$network, "mlnmr_data")) {
          dat_agd_contrast <- .unnest_integration(x$network$agd_contrast) %>%
            dplyr::mutate(.sample_size = .data$.sample_size / x$network$n_int)
        } else {
          dat_agd_contrast <- x$network$agd_contrast
        }

        # Only take necessary columns
        dat_agd_contrast <- get_model_data_columns(dat_agd_contrast, regression = x$regression, label = "AgD (contrast-based)")
      } else {
        dat_agd_contrast <- tibble::tibble()
      }

      if (has_ipd(x$network)) {
        dat_ipd <- x$network$ipd
        dat_ipd$.sample_size <- 1

        # A `.mu` column is expected if baseline risk meta-regression is used in this model
        dat_ipd$.mu <- 1

        # Only take necessary columns
        dat_ipd <- get_model_data_columns(dat_ipd, regression = x$regression, label = "IPD")
      } else {
        dat_ipd <- tibble::tibble()
      }

      dat_all <- dplyr::bind_rows(dat_agd_arm, dat_agd_contrast, dat_ipd)

      # Take the first row for each study. We will correct the design matrix below
      dat_studies <- dat_all %>%
        dplyr::group_by(.data$.study) %>%
        dplyr::slice(1)

    } else {
      # Produce relative effects for all studies in newdata

      dat_studies <- newdata

      # Check all variables are present
      regdat <- get_model_data_columns(dat_studies, regression = x$regression, label = "`newdata`")
    }

    # Get number of treatments, number of studies
    ntrt <- nlevels(x$network$treatments)
    nstudy <- nrow(dat_studies)

    # Expand rows for every treatment
    all_trts <- tidyr::expand_grid(.study = dat_studies$.study, .trt = x$network$treatments[-1])
    if (rlang::has_name(dat_studies, ".trt")) dat_studies <- dplyr::select(dat_studies, -".trt")
    dat_studies <- dplyr::left_join(all_trts, dat_studies, by = ".study")

    # Add in .trtclass if defined in network
    if (!is.null(x$network$classes)) {
      dat_studies$.trtclass <- x$network$classes[as.numeric(dat_studies$.trt)]
    }

    # Get model formula and design matrix
    nma_formula <- make_nma_formula(x$regression,
                                    consistency = x$consistency,
                                    classes = !is.null(x$network$classes),
                                    class_interactions = x$class_interactions)

    # If `newdata` was not supplied, relative effects are calculated for each study, and
    # the baseline risk meta-regression columns in the design matrix are 0/1 values.
    # Therefore, they should not be centered here.
    xbar <- x$xbar
    if (is.null(newdata)) {
      xbar <- xbar[names(xbar) != ".mu"]
    }

    X_list <- make_nma_model_matrix(nma_formula,
                                    dat_agd_arm = dat_studies,
                                    xbar = xbar,
                                    consistency = x$consistency,
                                    classes = !is.null(x$network$classes),
                                    newdata = TRUE)
    X_all <- X_list$X_agd_arm

    # Subset design matrix into EM columns and trt columns
    X_d <- X_all[, grepl("^(\\.trt|\\.contr)[^:]+$", colnames(X_all)), drop = FALSE]
    EM_regex <- "(^\\.trt(class)?.+\\:)|(\\:\\.trt(class)?.+$)"
    X_EM <- X_all[, grepl(EM_regex, colnames(X_all)), drop = FALSE]

    # If there are no EMs (regression model had no interactions with .trt) then
    # just return the treatment effects (not study specific)
    if (ncol(X_EM) == 0) {

      if (!predictive_distribution) {
        re_array <- as.array(as.stanfit(x), pars = "d")
      } else {
        # For predictive distribution, use delta_new instead of d
        re_array <- get_delta_new(x)
      }

      # Set treatments vector
      trtb <- x$network$treatments[-1]
      trta <- x$network$treatments[1]

      if (all_contrasts) {
        re_array <- make_all_contrasts(re_array, trt_ref = nrt)

        contrs <- utils::combn(x$network$treatments, 2)
        trtb <- contrs[2, ]
        trta <- contrs[1, ]

      } else if (!is.null(trt_ref) && trt_ref != nrt) {
        d_ref <- re_array[ , , paste0("d[", trt_ref, "]"), drop = FALSE]
        re_array <- sweep(re_array, 1:2, d_ref, FUN = "-")

        # Add in parameter for network ref trt in place of trt_ref
        re_array[ , , paste0("d[", trt_ref, "]")] <- -d_ref
        d_names <- dimnames(re_array)[[3]]
        d_names[d_names == paste0("d[", trt_ref, "]")] <- paste0("d[", nrt, "]")
        dimnames(re_array)[[3]] <- d_names

        # Reorder parameters
        d_names <- c(paste0("d[", nrt, "]"), d_names[d_names != paste0("d[", nrt, "]")])
        re_array <- re_array[ , , d_names, drop = FALSE]

        trt_reorder <- sort(forcats::fct_relevel(x$network$treatments, trt_ref))
        trtb <- trt_reorder[-1]
        trta <- trt_reorder[1]
      }

      # Fix up parameter names for predictive_distribution = TRUE
      if (predictive_distribution) {
        dimnames(re_array)[[3]] <- gsub("^d\\[", "delta_new[", dimnames(re_array)[[3]])
      }

      if (summary) {
        re_summary <- summary_mcmc_array(re_array, probs = probs)  %>%
          tibble::add_column(.trtb = trtb, .trta = trta, .before = 1)

        out <- list(summary = re_summary, sims = re_array)
      } else {
        out <- list(sims = re_array)
      }
    } else {

      # Split baseline risk meta-regression from other effect modifiers
      brmr <- grepl("(^\\.mu\\:)|(\\:\\.mu$)", colnames(X_EM)) & is.null(newdata)
      X_BRMR <- X_EM[, brmr, drop = FALSE]
      X_EM <- X_EM[, !brmr, drop = FALSE]

      # Which covariates are EMs
      EM_col_names <- stringr::str_remove(colnames(X_EM), EM_regex)
      EM_vars <- get_EM_vars(nma_formula)

      # Replace EM design matrix with study means if newdata is NULL
      if (is.null(newdata)) {

        # Apply centering if used
        dat_all_cen <- dat_all
        if (!is.null(x$xbar)) {
          cen_vars <- intersect(names(dat_all), names(x$xbar))
          dat_all_cen[, cen_vars] <- sweep(dat_all[, cen_vars, drop = FALSE], 2, x$xbar[cen_vars])
        }

        # Get model matrix of EM "main effects" - notably this expands out factors
        # into dummy variables so we can average those too
        EM_formula <- as.formula(paste0("~", paste(EM_vars, collapse = " + ")))

        # Calculate mean covariate values by study in the network
        X_study_means <- model.matrix(EM_formula, data = dat_all_cen) %>%
          tibble::as_tibble() %>%
          tibble::add_column(.study = dat_all$.study, .sample_size = dat_all$.sample_size, .before = 1) %>%
          dplyr::group_by(.data$.study) %>%
          dplyr::summarise_at(setdiff(colnames(.), c(".sample_size", ".study")),
                              ~weighted.mean(., w = .data$.sample_size)) %>%
          dplyr::select(!!! unique(EM_col_names)) %>%
          as.matrix()

        # Repeat columns across interaction columns in X_EM
        X_study_means_rep <- X_study_means[, EM_col_names, drop = FALSE]

        # Repeat study rows for the number of treatment parameters
        X_study_means_rep <- apply(X_study_means_rep, 2, rep, each = ntrt - 1)

        # Replace non-zero entries of design matrix X_EM with corresponding mean values
        # This works only because trt columns are 0/1, so interactions are just the covariate values
        nonzero <- X_EM != 0
        X_EM[nonzero] <- X_study_means_rep[nonzero]
      }

      # Name columns to match Stan parameters
      if (ncol(X_EM) > 0) {
        colnames(X_EM) <- paste0("beta[", colnames(X_EM), "]")
      }
      if (ncol(X_BRMR) > 0) {
        colnames(X_BRMR) <- paste0("beta[", colnames(X_BRMR), "]")
      }
      colnames(X_d) <- paste0("d[", x$network$treatments[-1], "]")

      X_EM_d <- cbind(X_EM, X_d)

      # Name rows by treatment for now (required for make_all_contrasts)
      rownames(X_EM_d) <- paste0("d[", dat_studies$.trt , "]")

      d_array <- as.array(x, pars = colnames(X_EM_d))
      if (predictive_distribution) {
        # For predictive distribution, use delta_new instead of d
        d_array[ , , colnames(X_d)] <- get_delta_new(x)
      }

      # Linear combination with posterior MCMC array
      re_array <- tcrossprod_mcmc_array(d_array, X_EM_d)

      if (any(brmr)) {
        # Centered sampled study-specific intercepts
        mu_array <- as.array(x, pars = paste0("mu[", x$network$studies , "]")) - x$xbar[[".mu"]]

        # Include the mean study-specific intercepts in the covariate values
        mu_means <- apply(mu_array, 3, mean)

        X_study_means <- cbind(
          X_study_means,
          .mu = mu_means[paste0("mu[", unique(dat_studies$.study), "]")]
        )

        brmr_array <- as.array(x, pars = colnames(X_BRMR))

        # Add to relative effects
        re_array <- re_array + tcrossprod_mcmc_array(brmr_array, X_BRMR) * (
          mu_array[, , paste0("mu[", as.character(dat_studies$.study), "]"), drop = FALSE]
        )
      }

      # Set treatments vector
      trtb <- x$network$treatments[-1]
      trta <- rep(x$network$treatments[1], times = ntrt - 1)

      # Produce all contrasts, if required
      if (all_contrasts) {
        n_contr <- ntrt * (ntrt - 1) / 2
        re_array_all <- array(dim = c(dim(re_array)[1:2],
                                      dplyr::n_distinct(dat_studies$.study) * n_contr),
                              dimnames = purrr::list_modify(dimnames(re_array),
                                                            parameters = paste(rep(1:dplyr::n_distinct(dat_studies$.study), each = n_contr),
                                                                               rep(1:n_contr, times = dplyr::n_distinct(dat_studies$.study)))))
        for (j in unique(dat_studies$.study)) {
          j_select <- dat_studies$.study == j
          j_select_all <- rep(unique(dat_studies$.study) == j, each = n_contr)
          j_contrs_all <- make_all_contrasts(re_array[ , , j_select, drop = FALSE], trt_ref = nrt)
          re_array_all[ , , j_select_all] <- j_contrs_all
          dimnames(re_array_all)[[3]][j_select_all] <- dimnames(j_contrs_all)[[3]]
        }
        re_array <- re_array_all

        contrs <- utils::combn(x$network$treatments, 2)
        trtb <- contrs[2, ]
        trta <- contrs[1, ]
      }

      # Add in study names to parameters
      parnames <- stringr::str_extract(dimnames(re_array)[[3]], "(?<=^d\\[)(.+)(?=\\]$)")
      if (all_contrasts) {
        study_parnames <- paste0("d[", rep(unique(dat_studies$.study), each = n_contr), ": ", parnames, "]")
      } else {
        study_parnames <- paste0("d[", dat_studies$.study, ": ", parnames, "]")
      }
      dimnames(re_array)[[3]] <- study_parnames

      # Rebase against ref_trt if given
      if (!is.null(trt_ref) && trt_ref != nrt) {
        for (j in unique(dat_studies$.study)) {
          j_pars <- dat_studies$.study == j
          d_ref <- re_array[ , , paste0("d[", j, ": ", trt_ref, "]"), drop = FALSE]
          re_array[ , , j_pars] <- sweep(re_array[ , , j_pars, drop = FALSE], 1:2, d_ref, FUN = "-")

          # Add in parameter for network ref trt in place of trt_ref
          re_array[ , , paste0("d[", j, ": ", trt_ref, "]")] <- -d_ref
          d_names <- dimnames(re_array)[[3]]
          d_names[d_names == paste0("d[", j, ": ", trt_ref, "]")] <- paste0("d[", j, ": ", nrt, "]")
          dimnames(re_array)[[3]] <- d_names

          # Reorder parameters
          d_names[j_pars] <- c(paste0("d[", j, ": ", nrt, "]"), d_names[j_pars & d_names != paste0("d[", j, ": ", nrt, "]")])
          re_array <- re_array[ , , d_names, drop = FALSE]

          trt_reorder <- sort(forcats::fct_relevel(x$network$treatments, trt_ref))
          trtb <- trt_reorder[-1]
          trta <- rep(trt_reorder[1], times = ntrt - 1)
        }
      }

      # Fix up parameter names for predictive_distribution = TRUE
      if (predictive_distribution) {
        dimnames(re_array)[[3]] <- gsub("^d\\[", "delta_new[", dimnames(re_array)[[3]])
      }

      # Create summary stats
      if (summary) {
        re_summary <- summary_mcmc_array(re_array, probs = probs) %>%
          tibble::add_column(.study =
                               if (all_contrasts) {
                                 rep(unique(dat_studies$.study), each = n_contr)
                               } else {
                                 dat_studies$.study
                               },
                             .trtb = rep(trtb, times = nstudy),
                             .trta = rep(trta, times = nstudy),
                             .before = 1)
      }

      # Prepare study covariate info
      if (is.null(newdata)) {
        study_EMs <- X_study_means

        # Uncenter if necessary
        if (!is.null(x$xbar)) {
          cen_vars <- intersect(colnames(study_EMs), names(x$xbar))
          study_EMs[, cen_vars] <- sweep(study_EMs[, cen_vars, drop = FALSE], 2, x$xbar[cen_vars], FUN = "+")
        }
      } else {
        study_EMs <- newdata[EM_vars]
      }

      study_EMs <- tibble::as_tibble(study_EMs) %>%
        tibble::add_column(.study = unique(dat_studies$.study), .before = 1)

      if (summary) {
        out <- list(summary = re_summary, sims = re_array, studies = study_EMs)
      } else {
        out <- list(sims = re_array, studies = study_EMs)
      }
    }
  }

  # Return nma_summary object
  if (summary) {
    class(out) <- "nma_summary"
    attr(out, "xlab") <- if (all_contrasts) "Contrast" else "Treatment"
    attr(out, "ylab") <- get_scale_name(likelihood = x$likelihood,
                                        link = x$link,
                                        measure = "relative",
                                        type = "link")
  }
  return(out)
}


#' Make all treatment contrasts
#'
#' @param d A 3D MCMC array of basic treatment effects
#' @param trt_ref String naming the reference treatment
#'
#' @return A 3D MCMC array of all contrasts
#' @noRd
make_all_contrasts <- function(d, trt_ref) {
  if (!is.array(d) || length(dim(d)) != 3) abort("Not a 3D MCMC array [Iterations, Chains, Treatments]")
  if (!rlang::is_string(trt_ref)) abort("`trt_ref` must be a single string")

  trts <- c(trt_ref, stringr::str_extract(dimnames(d)[[3]], "(?<=\\[)(.+)(?=\\]$)"))
  ntrt <- length(trts)

  d_ab <- utils::combn(ntrt, 2)

  dim_out <- dim(d)
  dim_out[3] <- ncol(d_ab)

  contrs <- array(NA_real_, dim = dim_out)

  contrs[ , , 1:(ntrt - 1)] <- d
  for (i in ntrt:ncol(d_ab)) {
    contrs[ , , i] <- d[ , , d_ab[2, i] - 1] - d[ , , d_ab[1, i] - 1]
  }

  new_dimnames <- dimnames(d)
  new_dimnames[[3]] <- paste0("d[", trts[d_ab[2, ]], " vs. ", trts[d_ab[1, ]], "]")
  dimnames(contrs) <- new_dimnames

  return(contrs)
}

#' Crossproduct for 3D MCMC arrays
#'
#' @param x A matrix
#' @param a A 3D MCMC array
#'
#' @return A 3D MCMC array
#' @noRd
tcrossprod_mcmc_array <- function(a, x) {
  if (!is.array(a) || length(dim(a)) != 3) abort("Not a 3D MCMC array [Iterations, Chains, Parameters]")

  dim_out <- c(dim(a)[1:2], NROW(x))
  dimnames_out <- dimnames(a)
  dimnames_out[[3]] <- if (!is.null(rownames(x))) rownames(x) else paste0("V", 1:NROW(x))
  out <- array(NA_real_, dim = dim_out)

  nchains <- dim(a)[2]

  if (inherits(x, "Matrix")) {
    for (i in 1:nchains) {
      out[ , i, ] <- as.matrix(Matrix::tcrossprod(a[ , i, ], x))
    }
  } else {
    for (i in 1:nchains) {
      out[ , i, ] <- tcrossprod(a[ , i, ], x)
    }
  }

  dimnames(out) <- dimnames_out

  return(out)
}

#' Sample predictive distribution of deltas for new study
#'
#' @param x Fitted RE stan_nma object
#' @param ...
#'
#' @return A 3D MCMC array
#' @noRd
get_delta_new <- function(x, ...) {
  ntrt <- nlevels(x$network$treatments)

  RE_cor <- matrix(0.5, nrow = ntrt - 1, ncol = ntrt - 1)
  diag(RE_cor) <- 1

  draws <- as.matrix(x, pars = c("d", "tau"))
  # Need to make d[] index numeric for rstan::gqs
  colnames(draws)[1:(ntrt - 1)] <- paste0("d[", 1:(ntrt - 1), "]")

  if (ntrt == 2) {
    # Work around rstan::gqs() bug when there is only d[1]
    draws <- draws[, c("d[1]", "d[1]", "tau")]
    colnames(draws) <- c("d[1]", "d[2]", "tau")
    delta_new <- rstan::gqs(stanmodels$predict_delta_new,
                            data = list(nt = 3,
                                        RE_cor = diag(2)),
                            draws = draws,
                            seed = rstan::get_seed(x$stanfit))

    delta_new <- as.array(delta_new)[ , , 1, drop = FALSE]
  } else {
    delta_new <- rstan::gqs(stanmodels$predict_delta_new,
                            data = list(nt = ntrt,
                                        RE_cor = RE_cor),
                            draws = draws,
                            seed = rstan::get_seed(x$stanfit))

    delta_new <- as.array(delta_new)
  }

  # Note: name these d instead of delta_new here so that they can be drop-in
  # replacements in relative_effects(), then fix up later
  dimnames(delta_new)[[3]] <- paste0("d[", levels(x$network$treatments)[-1], "]")
  class(delta_new) <- c("mcmc_array", class(delta_new))

  return(delta_new)
}

#' Smoking cessation data
#'
#' Data frame containing the results of 24 trials of 4 smoking cessation
#' treatments \insertCite{Hasselblad1998,TSD4}{multinma}.
#'
#' @format A data frame with 50 rows and 5 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{r}{total number of events}
#'   \item{n}{total number of individuals}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"smoking"

#' Plaque psoriasis data
#'
#' Two data frames, `plaque_psoriasis_ipd` and `plaque_psoriasis_agd`,
#' containing (simulated) individual patient data from four studies and
#' aggregate data from five studies \insertCite{Phillippo_thesis}{multinma}.
#' Outcomes are binary success/failure to achieve 75%, 90%, or 100% reduction in
#' symptoms on the Psoriasis Area and Severity Index (PASI) scale.
#'
#' @format The individual patient data are contained in a data frame
#'   `plaque_psoriasis_ipd` with 4118 rows, one per individual, and 16 variables:
#'   \describe{
#'     \item{studyc}{study name}
#'     \item{trtc_long}{treatment name (long format)}
#'     \item{trtc}{treatment name}
#'     \item{trtn}{numeric treatment code}
#'     \item{pasi75}{binary PASI 75 outcome}
#'     \item{pasi90}{binary PASI 90 outcome}
#'     \item{pasi100}{binary PASI 100 outcome}
#'     \item{age}{age (years)}
#'     \item{bmi}{body mass index (BMI)}
#'     \item{pasi_w0}{PASI score at week 0}
#'     \item{male}{male sex (TRUE or FALSE)}
#'     \item{bsa}{body surface area (percent)}
#'     \item{weight}{weight (kilograms)}
#'     \item{durnpso}{duration of psoriasis (years)}
#'     \item{prevsys}{previous systemic treatment (TRUE or FALSE)}
#'     \item{psa}{psoriatic arthritis (TRUE or FALSE)}
#'   }
#'
#'  The aggregate data are contained in a data frame `plaque_psoriasis_agd` with 15
#'  rows, one per study arm, and 26 variables:
#'   \describe{
#'     \item{studyc}{study name}
#'     \item{trtc_long}{treatment name (long format)}
#'     \item{trtc}{treatment name}
#'     \item{trtn}{numeric treatment code}
#'     \item{pasi75_r, pasi75_n}{PASI 75 outcome count and denominator}
#'     \item{pasi90_r, pasi90_n}{PASI 75 outcome count and denominator}
#'     \item{pasi100_r, pasi100_n}{PASI 75 outcome count and denominator}
#'     \item{sample_size_w0}{sample size at week zero}
#'     \item{age_mean, age_sd}{mean and standard deviation of age (years)}
#'     \item{bmi_mean, bmi_sd}{mean and standard deviation of BMI}
#'     \item{pasi_w0_mean, pasi_w0_sd}{mean and standard deviation of PASI score at week 0}
#'     \item{male}{percentage of males}
#'     \item{bsa_mean, bsa_sd}{mean and standard deviation of body surface area (percent)}
#'     \item{weight_mean, weight_sd}{mean and standard deviation of weight (kilograms)}
#'     \item{durnpso_mean, durnpso_sd}{mean and standard deviation of duration of psoriasis (years)}
#'     \item{prevsys}{percentage of individuals with previous systemic treatment}
#'     \item{psa}{percentage of individuals with psoriatic arthritis}
#'   }
#'
#' @references \insertAllCited{}
#'
#' @rdname plaque_psoriasis
#' @aliases plaque_psoriasis
#'

"plaque_psoriasis_ipd"

#' @rdname plaque_psoriasis
"plaque_psoriasis_agd"

#' Thrombolytic treatments data
#'
#' Data frame containing the results of 50 trials of 8 thrombolytic drugs
#' (streptokinase, SK; alteplase, t-PA; accelerated alteplase, Acc t-PA;
#' streptokinase plus alteplase, SK+tPA; reteplase, r-PA; tenocteplase, TNK;
#' urokinase, UK; anistreptilase, ASPAC) plus per-cutaneous transluminal
#' coronary angioplasty (PTCA)
#' \insertCite{Boland2003,Lu2006,TSD4}{multinma}. The number of
#' deaths in 30 or 35 days following acute myocardial infarction are recorded.
#'
#' @format A data frame with 102 rows and 5 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{r}{total number of events}
#'   \item{n}{total number of individuals}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"thrombolytics"

#' Incidence of diabetes in trials of antihypertensive drugs
#'
#' Data frame containing the number of new cases of diabetes in 22 trials of 6
#' antihypertensive drugs \insertCite{Elliott2007,TSD2}{multinma}. The trial
#' duration (in years) is also recorded.
#'
#' @format A data frame with 48 rows and 7 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{studyc}{study name}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{r}{total number of events}
#'   \item{n}{total number of individuals}
#'   \item{time}{trial follow-up (years)}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"diabetes"

#' Granulocyte transfusion in patients with neutropenia or neutrophil
#' dysfunction
#'
#' Data frame containing the number of deaths in 6 trials comparing transfusion
#' of granulocytes (white blood cells) to control
#' \insertCite{Stanworth2005}{multinma}. Previously used to demonstrate
#' informative prior distributions for the heterogeneity variance by
#' \insertCite{Turner2012;textual}{multinma}.
#'
#' @format A data frame with 12 rows and 4 variables:
#' \describe{
#'   \item{studyc}{study name}
#'   \item{trtc}{treatment name}
#'   \item{r}{total number of deaths}
#'   \item{n}{total number of individuals}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"transfusion"

#' Beta blockers to prevent mortality after MI
#'
#' Data frame containing the number of deaths in 22 trials comparing beta
#' blockers vs. control for preventing mortality after myocardial infarction
#' \insertCite{Carlin1992,TSD2}{multinma}.
#'
#' @format A data frame with 44 rows and 5 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{r}{total number of events}
#'   \item{n}{total number of individuals}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"blocker"

#' Reduced dietary fat to prevent mortality
#'
#' Data frame containing the number of deaths and person-years at risk in 10
#' trials comparing reduced fat diets vs. control (non-reduced fat diet) for
#' preventing mortality \insertCite{Hooper2000,TSD2}{multinma}.
#'
#' @format A data frame with 21 rows and 7 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{studyc}{study name}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{r}{number of events}
#'   \item{n}{number randomised}
#'   \item{E}{person-years at risk}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"dietary_fat"

#' Mean off-time reduction in Parkison's disease
#'
#' Data frame containing the mean off-time reduction in patients given dopamine
#' agonists as adjunct therapy in Parkinson's disease, from 7 trials comparing
#' four active drugs and placebo \insertCite{TSD2}{multinma}.
#'
#' @format A data frame with 15 rows and 7 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{trtn}{numeric treatment code (placebo = 1)}
#'   \item{y}{mean off-time reduction}
#'   \item{se}{standard error}
#'   \item{n}{sample size}
#'   \item{diff}{mean difference vs. treatment in reference arm}
#'   \item{se_diff}{standard error of mean difference, see details}
#' }
#'
#' @details This dataset may be analysed using either an arm-based likelihood
#'   using `y` and `se`, or a contrast-based likelihood using `diff` and
#'   `se_diff` (or a combination of the two across different studies).
#'
#'   The contrast-based data is formatted as described in [set_agd_contrast()].
#'   That is, for the chosen reference arm in each study, the mean difference
#'   `diff` is set to `NA`, and `se_diff` is set to the standard error `se` of
#'   the outcome on the reference arm.
#'
#' @references
#'   \insertAllCited{}
#'

"parkinsons"

#' Stroke prevention in atrial fibrillation patients
#'
#' Data frame containing the results of 26 trials comparing 17 treatments in 4
#' classes for the prevention of stroke in patients with atrial fibrillation
#' \insertCite{Cooper2009}{multinma}. The data are the corrected versions
#' given by \insertCite{gemtc;textual}{multinma}.
#'
#' @format A data frame with 63 rows and 11 variables:
#' \describe{
#'   \item{studyc}{study name}
#'   \item{studyn}{numeric study ID}
#'   \item{trtc}{treatment name}
#'   \item{trtn}{numeric treatment code}
#'   \item{trt_class}{treatment class}
#'   \item{r}{number of events}
#'   \item{n}{sample size}
#'   \item{E}{person-years at risk}
#'   \item{stroke}{proportion of individuals with prior stroke}
#'   \item{year}{year of study publication}
#'   \item{followup}{mean length of follow-up (years)}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"atrial_fibrillation"

#' Statins for cholesterol lowering
#'
#' Data frame containing the results of 19 trials comparing statins to placebo
#' or usual care \insertCite{TSD3}{multinma}. The number of deaths (all-cause
#' mortality) are recorded. In some studies the aim was primary prevention
#' (patients had no previous heart disease), and in others the aim was secondary
#' prevention (patients had previous heart disease).
#'
#' @format A data frame with 38 rows and 7 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{studyc}{study name}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{prevention}{primary or secondary prevention study}
#'   \item{r}{number of deaths}
#'   \item{n}{sample size}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"statins"

#' BCG vaccination
#'
#' Data frame containing the results of 13 trials comparing BCG vaccination to
#' no vaccination for preventing tuberculosis (TB)
#' \insertCite{TSD3,Berkey1995}{multinma}. The numbers of individuals diagnosed
#' with TB in each arm during the study follow-up period are recorded. The
#' absolute degrees latitude at which the study was conducted are also recorded.
#'
#' @format A data frame with 26 rows and 6 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{latitude}{absolute degrees latitude}
#'   \item{r}{number diagnosed with TB}
#'   \item{n}{sample size}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"bcg_vaccine"

#' HTA Plaque Psoriasis
#'
#' Data frame containing the results of 16 trials comparing 8 treatments for
#' moderate-to-severe plaque psoriasis from an HTA report
#' \insertCite{Woolacott2006}{multinma}, analysed in TSD2
#' \insertCite{TSD2}{multinma}. Outcomes are success/failure to achieve 50%,
#' 75%, or 90% reduction in symptoms on the Psoriasis Area and Severity Index
#' (PASI) scale. Some studies report all three ordered outcomes, others only one
#' or two. The latter are coded as missing values (see details).
#'
#' @format A data frame with 36 rows and 9 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{studyc}{study name}
#'   \item{year}{year of publication}
#'   \item{trtn}{numeric treatment code}
#'   \item{trtc}{treatment name}
#'   \item{sample_size}{sample size in each arm}
#'   \item{PASI50, PASI75, PASI90}{ordered multinomial outcome counts (exclusive, see details)}
#' }
#'
#' @details Outcome counts are "exclusive"; that is, for a study reporting all
#'   outcomes, the counts represent the categories 50 < PASI < 75, 75 < PASI <
#'   90, and 90 < PASI < 100, and are named by the lower end of the interval.
#'   (As opposed to "inclusive" counts, which would represent the overlapping
#'   categories PASI > 50, PASI > 70, and PASI > 90.) The count for the fourth
#'   category (the lowest), 0 < PASI < 50, is equal to `sample_size - PASI50 -
#'   PASI75 - PASI90`.
#'
#'   Missing values are used where studies only report a subset of the outcomes.
#'   For a study reporting only two outcomes, say 50 and 75, the counts
#'   represent 50 < PASI < 75 and 75 < PASI < 100. For a study reporting only
#'   one outcome, say PASI 75, the count represents 75 < PASI < 100.
#'
#' @references
#'   \insertAllCited{}
#'

"hta_psoriasis"

#' Newly diagnosed multiple myeloma
#'
#' Three data frames, `ndmm_ipd`, `ndmm_agd`, and `ndmm_agd_covs` containing
#' (simulated) individual patient data (IPD) from three studies and aggregate
#' data (AgD) from two studies on newly diagnosed multiple myeloma. The outcome
#' of interest is progression-free survival after autologous stem cell
#' transplant. The IPD studies in `ndmm_ipd` provide event/censoring times and
#' covariate values for each individual. The AgD studies provide reconstructed
#' event/censoring times from digitized Kaplan-Meier curves in `ndmm_agd` and
#' covariate summaries in `ndmm_agd_covs`, obtained from published trial
#' reports. The data are constructed to resemble those used by
#' \insertCite{Leahy2019;textual}{multinma}.
#'
#' @format The individual patient data are contained in a data frame `ndmm_ipd`
#'   with 1325 rows, one per individual, and 10 variables:
#'   \describe{
#'     \item{study, studyf}{study name}
#'     \item{trt, trtf}{treatment name}
#'     \item{eventtime}{event/censoring time}
#'     \item{status}{censoring indicator (0 = censored, 1 = event)}
#'     \item{age}{age (years)}
#'     \item{iss_stage3}{ISS stage 3 (0 = no, 1 = yes)}
#'     \item{response_cr_vgpr}{complete or very good partial response (0 = no, 1 = yes)}
#'     \item{male}{male sex (0 = no, 1 = yes)}
#'   }
#'
#' @references \insertAllCited{}
#'
#' @rdname ndmm
"ndmm_ipd"

#' @rdname ndmm
#' @format The reconstructed Kaplan-Meier data for the aggregate studies are
#'   contained in a data frame `ndmm_agd` with 2819 rows and 6 variables:
#'  \describe{
#'    \item{study, studyf}{study name}
#'    \item{trt, trtf}{treatment name}
#'    \item{eventtime}{event/censoring time}
#'    \item{status}{censoring indicator (0 = censored, 1 = event)}
#'  }
"ndmm_agd"

#' @rdname ndmm
#' @format The covariate summaries extracted from published reportes for the
#'   aggregate studies are contained in a data frame `ndmm_agd_covs` with 4
#'   rows, one per study arm, and 15 columns:
#'  \describe{
#'    \item{study, studyf}{study name}
#'    \item{trt, trtf}{treatment name}
#'    \item{sample_size}{sample size in each arm}
#'    \item{age_min, age_iqr_l, age_median, age_iqr_h, age_max, age_mean, age_sd}{summary statistics for age (years)}
#'    \item{iss_stage3}{proportion of participants with ISS stage 3}
#'    \item{response_cr_vgpr}{proportion of participants with complete or very good partial response}
#'    \item{male}{proportion of male participants}
#'  }
"ndmm_agd_covs"

#' Social Anxiety
#'
#' Data frame containing the results of 101 clinical trials comparing 41 first-line treatments for social anxiety disorder
#' in adults . The 41 treatments are further categorised into 17 distinct classes. The interventions of interest include oral drugs,
#' psychological or behavioural therapies, and combinations of pharmacological and psychological therapies.
#' Following Mayo-Wilson et al. \insertCite{mayo2014psychological}{multinma} the data are given as standardised mean differences (SMD) to allow comparison across the different scales used in each study.
#'
#' @format A data frame with 248 rows and 8 variables:
#' \describe{
#'   \item{studyn}{numeric study ID}
#'   \item{studyc}{study name}
#'   \item{trtn}{treatment ID}
#'   \item{y}{standardised mean difference of the outcome measure}
#'   \item{se}{standard error of the outcome measure}
#'   \item{trtc}{treatment name}
#'   \item{classn}{class ID}
#'   \item{classc}{class name}
#' }
#'
#' @references
#'   \insertAllCited{}
#'
"social_anxiety"

#' Certolizumab
#'
#' Data frame containing the results of 12 trials comparing 7 treatments for rheumatoid arthritis, analysed in \insertCite{TSD3}{multinma}.
#'
#' @format A data frame with 24 rows and 5 variables:
#' \describe{
#'   \item{study}{study name}
#'   \item{trt}{treatment name}
#'   \item{r}{number of patients achieving ACR50 at 6 months}
#'   \item{n}{sample size}
#'   \item{disease_duration}{mean disease duration in years}
#' }
#'
#' @references
#'   \insertAllCited{}
#'

"certolizumab"

#' The nma_prior class
#'
#' The `nma_prior` class is used to specify prior distributions.
#'
#' @rdname nma_prior-class
#' @name nma_prior-class
#' @aliases nma_prior
#'
#' @details Objects of class `nma_prior` have the following components:
#'   \describe{
#'   \item{`dist`}{Distribution name}
#'   \item{`fun`}{Name of constructor function, as string (e.g. `"normal"`)}
#'   \item{`...`}{Parameters of the distribution}
#'   }
#'
#' The distribution parameters, specified as named components in `...`, match
#' those in the constructor functions (see [priors]).
NULL

#' @export
#' @noRd
print.nma_prior <- function(x, ...) {
  if (x$dist == "flat (implicit)") {
    cglue("An implicit flat prior distribution over the entire parameter support.")
  } else {
    p <- purrr::list_modify(x, dist = purrr::zap(), fun = purrr::zap())
    if (stringr::str_starts(x$dist, "half-|Exponential")) p <- purrr::list_modify(p, location = purrr::zap())
    p <- p[!is.na(p)]
    cglue("A{if (stringr::str_starts(x$dist, '[aeiouAEIOU]')) 'n' else ''} {x$dist} prior distribution: {paste(names(p), p, sep = ' = ', collapse = ', ')}.")
  }
  invisible(x)
}


#' Summary of prior distributions
#'
#' Print a summary of prior distribution details.
#'
#' @param object Prior distribution as a `nma_prior` object
#' @param ... Additional arguments, not used
#' @param probs Numeric vector of probabilities to calculate prior intervals
#' @param digits Number of digits to display
#' @param trunc Optional numeric vector of length 2, giving the truncation
#'   limits of the prior distribution. Useful if a real-valued prior is assigned
#'   to a positive-valued parameter, then `trunc = c(0, Inf)` will give the
#'   correct prior intervals. By default, truncation is not used.
#'
#' @return A data frame is returned invisibly, giving the prior intervals
#' @export
#'
#' @examples
#' summary(normal(location = 0, scale = 1))
#' summary(half_normal(scale = 1))
#' summary(log_normal(location = -3.93, scale = 1.51))
#'
#' # Truncation limits may be set, for example to restrict a prior to positive values
#' summary(normal(location = 0.5, scale = 1), trunc = c(0, Inf))
#'
summary.nma_prior <- function(object, ..., probs = c(0.5, 0.95), digits = 2, trunc = NULL) {
  check_probs(probs)
  if (!rlang::is_scalar_integerish(digits, finite = TRUE))
    abort("`digits` must be a single integer.")
  if (!is.null(trunc) && !rlang::is_double(trunc, n = 2) || any(is.na(trunc)))
    abort("`trunc` must be a length 2 numeric vector of truncation limits.")

  if (object$dist == "flat (implicit)") {
    if (is.null(trunc)) trunc <- c(-Inf, Inf)
    cglue("A flat prior on the parameter support {round(trunc[1], digits)} to {round(trunc[2], digits)}.")
    invisible(tibble::tibble(probs = 1, lower = trunc[1], upper = trunc[2]))
  } else {
    prior <- get_tidy_prior(object, trunc = trunc, ...) %>%
      tidyr::expand_grid(probs = probs) %>%
      dplyr::group_by(.data[["dist"]], .data[["probs"]]) %>%
      {if (stringr::str_starts(object$dist, "half-|Exponential")) {
        dplyr::summarise(., qfun = paste0("q", .data[["dist"]]),
                         lower = 0,
                         upper = do.call(.data[["qfun"]], args = rlang::list2(p = .data[["probs"]], !!! .[["args"]][[1]])))
      } else {
        dplyr::summarise(., qfun = paste0("q", .data[["dist"]]),
                         lower = do.call(.data[["qfun"]], args = rlang::list2(p = (1 - .data[["probs"]]) / 2, !!! .[["args"]][[1]])),
                         upper = do.call(.data[["qfun"]], args = rlang::list2(p = 1 - (1 - .data[["probs"]]) / 2, !!! .[["args"]][[1]])))
      }}

    print(object)
    cglue("{prior$probs*100}% of the prior density lies between {round(prior$lower, digits)} and {round(prior$upper, digits)}.")

    invisible(prior[ , c("probs", "lower", "upper")])
  }
}

#' Kaplan-Meier curves of survival data
#'
#' This helper function constructs a ggplot2 geom to plot Kaplan-Meier curves
#' from a network containing survival or time-to-event outcomes. This is useful
#' for overlaying the "raw" survival data on the estimated survival functions
#' created with plotted with [plot.surv_nma_summary()], but can also be used
#' standalone to plot Kaplan-Meier curves before fitting a model.
#'
#' @param network A `nma_data` network object containing survival outcomes
#' @param ... Additional arguments passed to [survival::survfit()]
#' @param transform Character string giving the transformation to apply to the
#'   KM curves before plotting. The default is `"identity"` for no
#'   transformation; other options are `"cloglog"` for \eqn{\log(-\log(S))},
#'   `"log"` for \eqn{\log(S)}, or `"cumhaz"` for the cumulative hazard
#'   \eqn{-\log(S)}.
#' @param curve_args Optional list of arguments to customise the curves plotted
#' with [ggplot2::geom_step()]
#' @param cens_args Optional list of arguments to customise the censoring marks
#' plotted with [ggplot2::geom_point()]
#'
#' @return A ggplot2 geom list that can be added to a ggplot2 plot object
#' @export
#'
#' @template ex_ndmm_network
#' @examples
#' # Plot KM curves using ggplot2
#' library(ggplot2)
#'
#' # We need to create an empty ggplot object to add the curves to
#' ggplot() + geom_km(ndmm_net)
#'
#' # Adding plotting options, facets, axis labels, and a plot theme
#' ggplot() +
#'   geom_km(ndmm_net,
#'           curve_args = list(linewidth = 0.5),
#'           cens_args = list(size = 3, shape = 124)) +
#'   facet_wrap(vars(Study)) +
#'   labs(x = "Time", y = "Survival Probability") +
#'   theme_multinma()
#'
#' # Using the transform argument to produce log-log plots (e.g. to assess the
#' # proportional hazards assumption)
#' ggplot() +
#'   geom_km(ndmm_net, transform = "cloglog") +
#'   facet_wrap(vars(Study)) +
#'   theme_multinma()
#'
#' # Using the transform argument to produce cumulative hazard plots
#' ggplot() +
#'   geom_km(ndmm_net, transform = "cumhaz") +
#'   facet_wrap(vars(Study)) +
#'   theme_multinma()
#'
#' # This function can also be used to add KM data to plots of estimated survival
#' # curves from a fitted model, in a similar manner
#' @template ex_ndmm_example
#' @examples
#' # Plot estimated survival curves, and overlay the KM data
#' \donttest{
#' plot(predict(ndmm_fit, type = "survival")) + geom_km(ndmm_net)
#' }
geom_km <- function(network, ...,
                    transform = c("identity", "cloglog", "log", "cumhaz"),
                    curve_args = list(), cens_args = list()) {
  if (!inherits(network, "nma_data"))
    abort("`network` must be an `nma_data` object, as created by the functions `set_*`, `combine_network`, or `add_integration`.")

  if (!(identical(network$outcome$ipd, "survival") || identical(network$outcome$agd_arm, "survival")))
    abort("`network` does not contain survival outcomes.")

  transform <- rlang::arg_match(transform)

  if (!is.list(curve_args))
    abort("`curve_args` must be a list of arguments to pass to `geom_step()`.")

  if (!is.list(cens_args))
    abort("`cens_args` must be a list of arguments to pass to `geom_point()`.")

  dots <- list(...)

  # Get KM fits
  kmdat <- dplyr::bind_rows(network$ipd,
                            if (has_agd_arm(network)) tidyr::unnest(network$agd_arm, cols = ".Surv") else NULL) %>%
    dplyr::group_by(.data$.study, .data$.trt) %>%
    dplyr::group_modify(~dplyr::as_tibble(unclass(
      do.call(survival::survfit, rlang::dots_list(formula = .Surv ~ 1, !!! dots,  data = ., .homonyms = "last"))
      )[c("time", "n.censor", "surv", "std.err", "upper", "lower", "cumhaz")]))

  # Add S(0) = 1
  if (transform != "cloglog") kmdat <- dplyr::group_modify(kmdat, ~dplyr::add_row(., time = 0, n.censor = 0, surv = 1, std.err = 0, upper = 1, lower = 1, cumhaz = 0, .before = 0))

  kmdat <- dplyr::mutate(kmdat, Treatment = .data$.trt, Study = .data$.study)

  aes <- purrr::list_modify(
    ggplot2::aes(x = .data$time, y = .data$surv,
                 colour = .data$Treatment,
                 group = interaction(.data$Study, .data$Treatment)),
    !!! switch(transform,
           "cloglog" = ggplot2::aes(x = log(.data$time), y = log(-log(.data$surv))),
           "log" = ggplot2::aes(y = log(.data$surv)),
           "cumhaz" = ggplot2::aes(y = .data$cumhaz))
  )

  # Set geom args
  curve_args <- rlang::dots_list(mapping = aes,
                                 data = kmdat,
                                 !!! curve_args,
                                 linewidth = 0.25,
                                 .homonyms = "first")

  cens_args <- rlang::dots_list(mapping = aes,
                                data = dplyr::filter(kmdat, .data$n.censor >= 1),
                                !!! cens_args,
                                stroke = 0.25, shape = 3,
                                .homonyms = "first")

  # Output ggplot geoms
  list(do.call(ggplot2::geom_step, args = curve_args), do.call(ggplot2::geom_point, args = cens_args))
}

#' The stan_nma class
#'
#' The `stan_nma` and `stan_mlnmr` classes contains the results from running a
#' model with the function [nma()].
#'
#' @rdname stan_nma-class
#' @name stan_nma-class
#' @aliases stan_nma stan_mlnmr
#'
#' @details Objects of class `stan_nma` and `stan_mlnmr` have the following
#'   components:
#'   \describe{
#'   \item{`network`}{The network data from which the model was run (class
#'   [nma_data] for `stan_nma`, or class [mlnmr_data] for `stan_mlnmr`)}
#'   \item{`stanfit`}{The `stanfit` object returned by calling
#'   \code{\link[rstan:stanmodel-method-sampling]{sampling()}} for the model}
#'   \item{`trt_effects`}{Whether fixed or random effects were used (character
#'   string)}
#'   \item{`consistency`}{The consistency/inconsistency model used (character
#'   string)}
#'   \item{`regression`}{The regression model used (formula)}
#'   \item{`class_interactions`}{If treatment classes and a regression model are
#'   specified, the model used for interactions within each class (common,
#'   exchangeable, or independent)}
#'   \item{`xbar`}{A named vector of values used for centering}
#'   \item{`likelihood`}{The likelihood used (character string)}
#'   \item{`link`}{The link function used (character string)}
#'   \item{`priors`}{A list containing the priors used (as [nma_prior] objects)}
#'   \item{`basis`}{For `mspline` and `pexp` models, a named list of spline
#'    bases for each study}
#'   }
#'
#' The `stan_mlnmr` sub-class inherits from `stan_nma`, and differs only in the
#' class of the `network` object.
#'
NULL

#' Print `stan_nma` objects
#'
#' @param x A [stan_nma] object
#' @param ... Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}
#'
#' @return `x` is returned invisibly.
#'
#' @export
print.stan_nma <- function(x, ...) {
  if (inherits(x$network, "mlnmr_data")) type <- "ML-NMR"
  else type <- "NMA"
  cglue("A {x$trt_effects} effects {type} with a {x$likelihood} likelihood ({x$link} link).")
  if (x$likelihood %in% c("mspline", "pexp")) {
    deg <- switch(x$likelihood,
                  mspline = switch(attr(x$basis[[1]], 'degree'),
                                   "1" = "Piecewise constant",
                                   "2" = "Quadratic M-spline",
                                   "3" = "Cubic M-spline",
                                   paste('Degree', attr(x$basis[[1]], 'degree'), 'M-spline')),
                  pexp = 'Piecewise constant')
    cglue("{deg} baseline hazard with {length(attr(x$basis[[1]], 'knots'))} internal knots.")
  }
  if (x$consistency != "consistency") {
    if (x$consistency == "nodesplit")
      cglue("An inconsistency model ('{x$consistency}') was fitted, splitting the comparison {x$nodesplit[2]} vs. {x$nodesplit[1]}.")
    else
      cglue("An inconsistency model ('{x$consistency}') was fitted.")
  }
  if (!is.null(x$regression)) cglue("Regression model: {rlang::as_label(x$regression)}.")
  if (!is.null(x$aux_regression)) cglue("Auxiliary regression model: {rlang::as_label(x$aux_regression)}.")
  if ((!is.null(x$regression) || !is.null(x$aux_regression)) && !is.null(x$xbar)) {
    cglue("Centred covariates at the following overall mean values:")
    print(x$xbar)
  }
  if (length(setdiff(x$aux_by, ".study"))) cglue("Stratified baseline hazards by {glue::glue_collapse(x$aux_by, sep = ', ', last = ' and ')}.")

  sf <- as.stanfit(x)
  dots <- list(...)
  include <- "pars" %in% names(dots)
  dots <- rlang::dots_list(x = sf,
                           pars = c("log_lik", "resdev",
                                    "fitted_ipd",
                                    "fitted_agd_arm",
                                    "fitted_agd_contrast",
                                    "theta_bar_cum_agd_arm",
                                    "theta_bar_cum_agd_contrast",
                                    "theta2_bar_cum",
                                    "mu", "delta",
                                    if (!is.null(x$aux_regression) &&
                                        length(setdiff(colnames(attr(terms(x$aux_regression), "factor")), ".trt")) > 0) {
                                      if (x$likelihood %in% c("mspline", "pexp")) NULL else "d_aux"
                                    } else {
                                      "beta_aux"
                                    },
                                    "scoef"),
                           include = include,
                           use_cache = FALSE,
                           !!! dots,
                           .homonyms = "last")
  do.call(print, dots)
  invisible(x)
}

#' Posterior summaries from `stan_nma` objects
#'
#' Posterior summaries of model parameters in `stan_nma` objects may be produced
#' using the `summary()` method and plotted with the `plot()` method. NOTE: To
#' produce relative effects, absolute predictions, or posterior ranks, see
#' [relative_effects()], [predict.stan_nma()], [posterior_ranks()],
#' [posterior_rank_probs()].
#'
#' @param x,object A `stan_nma` object
#' @param ... Additional arguments passed on to other methods
#' @param pars,include See \code{\link[rstan:stanfit-method-extract]{rstan::extract()}}
#' @param probs Numeric vector of specifying quantiles of interest, default
#'   `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#' @param summary Logical, calculate posterior summaries? Default `TRUE`.
#'
#' @details The `plot()` method is a shortcut for `plot(summary(stan_nma))`. For
#'   details of plotting options, see [plot.nma_summary()].
#'
#' @return A [nma_summary] object
#' @export
#'
#' @seealso [plot.nma_summary()], [relative_effects()], [predict.stan_nma()],
#'   [posterior_ranks()], [posterior_rank_probs()]
#'
#' @examples
#' ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Summary and plot of all model parameters
#' summary(smk_fit_RE)
#' plot(smk_fit_RE)
#'
#' # Summary and plot of heterogeneity tau only
#' summary(smk_fit_RE, pars = "tau")
#' plot(smk_fit_RE, pars = "tau")
#'
#' # Customising plot output
#' plot(smk_fit_RE,
#'      pars = c("d", "tau"),
#'      stat = "halfeye",
#'      ref_line = 0)
#' }
summary.stan_nma <- function(object, ...,
                             pars, include,
                             probs = c(0.025, 0.25, 0.5, 0.75, 0.975)
                             ) {

  # Set defaults for pars, include
  if (missing(include)) {
    include <- !missing(pars)
  } else {
    if (!rlang::is_bool(include)) abort("`include` should be TRUE or FALSE")
  }
  if (missing(pars)) {
    pars <- c("log_lik", "resdev", "lp__")
    if (has_ipd(object$network)) pars <- c(pars, "fitted_ipd")
    if (has_agd_arm(object$network)) pars <- c(pars, "fitted_agd_arm")
    if (has_agd_contrast(object$network)) pars <- c(pars, "fitted_agd_contrast")
    if (inherits(object, "stan_mlnmr")) {
      if (has_agd_arm(object$network)) pars <- c(pars, "theta_bar_cum_agd_arm")
      if (has_agd_contrast(object$network)) pars <- c(pars, "theta_bar_cum_agd_contrast")
    }
    if (object$likelihood %in% c("bernoulli2", "binomial2")) {
      if (has_agd_arm(object$network)) pars <- c(pars, "theta2_bar_cum")
    }
  } else {
    if (!is.character(pars)) abort("`pars` should be a character vector")
  }

  sims <- as.array(object, pars = pars, include = include)
  sums <- summary_mcmc_array(sims, probs = probs)
  ss <- list(summary = sums, sims = sims)
  class(ss) <- c("nma_parameter_summary", "nma_summary")
  attr(ss, "xlab") <- "Parameter"
  attr(ss, "ylab") <- "Value"
  return(ss)
}

#' @param stat Character string specifying the `ggdist` plot stat to use,
#'   default `"pointinterval"`
#' @param orientation Whether the `ggdist` geom is drawn horizontally
#'   (`"horizontal"`) or vertically (`"vertical"`), default `"horizontal"`
#' @param ref_line Numeric vector of positions for reference lines, by default
#'   no reference lines are drawn
#' @rdname summary.stan_nma
#' @export
plot.stan_nma <- function(x, ...,
                          pars, include,
                          stat = "pointinterval",
                          orientation = c("horizontal", "vertical", "y", "x"),
                          ref_line = NA_real_) {

  # All checks carried out by downstream functions

  s <- summary(x, pars = pars, include = include)
  p <- plot(s, ..., stat = stat, orientation = orientation, ref_line = ref_line)
  return(p)
}

#' Plot prior vs posterior distribution
#'
#' Produce plots comparing the prior and posterior distributions of model
#' parameters.
#'
#' @param x A `stan_nma` object
#' @param ... Additional arguments passed on to methods
#' @param prior Character vector selecting the prior and posterior
#'   distribution(s) to plot. May include `"intercept"`, `"trt"`, `"het"`,
#'   `"reg"`, `"aux"`, `"class_mean"` or `"class_sd"` as appropriate.
#' @param post_args List of arguments passed on to [ggplot2::geom_histogram] to
#'   control plot output for the posterior distribution
#' @param prior_args List of arguments passed on to [ggplot2::geom_path] to
#'   control plot output for the prior distribution. Additionally, `n` controls
#'   the number of points the density curve is evaluated at (default `500`), and
#'   `p_limits` controls the endpoints of the curve as quantiles (default
#'   `c(.001, .999)`).
#' @param overlay String, should prior or posterior be shown on top? Default
#'   `"prior"`.
#' @param ref_line Numeric vector of positions for reference lines, by default
#'   no reference lines are drawn
#'
#' @return A `ggplot` object.
#' @export
#'
#' @details Prior distributions are displayed as lines, posterior distributions
#'   are displayed as histograms.
#'
#' @importFrom truncdist dtrunc ptrunc qtrunc
#'
#' @examples
#' ## Smoking cessation NMA
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Plot prior vs. posterior, by default all parameters are plotted
#' plot_prior_posterior(smk_fit_RE)
#'
#' # Plot prior vs. posterior for heterogeneity SD only
#' plot_prior_posterior(smk_fit_RE, prior = "het")
#'
#' # Customise plot
#' plot_prior_posterior(smk_fit_RE, prior = "het",
#'                      prior_args = list(colour = "darkred", linewidth = 2),
#'                      post_args = list(alpha = 0.6))
#' }
#'
plot_prior_posterior <- function(x, ...,
                                 prior = NULL,
                                 post_args = list(),
                                 prior_args = list(),
                                 overlay = c("prior", "posterior"),
                                 ref_line = NA_real_) {

  # Checks
  if (!inherits(x, "stan_nma"))
    abort("Not a `stan_nma` object.")

  priors_used <-
    c("intercept"[!is.null(x$priors$prior_intercept)],
      "trt"[!is.null(x$priors$prior_trt)],
      "het"[!is.null(x$priors$prior_het)],
      "reg"[!is.null(x$priors$prior_reg)],
      "aux"[!is.null(x$priors$prior_aux)],
      "aux_reg"[!is.null(x$priors$prior_aux_reg)],
      "class_mean"[!is.null(x$priors$prior_class_mean)],
      "class_sd"[!is.null(x$priors$prior_class_sd)])

  if (is.null(prior)) {
    prior <- priors_used
  } else if (!rlang::is_character(prior) || !all(prior %in% priors_used)) {
    abort(paste0("`prior` should be a character vector, with elements from ",
                paste(priors_used, collapse = ", ")))
  }

  if (!is.list(post_args))
    abort("`post_args` should be a list of arguments to pass to ggdist::stat_sample_slabinterval")

  if (!is.list(prior_args))
    abort("`prior_args` should be a list of arguments to pass to ggdist::stat_dist_slabinterval")

  overlay <- rlang::arg_match(overlay)

  if (!is.numeric(ref_line) || !is.null(dim(ref_line)))
    abort("`ref_line` should be a numeric vector.")

  if (x$likelihood %in% c("mspline", "pexp")) n_scoef <- ncol(x$basis[[1]])

  # Get prior details
  prior_dat <- vector("list", length(prior))
  for (i in seq_along(prior)) {
    if (prior[i] %in% c("het", "aux") || (prior[i] == "aux_reg" && x$likelihood %in% c("mspline", "pexp"))) trunc <- c(0, Inf)
    else trunc <- NULL

    prior_dat[[i]] <- get_tidy_prior(x$priors[[paste0("prior_", prior[i])]], trunc = trunc) %>%
      tibble::add_column(prior = prior[i])

    if (x$likelihood == "gengamma" && prior[i] == "aux") {
      prior_dat[[i]] <-
        dplyr::bind_rows(get_tidy_prior(x$priors$prior_aux$sigma, trunc = trunc),
                         get_tidy_prior(x$priors$prior_aux$k, trunc = trunc)) %>%
        tibble::add_column(prior = c("aux", "aux2"))

    } else {
      prior_dat[[i]] <- get_tidy_prior(x$priors[[paste0("prior_", prior[i])]], trunc = trunc) %>%
        tibble::add_column(prior = prior[i])
    }
  }

  prior_dat <- dplyr::bind_rows(prior_dat) %>%
    dplyr::mutate(par_base = dplyr::recode(.data$prior,
                                           intercept = "mu",
                                           trt = "d",
                                           het = "tau",
                                           reg = "beta",
                                           aux = switch(x$likelihood,
                                                        normal = "sigma",
                                                        ordered = "cc",
                                                        weibull = "shape",
                                                        gompertz = "shape",
                                                        `weibull-aft` = "shape",
                                                        lognormal = "sdlog",
                                                        loglogistic = "shape",
                                                        gamma = "shape",
                                                        gengamma = "sigma",
                                                        mspline = "sigma",
                                                        pexp = "sigma"),
                                           aux2 = switch(x$likelihood,
                                                         gengamma = "k"),
                                           aux_reg = switch(x$likelihood,
                                                            pexp =, mspline = "sigma_beta",
                                                            weibull =, gompertz =, `weibull-aft` =,
                                                            lognormal =, loglogistic =, gamma =,
                                                            gengamma = "beta_aux"),
                                           class_mean = "class_mean",
                                           class_sd = "class_sd"))

  # Add in omega parameter if node-splitting model, which uses prior_trt
  if (inherits(x, "nma_nodesplit")) {
    prior_dat <- dplyr::bind_rows(
      prior_dat,
      dplyr::filter(prior_dat, .data$prior == "trt") %>%
        dplyr::mutate(par_base = "omega")
    )
  }

  # Get parameter samples
  pars <- unique(prior_dat$par_base)

  draws <- tibble::as_tibble(as.matrix(x, pars = pars))

  # Transform heterogeneity samples to prior scale (SD, variance, precision)
  if ("het" %in% prior) {
    if (x$priors$prior_het_type == "var") {
      draws$tausq <- draws$tau^2
      draws <- dplyr::select(draws, -"tau")
      prior_dat$par_base <- dplyr::recode(prior_dat$par_base, tau = "tausq")
    } else if (x$priors$prior_het_type == "prec") {
      draws$prec <- draws$tau^-2
      draws <- dplyr::select(draws, -"tau")
      prior_dat$par_base <- dplyr::recode(prior_dat$par_base, tau = "prec")
    }
  }

  # For ordered likelihood, priors are specified on differences between cutoffs
  if (x$likelihood == "ordered" && "aux" %in% prior) {
    l_cat <- if (has_ipd(x$network)) colnames(x$network$ipd$.r) else colnames(x$network$agd_arm$.r)
    n_cat <- length(l_cat)

    if (n_cat <= 2) {
      draws <- dplyr::select(draws, -dplyr::starts_with("cc["))
      prior_dat <- dplyr::filter(prior_dat, prior != "aux")
    } else {
      for (i in 2:(n_cat-1)) {
        draws <- dplyr::mutate(draws, !! paste0("diff_cc[", l_cat[i+1], " - ", l_cat[i], "]") :=
                                        !! as.symbol(paste0("cc[", l_cat[i+1], "]")) - !! as.symbol(paste0("cc[", l_cat[i], "]")))
      }
      draws <- dplyr::select(draws, -dplyr::starts_with("cc["))
    }

    prior_dat <- dplyr::mutate(prior_dat, par_base = dplyr::recode(.data$par_base, cc = "diff_cc"))
  }

  draws <- tidyr::pivot_longer(draws, cols = dplyr::everything(),
                               names_to = "parameter", values_to = "value")

  draws$par_base <- stringr::str_remove(draws$parameter, "\\[.*\\]")
  draws$parameter <- forcats::fct_inorder(factor(draws$parameter))

  # Join prior name into posterior
  draws <- dplyr::left_join(draws, prior_dat[, c("par_base", "prior")], by = "par_base")

  # Calculate prior density lines
  if (rlang::has_name(prior_args, "p_limits")) {
    p_limits <- prior_args$p_limits
    prior_args <- purrr::list_modify(prior_args, p_limits = purrr::zap())
  } else {
    p_limits <- c(0.001, 0.999)
  }
  if (rlang::has_name(prior_args, "n")) {
    n <- prior_args$n
    prior_args <- purrr::list_modify(prior_args, n = purrr::zap())
  } else {
    n <- 501
  }

  xseq <- dens <- vector("list", nrow(prior_dat))
  for (i in seq_len(nrow(prior_dat))) {
    dist <- prior_dat$dist[[i]]
    args <- prior_dat$args[[i]]

    if (dist == "unif") {
      # lower <- max(args$min, -1e12)
      # upper <- min(args$max, 1e12)

      xseq[[i]] <- c(args$min, args$max)
      if (is.infinite(args$min) || is.infinite(args$max)) {
        dens[[i]] <- c(0, 0)
      } else {
        dens[[i]] <- dunif(xseq[[i]], min = args$min, max = args$max)
      }
    } else {
      lower <- eval(rlang::call2(paste0("q", dist), p = p_limits[1], !!! args))
      upper <- eval(rlang::call2(paste0("q", dist), p = p_limits[2], !!! args))

      xseq[[i]] <- seq(from = lower, to = upper, length.out = n)

      dens[[i]] <- eval(rlang::call2(paste0("d", dist), x = xseq[[i]], !!! args))
    }
  }

  prior_dat <- tibble::add_column(prior_dat, xseq = xseq, dens = dens)
  prior_dat <- tidyr::unnest(prior_dat, c("xseq", "dens"))

  # Repeat rows of prior_dat for each corresponding parameter
  if (packageVersion("dplyr") >= "1.1.1") {
    prior_dat <- dplyr::left_join(prior_dat,
                                  dplyr::distinct(draws, .data$par_base, .data$parameter),
                                  by = "par_base",
                                  relationship = "many-to-many")
  } else {
    prior_dat <- dplyr::left_join(prior_dat,
                                  dplyr::distinct(draws, .data$par_base, .data$parameter),
                                  by = "par_base")
  }

  # Construct plot
  xlim <- c(min(draws$value, 0), max(draws$value))

  p <- ggplot2::ggplot() +
    ggplot2::geom_vline(xintercept = ref_line, na.rm = TRUE, colour = "grey60") +
    ggplot2::coord_cartesian(xlim = xlim)

  g_prior <- rlang::call2(ggplot2::geom_line,
                          !!! rlang::dots_list(mapping = ggplot2::aes(x = .data$xseq, y = .data$dens),
                                               data = prior_dat,
                                               !!! prior_args,
                                               .homonyms = "last"))

  g_post <- rlang::call2(ggplot2::geom_histogram,
                         !!! rlang::dots_list(mapping = ggplot2::aes(y = ggplot2::after_stat(.data$density), x = .data$value, group = .data$parameter),
                                              data = draws,
                                              binwidth = function(x) diff(range(x)) / nclass.Sturges(x),
                                              boundary = 0,
                                              position = "identity",
                                              !!! post_args,
                                              .homonyms = "last"))

  if (overlay == "prior") {
    p <- p + eval(g_post) + eval(g_prior)
  } else {
    p <- p + eval(g_prior) + eval(g_post)
  }

  p <- p +
    ggplot2::facet_wrap("parameter", scales = "free") +
    theme_multinma()

  return(p)
}

#' Plot numerical integration error
#'
#' For ML-NMR models, plot the estimated numerical integration error over the
#' entire posterior distribution, as the number of integration points increases.
#' See \insertCite{methods_paper,Phillippo_thesis}{multinma} for details.
#'
#' @param x An object of type `stan_mlnmr`
#' @param ... Additional arguments passed to the `ggdist` plot stat.
#' @param stat Character string specifying the `ggdist` plot stat used to
#'   summarise the integration error over the posterior. Default is `"violin"`,
#'   which is equivalent to `"eye"` with some cosmetic tweaks.
#' @param orientation Whether the `ggdist` geom is drawn horizontally
#'   (`"horizontal"`) or vertically (`"vertical"`), default `"vertical"`
#' @param show_expected_rate Logical, show typical convergence rate \eqn{1/N}?
#'   Default `TRUE`.
#'
#' @details The total number of integration points is set by the `n_int`
#'   argument to [add_integration()], and the intervals at which integration
#'   error is estimated are set by the `int_thin` argument to [nma()]. The
#'   typical convergence rate of Quasi-Monte Carlo integration (as used here) is
#'   \eqn{1/N}, which by default is displayed on the plot output.
#'
#'   The integration error at each thinning interval \eqn{N_\mathrm{thin}}{N_thin} is
#'   estimated for each point in the posterior distribution by subtracting the
#'   final estimate (using all `n_int` points) from the estimate using only the
#'   first \eqn{N_\mathrm{thin}}{N_thin} points.
#'
#' # Note for survival models
#' This function is not supported for survival/time-to-event models. These do
#' not save cumulative integration points for efficiency reasons (both time and
#' memory).
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_network
#' @template ex_plaque_psoriasis_integration
#' @examples \donttest{
#' # Fit the ML-NMR model
#' pso_fit <- nma(pso_net,
#'                trt_effects = "fixed",
#'                link = "probit",
#'                likelihood = "bernoulli2",
#'                regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
#'                class_interactions = "common",
#'                prior_intercept = normal(scale = 10),
#'                prior_trt = normal(scale = 10),
#'                prior_reg = normal(scale = 10),
#'                init_r = 0.1,
#'                QR = TRUE,
#'                # Set the thinning factor for saving the cumulative results
#'                # (This also sets int_check = FALSE)
#'                int_thin = 8)
#' pso_fit
#'
#' # Plot numerical integration error
#' plot_integration_error(pso_fit)
#' }
plot_integration_error <- function(x, ...,
                                   stat = "violin",
                                   orientation = c("vertical", "horizontal", "x", "y"),
                                   show_expected_rate = TRUE) {
  # Checks
  if (!inherits(x, "stan_mlnmr"))
    abort("Expecting a `stan_mlnmr` object, created by fitting a ML-NMR model with numerical integration using the `nma()` function.")

  if (inherits(x, "stan_nma_surv"))
    abort("Not supported for survival models; cumulative integration points are not saved for efficiency reasons.")

  if (!rlang::is_bool(show_expected_rate))
    abort("`show_expected_rate` must be a logical value, TRUE or FALSE.")

  if (!rlang::is_string(stat))
    abort("`stat` should be a character string specifying the name of a ggdist stat")

  stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")

  if (violin <- stat == "violin") {
    stat <- "eye"
  }

  tb_geom <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
                      error = function(err) {
                        abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
                                    err, sep = "\n"))
                      })

  orientation <- rlang::arg_match(orientation)
  if (orientation == "x") orientation <- "vertical"
  else if (orientation == "y") orientation <- "horizontal"

  # Is a horizontal geom specified?
  horizontal <- orientation == "horizontal"

  # Get cumulative integration points
  twoparbin <- x$likelihood %in% c("binomial2", "bernoulli2")
  multi <- x$likelihood == "ordered"

  ipars <- c()
  if (has_agd_arm(x$network)) {
    ipars <- c(ipars, "theta_bar_cum_agd_arm")
  }
  if (has_agd_contrast(x$network)) {
    ipars <- c(ipars, "theta_bar_cum_agd_contrast")
  }
  if (twoparbin) {
    ipars <- c(ipars, "theta2_bar_cum")
  }

  if (!all(ipars %in% x$stanfit@sim$pars_oi))
    abort(paste0("Cumulative integration points not saved.\n",
                 "Re-run model with `int_thin > 0` and `int_check = FALSE` to use this feature."))

  int_dat <- as.data.frame(x, pars = ipars) %>%
    dplyr::mutate(.draw = 1:dplyr::n())

  colnames(int_dat) <- stringr::str_remove(colnames(int_dat), "_bar_cum(_agd_arm|_agd_contrast)?")

  n_int <- x$network$n_int

  rx <- if (multi) "^(theta2?)\\[(.+): (.+), ([0-9]+), (.+)\\]$" else "^(theta2?)\\[(.+): (.+), ([0-9]+)\\]$"

  int_dat <- tidyr::pivot_longer(int_dat, cols = -dplyr::one_of(".draw"),
                                 names_pattern = rx,
                                 names_to = if (multi) c("parameter", "study", "treatment", "n_int", "category") else c("parameter", "study", "treatment", "n_int"),
                                 names_transform = list(n_int = as.integer),
                                 values_to = "value")

  int_dat$study <- factor(int_dat$study, levels = levels(x$network$studies))
  int_dat$treatment <- factor(int_dat$treatment, levels = levels(x$network$treatments))
  if (multi) int_dat$category <- forcats::fct_inorder(factor(int_dat$category))

  # Estimate integration error by subtracting final value
  int_dat <- dplyr::left_join(dplyr::filter(int_dat, .data$n_int != max(.data$n_int)),
                              dplyr::filter(int_dat, .data$n_int == max(.data$n_int)) %>%
                                dplyr::rename(final_value = "value") %>%
                                dplyr::select(-"n_int"),
                              by = if (multi)  c("parameter", "study", "treatment", "category", ".draw") else c("parameter", "study", "treatment", ".draw")) %>%
    dplyr::mutate(diff = .data$value - .data$final_value)

  int_thin <- min(int_dat$n_int)

  # Reference convergence rates
  if (show_expected_rate) {
    conv_dat <- dplyr::bind_rows(
      tibble::tibble(n_int = seq(1, n_int, length.out = 501), diff = n_int^-1, group = "pos"),
      tibble::tibble(n_int = seq(1, n_int, length.out = 501), diff = -n_int^-1, group = "neg"))
  }

  # Create plot
  if (horizontal) {
    p <- ggplot2::ggplot(int_dat, ggplot2::aes(y = .data$n_int, x = .data$diff)) +
      ggplot2::geom_vline(xintercept = 0, colour = "grey60") +
      ggplot2::xlab("Estimated integration error") +
      ggplot2::ylab("Number of integration points") +
      ggplot2::coord_cartesian(xlim = range(int_dat$diff))
  } else {
    p <- ggplot2::ggplot(int_dat, ggplot2::aes(x = .data$n_int, y = .data$diff)) +
      ggplot2::geom_hline(yintercept = 0, colour = "grey60") +
      ggplot2::ylab("Estimated integration error") +
      ggplot2::xlab("Number of integration points") +
      ggplot2::coord_cartesian(ylim = range(int_dat$diff))
  }

  if (show_expected_rate) {
    p <- p +
      ggplot2::geom_line(ggplot2::aes(group = .data$group),
                         data = conv_dat,
                         colour = "grey60", linetype = 2)
  }

  # Custom format for violin plot
  v_args <-
    if (violin) {
      if (twoparbin) {
        list(point_interval = NULL,
             alpha = 0.8,
             slab_size = 0.5)
        } else {
          list(point_interval = NULL,
               slab_colour = "black",
               slab_size = 0.5)
        }
    } else list()

  if (twoparbin) {
    p <- p +
      do.call(tb_geom,
              args = rlang::dots_list(ggplot2::aes(colour = .data$parameter,
                                                   fill = .data$parameter,
                                                   slab_colour = .data$parameter),
                                      orientation = orientation,
                                      ...,
                                      !!! v_args,
                                      position = if (!horizontal) ggplot2::position_dodge(width = int_thin / 10) else "identity",
                                      .homonyms = "first")) +
      ggplot2::scale_colour_manual("Parameter",
                                   values = c(theta = "#113259", theta2 = "#55A480"),
                                   labels = function(x) dplyr::recode(x,
                                     theta = expression(bar(p)),
                                     theta2 = expression(bar(p)^2)),
                                   aesthetics = c("colour", "fill", "slab_colour"))
  } else {
    p <- p +
      do.call(tb_geom,
              args = rlang::dots_list(orientation = orientation, ..., !!! v_args, .homonyms = "first"))
  }

  if (multi) {
    p <- p + ggplot2::facet_grid(study + treatment ~ category)
  } else {
    p <- p + ggplot2::facet_wrap(~ study + treatment)
  }

  p <- p + theme_multinma()

  return(p)
}

#' as.stanfit
#'
#' Attempt to turn an object into a \code{\link[rstan:stanfit-class]{stanfit}} object.
#'
#' @param x an object
#' @param ... additional arguments
#'
#' @return A \code{\link[rstan:stanfit-class]{stanfit}} object.
#' @export
as.stanfit <- function(x, ...) {
  UseMethod("as.stanfit")
}

#' @export
#' @rdname as.stanfit
as.stanfit.stan_nma <- function(x, ...) {
  return(x[["stanfit"]])
}

#' @export
#' @rdname as.stanfit
as.stanfit.default <- function(x, ...) {
  abort(glue::glue("Cannot coerce object of class '{class(x)}' to 'stanfit'."))
}

#' Convert samples into arrays, matrices, or data frames
#'
#' Samples (post warm-up) from a `stan_nma` model object can be coerced into an
#' array, matrix, or data frame.
#'
#' @param x A `stan_nma` object
#' @param ... Additional arguments passed to \code{\link[rstan:as.array.stanfit]{as.array.stanfit()}}
#' @param pars Optional character vector of parameter names to include in output. If not specified, all parameters are used.
#' @param include Logical, are parameters in `pars` to be included (`TRUE`, default) or excluded (`FALSE`)?
#'
#' @return The `as.array()` method produces a 3D array \[Iteration, Chain,
#'   Parameter\] containing posterior samples of each parameter (as class
#'   [mcmc_array]). This has the side effect of enabling
#'   \code{\link[bayesplot:bayesplot-package]{bayesplot}} functions to
#'   seamlessly work on `stan_nma` objects.
#'
#'   The `as.data.frame()` method produces a data frame containing posterior
#'   samples of each parameter, combined over all chains.
#'
#'   The `as.matrix()` method produces a matrix containing posterior samples of
#'   each parameter, combined over all chains.
#'
#' @export
as.array.stan_nma <- function(x, ..., pars, include = TRUE) {
  if (!rlang::is_bool(include))
    abort("`include` must be TRUE or FALSE.")
  if (!missing(pars)) {
    if (!is.character(pars))
      abort("`pars` must be a character vector of parameters to include (or missing).")

    allpars <- c(x$stanfit@sim$pars_oi, x$stanfit@sim$fnames_oi)
    badpars <- setdiff(pars, allpars)
    if (length(badpars))
      abort(glue::glue("No parameter{if (length(badpars) > 1) 's' else ''} ",
                       glue::glue_collapse(glue::double_quote(badpars), sep = ", ", last = " or "), "."))

    # Extract from stanfit only parameters represented in pars
    if (include) {
      par_base <- stringr::str_remove(pars, "\\[.*$")
    } else {
      par_base <- setdiff(x$stanfit@sim$pars_oi, pars)
    }
    a <- as.array(as.stanfit(x), pars = par_base, ...)

    # Get parameter indices, respecting order of `pars`
    par_regex <- paste0("^\\Q", pars, "\\E(\\[|$)")
    par_select <- unique(unlist(lapply(par_regex, grep, x = dimnames(a)[[3]],
                                       perl = TRUE, invert = !include)))
    out <- a[ , , par_select, drop = FALSE]

  } else {
    out <- as.array(as.stanfit(x), ...)
  }

  class(out) <- c("mcmc_array", "array")

  return(out)
}

#' @rdname as.array.stan_nma
#' @export
as.data.frame.stan_nma <- function(x, ..., pars, include = TRUE) {
  return(as.data.frame(as.matrix(x, ..., pars = pars, include = include)))
}

#' @rdname as.array.stan_nma
#' @export
as_tibble.stan_nma <- function(x, ..., pars, include = TRUE) {
  return(tibble::as_tibble(as.matrix(x, ..., pars = pars, include = include)))
}

#' @rdname as.array.stan_nma
#' @method as.tibble stan_nma
#' @export
as.tibble.stan_nma <- function(x, ..., pars, include = TRUE) {
  return(tibble::as_tibble(as.matrix(x, ..., pars = pars, include = include)))
}

#' @rdname as.array.stan_nma
#' @export
as.matrix.stan_nma <- function(x, ..., pars, include = TRUE) {
  a <- as.array(x, ..., pars = pars, include = include)
  names_a <- dimnames(a)
  dim_a <- dim(a)
  dim(a) <- c(dim_a[1] * dim_a[2], dim_a[3])
  dimnames(a) <- names_a[-2]
  class(a) <- "matrix"
  return(a)
}

#' Model comparison using the `loo` package
#'
#' The \code{\link[loo:loo]{loo()}} and \code{\link[loo:waic]{waic()}} functions from the `loo`
#' package may be called directly on [stan_nma] and [stan_mlnmr] objects.
#'
#' @param x An object of class [stan_nma] or [stan_mlnmr]
#' @param ... Further arguments to \code{\link[rstan:stanfit-method-loo]{loo()}} or
#'   \code{\link[loo:waic]{waic()}}
#'
#' @rdname loo
#' @aliases loo
#' @method loo stan_nma
# Dynamically exported, see zzz.R
loo.stan_nma <- function(x, ...) {
  sf <- as.stanfit(x)
  return(rstan::loo(sf, ...))
}


#' @rdname loo
#' @aliases waic
#' @method waic stan_nma
# Dynamically exported, see zzz.R
waic.stan_nma <- function(x, ...) {
  ll <- as.array(x, pars = "log_lik")
  return(loo::waic(ll, ...))
}

#' Matrix of plots for a `stan_nma` object
#'
#' A [pairs()] method for `stan_nma` objects, which calls
#' \code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}} on the
#' underlying `stanfit` object.
#'
#' @param x An object of class `stan_nma`
#' @param ... Other arguments passed to
#'   \code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}}
#' @param pars Optional character vector of parameter names to include in
#'   output. If not specified, all parameters are used.
#' @param include Logical, are parameters in `pars` to be included (`TRUE`,
#'   default) or excluded (`FALSE`)?
#'
#' @return A grid of ggplot objects produced by
#'   \code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}}.
#' @export
#'
#' @examples \dontrun{
#' ## Parkinson's mean off time reduction
#' park_net <- set_agd_arm(parkinsons,
#'                         study = studyn,
#'                         trt = trtn,
#'                         y = y,
#'                         se = se,
#'                         sample_size = n)
#'
#' # Fitting a RE model
#' park_fit_RE <- nma(park_net,
#'                    trt_effects = "random",
#'                    prior_intercept = normal(scale = 100),
#'                    prior_trt = normal(scale = 100),
#'                    prior_het = half_normal(scale = 5))
#'
#' # We see a small number of divergent transition errors
#' # These do not go away entirely when adapt_delta is increased
#'
#' # Try to diagnose with a pairs plot
#' pairs(park_fit_RE, pars = c("mu[4]", "d[3]", "delta[4: 3]", "tau"))
#'
#' # Transforming tau onto log scale
#' pairs(park_fit_RE, pars = c("mu[4]", "d[3]", "delta[4: 3]", "tau"),
#'       transformations = list(tau = "log"))
#'
#' # The divergent transitions occur in the upper tail of the heterogeneity
#' # standard deviation. In this case, with only a small number of studies, there
#' # is not very much information to estimate the heterogeneity standard deviation
#' # and the prior distribution may be too heavy-tailed. We could consider a more
#' # informative prior distribution for the heterogeneity variance to aid
#' # estimation.
#' }
#'
pairs.stan_nma <- function(x, ..., pars, include = TRUE) {
  sf <- as.stanfit(x)
  post_array <- as.array(x, pars = pars, include = include)

  max_td <- sf@stan_args[[1]]$control$max_treedepth
  if (is.null(max_td)) max_td <- 10

  args <- rlang::dots_list(x = post_array,
                           np = bayesplot::nuts_params(sf),
                           lp = bayesplot::log_posterior(sf),
                           max_treedepth = max_td,
                           ...,
                           condition = bayesplot::pairs_condition(nuts = "accept_stat__"),
                           .homonyms = "first")

  thm <- ggplot2::theme_set(theme_multinma())
  out <- do.call(bayesplot::mcmc_pairs, args = args)
  ggplot2::theme_set(thm)
  return(out)
}

#' Set default values
#'
#' The `.default()` function is used internally to mark certain values as
#' default, so that the user may be notified when default values are being used.
#' For example, choosing a default reference treatment for a network, or using
#' default prior distributions. The function `.is_default()` checks whether an
#' argument/object is set to a default value. Neither of these functions are
#' intended to be called by the user.
#'
#' @param x An object
#'
#' @return For `.default()`, an identical object with additional attribute
#'   `.default`. For `.is_default()`, a logical value (`TRUE` or `FALSE`).
#' @export
#' @rdname default_values
#'
.default <- function(x = list()) {
  attr(x, ".default") <- TRUE
  return(x)
}

#' @export
#' @rdname default_values
.is_default <- function(x) {
  return(isTRUE(attr(x, ".default", exact = TRUE)))
}

#' Marginal treatment effects
#'
#' Generate population-average marginal treatment effects. These are formed from
#' population-average absolute predictions, so this function is a wrapper around
#' [predict.stan_nma()].
#'
#' @param object A `stan_nma` object created by [nma()].
#' @param ... Arguments passed to [predict.stan_nma()], for example to specify
#'   the covariate distribution and baseline risk for a target population, e.g.
#'   `newdata`, `baseline`, and related arguments. For survival outcomes, `type`
#'   can also be specified to determine the quantity from which to form a
#'   marginal effect. For example, `type = "hazard"` with `mtype = "ratio"`
#'   produces marginal hazard ratios, `type = "median"` with `mtype =
#'   "difference"` produces marginal median survival time differences, and so
#'   on.
#' @param mtype The type of marginal effect to construct from the average
#'   absolute effects, either `"difference"` (the default) for a difference of
#'   absolute effects such as a risk difference, `"ratio"` for a ratio of
#'   absolute effects such as a risk ratio, or `"link"` for a difference on the
#'   scale of the link function used in fitting the model such as a marginal log
#'   odds ratio.
#' @param all_contrasts Logical, generate estimates for all contrasts (`TRUE`),
#'   or just the "basic" contrasts against the network reference treatment
#'   (`FALSE`)? Default `FALSE`.
#' @param trt_ref Reference treatment to construct relative effects against, if
#'   `all_contrasts = FALSE`. By default, relative effects will be against the
#'   network reference treatment. Coerced to character string.
#' @param probs Numeric vector of quantiles of interest to present in computed
#'   summary, default `c(0.025, 0.25, 0.5, 0.75, 0.975)`
#' @param predictive_distribution Logical, when a random effects model has been
#'   fitted, should the predictive distribution for marginal effects in a new
#'   study be returned? Default `FALSE`.
#' @param summary Logical, calculate posterior summaries? Default `TRUE`.
#'
#' @return A [nma_summary] object if `summary = TRUE`, otherwise a list
#'   containing a 3D MCMC array of samples and (for regression models) a data
#'   frame of study information.
#' @export
#'
#' @examples ## Smoking cessation
#' @template ex_smoking_nma_re_example
#' @examples \donttest{
#' # Marginal risk difference in each study population in the network
#' marginal_effects(smk_fit_RE, mtype = "difference")
#'
#' # Since there are no covariates in the model, the marginal and conditional
#' # (log) odds ratios here coincide
#' marginal_effects(smk_fit_RE, mtype = "link")
#' relative_effects(smk_fit_RE)
#'
#' # Marginal risk differences in a population with 67 observed events out of
#' # 566 individuals on No Intervention, corresponding to a Beta(67, 566 - 67)
#' # distribution on the baseline probability of response
#' (smk_rd_RE <- marginal_effects(smk_fit_RE,
#'                                baseline = distr(qbeta, 67, 566 - 67),
#'                                baseline_type = "response",
#'                                mtype = "difference"))
#' plot(smk_rd_RE)
#' }
#'
#' ## Plaque psoriasis ML-NMR
#' @template ex_plaque_psoriasis_mlnmr_example
#' @examples \donttest{
#' # Population-average marginal probit differences in each study in the network
#' (pso_marg <- marginal_effects(pso_fit, mtype = "link"))
#' plot(pso_marg, ref_line = c(0, 1))
#'
#' # Population-average marginal probit differences in a new target population,
#' # with means and SDs or proportions given by
#' new_agd_int <- data.frame(
#'   bsa_mean = 0.6,
#'   bsa_sd = 0.3,
#'   prevsys = 0.1,
#'   psa = 0.2,
#'   weight_mean = 10,
#'   weight_sd = 1,
#'   durnpso_mean = 3,
#'   durnpso_sd = 1
#' )
#'
#' # We need to add integration points to this data frame of new data
#' # We use the weighted mean correlation matrix computed from the IPD studies
#' new_agd_int <- add_integration(new_agd_int,
#'                                durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
#'                                prevsys = distr(qbern, prob = prevsys),
#'                                bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
#'                                weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
#'                                psa = distr(qbern, prob = psa),
#'                                cor = pso_net$int_cor,
#'                                n_int = 64)
#'
#' # Population-average marginal probit differences of achieving PASI 75 in this
#' # target population, given a Normal(-1.75, 0.08^2) distribution on the
#' # baseline probit-probability of response on Placebo (at the reference levels
#' # of the covariates), are given by
#' (pso_marg_new <- marginal_effects(pso_fit,
#'                                   mtype = "link",
#'                                   newdata = new_agd_int,
#'                                   baseline = distr(qnorm, -1.75, 0.08)))
#' plot(pso_marg_new)
#' }
#'
#' ## Progression free survival with newly-diagnosed multiple myeloma
#' @template ex_ndmm_example
#' @examples \donttest{
#' # We can produce a range of marginal effects from models with survival
#' # outcomes, specified with the mtype and type arguments. For example:
#'
#' # Marginal survival probability difference at 5 years, all contrasts
#' marginal_effects(ndmm_fit, type = "survival", mtype = "difference",
#'                  times = 5, all_contrasts = TRUE)
#'
#' # Marginal difference in RMST up to 5 years
#' marginal_effects(ndmm_fit, type = "rmst", mtype = "difference", times = 5)
#'
#' # Marginal median survival time ratios
#' marginal_effects(ndmm_fit, type = "median", mtype = "ratio")
#'
#' # Marginal log hazard ratios
#' # With no covariates in the model, these are constant over time and study
#' # populations, and are equal to the log hazard ratios from relative_effects()
#' plot(marginal_effects(ndmm_fit, type = "hazard", mtype = "link"),
#'      # The hazard is infinite at t=0 in some studies, giving undefined logHRs at t=0
#'      na.rm = TRUE)
#'
#' # The NDMM vignette demonstrates the production of time-varying marginal
#' # hazard ratios from a ML-NMR model that includes covariates, see
#' # `vignette("example_ndmm")`
#'
#' # Marginal survival difference over time
#' plot(marginal_effects(ndmm_fit, type = "survival", mtype = "difference"))
#' }
#'
marginal_effects <- function(object,
                             ...,
                             mtype = c("difference", "ratio", "link"),
                             all_contrasts = FALSE, trt_ref = NULL,
                             probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                             predictive_distribution = FALSE,
                             summary = TRUE) {

  # Checks
  if (!inherits(object, "stan_nma")) abort("Expecting a `stan_nma` object, as returned by nma().")

  mtype <- rlang::arg_match(mtype)

  if (!rlang::is_bool(all_contrasts))
    abort("`all_contrasts` should be TRUE or FALSE.")

  if (!is.null(trt_ref)) {
    if (all_contrasts) {
      warn("Ignoring `trt_ref` when all_contrasts = TRUE.")
      trt_ref <- levels(object$network$treatments)[1]
    } else {
      if (length(trt_ref) > 1) abort("`trt_ref` must be length 1.")
      trt_ref <- as.character(trt_ref)
      lvls_trt <- levels(object$network$treatments)
      if (! trt_ref %in% lvls_trt)
        abort(sprintf("`trt_ref` does not match a treatment in the network.\nSuitable values are: %s",
                      ifelse(length(lvls_trt) <= 5,
                             paste0(lvls_trt, collapse = ", "),
                             paste0(paste0(lvls_trt[1:5], collapse = ", "), ", ..."))))
    }
  } else {
    trt_ref <- levels(object$network$treatments)[1]
  }

  if (!rlang::is_bool(summary))
    abort("`summary` should be TRUE or FALSE.")

  check_probs(probs)

  if(!rlang::is_bool(predictive_distribution))
    abort("`predictive_distribution` should be TRUE or FALSE")
  if (predictive_distribution && object$trt_effects != "random") predictive_distribution <- FALSE

  # Cannot produce marginal effects for inconsistency models
  if (object$consistency != "consistency")
    abort(glue::glue("Cannot produce marginal effects under inconsistency '{object$consistency}' model."))

  # Call predict to get absolute predictions
  if ("level" %in% ...names()) warn('Ignoring `level` argument, this is always "aggregate" for marginal effects.')

  if (object$likelihood %in% valid_lhood$survival) {
    pred <- rlang::eval_tidy(rlang::call2(
              stats::predict,
              !!! rlang::dots_list(object,
                                   level = "aggregate",
                                   summary = FALSE,
                                   predictive_distribution = predictive_distribution,
                                   !!! rlang::enquos(...),
                                   .homonyms = "first")))
  } else {
    if ("type" %in% ...names()) warn('Ignoring `type` argument, this is always "response" for marginal effects (except for survival models).')
    pred <- rlang::eval_tidy(rlang::call2(
              stats::predict,
              !!! rlang::dots_list(object,
                                   type = "response", level = "aggregate",
                                   summary = FALSE,
                                   predictive_distribution = predictive_distribution,
                                   !!! rlang::enquos(...),
                                   .homonyms = "first")))
  }

  # Set up output metadata
  pred_meta <- pred$summary
  vars <- intersect(c(".study", ".time", ".quantile", ".category"), colnames(pred_meta))

  if (".time" %in% vars) {
    pred_meta <- { if (".study" %in% vars) dplyr::group_by(pred_meta, .data$.study, .data$.trt) else dplyr::group_by(pred_meta, .data$.trt) } %>%
      dplyr::mutate(.time_id = 1:dplyr::n()) %>%
      dplyr::ungroup()

    vars <- c(vars, ".time_id")
  }

  out_meta <- pred_meta

  if (!all_contrasts) {
    out_meta <- dplyr::filter(out_meta, .data$.trt != trt_ref) %>%
      dplyr::mutate(.trtb = .data$.trt, .trta = factor(trt_ref, levels = levels(object$network$treatments))) %>%
      dplyr::select(dplyr::any_of(".study"), ".trtb", ".trta", dplyr::everything(), -".trt")
  } else {
    contrs <- utils::combn(object$network$treatments, 2)
    trtb <- contrs[2, ]
    trta <- contrs[1, ]
    out_meta <- dplyr::distinct(out_meta, dplyr::pick(dplyr::all_of(vars))) %>%
      dplyr::mutate(.trtb = list(trtb), .trta = list(trta))  %>%
      tidyr::unnest(cols = c(".trtb", ".trta")) %>%
      dplyr::select(dplyr::any_of(".study"), ".trtb", ".trta", dplyr::everything())
  }

  pred_meta$id <- 1:nrow(pred_meta)
  out_meta$id <- 1:nrow(out_meta)

  # Output parameter names
  pnames <- paste0("marg[",
                   if (rlang::has_name(out_meta, ".study")) paste0(out_meta$.study, ": ") else character(),
                   out_meta$.trtb,
                   if (all_contrasts) paste0(" vs. ", out_meta$.trta) else character(),
                   if (".time" %in% vars) paste0(", ", out_meta$.time_id)
                   else if (".quantile" %in% vars) paste0(", ", out_meta$.quantile)
                   else if (".category" %in% vars) paste0(", ", out_meta$.category)
                   else character()
                   , "]")

  # Set up output array
  d_out <- dim(pred$sims)
  d_out[[3]] <- nrow(out_meta)
  dn_out <- list(iterations = NULL, chains = NULL, parameters = pnames)
  out_array <- array(NA_real_, dim = d_out, dimnames = dn_out)

  # Set contrast function
  if (mtype == "difference") {
    cf <- function(b, a) b - a
  } else if (mtype == "ratio") {
    cf <- function(b, a) b / a
  } else if (mtype == "link") {
    cf <- function(b, a) link_fun(b, link = object$link) - link_fun(a, link = object$link)
  }

  cfv <- function(x) cf(x[2], x[1])
  mk_contr <- function(x) utils::combn(x, 2, FUN = cfv)

  # Calculate contrasts, looping over groups
  grps <- dplyr::distinct(out_meta, dplyr::pick(dplyr::all_of(vars)))

  for (i in 1:ifelse(length(vars), nrow(grps), 1)) {
    if (length(vars)) {
      grpi <- grps[i, ]
      predi <- dplyr::semi_join(pred_meta, grpi, by = vars)$id
      outi <- dplyr::semi_join(out_meta, grpi, by = vars)$id
    } else { # No grouping variables
      grpi <- tibble::tibble(.rows = 1)
      predi <- pred_meta$id
      outi <- out_meta$id
    }

    if (all_contrasts && nlevels(object$network$treatments) > 2) {
      out_array[ , , outi] <- aperm(apply(pred$sims[ , , predi], MARGIN = 1:2, FUN = mk_contr), c(2, 3, 1))
    } else {
      predi_ref <- dplyr::semi_join(pred_meta, dplyr::mutate(grpi, .trt = trt_ref), by = c(vars, ".trt"))$id
      predi_nonref <- setdiff(predi, predi_ref)
      out_array[ , , outi] <- sweep(pred$sims[ , , predi_nonref, drop = FALSE], MARGIN = 1:2, STATS = pred$sims[ , , predi_ref], FUN = cf)
    }
  }

  out_meta <- dplyr::select(out_meta, -"id")
  if (".time_id" %in% vars) out_meta <- dplyr::select(out_meta, -".time_id")

  # Summarise
  if (summary) {
    out_summary <- summary_mcmc_array(out_array, probs)
    out_summary <- dplyr::bind_cols(out_meta, out_summary)
  } else {
    out_summary <- out_meta
  }

  out <- list(summary = out_summary, sims = out_array)

  # Set attributes
  if (summary) {
    attr(out, "xlab") <- "Treatment"

    if (object$likelihood %in% valid_lhood$survival) {
      dots <- rlang::enquos(...)
      if (rlang::has_name(dots, "type")) type <- rlang::eval_tidy(dots$type)
      else type <- "survival"
    } else if (mtype == "link") {
      type <- "link"
    } else {
      type <- "response"
    }

    if (mtype == "difference") {
      attr(out, "ylab") <- paste0("Marginal ",
                                  get_scale_name(likelihood = object$likelihood,
                                                 link = object$link,
                                                 measure = "absolute",
                                                 type = type),
                                  " Difference")
    } else if (mtype == "ratio") {
      attr(out, "ylab") <- paste0("Marginal ",
                                  get_scale_name(likelihood = object$likelihood,
                                                 link = object$link,
                                                 measure = "absolute",
                                                 type = type),
                                  " Ratio")
    } else if (mtype == "link") {
      if (object$likelihood %in% valid_lhood$survival) {
        if (object$link == "log") {
          attr(out, "ylab") <- paste0("Marginal log ",
                                      get_scale_name(likelihood = object$likelihood,
                                                     link = object$link,
                                                     measure = "absolute",
                                                     type = type),
                                      " Ratio")
        } else {
          attr(out, "ylab") <- paste0("Marginal ", object$link, " ",
                                      get_scale_name(likelihood = object$likelihood,
                                                     link = object$link,
                                                     measure = "absolute",
                                                     type = type),
                                      " Difference ")
        }
      } else {
        attr(out, "ylab") <- paste0("Marginal ",
                                    get_scale_name(likelihood = object$likelihood,
                                                   link = object$link,
                                                   measure = "relative",
                                                   type = type))
      }
    }

    if (object$likelihood == "ordered") {
      class(out) <- c("ordered_nma_summary", "nma_summary")
    } else if (type %in% c("survival", "hazard", "cumhaz")) {
      class(out) <- c("surv_nma_summary", "nma_summary")
      attr(out, "xlab") <- "Time"
    } else {
      class(out) <- "nma_summary"
    }
  }
  return(out)
}

---
title: "Overview of Examples"
output: rmarkdown::html_vignette
link-citations: yes
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{*Overview of Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This package contains a number of vignettes, each one walking through an example analysis.
The table below gives an overview.

Many of these examples recreate analyses from the series of Technical Support Documents published by the NICE Decision Support Unit [@TSD_evsynth].
The exceptions are atrial fibrillation [@Cooper2009], white blood cell transfusion [@Turner2012], social anxiety [@Perren2025;@mayo2014psychological], and plaque psoriasis multilevel network meta-regression [@methods_paper;@Phillippo2022].


Title | Outcome type | Likelihood | Link function | Notable features
:-----|:-------------|:-----------|:--------------|:----------------
[Blocker](example_blocker.html) | Counts | Binomial | logit | Pairwise MA
[Dietary fat](example_dietary_fat.html) | Rates | Poisson | log | Analysis of log rate ratios from rate data
[Diabetes](example_diabetes.html) | Counts with time at risk | Binomial | cloglog | Analysis of log hazard ratios from count data with time at risk
[Parkinson's](example_parkinsons.html) | Continuous | Normal | Identity | Analysis of arm-based data, contrast-based data, and a mixture of both
[HTA plaque psoriasis](example_hta_psoriasis.html) | Ordered | Multinomial (ordered) | probit | Analysis of ordered categorical outcomes
[Statins](example_statins.html) | Counts | Binomial | logit | Meta-regression with subgroups
[BCG vaccine](example_bcg_vaccine.html) | Counts | Binomial | logit | Meta-regression with a continuous covariate, predictive distributions
[Smoking cessation](example_smoking.html) | Counts | Binomial | logit | Assessing inconsistency with unrelated mean effects and node-splitting models
[Thrombolytics](example_thrombolytics.html) | Counts | Binomial | logit | Assessing inconsistency with unrelated mean effects and node-splitting models
[Atrial fibrillation](example_atrial_fibrillation.html) | Counts | Binomial | logit | Meta-regression with shared class interactions
[WBC transfusion](example_transfusion.html) | Counts | Binomial | logit | Informative log-Normal prior on $\tau^2$
[ML-NMR plaque psoriasis](example_plaque_psoriasis.html) | Binary (IPD) and counts (AgD), and ordered categorical | Bernoulli (IPD) and two-parameter Binomial (AgD), and ordered multinomial | probit | Multilevel network meta-regression combining IPD and AgD
[ML-NMR newly diagnosed multiple myeloma](example_ndmm.html) | Time-to-event with censoring | M-spline baseline hazard | log | Multilevel network meta-regression combining IPD and AgD
[Social anxiety](example_social_anxiety.html) | Continuous | Normal | Identity | Model selection with class effects models
[Certolizumab](example_certolizumab.html) | Counts | Binomial | logit | Baseline risk meta-regression

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic.R
\name{dic}
\alias{dic}
\title{Deviance Information Criterion (DIC)}
\usage{
dic(x, penalty = c("pD", "pV"), ...)
}
\arguments{
\item{x}{A fitted model object, inheriting class \link{stan_nma}}

\item{penalty}{The method for estimating the effective number of parameters,
used to penalise model fit in the DIC. Either \code{"pD"} (the default), or
\code{"pV"}. For survival likelihoods only \code{"pV"} is currently available.}

\item{...}{Other arguments (not used)}
}
\value{
A \link{nma_dic} object.
}
\description{
Calculate the DIC for a model fitted using the \code{\link[=nma]{nma()}} function.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking FE NMA example if not already available
if (!exists("smk_fit_FE")) example("example_smk_fe", run.donttest = TRUE)
}
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Compare DIC of FE and RE models
(smk_dic_FE <- dic(smk_fit_FE))
(smk_dic_RE <- dic(smk_fit_RE))   # substantially better fit

# Plot residual deviance contributions under RE model
plot(smk_dic_RE)

# Check for inconsistency using UME model
}
\donttest{
# Run smoking UME NMA example if not already available
if (!exists("smk_fit_RE_UME")) example("example_smk_ume", run.donttest = TRUE)
}
\donttest{
# Compare DIC
smk_dic_RE
(smk_dic_RE_UME <- dic(smk_fit_RE_UME))  # no difference in fit

# Compare residual deviance contributions
plot(smk_dic_RE, smk_dic_RE_UME, show_uncertainty = FALSE)
}
}
\seealso{
\code{\link[=print.nma_dic]{print.nma_dic()}} for printing details, \code{\link[=plot.nma_dic]{plot.nma_dic()}} for
producing plots of residual deviance contributions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative_effects.R
\name{relative_effects}
\alias{relative_effects}
\title{Relative treatment effects}
\usage{
relative_effects(
  x,
  newdata = NULL,
  study = NULL,
  all_contrasts = FALSE,
  trt_ref = NULL,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
  predictive_distribution = FALSE,
  summary = TRUE
)
}
\arguments{
\item{x}{A \code{stan_nma} object created by \code{\link[=nma]{nma()}}}

\item{newdata}{Only used if a regression model is fitted. A data frame of
study details, one row per study, giving the covariate values at which to
produce relative effects. Column names must match variables in the
regression model. If \code{NULL}, relative effects are produced for all studies
in the network.}

\item{study}{Column of \code{newdata} which specifies study names, otherwise
studies will be labelled by row number.}

\item{all_contrasts}{Logical, generate estimates for all contrasts (\code{TRUE}),
or just the "basic" contrasts against the network reference treatment
(\code{FALSE})? Default \code{FALSE}.}

\item{trt_ref}{Reference treatment to construct relative effects against, if
\code{all_contrasts = FALSE}. By default, relative effects will be against the
network reference treatment. Coerced to character string.}

\item{probs}{Numeric vector of quantiles of interest to present in computed
summary, default \code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{predictive_distribution}{Logical, when a random effects model has been
fitted, should the predictive distribution for relative effects in a new
study be returned? Default \code{FALSE}.}

\item{summary}{Logical, calculate posterior summaries? Default \code{TRUE}.}
}
\value{
A \link{nma_summary} object if \code{summary = TRUE}, otherwise a list
containing a 3D MCMC array of samples and (for regression models) a data
frame of study information.
}
\description{
Generate (population-average) relative treatment effects. If a ML-NMR or
meta-regression model was fitted, these are specific to each study
population.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Produce relative effects
smk_releff_RE <- relative_effects(smk_fit_RE)
smk_releff_RE
plot(smk_releff_RE, ref_line = 0)

# Relative effects for all pairwise comparisons
relative_effects(smk_fit_RE, all_contrasts = TRUE)

# Relative effects against a different reference treatment
relative_effects(smk_fit_RE, trt_ref = "Self-help")

# Transforming to odds ratios
# We work with the array of relative effects samples
LOR_array <- as.array(smk_releff_RE)
OR_array <- exp(LOR_array)

# mcmc_array objects can be summarised to produce a nma_summary object
smk_OR_RE <- summary(OR_array)

# This can then be printed or plotted
smk_OR_RE
plot(smk_OR_RE, ref_line = 1)
}

## Plaque psoriasis ML-NMR
\donttest{
# Run plaque psoriasis ML-NMR example if not already available
if (!exists("pso_fit")) example("example_pso_mlnmr", run.donttest = TRUE)
}
\donttest{
# Produce population-adjusted relative effects for all study populations in
# the network
pso_releff <- relative_effects(pso_fit)
pso_releff
plot(pso_releff, ref_line = 0)

# Produce population-adjusted relative effects for a different target
# population
new_agd_means <- data.frame(
  bsa = 0.6,
  prevsys = 0.1,
  psa = 0.2,
  weight = 10,
  durnpso = 3)

relative_effects(pso_fit, newdata = new_agd_means)
}
}
\seealso{
\code{\link[=plot.nma_summary]{plot.nma_summary()}} for plotting the relative effects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{priors}
\alias{priors}
\alias{normal}
\alias{half_normal}
\alias{log_normal}
\alias{cauchy}
\alias{half_cauchy}
\alias{student_t}
\alias{half_student_t}
\alias{log_student_t}
\alias{exponential}
\alias{flat}
\title{Prior distributions}
\usage{
normal(location = 0, scale)

half_normal(scale)

log_normal(location, scale)

cauchy(location = 0, scale)

half_cauchy(scale)

student_t(location = 0, scale, df)

half_student_t(scale, df)

log_student_t(location, scale, df)

exponential(scale = 1/rate, rate = 1/scale)

flat()
}
\arguments{
\item{location}{Prior location. Typically prior mean (see details).}

\item{scale}{Prior scale. Typically prior standard deviation (see details).}

\item{df}{Prior degrees of freedom.}

\item{rate}{Prior rate.}
}
\value{
Object of class \link{nma_prior}.
}
\description{
These functions are used to specify prior distributions for the model parameters.
}
\details{
The \code{location} and \code{scale} parameters are typically the prior mean
and standard deviation, with the following exceptions:
\itemize{
\item For the Cauchy distribution \code{location} is the prior median and
\code{scale} is the prior scale.
\item For the log-Normal distribution, \code{location} and \code{scale} are the prior
mean and standard deviation of the logarithm.
}
\subsection{Compatibility with model parameters}{

The following table summarises which prior distributions may be used with
which model parameters. Essentially, priors that take only non-negative
values (e.g. half-Normal) may only be used for non-negative parameters
(heterogeneity SD/variance/precision, and any auxiliary parameter). If a
real-valued prior distribution is specified for a non-negative parameter, it
will be truncated at 0 to be non-negative.\tabular{lccccc}{
    \tab \strong{Intercept} \code{prior_intercept} \tab \strong{Treatment effects} \code{prior_trt} \tab \strong{Heterogeneity} \code{prior_het} \tab \strong{Regression coefficients} \code{prior_reg} \tab \strong{Auxiliary parameter} \code{prior_aux} \cr
   \strong{Normal} \code{normal()} \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
   \strong{half-Normal} \code{half_normal()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{log-Normal} \code{log_normal()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{Cauchy }\code{cauchy()} \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
   \strong{half-Cauchy} \code{half_cauchy()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{Student t} \code{student_t()} \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
   \strong{half-Student t} \code{half_student_t()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{log-Student t} \code{log_student_t()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{Exponential} \code{exponential()} \tab - \tab - \tab Yes \tab - \tab Yes \cr
   \strong{Flat} \code{flat()} \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
}


The \code{flat()} prior is a special case where no prior information is added to
the model, resulting in an implicit flat uniform prior distribution over the
entire support for a parameter. This will be an improper prior if the
parameter is unbounded, and is not generally advised. See the
\href{https://mc-stan.org/docs/stan-users-guide/some-differences-in-the-statistical-models-that-are-allowed.html}{Stan user's guide}
for more details.
}
}
\seealso{
\code{\link[=summary.nma_prior]{summary.nma_prior()}} for summarising details of prior
distributions. \code{\link[=plot_prior_posterior]{plot_prior_posterior()}} for plots comparing the prior and
posterior distributions of model parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{set_ipd}
\alias{set_ipd}
\title{Set up individual patient data}
\usage{
set_ipd(
  data,
  study,
  trt,
  y = NULL,
  r = NULL,
  E = NULL,
  Surv = NULL,
  trt_ref = NULL,
  trt_class = NULL
)
}
\arguments{
\item{data}{a data frame}

\item{study}{column of \code{data} specifying the studies, coded using integers,
strings, or factors}

\item{trt}{column of \code{data} specifying treatments, coded using integers,
strings, or factors}

\item{y}{column of \code{data} specifying a continuous outcome}

\item{r}{column of \code{data} specifying a binary outcome or Poisson outcome count}

\item{E}{column of \code{data} specifying the total time at risk for Poisson
outcomes}

\item{Surv}{column of \code{data} specifying a survival or time-to-event outcome,
using the \code{\link[=Surv]{Surv()}} function. Right/left/interval censoring and left
truncation (delayed entry) are supported.}

\item{trt_ref}{reference treatment for the network, as a single integer,
string, or factor. If not specified, a reasonable well-connected default
will be chosen (see details).}

\item{trt_class}{column of \code{data} specifying treatment classes, coded using
integers, strings, or factors. By default, no classes are specified.}
}
\value{
An object of class \link{nma_data}
}
\description{
Set up a network containing individual patient data (IPD). Multiple data
sources may be combined once created using \code{\link[=combine_network]{combine_network()}}.
}
\details{
By default, \code{trt_ref = NULL} and a network reference treatment will be chosen
that attempts to maximise computational efficiency and stability. If an
alternative reference treatment is chosen and the model runs slowly or has
low effective sample size (ESS) this may be the cause - try letting the
default reference treatment be used instead. Regardless of which treatment is
used as the network reference at the model fitting stage, results can be
transformed afterwards: see the \code{trt_ref} argument of
\code{\link[=relative_effects]{relative_effects()}} and \code{\link[=predict.stan_nma]{predict.stan_nma()}}.

All arguments specifying columns of \code{data} accept the following:
\itemize{
\item A column name as a character string, e.g. \code{study = "studyc"}
\item A bare column name, e.g. \code{study = studyc}
\item \code{dplyr::mutate()} style semantics for inline variable transformations, e.g. \code{study = paste(author, year)}
}
}
\examples{
# Set up network of plaque psoriasis IPD
head(plaque_psoriasis_ipd)

pso_net <- set_ipd(plaque_psoriasis_ipd,
                   study = studyc,
                   trt = trtc,
                   r = pasi75)

# Print network details
pso_net

# Plot network
plot(pso_net)

# Setting a different reference treatment
set_ipd(plaque_psoriasis_ipd,
        study = studyc,
        trt = trtc,
        r = pasi75,
        trt_ref = "PBO")
}
\seealso{
\code{\link[=set_agd_arm]{set_agd_arm()}} for arm-based aggregate data, \code{\link[=set_agd_contrast]{set_agd_contrast()}}
for contrast-based aggregate data, and \code{\link[=combine_network]{combine_network()}} for combining
several data sources in one network.

\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{as.stanfit}
\alias{as.stanfit}
\alias{as.stanfit.stan_nma}
\alias{as.stanfit.default}
\title{as.stanfit}
\usage{
as.stanfit(x, ...)

\method{as.stanfit}{stan_nma}(x, ...)

\method{as.stanfit}{default}(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{additional arguments}
}
\value{
A \code{\link[rstan:stanfit-class]{stanfit}} object.
}
\description{
Attempt to turn an object into a \code{\link[rstan:stanfit-class]{stanfit}} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mspline.R
\name{make_knots}
\alias{make_knots}
\title{Knot locations for M-spline baseline hazard models}
\usage{
make_knots(
  network,
  n_knots = 7,
  type = c("quantile", "quantile_common", "quantile_lumped", "quantile_longest", "equal",
    "equal_common")
)
}
\arguments{
\item{network}{A network object, containing survival outcomes}

\item{n_knots}{Non-negative integer giving the number of internal knots
(default \code{7})}

\item{type}{String specifying the knot location algorithm to use (see
details). The default used by \code{\link[=nma]{nma()}} is \code{"quantile"}, except when a
regression model is specified (using \code{aux_regression}) in which case the
default is \code{"quantile_common"}.}
}
\value{
A named list of vectors giving the knot locations in each study.
}
\description{
Several different algorithms are provided to calculate knot locations for
M-spline baseline hazard models. This function is called internally within
the \code{\link[=nma]{nma()}} function, but may be called directly by the user for more
control.
}
\details{
The \code{type} argument can be used to choose between different
algorithms for placing the knots:

\describe{
\item{\code{"quantile"}}{Creates separate knot locations for each study,
internal knots are placed at evenly-spaced quantiles of the observed event
times within each study.}
\item{\code{"quantile_lumped"}}{Creates a common set of knots for all studies,
calculated as evenly-spaced quantiles of the observed event times from all
studies lumped together.}
\item{\code{"quantile_common"}}{Creates a common set of knots for all studies,
taking quantiles of the quantiles of the observed event times within each
study. This often seems to result in a more even knot spacing than
\code{"quantile_lumped"}, particularly when follow-up is uneven across studies,
and may handle differing behaviour in the baseline hazard across studies
better than \code{"quantile_longest"}.}
\item{\code{"quantile_longest"}}{Creates a common set of knots for all studies,
using evenly-spaced quantiles of the observed event times in the longest
study.}
\item{\code{"equal"}}{Creates separate knot locations for each study, at
evenly-spaced times between the boundary knots in each study.}
\item{\code{"equal_common"}}{Creates a common set of knots for all studies, at
evenly-spaced times between the earliest entry time and last
event/censoring time in the network.}
}

Boundary knots are calculated as follows:
\itemize{
\item For separate knot locations in each study, boundary knots are placed at the
earliest entry time and last event/censoring time in each study.
\item For a common set of knots across all studies, boundary knots are placed at
the earliest entry time and last event/censoring time across all studies.
}

Models with regression on the spline coefficients (i.e. with \code{aux_regression}
specified) require a common set of knots across all studies.

Provided that a sufficient number of knots are used, model fit should be
largely unaffected by the knot locations. However, sampling difficulties can
sometimes occur if knot placement is poor, for example if a knot is placed
just before the last follow-up time in a study.
}
\examples{
# Set up newly-diagnosed multiple myeloma network

head(ndmm_ipd)
head(ndmm_agd)

ndmm_net <- combine_network(
  set_ipd(ndmm_ipd,
          study, trt,
          Surv = Surv(eventtime / 12, status)),
  set_agd_surv(ndmm_agd,
               study, trt,
               Surv = Surv(eventtime / 12, status),
               covariates = ndmm_agd_covs))

# The default knot locations
make_knots(ndmm_net, type = "quantile")

# Increasing the number of knots
make_knots(ndmm_net, n_knots = 10)

# Comparing alternative knot positioning algorithms
# Visualise these with a quick function
plot_knots <- function(network, knots) {
  ggplot2::ggplot() +
    geom_km(network) +
    ggplot2::geom_vline(ggplot2::aes(xintercept = .data$knot),
                        data = tidyr::pivot_longer(as.data.frame(knots), cols = dplyr::everything(),
                                                   names_to = "Study", values_to = "knot"),
                        linetype = 2, colour = "grey60") +
    ggplot2::facet_wrap(~Study) +
    theme_multinma()
}

plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile"))
plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_common"))
plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_lumped"))
plot_knots(ndmm_net, make_knots(ndmm_net, type = "quantile_longest"))
plot_knots(ndmm_net, make_knots(ndmm_net, type = "equal"))
plot_knots(ndmm_net, make_knots(ndmm_net, type = "equal_common"))

}
\seealso{
\code{\link[=knots.stan_nma]{knots.stan_nma()}} for obtaining the knots from a fitted model
object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{pairs.stan_nma}
\alias{pairs.stan_nma}
\title{Matrix of plots for a \code{stan_nma} object}
\usage{
\method{pairs}{stan_nma}(x, ..., pars, include = TRUE)
}
\arguments{
\item{x}{An object of class \code{stan_nma}}

\item{...}{Other arguments passed to
\code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}}}

\item{pars}{Optional character vector of parameter names to include in
output. If not specified, all parameters are used.}

\item{include}{Logical, are parameters in \code{pars} to be included (\code{TRUE},
default) or excluded (\code{FALSE})?}
}
\value{
A grid of ggplot objects produced by
\code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}}.
}
\description{
A \code{\link[=pairs]{pairs()}} method for \code{stan_nma} objects, which calls
\code{\link[bayesplot:MCMC-scatterplots]{bayesplot::mcmc_pairs()}} on the
underlying \code{stanfit} object.
}
\examples{
\dontrun{
## Parkinson's mean off time reduction
park_net <- set_agd_arm(parkinsons,
                        study = studyn,
                        trt = trtn,
                        y = y,
                        se = se,
                        sample_size = n)

# Fitting a RE model
park_fit_RE <- nma(park_net,
                   trt_effects = "random",
                   prior_intercept = normal(scale = 100),
                   prior_trt = normal(scale = 100),
                   prior_het = half_normal(scale = 5))

# We see a small number of divergent transition errors
# These do not go away entirely when adapt_delta is increased

# Try to diagnose with a pairs plot
pairs(park_fit_RE, pars = c("mu[4]", "d[3]", "delta[4: 3]", "tau"))

# Transforming tau onto log scale
pairs(park_fit_RE, pars = c("mu[4]", "d[3]", "delta[4: 3]", "tau"),
      transformations = list(tau = "log"))

# The divergent transitions occur in the upper tail of the heterogeneity
# standard deviation. In this case, with only a small number of studies, there
# is not very much information to estimate the heterogeneity standard deviation
# and the prior distribution may be too heavy-tailed. We could consider a more
# informative prior distribution for the heterogeneity variance to aid
# estimation.
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_summary-class.R
\name{plot.nma_summary}
\alias{plot.nma_summary}
\alias{plot.nma_parameter_summary}
\alias{plot.nma_rank_probs}
\alias{plot.surv_nma_summary}
\title{Plots of summary results}
\usage{
\method{plot}{nma_summary}(
  x,
  ...,
  stat = "pointinterval",
  orientation = c("horizontal", "vertical", "y", "x"),
  ref_line = NA_real_
)

\method{plot}{nma_parameter_summary}(
  x,
  ...,
  stat = "pointinterval",
  orientation = c("horizontal", "vertical", "y", "x"),
  ref_line = NA_real_
)

\method{plot}{nma_rank_probs}(x, ...)

\method{plot}{surv_nma_summary}(x, ..., stat = "lineribbon")
}
\arguments{
\item{x}{A \code{nma_summary} object}

\item{...}{Additional arguments passed on to the underlying \code{ggdist} plot
stat, see Details}

\item{stat}{Character string specifying the \code{ggdist} plot stat to use,
default \code{"pointinterval"}, except when plotting estimated
survival/hazard/cumulative hazard curves from survival models where the
default is \code{"lineribbon"}}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}), default \code{"horizontal"}}

\item{ref_line}{Numeric vector of positions for reference lines, by default
no reference lines are drawn}
}
\value{
A \code{ggplot} object.
}
\description{
The \code{plot} method for \code{nma_summary} objects is used to produce plots of
parameter estimates (when called on a \code{stan_nma} object or its summary),
relative effects (when called on the output of \code{\link[=relative_effects]{relative_effects()}}),
absolute predictions (when called on the output of \code{\link[=predict.stan_nma]{predict.stan_nma()}}),
posterior ranks and rank probabilities (when called on the output of
\code{\link[=posterior_ranks]{posterior_ranks()}} or \code{\link[=posterior_rank_probs]{posterior_rank_probs()}}).
}
\details{
Plotting is handled by \link[ggplot2:ggplot2-package]{ggplot2} and
the stats and geoms provided in the \link[ggdist:ggdist-package]{ggdist}
package. As a result, the output is very flexible. Any plotting stats
provided by \code{ggdist} may be used, via the argument \code{stat}.

The default uses
\code{\link[ggdist:stat_pointinterval]{ggdist::stat_pointinterval()}}, to
produce medians and 95\% Credible Intervals with 66\% inner bands. Additional
arguments in \code{...} are passed to the \code{ggdist} stat, to customise the
output. For example, to produce means and Credible Intervals, specify
\code{point_interval = "mean_qi"}. To produce an 80\% Credible Interval with no
inner band, specify \code{.width = c(0, 0.8)}.

Alternative stats can be specified to produce different summaries. For
example, specify \code{stat = "[half]eye"} to produce (half) eye plots, or \code{stat = "histinterval"} to produce histograms with intervals.

A full list of options and examples is found in the \code{ggdist} vignette
\code{vignette("slabinterval", package = "ggdist")}.

For survival/hazard/cumulative hazard curves estimated from survival
models, the default uses
\code{\link[ggdist:stat_lineribbon]{ggdist::stat_lineribbon()}} which
produces curves of posterior medians with 50\%, 80\%, and 95\% Credible
Interval bands. Again, additional arguments in \code{...} are passed to the
\code{ggdist} stat. For example, to produce posterior means and 95\% Credible
bands, specify \code{point_interval = "mean_qi"} and \code{.width = 0.95}.

A \code{ggplot} object is returned which can be further modified through the
usual \link[ggplot2:ggplot2-package]{ggplot2} functions to add further
aesthetics, geoms, themes, etc.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Produce relative effects
smk_releff_RE <- relative_effects(smk_fit_RE)
plot(smk_releff_RE, ref_line = 0)

# Customise plot options
plot(smk_releff_RE, ref_line = 0, stat = "halfeye")

# Further customisation is possible with ggplot commands
plot(smk_releff_RE, ref_line = 0, stat = "halfeye", slab_alpha = 0.6) +
  ggplot2::aes(slab_fill = ggplot2::after_stat(ifelse(x < 0, "darkred", "grey60")))

# Produce posterior ranks
smk_rank_RE <- posterior_ranks(smk_fit_RE, lower_better = FALSE)
plot(smk_rank_RE)

# Produce rank probabilities
smk_rankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE)
plot(smk_rankprob_RE)

# Produce cumulative rank probabilities
smk_cumrankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE,
                                           cumulative = TRUE)
plot(smk_cumrankprob_RE)

# Further customisation is possible with ggplot commands
plot(smk_cumrankprob_RE) +
  ggplot2::facet_null() +
  ggplot2::aes(colour = Treatment)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodesplit_summary-class.R
\name{print.nodesplit_summary}
\alias{print.nodesplit_summary}
\alias{as_tibble.nodesplit_summary}
\alias{as.tibble.nodesplit_summary}
\alias{as.data.frame.nodesplit_summary}
\title{Methods for \code{nodesplit_summary} objects}
\usage{
\method{print}{nodesplit_summary}(x, ..., digits = 2)

\method{as_tibble}{nodesplit_summary}(x, ..., nest = FALSE)

as.tibble.nodesplit_summary(x, ..., nest = FALSE)

\method{as.data.frame}{nodesplit_summary}(x, ...)
}
\arguments{
\item{x}{A \code{nodesplit_summary} object}

\item{...}{Additional arguments passed on to other methods}

\item{digits}{Integer number of digits to display}

\item{nest}{Whether to return a nested tibble, with the full \link{nma_summary}
and \link{nma_dic} objects, or to unnest their summaries, default \code{FALSE}}
}
\value{
A \code{data.frame} for \code{as.data.frame()}, a \code{tbl_df} for \code{as.tibble()}
and \code{as_tibble()}.

The \code{print()} method returns \code{x} invisibly.
}
\description{
The \code{as.data.frame()}, \code{as_tibble()}, and \code{as.tibble()} methods return the
node-splitting summaries in a data frame or tibble.
}
\seealso{
\code{\link[=plot.nodesplit_summary]{plot.nodesplit_summary()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{diabetes}
\alias{diabetes}
\title{Incidence of diabetes in trials of antihypertensive drugs}
\format{
A data frame with 48 rows and 7 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
\item{time}{trial follow-up (years)}
}
}
\usage{
diabetes
}
\description{
Data frame containing the number of new cases of diabetes in 22 trials of 6
antihypertensive drugs \insertCite{Elliott2007,TSD2}{multinma}. The trial
duration (in years) is also recorded.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{set_agd_contrast}
\alias{set_agd_contrast}
\title{Set up contrast-based aggregate data}
\usage{
set_agd_contrast(
  data,
  study,
  trt,
  y = NULL,
  se = NULL,
  sample_size = NULL,
  trt_ref = NULL,
  trt_class = NULL
)
}
\arguments{
\item{data}{a data frame}

\item{study}{column of \code{data} specifying the studies, coded using integers,
strings, or factors}

\item{trt}{column of \code{data} specifying treatments, coded using integers,
strings, or factors}

\item{y}{column of \code{data} specifying a continuous outcome}

\item{se}{column of \code{data} specifying the standard error for a continuous
outcome}

\item{sample_size}{column of \code{data} giving the sample size in each arm.
Optional, see details.}

\item{trt_ref}{reference treatment for the network, as a single integer,
string, or factor. If not specified, a reasonable well-connected default
will be chosen (see details).}

\item{trt_class}{column of \code{data} specifying treatment classes, coded using
integers, strings, or factors. By default, no classes are specified.}
}
\value{
An object of class \link{nma_data}
}
\description{
Set up a network containing contrast-based aggregate data (AgD), i.e.
summaries of relative effects between treatments such as log Odds Ratios.
Multiple data sources may be combined once created using \code{\link[=combine_network]{combine_network()}}.
}
\details{
Each study should have a single reference/baseline treatment,
against which relative effects in the other arm(s) are given. For the
reference arm, include a data row with continuous outcome \code{y} equal to
\code{NA}. If a study has three or more arms (so two or more relative effects),
set the standard error \code{se} for the reference arm data row equal to the
standard error of the mean outcome on the reference arm (this determines
the covariance of the relative effects, when expressed as differences in
mean outcomes between arms).

All arguments specifying columns of \code{data} accept the following:
\itemize{
\item A column name as a character string, e.g. \code{study = "studyc"}
\item A bare column name, e.g. \code{study = studyc}
\item \code{dplyr::mutate()} style semantics for inline variable transformations, e.g. \code{study = paste(author, year)}
}

By default, \code{trt_ref = NULL} and a network reference treatment will be chosen
that attempts to maximise computational efficiency and stability. If an
alternative reference treatment is chosen and the model runs slowly or has
low effective sample size (ESS) this may be the cause - try letting the
default reference treatment be used instead. Regardless of which treatment is
used as the network reference at the model fitting stage, results can be
transformed afterwards: see the \code{trt_ref} argument of
\code{\link[=relative_effects]{relative_effects()}} and \code{\link[=predict.stan_nma]{predict.stan_nma()}}.

The \code{sample_size} argument is optional, but when specified:
\itemize{
\item Enables automatic centering of predictors (\code{center = TRUE}) in \code{\link[=nma]{nma()}}
when a regression model is given for a network combining IPD and AgD
\item Enables production of study-specific relative effects, rank probabilities,
etc. for studies in the network when a regression model is given
\item Nodes in \code{\link[=plot.nma_data]{plot.nma_data()}} may be weighted by sample size
}
}
\examples{
# Set up network of Parkinson's contrast data
head(parkinsons)

park_net <- set_agd_contrast(parkinsons,
                             study = studyn,
                             trt = trtn,
                             y = diff,
                             se = se_diff,
                             sample_size = n)

# Print details
park_net

# Plot network
plot(park_net)
}
\seealso{
\code{\link[=set_ipd]{set_ipd()}} for individual patient data, \code{\link[=set_agd_arm]{set_agd_arm()}} for
arm-based aggregate data, and \code{\link[=combine_network]{combine_network()}} for combining several
data sources in one network.

\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{qgamma}
\alias{qgamma}
\alias{dgamma}
\alias{pgamma}
\title{The Gamma distribution}
\usage{
qgamma(
  p,
  shape,
  rate = 1,
  scale = 1/rate,
  lower.tail = TRUE,
  log.p = FALSE,
  mean,
  sd
)

dgamma(x, shape, rate = 1, scale = 1/rate, log = FALSE, mean, sd)

pgamma(
  q,
  shape,
  rate = 1,
  scale = 1/rate,
  lower.tail = TRUE,
  log.p = FALSE,
  mean,
  sd
)
}
\arguments{
\item{p}{vector of probabilities}

\item{shape, rate, scale, log, lower.tail, log.p}{see \link[stats:GammaDist]{stats::GammaDist}}

\item{mean, sd}{mean and standard deviation, overriding \code{shape} and
\code{rate} or \code{scale} if specified}

\item{x, q}{vector of quantiles}
}
\value{
Numeric vector of length equal to the maximum of the lengths of the
input arguments.
}
\description{
We provide convenient extensions of the \verb{[dpq]gamma} functions, which allow
the distribution to be specified in terms of its mean and standard deviation,
instead of shape and rate/scale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{nma_data-class}
\alias{nma_data-class}
\alias{nma_data}
\alias{mlnmr_data}
\alias{mlnmr_data-class}
\title{The nma_data class}
\description{
The \code{nma_data} class contains the data for a NMA in a standard format,
created using the functions \code{\link[=set_ipd]{set_ipd()}}, \code{\link[=set_agd_arm]{set_agd_arm()}},
\code{\link[=set_agd_contrast]{set_agd_contrast()}}, \code{\link[=set_agd_surv]{set_agd_surv()}}, or \code{\link[=combine_network]{combine_network()}}. The sub-class
\code{mlnmr_data} is created by the function \code{\link[=add_integration]{add_integration()}}, and further
contains numerical integration points for the aggregate data.
}
\details{
Objects of class \code{nma_data} have the following components:
\describe{
\item{\code{agd_arm}}{data from studies with aggregate data (arm format)}
\item{\code{agd_contrast}}{data from studies with aggregate data (contrast
format)}
\item{\code{ipd}}{data from studies with individual patient data}
\item{\code{treatments}}{treatment coding factor for entire network}
\item{\code{classes}}{treatment class coding factor (same length as \code{treatments}
for entire network)}
\item{\code{studies}}{study coding factor for entire network}
\item{\code{outcome}}{outcome type for each data source, named list}
}

The \code{agd_arm}, \code{agd_contrast}, and \code{ipd} components are
tibbles with the following columns:
\describe{
\item{\code{.study}}{study (as factor)}
\item{\code{.trt}}{treatment (as factor)}
\item{\code{.trtclass}}{treatment class (as factor), if specified}
\item{\code{.y}}{continuous outcome}
\item{\code{.se}}{standard error (continuous)}
\item{\code{.r}}{event count (discrete)}
\item{\code{.n}}{event count denominator (discrete, \code{agd_arm} only)}
\item{\code{.E}}{time at risk (discrete)}
\item{\code{.Surv}}{survival outcome of type \code{\link{Surv}} (time-to-event), nested by
study arm}
\item{\code{.sample_size}}{sample size (\verb{agd_*} only)}
\item{\code{...}}{other columns (typically covariates) from the original data
frame}
}

Objects of class \code{mlnmr_data} additionally have components:
\describe{
\item{\code{n_int}}{number of numerical integration points}
\item{\code{int_names}}{names of covariates with numerical integration points}
\item{\code{int_cor}}{correlation matrix for covariates used to generate
numerical integration points}
}

The \code{agd_arm} and \code{agd_contrast} tibbles have additional list columns with
prefix \code{.int_}, one for each covariate, which contain the numerical
integration points nested as length-\code{n_int} vectors within each row.
}
\seealso{
\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma.R
\name{RE_cor}
\alias{RE_cor}
\alias{which_RE}
\title{Random effects structure}
\usage{
RE_cor(study, trt, contrast, type = c("reftrt", "blshift"))

which_RE(study, trt, contrast, type = c("reftrt", "blshift"))
}
\arguments{
\item{study}{A vector of study IDs (integer, character, or factor)}

\item{trt}{A factor vector of treatment codes (or coercible as such), with
first level indicating the reference treatment}

\item{contrast}{A logical vector, of the same length as \code{study} and \code{trt},
indicating whether the corresponding data are in contrast rather than arm
format.}

\item{type}{Character string, whether to generate RE structure under the
"reference treatment" parameterisation, or the "baseline shift"
parameterisation.}
}
\value{
For \code{RE_cor()}, a correlation matrix of dimension equal to the number
of random effects deltas (excluding those that are set equal to zero).

For \code{which_RE()}, an integer vector of IDs indexing the rows and columns of
the correlation matrix returned by \code{RE_cor()}.
}
\description{
Use \code{RE_cor} to generate the random effects correlation matrix, under the
assumption of common heterogeneity variance (i.e. all within-study
correlations are 0.5). Use \code{which_RE} to return a vector of IDs for the RE
deltas (0 means no RE delta on this arm).
}
\examples{
RE_cor(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)))
RE_cor(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)), type = "blshift")
which_RE(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)))
which_RE(smoking$studyn, smoking$trtn, contrast = rep(FALSE, nrow(smoking)), type = "blshift")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{as.array.stan_nma}
\alias{as.array.stan_nma}
\alias{as.data.frame.stan_nma}
\alias{as_tibble.stan_nma}
\alias{as.tibble.stan_nma}
\alias{as.matrix.stan_nma}
\title{Convert samples into arrays, matrices, or data frames}
\usage{
\method{as.array}{stan_nma}(x, ..., pars, include = TRUE)

\method{as.data.frame}{stan_nma}(x, ..., pars, include = TRUE)

\method{as_tibble}{stan_nma}(x, ..., pars, include = TRUE)

\method{as.tibble}{stan_nma}(x, ..., pars, include = TRUE)

\method{as.matrix}{stan_nma}(x, ..., pars, include = TRUE)
}
\arguments{
\item{x}{A \code{stan_nma} object}

\item{...}{Additional arguments passed to \code{\link[rstan:as.array.stanfit]{as.array.stanfit()}}}

\item{pars}{Optional character vector of parameter names to include in output. If not specified, all parameters are used.}

\item{include}{Logical, are parameters in \code{pars} to be included (\code{TRUE}, default) or excluded (\code{FALSE})?}
}
\value{
The \code{as.array()} method produces a 3D array [Iteration, Chain,
Parameter] containing posterior samples of each parameter (as class
\link{mcmc_array}). This has the side effect of enabling
\code{\link[bayesplot:bayesplot-package]{bayesplot}} functions to
seamlessly work on \code{stan_nma} objects.

The \code{as.data.frame()} method produces a data frame containing posterior
samples of each parameter, combined over all chains.

The \code{as.matrix()} method produces a matrix containing posterior samples of
each parameter, combined over all chains.
}
\description{
Samples (post warm-up) from a \code{stan_nma} model object can be coerced into an
array, matrix, or data frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{multi}
\alias{multi}
\title{Multinomial outcome data}
\usage{
multi(..., inclusive = FALSE, type = c("ordered", "competing"))
}
\arguments{
\item{...}{Two or more numeric columns (or vectors) of category counts.
Argument names (optional) will be used to label the categories.}

\item{inclusive}{Logical, are ordered category counts inclusive (\code{TRUE}) or
exclusive (\code{FALSE})? Default \code{FALSE}. Only used when \code{type = "ordered"}.
See details.}

\item{type}{String, indicating whether categories are \code{"ordered"} or
\code{"competing"}. Currently only ordered categorical outcomes are supported by
the modelling functions in this package.}
}
\value{
A matrix of (exclusive) category counts
}
\description{
This function aids the specification of multinomial outcome data when setting
up a network with \code{\link[=set_agd_arm]{set_agd_arm()}} or \code{\link[=set_ipd]{set_ipd()}}. It takes a set of columns
(or, more generally, numeric vectors of the same length) of outcome counts in
each category, and binds these together to produce a matrix.
}
\details{
When specifying ordered categorical counts, these can either be
given as \emph{exclusive} counts (\code{inclusive = FALSE}, the default) where
individuals are only counted in the highest category they achieve, or
\emph{inclusive} counts (\code{inclusive = TRUE}) where individuals are counted in
every category up to and including the highest category achieved.
(Competing outcomes, by nature, are always specified as exclusive counts.)

\code{NA} values can be used to indicate categories/cutpoints that were not
measured.
}
\examples{
# These two data sets specify the same ordered categorical data for outcomes
# r0 < r1 < r2, but the first uses the "inclusive" format and the second the
# "exclusive" format.
df_inclusive <- tibble::tribble(~r0, ~r1, ~r2,
                                1, 1, 1,
                                5, 4, 1,
                                5, 2, 2,
                                10, 5, 0,
                                5, 5, 0,
                                7, NA, 6,   # Achieved r2 or not (no r1)
                                10, 4, NA)  # Achieved r1 or not (no r2)

df_exclusive <- tibble::tribble(~r0, ~r1, ~r2,
                                0, 0, 1,
                                1, 3, 1,
                                3, 0, 2,
                                5, 5, 0,
                                0, 5, 0,
                                1, NA, 6,   # Achieved r2 or not (no r1)
                                6, 4, NA)   # Achieved r1 or not (no r2)

(r_inclusive <- with(df_inclusive, multi(r0, r1, r2, inclusive = TRUE)))
(r_exclusive <- with(df_exclusive, multi(r0, r1, r2, inclusive = FALSE)))

# Counts are always stored in exclusive format
stopifnot(isTRUE(all.equal(r_inclusive, r_exclusive)))


## HTA Plaque Psoriasis
library(dplyr)

# Ordered outcomes here are given as "exclusive" counts
head(hta_psoriasis)

# Calculate lowest category count (failure to achieve PASI 50)
pso_dat <- hta_psoriasis \%>\%
  mutate(`PASI<50` = sample_size - rowSums(cbind(PASI50, PASI75, PASI90), na.rm = TRUE))

# Set up network
pso_net <- set_agd_arm(pso_dat,
                       study = paste(studyc, year),
                       trt = trtc,
                       r = multi(`PASI<50`, PASI50, PASI75, PASI90,
                                 inclusive = FALSE,
                                 type = "ordered"))

pso_net

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{thrombolytics}
\alias{thrombolytics}
\title{Thrombolytic treatments data}
\format{
A data frame with 102 rows and 5 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
}
}
\usage{
thrombolytics
}
\description{
Data frame containing the results of 50 trials of 8 thrombolytic drugs
(streptokinase, SK; alteplase, t-PA; accelerated alteplase, Acc t-PA;
streptokinase plus alteplase, SK+tPA; reteplase, r-PA; tenocteplase, TNK;
urokinase, UK; anistreptilase, ASPAC) plus per-cutaneous transluminal
coronary angioplasty (PTCA)
\insertCite{Boland2003,Lu2006,TSD4}{multinma}. The number of
deaths in 30 or 35 days following acute myocardial infarction are recorded.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{smoking}
\alias{smoking}
\title{Smoking cessation data}
\format{
A data frame with 50 rows and 5 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
}
}
\usage{
smoking
}
\description{
Data frame containing the results of 24 trials of 4 smoking cessation
treatments \insertCite{Hasselblad1998,TSD4}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{dlogt}
\alias{dlogt}
\alias{plogt}
\alias{qlogt}
\title{Log Student's t distribution}
\usage{
dlogt(x, df, location = 0, scale = 1)

plogt(q, df, location = 0, scale = 1)

qlogt(p, df, location = 0, scale = 1)
}
\arguments{
\item{x, q}{Vector of quantiles}

\item{df}{Degrees of freedom, greater than zero}

\item{location}{Location parameter}

\item{scale}{Scale parameter, greater than zero}

\item{p}{Vector of probabilities}
}
\value{
\code{dlogt()} gives the density, \code{plogt()} gives the distribution
function, \code{qlogt()} gives the quantile function.
}
\description{
Density, distribution, and quantile function for the log t distribution,
whose logarithm has degrees of freedom \code{df}, mean \code{location}, and standard
deviation \code{scale}.
}
\details{
If \eqn{\log(Y) \sim t_\nu(\mu, \sigma^2)}, then \eqn{Y} has a log t
distribution with \code{location} \eqn{\mu}, \code{scale} \eqn{\sigma}, and \code{df}
\eqn{\nu}.

The mean and all higher moments of the log t distribution are undefined or
infinite.

If \code{df = 1} then the distribution is a log Cauchy distribution. As \code{df}
tends to infinity, this approaches a log Normal distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodesplit_summary-class.R
\name{nodesplit_summary-class}
\alias{nodesplit_summary-class}
\alias{nodesplit_summary}
\title{The \code{nodesplit_summary} class}
\description{
The \code{nodesplit_summary} class contains posterior summary statistics for
node-splitting models, as a result of calling \code{summary()} on a
\code{nma_nodesplit} or \code{nma_nodesplit_df} object.
}
\details{
Objects of class \code{nodesplit_summary} are tibble data frames, with one row
for each node-split comparison and columns:
\describe{
\item{\code{trt1}, \code{trt2}}{Treatments forming the comparison}
\item{\code{summary}}{A list column containing \link{nma_summary} objects with the
posterior summaries and draws for each of the node-splitting parameters}
\item{\code{p_value}}{Bayesian p-value for inconsistency}
\item{\code{dic}}{A list column containing \link{nma_dic} objects, giving the model
fit statistics}
}

The parameters included in \code{summary} are:
\describe{
\item{\code{d_net}}{Network estimate from the corresponding consistency model,
if available}
\item{\code{d_dir}}{Direct estimate from the node-splitting model}
\item{\code{d_ind}}{Indirect estimate from the node-splitting model}
\item{\code{omega}}{Inconsistency factor \eqn{\omega = d_\mathrm{dir} -
  d_\mathrm{ind}}{\omega = d_dir - d_ind}}
\item{\code{tau}}{Heterogeneity standard deviation from the node-splitting
model, if a random effects model was fitted}
\item{\code{tau_consistency}}{Heterogeneity standard deviation from the
corresponding consistency model, if available and if a random effects model
was fitted}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_summary-class.R
\name{nma_summary-class}
\alias{nma_summary-class}
\alias{nma_summary}
\alias{nma_rank_probs}
\title{The \code{nma_summary} class}
\description{
The \code{nma_summary} class contains posterior summary statistics of model
parameters or other quantities of interest, and the draws used to obtain
these statistics.
}
\details{
Objects of class \code{nma_summary} have the following components:
\describe{
\item{summary}{A data frame containing the computed summary statistics.
Column \code{.trt} indicates the corresponding treatment, or columns \code{.trta} and
\code{.trtb} indicate the corresponding contrast (\code{.trtb} vs. \code{.trta}). If a
regression model was fitted with effect modifier interactions with
treatment, these summaries will be study-specific. In this case, the
corresponding study population is indicated in the \code{.study} column. If a
multinomial model was fitted, the \code{.category} column indicates the
corresponding category.}
\item{sims}{A 3D array [Iteration, Chain, Parameter] of MCMC
simulations}
\item{studies}{(Optional) A data frame containing study information,
printed along with the corresponding summary statistics if \code{summary}
contains a \code{.study} column. Should have a matching \code{.study} column.}
}

The following attributes may also be set:
\describe{
\item{xlab}{Label for x axis in plots, usually either \code{"Treatment"} or
\code{"Contrast"}.}
\item{ylab}{Label for y axis in plots, usually used for the scale e.g.
\code{"log Odds Ratio"}.}
}

The subclass \code{nma_rank_probs} is used by the function
\code{\link[=posterior_rank_probs]{posterior_rank_probs()}}, and contains posterior rank probabilities. This
subclass does not have a \code{sims} component, as the rank probabilities are
themselves posterior summaries of the ranks (i.e. they do not have a
posterior distribution). The posterior ranks from which the rank
probabilities are calculated may be obtained from \code{\link[=posterior_ranks]{posterior_ranks()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/default.R
\name{.default}
\alias{.default}
\alias{.is_default}
\title{Set default values}
\usage{
.default(x = list())

.is_default(x)
}
\arguments{
\item{x}{An object}
}
\value{
For \code{.default()}, an identical object with additional attribute
\code{.default}. For \code{.is_default()}, a logical value (\code{TRUE} or \code{FALSE}).
}
\description{
The \code{.default()} function is used internally to mark certain values as
default, so that the user may be notified when default values are being used.
For example, choosing a default reference treatment for a network, or using
default prior distributions. The function \code{.is_default()} checks whether an
argument/object is set to a default value. Neither of these functions are
intended to be called by the user.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{set_agd_arm}
\alias{set_agd_arm}
\title{Set up arm-based aggregate data}
\usage{
set_agd_arm(
  data,
  study,
  trt,
  y = NULL,
  se = NULL,
  r = NULL,
  n = NULL,
  E = NULL,
  sample_size = NULL,
  trt_ref = NULL,
  trt_class = NULL
)
}
\arguments{
\item{data}{a data frame}

\item{study}{column of \code{data} specifying the studies, coded using integers,
strings, or factors}

\item{trt}{column of \code{data} specifying treatments, coded using integers,
strings, or factors}

\item{y}{column of \code{data} specifying a continuous outcome}

\item{se}{column of \code{data} specifying the standard error for a continuous
outcome}

\item{r}{column of \code{data} specifying a binary or Binomial outcome count}

\item{n}{column of \code{data} specifying Binomial outcome numerator}

\item{E}{column of \code{data} specifying the total time at risk for Poisson
outcomes}

\item{sample_size}{column of \code{data} giving the sample size in each arm.
Optional, see details.}

\item{trt_ref}{reference treatment for the network, as a single integer,
string, or factor. If not specified, a reasonable well-connected default
will be chosen (see details).}

\item{trt_class}{column of \code{data} specifying treatment classes, coded using
integers, strings, or factors. By default, no classes are specified.}
}
\value{
An object of class \link{nma_data}
}
\description{
Set up a network containing arm-based aggregate data (AgD), such as event
counts or mean outcomes on each arm. Multiple data sources may be combined
once created using \code{\link[=combine_network]{combine_network()}}.
}
\details{
By default, \code{trt_ref = NULL} and a network reference treatment will be chosen
that attempts to maximise computational efficiency and stability. If an
alternative reference treatment is chosen and the model runs slowly or has
low effective sample size (ESS) this may be the cause - try letting the
default reference treatment be used instead. Regardless of which treatment is
used as the network reference at the model fitting stage, results can be
transformed afterwards: see the \code{trt_ref} argument of
\code{\link[=relative_effects]{relative_effects()}} and \code{\link[=predict.stan_nma]{predict.stan_nma()}}.

The \code{sample_size} argument is optional, but when specified:
\itemize{
\item Enables automatic centering of predictors (\code{center = TRUE}) in \code{\link[=nma]{nma()}}
when a regression model is given for a network combining IPD and AgD
\item Enables production of study-specific relative effects, rank probabilities,
etc. for studies in the network when a regression model is given
\item Nodes in \code{\link[=plot.nma_data]{plot.nma_data()}} may be weighted by sample size
}

If a Binomial outcome is specified and \code{sample_size} is omitted, \code{n} will be
used as the sample size by default. If a Multinomial outcome is specified and
\code{sample_size} is omitted, the sample size will be determined automatically
from the supplied counts by default.

All arguments specifying columns of \code{data} accept the following:
\itemize{
\item A column name as a character string, e.g. \code{study = "studyc"}
\item A bare column name, e.g. \code{study = studyc}
\item \code{dplyr::mutate()} style semantics for inline variable transformations, e.g. \code{study = paste(author, year)}
}
}
\examples{
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net


# Plot network
plot(smk_net)
}
\seealso{
\code{\link[=set_ipd]{set_ipd()}} for individual patient data, \code{\link[=set_agd_contrast]{set_agd_contrast()}} for
contrast-based aggregate data, and \code{\link[=combine_network]{combine_network()}} for combining
several data sources in one network.

\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ranks.R
\name{posterior_ranks}
\alias{posterior_ranks}
\alias{posterior_rank_probs}
\title{Treatment rankings and rank probabilities}
\usage{
posterior_ranks(
  x,
  newdata = NULL,
  study = NULL,
  lower_better = TRUE,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
  sucra = FALSE,
  summary = TRUE,
  subset = NULL
)

posterior_rank_probs(
  x,
  newdata = NULL,
  study = NULL,
  lower_better = TRUE,
  cumulative = FALSE,
  sucra = FALSE,
  subset = NULL
)
}
\arguments{
\item{x}{A \code{stan_nma} object created by \code{\link[=nma]{nma()}}}

\item{newdata}{Only used if a regression model is fitted. A data frame of
study details, one row per study, giving the covariate values at which to
produce relative effects. Column names must match variables in the
regression model. If \code{NULL}, relative effects are produced for all studies
in the network.}

\item{study}{Column of \code{newdata} which specifies study names, otherwise
studies will be labelled by row number.}

\item{lower_better}{Logical, are lower treatment effects better (\code{TRUE};
default) or higher better (\code{FALSE})? See details.}

\item{probs}{Numeric vector of quantiles of interest to present in computed
summary, default \code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{sucra}{Logical, calculate the surface under the cumulative ranking
curve (SUCRA) for each treatment? Default \code{FALSE}.}

\item{summary}{Logical, calculate posterior summaries? Default \code{TRUE}.}

\item{subset}{Names of treatments to include in the rank calculation, e.g. if
the decision set is smaller than the analysis set. By default, all
treatments in the network will be included.}

\item{cumulative}{Logical, return cumulative rank probabilities? Default is
\code{FALSE}, return posterior probabilities of each treatment having a given
rank. If \code{TRUE}, cumulative posterior rank probabilities are returned for
each treatment having a given rank or better.}
}
\value{
A \link{nma_summary} object if \code{summary = TRUE}, otherwise a list
containing a 3D MCMC array of samples and (for regression models) a data
frame of study information.
}
\description{
Produce posterior treatment rankings and rank probabilities from a fitted NMA
model. When a meta-regression is fitted with effect modifier interactions
with treatment, these will differ by study population.
}
\details{
The function \code{posterior_ranks()} produces posterior rankings, which
have a distribution (e.g. mean/median rank and 95\% Credible Interval). The
function \code{posterior_rank_probs()} produces rank probabilities, which give
the posterior probabilities of being ranked first, second, etc. out of all
treatments.

The argument \code{lower_better} specifies whether lower treatment
effects or higher treatment effects are preferred. For example, with a
negative binary outcome lower (more negative) log odds ratios are
preferred, so \code{lower_better = TRUE}. Conversely, for example, if treatments
aim to increase the rate of a positive outcome then \code{lower_better = FALSE}.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Produce posterior ranks
smk_rank_RE <- posterior_ranks(smk_fit_RE, lower_better = FALSE)
smk_rank_RE
plot(smk_rank_RE)

# Produce rank probabilities
smk_rankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE)
smk_rankprob_RE
plot(smk_rankprob_RE)

# Produce cumulative rank probabilities
smk_cumrankprob_RE <- posterior_rank_probs(smk_fit_RE, lower_better = FALSE,
                                           cumulative = TRUE)
smk_cumrankprob_RE
plot(smk_cumrankprob_RE)

# Further customisation is possible with ggplot commands
plot(smk_cumrankprob_RE) +
  ggplot2::facet_null() +
  ggplot2::aes(colour = Treatment)
}

## Plaque psoriasis ML-NMR
\donttest{
# Run plaque psoriasis ML-NMR example if not already available
if (!exists("pso_fit")) example("example_pso_mlnmr", run.donttest = TRUE)
}
\donttest{
# Produce population-adjusted rankings for all study populations in
# the network

# Ranks
pso_rank <- posterior_ranks(pso_fit)
pso_rank
plot(pso_rank)

# Rank probabilities
pso_rankprobs <- posterior_rank_probs(pso_fit)
pso_rankprobs
plot(pso_rankprobs)

# Cumulative rank probabilities
pso_cumrankprobs <- posterior_rank_probs(pso_fit, cumulative = TRUE)
pso_cumrankprobs
plot(pso_cumrankprobs)

# Produce population-adjusted rankings for a different target
# population
new_agd_means <- data.frame(
  bsa = 0.6,
  prevsys = 0.1,
  psa = 0.2,
  weight = 10,
  durnpso = 3)

# Ranks
posterior_ranks(pso_fit, newdata = new_agd_means)

# Rank probabilities
posterior_rank_probs(pso_fit, newdata = new_agd_means)

# Cumulative rank probabilities
posterior_rank_probs(pso_fit, newdata = new_agd_means,
                     cumulative = TRUE)
}
}
\seealso{
\code{\link[=plot.nma_summary]{plot.nma_summary()}} for plotting the ranks and rank probabilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{plot_prior_posterior}
\alias{plot_prior_posterior}
\title{Plot prior vs posterior distribution}
\usage{
plot_prior_posterior(
  x,
  ...,
  prior = NULL,
  post_args = list(),
  prior_args = list(),
  overlay = c("prior", "posterior"),
  ref_line = NA_real_
)
}
\arguments{
\item{x}{A \code{stan_nma} object}

\item{...}{Additional arguments passed on to methods}

\item{prior}{Character vector selecting the prior and posterior
distribution(s) to plot. May include \code{"intercept"}, \code{"trt"}, \code{"het"},
\code{"reg"}, \code{"aux"}, \code{"class_mean"} or \code{"class_sd"} as appropriate.}

\item{post_args}{List of arguments passed on to \link[ggplot2:geom_histogram]{ggplot2::geom_histogram} to
control plot output for the posterior distribution}

\item{prior_args}{List of arguments passed on to \link[ggplot2:geom_path]{ggplot2::geom_path} to
control plot output for the prior distribution. Additionally, \code{n} controls
the number of points the density curve is evaluated at (default \code{500}), and
\code{p_limits} controls the endpoints of the curve as quantiles (default
\code{c(.001, .999)}).}

\item{overlay}{String, should prior or posterior be shown on top? Default
\code{"prior"}.}

\item{ref_line}{Numeric vector of positions for reference lines, by default
no reference lines are drawn}
}
\value{
A \code{ggplot} object.
}
\description{
Produce plots comparing the prior and posterior distributions of model
parameters.
}
\details{
Prior distributions are displayed as lines, posterior distributions
are displayed as histograms.
}
\examples{
## Smoking cessation NMA
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Plot prior vs. posterior, by default all parameters are plotted
plot_prior_posterior(smk_fit_RE)

# Plot prior vs. posterior for heterogeneity SD only
plot_prior_posterior(smk_fit_RE, prior = "het")

# Customise plot
plot_prior_posterior(smk_fit_RE, prior = "het",
                     prior_args = list(colour = "darkred", linewidth = 2),
                     post_args = list(alpha = 0.6))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{atrial_fibrillation}
\alias{atrial_fibrillation}
\title{Stroke prevention in atrial fibrillation patients}
\format{
A data frame with 63 rows and 11 variables:
\describe{
\item{studyc}{study name}
\item{studyn}{numeric study ID}
\item{trtc}{treatment name}
\item{trtn}{numeric treatment code}
\item{trt_class}{treatment class}
\item{r}{number of events}
\item{n}{sample size}
\item{E}{person-years at risk}
\item{stroke}{proportion of individuals with prior stroke}
\item{year}{year of study publication}
\item{followup}{mean length of follow-up (years)}
}
}
\usage{
atrial_fibrillation
}
\description{
Data frame containing the results of 26 trials comparing 17 treatments in 4
classes for the prevention of stroke in patients with atrial fibrillation
\insertCite{Cooper2009}{multinma}. The data are the corrected versions
given by \insertCite{gemtc;textual}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bcg_vaccine}
\alias{bcg_vaccine}
\title{BCG vaccination}
\format{
A data frame with 26 rows and 6 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{latitude}{absolute degrees latitude}
\item{r}{number diagnosed with TB}
\item{n}{sample size}
}
}
\usage{
bcg_vaccine
}
\description{
Data frame containing the results of 13 trials comparing BCG vaccination to
no vaccination for preventing tuberculosis (TB)
\insertCite{TSD3,Berkey1995}{multinma}. The numbers of individuals diagnosed
with TB in each arm during the study follow-up period are recorded. The
absolute degrees latitude at which the study was conducted are also recorded.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_nodesplit-class.R
\name{nma_nodesplit-class}
\alias{nma_nodesplit-class}
\alias{nma_nodesplit}
\alias{nma_nodesplit_df}
\alias{nma_nodesplit_df-class}
\title{The nma_nodesplit class}
\description{
The \code{nma_nodesplit} and \code{nma_nodesplit_df} classes contains the results from
running a node-splitting model with the function \code{\link[=nma]{nma()}}.
}
\details{
Objects of class \code{nma_nodesplit} inherit from the \link{stan_nma} class,
and contain the results of fitting a single node-split model. They have one
additional component, \code{nodesplit}, which gives the comparison that was
node-split as a length 2 vector.

Objects of class \code{nma_nodesplit_df} are tibble data frames with one row
for each node-split comparison and columns:
\describe{
\item{\code{trt1}, \code{trt2}}{Treatments forming the comparison}
\item{\code{model}}{A list column containing the results of each model as a
\code{nma_nodesplit} object}
}
Optionally, there will be an additional row for the consistency model if
this was fitted (e.g. by \code{get_nodesplits(., include_consistency = TRUE)})
with \code{trt1} and \code{trt2} both \code{NA}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{plot_integration_error}
\alias{plot_integration_error}
\title{Plot numerical integration error}
\usage{
plot_integration_error(
  x,
  ...,
  stat = "violin",
  orientation = c("vertical", "horizontal", "x", "y"),
  show_expected_rate = TRUE
)
}
\arguments{
\item{x}{An object of type \code{stan_mlnmr}}

\item{...}{Additional arguments passed to the \code{ggdist} plot stat.}

\item{stat}{Character string specifying the \code{ggdist} plot stat used to
summarise the integration error over the posterior. Default is \code{"violin"},
which is equivalent to \code{"eye"} with some cosmetic tweaks.}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}), default \code{"vertical"}}

\item{show_expected_rate}{Logical, show typical convergence rate \eqn{1/N}?
Default \code{TRUE}.}
}
\value{
A \code{ggplot} object.
}
\description{
For ML-NMR models, plot the estimated numerical integration error over the
entire posterior distribution, as the number of integration points increases.
See \insertCite{methods_paper,Phillippo_thesis}{multinma} for details.
}
\details{
The total number of integration points is set by the \code{n_int}
argument to \code{\link[=add_integration]{add_integration()}}, and the intervals at which integration
error is estimated are set by the \code{int_thin} argument to \code{\link[=nma]{nma()}}. The
typical convergence rate of Quasi-Monte Carlo integration (as used here) is
\eqn{1/N}, which by default is displayed on the plot output.

The integration error at each thinning interval \eqn{N_\mathrm{thin}}{N_thin} is
estimated for each point in the posterior distribution by subtracting the
final estimate (using all \code{n_int} points) from the estimate using only the
first \eqn{N_\mathrm{thin}}{N_thin} points.
}
\section{Note for survival models}{
This function is not supported for survival/time-to-event models. These do
not save cumulative integration points for efficiency reasons (both time and
memory).
}

\examples{
## Plaque psoriasis ML-NMR
# Set up plaque psoriasis network combining IPD and AgD
library(dplyr)
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc \%in\% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)

pso_ipd <- pso_ipd \%>\%
  mutate(# Variable transformations
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd \%>\%
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

# Exclude small number of individuals with missing covariates
pso_ipd <- filter(pso_ipd, complete)

pso_net <- combine_network(
  set_ipd(pso_ipd,
          study = studyc,
          trt = trtc,
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd,
              study = studyc,
              trt = trtc,
              r = pasi75_r,
              n = pasi75_n,
              trt_class = trtclass)
)

# Print network details
pso_net

# Add integration points to the network
pso_net <- add_integration(pso_net,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  n_int = 64)

\donttest{
# Fit the ML-NMR model
pso_fit <- nma(pso_net,
               trt_effects = "fixed",
               link = "probit",
               likelihood = "bernoulli2",
               regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
               class_interactions = "common",
               prior_intercept = normal(scale = 10),
               prior_trt = normal(scale = 10),
               prior_reg = normal(scale = 10),
               init_r = 0.1,
               QR = TRUE,
               # Set the thinning factor for saving the cumulative results
               # (This also sets int_check = FALSE)
               int_thin = 8)
pso_fit

# Plot numerical integration error
plot_integration_error(pso_fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{social_anxiety}
\alias{social_anxiety}
\title{Social Anxiety}
\format{
A data frame with 248 rows and 8 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{treatment ID}
\item{y}{standardised mean difference of the outcome measure}
\item{se}{standard error of the outcome measure}
\item{trtc}{treatment name}
\item{classn}{class ID}
\item{classc}{class name}
}
}
\usage{
social_anxiety
}
\description{
Data frame containing the results of 101 clinical trials comparing 41 first-line treatments for social anxiety disorder
in adults . The 41 treatments are further categorised into 17 distinct classes. The interventions of interest include oral drugs,
psychological or behavioural therapies, and combinations of pharmacological and psychological therapies.
Following Mayo-Wilson et al. \insertCite{mayo2014psychological}{multinma} the data are given as standardised mean differences (SMD) to allow comparison across the different scales used in each study.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{print.nma_data}
\alias{print.nma_data}
\alias{print.mlnmr_data}
\title{Print \code{nma_data} objects}
\usage{
\method{print}{nma_data}(x, ..., n = 10)

\method{print}{mlnmr_data}(x, ..., n = 10)
}
\arguments{
\item{x}{\code{nma_data} object}

\item{...}{other options (not used)}

\item{n}{number of studies of each type to print}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print details of networks stored as \link{nma_data} objects, as created by
\code{\link[=set_ipd]{set_ipd()}}, \code{\link[=set_agd_arm]{set_agd_arm()}}, \code{\link[=set_agd_contrast]{set_agd_contrast()}}, \code{\link[=set_agd_surv]{set_agd_surv()}}, or
\code{\link[=combine_network]{combine_network()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{statins}
\alias{statins}
\title{Statins for cholesterol lowering}
\format{
A data frame with 38 rows and 7 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{prevention}{primary or secondary prevention study}
\item{r}{number of deaths}
\item{n}{sample size}
}
}
\usage{
statins
}
\description{
Data frame containing the results of 19 trials comparing statins to placebo
or usual care \insertCite{TSD3}{multinma}. The number of deaths (all-cause
mortality) are recorded. In some studies the aim was primary prevention
(patients had no previous heart disease), and in others the aim was secondary
prevention (patients had previous heart disease).
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_nodesplit-class.R
\name{print.nma_nodesplit_df}
\alias{print.nma_nodesplit_df}
\alias{print.nma_nodesplit}
\title{Print \code{nma_nodesplit_df} objects}
\usage{
\method{print}{nma_nodesplit_df}(x, ...)

\method{print}{nma_nodesplit}(x, ...)
}
\arguments{
\item{x}{A \link{nma_nodesplit_df} object}

\item{...}{Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print \code{nma_nodesplit_df} objects
}
\seealso{
The summary method \code{\link[=summary.nma_nodesplit_df]{summary.nma_nodesplit_df()}} summarises the
node-splitting results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{plot.nma_data}
\alias{plot.nma_data}
\title{Network plots}
\usage{
\method{plot}{nma_data}(
  x,
  ...,
  layout,
  circular,
  weight_edges = TRUE,
  weight_nodes = FALSE,
  show_trt_class = FALSE,
  level = c("treatment", "class"),
  nudge = 0
)
}
\arguments{
\item{x}{A \link{nma_data} object to plot}

\item{...}{Additional arguments passed to
\code{\link[ggraph:ggraph]{ggraph()}} and on to the layout function}

\item{layout}{The type of layout to create. Any layout accepted by
\code{\link[ggraph:ggraph]{ggraph()}} may be used, including all of the
layout functions provided by \link[igraph:igraph-package]{igraph}.}

\item{circular}{Whether to use a circular representation. See
\code{\link[ggraph:ggraph]{ggraph()}}.}

\item{weight_edges}{Weight edges by the number of studies? Default is \code{TRUE}.}

\item{weight_nodes}{Weight nodes by the total sample size? Default is
\code{FALSE}.}

\item{show_trt_class}{Colour treatment nodes by class, if \code{trt_class} is set?
Default is \code{FALSE}.}

\item{level}{Display network at the \code{"treatment"} (default) or \code{"class"} level.}

\item{nudge}{Numeric value to nudge the treatment labels away from the nodes
when \code{weight_nodes = TRUE}. Default is \code{0} (no adjustment to label
position). A small value like \code{0.1} is usually sufficient.}
}
\value{
A \code{ggplot} object, as produced by
\code{\link[ggraph:ggraph]{ggraph()}}.
}
\description{
Create a network plot from a \code{nma_data} network object.
}
\details{
The default is equivalent to \code{layout = "linear"} and \code{circular = TRUE}, which places the treatment nodes on a circle in the order defined by
the treatment factor variable. An alternative layout which may give good
results for simple networks is \code{"sugiyama"}, which attempts to minimise the
number of edge crossings.

\code{weight_nodes = TRUE} requires that sample sizes have been specified for
any aggregate data in the network, using the \code{sample_size} option of
\verb{set_agd_*()}.
}
\examples{
## Stroke prevention in atrial fibrillation
# Setting up the network
af_net <- set_agd_arm(atrial_fibrillation,
                      study = studyc,
                      trt = abbreviate(trtc, minlength = 3),
                      r = r,
                      n = n,
                      trt_class = trt_class)
af_net

# Basic plot
plot(af_net)

# Turn off weighting edges by number of studies
plot(af_net, weight_edges = FALSE)

# Turn on weighting nodes by sample size
plot(af_net, weight_nodes = TRUE)

# Colour treatment nodes by class
plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE)

# Nudge the treatment labels away from the nodes
plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE, nudge = 0.1)

# Output may be customised using standard ggplot commands
# For example, to display the legends below the plot:
plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE) +
  ggplot2::theme(legend.position = "bottom",
                 legend.box = "vertical",
                 legend.margin = ggplot2::margin(0, 0, 0, 0),
                 legend.spacing = ggplot2::unit(0.5, "lines"))

# Choosing a different ggraph layout, hiding some legends
plot(af_net, weight_nodes = TRUE, show_trt_class = TRUE,
     layout = "star") +
  ggplot2::guides(edge_width = "none", size = "none")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{certolizumab}
\alias{certolizumab}
\title{Certolizumab}
\format{
A data frame with 24 rows and 5 variables:
\describe{
\item{study}{study name}
\item{trt}{treatment name}
\item{r}{number of patients achieving ACR50 at 6 months}
\item{n}{sample size}
\item{disease_duration}{mean disease duration in years}
}
}
\usage{
certolizumab
}
\description{
Data frame containing the results of 12 trials comparing 7 treatments for rheumatoid arthritis, analysed in \insertCite{TSD3}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dietary_fat}
\alias{dietary_fat}
\title{Reduced dietary fat to prevent mortality}
\format{
A data frame with 21 rows and 7 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{number of events}
\item{n}{number randomised}
\item{E}{person-years at risk}
}
}
\usage{
dietary_fat
}
\description{
Data frame containing the number of deaths and person-years at risk in 10
trials comparing reduced fat diets vs. control (non-reduced fat diet) for
preventing mortality \insertCite{Hooper2000,TSD2}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{plaque_psoriasis_ipd}
\alias{plaque_psoriasis_ipd}
\alias{plaque_psoriasis}
\alias{plaque_psoriasis_agd}
\title{Plaque psoriasis data}
\format{
The individual patient data are contained in a data frame
\code{plaque_psoriasis_ipd} with 4118 rows, one per individual, and 16 variables:
\describe{
\item{studyc}{study name}
\item{trtc_long}{treatment name (long format)}
\item{trtc}{treatment name}
\item{trtn}{numeric treatment code}
\item{pasi75}{binary PASI 75 outcome}
\item{pasi90}{binary PASI 90 outcome}
\item{pasi100}{binary PASI 100 outcome}
\item{age}{age (years)}
\item{bmi}{body mass index (BMI)}
\item{pasi_w0}{PASI score at week 0}
\item{male}{male sex (TRUE or FALSE)}
\item{bsa}{body surface area (percent)}
\item{weight}{weight (kilograms)}
\item{durnpso}{duration of psoriasis (years)}
\item{prevsys}{previous systemic treatment (TRUE or FALSE)}
\item{psa}{psoriatic arthritis (TRUE or FALSE)}
}

The aggregate data are contained in a data frame \code{plaque_psoriasis_agd} with 15
rows, one per study arm, and 26 variables:
\describe{
\item{studyc}{study name}
\item{trtc_long}{treatment name (long format)}
\item{trtc}{treatment name}
\item{trtn}{numeric treatment code}
\item{pasi75_r, pasi75_n}{PASI 75 outcome count and denominator}
\item{pasi90_r, pasi90_n}{PASI 75 outcome count and denominator}
\item{pasi100_r, pasi100_n}{PASI 75 outcome count and denominator}
\item{sample_size_w0}{sample size at week zero}
\item{age_mean, age_sd}{mean and standard deviation of age (years)}
\item{bmi_mean, bmi_sd}{mean and standard deviation of BMI}
\item{pasi_w0_mean, pasi_w0_sd}{mean and standard deviation of PASI score at week 0}
\item{male}{percentage of males}
\item{bsa_mean, bsa_sd}{mean and standard deviation of body surface area (percent)}
\item{weight_mean, weight_sd}{mean and standard deviation of weight (kilograms)}
\item{durnpso_mean, durnpso_sd}{mean and standard deviation of duration of psoriasis (years)}
\item{prevsys}{percentage of individuals with previous systemic treatment}
\item{psa}{percentage of individuals with psoriatic arthritis}
}

An object of class \code{data.frame} with 15 rows and 26 columns.
}
\usage{
plaque_psoriasis_ipd

plaque_psoriasis_agd
}
\description{
Two data frames, \code{plaque_psoriasis_ipd} and \code{plaque_psoriasis_agd},
containing (simulated) individual patient data from four studies and
aggregate data from five studies \insertCite{Phillippo_thesis}{multinma}.
Outcomes are binary success/failure to achieve 75\%, 90\%, or 100\% reduction in
symptoms on the Psoriasis Area and Severity Index (PASI) scale.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{combine_network}
\alias{combine_network}
\title{Combine multiple data sources into one network}
\usage{
combine_network(..., trt_ref)
}
\arguments{
\item{...}{multiple data sources, as defined using the \verb{set_*} functions}

\item{trt_ref}{reference treatment for the entire network, as a string (or
coerced as such) referring to the levels of the treatment factor variable}
}
\value{
An object of class \link{nma_data}
}
\description{
Multiple data sources created using \code{\link[=set_ipd]{set_ipd()}}, \code{\link[=set_agd_arm]{set_agd_arm()}}, or
\code{\link[=set_agd_contrast]{set_agd_contrast()}} can be combined into a single network for analysis.
}
\examples{
## Parkinson's - combining contrast- and arm-based data
studies <- parkinsons$studyn
(parkinsons_arm <- parkinsons[studies \%in\% 1:3, ])
(parkinsons_contr <- parkinsons[studies \%in\% 4:7, ])

park_arm_net <- set_agd_arm(parkinsons_arm,
                            study = studyn,
                            trt = trtn,
                            y = y,
                            se = se,
                            sample_size = n)

park_contr_net <- set_agd_contrast(parkinsons_contr,
                                   study = studyn,
                                   trt = trtn,
                                   y = diff,
                                   se = se_diff,
                                   sample_size = n)

park_net <- combine_network(park_arm_net, park_contr_net)

# Print network details
park_net

# Plot network
plot(park_net, weight_edges = TRUE, weight_nodes = TRUE)

## Plaque Psoriasis - combining IPD and AgD in a network
# Set up plaque psoriasis network combining IPD and AgD
library(dplyr)
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc \%in\% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)

pso_ipd <- pso_ipd \%>\%
  mutate(# Variable transformations
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd \%>\%
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

# Exclude small number of individuals with missing covariates
pso_ipd <- filter(pso_ipd, complete)

pso_net <- combine_network(
  set_ipd(pso_ipd,
          study = studyc,
          trt = trtc,
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd,
              study = studyc,
              trt = trtc,
              r = pasi75_r,
              n = pasi75_n,
              trt_class = trtclass)
)

# Print network details
pso_net


# Plot network
plot(pso_net, weight_nodes = TRUE, weight_edges = TRUE, show_trt_class = TRUE)
}
\seealso{
\code{\link[=set_ipd]{set_ipd()}}, \code{\link[=set_agd_arm]{set_agd_arm()}}, and \code{\link[=set_agd_contrast]{set_agd_contrast()}} for defining
different data sources.

\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_array-class.R
\name{mcmc_array-class}
\alias{mcmc_array-class}
\alias{mcmc_array}
\alias{summary.mcmc_array}
\alias{print.mcmc_array}
\alias{plot.mcmc_array}
\alias{names.mcmc_array}
\alias{names<-.mcmc_array}
\title{Working with 3D MCMC arrays}
\usage{
\method{summary}{mcmc_array}(object, ..., probs = c(0.025, 0.25, 0.5, 0.75, 0.975))

\method{print}{mcmc_array}(x, ...)

\method{plot}{mcmc_array}(x, ...)

\method{names}{mcmc_array}(x)

\method{names}{mcmc_array}(x) <- value
}
\arguments{
\item{...}{Further arguments passed to other methods}

\item{probs}{Numeric vector of quantiles of interest}

\item{x, object}{A 3D MCMC array of class \code{mcmc_array}}

\item{value}{Character vector of replacement parameter names}
}
\value{
The \code{summary()} method returns a \link{nma_summary} object, the \code{print()}
method returns \code{x} invisibly. The \code{names()} method returns a character
vector of parameter names, and \verb{names()<-} returns the object with updated
parameter names. The \code{plot()} method is a shortcut for
\code{plot(summary(x), ...)}, passing all arguments on to \code{\link[=plot.nma_summary]{plot.nma_summary()}}.
}
\description{
3D MCMC arrays (Iterations, Chains, Parameters) are produced by \code{as.array()}
methods applied to \code{stan_nma} or \code{nma_summary} objects.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Working with arrays of posterior draws (as mcmc_array objects) is
# convenient when transforming parameters

# Transforming log odds ratios to odds ratios
LOR_array <- as.array(relative_effects(smk_fit_RE))
OR_array <- exp(LOR_array)

# mcmc_array objects can be summarised to produce a nma_summary object
smk_OR_RE <- summary(OR_array)

# This can then be printed or plotted
smk_OR_RE
plot(smk_OR_RE, ref_line = 1)

# Transforming heterogeneity SD to variance
tau_array <- as.array(smk_fit_RE, pars = "tau")
tausq_array <- tau_array^2

# Correct parameter names
names(tausq_array) <- "tausq"

# Summarise
summary(tausq_array)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc-examples.R
\name{example_ndmm}
\alias{example_ndmm}
\title{Example newly-diagnosed multiple myeloma}
\description{
Calling \code{example("example_ndmm")} will run a proportional
hazards Weibull NMA model on the newly-diagnosed multiple myeloma data,
using the code in the Examples section below. The resulting \code{stan_nma}
object \code{ndmm_fit} will then be available in the global environment.
}
\examples{
# Set up newly-diagnosed multiple myeloma network

head(ndmm_ipd)
head(ndmm_agd)

ndmm_net <- combine_network(
  set_ipd(ndmm_ipd,
          study, trt,
          Surv = Surv(eventtime / 12, status)),
  set_agd_surv(ndmm_agd,
               study, trt,
               Surv = Surv(eventtime / 12, status),
               covariates = ndmm_agd_covs))
\donttest{
# Fit Weibull (PH) model
ndmm_fit <- nma(ndmm_net, 
                likelihood = "weibull",
                prior_intercept = normal(scale = 100),
                prior_trt = normal(scale = 10),
                prior_aux = half_normal(scale = 10))

ndmm_fit
}
\dontshow{
if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
  assign("ndmm_net", ndmm_net, .GlobalEnv)
  assign("ndmm_fit", ndmm_fit, .GlobalEnv)
}
}
}
\keyword{examples}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blocker}
\alias{blocker}
\title{Beta blockers to prevent mortality after MI}
\format{
A data frame with 44 rows and 5 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
}
}
\usage{
blocker
}
\description{
Data frame containing the number of deaths in 22 trials comparing beta
blockers vs. control for preventing mortality after myocardial infarction
\insertCite{Carlin1992,TSD2}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{get_nodesplits}
\alias{get_nodesplits}
\alias{has_direct}
\alias{has_indirect}
\title{Direct and indirect evidence}
\usage{
get_nodesplits(network, include_consistency = FALSE)

has_direct(network, trt1, trt2)

has_indirect(network, trt1, trt2)
}
\arguments{
\item{network}{An \code{nma_data} object, as created by the functions \verb{set_*()} or
\code{combine_network()}.}

\item{include_consistency}{Logical, whether to include a row of \code{NA}s to
indicate that a consistency model (i.e. a model with no node-splitting)
should also be fitted by the \code{\link[=nma]{nma()}} function. Default is \code{FALSE} when
calling \code{get_nodesplits()} by hand, and \code{\link[=nma]{nma()}} sets this to \code{TRUE} by
default.}

\item{trt1, trt2}{Treatments, each as a single integer, string, or factor}
}
\value{
For \code{has_direct()} and \code{has_indirect()}, a single logical value. For
\code{get_nodesplits()}, a data frame with two columns giving the comparisons
for node-splitting.
}
\description{
Determine whether two treatments in a network are connected by direct and/or
indirect evidence, and generate a list of comparisons with both direct and
indirect evidence (i.e. potential inconsistency) for node-splitting.
}
\details{
The list of comparisons for node-splitting is generated following
the algorithm of \insertCite{Valkenhoef2016;textual}{multinma}. A
comparison between two treatments has the potential for inconsistency, and
is thus considered for node-splitting, if the comparison has both direct
evidence and \emph{independent} indirect evidence.

The notion of independent indirect evidence is necessary when multi-arm
trials are present, since by design these trials are internally consistent.
A comparison between two treatments has independent indirect evidence if,
after removing all studies comparing the two treatments from the network,
the two treatments are still connected by a path of evidence. This is the
criterion considered by the \code{has_indirect()} function.
}
\examples{
# Parkinsons example
park_net <- set_agd_arm(parkinsons,
                        study = studyn,
                        trt = trtn,
                        y = y,
                        se = se,
                        trt_ref = 1)

# View the network plot
plot(park_net)

# The 4 vs. 5 comparison is a spur on the network
has_direct(park_net, 4, 5)
has_indirect(park_net, 4, 5)

# 1 and 5 are not directly connected
has_direct(park_net, 1, 5)
has_indirect(park_net, 1, 5)

# The 1 vs. 2 comparison does not have independent indirect evidence, since
# the 1-2-4 loop is a multi-arm study
has_indirect(park_net, 1, 2)

# Get a list of comparisons with potential inconsistency for node-splitting
get_nodesplits(park_net)

# See van Valkenhoef (2016) for a discussion of this example
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{loo.stan_nma}
\alias{loo.stan_nma}
\alias{loo}
\alias{waic.stan_nma}
\alias{waic}
\title{Model comparison using the \code{loo} package}
\usage{
\method{loo}{stan_nma}(x, ...)

\method{waic}{stan_nma}(x, ...)
}
\arguments{
\item{x}{An object of class \link{stan_nma} or \link{stan_mlnmr}}

\item{...}{Further arguments to \code{\link[rstan:stanfit-method-loo]{loo()}} or
\code{\link[loo:waic]{waic()}}}
}
\description{
The \code{\link[loo:loo]{loo()}} and \code{\link[loo:waic]{waic()}} functions from the \code{loo}
package may be called directly on \link{stan_nma} and \link{stan_mlnmr} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc-examples.R
\name{example_pso_mlnmr}
\alias{example_pso_mlnmr}
\title{Example plaque psoriasis ML-NMR}
\description{
Calling \code{example("example_pso_mlnmr")} will run a ML-NMR model
with the plaque psoriasis IPD and AgD, using the code in the Examples
section below. The resulting \code{stan_nma} object \code{pso_fit} will then be
available in the global environment.
}
\details{
Plaque psoriasis ML-NMR for use in examples.
}
\examples{
# Set up plaque psoriasis network combining IPD and AgD
library(dplyr)
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc \%in\% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)

pso_ipd <- pso_ipd \%>\%
  mutate(# Variable transformations
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd \%>\%
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

# Exclude small number of individuals with missing covariates
pso_ipd <- filter(pso_ipd, complete)

pso_net <- combine_network(
  set_ipd(pso_ipd,
          study = studyc,
          trt = trtc,
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd,
              study = studyc,
              trt = trtc,
              r = pasi75_r,
              n = pasi75_n,
              trt_class = trtclass)
)

# Print network details
pso_net

# Add integration points to the network
pso_net <- add_integration(pso_net,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  n_int = 64)

\donttest{
# Fitting a ML-NMR model.
# Specify a regression model to include effect modifier interactions for five
# covariates, along with main (prognostic) effects. We use a probit link and
# specify that the two-parameter Binomial approximation for the aggregate-level
# likelihood should be used. We set treatment-covariate interactions to be equal
# within each class. We narrow the possible range for random initial values with
# init_r = 0.1, since probit models in particular are often hard to initialise.
# Using the QR decomposition greatly improves sampling efficiency here, as is
# often the case for regression models.
pso_fit <- nma(pso_net, 
               trt_effects = "fixed",
               link = "probit",
               likelihood = "bernoulli2",
               regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
               class_interactions = "common",
               prior_intercept = normal(scale = 10),
               prior_trt = normal(scale = 10),
               prior_reg = normal(scale = 10),
               init_r = 0.1,
               QR = TRUE)
pso_fit
}

\dontshow{
if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
  assign("pso_net", pso_net, .GlobalEnv)
  assign("pso_fit", pso_fit, .GlobalEnv)
}
}
}
\keyword{examples}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{is_network_connected}
\alias{is_network_connected}
\title{Check network connectedness}
\usage{
is_network_connected(network)
}
\arguments{
\item{network}{An \code{nma_data} object, as created by the functions \verb{set_*()} or
\code{combine_network()}.}
}
\value{
Logical \code{TRUE} or \code{FALSE}
}
\description{
Check whether a network is connected - whether there is a path of study
evidence linking every pair of treatments in the network.
}
\details{
Models will still run with disconnected networks. However, estimated
relative effects between treatments across disconnected parts of the
network will be entirely based on the prior distribution (typically very
uncertain), as there is no information to update the prior distribution.
Relative effects within each connected sub-network will be estimated as if
each sub-network had been analysed separately.
}
\examples{
## Smoking cessation
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net

is_network_connected(smk_net)  # TRUE, network is connected
## A disconnected network
disc_net <- set_agd_arm(smoking[smoking$studyn \%in\% c(15, 21), ],
                        study = studyn,
                        trt = trtc,
                        r = r,
                        n = n)
is_network_connected(disc_net)  # FALSE, network is disconnected
disc_net
plot(disc_net)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{dgent}
\alias{dgent}
\alias{pgent}
\alias{qgent}
\title{Generalised Student's t distribution (with location and scale)}
\usage{
dgent(x, df, location = 0, scale = 1)

pgent(q, df, location = 0, scale = 1)

qgent(p, df, location = 0, scale = 1)
}
\arguments{
\item{x, q}{Vector of quantiles}

\item{df}{Degrees of freedom, greater than zero}

\item{location}{Location parameter}

\item{scale}{Scale parameter, greater than zero}

\item{p}{Vector of probabilities}
}
\value{
\code{dgent()} gives the density, \code{pgent()} gives the distribution
function, \code{qgent()} gives the quantile function.
}
\description{
Density, distribution, and quantile function for the generalised t
distribution with degrees of freedom \code{df}, shifted by \code{location} and scaled
by \code{scale}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{dlogitnorm}
\alias{dlogitnorm}
\alias{plogitnorm}
\alias{qlogitnorm}
\title{The logit Normal distribution}
\usage{
dlogitnorm(x, mu = 0, sigma = 1, ..., mean, sd)

plogitnorm(q, mu = 0, sigma = 1, ..., mean, sd)

qlogitnorm(p, mu = 0, sigma = 1, ..., mean, sd)
}
\arguments{
\item{x, q}{vector of quantiles}

\item{mu, sigma}{location and scale parameters, on the logit scale}

\item{...}{additional arguments, passed to \verb{[dpq]norm()}}

\item{mean, sd}{mean and standard deviation, overriding \code{mu} and \code{sigma} if
specified}

\item{p}{vector of probabilities}
}
\value{
Numeric vector of length equal to the maximum of the lengths of the
input arguments.
}
\description{
Density, distribution function, and quantile function for the logit Normal
distribution. The location and scale parameters of the distribution are \code{mu}
and \code{sigma}, which are the mean and standard deviation on the logit scale.
For convenience, the distribution may also be specified in terms of its mean
and standard deviation, instead of its logit-mean and logit-sd.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adapt_delta.R
\name{adapt_delta}
\alias{adapt_delta}
\title{Target average acceptance probability}
\description{
The Stan control argument \code{adapt_delta} sets the target average acceptance
probability for the No-U-Turn Sampler (NUTS) used by Stan.
}
\details{
The default value of \code{adapt_delta} used by \code{\link[=nma]{nma()}} is 0.8 for fixed
effect models, and 0.95 for random effects models.

You should not need to change \code{adapt_delta} unless you see a warning
message about divergent transitions. Increasing \code{adapt_delta} from the
default to a value closer to 1 means that Stan will use a smaller step
size, making sampling slower but more robust, and resulting in fewer
divergent transitions.

For more details see the Stan documentation available from
\url{https://mc-stan.org/users/documentation/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic-class.R
\name{nma_dic-class}
\alias{nma_dic-class}
\alias{nma_dic}
\title{The nma_dic class}
\description{
The \code{nma_dic} class contains details of the Deviance Information Criterion
(DIC), produced using the \code{\link[=dic]{dic()}} function.
}
\details{
Objects of class \code{nma_dic} have the following components:
\describe{
\item{\code{dic}}{The DIC value}
\item{\code{pd}, \code{pv}}{The effective number of parameters}
\item{\code{resdev}}{The total residual deviance}
\item{\code{pointwise}}{A list of data frames containing the pointwise
contributions for the IPD and AgD.}
\item{\code{resdev_array}}{A 3D MCMC array [Iterations, Chains, Parameters] of
posterior residual deviance samples.}
}
}
\seealso{
\code{\link[=dic]{dic()}}, \code{\link[=print.nma_dic]{print.nma_dic()}}, \code{\link[=plot.nma_dic]{plot.nma_dic()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_multinma.R
\name{theme_multinma}
\alias{theme_multinma}
\title{Plot theme for multinma plots}
\usage{
theme_multinma(...)
}
\arguments{
\item{...}{Arguments passed to
\code{\link[ggplot2:ggtheme]{ggplot2::theme_light()}}}
}
\value{
A \code{ggplot2} theme
}
\description{
A simple \code{ggplot2} theme for plots in the \code{multinma} package.
}
\examples{
library(ggplot2)
theme_set(theme_multinma())

}
\seealso{
\code{\link[ggplot2:theme]{ggplot2::theme()}}, \code{\link[ggplot2:theme_get]{ggplot2::theme_set()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data.R
\name{set_agd_surv}
\alias{set_agd_surv}
\title{Set up aggregate survival data}
\usage{
set_agd_surv(
  data,
  study,
  trt,
  Surv,
  covariates = NULL,
  trt_ref = NULL,
  trt_class = NULL
)
}
\arguments{
\item{data}{a data frame}

\item{study}{column of \code{data} specifying the studies, coded using integers,
strings, or factors}

\item{trt}{column of \code{data} specifying treatments, coded using integers,
strings, or factors}

\item{Surv}{column of \code{data} specifying a survival or time-to-event outcome,
using the \code{\link[=Surv]{Surv()}} function. Right/left/interval censoring and left
truncation (delayed entry) are supported.}

\item{covariates}{data frame of covariate summary statistics for each study
or study arm, with corresponding \code{study} and \code{trt} columns to match to
those in \code{data}}

\item{trt_ref}{reference treatment for the network, as a single integer,
string, or factor. If not specified, a reasonable well-connected default
will be chosen (see details).}

\item{trt_class}{column of \code{data} specifying treatment classes, coded using
integers, strings, or factors. By default, no classes are specified.}
}
\value{
An object of class \link{nma_data}
}
\description{
Set up a network containing aggregate survival data (AgD) in the form of
event/censoring times (e.g. reconstructed from digitized Kaplan-Meier curves)
and covariate summary statistics from each study. Multiple data sources may be
combined once created using \code{\link[=combine_network]{combine_network()}}.
}
\details{
By default, \code{trt_ref = NULL} and a network reference treatment will be chosen
that attempts to maximise computational efficiency and stability. If an
alternative reference treatment is chosen and the model runs slowly or has
low effective sample size (ESS) this may be the cause - try letting the
default reference treatment be used instead. Regardless of which treatment is
used as the network reference at the model fitting stage, results can be
transformed afterwards: see the \code{trt_ref} argument of
\code{\link[=relative_effects]{relative_effects()}} and \code{\link[=predict.stan_nma]{predict.stan_nma()}}.

All arguments specifying columns of \code{data} accept the following:
\itemize{
\item A column name as a character string, e.g. \code{study = "studyc"}
\item A bare column name, e.g. \code{study = studyc}
\item \code{dplyr::mutate()} style semantics for inline variable transformations, e.g. \code{study = paste(author, year)}
}
}
\examples{
## Newly diagnosed multiple myeloma

head(ndmm_agd)  # Reconstructed Kaplan-Meier data
ndmm_agd_covs   # Summary covariate information on each arm

set_agd_surv(ndmm_agd,
             study = studyf,
             trt = trtf,
             Surv = Surv(eventtime, status),
             covariates = ndmm_agd_covs)

}
\seealso{
\code{\link[=set_ipd]{set_ipd()}} for individual patient data, \code{\link[=set_agd_contrast]{set_agd_contrast()}} for
contrast-based aggregate data, and \code{\link[=combine_network]{combine_network()}} for combining
several data sources in one network.

\code{\link[=print.nma_data]{print.nma_data()}} for the print method displaying details of the
network, and \code{\link[=plot.nma_data]{plot.nma_data()}} for network plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodesplit_summary-class.R
\name{plot.nodesplit_summary}
\alias{plot.nodesplit_summary}
\title{Plots of node-splitting models}
\usage{
\method{plot}{nodesplit_summary}(
  x,
  ...,
  pars = "d",
  stat = "dens_overlay",
  orientation = c("horizontal", "vertical", "y", "x"),
  ref_line = NA_real_
)
}
\arguments{
\item{x}{A \code{nodesplit_summary} object.}

\item{...}{Additional arguments passed on to the underlying \code{ggdist} plot
stat, see Details.}

\item{pars}{Character vector specifying the parameters to include in the
plot, choices include \code{"d"} for the direct, indirect, and network estimates
of relative effects, \code{"omega"} for the inconsistency factor, and \code{"tau"}
for heterogeneity standard deviation in random effects models. Default is
\code{"d"}.}

\item{stat}{Character string specifying the \code{ggdist} plot stat to use. The
default \code{"dens_overlay"} is a special case, producing an overlaid density
plot.}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}), default \code{"horizontal"}.}

\item{ref_line}{Numeric vector of positions for reference lines, by default
no reference lines are drawn.}
}
\value{
A \code{ggplot} object.
}
\description{
Produce summary plots of node-splitting models
}
\details{
Plotting is handled by \link[ggplot2:ggplot2-package]{ggplot2} and
the stats and geoms provided in the \link[ggdist:ggdist-package]{ggdist}
package. As a result, the output is very flexible. Any plotting stats
provided by \code{ggdist} may be used, via the argument \code{stat}. The default
\code{"dens_overlay"} is a special exception, which uses
\code{\link[ggplot2:geom_density]{ggplot2::geom_density()}}, to plot
overlaid densities. Additional arguments in \code{...} are passed to the
\code{ggdist} stat, to customise the output.

Alternative stats can be specified to produce different summaries. For
example, specify \code{stat = "[half]eye"} to produce (half) eye plots, or \code{stat = "pointinterval"} to produce point estimates and credible intervals.

A full list of options and examples is found in the \code{ggdist} vignette
\code{vignette("slabinterval", package = "ggdist")}.

A \code{ggplot} object is returned which can be further modified through the
usual \link[ggplot2:ggplot2-package]{ggplot2} functions to add further
aesthetics, geoms, themes, etc.
}
\examples{
\donttest{
# Run smoking node-splitting example if not already available
if (!exists("smk_fit_RE_nodesplit")) example("example_smk_nodesplit", run.donttest = TRUE)
}
\donttest{
# Summarise the node-splitting results
(smk_nodesplit_summary <- summary(smk_fit_RE_nodesplit))

# Plot the node-splitting results
plot(smk_nodesplit_summary)

# Plot the inconsistency factors instead, change the plot stat to half-eye,
# and add a reference line at 0
plot(smk_nodesplit_summary, pars = "omega", stat = "halfeye", ref_line = 0)

# Plot a comparison of the heterogeneity under the node-split models vs.
# the consistency model
plot(smk_nodesplit_summary, pars = "tau")
}
}
\seealso{
\code{\link[=summary.nma_nodesplit_df]{summary.nma_nodesplit_df()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_prior-class.R
\name{summary.nma_prior}
\alias{summary.nma_prior}
\title{Summary of prior distributions}
\usage{
\method{summary}{nma_prior}(object, ..., probs = c(0.5, 0.95), digits = 2, trunc = NULL)
}
\arguments{
\item{object}{Prior distribution as a \code{nma_prior} object}

\item{...}{Additional arguments, not used}

\item{probs}{Numeric vector of probabilities to calculate prior intervals}

\item{digits}{Number of digits to display}

\item{trunc}{Optional numeric vector of length 2, giving the truncation
limits of the prior distribution. Useful if a real-valued prior is assigned
to a positive-valued parameter, then \code{trunc = c(0, Inf)} will give the
correct prior intervals. By default, truncation is not used.}
}
\value{
A data frame is returned invisibly, giving the prior intervals
}
\description{
Print a summary of prior distribution details.
}
\examples{
summary(normal(location = 0, scale = 1))
summary(half_normal(scale = 1))
summary(log_normal(location = -3.93, scale = 1.51))

# Truncation limits may be set, for example to restrict a prior to positive values
summary(normal(location = 0.5, scale = 1), trunc = c(0, Inf))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic-class.R
\name{plot.nma_dic}
\alias{plot.nma_dic}
\title{Plots of model fit diagnostics}
\usage{
\method{plot}{nma_dic}(
  x,
  y,
  ...,
  type = c("resdev", "leverage"),
  show_uncertainty = TRUE,
  stat = "pointinterval",
  orientation = c("vertical", "horizontal", "x", "y"),
  dic_contours = 1:4
)
}
\arguments{
\item{x}{A \code{nma_dic} object}

\item{y}{(Optional) A second \code{nma_dic} object, to produce "dev-dev" plots for
model comparison.}

\item{...}{Additional arguments passed on to other methods}

\item{type}{With a single \code{nma_dic} object, whether to show the residual
deviance contribution for each data point (\code{"resdev"}, the default), or a
leverage plot (\code{"leverage"}).}

\item{show_uncertainty}{Logical, show uncertainty with a \code{ggdist} plot
stat? Default \code{TRUE}. Ignored for \code{type = "leverage"}.}

\item{stat}{Character string specifying the \code{ggdist} plot stat to use if
\code{show_uncertainty = TRUE}, default \code{"pointinterval"}. If \code{y} is provided,
currently only \code{"pointinterval"} is supported.}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}). Only used for residual
deviance plots, default \code{"vertical"}.}

\item{dic_contours}{Numeric vector of DIC contribution contours to show on
leverage plots, default \code{1:4}.}
}
\value{
A \code{ggplot} object.
}
\description{
The \code{plot()} method for \link{nma_dic} objects produced by \code{\link[=dic]{dic()}} produces
several useful diagnostic plots for checking model fit and model comparison.
Further detail on these plots and their interpretation is given by
\insertCite{TSD2;textual}{multinma}.
}
\details{
When a single \code{nma_dic} object is given, the default plot (\code{type = "resdev"}) shows the residual deviance contribution for each data point.
For a good fitting model, each data point is expected to have a residual
deviance equal to its degrees of freedom (typically 1, except for multi-arm
trials in contrast format or multinomial outcomes); larger values indicate
data points that are fit poorly by the model. A leverage plot can be
produced with \code{type = "leverage"}, plotting the leverages (contributions to
\eqn{p_D}) against the signed square root residual deviance, both
standardised by the degrees of freedom. Contours for different DIC
contribution cutoffs are also shown; points which lie outside the DIC=3
contour are generally identified as contributing to a model's poor fit.

When two \code{nma_dic} objects are given, a "dev-dev" plot comparing the
residual deviance contributions under each model is produced. Data points
with residual deviance contributions lying on the line of equality are fit
equally well under either model. Data points lying below the line of
equality indicate better fit under the second model (\code{y}); conversely, data
points lying above the line of equality indicate better fit under the first
model (\code{x}). A common use case is to compare a standard consistency model
(fitted using \code{\link[=nma]{nma()}} with \code{consistency = "consistency"}) with an unrelated
mean effects (UME) inconsistency model (fitted using \code{\link[=nma]{nma()}} with
\code{consistency = "ume"}), to check for potential inconsistency.

See \insertCite{TSD2;textual}{multinma} for further details.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking FE NMA example if not already available
if (!exists("smk_fit_FE")) example("example_smk_fe", run.donttest = TRUE)
}
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Compare DIC of FE and RE models
(smk_dic_FE <- dic(smk_fit_FE))
(smk_dic_RE <- dic(smk_fit_RE))   # substantially better fit

# Plot residual deviance contributions under RE model
plot(smk_dic_RE)

# Further customisation is possible using ggplot commands
# For example, highlighting data points with residual deviance above a certain threshold
plot(smk_dic_RE) +
  ggplot2::aes(colour = ggplot2::after_stat(ifelse(y > 1.5, "darkorange", "black"))) +
  ggplot2::scale_colour_identity()

# Or by posterior probability, for example here a central probability of 0.6
# corresponds to a lower tail probability of (1 - 0.6)/2 = 0.2
plot(smk_dic_RE, .width = c(0.6, 0.95)) +
  ggplot2::aes(colour = ggplot2::after_stat(ifelse(ymin > 1, "darkorange", "black"))) +
  ggplot2::scale_colour_identity()

# Leverage plot for RE model
plot(smk_dic_RE, type = "leverage")

# Check for inconsistency using UME model
}
\donttest{
# Run smoking UME NMA example if not already available
if (!exists("smk_fit_RE_UME")) example("example_smk_ume", run.donttest = TRUE)
}
\donttest{
# Compare DIC
smk_dic_RE
(smk_dic_RE_UME <- dic(smk_fit_RE_UME))  # no difference in fit

# Compare residual deviance contributions with a "dev-dev" plot
plot(smk_dic_RE, smk_dic_RE_UME)

# By default the dev-dev plot can be a little cluttered
# Hiding the credible intervals
plot(smk_dic_RE, smk_dic_RE_UME, show_uncertainty = FALSE)

# Changing transparency
plot(smk_dic_RE, smk_dic_RE_UME, point_alpha = 0.5, interval_alpha = 0.1)
}
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mspline.R
\name{knots.stan_nma}
\alias{knots.stan_nma}
\title{Knot locations for a fitted model}
\usage{
\method{knots}{stan_nma}(Fn, type = c("all", "internal", "boundary"), ...)
}
\arguments{
\item{Fn}{A fitted \link{stan_nma} object}

\item{type}{String, indicating whether to return all knots (\code{"all"}, the
default), or only the internal knots (\code{"internal"}) or boundary knots
(\code{"boundary"}).}

\item{...}{Other arguments, passed on to \code{\link[splines2:knots]{splines2::knots()}}}
}
\value{
A list of vectors of knot locations, for each study in the network.
}
\description{
Obtain the knot locations from a fitted M-spline or piecewise exponential
model.
}
\seealso{
\code{\link[=make_knots]{make_knots()}} for constructing knots for a network object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_nodesplit-class.R
\name{summary.nma_nodesplit_df}
\alias{summary.nma_nodesplit_df}
\alias{summary.nma_nodesplit}
\alias{plot.nma_nodesplit}
\alias{plot.nma_nodesplit_df}
\title{Summarise the results of node-splitting models}
\usage{
\method{summary}{nma_nodesplit_df}(
  object,
  consistency = NULL,
  ...,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975)
)

\method{summary}{nma_nodesplit}(
  object,
  consistency = NULL,
  ...,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975)
)

\method{plot}{nma_nodesplit}(x, consistency = NULL, ...)

\method{plot}{nma_nodesplit_df}(x, consistency = NULL, ...)
}
\arguments{
\item{consistency}{Optional, a \code{stan_nma} object for the corresponding fitted
consistency model, to display the network estimates alongside the direct
and indirect estimates. The fitted consistency model present in the
\code{nma_nodesplit_df} object will be used if this is present (see
\code{\link[=get_nodesplits]{get_nodesplits()}}).}

\item{...}{Additional arguments passed on to other methods}

\item{probs}{Numeric vector of specifying quantiles of interest, default
\code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{x, object}{A \code{nma_nodesplit} or \code{nma_nodesplit_df} object}
}
\value{
A \link{nodesplit_summary} object
}
\description{
Posterior summaries of node-splitting models (\code{nma_nodesplit} and
\code{nma_nodesplit_df} objects) can be produced using the \code{summary()} method, and
plotted using the \code{plot()} method.
}
\details{
The \code{plot()} method is a shortcut for \code{plot(summary(nma_nodesplit))}. For
details of plotting options, see \code{\link[=plot.nodesplit_summary]{plot.nodesplit_summary()}}.
}
\examples{
\donttest{
# Run smoking node-splitting example if not already available
if (!exists("smk_fit_RE_nodesplit")) example("example_smk_nodesplit", run.donttest = TRUE)
}
\donttest{
# Summarise the node-splitting results
summary(smk_fit_RE_nodesplit)

# Plot the node-splitting results
plot(smk_fit_RE_nodesplit)
}
}
\seealso{
\code{\link[=plot.nodesplit_summary]{plot.nodesplit_summary()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mspline.R
\name{dmspline}
\alias{dmspline}
\alias{pmspline}
\alias{qmspline}
\alias{hmspline}
\alias{Hmspline}
\alias{rmst_mspline}
\title{Distribution functions for M-spline baseline hazards}
\usage{
dmspline(x, basis, scoef, rate, log = FALSE)

pmspline(q, basis, scoef, rate, lower.tail = TRUE, log.p = FALSE)

qmspline(p, basis, scoef, rate, lower.tail = TRUE, log.p = FALSE)

hmspline(x, basis, scoef, rate, log = FALSE)

Hmspline(x, basis, scoef, rate, log = FALSE)

rmst_mspline(t, basis, scoef, rate, start = 0)
}
\arguments{
\item{x, q}{Vector of quantiles}

\item{basis}{M-spline basis produced by \code{\link[splines2:mSpline]{splines2::mSpline()}}}

\item{scoef}{Vector (or matrix) of spline coefficients with length (or number
of columns) equal to the dimension of \code{basis}}

\item{rate}{Vector of rate parameters}

\item{log, log.p}{Logical; if \code{TRUE}, probabilities \code{p} are given as
\eqn{\log(p)}}

\item{lower.tail}{Logical; if \code{TRUE} (the default), probabilities are
\eqn{P(X \le x)}, otherwise \eqn{P(X > x)}}

\item{p}{Vector of probabilities}

\item{t}{Vector of times to which the restricted mean survival time is
calculated}

\item{start}{Optional left-truncation time or times. The returned restricted
mean survival will be conditioned on survival up to this time}
}
\value{
\code{dmspline()} gives the density, \code{pmspline()} gives the distribution
function (CDF), \code{qmspline()} gives the quantile function (inverse-CDF),
\code{hmspline()} gives the hazard function, \code{Hmspline()} gives the cumulative
hazard function, and \code{rmst_mspline()} gives restricted mean survival times.
}
\description{
Density, distribution, quantile, hazard, cumulative hazard, and restricted
mean survival time functions for the M-spline baseline hazards model.
}
\details{
Survival models with a flexible M-spline on the baseline hazard are
described by \insertCite{Phillippo_mspline;textual}{multinma}.
Piecewise-exponential baseline hazards are a special case where the degree
of the M-spline polynomial is 0.

The d/p/h/H functions are calculated from their definitions. \code{qmspline()}
uses numerical inversion via \code{\link[flexsurv:qgeneric]{flexsurv::qgeneric()}}. \code{rmst_mspline()}uses
numerical integration via \code{\link[flexsurv:rmst_generic]{flexsurv::rmst_generic()}}, except for the
special case of the piecewise-exponential hazard (i.e. degree 0 M-splines)
which uses the explicit formula from
\insertCite{Royston2013;textual}{multinma}.

Beyond the boundary knots, the hazard is assumed to be constant. (This
differs from the approach in \code{\link[splines2:mSpline]{splines2::mSpline()}} that extrapolates the
polynomial basis functions, which is numerically unstable and highly
dependent on the data just before the boundary knots.) As with all
extrapolation, care should be taken when evaluating the splines at times
beyond the boundary knots (either directly through the d/p/h/H/rmst
functions, or indirectly by requesting quantiles with \code{qmspline()} that
correspond to times beyond the boundary knots). For this reason evaluating
the (unrestricted) mean survival time is not generally recommended as this
requires integrating over an infinite time horizon (i.e. \code{rmst_mspline()}
with \code{t = Inf}).
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mspline.R
\name{softmax}
\alias{softmax}
\alias{inv_softmax}
\title{Softmax transform}
\usage{
softmax(x)

inv_softmax(p)
}
\arguments{
\item{x}{\eqn{K-1} vector of reals}

\item{p}{\eqn{K} vector simplex}
}
\value{
\code{softmax()} returns a vector of length \eqn{K} that is a simplex.
\code{inv_softmax()} returns a vector of reals of length \eqn{K-1}.
}
\description{
The softmax transform is a multivariate generalisation of the logit
transform. \code{softmax()} maps a vector of \eqn{K-1} values on the real line to a
\eqn{K}-simplex (i.e. values between 0 and 1, that sum to 1). \code{inv_softmax()}
provides the inverse transform, mapping a \eqn{K}-simplex vector to a vector of
\eqn{K-1} real values.
}
\examples{
x <- c(-1, 3, -0.5, 2)
(p <- softmax(x))
sum(p)
inv_softmax(p)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hta_psoriasis}
\alias{hta_psoriasis}
\title{HTA Plaque Psoriasis}
\format{
A data frame with 36 rows and 9 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{year}{year of publication}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{sample_size}{sample size in each arm}
\item{PASI50, PASI75, PASI90}{ordered multinomial outcome counts (exclusive, see details)}
}
}
\usage{
hta_psoriasis
}
\description{
Data frame containing the results of 16 trials comparing 8 treatments for
moderate-to-severe plaque psoriasis from an HTA report
\insertCite{Woolacott2006}{multinma}, analysed in TSD2
\insertCite{TSD2}{multinma}. Outcomes are success/failure to achieve 50\%,
75\%, or 90\% reduction in symptoms on the Psoriasis Area and Severity Index
(PASI) scale. Some studies report all three ordered outcomes, others only one
or two. The latter are coded as missing values (see details).
}
\details{
Outcome counts are "exclusive"; that is, for a study reporting all
outcomes, the counts represent the categories 50 < PASI < 75, 75 < PASI <
90, and 90 < PASI < 100, and are named by the lower end of the interval.
(As opposed to "inclusive" counts, which would represent the overlapping
categories PASI > 50, PASI > 70, and PASI > 90.) The count for the fourth
category (the lowest), 0 < PASI < 50, is equal to \code{sample_size - PASI50 - PASI75 - PASI90}.

Missing values are used where studies only report a subset of the outcomes.
For a study reporting only two outcomes, say 50 and 75, the counts
represent 50 < PASI < 75 and 75 < PASI < 100. For a study reporting only
one outcome, say PASI 75, the count represents 75 < PASI < 100.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{add_integration}
\alias{add_integration}
\alias{add_integration.default}
\alias{add_integration.data.frame}
\alias{add_integration.nma_data}
\alias{unnest_integration}
\title{Add numerical integration points to aggregate data}
\usage{
add_integration(x, ...)

\method{add_integration}{default}(x, ...)

\method{add_integration}{data.frame}(
  x,
  ...,
  cor = NULL,
  cor_adjust = NULL,
  n_int = 64L,
  int_args = list()
)

\method{add_integration}{nma_data}(
  x,
  ...,
  cor = NULL,
  cor_adjust = NULL,
  n_int = 64L,
  int_args = list()
)

unnest_integration(data)
}
\arguments{
\item{x}{An \code{nma_data} object, as created by the \verb{set_*()} functions or
\code{combine_network()}, or data frame}

\item{...}{Distributions for covariates, see "Details"}

\item{cor}{Correlation matrix to use for generating the integration points.
By default, this takes a weighted correlation matrix from all IPD studies.
Rows and columns should match the order of covariates specified in \code{...}.}

\item{cor_adjust}{Adjustment to apply to the correlation matrix given by
\code{cor} (or computed from the IPD if \code{cor = NULL}) to obtain the Gaussian
copula correlations, either \code{"spearman"}, \code{"pearson"}, or \code{"none"}, see
"Details". The default when \code{cor = NULL} is \code{"spearman"}, otherwise the
default is \code{"pearson"}.}

\item{n_int}{Number of integration points to generate, default 64. Powers of
2 are recommended, as these are expected to be particularly efficient for
QMC integration.}

\item{int_args}{A named list of arguments to pass to
\code{\link[randtoolbox:quasiRNG]{sobol()}}}

\item{data}{Data frame with nested integration points, stored in list
columns as \verb{.int_<variable name>}}
}
\value{
For the \code{nma_data} method, an object of class \link{nma_data}. For the
\code{data.frame} method, the input data frame is returned (as a \code{tibble}) with
an added column for each covariate (prefixed with ".int_"), containing the
numerical integration points nested as length-\code{n_int} vectors within each
row. For \code{unnest_integration()}, a data frame with integration points
unnested.
}
\description{
The \code{add_integration()} generic creates Quasi-Monte Carlo numerical
integration points using a Gaussian copula and Sobol' sequences, as described
in \insertCite{methods_paper;textual}{multinma}. Methods are available for
networks stored in \code{nma_data} objects, and for data frames. The function
\code{unnest_integration()} unnests integration points stored in a data frame, to
aid plotting or other exploration.
}
\details{
The arguments passed to \code{...} specify distributions for the
covariates. Argument names specify the name of the covariate, which should
match a covariate name in the IPD (if IPD are present). The required
marginal distribution is then specified using the function \code{\link[=distr]{distr()}}.

The argument \code{cor_adjust} specifies how the correlation matrix given by
\code{cor} (or computed from the IPD if \code{cor = NULL}) is adjusted to obtain the
correlation matrix for the Gaussian copula, using the formulae in
\insertCite{Xiao2018;textual}{multinma}.
\itemize{
\item \code{cor_adjust = "spearman"} should be used when the correlations \code{cor} have
been computed using Spearman's rank correlation. Correlations between
continuous covariates will be reproduced exactly by the integration points.
Correlations between discrete covariates will be reproduced approximately.
This is the default when \code{cor = NULL} and correlations are calculated from
the IPD studies.
\item \code{cor_adjust = "pearson"} should be used when the correlations \code{cor} have
been computed using Pearson's product-moment correlation. Correlations between
Normal covariates will be reproduced exactly by the integration points, all
others will be reproduced approximately. Correlations between discrete
covariates will be reproduced approximately (and identically to \code{cor_adjust   = "spearman"}). This is the default when \code{cor} is provided by the user,
since \code{\link[=cor]{cor()}} defaults to \code{method = "pearson"} and Pearson correlations are
most likely reported in published data. However, we recommend providing
Spearman correlations (e.g. from \code{cor(., method = "spearman")}) and using
\code{cor_adjust = "spearman"} where possible.
\item \code{cor_adjust = "none"} allows the user to specify the correlation matrix
for the Gaussian copula directly; no adjustment is applied.
\item \code{cor_adjust = "legacy"} is also available, which reproduces exactly the
behaviour from version 0.3.0 and earlier. This is similar to \code{cor_adjust =   "none"}, but unadjusted Spearman correlations are used if \code{cor = NULL}.
}

When adding integration points to a network object the correlation matrix
used is stored in \verb{$int_cor}, and the copula correlation matrix and
adjustment used are stored as attributes of \verb{$int_cor}. If this correlation
matrix is passed again to \code{add_integration()} (e.g. to reuse the
correlations for an external target population) this will be detected, and
the correct setting for \code{cor_adjust} will automatically be applied.
}
\examples{
## Plaque psoriasis ML-NMR - network setup and adding integration points
# Set up plaque psoriasis network combining IPD and AgD
library(dplyr)
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc \%in\% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)

pso_ipd <- pso_ipd \%>\%
  mutate(# Variable transformations
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd \%>\%
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

# Exclude small number of individuals with missing covariates
pso_ipd <- filter(pso_ipd, complete)

pso_net <- combine_network(
  set_ipd(pso_ipd,
          study = studyc,
          trt = trtc,
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd,
              study = studyc,
              trt = trtc,
              r = pasi75_r,
              n = pasi75_n,
              trt_class = trtclass)
)

# Print network details
pso_net

# Add integration points to the network
pso_net <- add_integration(pso_net,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  n_int = 64)


## Adding integration points to a data frame, e.g. for prediction
# Define a data frame of covariate summaries
new_agd_int <- data.frame(
  bsa_mean = 0.6,
  bsa_sd = 0.3,
  prevsys = 0.1,
  psa = 0.2,
  weight_mean = 10,
  weight_sd = 1,
  durnpso_mean = 3,
  durnpso_sd = 1)

# Adding integration points, using the weighted average correlation matrix
# computed for the plaque psoriasis network
new_agd_int <- add_integration(new_agd_int,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  cor = pso_net$int_cor,
  n_int = 64)

# Here, since we reused the correlation matrix pso_net$int_cor from the
# network, the correct setting of cor_adjust = "spearman" is automatically
# applied

new_agd_int

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{stan_nma-class}
\alias{stan_nma-class}
\alias{stan_nma}
\alias{stan_mlnmr}
\title{The stan_nma class}
\description{
The \code{stan_nma} and \code{stan_mlnmr} classes contains the results from running a
model with the function \code{\link[=nma]{nma()}}.
}
\details{
Objects of class \code{stan_nma} and \code{stan_mlnmr} have the following
components:
\describe{
\item{\code{network}}{The network data from which the model was run (class
\link{nma_data} for \code{stan_nma}, or class \link{mlnmr_data} for \code{stan_mlnmr})}
\item{\code{stanfit}}{The \code{stanfit} object returned by calling
\code{\link[rstan:stanmodel-method-sampling]{sampling()}} for the model}
\item{\code{trt_effects}}{Whether fixed or random effects were used (character
string)}
\item{\code{consistency}}{The consistency/inconsistency model used (character
string)}
\item{\code{regression}}{The regression model used (formula)}
\item{\code{class_interactions}}{If treatment classes and a regression model are
specified, the model used for interactions within each class (common,
exchangeable, or independent)}
\item{\code{xbar}}{A named vector of values used for centering}
\item{\code{likelihood}}{The likelihood used (character string)}
\item{\code{link}}{The link function used (character string)}
\item{\code{priors}}{A list containing the priors used (as \link{nma_prior} objects)}
\item{\code{basis}}{For \code{mspline} and \code{pexp} models, a named list of spline
bases for each study}
}

The \code{stan_mlnmr} sub-class inherits from \code{stan_nma}, and differs only in the
class of the \code{network} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma.R
\name{nma}
\alias{nma}
\title{Network meta-analysis models}
\usage{
nma(
  network,
  consistency = c("consistency", "ume", "nodesplit"),
  trt_effects = c("fixed", "random"),
  regression = NULL,
  class_interactions = c("common", "exchangeable", "independent"),
  class_effects = c("independent", "common", "exchangeable"),
  class_sd = c("independent", "common"),
  likelihood = NULL,
  link = NULL,
  ...,
  nodesplit = get_nodesplits(network, include_consistency = TRUE),
  prior_intercept = .default(normal(scale = 100)),
  prior_trt = .default(normal(scale = 10)),
  prior_het = .default(half_normal(scale = 5)),
  prior_het_type = c("sd", "var", "prec"),
  prior_reg = .default(normal(scale = 10)),
  prior_aux = .default(),
  prior_aux_reg = .default(),
  prior_class_mean = .default(normal(scale = 10)),
  prior_class_sd = .default(half_normal(scale = 5)),
  aux_by = NULL,
  aux_regression = NULL,
  QR = FALSE,
  center = TRUE,
  adapt_delta = NULL,
  int_thin = 0,
  int_check = TRUE,
  mspline_degree = 3,
  n_knots = 7,
  knots = NULL,
  mspline_basis = NULL
)
}
\arguments{
\item{network}{An \code{nma_data} object, as created by the functions \verb{set_*()},
\code{combine_network()}, or \code{add_integration()}}

\item{consistency}{Character string specifying the type of (in)consistency
model to fit, either \code{"consistency"}, \code{"ume"}, or \code{"nodesplit"}}

\item{trt_effects}{Character string specifying either \code{"fixed"} or \code{"random"}
effects}

\item{regression}{A one-sided model formula, specifying the prognostic and
effect-modifying terms for a regression model. Any references to treatment
should use the \code{.trt} special variable, for example specifying effect
modifier interactions as \code{variable:.trt} (see details). The special
variable \code{.mu} can be used for baseline risk meta-regression: \code{~.mu:.trt}.}

\item{class_interactions}{Character string specifying whether effect modifier
interactions are specified as \code{"common"}, \code{"exchangeable"}, or
\code{"independent"}.}

\item{class_effects}{Character string specifying a model for treatment class effects,
either \code{"independent"} (the default), \code{"exchangeable"}, or \code{"common"}.}

\item{class_sd}{Character string specifying whether the class standard deviations in a
class effects model should be \code{"independent"} (i.e. separate for each class, the default),
or \code{"common"} (i.e. shared across all classes). Alternatively this can be a list of
character vectors, each of which describe a set classes for which to share a common class SD;
any list names will be used to name the output parameters, otherwise the name will be taken
from the first class in each set.}

\item{likelihood}{Character string specifying a likelihood, if unspecified
will be inferred from the data (see details)}

\item{link}{Character string specifying a link function, if unspecified will
default to the canonical link (see details)}

\item{...}{Further arguments passed to
\code{\link[rstan:stanmodel-method-sampling]{sampling()}}, such as \code{iter},
\code{chains}, \code{cores}, etc.}

\item{nodesplit}{For \code{consistency = "nodesplit"}, the comparison(s) to split
in the node-splitting model(s). Either a length 2 vector giving the
treatments in a single comparison, or a 2 column data frame listing
multiple treatment comparisons to split in turn. By default, all possible
comparisons will be chosen (see \code{\link[=get_nodesplits]{get_nodesplits()}}).}

\item{prior_intercept}{Specification of prior distribution for the intercept}

\item{prior_trt}{Specification of prior distribution for the treatment
effects}

\item{prior_het}{Specification of prior distribution for the heterogeneity
(if \code{trt_effects = "random"})}

\item{prior_het_type}{Character string specifying whether the prior
distribution \code{prior_het} is placed on the heterogeneity standard deviation
\eqn{\tau} (\code{"sd"}, the default), variance \eqn{\tau^2} (\code{"var"}), or
precision \eqn{1/\tau^2} (\code{"prec"}).}

\item{prior_reg}{Specification of prior distribution for the regression
coefficients (if \code{regression} formula specified)}

\item{prior_aux}{Specification of prior distribution for the auxiliary
parameter, if applicable (see details). For \code{likelihood = "gengamma"} this
should be a list of prior distributions with elements \code{sigma} and \code{k}.}

\item{prior_aux_reg}{Specification of prior distribution for the auxiliary
regression parameters, if \code{aux_regression} is specified (see details).}

\item{prior_class_mean}{Specification of prior distribution for the
treatment class means (if \code{class_effects = "exchangeable"}).}

\item{prior_class_sd}{Specification of prior distribution for the
treatment class standard deviations (if \code{class_effects = "exchangeable"}).}

\item{aux_by}{Vector of variable names listing the variables to stratify the
auxiliary parameters by. Currently only used for survival models, see
details. Cannot be used with \code{aux_regression}.}

\item{aux_regression}{A one-sided model formula giving a regression model for
the auxiliary parameters. Currently only used for survival models, see
details. Cannot be used with \code{aux_by}.}

\item{QR}{Logical scalar (default \code{FALSE}), whether to apply a QR
decomposition to the model design matrix}

\item{center}{Logical scalar (default \code{TRUE}), whether to center the
(numeric) regression terms about the overall means}

\item{adapt_delta}{See \link{adapt_delta} for details}

\item{int_thin}{A single integer value, the thinning factor for returning
cumulative estimates of integration error. Saving cumulative estimates is
disabled by \code{int_thin = 0}, which is the default.}

\item{int_check}{Logical, check sufficient accuracy of numerical integration
by fitting half of the chains with \code{n_int/2}? When \code{TRUE}, \code{Rhat} and
\code{n_eff} diagnostic warnings will be given if numerical integration has not
sufficiently converged (suggesting increasing \code{n_int} in
\code{\link[=add_integration]{add_integration()}}). Default \code{TRUE}, but disabled (\code{FALSE}) when
\code{int_thin > 0}.}

\item{mspline_degree}{Non-negative integer giving the degree of the M-spline
polynomial for \code{likelihood = "mspline"}. Piecewise exponential hazards
(\code{likelihood = "pexp"}) are a special case with \code{mspline_degree = 0}.}

\item{n_knots}{For \code{mspline} and \code{pexp} likelihoods, a non-negative integer
giving the number of internal knots for partitioning the baseline hazard
into intervals. The knot locations within each study will be determined by
the corresponding quantiles of the observed event times, plus boundary
knots at the earliest entry time (0 with no delayed entry) and the maximum
event/censoring time. For example, with \code{n_knots = 3}, the internal knot
locations will be at the 25\%, 50\%, and 75\% quantiles of the observed event
times. The default is \code{n_knots = 7}; overfitting is avoided by shrinking
towards a constant hazard with a random walk prior (see details). If
\code{aux_regression} is specified then a single set of knot locations will be
calculated across all studies in the network. See \code{\link[=make_knots]{make_knots()}} for more
details on the knot positioning algorithms. Ignored when \code{knots} is
specified.}

\item{knots}{For \code{mspline} and \code{pexp} likelihoods, a named list of numeric
vectors of knot locations (including boundary knots) for each of the
studies in the network. Currently, each vector must have the same length
(i.e. each study must use the same number of knots). Alternatively, a
single numeric vector of knot locations can be provided which will be
shared across all studies in the network. If unspecified (the default), the
knots will be chosen based on \code{n_knots} as described above. If
\code{aux_regression} is specified then \code{knots} should be a single numeric
vector of knot locations which will be shared across all studies in the
network. \code{\link[=make_knots]{make_knots()}} can be used to help specify \code{knots} directly, or to
investigate knot placement prior to model fitting.}

\item{mspline_basis}{Instead of specifying \code{mspline_degree} and \code{n_knots} or
\code{knots}, a named list of M-spline bases (one for each study) can be
provided with \code{mspline_basis} which will be used directly. In this case,
all other M-spline options will be ignored.}
}
\value{
\code{nma()} returns a \link{stan_nma} object, except when \code{consistency = "nodesplit"} when a \link{nma_nodesplit} or \link{nma_nodesplit_df} object is
returned. \code{nma.fit()} returns a \code{\link[rstan:stanfit-class]{stanfit}}
object.
}
\description{
The \code{nma} function fits network meta-analysis and (multilevel) network
meta-regression models in Stan.
}
\details{
When specifying a model formula in the \code{regression} argument, the
usual formula syntax is available (as interpreted by \code{\link[=model.matrix]{model.matrix()}}). The
only additional requirement here is that the special variable \code{.trt} should
be used to refer to treatment. For example, effect modifier interactions
should be specified as \code{variable:.trt}. Prognostic (main) effects and
interactions can be included together compactly as \code{variable*.trt}, which
expands to \code{variable + variable:.trt} (plus \code{.trt}, which is already in the
NMA model).

For the advanced user, the additional specials \code{.study} and \code{.trtclass} are
also available, and refer to studies and (if specified) treatment classes
respectively. When node-splitting models are fitted (\code{consistency = "nodesplit"}) the special \code{.omega} is available, indicating the arms to
which the node-splitting inconsistency factor is added.

See \code{\link[multinma:priors]{?priors}} for details on prior
specification. Default prior distributions are available, but may not be
appropriate for the particular setting and will raise a warning if used. No
attempt is made to tailor these defaults to the data provided. Please
consider appropriate prior distributions for the particular setting,
accounting for the scales of outcomes and covariates, etc. The function
\code{\link[=plot_prior_posterior]{plot_prior_posterior()}} may be useful in examining the influence of the
chosen prior distributions on the posterior distributions, and the
\code{\link[multinma:summary.nma_prior]{summary()}} method for \code{nma_prior}
objects prints prior intervals.
}
\section{Likelihoods and link functions}{

Currently, the following likelihoods and link functions are supported for
each data type:\tabular{lll}{
   \strong{Data type} \tab \strong{Likelihood} \tab \strong{Link function} \cr
   \strong{Binary} \tab \code{bernoulli}, \code{bernoulli2} \tab \code{logit}, \code{probit}, \code{cloglog} \cr
   \strong{Count} \tab \code{binomial}, \code{binomial2} \tab \code{logit}, \code{probit}, \code{cloglog} \cr
   \strong{Rate} \tab \code{poisson} \tab \code{log} \cr
   \strong{Continuous} \tab \code{normal} \tab \code{identity}, \code{log} \cr
   \strong{Ordered} \tab \code{ordered} \tab \code{logit}, \code{probit}, \code{cloglog} \cr
   \strong{Survival} \tab \code{exponential}, \code{weibull}, \code{gompertz}, \code{exponential-aft}, \code{weibull-aft}, \code{lognormal}, \code{loglogistic}, \code{gamma}, \code{gengamma}, \code{mspline}, \code{pexp} \tab \code{log} \cr
}


The \code{bernoulli2} and \code{binomial2} likelihoods correspond to a two-parameter
Binomial likelihood for arm-based AgD, which more closely matches the
underlying Poisson Binomial distribution for the summarised aggregate
outcomes in a ML-NMR model than the typical (one parameter) Binomial
distribution \insertCite{@see @methods_paper}{multinma}.

When a \code{cloglog} link is used, including an offset for log follow-up time
(i.e. \code{regression = ~offset(log(time))}) results in a model on the log
hazard \insertCite{@see @TSD2}{multinma}.

For survival data, all accelerated failure time models (\code{exponential-aft},
\code{weibull-aft}, \code{lognormal}, \code{loglogistic}, \code{gamma}, \code{gengamma}) are
parameterised so that the treatment effects and any regression parameters
are log Survival Time Ratios (i.e. a coefficient of \eqn{\log(2)} means
that the treatment or covariate is associated with a doubling of expected
survival time). These can be converted to log Acceleration Factors using
the relation \eqn{\log(\mathrm{AF}) = -\log(\mathrm{STR})} (or equivalently
\eqn{\mathrm{AF} = 1/\mathrm{STR}}).

Further details on each likelihood and link function are given by
\insertCite{TSD2;textual}{multinma}. Details of survival models are given
by \insertCite{Phillippo_survival;textual}{multinma}.
}

\section{Auxiliary parameters}{

Auxiliary parameters are only present in the following models.
\subsection{Normal likelihood with IPD}{

When a Normal likelihood is fitted to IPD, the auxiliary parameters are the
arm-level standard deviations \eqn{\sigma_{jk}} on treatment \eqn{k} in
study \eqn{j}.
}

\subsection{Ordered multinomial likelihood}{

When fitting a model to \eqn{M} ordered outcomes, the auxiliary parameters
are the latent cutoffs between each category, \eqn{c_0 < c_1 < \dots <
  c_M}. Only \eqn{c_2} to \eqn{c_{M-1}} are estimated; we fix \eqn{c_0 =
  -\infty}, \eqn{c_1 = 0}, and \eqn{c_M = \infty}. When specifying priors for
these latent cutoffs, we choose to specify priors on the \emph{differences}
\eqn{c_{m+1} - c_m}. Stan automatically truncates any priors so that the
ordering constraints are satisfied.
}

\subsection{Survival (time-to-event) likelihoods}{

All survival likelihoods except the \code{exponential} and \code{exponential-aft}
likelihoods have auxiliary parameters. These are typically study-specific
shape parameters \eqn{\gamma_j>0}, except for the \code{lognormal} likelihood
where the auxiliary parameters are study-specific standard deviations on
the log scale \eqn{\sigma_j>0}.

The \code{gengamma} likelihood has two sets of auxiliary parameters,
study-specific scale parameters \eqn{\sigma_j>0} and shape parameters
\eqn{k_j}, following the parameterisation of
\insertCite{Lawless1980;textual}{multinma}, which permits a range of
behaviours for the baseline hazard including increasing, decreasing,
bathtub and arc-shaped hazards. This parameterisation is related to that
discussed by \insertCite{Cox2007;textual}{multinma} and implemented in the
\code{flexsurv} package with \eqn{Q = k^{-0.5}}. The parameterisation used here
effectively bounds the shape parameter \eqn{k} away from numerical
instabilities as \eqn{k \rightarrow \infty} (i.e. away from \eqn{Q
  \rightarrow 0}, the log-Normal distribution) via the prior distribution.
Implicitly, this parameterisation is restricted to \eqn{Q > 0} and so
certain survival distributions like the inverse-Gamma and inverse-Weibull
are not part of the parameter space; however, \eqn{Q > 0} still encompasses
the other survival distributions implemented in this package.

For the \code{mspline} and \code{pexp} likelihoods, the auxiliary parameters are the
spline coefficients for each study. These form a unit simplex (i.e. lie
between 0 and 1, and sum to 1), and are given a random walk prior
distribution \insertCite{Phillippo_mspline}{multinma}. \code{prior_aux}
specifies the hyperprior on the random walk standard deviation \eqn{\sigma}
which controls the level of smoothing of the baseline hazard, with
\eqn{\sigma = 0} corresponding to a constant baseline hazard.

The auxiliary parameters can be stratified by additional factors through
the \code{aux_by} argument. For example, to allow the shape of the baseline
hazard to vary between treatment arms as well as studies, use \code{aux_by = c(".study", ".trt")}. (Technically, \code{.study} is always included in the
stratification even if omitted from \code{aux_by}, but we choose here to make
the stratification explicit.) This is a common way of relaxing the
proportional hazards assumption. The default is equivalent to \code{aux_by = ".study"} which stratifies the auxiliary parameters by study, as described
above.

A regression model may be specified on the auxiliary parameters using
\code{aux_regression}. This is useful if we wish to model departures from
non-proportionality, rather than allowing the baseline hazards to be
completely independent using \code{aux_by}. This is necessary if absolute
predictions (e.g. survival curves) are required in a population for
unobserved combinations of covariates; for example, if \code{aux_by = .trt} then
absolute predictions may only be produced for the observed treatment arms
in each study population, whereas if \code{aux_regression = ~.trt} then absolute
predictions can be produced for all treatments in any population. For
\code{mspline} and \code{pexp} likelihoods, the regression coefficients are smoothed
over time using a random walk prior to avoid overfitting
\insertCite{Phillippo_mspline}{multinma}: \code{prior_aux_reg} specifies the
hyperprior for the random walk standard deviation. For other parametric
likelihoods, \code{prior_aux_reg} specifies the prior for the auxiliary
regression coefficients.
}
}

\examples{
## Smoking cessation NMA
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net

\donttest{
# Fitting a fixed effect model
smk_fit_FE <- nma(smk_net,
                  trt_effects = "fixed",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100))

smk_fit_FE
}

\donttest{
# Fitting a random effects model
smk_fit_RE <- nma(smk_net, 
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5))

smk_fit_RE
}

\donttest{
# Fitting an unrelated mean effects (inconsistency) model
smk_fit_RE_UME <- nma(smk_net, 
                      consistency = "ume",
                      trt_effects = "random",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_het = normal(scale = 5))

smk_fit_RE_UME
}

\donttest{
# Fitting all possible node-splitting models
smk_fit_RE_nodesplit <- nma(smk_net, 
                            consistency = "nodesplit",
                            trt_effects = "random",
                            prior_intercept = normal(scale = 100),
                            prior_trt = normal(scale = 100),
                            prior_het = normal(scale = 5))
}

\donttest{
# Summarise the node-splitting results
summary(smk_fit_RE_nodesplit)
}

## Plaque psoriasis ML-NMR
# Set up plaque psoriasis network combining IPD and AgD
library(dplyr)
pso_ipd <- filter(plaque_psoriasis_ipd,
                  studyc \%in\% c("UNCOVER-1", "UNCOVER-2", "UNCOVER-3"))

pso_agd <- filter(plaque_psoriasis_agd,
                  studyc == "FIXTURE")

head(pso_ipd)
head(pso_agd)

pso_ipd <- pso_ipd \%>\%
  mutate(# Variable transformations
    bsa = bsa / 100,
    prevsys = as.numeric(prevsys),
    psa = as.numeric(psa),
    weight = weight / 10,
    durnpso = durnpso / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker"),
    # Check complete cases for covariates of interest
    complete = complete.cases(durnpso, prevsys, bsa, weight, psa)
  )

pso_agd <- pso_agd \%>\%
  mutate(
    # Variable transformations
    bsa_mean = bsa_mean / 100,
    bsa_sd = bsa_sd / 100,
    prevsys = prevsys / 100,
    psa = psa / 100,
    weight_mean = weight_mean / 10,
    weight_sd = weight_sd / 10,
    durnpso_mean = durnpso_mean / 10,
    durnpso_sd = durnpso_sd / 10,
    # Treatment classes
    trtclass = case_when(trtn == 1 ~ "Placebo",
                         trtn \%in\% c(2, 3, 5, 6) ~ "IL blocker",
                         trtn == 4 ~ "TNFa blocker")
  )

# Exclude small number of individuals with missing covariates
pso_ipd <- filter(pso_ipd, complete)

pso_net <- combine_network(
  set_ipd(pso_ipd,
          study = studyc,
          trt = trtc,
          r = pasi75,
          trt_class = trtclass),
  set_agd_arm(pso_agd,
              study = studyc,
              trt = trtc,
              r = pasi75_r,
              n = pasi75_n,
              trt_class = trtclass)
)

# Print network details
pso_net

# Add integration points to the network
pso_net <- add_integration(pso_net,
  durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
  prevsys = distr(qbern, prob = prevsys),
  bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
  weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
  psa = distr(qbern, prob = psa),
  n_int = 64)

\donttest{
# Fitting a ML-NMR model.
# Specify a regression model to include effect modifier interactions for five
# covariates, along with main (prognostic) effects. We use a probit link and
# specify that the two-parameter Binomial approximation for the aggregate-level
# likelihood should be used. We set treatment-covariate interactions to be equal
# within each class. We narrow the possible range for random initial values with
# init_r = 0.1, since probit models in particular are often hard to initialise.
# Using the QR decomposition greatly improves sampling efficiency here, as is
# often the case for regression models.
pso_fit <- nma(pso_net, 
               trt_effects = "fixed",
               link = "probit",
               likelihood = "bernoulli2",
               regression = ~(durnpso + prevsys + bsa + weight + psa)*.trt,
               class_interactions = "common",
               prior_intercept = normal(scale = 10),
               prior_trt = normal(scale = 10),
               prior_reg = normal(scale = 10),
               init_r = 0.1,
               QR = TRUE)
pso_fit
}

## Newly-diagnosed multiple myeloma NMA
# Set up newly-diagnosed multiple myeloma network

head(ndmm_ipd)
head(ndmm_agd)

ndmm_net <- combine_network(
  set_ipd(ndmm_ipd,
          study, trt,
          Surv = Surv(eventtime / 12, status)),
  set_agd_surv(ndmm_agd,
               study, trt,
               Surv = Surv(eventtime / 12, status),
               covariates = ndmm_agd_covs))
\donttest{
# Fit Weibull (PH) model
ndmm_fit <- nma(ndmm_net, 
                likelihood = "weibull",
                prior_intercept = normal(scale = 100),
                prior_trt = normal(scale = 10),
                prior_aux = half_normal(scale = 10))

ndmm_fit
}
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{distr}
\alias{distr}
\title{Specify a general marginal distribution}
\usage{
distr(qfun, ...)
}
\arguments{
\item{qfun}{an inverse CDF, either as a function name or a string}

\item{...}{parameters of the distribution as arguments to \code{qfun}, these will
be quoted and evaluated later in the context of the aggregate data sources}
}
\value{
An object of class \link{distr}.
}
\description{
\code{distr()} is used within the function \code{\link[=add_integration]{add_integration()}} to specify marginal
distributions for the covariates, via a corresponding inverse CDF. It is also
used in \code{\link[=predict.stan_nma]{predict.stan_nma()}} to specify a distribution for the baseline
response (intercept) when predicting absolute outcomes.
}
\details{
The function \code{qfun} should have a formal argument called \code{p}. This
restriction serves as a crude check for inverse CDFs (e.g. an error will be
given if \code{dnorm} is used instead of \code{qnorm}). If a user-written CDF is
supplied, it must have an argument \code{p} which takes a vector of
probabilities.
}
\examples{
## Specifying marginal distributions for integration

df <- data.frame(x1_mean = 2, x1_sd = 0.5, x2 = 0.8)

# Distribution parameters are evaluated in the context of the data frame
add_integration(df,
                x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
                x2 = distr(qbern, prob = x2),
                cor = diag(2))

}
\seealso{
\code{\link[=add_integration]{add_integration()}} where \code{distr()} is used to specify marginal
distributions for covariates to integrate over, and \code{\link[=predict.stan_nma]{predict.stan_nma()}}
where \code{distr()} is used to specify a distribution on the baseline response.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{as.igraph.nma_data}
\alias{as.igraph.nma_data}
\alias{as_tbl_graph.nma_data}
\title{Convert networks to graph objects}
\usage{
\method{as.igraph}{nma_data}(x, ..., collapse = TRUE)

\method{as_tbl_graph}{nma_data}(x, ...)
}
\arguments{
\item{x}{An \link{nma_data} object to convert}

\item{...}{Additional arguments}

\item{collapse}{Logical, collapse edges over studies? Default \code{TRUE}, only
one edge is produced for each comparison (by IPD or AgD study type) with a
\code{.nstudy} attribute giving the number of studies making that comparison. If
\code{FALSE}, repeated edges are added for each study making the comparison.}
}
\value{
An \code{igraph} object for \code{as.igraph()}, a \code{tbl_graph} object for
\code{as_tbl_graph()}.
}
\description{
The method \code{as.igraph()} converts \code{nma_data} objects into the form used by
the \link[igraph:igraph-package]{igraph} package. The method \code{as_tbl_graph()}
converts \code{nma_data} objects into the form used by the
\link[ggraph:ggraph-package]{ggraph} and
\link[tidygraph:tidygraph-package]{tidygraph} packages.
}
\examples{
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net

# Convert to igraph object
igraph::as.igraph(smk_net)  # Edges combined by default
igraph::as.igraph(smk_net, collapse = FALSE)  # Without combining edges

# Convert to tbl_graph object
tidygraph::as_tbl_graph(smk_net)  # Edges combined by default
tidygraph::as_tbl_graph(smk_net, collapse = FALSE)  # Without combining edges
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{transfusion}
\alias{transfusion}
\title{Granulocyte transfusion in patients with neutropenia or neutrophil
dysfunction}
\format{
A data frame with 12 rows and 4 variables:
\describe{
\item{studyc}{study name}
\item{trtc}{treatment name}
\item{r}{total number of deaths}
\item{n}{total number of individuals}
}
}
\usage{
transfusion
}
\description{
Data frame containing the number of deaths in 6 trials comparing transfusion
of granulocytes (white blood cells) to control
\insertCite{Stanworth2005}{multinma}. Previously used to demonstrate
informative prior distributions for the heterogeneity variance by
\insertCite{Turner2012;textual}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{qbern}
\alias{qbern}
\alias{pbern}
\alias{dbern}
\title{The Bernoulli Distribution}
\usage{
qbern(p, prob, lower.tail = TRUE, log.p = FALSE)

pbern(q, prob, lower.tail = TRUE, log.p = FALSE)

dbern(x, prob, log = FALSE)
}
\arguments{
\item{p}{vector of probabilities}

\item{prob}{probability of success}

\item{lower.tail, log.p, log}{see \link[stats:Binomial]{stats::Binomial}}

\item{x, q}{vector of quantiles}
}
\value{
Numeric vector of length equal to the maximum of the lengths of the
input arguments.
}
\description{
The density function \code{dbern()}, distribution function \code{pbern()}, and quantile
function \code{qbern()} for a Bernoulli distribution, with success probability
\code{prob}. These are equivalent to \code{dbinom(p, 1, prob)}, \code{pbinom(p, 1, prob)}
and \code{qbinom(p, 1, prob)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_summary-class.R
\name{print.nma_summary}
\alias{print.nma_summary}
\alias{as.data.frame.nma_summary}
\alias{as.tibble.nma_summary}
\alias{as_tibble.nma_summary}
\alias{as.array.nma_summary}
\alias{as.matrix.nma_summary}
\alias{as.array.nma_rank_probs}
\alias{as.matrix.nma_rank_probs}
\title{Methods for \code{nma_summary} objects}
\usage{
\method{print}{nma_summary}(x, ..., digits = 2, pars, include = TRUE)

\method{as.data.frame}{nma_summary}(x, ...)

\method{as.tibble}{nma_summary}(x, ...)

\method{as_tibble}{nma_summary}(x, ...)

\method{as.array}{nma_summary}(x, ...)

\method{as.matrix}{nma_summary}(x, ...)

\method{as.array}{nma_rank_probs}(x, ...)

\method{as.matrix}{nma_rank_probs}(x, ...)
}
\arguments{
\item{x}{A \code{nma_summary} object}

\item{...}{Additional arguments passed on to other methods}

\item{digits}{Integer number of digits to display}

\item{pars}{Character vector of parameters to display in the printed summary}

\item{include}{Logical, are parameters named in \code{pars} included (\code{TRUE}) or excluded (\code{FALSE})}
}
\value{
A \code{data.frame} for \code{as.data.frame()}, a \code{tbl_df} for \code{as.tibble()}
and \code{as_tibble()}, a \code{matrix} for \code{as.matrix()}, and an \code{mcmc_array} for
\code{as.array()}.

The \code{print()} method returns \code{x} invisibly.
}
\description{
The \code{as.data.frame()}, \code{as_tibble()}, and \code{as.tibble()} methods return the
posterior summary statistics in a data frame or tibble. The \code{as.matrix()}
method returns a matrix of posterior draws. The \code{as.array()} method returns a
3D array [Iteration, Chain, Parameter] of posterior draws (as class
\link{mcmc_array}).
}
\seealso{
\code{\link[=plot.nma_summary]{plot.nma_summary()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic-class.R
\name{print.nma_dic}
\alias{print.nma_dic}
\alias{as.data.frame.nma_dic}
\alias{as.tibble.nma_dic}
\alias{as_tibble.nma_dic}
\alias{as.array.nma_dic}
\alias{as.matrix.nma_dic}
\title{Methods for \code{nma_dic} objects}
\usage{
\method{print}{nma_dic}(x, digits = 1, ...)

\method{as.data.frame}{nma_dic}(x, ...)

\method{as.tibble}{nma_dic}(x, ...)

\method{as_tibble}{nma_dic}(x, ...)

\method{as.array}{nma_dic}(x, ...)

\method{as.matrix}{nma_dic}(x, ...)
}
\arguments{
\item{x}{An object of class \link{nma_dic}}

\item{digits}{Integer number of digits to display}

\item{...}{Additional arguments passed on to other methods}
}
\value{
A \code{data.frame} for \code{as.data.frame()}, a \code{tbl_df} for \code{as.tibble()}
and \code{as_tibble()}, a \code{matrix} for \code{as.matrix()}, and an \code{mcmc_array} for
\code{as.array()}.

The \code{print()} method returns \code{x} invisibly.
}
\description{
The \code{print()} method prints details of DIC model fit statistics, computed by
the  \code{\link[=dic]{dic()}} function. The \code{as.data.frame()}, \code{as_tibble()}, and \code{as.tibble()} methods return the
pointwise contributions to the DIC and $p_D$ in a data frame or tibble. The \code{as.array()} and \code{as.matrix()}
methods returns a 3D MCMC array (as class \link{mcmc_array}) or matrix of posterior draws of the residual deviances.
}
\seealso{
\code{\link[=dic]{dic()}}, \code{\link[=plot.nma_dic]{plot.nma_dic()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ndmm_ipd}
\alias{ndmm_ipd}
\alias{ndmm_agd}
\alias{ndmm_agd_covs}
\title{Newly diagnosed multiple myeloma}
\format{
The individual patient data are contained in a data frame \code{ndmm_ipd}
with 1325 rows, one per individual, and 10 variables:
\describe{
\item{study, studyf}{study name}
\item{trt, trtf}{treatment name}
\item{eventtime}{event/censoring time}
\item{status}{censoring indicator (0 = censored, 1 = event)}
\item{age}{age (years)}
\item{iss_stage3}{ISS stage 3 (0 = no, 1 = yes)}
\item{response_cr_vgpr}{complete or very good partial response (0 = no, 1 = yes)}
\item{male}{male sex (0 = no, 1 = yes)}
}

The reconstructed Kaplan-Meier data for the aggregate studies are
contained in a data frame \code{ndmm_agd} with 2819 rows and 6 variables:
\describe{
\item{study, studyf}{study name}
\item{trt, trtf}{treatment name}
\item{eventtime}{event/censoring time}
\item{status}{censoring indicator (0 = censored, 1 = event)}
}

The covariate summaries extracted from published reportes for the
aggregate studies are contained in a data frame \code{ndmm_agd_covs} with 4
rows, one per study arm, and 15 columns:
\describe{
\item{study, studyf}{study name}
\item{trt, trtf}{treatment name}
\item{sample_size}{sample size in each arm}
\item{age_min, age_iqr_l, age_median, age_iqr_h, age_max, age_mean, age_sd}{summary statistics for age (years)}
\item{iss_stage3}{proportion of participants with ISS stage 3}
\item{response_cr_vgpr}{proportion of participants with complete or very good partial response}
\item{male}{proportion of male participants}
}
}
\usage{
ndmm_ipd

ndmm_agd

ndmm_agd_covs
}
\description{
Three data frames, \code{ndmm_ipd}, \code{ndmm_agd}, and \code{ndmm_agd_covs} containing
(simulated) individual patient data (IPD) from three studies and aggregate
data (AgD) from two studies on newly diagnosed multiple myeloma. The outcome
of interest is progression-free survival after autologous stem cell
transplant. The IPD studies in \code{ndmm_ipd} provide event/censoring times and
covariate values for each individual. The AgD studies provide reconstructed
event/censoring times from digitized Kaplan-Meier curves in \code{ndmm_agd} and
covariate summaries in \code{ndmm_agd_covs}, obtained from published trial
reports. The data are constructed to resemble those used by
\insertCite{Leahy2019;textual}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marginal_effects.R
\name{marginal_effects}
\alias{marginal_effects}
\title{Marginal treatment effects}
\usage{
marginal_effects(
  object,
  ...,
  mtype = c("difference", "ratio", "link"),
  all_contrasts = FALSE,
  trt_ref = NULL,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
  predictive_distribution = FALSE,
  summary = TRUE
)
}
\arguments{
\item{object}{A \code{stan_nma} object created by \code{\link[=nma]{nma()}}.}

\item{...}{Arguments passed to \code{\link[=predict.stan_nma]{predict.stan_nma()}}, for example to specify
the covariate distribution and baseline risk for a target population, e.g.
\code{newdata}, \code{baseline}, and related arguments. For survival outcomes, \code{type}
can also be specified to determine the quantity from which to form a
marginal effect. For example, \code{type = "hazard"} with \code{mtype = "ratio"}
produces marginal hazard ratios, \code{type = "median"} with \code{mtype = "difference"} produces marginal median survival time differences, and so
on.}

\item{mtype}{The type of marginal effect to construct from the average
absolute effects, either \code{"difference"} (the default) for a difference of
absolute effects such as a risk difference, \code{"ratio"} for a ratio of
absolute effects such as a risk ratio, or \code{"link"} for a difference on the
scale of the link function used in fitting the model such as a marginal log
odds ratio.}

\item{all_contrasts}{Logical, generate estimates for all contrasts (\code{TRUE}),
or just the "basic" contrasts against the network reference treatment
(\code{FALSE})? Default \code{FALSE}.}

\item{trt_ref}{Reference treatment to construct relative effects against, if
\code{all_contrasts = FALSE}. By default, relative effects will be against the
network reference treatment. Coerced to character string.}

\item{probs}{Numeric vector of quantiles of interest to present in computed
summary, default \code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{predictive_distribution}{Logical, when a random effects model has been
fitted, should the predictive distribution for marginal effects in a new
study be returned? Default \code{FALSE}.}

\item{summary}{Logical, calculate posterior summaries? Default \code{TRUE}.}
}
\value{
A \link{nma_summary} object if \code{summary = TRUE}, otherwise a list
containing a 3D MCMC array of samples and (for regression models) a data
frame of study information.
}
\description{
Generate population-average marginal treatment effects. These are formed from
population-average absolute predictions, so this function is a wrapper around
\code{\link[=predict.stan_nma]{predict.stan_nma()}}.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Marginal risk difference in each study population in the network
marginal_effects(smk_fit_RE, mtype = "difference")

# Since there are no covariates in the model, the marginal and conditional
# (log) odds ratios here coincide
marginal_effects(smk_fit_RE, mtype = "link")
relative_effects(smk_fit_RE)

# Marginal risk differences in a population with 67 observed events out of
# 566 individuals on No Intervention, corresponding to a Beta(67, 566 - 67)
# distribution on the baseline probability of response
(smk_rd_RE <- marginal_effects(smk_fit_RE,
                               baseline = distr(qbeta, 67, 566 - 67),
                               baseline_type = "response",
                               mtype = "difference"))
plot(smk_rd_RE)
}

## Plaque psoriasis ML-NMR
\donttest{
# Run plaque psoriasis ML-NMR example if not already available
if (!exists("pso_fit")) example("example_pso_mlnmr", run.donttest = TRUE)
}
\donttest{
# Population-average marginal probit differences in each study in the network
(pso_marg <- marginal_effects(pso_fit, mtype = "link"))
plot(pso_marg, ref_line = c(0, 1))

# Population-average marginal probit differences in a new target population,
# with means and SDs or proportions given by
new_agd_int <- data.frame(
  bsa_mean = 0.6,
  bsa_sd = 0.3,
  prevsys = 0.1,
  psa = 0.2,
  weight_mean = 10,
  weight_sd = 1,
  durnpso_mean = 3,
  durnpso_sd = 1
)

# We need to add integration points to this data frame of new data
# We use the weighted mean correlation matrix computed from the IPD studies
new_agd_int <- add_integration(new_agd_int,
                               durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                               prevsys = distr(qbern, prob = prevsys),
                               bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
                               weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
                               psa = distr(qbern, prob = psa),
                               cor = pso_net$int_cor,
                               n_int = 64)

# Population-average marginal probit differences of achieving PASI 75 in this
# target population, given a Normal(-1.75, 0.08^2) distribution on the
# baseline probit-probability of response on Placebo (at the reference levels
# of the covariates), are given by
(pso_marg_new <- marginal_effects(pso_fit,
                                  mtype = "link",
                                  newdata = new_agd_int,
                                  baseline = distr(qnorm, -1.75, 0.08)))
plot(pso_marg_new)
}

## Progression free survival with newly-diagnosed multiple myeloma
\donttest{
# Run newly-diagnosed multiple myeloma example if not already available
if (!exists("ndmm_fit")) example("example_ndmm", run.donttest = TRUE)
}
\donttest{
# We can produce a range of marginal effects from models with survival
# outcomes, specified with the mtype and type arguments. For example:

# Marginal survival probability difference at 5 years, all contrasts
marginal_effects(ndmm_fit, type = "survival", mtype = "difference",
                 times = 5, all_contrasts = TRUE)

# Marginal difference in RMST up to 5 years
marginal_effects(ndmm_fit, type = "rmst", mtype = "difference", times = 5)

# Marginal median survival time ratios
marginal_effects(ndmm_fit, type = "median", mtype = "ratio")

# Marginal log hazard ratios
# With no covariates in the model, these are constant over time and study
# populations, and are equal to the log hazard ratios from relative_effects()
plot(marginal_effects(ndmm_fit, type = "hazard", mtype = "link"),
     # The hazard is infinite at t=0 in some studies, giving undefined logHRs at t=0
     na.rm = TRUE)

# The NDMM vignette demonstrates the production of time-varying marginal
# hazard ratios from a ML-NMR model that includes covariates, see
# `vignette("example_ndmm")`

# Marginal survival difference over time
plot(marginal_effects(ndmm_fit, type = "survival", mtype = "difference"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{summary.stan_nma}
\alias{summary.stan_nma}
\alias{plot.stan_nma}
\title{Posterior summaries from \code{stan_nma} objects}
\usage{
\method{summary}{stan_nma}(object, ..., pars, include, probs = c(0.025, 0.25, 0.5, 0.75, 0.975))

\method{plot}{stan_nma}(
  x,
  ...,
  pars,
  include,
  stat = "pointinterval",
  orientation = c("horizontal", "vertical", "y", "x"),
  ref_line = NA_real_
)
}
\arguments{
\item{...}{Additional arguments passed on to other methods}

\item{pars, include}{See \code{\link[rstan:stanfit-method-extract]{rstan::extract()}}}

\item{probs}{Numeric vector of specifying quantiles of interest, default
\code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{x, object}{A \code{stan_nma} object}

\item{stat}{Character string specifying the \code{ggdist} plot stat to use,
default \code{"pointinterval"}}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}), default \code{"horizontal"}}

\item{ref_line}{Numeric vector of positions for reference lines, by default
no reference lines are drawn}

\item{summary}{Logical, calculate posterior summaries? Default \code{TRUE}.}
}
\value{
A \link{nma_summary} object
}
\description{
Posterior summaries of model parameters in \code{stan_nma} objects may be produced
using the \code{summary()} method and plotted with the \code{plot()} method. NOTE: To
produce relative effects, absolute predictions, or posterior ranks, see
\code{\link[=relative_effects]{relative_effects()}}, \code{\link[=predict.stan_nma]{predict.stan_nma()}}, \code{\link[=posterior_ranks]{posterior_ranks()}},
\code{\link[=posterior_rank_probs]{posterior_rank_probs()}}.
}
\details{
The \code{plot()} method is a shortcut for \code{plot(summary(stan_nma))}. For
details of plotting options, see \code{\link[=plot.nma_summary]{plot.nma_summary()}}.
}
\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Summary and plot of all model parameters
summary(smk_fit_RE)
plot(smk_fit_RE)

# Summary and plot of heterogeneity tau only
summary(smk_fit_RE, pars = "tau")
plot(smk_fit_RE, pars = "tau")

# Customising plot output
plot(smk_fit_RE,
     pars = c("d", "tau"),
     stat = "halfeye",
     ref_line = 0)
}
}
\seealso{
\code{\link[=plot.nma_summary]{plot.nma_summary()}}, \code{\link[=relative_effects]{relative_effects()}}, \code{\link[=predict.stan_nma]{predict.stan_nma()}},
\code{\link[=posterior_ranks]{posterior_ranks()}}, \code{\link[=posterior_rank_probs]{posterior_rank_probs()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc-examples.R
\name{example_smk_re}
\alias{example_smk_re}
\title{Example smoking RE NMA}
\description{
Calling \code{example("example_smk_re")} will run a random effects
NMA model with the smoking cessation data, using the code in the Examples
section below. The resulting \code{stan_nma} object \code{smk_fit_RE} will then be
available in the global environment.
}
\details{
Smoking RE NMA for use in examples.
}
\examples{
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net

\donttest{
# Fitting a random effects model
smk_fit_RE <- nma(smk_net, 
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5))

smk_fit_RE
}

\dontshow{
if (requireNamespace("pkgdown", quietly = TRUE) && pkgdown::in_pkgdown()) {
  assign("smk_net", smk_net, .GlobalEnv)
  assign("smk_fit_RE", smk_fit_RE, .GlobalEnv)
}
}
}
\keyword{examples}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{print.stan_nma}
\alias{print.stan_nma}
\title{Print \code{stan_nma} objects}
\usage{
\method{print}{stan_nma}(x, ...)
}
\arguments{
\item{x}{A \link{stan_nma} object}

\item{...}{Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print \code{stan_nma} objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{parkinsons}
\alias{parkinsons}
\title{Mean off-time reduction in Parkison's disease}
\format{
A data frame with 15 rows and 7 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code (placebo = 1)}
\item{y}{mean off-time reduction}
\item{se}{standard error}
\item{n}{sample size}
\item{diff}{mean difference vs. treatment in reference arm}
\item{se_diff}{standard error of mean difference, see details}
}
}
\usage{
parkinsons
}
\description{
Data frame containing the mean off-time reduction in patients given dopamine
agonists as adjunct therapy in Parkinson's disease, from 7 trials comparing
four active drugs and placebo \insertCite{TSD2}{multinma}.
}
\details{
This dataset may be analysed using either an arm-based likelihood
using \code{y} and \code{se}, or a contrast-based likelihood using \code{diff} and
\code{se_diff} (or a combination of the two across different studies).

The contrast-based data is formatted as described in \code{\link[=set_agd_contrast]{set_agd_contrast()}}.
That is, for the chosen reference arm in each study, the mean difference
\code{diff} is set to \code{NA}, and \code{se_diff} is set to the standard error \code{se} of
the outcome on the reference arm.
}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.stan_nma}
\alias{predict.stan_nma}
\alias{predict.stan_nma_surv}
\title{Predictions of absolute effects from NMA models}
\usage{
\method{predict}{stan_nma}(
  object,
  ...,
  baseline = NULL,
  newdata = NULL,
  study = NULL,
  type = c("link", "response"),
  level = c("aggregate", "individual"),
  baseline_trt = NULL,
  baseline_type = c("link", "response"),
  baseline_level = c("individual", "aggregate"),
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
  predictive_distribution = FALSE,
  expand = TRUE,
  summary = TRUE,
  progress = FALSE,
  trt_ref = NULL
)

\method{predict}{stan_nma_surv}(
  object,
  times = NULL,
  ...,
  baseline_trt = NULL,
  baseline = NULL,
  aux = NULL,
  newdata = NULL,
  study = NULL,
  type = c("survival", "hazard", "cumhaz", "mean", "median", "quantile", "rmst", "link"),
  quantiles = c(0.25, 0.5, 0.75),
  level = c("aggregate", "individual"),
  times_seq = NULL,
  probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
  predictive_distribution = FALSE,
  expand = TRUE,
  summary = TRUE,
  progress = interactive(),
  trt_ref = NULL
)
}
\arguments{
\item{object}{A \code{stan_nma} object created by \code{\link[=nma]{nma()}}.}

\item{...}{Additional arguments, passed to \code{\link[=uniroot]{uniroot()}} for regression models
if \code{baseline_level = "aggregate"}.}

\item{baseline}{An optional \code{\link[=distr]{distr()}} distribution for the baseline response
(i.e. intercept), about which to produce absolute effects. Can also be a
character string naming a study in the network to take the estimated
baseline response distribution from. If \code{NULL}, predictions are produced
using the baseline response for each study in the network with IPD or
arm-based AgD.

For regression models, this may be a list of \code{\link[=distr]{distr()}} distributions (or
study names in the network to use the baseline distributions from) of the
same length as the number of studies in \code{newdata}, possibly named by the
studies in \code{newdata} or otherwise in order of appearance in \code{newdata}.

Use the \code{baseline_type} and \code{baseline_level} arguments to specify whether
this distribution is on the response or linear predictor scale, and (for
ML-NMR or models including IPD) whether this applies to an individual at
the reference level of the covariates or over the entire \code{newdata}
population, respectively. For example, in a model with a logit link with
\code{baseline_type = "link"}, this would be a distribution for the baseline log
odds of an event. For survival models, \code{baseline} always corresponds to the
intercept parameters in the linear predictor (i.e. \code{baseline_type} is
always \code{"link"}, and \code{baseline_level} is \code{"individual"} for IPD NMA or
ML-NMR, and \code{"aggregate"} for AgD NMA).

Use the \code{baseline_trt} argument to specify which treatment this
distribution applies to.}

\item{newdata}{Only required if a regression model is fitted and \code{baseline}
is specified. A data frame of covariate details, for which to produce
predictions. Column names must match variables in the regression model.

If \code{level = "aggregate"} this should either be a data frame with integration
points as produced by \code{\link[=add_integration]{add_integration()}} (one row per study), or a data
frame with individual covariate values (one row per individual) which are
summarised over.

If \code{level = "individual"} this should be a data frame of individual
covariate values, one row per individual.

If \code{NULL}, predictions are produced for all studies with IPD and/or
arm-based AgD in the network, depending on the value of \code{level}.}

\item{study}{Column of \code{newdata} which specifies study names or IDs. When not
specified: if \code{newdata} contains integration points produced by
\code{\link[=add_integration]{add_integration()}}, studies will be labelled sequentially by row;
otherwise data will be assumed to come from a single study.}

\item{type}{Whether to produce predictions on the \code{"link"} scale (the
default, e.g. log odds) or \code{"response"} scale (e.g. probabilities).

For survival models, the options are \code{"survival"} for survival
probabilities (the default), \code{"hazard"} for hazards, \code{"cumhaz"} for
cumulative hazards, \code{"mean"} for mean survival times, \code{"quantile"} for
quantiles of the survival time distribution, \code{"median"} for median survival
times (equivalent to \code{type = "quantile"} with \code{quantiles = 0.5}), \code{"rmst"}
for restricted mean survival times, or \code{"link"} for the linear predictor.
For \code{type = "survival"}, \code{"hazard"} or \code{"cumhaz"}, predictions are given at
the times specified by \code{times} or at the event/censoring times in the
network if \code{times = NULL}. For \code{type = "rmst"}, the restricted time horizon
is specified by \code{times}, or if \code{times = NULL} the earliest last follow-up
time amongst the studies in the network is used. When \code{level = "aggregate"}, these all correspond to the standardised survival function
(see details).}

\item{level}{The level at which predictions are produced, either
\code{"aggregate"} (the default), or \code{"individual"}. If \code{baseline} is not
specified, predictions are produced for all IPD studies in the network if
\code{level} is \code{"individual"} or \code{"aggregate"}, and for all arm-based AgD
studies in the network if \code{level} is \code{"aggregate"}.}

\item{baseline_trt}{Treatment to which the \code{baseline} response distribution
refers, if \code{baseline} is specified. By default, the baseline response
distribution will refer to the network reference treatment. Coerced to
character string.}

\item{baseline_type}{When a \code{baseline} distribution is given, specifies
whether this corresponds to the \code{"link"} scale (the default, e.g. log odds)
or \code{"response"} scale (e.g. probabilities). For survival models, \code{baseline}
always corresponds to the intercept parameters in the linear predictor
(i.e. \code{baseline_type} is always \code{"link"}).}

\item{baseline_level}{When a \code{baseline} distribution is given, specifies
whether this corresponds to an individual at the reference level of the
covariates (\code{"individual"}, the default), or from an (unadjusted) average
outcome on the reference treatment in the \code{newdata} population
(\code{"aggregate"}). Ignored for AgD NMA, since the only option is
\code{"aggregate"} in this instance. For survival models, \code{baseline} always
corresponds to the intercept parameters in the linear predictor (i.e.
\code{baseline_level} is \code{"individual"} for IPD NMA or ML-NMR, and \code{"aggregate"}
for AgD NMA).}

\item{probs}{Numeric vector of quantiles of interest to present in computed
summary, default \code{c(0.025, 0.25, 0.5, 0.75, 0.975)}}

\item{predictive_distribution}{Logical, when a random effects model has been
fitted, should the predictive distribution for absolute effects in a new
study be returned? Default \code{FALSE}.}

\item{expand}{Logical, expand out predictions for every treatment (\code{TRUE}),
or only produce predictions for observed treatments (\code{FALSE}). Default
\code{TRUE}.}

\item{summary}{Logical, calculate posterior summaries? Default \code{TRUE}.}

\item{progress}{Logical, display progress for potentially long-running
calculations? Population-average predictions from ML-NMR models are
computationally intensive, especially for survival outcomes. Currently the
default is to display progress only when running interactively and
producing predictions for a survival ML-NMR model.}

\item{trt_ref}{Deprecated, renamed to \code{baseline_trt}.}

\item{times}{A numeric vector of times to evaluate predictions at.
Alternatively, if \code{newdata} is specified, \code{times} can be the name of a
column in \code{newdata} which contains the times. If \code{NULL} (the default) then
predictions are made at the event/censoring times from the studies included
in the network (or according to \code{times_seq}). Only used if \code{type} is
\code{"survival"}, \code{"hazard"}, \code{"cumhaz"} or \code{"rmst"}.}

\item{aux}{An optional \code{\link[=distr]{distr()}} distribution for the auxiliary parameter(s)
in the baseline hazard (e.g. shapes). Can also be a character string naming
a study in the network to take the estimated auxiliary parameter
distribution from. If \code{NULL}, predictions are produced using the parameter
estimates for each study in the network with IPD or arm-based AgD.

For regression models, this may be a list of \code{\link[=distr]{distr()}} distributions (or
study names in the network to use the auxiliary parameters from) of the
same length as the number of studies in \code{newdata}, possibly named by the
study names or otherwise in order of appearance in \code{newdata}.}

\item{quantiles}{A numeric vector of quantiles of the survival time
distribution to produce estimates for when \code{type = "quantile"}.}

\item{times_seq}{A positive integer, when specified evaluate predictions at
this many evenly-spaced event times between 0 and the latest follow-up time
in each study, instead of every observed event/censoring time. Only used if
\code{newdata = NULL} and \code{type} is \code{"survival"}, \code{"hazard"} or \code{"cumhaz"}. This
can be useful for plotting survival or (cumulative) hazard curves, where
prediction at every observed even/censoring time is unnecessary and can be
slow. When a call from within \code{plot()} is detected, e.g. like
\code{plot(predict(fit, type = "survival"))}, \code{times_seq} will default to 50.}
}
\value{
A \link{nma_summary} object if \code{summary = TRUE}, otherwise a list
containing a 3D MCMC array of samples and (for regression models) a data
frame of study information.
}
\description{
Obtain predictions of absolute effects from NMA models fitted with \code{\link[=nma]{nma()}}.
For example, if a model is fitted to binary data with a logit link, predicted
outcome probabilities or log odds can be produced. For survival models,
predictions can be made for survival probabilities, (cumulative) hazards,
(restricted) mean survival times, and quantiles including median survival
times.
When an IPD NMA or ML-NMR model has been fitted, predictions can be
produced either at an individual level or at an aggregate level.
Aggregate-level predictions are population-average absolute effects; these
are marginalised or standardised over a population. For example, average
event probabilities from a logistic regression, or marginal (standardised)
survival probabilities from a survival model.
}
\section{Aggregate-level predictions from IPD NMA and ML-NMR models}{
Population-average absolute effects can be produced from IPD NMA and ML-NMR
models with \code{level = "aggregate"}. Predictions are averaged over the target
population (i.e. standardised/marginalised), either by (numerical)
integration over the joint covariate distribution (for AgD studies in the
network for ML-NMR, or AgD \code{newdata} with integration points created by
\code{\link[=add_integration]{add_integration()}}), or by averaging predictions for a sample of individuals
(for IPD studies in the network for IPD NMA/ML-NMR, or IPD \code{newdata}).

For example, with a binary outcome, the population-average event
probabilities on treatment \eqn{k} in study/population \eqn{j} are
\deqn{\bar{p}_{jk} = \int_\mathfrak{X} p_{jk}(\mathbf{x}) f_{jk}(\mathbf{x})
d\mathbf{x}}
for a joint covariate distribution \eqn{f_{jk}(\mathbf{x})} with
support \eqn{\mathfrak{X}} or
\deqn{\bar{p}_{jk} = \sum_i p_{jk}(\mathbf{x}_i)}
for a sample of individuals with covariates \eqn{\mathbf{x}_i}.

Population-average absolute predictions follow similarly for other types of
outcomes, however for survival outcomes there are specific considerations.
\subsection{Standardised survival predictions}{

Different types of population-average survival predictions, often called
standardised survival predictions, follow from the \strong{standardised survival
function} created by integrating (or equivalently averaging) the
individual-level survival functions at each time \eqn{t}:
\deqn{\bar{S}_{jk}(t) = \int_\mathfrak{X} S_{jk}(t | \mathbf{x}) f_{jk}(\mathbf{x})
d\mathbf{x}}
which is itself produced using \code{type = "survival"}.

The \strong{standardised hazard function} corresponding to this standardised
survival function is a weighted average of the individual-level hazard
functions
\deqn{\bar{h}_{jk}(t) = \frac{\int_\mathfrak{X} S_{jk}(t | \mathbf{x}) h_{jk}(t | \mathbf{x}) f_{jk}(\mathbf{x})
d\mathbf{x} }{\bar{S}_{jk}(t)}}
weighted by the probability of surviving to time \eqn{t}. This is produced
using \code{type = "hazard"}.

The corresponding \strong{standardised cumulative hazard function} is
\deqn{\bar{H}_{jk}(t) = -\log(\bar{S}_{jk}(t))}
and is produced using \code{type = "cumhaz"}.

\strong{Quantiles and medians} of the standardised survival times are found by
solving
\deqn{\bar{S}_{jk}(t) = 1-\alpha}
for the \eqn{\alpha\%} quantile, using numerical root finding. These are
produced using \code{type = "quantile"} or \code{"median"}.

\strong{(Restricted) means} of the standardised survival times are found by
integrating
\deqn{\mathrm{RMST}_{jk}(t^*) = \int_0^{t^*} \bar{S}_{jk}(t) dt}
up to the restricted time horizon \eqn{t^*}, with \eqn{t^*=\infty} for mean
standardised survival time. These are produced using \code{type = "rmst"} or
\code{"mean"}.
}
}

\examples{
## Smoking cessation
\donttest{
# Run smoking RE NMA example if not already available
if (!exists("smk_fit_RE")) example("example_smk_re", run.donttest = TRUE)
}
\donttest{
# Predicted log odds of success in each study in the network
predict(smk_fit_RE)

# Predicted probabilities of success in each study in the network
predict(smk_fit_RE, type = "response")

# Predicted probabilities in a population with 67 observed events out of 566
# individuals on No Intervention, corresponding to a Beta(67, 566 - 67)
# distribution on the baseline probability of response, using
# `baseline_type = "response"`
(smk_pred_RE <- predict(smk_fit_RE,
                        baseline = distr(qbeta, 67, 566 - 67),
                        baseline_type = "response",
                        type = "response"))
plot(smk_pred_RE, ref_line = c(0, 1))

# Predicted probabilities in a population with a baseline log odds of
# response on No Intervention given a Normal distribution with mean -2
# and SD 0.13, using `baseline_type = "link"` (the default)
# Note: this is approximately equivalent to the above Beta distribution on
# the baseline probability
(smk_pred_RE2 <- predict(smk_fit_RE,
                         baseline = distr(qnorm, mean = -2, sd = 0.13),
                         type = "response"))
plot(smk_pred_RE2, ref_line = c(0, 1))
}

## Plaque psoriasis ML-NMR
\donttest{
# Run plaque psoriasis ML-NMR example if not already available
if (!exists("pso_fit")) example("example_pso_mlnmr", run.donttest = TRUE)
}
\donttest{
# Predicted probabilities of response in each study in the network
(pso_pred <- predict(pso_fit, type = "response"))
plot(pso_pred, ref_line = c(0, 1))

# Predicted probabilites of response in a new target population, with means
# and SDs or proportions given by
new_agd_int <- data.frame(
  bsa_mean = 0.6,
  bsa_sd = 0.3,
  prevsys = 0.1,
  psa = 0.2,
  weight_mean = 10,
  weight_sd = 1,
  durnpso_mean = 3,
  durnpso_sd = 1
)

# We need to add integration points to this data frame of new data
# We use the weighted mean correlation matrix computed from the IPD studies
new_agd_int <- add_integration(new_agd_int,
                               durnpso = distr(qgamma, mean = durnpso_mean, sd = durnpso_sd),
                               prevsys = distr(qbern, prob = prevsys),
                               bsa = distr(qlogitnorm, mean = bsa_mean, sd = bsa_sd),
                               weight = distr(qgamma, mean = weight_mean, sd = weight_sd),
                               psa = distr(qbern, prob = psa),
                               cor = pso_net$int_cor,
                               n_int = 64)

# Predicted probabilities of achieving PASI 75 in this target population, given
# a Normal(-1.75, 0.08^2) distribution on the baseline probit-probability of
# response on Placebo (at the reference levels of the covariates), are given by
(pso_pred_new <- predict(pso_fit,
                         type = "response",
                         newdata = new_agd_int,
                         baseline = distr(qnorm, -1.75, 0.08)))
plot(pso_pred_new, ref_line = c(0, 1))
}

## Progression free survival with newly-diagnosed multiple myeloma
\donttest{
# Run newly-diagnosed multiple myeloma example if not already available
if (!exists("ndmm_fit")) example("example_ndmm", run.donttest = TRUE)
}
\donttest{
# We can produce a range of predictions from models with survival outcomes,
# chosen with the type argument to predict

# Predicted survival probabilities at 5 years
predict(ndmm_fit, type = "survival", times = 5)

# Survival curves
plot(predict(ndmm_fit, type = "survival"))

# Hazard curves
# Here we specify a vector of times to avoid attempting to plot infinite
# hazards for some studies at t=0
plot(predict(ndmm_fit, type = "hazard", times = seq(0.001, 6, length.out = 50)))

# Cumulative hazard curves
plot(predict(ndmm_fit, type = "cumhaz"))

# Survival time quantiles and median survival
predict(ndmm_fit, type = "quantile", quantiles = c(0.2, 0.5, 0.8))
plot(predict(ndmm_fit, type = "median"))

# Mean survival time
predict(ndmm_fit, type = "mean")

# Restricted mean survival time
# By default, the time horizon is the shortest follow-up time in the network
predict(ndmm_fit, type = "rmst")

# An alternative restriction time can be set using the times argument
predict(ndmm_fit, type = "rmst", times = 5)  # 5-year RMST
}
}
\seealso{
\code{\link[=plot.nma_summary]{plot.nma_summary()}} for plotting the predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_km.R
\name{geom_km}
\alias{geom_km}
\title{Kaplan-Meier curves of survival data}
\usage{
geom_km(
  network,
  ...,
  transform = c("identity", "cloglog", "log", "cumhaz"),
  curve_args = list(),
  cens_args = list()
)
}
\arguments{
\item{network}{A \code{nma_data} network object containing survival outcomes}

\item{...}{Additional arguments passed to \code{\link[survival:survfit]{survival::survfit()}}}

\item{transform}{Character string giving the transformation to apply to the
KM curves before plotting. The default is \code{"identity"} for no
transformation; other options are \code{"cloglog"} for \eqn{\log(-\log(S))},
\code{"log"} for \eqn{\log(S)}, or \code{"cumhaz"} for the cumulative hazard
\eqn{-\log(S)}.}

\item{curve_args}{Optional list of arguments to customise the curves plotted
with \code{\link[ggplot2:geom_path]{ggplot2::geom_step()}}}

\item{cens_args}{Optional list of arguments to customise the censoring marks
plotted with \code{\link[ggplot2:geom_point]{ggplot2::geom_point()}}}
}
\value{
A ggplot2 geom list that can be added to a ggplot2 plot object
}
\description{
This helper function constructs a ggplot2 geom to plot Kaplan-Meier curves
from a network containing survival or time-to-event outcomes. This is useful
for overlaying the "raw" survival data on the estimated survival functions
created with plotted with \code{\link[=plot.surv_nma_summary]{plot.surv_nma_summary()}}, but can also be used
standalone to plot Kaplan-Meier curves before fitting a model.
}
\examples{
# Set up newly-diagnosed multiple myeloma network

head(ndmm_ipd)
head(ndmm_agd)

ndmm_net <- combine_network(
  set_ipd(ndmm_ipd,
          study, trt,
          Surv = Surv(eventtime / 12, status)),
  set_agd_surv(ndmm_agd,
               study, trt,
               Surv = Surv(eventtime / 12, status),
               covariates = ndmm_agd_covs))
# Plot KM curves using ggplot2
library(ggplot2)

# We need to create an empty ggplot object to add the curves to
ggplot() + geom_km(ndmm_net)

# Adding plotting options, facets, axis labels, and a plot theme
ggplot() +
  geom_km(ndmm_net,
          curve_args = list(linewidth = 0.5),
          cens_args = list(size = 3, shape = 124)) +
  facet_wrap(vars(Study)) +
  labs(x = "Time", y = "Survival Probability") +
  theme_multinma()

# Using the transform argument to produce log-log plots (e.g. to assess the
# proportional hazards assumption)
ggplot() +
  geom_km(ndmm_net, transform = "cloglog") +
  facet_wrap(vars(Study)) +
  theme_multinma()

# Using the transform argument to produce cumulative hazard plots
ggplot() +
  geom_km(ndmm_net, transform = "cumhaz") +
  facet_wrap(vars(Study)) +
  theme_multinma()

# This function can also be used to add KM data to plots of estimated survival
# curves from a fitted model, in a similar manner
\donttest{
# Run newly-diagnosed multiple myeloma example if not already available
if (!exists("ndmm_fit")) example("example_ndmm", run.donttest = TRUE)
}
# Plot estimated survival curves, and overlay the KM data
\donttest{
plot(predict(ndmm_fit, type = "survival")) + geom_km(ndmm_net)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinma-package.R
\docType{package}
\name{multinma-package}
\alias{multinma-package}
\alias{multinma}
\title{multinma: A Package for Network Meta-Analysis of Individual and Aggregate
Data in Stan}
\description{
\if{html}{\figure{logo.svg}{options: width="120" style="float: right;" alt="multinma logo"}}
An R package for performing network meta-analysis and network meta-regression
with aggregate data, individual patient data, or mixtures of both.
}
\details{
Network meta-analysis (NMA) combines (aggregate) data from multiple
studies on multiple treatments in order to produce consistent estimates of
relative treatment effects between each pair of treatments in the network
\insertCite{TSD2}{multinma}.

Network meta-regression (NMR) extends NMA to include covariates, allowing
adjustment for differences in effect-modifying variables between studies
\insertCite{TSD3}{multinma}. NMR is typically performed using aggregate
data (AgD), which lacks power and is prone to ecological bias. NMR with
individual patient data (IPD) is the gold standard, if data are available.

Multilevel network meta-regression (ML-NMR) allows IPD and AgD to be
incorporated together in a network meta-regression
\insertCite{methods_paper,Phillippo_thesis}{multinma}. As in IPD NMR, an
individual-level regression model is defined. AgD studies are then fitted
by integrating the individual-level model over the respective covariate
distributions. This correctly links the two levels of the model (instead of
"plugging in" mean covariate values), avoiding aggregation bias.
Population-adjusted treatment effects \insertCite{TSD18}{multinma} can be
produced for any study population in the network, or for an external target
population.

Models are estimated in a Bayesian framework using Stan
\insertCite{Carpenter2017}{multinma}. Quasi-Monte Carlo numerical
integration based on Sobol' sequences is used for the integration in ML-NMR
models, with a Gaussian copula to account for correlations between
covariates \insertCite{methods_paper,Phillippo_thesis}{multinma}.
}
\section{Getting Started}{

A good place to start is with the package vignettes which walk through
example analyses, see \code{vignette("vignette_overview")} for an overview.
The series of NICE Technical Support Documents on evidence synthesis gives
a detailed introduction to network meta-analysis:

\insertRef{TSD_evsynth}{multinma}

Multilevel network meta-regression is set out in the following methods paper:

\insertRef{methods_paper}{multinma}
}

\references{
\insertAllCited{}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dmphillippo.github.io/multinma/}
  \item \url{https://github.com/dmphillippo/multinma}
  \item Report bugs at \url{https://github.com/dmphillippo/multinma/issues}
}

}
\author{
\strong{Maintainer}: David M. Phillippo \email{david.phillippo@bristol.ac.uk} (\href{https://orcid.org/0000-0003-2672-7841}{ORCID})

Other contributors:
\itemize{
  \item Samuel J. Perren \email{samuel.perren@bristol.ac.uk} (\href{https://orcid.org/0009-0005-1921-6957}{ORCID}) [contributor]
  \item Niels Dunnewind (\href{https://orcid.org/0009-0009-8675-9109}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_prior-class.R
\name{nma_prior-class}
\alias{nma_prior-class}
\alias{nma_prior}
\title{The nma_prior class}
\description{
The \code{nma_prior} class is used to specify prior distributions.
}
\details{
Objects of class \code{nma_prior} have the following components:
\describe{
\item{\code{dist}}{Distribution name}
\item{\code{fun}}{Name of constructor function, as string (e.g. \code{"normal"})}
\item{\code{...}}{Parameters of the distribution}
}

The distribution parameters, specified as named components in \code{...}, match
those in the constructor functions (see \link{priors}).
}
